#!/bin/bash

# build_dep.sh creates a debian .deb package from an iqportal app directory
#
# example:
#	 workspace/pvcycle $ ../iqportal/related/bin/build_deb.sh -r testing-virtual

set -e

project_name=""
version=""
verbose=1
release=""
project_dir="$(pwd)"
iqportal_dir="$HOME/workspace/iqportal"
package_name=""
debug=0
include_tests=0

log() {
	[ $verbose -eq 1 ] && echo $1
}

main() {

	# check requirements
	[ -z $release ] && usage && exit 1

	# get tag based on last tag created
	if [[ $version == "" ]]; then
		version=$(git describe --tags --abbrev=0)
	fi

	# versions in debian cant have underscores
	version=$(echo $version | sed 's/_/-/g')

	project_name=$(basename $project_dir)
	package_name="$project_name-$release"

	echo "Building deb for $package_name"
	echo " - version: $version"
	echo " - release $release"
	echo " - project_dir $project_dir"

	app_dirs="$(find $project_dir -maxdepth 1 -type d)"

	# if building iqportal then append the related/libs dirs to app_dirs
	if [[ $project_name == "iqportal" ]]; then
		app_dirs="$app_dirs $(find related/libs/ -maxdepth 1 -type d)"
	fi

	clean
	build
	package

}

clean() {

	if [[ $debug != 1 ]]; then
		if [ ! -z "$(git diff)" ]; then
			echo 'git is dirty, aborting.. (need to commit?)'
			exit 1
		fi

		# clean project python dirs
		for i in $(ls $project_dir); do
			if [ -e "$project_dir/$i/__init__.py" ]; then
				log "git clean $i"
				git clean -x -d -f "$project_dir/$i" > /dev/null
			fi
		done

		# todo: git hard reset?

	fi

	# clean package dir
	package_dir="$project_dir/deb"

	log "Cleaning package_dir $package_dir"
	if [ -e $package_dir ]; then
		sudo rm -rf $package_dir
	fi
	mkdir -p $package_dir

}


build() {

	for dir in $app_dirs; do
		if [ -e "$dir/__init__.py" ]; then

			log "Compiling pyc files in $dir"
			python -m compileall $dir > /dev/null

			log "Compressing JS in $dir"
			$iqportal_dir/related/yuicompressor/compress.sh $dir > /dev/null

			log "Compiling .po files in $dir"
			for po_file in $(find $dir -name *.po); do
				mo_file=$(echo $po_file | sed -e 's/\.po$/.mo/g')
				msgfmt -o $mo_file $po_file > /dev/null
			done


		fi
	done

}

package() {

	dest_fs="/var/www/iqportal-$release.basesoft.com"
	dest_fs_escaped=$(echo $dest_fs | sed -e 's/[\/&]/\\&/g')

	dest="${package_dir}${dest_fs}"
	dest_escaped=$(echo $dest | sed -e 's/[\/&]/\\&/g')

	dest_lib="$dest/lib"
	dest_lib_escaped=$(echo $dest_lib | sed -e 's/[\/&]/\\&/g')

	project_dir_escaped=$(echo $project_dir | sed -e 's/[\/&]/\\&/g')
	mkdir -p $dest_lib

	site_media=$dest/site_media
	mkdir -p $site_media

	# copy all python directories into the package dir
	for dir in $app_dirs; do
		if [ -e "$dir/__init__.py" ]; then

			log "Copying $dir"
			/bin/cp -ar $dir $dest_lib/
		fi
	done

	# if packaging iqportal..
	if [[ $project_name == "iqportal" ]]; then

		log "Adding .wsgi, vars etc"

		for f in "iqportal.wsgi" "vars"; do
			cp $iqportal_dir/related/deb/$f $dest/
			sed -i "s/%DEST%/$dest_fs_escaped\/lib/g" $dest/$f
			sed -i "s/%SETTINGS%/basesoft.settings.${release}/g" $dest/$f
		done

		log "Adding manage.py"
		cp $iqportal_dir/related/bin/manage.py $dest/

		log "Adding scheduler scripts"
		script=$package_dir/etc/init.d/iqportal-scheduler-$release
		mkdir -p $package_dir/etc/init.d/
		cp $iqportal_dir/related/deb/init.d/scheduler $script
		sed -i "s/%RELEASE%/${release}/g" $script
		sed -i "s/%DEST%/$dest_fs_escaped/g" $script
		cp $iqportal_dir/related/bin/scheduler_server.tac $dest/

		# set version
		sed -i "s/VERSION = 'trunk'/VERSION = '$version'/" $dest_lib/basesoft/settings/settings.py

	fi

	log "Adding bin scripts"
	mkdir -p $dest/bin/$project_name
	cp -r $project_dir/related/bin/* $dest/bin/$project_name

	# fix all media in package dir
	for media_dir in $(find $dest_lib -type d -name media); do

		# skip nested media dirs
		if [[ $media_dir == */media/* ]]; then
			continue;
		fi


		# remove $dest from media_dir so it will look something like 'iqportal/client/core/media',
		# remove the last /media part,
		# and convert slashes to dots
		new_media_dir=$(echo $media_dir | sed "s/^$dest_lib_escaped\///g" | sed 's/\/media$//g' | sed 's/\//./g' )
		mkdir -p $site_media/$new_media_dir

		log "Working on 'media' for $new_media_dir"

		# extensions that get moved to site_media
		media_files="jpg png gif svg pdf swf ico eot woff ttf html"

		# extensions that get copied
		media_files_no_rm="html"

		# recursive copy media files
		rsync_args=""; for ext in $media_files $media_files_no_rm; do rsync_args="$rsync_args --include *.$ext"; done
		rsync -r -m $rsync_args -f 'hide,! */' $media_dir/* $site_media/$new_media_dir

		# remove media files that shouldnt stay
		find_args=""; for ext in $media_files; do find_args="${find_args}$ext|"; done
		find $media_dir \
			-type f \
			-regextype posix-extended -iregex ".*\.(${find_args%?})\$" \
			-exec rm {} \;

		# application specific logic

		if [[ $new_media_dir == "basesoft.web.core" ]]; then
			rm $media_dir/fonts/fontello/config.json
		fi

		if [[ $new_media_dir == "basesoft.web.contrib.fusioncharts" ]]; then

			rm $media_dir/fusioncharts/firebug-lite.js

			# fusion charts wants to load js files from media
			cp $media_dir/fusioncharts/*.js $site_media/$new_media_dir/fusioncharts
		fi

	done

	log "Removing .po files"
	find $dest_lib -type f -name '*.po' -exec rm {} \;

	if [[ $include_tests != 1 ]]; then

		log "Removing test dirs"
		for test_dir in $(find $dest_lib -type d -name 'test' -or -name 'selenium'); do
			[ -d $test_dir ] && rm -rf $test_dir
		done

	fi

	log "Cleaning debug files"
	find $dest_lib -regextype posix-extended -regex '.*debug.(js|css)'	-exec rm {} \;

	log "Copying DEBIAN files"
	cp -ar $iqportal_dir/related/deb/DEBIAN $package_dir
	sed -i "s/%PACKAGE%/$package_name/g" $package_dir/DEBIAN/control
	sed -i "s/%VERSION%/$version/g" $package_dir/DEBIAN/control
	sed -i "s/%DEST%/$dest_fs_escaped/g" $package_dir/DEBIAN/postinst

	log "Setting permissions"
	sudo chown -R root:root $package_dir

	deb="${package_name}-${version}_all.deb"
	echo "Packaging to $deb"
	dpkg-deb --build $package_dir > /dev/null
	mv deb.deb $deb

	# remove package dir
	if [[ $debug != 1 ]]; then
		sudo rm -rf $package_dir
	fi


}

usage() {
	echo "Usage: build_deb.sh -r release [ -d project_dir ] [ -q iqportal_dir ] [ -t tag ]"
	echo "Example: build_deb.sh -r acceptance"
	echo
	echo "build_deb.sh creates a debian package from an iqportal project"
}

while getopts "h?vr:d:q:t:ni:" opt; do
		case "$opt" in
		h|\?)
				usage
				;;
		v)	verbose=1
				;;
		r)	release=$OPTARG
				;;
		d)	project_dir=$OPTARG
				;;
		q)	iqportal_dir=$OPTARG
				;;
		t)	version=$OPTARG
				;;
		n)	debug=1
				;;
		i)	include_tests=$OPTARG
				;;
		esac
done

shift $((OPTIND-1))

[ "$1" = "--" ] && shift

main
