#!/usr/bin/env python

# this script loads the iqportal settings file
# and for each application runs the messages make/compile
# for generation of translations

import logging
logging.basicConfig(level=logging.DEBUG)

import getopt, sys, os
from django.conf import settings
from django.core.management.commands import makemessages, compilemessages

import commands

# make a list of all locales
locales = []
for i in settings.LANGUAGES:
    locales.append(i[0])

def my_import(name):
    mod = __import__(name)
    components = name.split('.')
    for comp in components[1:]:
        mod = getattr(mod, comp)
    return mod


def get_app_paths():
    rv = []
    for app in settings.INSTALLED_APPS:
        #if app.startswith('django'):
        #    continue
        mod = my_import(app)
        rv.append(mod.__path__[0])
    print rv
    return rv

def help():
    print 'Compiles or generates iqportal translation files'
    print 'Usage: messages.py [--generate] [--compile]'

def generate():
    for path in get_app_paths():

        # change the cwd
        os.chdir(path)

        # for each locale
        for l in locales:

            # for django and djangojs
            for x in [('django', 'py'), ('djangojs', '.js')]:
            #for x in [('django', '')]:
                print 'Generating messages for %s,%s,%s in %s' % (l, x[0], x[1], path)
                makemessages.make_messages(l, x[0], extensions=[x[1]])

def compile():
    for path in get_app_paths():

        # integrity check
        global locales
        for locale in locales:
            for file in ['django', 'djangojs']:
                po_file = "%s/locale/%s/LC_MESSAGES/%s.po" % (path, locale, file)
                if os.path.exists(po_file):
                    f = open(po_file)
                    contents = f.read()

                    if contents.find("nplurals=INTEGER") != -1 or contents.find("plural=EXPRESSION") != -1:
                        raise Exception("invalid plural forms in %s " % po_file)



        # change the cwd
        os.chdir(path)

        # for each locale
        for l in locales:
            logging.info('Compiling messages for %s in %s', l, path)
            compilemessages.compile_messages(sys.stderr, locale=l)


def export(locale):

    global locales
    if locale not in locales:
        raise Exception('Cannot export locale [%s]: not in setting.locales [%s]', (locale, str(locales)))

    files = ""

    for app_path in get_app_paths():

        if app_path.split('/')[-1] == 'test':
            print 'skipping test app: %s' % app_path
            continue

        for file in ['django', 'djangojs']:

            po_file = "%s/locale/%s/LC_MESSAGES/%s.po" % (app_path, locale, file)

            if os.path.exists(po_file):
                files += "%s " % po_file

    outfile = "iqportal_%s.po" % locale

    # this command concatenates all found po files into one big one
    # the --use-first makes sure we dont get translation conflicts (no fuzzy) and headers will be kept clean
    # http://www.gnu.org/software/hello/manual/gettext/msgcat-Invocation.html#msgcat-Invocation
    cmd = "msgcat --use-first -o %s %s" % (outfile, files)

    print
    print 'running command: %s' % cmd
    print

    status, output = commands.getstatusoutput(cmd)
    if status != 0:
        raise Exception('msgcat failed: %s: %s' % (cmd, output))

    print "output written to %s" % outfile

def import_(locale, updated_file):

    global locales
    if locale not in locales:
        raise Exception('Cannot import locale [%s]: not in setting.locales [%s]', (locale, str(locales)))


    for app_path in get_app_paths():

        if app_path.split('/')[-1] == 'test':
            print 'skipping %s' % app_path
            continue

        for file in ['django', 'djangojs']:

            po_file = "%s/locale/%s/LC_MESSAGES/%s.po" % (app_path, locale, file)
            if os.path.exists(po_file):

                print 'working on %s' % po_file

                # This command merges the translator updated file together with the existing
                # during that it removes comments and double newlines
                # from http://www.gnu.org/software/hello/manual/gettext/Using-Compendia.html
                cmd = "msgmerge -N %s %s  | sed -e '/^#~/d' | sed -e '/^#, fuzzy/d' | pcregrep -v -M '^\n\n' > /tmp/iqportal_tmp.po && mv /tmp/iqportal_tmp.po %s" % (updated_file, po_file, po_file)

                status, output = commands.getstatusoutput(cmd)
                if status != 0:
                    raise Exception('msgmerge failed: %s: %s' % (cmd, output))
            else:
                print 'no such file: %s' % po_file

    print 'import done.. now run compile'



def main():
    try:
        opts = getopt.getopt(sys.argv[1:], "hgc", ["help", "generate", "compile", "export", "import", "locale=", "file="])[0]
    except getopt.GetoptError, err:
        print str(err) # will print something like "option -a not recognized"
        help()
        sys.exit(2)

    command = None
    locale = None
    file = None

    for o, a in opts:
        if o in ("-h", "--help"):
            help()
            sys.exit()
        elif o in ("-g", "--generate"):
            command = 'generate'
        elif o in ("-c", "--compile"):
            command = 'compile'
        elif o in ("--export"):
            command = 'export'
        elif o in ("--import"):
            command = 'import'
        elif o in ("--locale"):
            locale = a
        elif o in ("--file"):
            file = a

    if command == 'generate':
        generate()

    elif command == 'compile':
        compile()

    elif command == 'export':
        if locale is None:
            raise Exception('missing --locale argument')
        export(locale)

    elif command == 'import':

        if locale is None:
            raise Exception('missing --locale argument')

        if file is None:
            raise Exception('missing --file argument')

        import_(locale, file)

    else:
        raise Exception('nothing to do')



if __name__ == "__main__":
    main()


