# Copyright (c) 2008 BaseSoft Solutions B.V.
# See LICENSE for details.

"""
This contains the classes needed to run the Scheduler server. Its primary used from a .tac file.
@see: http://twistedmatrix.com/projects/core/documentation/howto/servers.html
"""

import datetime, collections

import twisted
from twisted.python.versions import Version
from twisted.web import server, xmlrpc
from twisted.application import service
from twisted.internet import task as twisted_internet_task
from twisted.internet import threads
from twisted.python.reflect import namedAny

from django.conf import settings

from basesoft.scheduler import task as basesoft_scheduler_task


from basesoft.web.core.util.db import Session
from basesoft.web.core.models import SchedulerTask as SaTask
from sqlalchemy.orm.exc import NoResultFound
from types import NoneType

from basesoft import logging
import xmlrpclib
from basesoft.scheduler.crontab import CronTab
from basesoft.util.virtual_date import today, now
logger = logging.getLogger('basesoft.scheduler')

from twisted.internet import reactor

class SchedulerWebSite(server.Site):
    """
    Extends xmlrpc.Site: A web site managing log, sessions and resources
    Wonder if we even need this actually.. well maybe for http authentication someday
    """

    def __init__(self, resource):
        server.Site.__init__(self, resource)

class SchedulerXmlRpc(xmlrpc.XMLRPC):
    """
    This is a http xmlrpc server which we can use to query the status of the scheduler. Like which tasks are running
    and maybe we can let these return information about their status (like how many emails sent).
    """

    def __init__(self, sched_service):
        """
        Constructor receives a reference to the running SchedulerService
        """
        xmlrpc.XMLRPC.__init__(self, allowNone=True)
        self._sched_service = sched_service

    def xmlrpc_echo(self, *args):
        """xml rpc test method: return all passed args."""
        return args

    def xmlrpc_list_registered_tasks(self, check_running=False):
        """
        Returns a list of names of registered tasks with id's which can be used to retrieve history
        """
        return self._sched_service.list_registered_tasks(check_running=check_running)

    def xmlrpc_start_task(self, task_id_or_name, params):
        """
        Starts the selected task_id (as a single run).
        @param params: optional dictionary argument that gets passed into the task consstructor. usefull for task triggering
        @return process_id
        """
        return self._sched_service.start_task(task_id_or_name, params)

    def xmlrpc_enable_task(self, task_id, enable=True):
        """
        enables the selected task_id. set enable=False for disable
        """
        self._sched_service.enable_task(task_id, enable)

    def xmlrpc_get_task_results_by_name(self, names):
        if not isinstance(names, list):
            names = [names]

        return self._sched_service.list_registered_tasks_by_names(names)

    def xmlrpc_get_task_results(self, task_id):
        """
        Returns the known results for the given task_id
        """
        return self._sched_service.get_task_results(task_id)

    def xmlrpc_list_processes(self):
        """
        Returns a list of running processes in the form
        """
        return self._sched_service.list_processes()

    def xmlrpc_list_task_queue_tasks(self):
        return self._sched_service.list_task_queue_tasks()

    def xmlrpc_get_process_status(self, process_id):
        """
        Calls the get_status method on the running process_id
        """
        return self._sched_service.get_process_status(process_id)

    def xmlrpc_send_process_signal(self, process_id, signal):
        """
        Sends the given signal to the processes. Depending how the task is implemented this can make it stop, pause, etc
        """
        return self._sched_service.send_process_signal(process_id, signal)

class SchedulerService(service.Service):
    """
    Next to the SchedulerWebSite serive this Service is also running in the schedule server.
    This is the beating heart of the scheduler that is responsable for spawning tasks in their interval.
    """

    # number of results to keep in result history.. per task!
    MAX_RESULTS = 20

    # the number of concurrent tasks to run
    MAX_CONCURRENT_TASKS = 5

    # number of seconds between runs of queue cleaner to run
    QUEUE_CLEANER_REPAWN = 2

    def __init__(self, db_engine):

        # this is a counter that gets incremented and assigned for every task
        self.__process_counter = 0

        # loaded tasks
        self._tasks = []

        # running processes
        self._processes = []

        # the run queue
        self._queue = collections.deque([])

        # for accounting
        self._running_tasks = []

        # results keeps result values per task
        self._results = {}

        # task queue, used for queue-ing based on task type
        self._task_queue = {}

        # twisted version check.. it seems older twister versions didnt clean up well the threads that
        # exited with an exception. Ticket #567
        if twisted.version < Version('twisted', 8,2,0):
            raise Exception('Twisted version must be higher than 8.2.0')

        # db task that can only be used in the server (not in tasks)
        self._sa_session = Session()


    def startService(self):
        """
        Initialies the scheduler by looping the django installed applications and reading their scheduler module. In
        there tasks can be defined wich are scheduled to run here.
        """

        logger.debug('SchedulerService: service starting..');
        service.Service.startService(self)

        for app in settings.INSTALLED_APPS:
            try:
                module = namedAny(app+'.schedule')
            except AttributeError:
                continue

            if not hasattr(module, 'tasks'):
                logger.warning('SchedulerService: WARNING: scheduler module does not have tasks defined: %s' % module )
                continue

            logger.debug('SchedulerService: processing schedule module: %s with tasks %s' % (module, module.tasks) )
            for ScheduleTask in module.tasks:

                try:
                    db_task = self._sa_session.query(SaTask).filter(SaTask.name==self._task_name(ScheduleTask)).one()
                    ScheduleTask.ENABLED = db_task.enabled
                except NoResultFound:
                    ScheduleTask.ENABLED = False # dont start tasks by default

                logger.info('SchedulerService: task %s to enabled=%s' % (self._task_name(ScheduleTask), ScheduleTask.ENABLED))

                if not issubclass(ScheduleTask, basesoft_scheduler_task.ScheduleTask):
                    logger.error('SchedulerService: ERROR: %s is not subclass of ScheduleTask' % ScheduleTask)

                #Start the queue cleaner, runs every QUEUE_CLEANER_REPAWN secs
                queue_cleaner = twisted_internet_task.LoopingCall(self._clear_queue)
                queue_cleaner.start(SchedulerService.QUEUE_CLEANER_REPAWN, False)

                # Start the task for its interval. This calls self._execute_task with the ScheduleTask for every
                # ScheduleTask.INTERVAL seconds. The callback takes care of spawning the task in a new tread.
                # If the interval is 0 it wont be scheduled but it could be started manually
                if isinstance(ScheduleTask.INTERVAL, (int, long)):

                    if ScheduleTask.INTERVAL > 0:
                        logger.debug('ScheduleTask.INTERVAL: %s' % ScheduleTask.INTERVAL)
                        loop_task = twisted_internet_task.LoopingCall(self._execute_task, ScheduleTask)
                        loop_task.start(ScheduleTask.INTERVAL, False)

                # if INTERVAL is of type CronTab then schedule it for the next run
                elif isinstance(ScheduleTask.INTERVAL, CronTab):

                    # wrapper gets executed when the next task should be ran and also schedules a new run
                    def _wrapper(_Task):
                        self._execute_task(_Task)
                        _schedule_next_cron(_Task)

                    # function that schedules the next run, called after each run
                    def _schedule_next_cron(_Task):
                        next_run = _Task.INTERVAL.next( now=now() )
                        c = reactor
                        c.callLater(next_run, _wrapper, _Task)

                    _schedule_next_cron(ScheduleTask)

                else:
                    raise Exception('Invalid INTERVAL')

                # append task class to known task
                logger.debug('registring task %s' % ScheduleTask)
                self._tasks.append(ScheduleTask)

                # reserve a place in the task results
                self._results[self._task_name(ScheduleTask)] = []

                logger.debug('SchedulerService: scheduled task %s with interval %s' % (ScheduleTask, ScheduleTask.INTERVAL))

    def stopService(self):
        """
        TODO: clean up all running tasks
        """
        logger.debug('SchedulerService: stopService()')

        return None

    def _task_name(self, ScheduleTask):
        """
        returns the unique name for the given task
        @param ScheduleTask: class reference to task
        """
        return '%s.%s' % (ScheduleTask.__module__, ScheduleTask.__name__)

    def _clear_queue(self):
        """
        This function is called every QUEUE_CLEANER_REPAWN to clear any backlog of tasks
        in the queue
        """
        dequeued = True

        while dequeued == True:
            if len(self._running_tasks) < SchedulerService.MAX_CONCURRENT_TASKS:
                try:
                    self._run_in_thread(self._queue.popleft())
                except IndexError as e:
                    dequeued = False
            else:
                dequeued = False

    def _queue_for_task(self, task):

        ScheduleTask = task.__class__

        if not self._task_queue.has_key(ScheduleTask):
            self._task_queue[ScheduleTask] = []

        # The queue of this Task type is full, notify the through task
        if len(self._task_queue[ScheduleTask]) >= ScheduleTask.MAX_COMPARABLE_TASKS_IN_QUEUE:
            logger.info('SchedulerService: INFO: Task based queue is full.')
            task.queue_full_notification()
            self._processes.remove(task)
            return

        logger.info('SchedulerService: INFO: Task added to to task based queue.')
        self._task_queue[ScheduleTask].insert(0, task)

    def _next_for_task(self, SchedulerTask):
        if not self._task_queue.has_key(SchedulerTask):
            return

        if len(self._task_queue[SchedulerTask]) == 0:
            logger.info('SchedulerService: INFO: No more Tasks in task based queue')
            return

        logger.info('SchedulerService: INFO: Next task for %s added to Scheduler queue' % SchedulerTask)
        self._queue.append(self._task_queue[SchedulerTask].pop())

    def _execute_task(self, ScheduleTask, params=None):
        """
        Gets called for the every ScheduleTask.INTERVAL seconds. Instanciate the task and call its execute method in a
        new thread, or queue it

        @returns process_id
        """

        # dont run task if its not enabled
        if not ScheduleTask.ENABLED:
            logger.debug('_execute_task: task not enabled: %s' % ScheduleTask)
            return

        #check if we need to check for duplicates
        #we need to check this again to catch the periodic
        #running of tasks on an INTERVAL
        if not ScheduleTask.ALLOW_DUPLICATES:
            for x in self._processes:
                if isinstance(x, ScheduleTask):
                    logger.info('SchedulerService: INFO: Dropping duplicate task %s' % ScheduleTask)
                    return

        # incremenent task counter if new task
        self.__process_counter = self.__process_counter + 1

        # instanciate task
        task = ScheduleTask(process_id=self.__process_counter, start_time=datetime.datetime.now(), params=params)

        logger.debug('SchedulerService: task started %s' % task)

        if ScheduleTask.QUEUE_IF_EXISTS:
            for x in self._processes:
                if isinstance(x, ScheduleTask):
                    self._processes.append(task)
                    self._queue_for_task(task)
                    return self.__process_counter

        # append task to running processes
        self._processes.append(task)

        #check to see if we have MAX_CONCURRENT_TASKS already
        if len(self._running_tasks) >= SchedulerService.MAX_CONCURRENT_TASKS:
            self._queue.append(task)
            return self.__process_counter

        self._run_in_thread(task)

        # return process_id
        return self.__process_counter

    def _run_in_thread(self, task):
        """
        Spins off the task to a thread
        """

        deferred = threads.deferToThread(task.execute_wrapper)
        self._running_tasks.append(task.process_id)

        def cbTaskFinished(task_rv, task):
            """
            Callback executed when the task's execute method has been executed.
            @param task_rv: return value from the task's execute method
            @param task: task instance which has been executed
            """
            #logger.debug('SchedulerService: task finished: %s' % task)

            # remove the task from the running processes
            self._processes.remove(task)
            self._running_tasks.remove(task.process_id)

            # record task to result history if the return value is not None or a non empty list
            if task_rv is not None or (isinstance(task_rv, list) and len(task_rv) > 0):
                self._add_result(task, task_rv)

            if task.QUEUE_IF_EXISTS:
                self._next_for_task(task.__class__)

        def ebTaskFinished(failure, task):

            logger.error('SchedulerService: ERROR: task finished with error: %s' % failure)
            logger.error(failure.getBriefTraceback())

            try:
                self._processes.remove(task)
                self._running_tasks.remove(task.process_id)

                # make string from failure.. maybe do this better (with stacetrack)
                # so we have more debugging info
                failure_str = str(failure)

                # When DEBUG is False, send an error message to the admins. (similar to how django reports errors)
                if not settings.DEBUG:
                    from django.core.mail import mail_admins

                    subject = 'Error (SchedulerServer)'
                    message = failure_str
                    mail_admins(subject, message, fail_silently=True)

                self._add_result(task, failure_str, failure=True)
            except Exception, e:
                # if we come here the it was unable to save the result so just log it
                logger.error('SchedulerService: ERROR: %s' % e.message)
                raise e

        # hookup callbacks
        deferred.addCallback(cbTaskFinished, task)
        deferred.addErrback(ebTaskFinished, task)

    def _get_task(self, task_id):
        """
        gets the Task for the given task_id
        """
        try:
            return self._tasks[task_id]
        except IndexError:
            raise NoSuchTaskException('No such task_id: %s' % task_id)

    def _get_task_by_name(self, task_name):
        for task in self._tasks:
            if self._task_name(task) == task_name:
                return task
        raise SchedulerException('_get_task_by_name: task not found: task_name')

    def _get_process(self, process_id):
        """
        Returns the process object in the current running processes for the given process_id
        """
        for process in self._processes:
            if process_id == process.process_id:
                return process

        raise NoSuchProcessException('No such process_id: %s' % process_id)

    def _add_result(self, task, task_rv, failure=False):
        """
        Adds an entry to the result list
        @param task: task instance
        @param task_rv: result of execute() task or exception data if failure
        @param failure: boolean
        """
        task_name = self._task_name(task.__class__)

        # make space if history is too big
        if len(self._results[task_name]) >= SchedulerService.MAX_RESULTS:
            self._results[task_name].pop(0)

        # add result
        self._results[task_name].append({
            'start_time': task.start_time,
            'end_time': datetime.datetime.now(),
            'result': unicode(task_rv),
            'failure': failure
        })

    def list_registered_tasks(self, check_running=False):
        """
        Returns a list of all registered tasks which can be used for history or manual starting
        """
        rv = []
        for i in range(len(self._tasks)):
            name = self._task_name(self._tasks[i])
            rv.append({
                'task_id': i,
                'name': name,
                'interval': unicode(self._tasks[i].INTERVAL),
                'enabled': self._tasks[i].ENABLED,
                'results': self.get_task_results(i),
                'is_running': None if not check_running else self._is_running(name)
            })
        return rv

    def list_registered_tasks_by_names(self, names, check_running=False):
        rv = []
        for task in names:
            Task = self._get_task_by_name(task)
            rv.append({
                'name': task,
                'interval': unicode(Task.INTERVAL),
                'enabled': Task.ENABLED,
                'results': self.get_task_results(task),
                'is_running': None if not check_running else self._is_running(task)
            })


        return rv

    def enable_task(self, task_id, enable=True):
        task = self._get_task(task_id)
        task.ENABLED = enable

        task_name = self._task_name(task)

        try:
            db_task = self._sa_session.query(SaTask).filter(SaTask.name==task_name).one()
        except NoResultFound:
            db_task = SaTask()
            db_task.name = task_name

        db_task.enabled = enable

        self._sa_session.add(db_task)
        self._sa_session.flush()


    def start_task(self, task, params=None):
        """
        Starts the task for the given task_id
        """

        assert isinstance(params, (NoneType, dict))

        if isinstance(task, int):
            Task = self._get_task(task)
        elif isinstance(task, (str, unicode)):
            Task = self._get_task_by_name(task)
        else:
            raise SchedulerException('start_task: task argument must be int or str')

        if not Task.ENABLED:
            raise TaskNotEnabledException('start_task: cannot start task: task is not enabled')

        #check if we need to check for duplicates
        if not Task.ALLOW_DUPLICATES:
            for x in self._processes:
                if isinstance(x, Task):
                    logger.error('SchedulerService: ERROR: Duplicate Task %s' % Task)
                    raise DuplicateTaskException('Duplicate task: %s' % Task)

        if Task.QUEUE_IF_EXISTS:
            if self._task_queue.has_key(Task) and len(self._task_queue[Task]) >= Task.MAX_COMPARABLE_TASKS_IN_QUEUE:
                raise MaximumTaskQueueException('Task queue is full')

        logger.info('start_task: starting task %s with params %s' % (Task, params))

        return self._execute_task(Task, params)

    def get_task_results(self, task):
        """
        Returns task results for the given task_id
        """
        if isinstance(task, int):
            Task = self._get_task(task)
        elif isinstance(task, (str, unicode)):
            Task = self._get_task_by_name(task)
        return self._results[self._task_name(Task)]

    def list_processes(self):
        """
        Returns information about the current running processes
        """
        rv = []
        for process in self._processes:
            rv.append({
                'process_id': process.process_id,
                'name': self._task_name(process.__class__),
                'start_time': process.start_time,
            })

        return rv

    def _is_running(self, task):
        processes = self.list_processes()
        for p in processes:
            if p['name'] == task:
                return True
        return False

    def list_task_queue_tasks(self):
        rv = {}
        for t, tasks in self._task_queue.iteritems():
            rv[self._task_name(t)] = len(tasks)
        return rv

    def get_process_status(self, process_id):
        """
        Calls get_status() on the process for the given process_id
        """
        process = self._get_process(process_id)
        return process.get_status()

    def send_process_signal(self, process_id, signal):
        """
        Sends the given signal to the process
        """
        process = self._get_process(process_id)
        assert isinstance(signal, int)

        logger.info('SchedulerService: sending signal %s to process %s' % (signal, process))
        return process.set_signal(signal)


class SchedulerException(xmlrpclib.Fault):
    def __init__(self, error_str):
        super(SchedulerException, self).__init__(-1, error_str)

class TaskNotEnabledException(SchedulerException): pass
class NoSuchTaskException(SchedulerException): pass
class NoSuchProcessException(SchedulerException): pass
class DuplicateTaskException(SchedulerException): pass
class MaximumTaskQueueException(SchedulerException): pass
