# Copyright (c) 2008 BaseSoft B.V.
# See LICENSE for details.

"""
This is the Controller for the Account Application
"""
from basesoft.web.contrib.license.controllers import LicenseController

# import used python libraries
import random, re
from hashlib import sha256 as sha

# import sqlalchemy stuff
from sqlalchemy import and_, func, distinct
from sqlalchemy.orm import  join

# import django stuff
from django.utils.translation import ugettext as _
from django.utils import simplejson
from django.conf import settings
from django.core.exceptions import ValidationError

# import exception
from basesoft.web.core.exceptions import PublicUserException, SecurityException, PublicSecurityException

# import controller stuff.
from basesoft.web.core.util.controllers import AbstractController, transaction_decorator, StateManager,\
    LicenseConfigController
from basesoft.web.core.jsonrpc.dispatcher import json_object_hook, JsonRpcJSONEncoder

# import the required SQLAlchemy models
from basesoft.web.contrib.contact.models import Person
from basesoft.web.contrib.contact.controllers import SimpleContactController
from basesoft.web.contrib.license.models import Group, License, Domain
from basesoft.web.contrib.account.models import AccountActivation, GroupAccount, Account, AccountCustomQueryPreset,\
    AccountActivationMail, LostPasswordMail
from basesoft.web.contrib.dashboard.models import DashboardAccount, Dashboard
from basesoft.web.contrib.branding.controllers import SiteController
from basesoft.web.contrib.mailing.models import MailGroup
from basesoft.web.core.input.validation.plugins import Validation

class UniqueEmailAddress(Validation):
    def check(self, params={"db_session": None}):
        if params['db_session'] is None:
            from basesoft.web.core.util.db import Session
            db_session = Session()
        else:
            db_session = params['db_session']

        a = db_session.query(Account).join(Person).filter(Person.email==self._value).first()
        if a is not None:
            raise ValidationError(_('An account is already registered for the provided email address.'))

class AccountController(SimpleContactController):
    """
    """
    CHECK_ACTIVE_ACCOUNT = False
    logger_name = 'basesoft.account'

    def logout(self, request):
        """
        Logout the current user
        @param requst: the request object
        @return: the modified request
        """
        if request.session.has_key('user_id'):
            uid = request.session['user_id']
        else:
            uid = None

        # TODO: shouldnt this just remove all in session? -see #3164
        for key in ('user_id', 'username', 'group_id', 'config', 'keep_logged_in', 'clm_locales',
                    'impersonator_user_id', 'impersonator_keep_logged_in'):
            try:
                del request.session[key]
            except KeyError:
                pass

        # set this in the session so we can update the session table entries with the logout date and also set back the
        # account_id column, otherwise it will be null and the login overview won't make the correct link between
        # account and the expired session
        request.session['logged_out'] = uid

        return request

    def login_as(self, request, current_username, username, check_ownership=True):
        """
        Log in as username, with checks to see if the current_username is allowed to do this
        @param request: the request object
        @param current_username: the username of the user that impersonates another user
        @param username: the username of the account we need to log in as
        @param check_ownership: Boolean. if True, checks are made to see if current_username is allowed to switch to
            username. The only time this is False is when after log out we switch back to the admin user; this is needed
            because the user we switched to doesn't necessarily need to have any permissions or visibility of the super
            users that switched to it
        """

        if check_ownership:
            self.assert_permissions('switch_account')
            self.assert_any_permissions('view_domain', 'view_license', 'view_all_domains')

        impersonated_account = self.validate_account_no_password(name=username) # validate the impersonated account

        if impersonated_account == False:
            return False

        if check_ownership:
            # check if the current_username is allowed to impersonate username
            accounts = self.get_accounts(impersonated_account.id)
            if len(accounts) == 0:
                return False

        return impersonated_account

    @transaction_decorator
    def remove_account(self, _id):
        self.assert_permissions('delete_account')

        if _id == self.get_account().id:
            raise PublicSecurityException(_('Cannot delete your own account'))

        account = self.get_accounts(_id)
        if len(account) == 0:
            raise PublicSecurityException(_('No access to the provided account'))

        account = account[0]
        self.logger.warn("Removing account %s." % account.id)
        self.sa_session.delete(account)
        return True

    def get_accounts(self, id=None, polymorphic_type=Account):
        """
        Return a list with Accounts.
        """
        # with view all domains, can get all accounts from all domains
        if self.has_permissions('view_all_domains'):
            if id is None:
                return self.sa_session.query(polymorphic_type).all()
            if not isinstance(id, list):
                id = [id]
            return self.sa_session.query(polymorphic_type).filter(polymorphic_type.id.in_(id)).all()

        current_license = self._get_default_group().license

        # TODO: if current license is reseller, we should use webshop controller

        # with view_domain, can get accounts from current domain
        if self.has_permissions('view_domain'):
            filter = License.domain_id==current_license.domain.id
            if id is not None:
                if not isinstance(id, list):
                    id = [id]
                filter = and_(filter, polymorphic_type.id.in_(id))
            fo = join(polymorphic_type, GroupAccount, polymorphic_type.id==GroupAccount.account_id).join(
                 Group, GroupAccount.group_id==Group.id).join(
                 License, Group.license_id==License.id)
            return self.sa_session.query(polymorphic_type).select_from(fo).filter(filter).all()

        # getting accounts it's only possible with view_license permission
        self.assert_permissions('view_license')
        filter = Group.license==current_license
        if id is not None:
            if not isinstance(id, list):
                id = [id]
            filter = and_(filter, polymorphic_type.id.in_(id))
        fo = join(polymorphic_type, GroupAccount, polymorphic_type.id==GroupAccount.account_id).join(
             Group, GroupAccount.group_id==Group.id)
        return self.sa_session.query(polymorphic_type).select_from(fo).filter(filter).all()

    def get_account_by(self, account_name=None, email=None):
        """
        Can be expanded and maybe add some more flexible way of filtering.
        """
        if account_name is None and email is None:
            raise PublicUserException('No account name or email was provided')

        q = self.sa_session.query(Account)
        filter = None

        if account_name is not None:
            filter = Account.name==account_name

        if email is not None:
            q = q.join(Person)
            if filter is None:
                filter = Person.email==email
            else:
                filter = and_(filter,Person.email==email)

        result = q.filter(filter).all()
        if len(result) != 1:
            raise PublicUserException('Incorrect name / email combination provided')
        return result.pop()

    def get_account_state(self, id):
        """
        This returns the account state
        """
        if isinstance(id, Account):
            account = id
        else:
            account = self.get_accounts(id).pop()
        rv = {
            'account': {
                'account_name': account.name,
                'groups': [g.group.id for g in account.groups],
                'default_group': account.default_group.id
            },
            'person': {'address_data': {}}
        }
        if account.person is not None:
            rv['person'] = self._load_person_state(account.person.id)
        return rv

    def get_groups_for_account(self, account, group_id=None):
        """
        """
        assert isinstance(account, Account)

        filter = GroupAccount.account==account
        if group_id is not None:
            if not isinstance(group_id, list):
                group_id = [group_id]
            filter = and_(filter, GroupAccount.group_id.in_(group_id))

        return self.sa_session.query(GroupAccount).filter(filter).all()

    def save_groups_for_account(self, account, groups, default_group, default_group_active=True):
        """
        Save account groups. It also checks for account dashboards and 'moves' them accordingly.
        """
        groups = [int(id) for id in groups]
        default_group = int(default_group)
        assert default_group in groups
        if not isinstance(account, Account):
            a = self.get_accounts(account).pop()
            account = a
        lc = LicenseController(self.sa_session, account_id=self.get_account().id)
        allowed = [g.id for g in lc.get_groups()]

        current = []
        new = []
        to_delete = []
        dg = None

        for g in account.groups:
            if g.group.id not in groups:
                to_delete.append(g)
            else:
                current.append(g.group.id)
                if g.group.id == default_group and not g.is_default:
                    g.is_default = True
                    self.sa_session.add(g)
                    dg = g.group
                if g.group.id != default_group and g.is_default:
                    g.is_default = False
                    self.sa_session.add(g)

        for group_id in groups:
            assert group_id in allowed
            if group_id not in current:
                new.append(group_id)

        groups = lc.get_groups(new)
        for g in groups:
            ga = GroupAccount()
            ga.group = g
            ga.account = account
            if g.id == default_group:
                ga.is_default = True
                ga.active = default_group_active
                dg = g
            else:
                ga.active = False
            self.sa_session.add(ga)

        to_delete_groups = [ga.group for ga in to_delete]

        if len(to_delete) > 0:
            for da in account.dashboards:
                dashboard = da.dashboard
                dashboard_groups = []
                for gg in dashboard.groups:
                    if gg.group not in dashboard_groups:
                        dashboard_groups.append(gg.group)

                # if none of the dashboards groups were deleted, do nothing
                in_deleted = False
                for gg in dashboard_groups:
                    if gg in to_delete_groups:
                        in_deleted = True
                        break
                if not in_deleted:
                    continue

                d_accs = self.sa_session.query(DashboardAccount).filter(and_(
                         DashboardAccount.dashboard==dashboard, DashboardAccount.account!=account)).all()
                # if this dashboard is not linked to other accounts also, just revoke privileges for the dashboard's
                # deleted groups and add privs for account's default group (if dashboard does not already have privs
                # on one of accounts groups)
                if len(d_accs) == 0:
                    for gg in dashboard_groups:
                        if gg in to_delete_groups:
                            self.revoke_privilege_from(dashboard, Dashboard.all_privileges, group=True, target_group=gg,
                                                       check_own_privs=False)
                    self.grant_privilege_to(dashboard, Dashboard.all_privileges, group=True, target_group=dg,
                                            check_own_privs=False)
                    continue
                # if the dashboard is linked to other accounts, then we revoke only for the groups that are not linked to
                # any of the other dashboard's accounts and we grant for the account's default group
                keep = []
                for d_acc in d_accs:
                    ac_grs = [gr.group for gr in d_acc.account.groups]
                    for gr in ac_grs:
                        if gr in to_delete_groups:
                            keep.append(gr)
                for gr in to_delete_groups:
                    if gr not in keep:
                        self.revoke_privilege_from(dashboard, Dashboard.all_privileges, group=True, target_group=gr,
                                                   check_own_privs=False)
                self.grant_privilege_to(dashboard, Dashboard.all_privileges, group=True, target_group=dg, check_own_privs=False)

        for g in to_delete:
            self.sa_session.delete(g)

        self.sa_session.flush()

    def get_account_activation(self, account,
                               type=AccountActivation.ACTIVATION_TYPE_NEW_ACCOUNT):
        """
        Get the account activation entry for the provided account.
        """
        assert isinstance(account,Account)

        aa = self.sa_session.query(AccountActivation).filter(
                 and_(AccountActivation.account==account,
                      AccountActivation.activation_type==type,
                      AccountActivation.activation_stage!=AccountActivation.ACTIVATION_STAGE_USED)).all()
        for a in aa:
            if not a.activation_key_expired():
                return a
        return None

    def get_account_activation_by_key(self, key,
                                      type=AccountActivation.ACTIVATION_TYPE_NEW_ACCOUNT):
        """
        Gets the AccountActivation object by key and type.
        """
        aa = self.sa_session.query(AccountActivation).filter(
                 and_(AccountActivation.activation_key==key,
                      AccountActivation.activation_type==type))
        if aa.count() == 0:
            raise PublicUserException(_('Incorrect activation key'))
        return aa.one()

    @transaction_decorator
    def save_or_update_group_account(self, group, account,
                                     is_default=False, is_active=False):
        """
        Saves or updates the group account relation. This should only be used for new accounts for which other checks
        (like dashboards) are not required.
        @param group: C{Group}
        @param account: C{Account}
        @param is_default: C{boolean}
        @param is_active: C{boolean}
        """
        # Check if the relation already exists.
        gas = self.sa_session.query(GroupAccount).filter(
                  and_(GroupAccount.group == group,
                       GroupAccount.account == account)).all()
        # If so, use it.
        if len(gas) == 1:
            ga = gas.pop()
        else:
            ga = GroupAccount()
            ga.group = group
            ga.account = account

        # set the changeable properties.
        ga.is_default = is_default
        ga.active = is_active
        self.sa_session.add(ga)
        return ga

    @transaction_decorator
    def create_account_from_license_contact(self, license, activate_account=False):
        """
        We use the contact person on the provided license to create an inactive
        account, with a random account name.
        """
        self.assert_permissions('create_account')

        assert isinstance(license, License)
        assert isinstance(license.default_group, Group)

        if license.company is None:
            raise PublicUserException(_('No contact information found'))

        account_name = self.generate_account_name_from_person(license.company.contact_person)
        password = self.generate_random_password()
        account = self.create_account(account_name, password, license.company.contact_person)

        self.save_or_update_group_account(license.default_group, account, is_default=True)
        if activate_account:
            # we need an entry in AccountActivation here to send
            # the newly created account an email so they can change their password.
            self.save_account_activation(account,
                AccountActivation.ACTIVATION_TYPE_NEW_ACCOUNT_FROM_LICENSE)

        return account

    @transaction_decorator
    def save_account_from_state(self, state, account_id=None, license_controller=None, polymorphic_type=None):
        """
        @param license_controller: C{LicenseController} or C{ResellerController}
        """
        assert isinstance(state, dict)
        assert isinstance(state['account'], dict)
        assert isinstance(state['person'], dict)
        assert isinstance(state['account']['groups'], list)
        assert state['account'].has_key('default_group') and state['account']['default_group'] not in ['', None]

        person_id = None
        account = None

        if account_id is not None:
            self.assert_permissions('update_account')
            account = self.get_accounts(account_id).pop()
            if account.person is not None:
                person_id = account.person.id
            if polymorphic_type is not None:
                assert isinstance(account, polymorphic_type)
        else:
            self.assert_permissions('create_account')

        if license_controller is None:
            from basesoft.web.contrib.license.controllers import LicenseController
            license_controller = LicenseController(self.sa_session, account_id=self._get_account().id)

        ac = state['account']
        person = self._save_or_update_person(person_id, state['person'])

        if account is None:
            password = self.generate_random_password()
            account = self.create_account(ac['account_name'], password, person, polymorphic_type=polymorphic_type)
        else:
            account.name = ac['account_name']
            self.sa_session.add(account)

        # if the account is new, it will be inactive for the default group
        dga = False
        if account_id is not None:
            for ga in account.groups:
                # account will be active for default group only if before update it was also active
                if ga.is_default and ga.active:
                    dga = True

        self.save_groups_for_account(account, state['account']['groups'], state['account']['default_group'], dga)
        return account

    @transaction_decorator
    def create_inactive_account(self, name, password, email, reseller=None):
        """
        This creates an Inactive account with the whole license tree.
         * Account
         * Group
         * License
         * Domain
        """

        if self._email_address_exists(email):
            # TODO: schedule duplicate registration email. (?)
            raise PublicUserException(_('Email address already used for a registration'))

        if not self.is_unique_account_name(name):
            raise PublicUserException('Account name already exists, %s' % name)

        # use the simple contact controller to save the contact person info.
        contact = self._save_or_update_person(None, {'email': email})

        # get domain based on the current site
        sc = SiteController(self.sa_session, request=self.request)
        domain = sc.get_current_site().domain

        if domain is None:
            self.logger.warn("Creating account with new domain")
            domain = Domain()
            domain.name = name
        else:
            domain.is_active()

        license = License()
        license.domain = domain
        license.name = name
        license.contact_person = contact

        group = Group()
        group.license = license
        group.is_default = True
        group.name = Group.DEFAULT_GROUP_NAME

        # create the account
        account = self.create_account(name, password, contact)
        # link the account to the group and set it as default.
        self.save_or_update_group_account(group, account, is_default=True)

        # add an entry to the AccountActivation table.
        self.save_account_activation(account,
            AccountActivation.ACTIVATION_TYPE_NEW_ACCOUNT)

        return account


    @transaction_decorator
    def reset_account_activation(self, account):
        """
        Resets the account activation for the given account if possible.
        """
        assert isinstance(account, Account)

        # try to de-activate all groups for account
        group_accounts = self.get_groups_for_account(account)
        for ga in group_accounts:

            # check if any of the groups has a license with company attached, then its a nogo!
            if ga.group.license.company is not None:
                raise PublicUserException(_('Cannot reset account %s: Company details exist' % account.name))

            self.save_or_update_group_account(ga.group, account, ga.is_default, is_active=False)

        # remove any existing account activation
        aa = self.get_account_activation(account)
        if aa is not None:
            self.sa_session.delete(aa)

        # add an entry to the AccountActivation table.
        self.save_account_activation(account,
            AccountActivation.ACTIVATION_TYPE_NEW_ACCOUNT)


    def __check_activation_key_availability(self, aa):
        """
        Private method that checks and logs the activation key availability.
        """
        activation_key = aa.activation_key

        if aa.activation_key_expired():
            self.logger.info('Activation: expired activation key: %s' % activation_key)
            raise PublicUserException(_('Expired activation key'))
        if aa.activation_stage == AccountActivation.ACTIVATION_STAGE_NEW:
            self.logger.info('Activation: Trying to access an activation key that has\'t been sent: %s' % activation_key)
            # this is a security exception, how did the user get the activation key when it wasn't even sent?
            raise SecurityException('Trying to access an activation key that has\'t been sent')
        if aa.activation_stage == AccountActivation.ACTIVATION_STAGE_SENDING:
            self.logger.info('Activation: Trying to access an activation key that is currently being sent: %s' % activation_key)
            # this is a security exception, how did the user get the activation key when it wasn't even sent?
            raise SecurityException('Trying to access an activation_key that is currently being sent')
        if aa.activation_stage == AccountActivation.ACTIVATION_STAGE_USED:
            self.logger.info('Activation: The activation key was used and is no longer valid: %s' % activation_key)
            raise PublicUserException(_('The activation key was already used and it is no longer valid'))


    def __validate_key(self, activation_key):
        # Make sure the key we're trying conforms to the pattern of a
        # SHA1 hash; if it doesn't, no point trying to look it up in
        # the database.
        SHA1_RE = re.compile('^[a-f0-9]{8}$')
        if not SHA1_RE.search(activation_key):
            self.logger.info('Registration: invalid activation key: %s' % activation_key)
            raise PublicUserException(_('Incorrect activation key'))

        # get the account activation object
        aa = self.get_account_activation_by_key(activation_key,
                 AccountActivation.ACTIVATION_TYPE_NEW_ACCOUNT)

        self.__check_activation_key_availability(aa)
        return aa

    @transaction_decorator
    def save_person_info(self, activation_key, state):

        aa = self.__validate_key(activation_key)

        self._save_or_update_person(aa.account.person.id, state)

        return True

    def can_active(self, activation_key):
        aa = self.__validate_key(activation_key)

        if not aa.account.default_group.license.domain.has_setting(Domain.SETTING_REQUIRE_PERSON_DETAILS_ON_ACCOUNT_ACTIVATION):
            return True

        if aa.account.person.last_name is None:
            return self._load_person_state(aa.account.person)
        return True

    @transaction_decorator
    def activate_account(self, activation_key):
        """
        Validates an activation key and activates the corresponding
        ``Account`` if valid.

        If the key is valid and has not expired, returns the ``Account``
        after activating.

        If the key is not valid or has expired, raises exception.

        If the key is valid but the ``Account`` is already active,
        raises exception.
        """

        aa = self.__validate_key(activation_key)
        if self.can_active(activation_key) != True:
            raise PublicUserException(_('Please provide all person details.'))

        ga = self.__set_account_active(aa)

        # copy permissions from role if set on the domain
        role = ga.group.license.domain.registrant_role
        if role is not None:
            ga.group.license.permissions = role.permissions
            ga.group.permissions = role.permissions
            self.sa_session.add(ga.group)

            if (len(role.license_configs)):
                lc = LicenseConfigController(self.sa_session, scope=ga.group.license)
                for license_config in role.license_configs:
                    lc.set(license_config.config.name, license_config.value)

            if role.dashboard is not None:
                from basesoft.web.contrib.dashboard.controllers import DashboardStateController
                manager = StateManager(None, aa.account.id, None)
                dbsc = DashboardStateController(self.sa_session, manager)
                dbsc.clone(role.dashboard, role.dashboard.name, aa.account, make_default=True)

        # mark the account activation as used
        aa.activation_stage = AccountActivation.ACTIVATION_STAGE_USED
        self.sa_session.add(aa)

        self.logger.info("activate_account: activated account [%s] " % aa.account.id)

        return ga.account

    def __set_account_active(self, activation):
        """
        Set an account as active for the default group. This method is called when activating a new account
        and also when activating an account created from a license contact
        """
        account = activation.account
        activation_key = activation.activation_key
        group_accounts = self.get_groups_for_account(account)
        if len(group_accounts) != 1:
            for g in group_accounts:
                if g.is_default:
                    group_accounts = [g]
                    break
            if len(group_accounts) != 1:
                self.logger.info('Registration: could not determine default group for key: %s' \
                        % activation_key)
                raise PublicUserException(_('Unable to determine default group'))

        group_account = group_accounts.pop()
        if group_account.active == True:
            self.logger.info('Registration: account is already active for activation key: %s' \
                        % activation_key)
            raise PublicUserException(_('Account is already active'))

        return self.save_or_update_group_account(group_account.group,
                                          group_account.account,
                                          # save_or_update_group_account has the default value of is_default False.
                                          # So everytime we make an account active it will automaticly make it is_default
                                          # false if I don't add this.
                                          is_default=group_account.is_default,
                                          is_active=True)

    def validate_account(self, name, password):
        """
        Checks if username and password match and the account is active for at least one group
        @param name: the username of the account that needs to be validated
        @param password: the password of the account
        @return: the Account object for the username if it's valid, or False if invalid
        """

        hash_password = self.hash_password(password)
        q = self.sa_session.query(Account).join(GroupAccount).filter(
                 and_(Account.name==name,
                      Account.password==hash_password,
                      GroupAccount.active==True))

        if q.count() <= 0:
            self.logger.info('no match found for name=%s, password=%s' % (name, password))
            return False

        account = q.first()
        active = False
        for ag in account.groups:
            if ag.group.license.is_active:
                active = True
                break

        if active:
            return account

        self.logger.info('no activate licenses found for name=%s' % (name))
        raise PublicUserException(_('No active licenses found for this account.'))

    def validate_account_no_password(self, name):
        """
        Checks if username and password match and the account is active for at least one group
        @param name: the username of the account that needs to be validated
        @return: the Account object for the username if it's valid, or False if invalid
        """

        q = self.sa_session.query(GroupAccount).join(GroupAccount.account).filter(
            and_(Account.name==name, GroupAccount.active==True))

        if q.count() <= 0:
            self.logger.info('no active match found for name=%s' % name)
            return False

        return q.first().account

    @transaction_decorator
    def forgot_credentials(self, account_name=None, email=None):
        """
        Sets an activation_key for changing the password.
        """
        account = self.get_account_by(account_name=account_name, email=email)

        group_accounts = self.get_groups_for_account(account)
        active = [ga for ga in group_accounts if ga.active]

        # not active for any group then we don't continue.
        if len(active)==0:
            raise PublicUserException(_('Requesting lost password for a non active account'))

        self.save_account_activation(account,
                 AccountActivation.ACTIVATION_TYPE_CHANGE_PASSWORD)
        return account

    @transaction_decorator
    def activate_change_password(self, activation_key, name, new_password):
        """
        Change password for an account which has activation_key and username match.
        Checks the activation key type: if it is for a new account created from a license contact,
        it will also activate the account before changing the password, else it checks if the account
        is active for a certain group and raises an exception otherwise
        """
        # Make sure the key we're trying conforms to the pattern of a
        # SHA1 hash; if it doesn't, no point trying to look it up in
        # the database.
        SHA1_RE = re.compile('^[a-f0-9]{8}$')
        if not SHA1_RE.search(activation_key):
            raise PublicUserException(_('Invalid activation key for new password'))

        # get the account activation object
        aa = self.sa_session.query(AccountActivation).filter(
                 AccountActivation.activation_key==activation_key).one()

        if aa.activation_type == AccountActivation.ACTIVATION_TYPE_NEW_ACCOUNT:
            self.logger.info('Change Credentials: used an activation key of type new account: %s' % activation_key)
            raise PublicUserException(_('Incorrect activation key'))

        if aa.account.name != name:
            raise PublicUserException(_('Incorrect account name supplied'))

        self.__check_activation_key_availability(aa)

        if aa.activation_type == AccountActivation.ACTIVATION_TYPE_CHANGE_PASSWORD:
            account = aa.account
            group_accounts = self.get_groups_for_account(account)
            active = [ga for ga in group_accounts if ga.active]

            # not active for any group then we don't continue.
            if len(active)==0:
                raise PublicUserException(_('Not an active account'))

            # new password cannot be the same as the old one
            if self.hash_password(new_password) == account.password:
                raise PublicUserException(_('New password cannot be the same as the old one'))

        if aa.activation_type == AccountActivation.ACTIVATION_TYPE_NEW_ACCOUNT_FROM_LICENSE:
            # need to activate the account first
            account = self.__set_account_active(aa).account

        hash_password = self.hash_password(new_password)
        account.password = hash_password
        self.sa_session.add(account)

        # mark the account activation as used
        aa.activation_stage = AccountActivation.ACTIVATION_STAGE_USED
        self.sa_session.add(aa)

        return aa.account

    @transaction_decorator
    def save_account_activation(self, account, activation_type=None):
        """
        Creates a ``AccountActivation`` profile for a given
        ``Account``. Returns the ``AccountActivation``.

        The activation key for the ``AccountActivation`` will be a
        SHA1 hash, generated from a combination of the ``Account``'s
        name and a random salt.
        @param type: for what is the activation key (for a new account or for
                                                     a forgot password)
        @type type:SmallInteger, 0-NEW_ACCOUNT, 1-CHANGE_PASSWORD
        """
        if activation_type is None:
            activation_type = AccountActivation.ACTIVATION_TYPE_NEW_ACCOUNT

        salt = sha(str(random.random())).hexdigest()[:5]
        activation_key = sha(salt + account.name).hexdigest()[:8]
        aa = AccountActivation()
        aa.activation_key = activation_key
        aa.activation_type = activation_type
        aa.account = account
        self.sa_session.add(aa)
        return aa

    def _email_address_exists(self, email):
        """
        Checks if email address was already used for another account
        @protected
        """
        q = self.sa_session.query(Account).join(Person).filter(
                 and_(Person.email == email))
        return (q.count() > 0)

    @transaction_decorator
    def update_last_login(self, account):
        """
        Updates the last login date/time for an account
        @param account: L{Account}
        """
        assert isinstance(account, Account)
        import datetime

        account.last_login = datetime.datetime.now()
        self.sa_session.add(account)

    @transaction_decorator
    def create_account(self, name, password, person=None, locale=None, polymorphic_type=None):
        """
        @param name: {String}
        @param password: {String} hashed password
        @param person: {Person} or None
        @param locale: {Locale} or None
        """
        if not self.is_unique_account_name(name):
            raise PublicUserException('Account name already exists, %s' % name)
        if polymorphic_type is None:
            account = Account()
        else:
            account = polymorphic_type()

        account.name = name
        account.password = self.hash_password(password)
        account.person = person
        account.locale = locale
        self.sa_session.add(account)
        return account

    def is_unique_account_name(self, name):
        """
        Check to see whether or not the provided name is already in use by
        another account.
        """
        result = self.sa_session.query(Account).filter(Account.name == name).all()
        return len(result) == 0

    def generate_account_name_from_person(self, person):
        """
        This takes the last name of the person and makes it 10 characters long,
        whereby up to the first 6 characters of the persons last name are used
        and the rest is filled with random numerical entries.
        @param person: C{Person}
        @return account_name: C{str}
        """
        import math
        name = person.last_name[:6]
        addition = 10 - len(name)

        def randomize(number):
            min = math.pow(10, number - 1)
            max = math.pow(10, number) - 1
            return random.randrange(min, max)

        account_name = name + '%s' % randomize(addition)

        dupe = True
        while dupe:
            if self.is_unique_account_name(account_name):
                dupe = False
                break
            account_name = name + '%s' % randomize(addition)
        return account_name

    def hash_password(self, password):
        """
        Hash the given password
        @param password: raw password
        @type password: C{String}
        """
        return sha(password.encode('utf-8')).hexdigest()

    def generate_random_password(self, length=8):
        """
        TODO: implement (#385)
        """
        return 'randompasswdstring'

    def get_account_activation_mail(self, group):
        return self.__get_mail(AccountActivationMail, group)

    def get_lost_password_mail(self, group):
        return self.__get_mail(LostPasswordMail, group)

    def __get_mail(self, _type, group):
        """
        Get the correct mail for the group (#3167):
         * Check if a mail is available on the license level
         * Check if a mail is available on the domain level
         * Check if a mail is available on reseller level (if license.reseller)
         * Get the mail on BaseSoft level (!)
        """
        assert _type in [AccountActivationMail, LostPasswordMail]

        license = group.license
        domain = license.domain
        reseller = domain.reseller
        site_owner = self.sa_session.query(License).filter(License.name==settings.SITE_OWNER_LICENSE_NAME).one()

        domain_mail = None
        reseller_mail = None
        site_owner_mail = None

        filter, fo = self.get_query_filter_on_privileges(_type, MailGroup, priv=_type.PRIV_USE_FOR_SENDING, groups=[group])
        mails = self.sa_session.query(_type).select_from(fo).filter(filter).all()
        for mail in mails:
            for g in mail.groups:
                if g.privilege!=_type.PRIV_USE_FOR_SENDING:
                    continue
                if g.group.license == license:
                    return mail
                if g.group.license.domain == domain and domain_mail is None:
                    domain_mail = mail
                    continue
                if g.group.license == reseller and reseller_mail is None:
                    reseller_mail = mail
                    continue
                if g.group.license == site_owner and site_owner_mail is None:
                    site_owner_mail = mail

        if domain_mail is not None:
            return domain_mail
        if reseller_mail is not None:
            return reseller_mail
        if site_owner_mail is not None:
            return site_owner_mail

        return None

class AccountCustomQueryPresetController(AbstractController):

    def get_presets(self, id=None):
        return self.get_presets_by('id', id)

    def get_presets_by(self, field=None, values=None):
        self.assert_permissions('set_custom_query_preset')
        account = self.get_account()
        filter = AccountCustomQueryPreset.account==account
        if field is not None:
            assert field in ['label', 'id', 'custom_query']
            if values is not None:
                if not isinstance(values, list):
                    values = [values]
                filter = and_(filter, getattr(AccountCustomQueryPreset, field).in_(values))
        return self.sa_session.query(AccountCustomQueryPreset).filter(filter).all()

    @transaction_decorator
    def save_or_update_preset(self, state):
        """
        Saves custom query preset
        """
        assert state.has_key('custom_query') and state['custom_query'] not in [None, '']
        assert state.has_key('label') and state['label'] not in [None, '']

        id = None
        if state.has_key('id') and state['id'] is not None:
            id = state['id']
        if id is None:
            self.assert_permissions('set_custom_query_preset')
            preset = AccountCustomQueryPreset()
            preset.account = self.get_account()
            preset.custom_query = state['custom_query']
        else:
            preset = self.get_presets(id)[0]
            if preset.custom_query == state['custom_query'] and not preset.is_default and state['is_default']:
                self._unset_default_preset(state['custom_query'])
        preset.label = state['label']
        preset.is_default = state['is_default']

        preset.tables_state = simplejson.dumps(state['tables_state'], ensure_ascii=False, cls=JsonRpcJSONEncoder)
        preset.filters_state = simplejson.dumps(state['filters_state'], ensure_ascii=False, cls=JsonRpcJSONEncoder)

        self.sa_session.add(preset)
        return preset

    def _unset_default_preset(self, custom_query):
        preset = self.sa_session.query(AccountCustomQueryPreset).filter(and_(
                 AccountCustomQueryPreset.account==self.get_account(), AccountCustomQueryPreset.custom_query==custom_query,
                 AccountCustomQueryPreset.is_default==True)).all()
        if len(preset) == 0:
            return
        preset = preset[0]
        preset.is_default = False
        self.sa_session.add(preset)

    def load_preset(self, id):
        """
        Load preset
        """
        if isinstance(id, AccountCustomQueryPreset):
            preset = id
        else:
            preset = self.get_presets(id)[0]
        return {'id': preset.id,
                'account_id': preset.account_id,
                'label': preset.label,
                'custom_query': preset.custom_query,
                'is_default': preset.is_default,
                'tables_state': simplejson.loads(preset.tables_state, object_hook=json_object_hook),
                'filters_state': simplejson.loads(preset.filters_state, object_hook=json_object_hook)
                }
