# Copyright (c) 2010 BaseSoft B.V.
# See LICENSE for details.

from sqlalchemy import case, func, or_
from django.utils.translation import ugettext as _

from basesoft.web.core.util.custom_query import CustomQuery, CustomQueryField, CustomQueryModel, FILTER_EQUAL

from basesoft.web.contrib.account.models import Account, GroupAccount, Config, AccountConfig
from basesoft.web.contrib.contact.models import Person
from basesoft.web.contrib.license.models import License, Domain, Group, Permission, GroupPermission, PermissionCategory
from basesoft.web.contrib.account.controllers import AccountController

class AccountCustomQuery(CustomQuery):
    def validate_access(self):
        self.assert_any_permissions('view_domain', 'view_license', 'view_all_domains')

    def tables(self):
        return (
            CustomQueryModel(Account),
            CustomQueryModel(GroupAccount, onclause=GroupAccount.account_id==Account.id),
            CustomQueryModel(Group, onclause=GroupAccount.group_id==Group.id),
            CustomQueryModel(License, onclause=Group.license_id==License.id),
            CustomQueryModel(Domain, onclause=License.domain_id==Domain.id),
            CustomQueryModel(Person, onclause=Account.person_id==Person.id, use_outerjoin=True),
        )

    def fields(self):
        return (
            CustomQueryField(Account.id, _('ID'), is_filter=True),
            CustomQueryField(Account.name, _('Account'), is_filter=True),
            CustomQueryField(func.concat(case([(Person.first_name!=None, Person.first_name)], else_=""),
                                        " ",
                                        case([(Person.middle_name!=None, Person.middle_name)], else_=""),
                                        " ",
                                        case([(Person.last_name!=None, Person.last_name)], else_=""),
                                        ), _('Name'), key='person_name', is_filter=True),
            CustomQueryField(Account.name, _('Switch to user'), key='switch_to_user'),
            CustomQueryField(Account.polymorphic_type, _('Type'), in_select=False, is_filter=True),
            CustomQueryField(Person.email, _('e-Mail'), is_filter=True),
            CustomQueryField(GroupAccount.active, _('Active'), key='active'),
            CustomQueryField(GroupAccount.is_default, _('Is default group'), key='is_default'),
            CustomQueryField(Group.id, _('Group ID'),  is_filter=True, filter_options=Group.name),
            CustomQueryField(Group.name, _('Group'), key='group_name'),
            CustomQueryField(Account.id, _('Permissions'), key='permissions'),
            CustomQueryField(License.name, _('License'), is_group_by=True),
            CustomQueryField(License.polymorphic_type, _('Type'), is_filter=True),
            CustomQueryField(License.id, _('License'), key="license_id", in_select=False, is_filter=True, filter_options=License.name),
            CustomQueryField(Domain.name, _('Domain'), is_filter=True, is_group_by=True),
            CustomQueryField(Account.last_login, _('Last login')),
            CustomQueryField(Account.id, _('Config'), key='account_config'),
        )

    def set_where(self):

        if self.has_permissions('view_all_domains'):
            return None
        dg = self.get_default_group()
        if self.has_permissions('view_domain'):
            # filter on current domain
            self.where = Domain.id==dg.license.domain.id
            return

        licenses = []
        for g in self.get_active_groups():
            if self.has_permissions('view_license', group_id=g.id):
                licenses.append(g.license.id)

        if len(licenses) > 0:
            self.where = License.id.in_(licenses)


class AccountConfigModel(CustomQueryModel):

    filter_account_id = None

    def __init__(self, *args, **kwargs):
        CustomQueryModel.__init__(self, Config, *args, **kwargs)

    def process(self, custom_query, fo, f, initial):

        from sqlalchemy.orm import outerjoin
        from sqlalchemy.sql.expression import and_, or_

        fo, f, initial = CustomQueryModel.process(self, custom_query, fo, f, initial)
        fo = outerjoin(fo, AccountConfig, onclause=(
            and_(Config.id == AccountConfig.config_id,
                or_(AccountConfig.account_id == self.filter_account_id, AccountConfig.account_id==None)))
        )
        fo = outerjoin(fo, Account, onclause=AccountConfig.account_id==Account.id)

        return fo, f, initial


class AccountConfigCustomQuery(CustomQuery):

    def __init__(self, *args, **kwargs):
        self._table = AccountConfigModel()
        CustomQuery.__init__(self, *args, **kwargs)

    def _apply_filters(self, filters, static=False):
        if filters.has_key('account_id'):
            self._table.filter_account_id = filters['account_id']['values'][0]
            del filters['account_id']

        return CustomQuery._apply_filters(self, filters, static)

    def tables(self):
        return (
            self._table,
        )

    def fields(self):
        return (
            CustomQueryField(Config.id, _('ID'), key='id'),
            CustomQueryField(Account.name, _('Account'), key='account', is_group_by=True),
            CustomQueryField(AccountConfig.account_id, _('Account ID'), key='account_id', in_select=False, filter_is_unique=True, is_filter=True, filter_options=[{'value': p.id, 'label': p.name} for p in self.get_accounts()]),
            CustomQueryField(Config.name, _('Name'), key='name', is_filter=True),
            CustomQueryField(Config.type, _('Type')),
            CustomQueryField(Config.default, _('Default'), key='default'),
            CustomQueryField(case([(AccountConfig.value==None,  Config.default)],  else_=AccountConfig.value), _('Value'), key='value'),
        )

    def get_accounts(self):
        ac = AccountController(self.sa_session, account_id=self.get_account().id)
        return ac.get_accounts()

    order_by = (Config.name, )

class AccountPermissionsCustomQuery(CustomQuery):

    def tables(self):
        return (
            CustomQueryModel(Permission),
            CustomQueryModel(PermissionCategory, onclause=Permission.category_id==PermissionCategory.id),
            CustomQueryModel(GroupPermission, onclause=GroupPermission.permission_id==Permission.id),
            CustomQueryModel(GroupAccount, onclause=GroupPermission.group_id==GroupAccount.group_id),
            CustomQueryModel(Group, onclause=GroupAccount.group_id==Group.id),
            CustomQueryModel(License, onclause=Group.license_id==License.id),
            CustomQueryModel(Domain, onclause=License.domain_id==Domain.id),
        )

    def fields(self):
        return (
            CustomQueryField(GroupAccount.account_id, _('Account ID'), key='account_id'),
            CustomQueryField(Permission.id, _('Permission ID'), key='permission_id'),
            CustomQueryField(PermissionCategory.name, _('Category'), is_filter=True, is_group_by=True),
            CustomQueryField(Permission.title, _('Permission'), is_filter=True),
            CustomQueryField(func.group_concat(' ', Group.name), _('Groups'), key='groups'),
        )

    def set_where(self):

        if self.has_permissions('view_all_domains'):
            return None
        dg = self.get_default_group()
        if self.has_permissions('view_domain'):
            # filter on current domain
            self.where = Domain.id==dg.license.domain.id
            return

        licenses = []
        for g in self.get_active_groups():
            if self.has_permissions('view_license', group_id=g.id):
                licenses.append(g.license.id)

        if len(licenses) > 0:
            self.where = or_(License.id.in_(licenses), GroupAccount.account_id==self.get_account().id)
        else:
            self.where = GroupAccount.account_id==self.get_account().id
    group_by = [Permission.id, ]

custom_queries = (AccountCustomQuery, AccountConfigCustomQuery, AccountPermissionsCustomQuery)
