# Copyright (c) 2010 BaseSoft B.V.
# See LICENSE for details.

from basesoft.web.contrib.account.controllers import AccountController, AccountCustomQueryPresetController,\
    UniqueEmailAddress
from django.conf import settings
from basesoft.web.core.exceptions import PublicUserException, PublicOutDatedSourcesException
from basesoft.web.contrib.account.models import Account

from django.utils.translation import ugettext as _
from basesoft.web.core.forms.handler import FormValidator
from basesoft.web.core.input.validation import plugins
from basesoft.web.contrib.account import EULA
from basesoft.web.contrib.wiki.parser import Parser

class RandomPasswordRPCHandler:
    def __init__(self, request, **kwargs):
        self.sa_session = request.Session()

    def get_password(self):
        """
        Generate a password that's valid by craclikb.FascistCheck standards
        """
        from basesoft.util.password import generate_password, password_pronunciation, is_password_fascist_valid

        password = generate_password(12, 12, use_punctuation=True)
        while not is_password_fascist_valid(password):
            password = generate_password()

        return {
            'password': password,
            'pronunciation': password_pronunciation(password)
        }

class RegistrationCompletionRPCHandler:
    def __init__(self, request, **kwargs):
        self.request = request

    def validate(self, values):
        form_handler = FormValidator({
            'first_name': [(plugins.IQPortalName, {}), (plugins.Required, {})],
            'last_name': [(plugins.IQPortalName, {}), (plugins.Required, {})],
            'phone': [(plugins.Required, {})],
            'gender': [(plugins.Required, {})],
            'email': [(plugins.Email, {}), (plugins.Required, {})],
            'preferred_locale': [(plugins.Required, {})],
        })
        return form_handler.validate(values)

class KeepAliveRPCHandler:
    def __init__(self, request, **kwargs):
        self.sa_session = request.Session()
        self.request = request
        self.controller = AccountController(request.Session())

    def keepalive(self, user_id, idle, version):
        """
        update the session to increase the expiry_date if necessary
        @param user_id: the id of the user that's logged (from the env)
        @param idle: int - how many idle minutes we have on the clientside
        @return: a boolean indicating whether we were logged out or not because of idle
        """

        # user is not logged in, no need to check the session
        if not user_id:
            return False

        # check version and notify user trough an exception
        if version != settings.VERSION:
            raise PublicOutDatedSourcesException(_('There is a new version of IQPortal available. Please save your work and refresh your browser.'))

        # the client thinks it's logged (there was a user in the env), so if we don't have a user_id in the session
        # it means the user was logged out
        if not self.request.session.has_key('user_id'):
            return True

        account = self.sa_session.query(Account).filter(Account.id==self.request.session['user_id']).one()

        try:
            account.default_group.license.domain.is_active()
        except PublicUserException:
            # TODO: give the user a warning ahead of time in case of a de-activation for updates.. See ticket #1812
            # logout the user, their domain is not active.
            self.controller.logout(self.request)
            self.request.session.save()
            raise

        timeout = settings.SESSION_COOKIE_AGE # timeout for expiry, in seconds
        if self.request.session.has_key('keep_logged_in') and self.request.session['keep_logged_in'] == True:
            timeout = settings.KEEP_SIGNED_COOKIE_AGE

        # we only use 75% of the timeout, to leave a margin for the client
        max_idle = int((timeout * 75 / 100) / 60)
        if idle >= max_idle:
            # we were idling for more than the session age, so we log out and we tell the clientside we're logged out
            # so a login dialog can be displayed
            self.controller.logout(self.request)
            self.request.session.save() # save the session so it's marked as logged out
            return True

        else:
            # we're still good, update the session with a counter so the client receives a new cookie
            if not self.request.session.has_key('counter'):
                self.request.session['counter'] = 0
            self.request.session['counter'] += 1

            return False

class EULAHandler:
    def __init__(self, request, **kwargs):
        self.sa_session = request.Session()
        self.request = request

    def get_eula(self):
        p = Parser()
        return p.render_to_html(EULA)

class PersonAccountHandler:
    def __init__(self, request, **kwargs):
        self.sa_session = request.Session()
        self.request = request
        self.controller = AccountController(request.Session())

    def validate(self, args):
        form_handler = FormValidator({
            'email': [(plugins.Required, {}), (UniqueEmailAddress, {'db_session': self.sa_session})]
        })
        return form_handler.validate(args)

    def validate_update(self, args):
        form_handler = FormValidator({
            'email': [(plugins.Required, {}),]
        })
        return form_handler.validate(args)

class AccountCQHandler:
        """
        Handles account CQ presets.
        """
        def __init__(self, request, **kwargs):
            sa_session = request.Session()
            user_id = request.session['user_id']
            self.controller = AccountCustomQueryPresetController(sa_session, account_id=user_id)

        def save_preset(self, state):
            preset = self.controller.save_or_update_preset(state)
            # send new state back
            return self.load_preset(preset)

        def load_preset(self, id):
            return self.controller.load_preset(id)

        def get_presets(self, cq_name, get_default=True):
            return [{'value': p.id, 'label': p.label, 'default': get_default and p.is_default or False}
                    for p in self.controller.get_presets_by('custom_query', cq_name)]

handlers = (RandomPasswordRPCHandler, KeepAliveRPCHandler, AccountCQHandler, RegistrationCompletionRPCHandler, PersonAccountHandler, EULAHandler, )
