# Copyright (c) 2008 BaseSoft B.V.
# See LICENSE for details.

"""
Contains the objects, functions and variables to setup the IQPortal database for registration app,
and to view information from the data models stored within it
"""

import datetime
from sqlalchemy.orm import mapper, relation
from sqlalchemy import Column, Integer, UnicodeText, SmallInteger, String, ForeignKey, DateTime
from basesoft.sqlalchemy.table import IQTable

from basesoft.web.contrib.license.hooks import PrivilegeObjectMapper
from basesoft.web.contrib.i18n import models as i18n_models
from basesoft.web.contrib.contact import models as contact_models
from basesoft.web.contrib.license import models as license_models
from basesoft.web.contrib.account import models as account_models
from django.utils.translation import ugettext as _

__mapping_defined = False
__tables_defined = False
tables = {}

def reset():
    global __mapping_defined
    global __tables_defined

    __mapping_defined = False
    __tables_defined = False

def get_tables():
    """
    """
    return tables

def do_tables(metadata):
    global __tables_defined
    if not __tables_defined:
        # account
        tblAccount = IQTable('tblAccount', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('polymorphic_type', String(255), nullable=False),
            Column('name', String(40), nullable=False, unique=True),
            Column('password', String(70), nullable=False),
            Column('person_id', Integer, ForeignKey('tblPerson.id')),
            Column('locale_id', Integer, ForeignKey('tblLocale.id')),
            Column('dashboard', UnicodeText, nullable=True),
            Column('last_login', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now),
        )

        tblGroupAccount = IQTable('tblGroupAccount', metadata,
            Column('group_id', Integer, ForeignKey('tblGroup.id', ondelete='CASCADE'), primary_key=True),
            Column('account_id', Integer, ForeignKey('tblAccount.id', ondelete='CASCADE'), primary_key=True),
            Column('active', SmallInteger, default=0),
            Column('is_default', SmallInteger, default=0),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now),
        )

        # activation keys for accounts
        tblAccountActivation = IQTable('tblAccountActivation', metadata,
            Column('id',Integer, primary_key=True, autoincrement=True),
            Column('account_id', Integer, ForeignKey('tblAccount.id', ondelete='CASCADE')),
            Column('activation_key', String(8), unique=True),
            Column('activation_type', SmallInteger, default=1), # See model for types
            Column('activation_stage',SmallInteger, default=0), # See model for stages
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now),
        )

        tblAccountActivationMail = IQTable('tblAccountActivationMail', metadata,
            Column('mail_id', Integer, ForeignKey('tblMail.id', ondelete='CASCADE'), primary_key=True),
            Column('fallback_locale_id', Integer, ForeignKey('tblLocale.id'), nullable=False),
        )

        tblLostPasswordMail = IQTable('tblLostPasswordMail', metadata,
            Column('mail_id', Integer, ForeignKey('tblMail.id', ondelete='CASCADE'), primary_key=True),
            Column('fallback_locale_id', Integer, ForeignKey('tblLocale.id'), nullable=False),
        )

        tblConfig = IQTable('tblConfig', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('name', String(255), nullable=False, unique=True),
            Column('description', UnicodeText, nullable=True),
            Column('type', String(40), nullable=False),
            Column('default', String(40), nullable=True),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )
        tblAccountConfig = IQTable('tblAccountConfig', metadata,
            Column('account_id', Integer, ForeignKey('tblAccount.id', ondelete='CASCADE'), primary_key=True),
            Column('config_id', Integer, ForeignKey('tblConfig.id', ondelete='CASCADE'), primary_key=True),
            Column('value', String(40), nullable=True),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        tblAccountCustomQueryPreset = IQTable('tblAccountCustomQueryPreset', metadata,
            Column('id',Integer, primary_key=True, autoincrement=True),
            Column('account_id', Integer, ForeignKey('tblAccount.id', ondelete='CASCADE'), nullable=False),
            Column('custom_query', String(255)),
            Column('label', String(100)),
            Column('is_default', SmallInteger, default=0, nullable=False),
            Column('tables_state', UnicodeText),
            Column('filters_state', UnicodeText),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now),
        )

        tables['tblAccount'] = tblAccount
        tables['tblGroupAccount'] = tblGroupAccount
        tables['tblAccountActivation'] = tblAccountActivation
        tables['tblAccountActivationMail'] = tblAccountActivationMail
        tables['tblLostPasswordMail'] = tblLostPasswordMail
        tables['tblConfig'] = tblConfig
        tables['tblAccountConfig'] = tblAccountConfig
        tables['tblAccountCustomQueryPreset'] = tblAccountCustomQueryPreset
        __tables_defined = True


def do_mappings():
    """
    This function is only called by the django middleware
    when it is needed
    """
    global __mapping_defined
    if not __mapping_defined:

        from basesoft.web.contrib.mailing import models as mailing_models
        from basesoft.web.contrib.mailing.sa_datamapper import get_tables as mail_tables
        from basesoft.web.contrib.dashboard.models import DashboardAccount

        mapper(account_models.Account, tables['tblAccount'],
            properties = {
                'person': relation(contact_models.Person, backref='accounts'),
                'locale': relation(i18n_models.Locale),
                'groups': relation(account_models.GroupAccount, passive_deletes=True),
                'dashboards': relation(DashboardAccount, passive_deletes=True),
                'direct_groups': relation(license_models.Group, backref='accounts', secondary=tables['tblGroupAccount'])
            },
            polymorphic_on=tables['tblAccount'].c.polymorphic_type,
            polymorphic_identity='Account'
        )

        mapper(account_models.GroupAccount, tables['tblGroupAccount'],
               properties = {
                        'group': relation(license_models.Group, passive_deletes=True),
                        'account': relation(account_models.Account, passive_deletes=True),
                        }
            )
        mapper(account_models.AccountActivation, tables['tblAccountActivation'],
               properties = {
                        'account': relation(account_models.Account),
                        }
            )

        mapper(account_models.AccountActivationMail, tables['tblAccountActivationMail'],
            extension = PrivilegeObjectMapper(),
            properties = {
                'fallback_locale': relation(i18n_models.Locale),
                'locales': relation(mailing_models.MailLocale, order_by=mail_tables()['tblMailLocale'].c.locale_id, passive_deletes=True)
            },
            polymorphic_identity='AccountActivationMail',
            inherits=mailing_models.Mail
        )

        mapper(account_models.LostPasswordMail, tables['tblLostPasswordMail'],
            extension = PrivilegeObjectMapper(),
            properties = {
                'fallback_locale': relation(i18n_models.Locale),
                'locales': relation(mailing_models.MailLocale, order_by=mail_tables()['tblMailLocale'].c.locale_id, passive_deletes=True)
            },
            polymorphic_identity='LostPasswordMail',
            inherits=mailing_models.Mail
        )

        mapper(account_models.Config, tables['tblConfig'])
        mapper(account_models.AccountConfig, tables['tblAccountConfig'],
               properties = {
                        'config': relation(account_models.Config, passive_deletes=True),
                        'account': relation(account_models.Account, passive_deletes=True),
                        }
            )

        mapper(account_models.AccountCustomQueryPreset, tables['tblAccountCustomQueryPreset'],
               properties = {
                        'account': relation(account_models.Account, passive_deletes=True),
                        }
            )

        __mapping_defined = True

def app_permissions_category():
    """
    Returns a category for all permissions defined for this app
    """
    return license_models.PermissionCategory(u'Account')

def app_permissions():
    """
    The permissions used within this application. Can be sync'd with the
    `iqsyncpermissions` command.
    """
    return [
        # account
        license_models.Permission(u"create_account", u"""
            This gives the right to create new account.
            """, _("Create account")),
        license_models.Permission(u"update_account", u"""
            This gives the right to update an account.
            """, _("Update account")),
        license_models.Permission(u"delete_account", u"""
            This gives the right to delete an account.
            """, _("Delete account")),

        # account config manager
        license_models.Permission(u'account_config_manager', u"""
            This gives the right to manage accounts configs.
            """, _("Manage account configurations")),

        license_models.Permission(u'switch_account', u"""
            This gives the right to switch to an account session.
            """, _("Switch to another account")),

        # account custom query presets
        license_models.Permission(u'set_custom_query_preset', u"""
            This gives the right to save presets for custom queries.
            """, _("Set customquery preset")),

        # mail
        license_models.Permission(u"create_account_mail", u"""
            This gives the right to create new account mails.
            """, _("Create account mails")),
        license_models.Permission(u"view_account_mail", u"""
            This gives the right to view account mails.
            """, _("View account mails")),
        license_models.Permission(u"update_account_mail", u"""
            This gives the right to update existing account mails.
            """, _("Update account mails")),
        license_models.Permission(u"delete_account_mail", u"""
            This gives the right to delete account mails.
            """, _("Delete account mails")),
        ]


def app_configs():

    configs = []

    # semicolon seperated list of js_modules to be loaded after login
    config = account_models.Config()
    config.name = 'basesoft.account.post_login.js_modules'
    config.type = 'string'
    config.default = None
    config.description = u'extra script to execute after login'
    configs.append(config)

    config = account_models.Config()
    config.name = 'basesoft.web.core.effects.enabled'
    config.type = 'boolean'
    config.default = 'False'
    config.description = u'Enable visual effects throughout the site'
    configs.append(config)

    return configs

