# Copyright (c) 2014 BaseSoft B.V.
# See LICENSE for details.

"""
This file contains the objects, functions and variables to setup the compose application.
"""
import datetime
from sqlalchemy import and_, Column, Integer, ForeignKey, String, \
                       DateTime, UnicodeText, Unicode, SmallInteger, UniqueConstraint
from sqlalchemy.dialects.mysql import MEDIUMTEXT
from basesoft.sqlalchemy.table import IQTable

from basesoft.web.contrib.license.hooks import PrivilegeObjectMapper
from basesoft.web.contrib.license import models as license_models
from django.utils.translation import ugettext as _

from basesoft.web.contrib.i18n import models as i18n_models
from sqlalchemy.types import Boolean
from basesoft.web.contrib.announce.models import Announcement, AnnouncementGroup, AnnouncementAccount
from basesoft.web.contrib.wiki.models import Wiki
from basesoft.web.contrib.account.models import Account

__mapping_defined = False
__tables_defined = False
tables = {}

def reset():
    global __mapping_defined
    global __tables_defined

    __mapping_defined = False
    __tables_defined = False

def get_tables():
    return tables

def do_tables(metadata):
    global __tables_defined
    if not __tables_defined:

        tblAnnouncement = IQTable('tblAnnouncement', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),

            Column('wiki_id', Integer, ForeignKey('tblWiki.id', ondelete="CASCADE"), nullable=False),
            Column('publish_date', DateTime(), nullable=False), # date when announcement becomes visible for targets
            Column('expire_date', DateTime(), nullable=True), # date when announcement is not visible anymore (optional)
            Column('status', Integer, nullable=False), # status: pending/published
            Column('label', String(255), nullable=False), # label for internal usage (title is multilingual)
            Column('must_read', Boolean, nullable=False), # boolean if the target must clicky

            Column('deleted', SmallInteger, nullable=False, default=0),
            Column('archived', SmallInteger, nullable=False, default=0),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
            )

        # owner of the announcement.. and holds which group may approve?
        tblAnnouncementGroup = IQTable('tblAnnouncementGroup', metadata,
            Column('announcement_id', Integer, ForeignKey('tblAnnouncement.id', ondelete="CASCADE"), primary_key=True),
            Column('group_id', Integer, ForeignKey('tblGroup.id'), primary_key=True),
            Column('privilege', Integer, nullable=False, primary_key=True, autoincrement=False),
            Column('group_has_privilege', SmallInteger, default=0),
            Column('license_has_privilege', SmallInteger, default=0),
            Column('domain_has_privilege', SmallInteger, default=0),
            Column('world_has_privilege', SmallInteger, default=0),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        # if an announcement has must_read set, keep track which groups have read it
        tblAnnouncementAccount = IQTable('tblAnnouncementAccount', metadata,
            Column('announcement_id', Integer, ForeignKey('tblAnnouncement.id', ondelete="CASCADE"), primary_key=True),
            Column('account_id', Integer, ForeignKey('tblAccount.id'), primary_key=True),
        )

        tables['tblAnnouncement'] = tblAnnouncement
        tables['tblAnnouncementGroup'] = tblAnnouncementGroup
        tables['tblAnnouncementAccount'] = tblAnnouncementAccount

        __tables_defined = True

def do_mappings():
    """
    This function is called by Django middleware when needed.
    """
    global __mapping_defined
    if not __mapping_defined:
        from sqlalchemy.orm import mapper, relation

        mapper(Announcement, tables['tblAnnouncement'], properties = {
            'groups': relation(AnnouncementGroup, backref='announcement', cascade="all", passive_deletes=True),
            'wiki': relation(Wiki),
        })

        mapper(AnnouncementGroup, tables['tblAnnouncementGroup'], properties = {
            'group': relation(license_models.Group)
        })

        mapper(AnnouncementAccount, tables['tblAnnouncementAccount'], properties = {
            'announcement': relation(Announcement),
            'account': relation(Account),
        })

        __mapping_defined = True


def fill_data(conn):
    pass

def app_permissions_category():
    """
    Returns a category for all permissions defined for this app
    """
    return license_models.PermissionCategory(u'Announcement')

def app_permissions():
    """
    The permissions used within this application. Can be sync'd with the
    `iqsyncpermissions` command.
    """
    return [
        license_models.Permission(u'manage_announcements', u"""
            This gives the right to manage announcements
            """, _("Manage announcements")),
        license_models.Permission(u'receive_announcements', u"""
            This gives the right to view received announcements
            """, _("Receive announcements")),
    ]

def app_configs():
    """
    The account configs available for this application. This can be sync'd with the
    `iqsyncpermissions` command.
    """
    configs = []

    return configs


