# Copyright (c) 2012 BaseSoft B.V.
# See LICENSE for details.

import datetime
from sqlalchemy import and_
from hashlib import sha256 as sha
import random
from django.utils.translation import ugettext as _

from basesoft.web.core.util.controllers import AbstractController, transaction_decorator,\
    AbstractStateController, get_models_of_type
from basesoft.web.core.exceptions import PublicUserException, SecurityException, PublicSystemException, SystemException
from basesoft.web.contrib.mailing.controllers import BaseMailingStateController, MailBaseController
from basesoft.web.contrib.compose.models import ComposePublication, ComposeInvitee, ComposePublicationMailing,\
    ComposePublicationContactList
from sqlalchemy.exc import IntegrityError
from basesoft.web.contrib.mailing.models import MailingContactList

_compose_publication_map = None

def get_compose_publication_map():
    """
    Create a map of child models of ComposePublication with the name as a key and model object as a value.
    """
    global _compose_publication_map
    if _compose_publication_map is not None:
        return _compose_publication_map
    _map = {}
    for model in get_models_of_type(ComposePublication, exclude_parent=True):
        _map[model['model'].__name__] = model

    _compose_publication_map = _map
    return _map

class PublicationStateController(AbstractStateController):
    """
    FIXME: end_date can only be changed if the publication is active or in the future
    """

    _instance_class = ComposePublication

    def __init__(self, session, manager):
        AbstractStateController.__init__(self, session, manager)
        self.publicationMailingStateController = PublicationMailingStateController(self.sa_session, manager)

    def get_publications(self, id=None, priv=ComposePublication.PRIV_VIEW, type=None):
        if type is None:
            type = self._instance_class
        return self.get_privileged_object_by_id(type, id, priv)

    def generate_unique_hash(self):
        """
        Generates a unique hash.
        This has to be unique among anonymous publication hashed but also among the invitees hashes since
        it will be copied there when publication is set to online.
        """
        import random
        from hashlib import sha256 as sha
        while True:
            hash = sha(str(random.random())).hexdigest()[:8]
            if len(self.sa_session.query(ComposePublication).filter(ComposePublication.anonymous_hash == hash).all()) == 0 \
                and len(self.sa_session.query(ComposeInvitee).filter(ComposeInvitee.invitee_hash == hash).all()) == 0:
                break
        return hash

    def set_publication(self, state, group_id=None):
        """
        Method for saving a publication. App specific permissions should be checked in the child.
        """
        self._assert_state(state, None, dict)
        self._assert_state(state, 'begin_date', datetime.datetime)
        self._assert_state(state, 'status', (long, int))

        if state['end_date'] is not None and state['end_date'] != "":
            self._assert_state(state, 'end_date', datetime.datetime)

        # make sure show_page_nrs is an int cuz (form sends it as a string)
        if state['show_page_nrs'] is not None:
            state['show_page_nrs'] = int(state['show_page_nrs'])

        self._assert_state(state, 'label', unicode)
        self._assert_state(state, 'contact_lists', list)
        self._assert_state(state, 'anonymous', bool)
        self._assert_state(state, 'show_page_nrs', (int, None))

        # viewer_print and viewer_pdf are sent as booleans by the client
        # but stored bitwise in the db.. convert its values here
        if state.has_key('viewer_print') and state['viewer_print']:
            self._assert_state(state, 'viewer_print', (bool, ))
            state['viewer_print'] = ComposePublication.VIEWER_PRINT_PRINTER
        else:
            state['viewer_print'] = 0

        if state.has_key('viewer_pdf') and state['viewer_pdf']:
            self._assert_state(state, 'viewer_pdf', (bool, ))
            state['viewer_pdf'] = ComposePublication.VIEWER_PRINT_PDF
        else:
            state['viewer_pdf'] = 0

        # contact lists should be ints
        for i in range(len(state['contact_lists'])):
            try:
                state['contact_lists'][i] = int(state['contact_lists'][i])
                self._assert_state(state['contact_lists'], i, (long, int))
            except ValueError:
                raise PublicSystemException("Wrong type for key '%s' in state '%s'." % (i, state['contact_lists']))

        # check if all dates are ok
        self.check_dates(state)

        # get the publication object
        publication = AbstractStateController._instance_from_state(self, state)

        if state['dbID'] is not None:
            self.assert_privileges_on_object(publication, ComposePublication.PRIV_UPDATE)
            # scheduling a publication (setting status from NEW to SCHEDULED) requires
            # 'schedule_publication' priv; permission should be checked per app
            if (publication.status == ComposePublication.STATUS_NEW and \
                state['status'] == ComposePublication.STATUS_SCHEDULED):
                self.assert_privileges_on_object(publication, ComposePublication.PRIV_SCHEDULE)



        # in case the publication is ONLINE or FINISHED the user can only update end_date (online only), label, show_page_nrs,
        #
        # online only: add new mailings and, in case the existing mailings have send_date > now, edit the existing mailings
        # (the mailing checks are done in the PublicationMailingStateController)
        #
        if publication.status in (ComposePublication.STATUS_ONLINE, ComposePublication.STATUS_FINISHED):

            if publication.begin_date != state['begin_date']:
                raise PublicUserException(_('You cannot update the begin date of a publication that is online or finished.'))

            if publication.status == ComposePublication.STATUS_FINISHED and publication.end_date != state['end_date']:
                raise PublicUserException(_('You cannot update the end date of a publication that is finished.'))

            # in this case, the user can only end a publication or put it back online if it was finished
            if state['status'] != ComposePublication.STATUS_ONLINE and \
               state['status'] != ComposePublication.STATUS_FINISHED:
                raise PublicUserException(_('You cannot change the status of an online publication.'))

            if publication.anonymous != state['anonymous']:
                raise PublicUserException(_('You cannot change the anonymous state of a publication that is online.'))

        publication.begin_date = state['begin_date']
        publication.end_date = state['end_date']
        publication.status = state['status']
        publication.label = state['label']
        publication.show_page_nrs = state['show_page_nrs'] or 0
        publication.show_compose_label = True if not state.has_key('show_compose_label') else state['show_compose_label']
        publication.show_publication_label = True if not state.has_key('show_publication_label') else state['show_publication_label']
        # viewer_print in db is saved bitwise
        publication.viewer_print = state['viewer_print'] + state['viewer_pdf']

        # exit url
        if state.has_key('exit_url') and state['exit_url'] is not None and state['exit_url'] != '' and not state['exit_url'].startswith('http://'):
            raise PublicUserException(_('Exit URL must start with http://'))
        if state.has_key('exit_url'):
            publication.exit_url = state['exit_url']

        publication.anonymous = state['anonymous']
        # the anonymous hash can never be updated or created from state
        if state['anonymous']:
            # generate a hash only if there wasn't one set yet
            if state['dbID'] == None or publication.anonymous_hash is None:
                publication.anonymous_hash = self.generate_unique_hash()
        else:
            # so when updating a publication setting it from anonymous true to false
            # the hash will be removed from the table
            publication.anonymous_hash = None

        contact_lists = state['contact_lists']
        if isinstance(contact_lists, list) and len(contact_lists) > 0:
            if publication.status == ComposePublication.STATUS_ONLINE:
                for c in publication.contact_lists:
                    if c.id not in contact_lists:
                        raise PublicUserException(_('Cannot remove contact list %s from an ONLINE publication' % c.id))
            from basesoft.web.contrib.contact.controllers import ContactController
            cc = ContactController(self.sa_session, account_id=self.get_account().id)
            cls = cc.get_contact_lists(contact_lists)
            publication.contact_lists = cls # invitees for the new contact lists will be set by a task
        else:
            if len(publication.contact_lists) > 0 and publication.status == ComposePublication.STATUS_ONLINE:
                raise PublicUserException(_('Cannot remove contact lists from an ONLINE publication'))
            publication.contact_lists = []

        publication = self.publicationMailingStateController.save_instance_mailings(publication, 'publication', state['mailings'])
        if not publication.anonymous:
            for m in publication.mailings:
                for c in m.contact_lists:
                    if c not in publication.contact_lists:
                        raise PublicUserException(_('Cannot use contact lists for mailings which are not added to the publication.'))

        return publication

    def check_dates(self, state):
        """
        Public method for checking the publication begin/end dates and also the mailings send date
        """
        if not state.has_key('begin_date') or not state.has_key('end_date'):
            return
        if isinstance(state['end_date'], datetime.datetime) and state['begin_date'] >= state['end_date']:
            raise PublicUserException(_('The publication\'s begin date(%(bd)s) must be before the end date(%(ed)s)') \
                                                          % { 'bd': state['begin_date'],
                                                              'ed': state['end_date']})
        if not state.has_key('mailings'):
            return
        for i in range(len(state['mailings'])):
            mailing = state['mailings'][i]
            if not mailing.has_key('send_date') or not mailing['send_date']:
                continue
            if mailing['send_date'] < state['begin_date']:
                raise PublicUserException(_('The mailing\'s send date(%(sd)s) must be after the publication\'s begin date(%(bd)s)') \
                                                              % {'sd': mailing['send_date'],
                                                                 'bd': state['begin_date']})
            if isinstance(state['end_date'], datetime.datetime) and mailing['send_date'] > state['end_date']:
                raise PublicUserException(_('The mailing\'s send date(%(sd)s) must be before the publication\'s end date(%(ed)s)') \
                                                              % {'sd': mailing['send_date'],
                                                                 'ed': state['end_date']})

    def load(self, publication):
        """
        Loads and returns the state for the given publication_id. App specific permissions will be checked in child.
        """
        if not isinstance(publication, self._instance_class):
            publication = self._instance_from_id(publication)

        state = {
            "dbID": publication.id,
            "domID": None,
            "begin_date": publication.begin_date,
            "end_date": publication.end_date,
            "label": publication.label,
            "show_page_nrs": publication.show_page_nrs,

            "viewer_print": publication.viewer_print & ComposePublication.VIEWER_PRINT_PRINTER == ComposePublication.VIEWER_PRINT_PRINTER,
            "viewer_pdf": publication.viewer_print & ComposePublication.VIEWER_PRINT_PDF == ComposePublication.VIEWER_PRINT_PDF,

            "exit_url": publication.exit_url,

            "anonymous": bool(publication.anonymous),
            "anonymous_hash": publication.anonymous_hash,
            "show_compose_label": publication.show_compose_label,
            "show_publication_label": publication.show_publication_label,
            "status": publication.status,
            'contact_lists': [cl.id for cl in publication.contact_lists],
            'mailings': [],
            'polymorphic_type': publication.polymorphic_type
        }

        for mailing in publication.mailings:
            state['mailings'].append(self.publicationMailingStateController.load(mailing))

        return state

class PublicationMailingStateController(BaseMailingStateController):

    # state model for iquest
    _instance_class = ComposePublicationMailing

    # this is the iquest application mail controller
    _mail_controller_class = MailBaseController

class ComposeInviteeController(AbstractController):
    """
    Compose Invitee controller
    """

    def get_invitee(self, id):
        return self.sa_session.query(ComposeInvitee).filter(ComposeInvitee.id==id).first()

    def save_or_update_invitees(self, publication):
        """
        Creates/updates invitees.
        This can also be used after the publication has been set online and some contacts are missing.
        """

        # check for online status of publication.
        if publication.status != ComposePublication.STATUS_ONLINE:
            raise SystemException('Cannot add invitees for publication which is not online.')

        contact_lists = self.sa_session.query(ComposePublicationContactList).filter(
                        and_(ComposePublicationContactList.publication==publication,
                             ComposePublicationContactList.is_processed.in_([ComposePublicationContactList.STATUS_NOT_PROCESSED,
                             ComposePublicationContactList.STATUS_REPROCESS]))).all()

        if len(contact_lists) == 0:
            return []

        reprocess = [cl for cl in contact_lists if cl.is_processed==ComposePublicationContactList.STATUS_REPROCESS]

        # since multiple contact lists could be already in the query result, update the status for all of them now
        # (because saving invitees could take a while) so they won't be selected next time
        self.set_publication_contact_lists_status(contact_lists, ComposePublicationContactList.STATUS_PROCESSING)

        rv = []
        total = []
        for cl in contact_lists:
            i, c = self._create_invitees(cl)
            rv = rv + i
            total = total + c
            self.set_publication_contact_lists_status(cl, ComposePublicationContactList.STATUS_PROCESSED)

        # update the number of invitees for the publication
        publication.number_of_invitees = len(total)
        self.sa_session.add(publication)
        self.sa_session.flush()

        for cl in reprocess:
            # if reprocess we should also set the Mailing with the last send date in the past back to SCHEDULED
            self._reset_mailing_status(cl)

        return rv

    def _reset_mailing_status(self, compose_contact_list):
        p = compose_contact_list.publication
        cl = compose_contact_list.contact_list

        mailing = self.sa_session.query(ComposePublicationMailing).join(MailingContactList, ComposePublicationMailing.mailing_id==MailingContactList.mailing_id).filter(
            and_(
                # No reason to exclude still pending mailings here. We dont want to flood the newly attached Contacts with mailings.
                ComposePublicationMailing.status.in_([ComposePublicationMailing.STATUS_SENT, ComposePublicationMailing.STATUS_SCHEDULED]),
                ComposePublicationMailing.publication==p,
                MailingContactList.contact_list==cl,
                ComposePublicationMailing.send_date<datetime.datetime.now()
            )
        ).order_by(ComposePublicationMailing.send_date.desc()).first()
        if mailing is None:
            return

        # this mailing has a grace period, we should check if it hasnt expired yet. (#4144)
        # if the grace period is None we should always send to new invitees.
        if mailing.sending_grace_period is not None:
            # if the current date is larger than or equal to the grace period we should not resent this email.
            if datetime.datetime.now().date() >= (mailing.send_date + datetime.timedelta(days=mailing.sending_grace_period)).date():
                return

        mailing.status = ComposePublicationMailing.STATUS_SCHEDULED
        self.sa_session.add(mailing)
        self.sa_session.flush()

    def _create_invitees(self, publication_contact_list):
        publication = publication_contact_list.publication
        # list of already invited contacts.
        contacts = [i.contact.id for i in publication.invitees if i.contact is not None]

        rv = []

        cl = publication_contact_list.contact_list
        for c in cl.contacts:
            # if we already have invited this contact or it is archived or deleted, we don't do it again.
            if c.id in contacts or c.archived == 1 or c.deleted == 1:
                continue

            invitee = ComposeInvitee()
            invitee.contact = c
            invitee.invitee_hash = sha(str(random.random())).hexdigest()[:8]
            invitee.publication = publication

            while True:
                try:
                    self.sa_session.add(invitee)
                    self.sa_session.flush()
                    break
                except IntegrityError, e: # hash must be unique
                    if str(e).find("Duplicate entry '%s'" % invitee.invitee_hash) < 0:
                        raise
                    invitee.invitee_hash = sha(str(random.random())).hexdigest()[:8]

            rv.append(invitee)
            contacts.append(c.id)
        return rv, contacts

    @transaction_decorator
    def set_publication_contact_lists_status(self, publication_contact_lists, status):
        if not isinstance(publication_contact_lists, list):
            publication_contact_lists = [publication_contact_lists]
        for pcl in publication_contact_lists:
            pcl.is_processed = status
            self.sa_session.add(pcl)

    def send_mail_to_invitee(self, invitee_ids, mail_id, mail_from_text=None):

        self.assert_permissions('send_separate_mail_to_invites')
        if not isinstance(invitee_ids, list):
            invitee_ids = [invitee_ids]

        mail_controller = MailBaseController(self.sa_session, account_id=self.get_account().id)
        from basesoft.web.contrib.mailing.schedule import HtmlMailTask
        mail_task = HtmlMailTask()

        mail = mail_controller.get_mails(mail_id)
        if len(mail) == 0:
            raise SecurityException('Invalid mail id %s' % mail_id)
        mail = mail[0]

        for invitee_id in invitee_ids:
            invitee = self.get_invitee(invitee_id)

            contact = invitee.contact
            license = mail.groups[0].group.license

            if mail_from_text not in [None, '']:
                mail_from = '"%s" <noreply-%s@iqportalmail.com>' % (mail_from_text, invitee.invitee_hash)
            else:
                mail_from = 'noreply-%s@iqportalmail.com' % invitee.invitee_hash

            # Build the message for the contact and mail
            msg, result = mail_task.build_message(contact.person, mail, context={
                'publication': invitee.publication,
                'contact': contact,
                'license': license,
            }, mail_from=mail_from)
            mail_task.send_mail(contact, msg, result, mail_from)
