# Copyright (c) 2012 BaseSoft B.V.
# See LICENSE for details.

from sqlalchemy.sql import and_

from sqlalchemy.orm.session import Session
from django.utils.translation import ugettext, ugettext_lazy as _

from basesoft.web.contrib.mailing import models as mailing_models
from basesoft.web.core.models import PrivilegeObject

class ComposePublication(PrivilegeObject):
    """
    @DynamicAttrs
    """

    # privilege to schedule a publication (that's setting its status to 'scheduled')
    PRIV_SCHEDULE = 32
    all_privileges = (PrivilegeObject.PRIV_VIEW, PrivilegeObject.PRIV_UPDATE, \
                      PrivilegeObject.PRIV_DELEGATE_PRIVILEGES, PrivilegeObject.PRIV_ARCHIVE,\
                      PrivilegeObject.PRIV_DELETE, PRIV_SCHEDULE)

    STATUS_NEW = 0      # publication new (wont go online)
    STATUS_SCHEDULED = 1    # waiting for scheduler
    STATUS_ONLINE = 2       # respondents can access publication
    STATUS_FINISHED = 3     # no more next periods (will remain this status)

    status_i18n = {
        STATUS_NEW: _('New publication'),
        STATUS_SCHEDULED: _('The publication is scheduled'),
        STATUS_ONLINE: _('The publication is online'),
        STATUS_FINISHED: _('The publication is expired')
    }

    PAGE_NUMBERING_NONE = 0
    PAGE_NUMBERING_AS_NUMBERS = 1
    PAGE_NUMBERING_AS_PROGRESS = 2

    VIEWER_PRINT_PRINTER = 1
    VIEWER_PRINT_PDF = 2

    page_numbering_i18n = {
        PAGE_NUMBERING_NONE: _('None'),
        PAGE_NUMBERING_AS_NUMBERS: _('Show page numbers'),
        PAGE_NUMBERING_AS_PROGRESS: _('Show progress bar'),
    }

    _name_property = 'label'

    NAME_I18N = _('Compose publication')

    def approved_for_sending(self):
        return True

    def get_group_object(self):
        return (ComposePublicationGroup, 'compose_publication')

    def get_privileges(self):
        return PrivilegeObject.get_privileges(self) + [
            (self.PRIV_SCHEDULE, ugettext('Schedule privileges')),
        ]

    def __repr__(self):
        return '<ComposePublication id=%s>' % (self.id)

class ComposePublicationGroup(object):
    """
    Model representing the link between compose publications and groups.
    @DynamicAttrs
    """

class ComposePublicationLocale(object):
    """
    @DynamicAttrs
    """

class ComposePublicationContactList(object):
    """
    Model representing the link between Compose publications and contact_lists.
    @DynamicAttrs
    """
    STATUS_NOT_PROCESSED = 0
    STATUS_PROCESSING = 1
    STATUS_PROCESSED = 2
    # processed contact lists that have new contacts added
    STATUS_REPROCESS = 3

class ComposeMail(mailing_models.Mail):
    """
    This object is not linked directly to a database object, it just defines Compose related replacement fields.
    """
    @staticmethod
    def replacement_fields():
        from basesoft.web.contrib.contact.models import Company
        x = mailing_models.Mail.replacement_fields()
        x.update({
            ugettext('Company'): {
                'company.name': Company.cols_i18n['name']
            }
        })
        return x

    def _replace_field(self, person, field_name, context):

        def format(company, field_name):
            if company is None:
                return ''
            return_value = getattr(company, field_name)

            if return_value is None: return_value = ''

            return "%s" % return_value

        replacement_fields = {
            'company.name' : 'name'
        }

        if context.has_key('contact') and field_name in replacement_fields.keys():
            return format(context['contact'].company, replacement_fields[field_name])


        return mailing_models.Mail._replace_field(self, person, field_name)

class ComposePublicationMailing(mailing_models.Mailing):
    """
    This is the model for a link between publications and mailings
    @DynamicAttrs
    """
    FILTER_RESPONDENTS = 1 # this will filter out respondents; this will send to NON-RESPONDENTS only.
    FILTER_NON_RESPONDENTS = 2 # this will filter out non-respondents; this will send to RESPONDENTS only.
    # this will filter out non-respondents and backoffice respondents; this will send to internet RESPONDENTS only.
    FILTER_NON_RESPONDENTS_AND_BACKOFFICE = 3

    def get_filters(self):
        return mailing_models.Mailing.get_filters(self) + [
            (self.FILTER_RESPONDENTS, ugettext('Exclude respondents')),
            (self.FILTER_NON_RESPONDENTS, ugettext('Exclude non-respondents')),
            (self.FILTER_NON_RESPONDENTS_AND_BACKOFFICE, ugettext('Exclude non-respondents and back-office respondents')),
        ]
    def approved_for_sending(self):
        """
        Make sure that the publication is online and all contact lists have been processed, else we do not send the mailing.
        """
        # publication needs to be online
        if self.publication.status != ComposePublication.STATUS_ONLINE:
            return False
        if not self.publication.approved_for_sending():
            return False

        session = Session.object_session(self)

        # all mailing contact lists that are linked to a publication need to be processed (invitees created)
        clids = [cl.id for cl in self.contact_lists]
        q = session.query(ComposePublicationContactList).filter(and_(
            ComposePublicationContactList.publication==self.publication,\
            ComposePublicationContactList.contact_list_id.in_(clids),\
            ComposePublicationContactList.is_processed!=ComposePublicationContactList.STATUS_PROCESSED
            )).all()

        if len(q) > 0:
            return False
        return True

    def mail_from(self, contact):
        """
        """
        session = Session.object_session(self)
        q = session.query(ComposeInvitee).filter(and_(ComposeInvitee.publication==self.publication,\
                                                     ComposeInvitee.contact==contact)).all()
        # this contact is not linked to a publication (anonymous)
        if len(q) == 0:
            if self.mail_from_text not in [None, '']:
                return '"%s" <noreply@iqportalmail.com>' % self.mail_from_text
            return 'noreply@iqportalmail.com'
        if self.mail_from_text not in [None, '']:
            return '"%s" <noreply-%s@iqportalmail.com>' % (self.mail_from_text, q[0].invitee_hash)
        return 'noreply-%s@iqportalmail.com' % q[0].invitee_hash

class ComposeInvitee(object):
    """
    @DynamicAttrs
    """

    id = None
    publication = None
    contact = None
    invitee_hash = None

    def __repr__(self):
        return '<ComposeInvite id=%s>' % self.id
