# Copyright (c) 2012 BaseSoft B.V.
# See LICENSE for details.

from basesoft.web.contrib.contact.controllers import ContactController
from basesoft.web.contrib.compose.controllers import PublicationStateController, ComposeInviteeController
from basesoft.web.core.util.controllers import StateManager
from basesoft.web.contrib.compose.models import ComposePublication
from basesoft.web.core.exceptions import PublicSecurityException
from basesoft.web.contrib.compose.controllers import get_compose_publication_map

class ComposePublicationHandler:
    """
    Handles rpc calls for the compose publication.
    """
    controller_class = PublicationStateController

    def __init__(self, request, **kwargs):
        self.request = request
        self.session = request.Session()
        self.account_id = request.session['user_id']
        locale = request.session['config']['basesoft.web.contrib.locale']
        self.manager = StateManager(locale, self.account_id, None)
        self.controller = self.controller_class(self.session, self.manager)

    def get_child_models(self, parent_model='ComposePublication', exclude_parent=False, name=None, include_models=True):
        """
        Get all child models of parent_model model for which the user
        has permission to create a publication.
        """
        from basesoft.web.core.util.controllers import get_models_of_type

        permitted_models = []
        model = get_compose_publication_map()[parent_model]['model']
        report_models = get_models_of_type(model, exclude_parent=exclude_parent)

        for model in report_models:
            if self.controller.has_permissions(*model['model'].permissions):
                permitted_models.append(model)

        if len(permitted_models) == 0:
            raise PublicSecurityException('No publication permissions.')

        return [{'label': model['label'], 'value': model['value']} for model in permitted_models]

    def get_js_module(self, poly_type):
        """
        Get the js_module of a certain publication type using the polymorphic type.
        """
        model = get_compose_publication_map()[poly_type]['model']
        return model.js_module

    def check_constraints(self, state):
        """
        This is used to check all constraints when setting a publication:
         * publication dates constraints
         * mailings: if contact lists have at least one contact with email address set
        """
        self.controller.check_dates(state)
        if state.has_key("mailings") and len(state["mailings"]) > 0:
            cl_ids = []
            for mailing in state["mailings"]:
                for cl in mailing["contact_lists"]:
                    cl_ids.append(cl)
            contact_controller = ContactController(self.session, account_id=self.account_id)
            cl_objs = contact_controller.get_contact_lists(id=cl_ids)
            contact_controller.check_contact_lists_email(cl_objs)

    def get_page_number_options(self):
        return [{'label': v, 'value': i } for i, v in ComposePublication.page_numbering_i18n.iteritems()]

    def save(self, publication_state, group_id=None):
        """
        Save a publication state.
        Returns the new publication state.
        """
        publication = self.controller.save(publication_state, group_id)
        return self.controller.load(publication.id)

    def mark_as_deleted(self, publication_id):
        """
        Delete compose publications.
        @param publication_id: int
        """
        self.controller.mark_as_deleted(publication_id)

    def save_state_property(self, publication_id, state_properties):
        """
        Method receives a publication id and a state properties dict.
        The state_properties dict contains state keys and values to be changed.
        This is only about publication state, not including the mailing part of it.
        """
        state = self.controller.load(publication_id)

        for key in state_properties:
            # the state controller needs a unicode domID
            state["domID"] = unicode(state["dbID"])
            for m in state["mailings"]:
                m["domID"] = unicode(m["dbID"])
            # set the new value for the state property that needs to be updated
            if state.has_key(key):
                state[key] = state_properties[key]
        self.controller.save(state)
        return state

    def get_state(self, _id):
        publication = self.session.query(ComposePublication).filter(ComposePublication.id == _id).one()
        controller_obj = publication.get_controller_instance()
        controller = controller_obj(self.session, self.manager)
        return controller.load(_id)

    def load_state(self, publication_id):
        """
        Returns a publication state
        """
        return self.controller.load(publication_id)

class ComposeInviteeHandler:

    def __init__(self, request, **kwargs):
        self.request = request
        self.session = request.Session()
        self.account_id = request.session['user_id']
        self.controller = ComposeInviteeController(self.session, account_id=self.account_id)

    def send_mail_to_invitee(self, args):
        assert args.has_key('invitees') and args.has_key('mail_id')
        ft = None
        if args.has_key('mail_from_text'):
            ft = args['mail_from_text']
        return self.controller.send_mail_to_invitee(args['invitees'], args['mail_id'], ft)

handlers = (ComposePublicationHandler, ComposeInviteeHandler, )