# Copyright (c) 2012 BaseSoft B.V.
# See LICENSE for details.

"""
This file contains the objects, functions and variables to setup the compose application.
"""
import datetime
from sqlalchemy import Column, Integer, ForeignKey, String, \
                       DateTime, UnicodeText, Boolean, SmallInteger, UniqueConstraint
from sqlalchemy.dialects.mysql import MEDIUMTEXT
from basesoft.sqlalchemy.table import IQTable

from basesoft.web.contrib.license.hooks import PrivilegeObjectMapper
from basesoft.web.contrib.license import models as license_models
from django.utils.translation import ugettext as _

from basesoft.web.contrib.i18n import models as i18n_models

__mapping_defined = False
__tables_defined = False
tables = {}

def reset():
    global __mapping_defined
    global __tables_defined

    __mapping_defined = False
    __tables_defined = False

def get_tables():
    return tables

def do_tables(metadata):
    global __tables_defined
    if not __tables_defined:

        tblComposePublication = IQTable('tblComposePublication', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('polymorphic_type', String(255), nullable=False),
            Column('begin_date', DateTime()),
            Column('end_date', DateTime(), nullable=True),
            Column('status', Integer, nullable=False, default=0),
            Column('label', String(255), nullable=True),
            Column('anonymous',SmallInteger, nullable=False, default=False),
            Column('anonymous_hash', String(8), index=True), # contains a unique sha1 hash used to create anonymous links
            Column('show_page_nrs', SmallInteger, nullable=False, default=True),
            # if viewer has the option to print the module
            Column('viewer_print', SmallInteger, nullable=False, default=True),
            Column('number_of_invitees', Integer, nullable=True), # cache of the number of invitees for quicker overviews
            Column('show_compose_label', Boolean, nullable=False, default=True),
            Column('show_publication_label', Boolean, nullable=False, default=True),
            #Column('state', MEDIUMTEXT, nullable=True),
            Column('exit_url', UnicodeText, nullable=True),
            Column('deleted', SmallInteger, nullable=False, default=0),
            Column('archived', SmallInteger, nullable=False, default=0),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
            )

        # This is the link between a group of accounts and the publication, this will be used for access rights.
        tblComposePublicationGroup = IQTable('tblComposePublicationGroup', metadata,
            Column('compose_publication_id', Integer, ForeignKey('tblComposePublication.id', ondelete="CASCADE"), primary_key=True),
            Column('group_id', Integer, ForeignKey('tblGroup.id'), primary_key=True),
            Column('privilege', Integer, nullable=False, primary_key=True, autoincrement=False),
            Column('group_has_privilege', SmallInteger, default=0),
            Column('license_has_privilege', SmallInteger, default=0),
            Column('domain_has_privilege', SmallInteger, default=0),
            Column('world_has_privilege', SmallInteger, default=0),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
            )

        # holds a publication state per locale
        tblComposePublicationLocale = IQTable('tblComposePublicationLocale', metadata,
            Column('compose_publication_id', Integer, ForeignKey('tblComposePublication.id', ondelete="CASCADE"), primary_key=True),
            Column('locale_id', Integer, ForeignKey('tblLocale.id'), primary_key=True),
            Column('state', MEDIUMTEXT, nullable=True),
            )

        # publication-mailing inherits from mailing
        tblComposePublicationMailing = IQTable('tblComposePublicationMailing', metadata,
            Column('mailing_id', Integer, ForeignKey('tblMailing.id'), primary_key=True),
            Column('publication_id', Integer, ForeignKey('tblComposePublication.id', ondelete='CASCADE'), primary_key=True),
            Column('sending_grace_period', Integer, nullable=True, default=0),
        )

        # ComposeContactList holds all the contact lists for which a compose module will be published.
        # The contacts of these contact lists get copied into ComposeInvitee on the moment of schedule.
        # When a compose module is anonymous nothing gets inserted here.
        tblComposePublicationContactList = IQTable('tblComposePublicationContactList', metadata,
            Column('publication_id', Integer, ForeignKey('tblComposePublication.id'), primary_key=True),
            Column('contact_list_id', Integer, ForeignKey('tblContactList.id'), primary_key=True),
            Column('is_processed', SmallInteger, nullable=False, default=0),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
            )

        # tblComposeInvitee gets filled with links between contact and ComposePublication. This happens when
        # the scheduler publishes the compose mosule.
        # When a compose module is anonymous there will only be one entry in here where the contact_id is null.
        tblComposeInvitee = IQTable('tblComposeInvitee', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('publication_id', Integer, ForeignKey('tblComposePublication.id'), nullable=False),
            Column('contact_id', Integer, ForeignKey('tblContact.id'), nullable=True),
            Column('invitee_hash', String(8), unique=True, index=True), # contains a random part of a sha1 hash used to send out to participants
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now),
            UniqueConstraint('publication_id', 'contact_id'),
            )

        tables['tblComposePublication'] = tblComposePublication
        tables['tblComposePublicationGroup'] = tblComposePublicationGroup
        tables['tblComposePublicationLocale'] = tblComposePublicationLocale
        tables['tblComposePublicationMailing'] = tblComposePublicationMailing
        tables['tblComposePublicationContactList'] = tblComposePublicationContactList
        tables['tblComposeInvitee'] = tblComposeInvitee


        __tables_defined = True

def do_mappings():
    """
    This function is called by Django middleware when needed.
    """
    global __mapping_defined
    if not __mapping_defined:
        from sqlalchemy.orm import mapper, relation
        from basesoft.web.contrib.compose import models as compose_models
        from basesoft.web.contrib.contact import models as contact_models
        from basesoft.web.contrib.mailing import models as mailing_models

        mapper(compose_models.ComposePublicationContactList, tables['tblComposePublicationContactList'], properties = {
            'publication': relation(compose_models.ComposePublication),
            'contact_list': relation(contact_models.ContactList)
        })

        mapper(compose_models.ComposePublication, tables['tblComposePublication'],
            extension = PrivilegeObjectMapper(),
            properties = {
                'invitees': relation(compose_models.ComposeInvitee),
                'groups': relation(compose_models.ComposePublicationGroup, backref='compose_publication', cascade="all", passive_deletes=True),
                'contact_lists': relation(contact_models.ContactList, secondary=tables['tblComposePublicationContactList']),
                'mailings': relation(compose_models.ComposePublicationMailing, passive_deletes=True),
                },
            polymorphic_identity='ComposePublication',
            polymorphic_on=tables['tblComposePublication'].c.polymorphic_type,
        )

        mapper(compose_models.ComposePublicationGroup, tables['tblComposePublicationGroup'], properties = {
            'group': relation(license_models.Group)
        })

        mapper(compose_models.ComposePublicationLocale, tables['tblComposePublicationLocale'], properties = {
            'publication': relation(compose_models.ComposePublication),
            'locale': relation(i18n_models.Locale),
        })

        mapper(compose_models.ComposePublicationMailing, tables['tblComposePublicationMailing'],
            properties = {
                'publication': relation(compose_models.ComposePublication),
            },
            polymorphic_identity='ComposePublicationMailing',
            inherits=mailing_models.Mailing
        )

        mapper(compose_models.ComposeInvitee, tables['tblComposeInvitee'], properties={
               'contact': relation(contact_models.Contact),
               'publication': relation(compose_models.ComposePublication),
        })

        __mapping_defined = True


def fill_data(conn):
    pass

def app_permissions_category():
    """
    Returns a category for all permissions defined for this app
    """
    return license_models.PermissionCategory(u'Compose')

def app_permissions():
    """
    The permissions used within this application. Can be sync'd with the
    `iqsyncpermissions` command.
    """
    return [
        license_models.Permission(u'send_separate_mail_to_invites', u"""
            This gives the right to directly send a separate mail to an invitee oustide the mailing.
            """, _("Send separate mail to invitees"))
        ]

def app_configs():
    """
    The account configs available for this application. This can be sync'd with the
    `iqsyncpermissions` command.
    """
    from basesoft.web.contrib.account import models as account_models
    configs = []

    config = account_models.Config()
    config.name = 'basesoft.web.compose.publication_exit_url'
    config.type = 'string'
    config.default = ''
    config.description = u'Default for exit URL in publications '
    configs.append(config)

    return configs


