# Copyright (c) 2012 BaseSoft B.V.
# See LICENSE for details.

from datetime import datetime

from django.shortcuts import render_to_response
from django.template.context import RequestContext
from django.template import TemplateDoesNotExist
from django.utils import simplejson

from basesoft.web.contrib.account.decorators import login_required
from basesoft.web.core.forms.handler import FormValidator
from basesoft.web.core.jsonrpc.dispatcher import JsonRpcDispatcher
from basesoft.web.core.views import loader_get
from basesoft.web.contrib.license.controllers import LicenseController
from basesoft.web.contrib.branding.controllers import LicenseThemeController
from basesoft.web.core.util.controllers import LicenseConfigController
from basesoft.web.contrib.file.models import ImageFile

@login_required
def publication_validation_rpc(request):
    """View that handles form validation in the publication wizard"""

    from basesoft.web.core.input.validation import plugins, group_plugins
    form_handler = FormValidator(

        fields = {
            'begin_date': [(plugins.Required, {})],
            'label': [(plugins.Required, {})],
            'exit_url': [(plugins.URL, {})],
        },

        group_plugins = [(group_plugins.Required, {'contact_lists': {'anonymous': False}})]
        )
    dispatcher = JsonRpcDispatcher()
    dispatcher.register_instance(form_handler)
    return dispatcher.handle_request(request)

@login_required
def publication_mailing_validation_rpc(request):
    """View that handles form validation in the publication wizard"""

    from basesoft.web.core.input.validation import group_plugins
    form_handler = FormValidator(
        group_plugins = [(group_plugins.Required, {'mail_id': {'send_date': True},
                                                   'contact_lists': {'mail_id': True},
                                                   'send_date': {'contact_lists': True}
                                                   })
                         ]
        )
    dispatcher = JsonRpcDispatcher()
    dispatcher.register_instance(form_handler)
    return dispatcher.handle_request(request)

def bootstrap(request, version, type_, extra_requires):
    """
    bootstrap view called by the compose html template
    """

    requires = ['iq_core', 'iq_broker', 'iq_loader']

    # parse extra_requires
    if extra_requires == '':
        extra_requires = []
    else:
        extra_requires = extra_requires.split(',')
    requires.extend(extra_requires)

    return loader_get(request, version, type_, '', ','.join(requires))


def compose_runner(request, requires, compose_vars, title, licensee=None):
    """
    Abstract view called by iquest/report/etc preview panel.
    This renders a template based on the current license
    """

    assert isinstance(compose_vars, dict)
    assert isinstance(requires, (list, tuple))

    sa_session = request.Session()

    css_requires = list(requires)
    css_requires.append('iq_compose_runner_scss')

    vars_ = {}
    vars_['current_date'] = datetime.now()
    vars_['compose_vars_json'] =  simplejson.dumps(compose_vars)
    vars_['js_requires'] = ','.join(requires)
    vars_['css_requires'] = ','.join(css_requires)
    vars_['title'] = title
    vars_['print'] = compose_vars.has_key('print') and compose_vars['print']
    vars_['show_compose_label'] = compose_vars.has_key('show_compose_label') and compose_vars['show_compose_label']
    vars_['show_publication_label'] = compose_vars.has_key('show_publication_label') and compose_vars['show_publication_label']

    if compose_vars.has_key('compose_name'):
        vars_['compose_name'] =  compose_vars['compose_name']

    if compose_vars.has_key('publication_label'):
        vars_['publication_label'] =  compose_vars['publication_label']

    if compose_vars.has_key('currentLocale'):
        vars_['currentLocale'] =  compose_vars['currentLocale']

    if compose_vars.has_key('availableLocales'):
        vars_['availableLocales'] =  compose_vars['availableLocales']

    account_id = None
    if request.session.has_key('user_id') and request.session['user_id'] is not None:
        account_id = vars_['account_id'] = request.session['user_id']

    licensee_logo = None
    lc = LicenseController(sa_session)
    site_licensee = lc.get_license_for_request(request)
    if licensee is None:
        licensee = site_licensee.http_host

    lcc = LicenseConfigController(sa_session, scope=site_licensee)
    img = lcc.get('basesoft.license.company.logo')
    if img is not None:
        licensee_logo = sa_session.query(ImageFile).get(img).file_hash

    vars_['licensee'] = licensee
    vars_['licensee_logo'] = licensee_logo

    ltc = LicenseThemeController(sa_session, account_id=account_id, request=request)
    current_theme = ltc.get_current_theme()
    vars_['current_theme'] = current_theme

    try:
        r = render_to_response('licensee/%s/compose.html' % licensee, vars_, context_instance=RequestContext(request))
        print 'compose_runner: rendering licensee template'
        return r

    except TemplateDoesNotExist:
        print 'compose_runner: rendering default template'
        return render_to_response('compose_base.html', vars_, context_instance=RequestContext(request))


