# Copyright (c) 2008 BaseSoft B.V.
# See LICENSE for details.

from django.utils.translation import ugettext_lazy as _, ugettext
from basesoft.web.core.models import PrivilegeObject

class Address(object):
    """
    Model representing an address.
    @DynamicAttrs
    """

    cols_i18n = {
       "id": _("Address ID"),
       "postcode": _("Postal code"),
       "street": _("Street"),
       "street_2": _("Street (line 2)"),
       "city": _("City"),
       "state": _("State"),
       "created": _("Created"),
       "last_updated": _("Last updated")
    }

available_cols = []

class Person(object):
    """
    This model represents a person
    @DynamicAttrs
    """

    NAME_I18N = _('Person')
    STR = 'Person'
    JS_MODULE = 'iq_person_form'

    first_name = None
    middle_name = None
    last_name = None
    title = None
    gender = None
    email = None
    position = None
    department = None
    phone = None
    mobile_phone = None
    fax = None
    birth_date = None
    skype_name = None

    cols_i18n = {
       "first_name": _("First name"),
       "middle_name": _("Middle name"),
       "last_name": _("Last name"),
       "title": _("Title"),
       "gender": _("Gender"),
       "email": _("Email"),
       "position": _("Position"),
       "department": _("Department"),
       "phone": _("Phone"),
       "mobile_phone": _("Mobile phone"),
       "fax": _("Fax"),
       "birth_date": _("Birth date"),
       'preferred_locale': _("Language"),
       'skype_name': _('Skype name'),
    }

    MALE = 'male'
    FEMALE = 'female'

    gender_options = (MALE, FEMALE)
    # possible values for gender
    gender_map = {MALE: ['m', 'male', 'man'],
                  FEMALE: ['f', 'female', 'vrouw', 'woman']}

    def __repr__(self):
        return '<Person first_name="%s", last_name="%s" id="%s">' % (self.first_name, self.last_name, self.id)

    def _get_full_name(self):
        """
        Returns the full name of a person. like: "Mr. Martijn van den Broek"
        """
        return " ".join([e for e in [self.title, self.first_name, self.middle_name, self.last_name] if e is not None and e.strip() != ''])

    full_name = property(_get_full_name)

    def _get_full_name_no_title(self):
        return " ".join([e for e in [self.first_name, self.middle_name, self.last_name] if e is not None and e.strip() != ''])

    person_name = property(_get_full_name_no_title)

class Company(object):
    """
    This model represents a Company
    @DynamicAttrs
    """
    NAME_I18N = _('Company')
    STR = 'Company'

    cols_i18n = {
       "name": _("Company name"),
       "vat_no": _("VAT number"),
       "website": _("Website")
    }

    # this includes all country specific fields.
    full_cols_i18n = {
       "name": _("Company name"),
       "vat_no": _("VAT number"),
       "website": _("Website"),
       "chamber_of_commerce_nr": _("Chamber of commerce number"),
       "es_cif_nr": _("CIF number"),
       "fr_rcs_number": _("RCS number"),
       "fr_rcs_location": _("RCS location"),
       "fr_company_type": _("Company type"),
       "it_fiscal_code": _("Fiscal code"),
    }

    PERMISSIONS = None
    JS_MODULE = 'iq_company_form'

    TEMPORARY_STATUS_NONE = 0
    TEMPORARY_STATUS_APPLIED = 1
    TEMPORARY_STATUS_REJECTED = 2

    def __repr__(self):
        return '<Company name=%s id=%s>' % (self.name, self.id)

class CompanyAddress(object):
    """
    This model represents a Company and Address connection
    @DynamicAttrs
    """

class BankAccount(object):
    """
    This model represents a Bank Account
    @DynamicAttrs
    """
    cols_i18n = {
        "id": _("Bank account ID"),
        "number": _("Bank account number"),
        "iban": _("IBAN"),
        "bic": _("BIC"),
        "created": _("Created"),
        "last_updated": _("Last updated")
    }

class CompanyBankAccount(object):
    """
    This model represents a Company and Bank Account connection
    @DynamicAttrs
    """

class Contact(PrivilegeObject):
    """
    Model representing a Contact.
    @DynamicAttrs
    """
    NAME_I18N = _('Contact')
    PERMISSIONS = None
    STR = 'Contact'

    mark_as_deleted_permission = 'delete_contact'
    mark_as_archived_permission = 'archive_contact'

    person_instance_class = Person
    company_instance_class = Company
    can_set_unique_fields = True

    cols_i18n = {
        "contact_email": _("Email"),
        "client_last_update": _("Last update"),
    }

    def get_group_object(self):
        return (ContactGroup, 'contact')

    def __repr__(self):
        return '<Contact id=%s>' % self.id

class ContactGroup(object):
    """
    Model representing the link between contacts and groups. Used for permissions.
    @DynamicAttrs
    """
    def __repr__(self):
        return '<ContactGroup>'

class ContactList(PrivilegeObject):
    """
    Model representing a Contact List.
    @DynamicAttrs
    """
    # privilege for linking contacts to a contact list
    PRIV_LINK_CONTACTS = 32
    # privilege for unlinking contacts from a contact list
    PRIV_UNLINK_CONTACTS = 64

    mark_as_deleted_permission = 'delete_contact_list'
    mark_as_archived_permission = 'update_contact_list'

    def get_group_object(self):
        return (ContactListGroup, 'contact_list')

    all_privileges = (PrivilegeObject.PRIV_VIEW, PrivilegeObject.PRIV_UPDATE, PrivilegeObject.PRIV_ARCHIVE, PrivilegeObject.PRIV_DELETE,\
                      PrivilegeObject.PRIV_DELEGATE_PRIVILEGES, PRIV_LINK_CONTACTS, PRIV_UNLINK_CONTACTS)

    def get_privileges(self):
        privs = PrivilegeObject.get_privileges(self)
        return privs + [
            (self.PRIV_LINK_CONTACTS, ugettext('Link contact privileges')),
            (self.PRIV_UNLINK_CONTACTS, ugettext('Unlink contact privileges'))
        ]

    def __repr__(self):
        return '<ContactList id=%s>' % self.id

class ContactListGroup(object):
    """
    Model representing the link between contact lists and groups.
    @DynamicAttrs
    """
    def __repr__(self):
        return '<ContactListGroup>'

class ContactListContact(object):
    """
    Link between contact lists and contacts
    @DynamicAttrs
    """
    CONTACT_RECEIVES_ALL_MAIL = 0
    CONTACT_RECEIVES_NO_MAIL = 1

    def __repr__(self):
        return '<ContactListContact>'

class ContactFieldMapping(PrivilegeObject):
    """
    Model representing a Contact Field Mapping object.
    Used to determine which fields from a contact import file should go to which DB field.
    @DynamicAttrs
    """
    def get_group_object(self):
        return (ContactFieldMappingGroup, 'contact_field_mapping')

    def __repr__(self):
        return '<ContactFieldMapping id=%s>' % self.id

class ContactFieldMappingGroup(object):
    """
    @DynamicAttrs
    """
    def __repr__(self):
        return '<ContactFieldMappingGroup>'


