# Copyright (c) 2008 BaseSoft B.V.
# See LICENSE for details.

"""
This defines the tables for the contact app.
TODO: make a link between persons and bank account information, currently
      we only support bank accounts to be linked with Companies.
"""

import datetime

from sqlalchemy.orm import mapper, relation
from sqlalchemy import Column, Integer, String, Unicode, ForeignKey, DateTime, SmallInteger, Date, Enum, UnicodeText
from basesoft.sqlalchemy.table import IQTable

from basesoft.web.contrib.license.hooks import PrivilegeObjectMapper
from basesoft.web.contrib.i18n import models as i18n_models
from basesoft.web.contrib.license import models as license_models
from basesoft.web.contrib.contact import models as contact_models
from django.utils.translation import ugettext as _
from basesoft.web.contrib.contact.hooks import ContactListMapper

__mapping_defined = False
__tables_defined = False
tables = {}

def reset():
    global __mapping_defined
    global __tables_defined

    __mapping_defined = False
    __tables_defined = False

def get_tables():
    """
    """
    return tables

def do_tables(metadata):
    global __tables_defined
    if not __tables_defined:
        # address
        tblAddress = IQTable('tblAddress', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('country_id', Integer, ForeignKey('tblCountry.id')),
            Column('postcode', String(255), nullable=True),
            Column('street', Unicode(255), nullable=True),
            Column('street_2', Unicode(255), nullable=True),
            Column('city', Unicode(255), nullable=True),
            Column('state', Unicode(255), nullable=True),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now),
        )

        # person
        tblPerson = IQTable('tblPerson', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('polymorphic_type', String(255), nullable=False),
            Column('first_name', Unicode(255), nullable=True),
            Column('middle_name', Unicode(255), nullable=True),
            Column('last_name', Unicode(255), nullable=True),
            Column('title', Unicode(255), nullable=True),
            Column('gender', Enum('male', 'female'), nullable=True),
            Column('email', Unicode(255)),
            Column('position', Unicode(255), nullable=True),
            Column('department', Unicode(255), nullable=True),
            Column('phone', String(255), nullable=True),
            Column('mobile_phone', String(255), nullable=True),
            Column('fax', String(255), nullable=True),
            Column('birth_date', Date(), nullable=True),
            Column('skype_name', Unicode(255), nullable=True),
            Column('address_id', Integer, ForeignKey('tblAddress.id'), nullable=True),
            Column('preferred_locale_id', Integer, ForeignKey('tblLocale.id'), nullable=True),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now),
        )

        # company
        tblCompany = IQTable('tblCompany', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('polymorphic_type', String(255), nullable=False),
            Column('name', Unicode(255)),
            Column('vat_no', String(70)),
            Column('invoice_person_id', Integer, ForeignKey('tblPerson.id')),
            Column('contact_person_id', Integer, ForeignKey('tblPerson.id')),
            Column('website', Unicode(70)),
            Column('chamber_of_commerce_nr', String(70), nullable=True),
            Column('es_cif_nr', String(70), nullable=True), # spanish CIF
            Column('fr_rcs_number', String(70), nullable=True), # french
            Column('fr_rcs_location', Unicode(70), nullable=True), # french
            Column('fr_company_type', Unicode(70), nullable=True), # french
            Column('it_fiscal_code', String(70), nullable=True), # italy
            Column('is_temp', SmallInteger, nullable=False, default=0), # is used for master data changes
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now),
        )

        # company address
        tblCompanyAddress = IQTable('tblCompanyAddress', metadata,
            Column('company_id', Integer, ForeignKey('tblCompany.id', ondelete='CASCADE'), primary_key=True),
            Column('address_id', Integer, ForeignKey('tblAddress.id'), primary_key=True),
            Column('is_invoice', SmallInteger),
            Column('is_delivery', SmallInteger),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now),
        )

        # bank account
        tblBankAccount = IQTable('tblBankAccount', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('number', String(40), nullable=False),
            Column('iban', String(70), nullable=True),
            Column('bic', String(70), nullable=False), #ISO 9362 - SWIFT-BIC, BIC code or SWIFT code

            Column('beneficiary', Unicode(255), nullable=True),
            Column('bank_name', Unicode(255), nullable=True),
            Column('note', Unicode(255), nullable=True),

            Column('address_id', Integer, ForeignKey('tblAddress.id'), nullable=True),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now,
                   onupdate=datetime.datetime.now),
        )

        # company bank account
        tblCompanyBankAccount = IQTable('tblCompanyBankAccount', metadata,
            Column('company_id', Integer, ForeignKey('tblCompany.id', ondelete='CASCADE'), primary_key=True),
            Column('bank_account_id', Integer, ForeignKey('tblBankAccount.id'), primary_key=True),
            Column('is_default', SmallInteger),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now),
        )

        tblContact = IQTable('tblContact', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('polymorphic_type', String(255), nullable=False),
            Column('person_id', Integer, ForeignKey('tblPerson.id'), nullable=True),
            Column('company_id', Integer, ForeignKey('tblCompany.id'), nullable=True),
            Column('email', Unicode(255), nullable=True, default=None),
            Column('client_last_update', Date(), nullable=True), # see ticket #3169
            Column('deleted', SmallInteger, nullable=False, default=0),
            Column('archived', SmallInteger, nullable=False, default=0),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
            )

        tblContactGroup = IQTable('tblContactGroup', metadata,
            Column('contact_id', Integer, ForeignKey('tblContact.id', ondelete='CASCADE'), primary_key=True),
            Column('group_id', Integer, ForeignKey('tblGroup.id'), primary_key=True),
            Column('privilege', Integer, nullable=False, primary_key=True, autoincrement=False),
            Column('group_has_privilege', SmallInteger, default=0),
            Column('license_has_privilege', SmallInteger, default=0),
            Column('domain_has_privilege', SmallInteger, default=0),
            Column('world_has_privilege', SmallInteger, default=0),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
            )

        tblContactList = IQTable('tblContactList', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('name', Unicode(255), nullable=False),
            Column('deleted', SmallInteger, nullable=False, default=0),
            Column('archived', SmallInteger, nullable=False, default=0),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
            )

        tblContactListGroup = IQTable('tblContactListGroup', metadata,
            Column('contact_list_id', Integer, ForeignKey('tblContactList.id', ondelete='CASCADE'), primary_key=True),
            Column('group_id', Integer, ForeignKey('tblGroup.id'), primary_key=True),
            Column('privilege', Integer, nullable=False, primary_key=True, autoincrement=False),
            Column('group_has_privilege', SmallInteger, default=0),
            Column('license_has_privilege', SmallInteger, default=0),
            Column('domain_has_privilege', SmallInteger, default=0),
            Column('world_has_privilege', SmallInteger, default=0),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
            )

        tblContactListContact = IQTable('tblContactListContact', metadata,
            Column('contact_list_id', Integer, ForeignKey('tblContactList.id'), primary_key=True),
            Column('contact_id', Integer, ForeignKey('tblContact.id'), primary_key=True),
            Column('mail_settings', SmallInteger, default=0),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
            )

        tblContactFieldMapping = IQTable('tblContactFieldMapping', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('name', Unicode(255), nullable=False),
            Column('contact_type', Unicode(255), nullable=False, default='Contact'),
            Column('unique_fields', UnicodeText, nullable=True),
            Column('mapping', UnicodeText, nullable=False),
            Column('deleted', SmallInteger, nullable=False, default=0),
            Column('archived', SmallInteger, nullable=False, default=0),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        tblContactFieldMappingGroup = IQTable('tblContactFieldMappingGroup', metadata,
            Column('contact_field_mapping_id', Integer, ForeignKey('tblContactFieldMapping.id', ondelete='CASCADE'), primary_key=True),
            Column('group_id', Integer, ForeignKey('tblGroup.id'), primary_key=True),
            Column('privilege', Integer, nullable=False, primary_key=True, autoincrement=False),
            Column('group_has_privilege', SmallInteger, default=0),
            Column('license_has_privilege', SmallInteger, default=0),
            Column('domain_has_privilege', SmallInteger, default=0),
            Column('world_has_privilege', SmallInteger, default=0),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        tables['tblAddress'] = tblAddress
        tables['tblPerson'] = tblPerson
        tables['tblCompany'] = tblCompany
        tables['tblCompanyAddress'] = tblCompanyAddress
        tables['tblBankAccount'] = tblBankAccount
        tables['tblCompanyBankAccount'] = tblCompanyBankAccount
        tables['tblContact'] = tblContact
        tables['tblContactGroup'] = tblContactGroup
        tables['tblContactList'] = tblContactList
        tables['tblContactListGroup'] = tblContactListGroup
        tables['tblContactListContact'] = tblContactListContact

        tables['tblContactFieldMapping'] = tblContactFieldMapping
        tables['tblContactFieldMappingGroup'] = tblContactFieldMappingGroup

        __tables_defined = True

def do_mappings():
    """
    This function is only called by the django middleware
    when it is needed
    """
    global __mapping_defined
    if not __mapping_defined:

        mapper(contact_models.Address, tables['tblAddress'],
               properties = {
                        'country': relation(i18n_models.Country)
                        }
               )
        mapper(contact_models.Person, tables['tblPerson'],
               properties = {
                        'address': relation(contact_models.Address),
                        'preferred_locale': relation(i18n_models.Locale),
                        },
                polymorphic_on=tables['tblPerson'].c.polymorphic_type,
                polymorphic_identity='Person'
               )

        mapper(contact_models.Company, tables['tblCompany'],
               properties = {
                    'invoice_person': relation(contact_models.Person,
                        primaryjoin=tables['tblCompany'].c.invoice_person_id==tables['tblPerson'].c.id),
                    'contact_person': relation(contact_models.Person,
                        primaryjoin=tables['tblCompany'].c.contact_person_id==tables['tblPerson'].c.id),
                    'addresses': relation(contact_models.CompanyAddress, passive_deletes=True),
                    'bank_accounts': relation(contact_models.CompanyBankAccount, passive_deletes=True),
                },
                polymorphic_on=tables['tblCompany'].c.polymorphic_type,
                polymorphic_identity='Company'
               )
        mapper(contact_models.CompanyAddress, tables['tblCompanyAddress'],
               properties = {
                        'company': relation(contact_models.Company, cascade="all"),
                        'address': relation(contact_models.Address)
                        }
               )
        mapper(contact_models.BankAccount, tables['tblBankAccount'],
            properties = {
                'address': relation(contact_models.Address),
            }
        )
        mapper(contact_models.CompanyBankAccount, tables['tblCompanyBankAccount'],
               properties = {
                        'company': relation(contact_models.Company, cascade="all"),
                        'bank_account': relation(contact_models.BankAccount)
                        }
               )

        mapper(contact_models.Contact, tables['tblContact'],
               extension = PrivilegeObjectMapper(),
               properties = {
                        'person': relation(contact_models.Person),
                        'company': relation(contact_models.Company),
                        'groups': relation(contact_models.ContactGroup, backref='contact', cascade="all", passive_deletes=True),
                        },
                polymorphic_on=tables['tblContact'].c.polymorphic_type,
                polymorphic_identity='Contact'
               )

        mapper(contact_models.ContactGroup, tables['tblContactGroup'],
               properties = {
                        'group': relation(license_models.Group)
                        })

        mapper(contact_models.ContactList, tables['tblContactList'],
               extension = ContactListMapper(),
               properties = {
                        'contacts': relation(contact_models.Contact, secondary=tables['tblContactListContact'], backref="contact_lists"),
                        'contact_list_contacts': relation(contact_models.ContactListContact, backref="contact_list"),
                        'groups': relation(contact_models.ContactListGroup, backref='contact_list', cascade="all", passive_deletes=True),
                        }
            )
        mapper(contact_models.ContactListGroup, tables['tblContactListGroup'],
               properties = {
                         'group': relation(license_models.Group)
                         })
        mapper(contact_models.ContactListContact, tables['tblContactListContact'],
               properties = {
                        'contact': relation(contact_models.Contact)
        })

        mapper(contact_models.ContactFieldMapping, tables['tblContactFieldMapping'],
               extension = PrivilegeObjectMapper(),
               properties = {
                    'groups': relation(contact_models.ContactFieldMappingGroup, backref='contact_field_mapping', cascade="all", passive_deletes=True),
                }
        )

        mapper(contact_models.ContactFieldMappingGroup, tables['tblContactFieldMappingGroup'],
               properties = {
                         'group': relation(license_models.Group)
                         })

        __mapping_defined = True

def app_permissions_category():
    """
    Returns a category for all permissions defined for this app
    """
    return license_models.PermissionCategory(u'Contacts')

def app_permissions():
    """
    The permissions used within this application. Can be sync'd with the
    `iqsyncpermissions` command.
    """
    return [
        license_models.Permission(u'view_contact', u"""
            This gives the right to view a contact.
            Note that group, license and domain level
            access define which Contact the group has privileges on.
            """, _("View contacts")),
        license_models.Permission(u'create_contact', u"""
            This gives the right to create a contact.
            Note that group, license and domain level
            access define which Contact the group has privileges on.
            """, _("Create contacts")),
        license_models.Permission(u'update_contact', u"""
            This gives the right to update a contact.
            Note that group, license and domain level
            access define which Contact the group has privileges on.
            """, _("Update contacts")),
        license_models.Permission(u'delete_contact', u"""
            This gives the right to delete a contact.
            Note that group, license and domain level
            access define which Contact the group has privileges on.
            """, _("Delete contacts")),
        license_models.Permission(u'archive_contact', u"""
            This gives the right to archive a contact.
            Note that group, license and domain level
            access define which Contact the group has privileges on.
            """, _("Archive contacts")),
        license_models.Permission(u'import_contacts', u"""
            This gives the right to import a contact from file.
            Note that group, license and domain level
            access define which Contact the group has privileges on.
            """, _("Import contacts")),

        license_models.Permission(u'view_contact_list', u"""
            This gives the right to view a contact list.
            Note that group, license and domain level
            access define which ContactList the group has privileges on.
            """, _("View contact lists")),
        license_models.Permission(u'create_contact_list', u"""
            This gives the right to create a contact list.
            Note that group, license and domain level
            access define which ContactList the group has privileges on.
            """, _("Create contact lists")),
        license_models.Permission(u'update_contact_list', u"""
            This gives the right to update a contact list.
            Note that group, license and domain level
            access define which ContactList the group has privileges on.
            """, _("Update contact lists")),
        license_models.Permission(u'delete_contact_list', u"""
            This gives the right to delete a contact list.
            Note that group, license and domain level
            access define which ContactList the group has privileges on.
            """, _("Delete contact lists")),
        license_models.Permission(u'link_to_contact_list', u"""
            This gives the right to attach contacts to a contact list.
            This requires at least the view_contact right on each of the
            contacts that will be attached to the contact list
            Note that group, license and domain level
            access define which ContactList the group has privileges on.
            """, _("Link contacts to contact lists")),
        license_models.Permission(u'unlink_from_contact_list', u"""
            This gives the right to remove contacts to a contact list.
            This requires at least the view_contact right on each of the
            contacts that will be removed to the contact list
            Note that group, license and domain level
            access define which ContactList the group has privileges on.
            """, _("Unlink contacts from contact lists")),

        license_models.Permission(u'use_field_mapping_presets', u"""
            This gives the rights to use predefined field mappings for contact imports.
            """, _('Use contact field mapping presets')),
        license_models.Permission(u'manage_field_mapping_presets', u"""
            This gives the rights to create/update/delete predefined field mappings for contact imports.
            """, _('Manage contact field mapping presets')),

        license_models.Permission(u'automated_imports', u"""
            This gives the rights to use automated imports by placing a file on the server.
            """, _('Use automated contact import.')),

        ]

def app_usage_options():
    """
    The usage options used within this application. Can be sync'd with the
    `iqsyncpermissions` command.
    """
    return [
        license_models.UsageOption(u'max_number_of_contacts', u"""
            This defines the contacts usage option; use this to limit on number of contacts.
            """, _("Maximum numbers of contacts"))
    ]

def app_configs():
    """
    The account configs available for this application. This can be sync'd with the
    `iqsyncpermissions` command.
    """
    from basesoft.web.contrib.account import models as account_models
    configs = []

    config = account_models.Config()
    config.name = 'contact.client_last_update.date_format'
    config.type = 'string'
    config.default = '%Y-%m-%d'
    config.description = u'Date format used to compare the Client Last Update date.'
    configs.append(config)

    config = account_models.Config()
    config.name = 'contact.import.default.field.mapping.id'
    config.type = 'integer'
    config.default = 0
    config.description = u'The default field mapping id to be used for importing contacts.'
    configs.append(config)

    config = account_models.Config()
    config.name = 'contact.import.default.contact.list.id'
    config.type = 'integer'
    config.default = 0
    config.description = u'The default contact list id to be used for importing contacts.'
    configs.append(config)

    return configs

