# Copyright (c) 2011 BaseSoft B.V.
# See LICENSE for details.

from sqlalchemy.orm.exc import NoResultFound
from sqlalchemy import and_, func, distinct

from basesoft.web.core.util.controllers import AbstractStateController, transaction_decorator
from basesoft.web.contrib.dashboard import models as dashboard_models
from basesoft.util.ordered_dict import OrderedDict
from basesoft.web.core.exceptions import SecurityException, PublicUserException, PublicSecurityException
from basesoft.web.contrib.dashboard.models import DashboardAccount, Dashboard, DashboardGroup,\
    DashboardWidget, DashboardTab
from basesoft.web.contrib.account.models import GroupAccount, Account
from basesoft.web.core.models import PrivilegeObject
from basesoft.web.contrib.account.controllers import AccountController
from basesoft.web.contrib.license.controllers import LicenseController

from django.utils import simplejson
from basesoft.web.core.jsonrpc.dispatcher import json_object_hook, JsonRpcJSONEncoder

from basesoft import logging
logger = logging.getLogger("basesoft.web.contrib.dashboard")

class DashboardStateController(AbstractStateController):
    """
    """

    _instance_class = dashboard_models.Dashboard

    def __init__(self, session, manager, *args, **kwargs):
        AbstractStateController.__init__(self, session, manager)
        self.tabStateController = TabStateController(session, manager)

    def get_dashboards(self, id=None):

        where = None
        if id:
            where = and_(Dashboard.id == id)

        dashboards = []

        group_dashboards = self._get_privileged_object(dashboard_models.Dashboard, where=where)

        account_dashboards = self.sa_session.query(Dashboard) \
            .join(DashboardAccount) \
            .filter(DashboardAccount.account==self.get_account()) \
            .all()

        for group_dashboard in group_dashboards:

            # only add the dashboard if the current account has it assigned personally OR
            # if there is no account that has this dashboard assigned

            my_group_ids = [ga.group_id for ga in self.get_account().groups]

            cnt = self.sa_session.query(DashboardAccount) \
                .join(Account) \
                .join(GroupAccount) \
                .filter(DashboardAccount.dashboard==group_dashboard) \
                .filter(GroupAccount.group_id.in_( my_group_ids ) ) \
                .count()

            if group_dashboard in account_dashboards or cnt == 0:
                dashboards.append(group_dashboard)

        # set readonly and shared flags on all dashboards
        for d in dashboards:
            d.is_readonly = not self.has_privileges_on_object(d, PrivilegeObject.PRIV_UPDATE)
            a_cnt = self.sa_session.query(DashboardAccount).filter(DashboardAccount.dashboard==d).count()
            g_cnt = self.sa_session.query(func.count(distinct(DashboardGroup.group_id)).label('cnt') ).filter(DashboardGroup.dashboard==d).filter(DashboardGroup.group_has_privilege==1).one().cnt
            d.is_shared = (a_cnt > 1 or g_cnt > 1)

        return dashboards

    def _get_sharing_accounts(self, dashboard, group):

        license_c = LicenseController(self.sa_session,account_id=self.account_id)
        accounts = license_c.get_accounts_for_group(group)
        dashboard_accounts = self.sa_session.query(Account).join(DashboardAccount).filter(DashboardAccount.dashboard==dashboard).all()

        available_accounts = [a for a in accounts if not a in dashboard_accounts]
        selected_accounts = [a for a in accounts if a in dashboard_accounts]

        return available_accounts, selected_accounts


    def get_sharing(self, dashboard_id):
        """
        returns a set of groups with accounts and to which it has been shared
        """

        assert dashboard_id is not None

        dashboard = self.sa_session.query(Dashboard).get(dashboard_id)

        rv = []

        license_c = LicenseController(self.sa_session,account_id=self.account_id)

        for group in license_c.get_groups_for_domain():

            filter, fo = self.get_query_filter_on_privileges(Dashboard, DashboardGroup, Dashboard.PRIV_VIEW, groups=[group])
            view = bool(self.sa_session.query(Dashboard).select_from(fo).filter(filter).filter(Dashboard.id==dashboard_id).count())

            edit = False
            if view:
                filter, fo = self.get_query_filter_on_privileges(Dashboard, DashboardGroup, Dashboard.PRIV_UPDATE, groups=[group])
                edit = bool(self.sa_session.query(Dashboard).select_from(fo).filter(filter).filter(Dashboard.id==dashboard_id).count())

            available_accounts, selected_accounts = self._get_sharing_accounts(dashboard, group)

            val = {
                'group_id': group.id,
                'group_name': '%s - %s' % (group.license.name, group.name),
                'viewable': view,
                'editable': edit,
                'available_accounts': [{'label': a.name, 'value': a.id} for a in available_accounts],
                'selected_accounts': [{'label': a.name, 'value': a.id} for a in selected_accounts],
            }

            rv.append(val)

        return rv

    def save_sharing(self, dashboard, group, viewable, editable, account_ids):
        """
        shares the dashboard to the group with the permissions

        @param dashboard
        @param group
        @param viewable boolean
        @param viewable boolean
        @param viewable account_ids list
        """


        license_c = LicenseController(self.sa_session,account_id=self.account_id)

        if viewable:
            license_c.grant_privilege_to(dashboard, [PrivilegeObject.PRIV_VIEW], group=True, target_group=group)

            if editable:
                license_c.grant_privilege_to(dashboard, [PrivilegeObject.PRIV_UPDATE], group=True, target_group=group)
            else:
                license_c.revoke_privilege_from(dashboard, [PrivilegeObject.PRIV_UPDATE], group=True, target_group=group)
        else:
            license_c.revoke_privilege_from(dashboard, [PrivilegeObject.PRIV_VIEW, PrivilegeObject.PRIV_UPDATE], group=True, target_group=group)

        selected_accounts = self._get_sharing_accounts(dashboard, group)[1]
        selected_accounts_ids = [a.id for a in selected_accounts]
        delete_accounts = [a_id for a_id in selected_accounts_ids if a not in account_ids]

        ac = AccountController(self.sa_session, account_id=self._get_account().id)
        accounts = ac.get_accounts(account_ids)

        for ac in self.sa_session.query(DashboardAccount) \
            .filter(DashboardAccount.account_id.in_(delete_accounts)) \
            .filter(DashboardAccount.dashboard == dashboard):
            self.sa_session.delete(ac)

        for account in accounts:

            if not self.sa_session.query(DashboardAccount) \
                .filter(DashboardAccount.account == account) \
                .filter(DashboardAccount.dashboard == dashboard) \
                .count():

                da = DashboardAccount()
                da.account = account
                da.dashboard = dashboard

                self.sa_session.add(da)

                logger.info('share_dashboard: shared %s to account %s' % (dashboard, account))

        self.sa_session.flush()

    @transaction_decorator
    def save_name(self, id, name):
        dbs = self.get_dashboards(id)
        if len(dbs) == 0:
            raise PublicSecurityException(_('No permission to alter the dashboard name'));
        db = dbs.pop()
        self.assert_privileges_on_object(db, Dashboard.PRIV_UPDATE)

        db.name = name
        self.sa_session.add(db)
        return True

    @transaction_decorator
    def save_state(self, state, account=None):
        """
        Save the dashboard state to the account.
        # TODO - #3173
        """
        self.assert_permissions('save_dashboard')
        return self.__save_state(state, account)

    @transaction_decorator
    def __save_state(self, state, account=None):
        AbstractStateController.reset_state_map()

        state = state['dashboard']

        # if no account is provided, get the logged in account.
        if account is None:
            account = self._get_account()

        self._assert_state(state, None, dict)
        self._assert_state(state, 'domID', unicode)
        self._assert_state(state, 'name', unicode)
        self._assert_state(state, 'objectName', unicode, 'Dashboard')
        self._assert_state(state, 'children', dict)
        self._assert_state(state['children'], 'tabs', list)

        if state['dbID'] is not None:
            dids = [d.id for d in self.get_dashboards()]
            if int(state['dbID']) not in dids:
                raise SecurityException('Invalid dashboard id %s for account id %s' % (state['dbID'], account.id))

        dashboard = AbstractStateController._instance_from_state(self, state)
        dashboard.active_index = state['activeIndex']
        dashboard.name = state['name']
        AbstractStateController.save(self, dashboard, state)

        if state['dbID'] is None:
            self.save_default_privilege_object_group(dashboard)
        else:
            self.assert_privileges_on_object(dashboard, PrivilegeObject.PRIV_UPDATE)

        # delete any removed tabs and process
        self._delete_children(dashboard.tabs, state['children']['tabs'])
        for i in range(len(state['children']['tabs'])):
            self.tabStateController.save(dashboard, state['children']['tabs'][i], i)

        # acc_d stays None when the current user doesnt have any personal dashboards
        acc_d = None

        if state['dbID'] is None:
            acc_d = dashboard_models.DashboardAccount()
            acc_d.account = account
            acc_d.dashboard = dashboard
            acc_d.is_default = False

            self.sa_session.add(acc_d)
        else:
            for ad in account.dashboards:
                if ad.dashboard.id == int(state['dbID']):
                    acc_d = ad
                    break

        # this dashboard was not default and now it is
        if acc_d and not acc_d.is_default and state['is_default']:
            # unset the old default one
            for ad in account.dashboards:
                if ad.is_default:
                    ad.is_default = False
                    self.sa_session.add(ad)
                    break
            acc_d.is_default = True

            self.sa_session.add(acc_d)

        #

        #linked_items = state['linkedItems']
        #print linked_items
        #print AbstractStateController.save_state_id_map

        # At this point the whole state has been procesed and modules have been saved to the db. What is left to
        # do is flush the delete_pool because we are sure that these objects arent used anymore.
        AbstractStateController.commit_delete_pool(self.sa_session)

        self._canvas_post_save()

        # return the id map
        return AbstractStateController.save_state_id_map

    def load_state(self, id=None):
        """
        It always loads the dashboard of the current user
        """

        self.assert_permissions('view_dashboard')
        return self.__load_state(id)

    def __load_state(self, id=None):
        account = self._get_account()
        dashboard = None
        if id is None:
            for ad in account.dashboards:
                if ad.is_default and not ad.dashboard.deleted and not ad.dashboard.archived:
                    dashboard = ad.dashboard
                    break
            is_default = True
        elif isinstance(id, Dashboard):
            dashboard = id
            is_default = False
        else:

            if not self.get_dashboards(id):
                raise SecurityException('Trying to load invalid dashboard id %s for account id %s' % (id, account.id))

            dashboard = self._instance_from_id(id)

            try:
                is_default = self.sa_session.query(DashboardAccount) \
                .filter(DashboardAccount.account==self.get_account()) \
                .filter(DashboardAccount.dashboard==dashboard).one().is_default
            except NoResultFound:
                is_default = False

        if dashboard is None:
            dashboards = self.get_dashboards()
            if len(dashboards) == 0:
                return None
            dashboard = dashboards.pop()
            is_default = False

        state = {
            'dashboard': {
                'objectName': u'Dashboard',
                'dbID': dashboard.id,
                'name': dashboard.name,
                'is_default': is_default,
                'activeIndex': dashboard.active_index,
                'is_readonly': not self.has_privileges_on_object(dashboard, PrivilegeObject.PRIV_UPDATE),
                'children': {
                    'tabs': []
                }
            }
        }

        for index, tab in enumerate(dashboard.tabs):
            state['dashboard']['children']['tabs'].append(self.tabStateController.load(tab.id, index==dashboard.active_index))

        return state

    def get_widgets(self, show_hidden=True):
        widgets = self.tabStateController.blockStateController.columnStateController.widgets

        keys = ['label', 'name', 'icon', 'category', 'params', 'requires', 'app']
        ret = []
        for controller in widgets.values():
            widget = controller(self.sa_session, self.manager)
            if not show_hidden and widget.hidden:
                continue

            if widget.has_permissions(self):
                rv = {}
                for var in keys:
                    if hasattr(widget, var):
                        rv[var] = getattr(widget, var)
                ret.append(rv)
        return ret

    def clone(self, dashboard, new_label, account=None, make_default=False):
        # clone can be used by specific actions in which case the controller is instantiated without an account to skip permision checks.
        if self.get_account() is None:
            self.assert_permissions('view_dashboard', 'save_dashboard')
        state = self.__load_state(dashboard)
        self._clone_dom_counter = 0

        self._alter_state_for_clone(state['dashboard'])
        state['dashboard']['name'] = new_label

        if make_default:
            state['dashboard']['is_default'] = True

        dashboard = self.__save_state(state, account)

        AbstractStateController.reset_state_map()

        return dashboard

    def _alter_state_for_clone(self, state):
        """
        recursive unsets dbID's for the given state
        """
        state['domID'] = 'dom-%s' % self._clone_dom_counter
        self._clone_dom_counter = self._clone_dom_counter + 1
        state['dbID'] = None


        if (state.has_key('children')):
            for child_attr in ['tabs', 'blocks', 'columns', 'widgets']:
                if state['children'].has_key(child_attr):
                    for child in state['children'][child_attr]:
                        self._alter_state_for_clone(child)

class TabStateController(AbstractStateController):
    """
    Takes care of tab state logic
    """

    _instance_class = dashboard_models.DashboardTab

    def __init__(self, session, manager):
        AbstractStateController.__init__(self, session, manager)
        self.blockStateController = BlockStateController(session, manager)

    @transaction_decorator
    def save(self, dashboard, state, position):
        assert isinstance(dashboard, dashboard_models.Dashboard)
        self._assert_state(state, None, dict)
        assert isinstance(position, int)
        self._assert_state(state, 'objectName', unicode, 'DashboardTab')

        self._assert_state(state, 'children', dict)
        self._assert_state(state['children'], 'blocks', list)

        children = True
        # call parent which takes care of common compose logic
        if state['dbID'] != None and not state['loaded']:
            children = False
            tab = AbstractStateController._instance_from_id(self, state['dbID'])
        else:
            tab = AbstractStateController._instance_from_state(self, state)

        # reference method arguments
        tab.dashboard = dashboard
        tab.position = position
        tab.label = state['label']

        # call parent save that actually writes to the db
        AbstractStateController.save(self, tab, state)

        # only bubble to children if the tab was loaded at any time during its availability
        if children:
            # delete any removed blocks
            self._delete_children(tab.blocks, state['children']['blocks'])
            # continue processing blocks
            for i in range(len(state['children']['blocks'])):
                self.blockStateController.save(tab, state['children']['blocks'][i], i)

        # expire webshop (so it will update webshop.tabs)
        self.sa_session.expire(dashboard)

    def load(self, id, load_children=True):
        tab = self._instance_from_id(id)

        state = {
            "objectName": u"DashboardTab",
            "dbID": tab.id,
            "position": tab.position,
            "label": tab.label,
            "loaded": load_children,
            "children": {
                'blocks': []
            }
        }

        if load_children:
            for block in tab.blocks:
                state['children']['blocks'].append(self.blockStateController.load(block.id))

        return state


    def copy_tab_to_dasboard(self, tab_state, dashboard_id):

        assert isinstance(tab_state, dict)
        assert isinstance(dashboard_id, int)

        dc = DashboardStateController(self.sa_session, self.manager)
        dashboard = dc.get_dashboards(dashboard_id)
        if len(dashboard) == 0 or dashboard[0].is_readonly:
            raise SecurityException('copy_tab_to_dasboard: no access to dashboard %s' % dashboard_id)
        dashboard = dashboard[0]
        if tab_state['dbID'] is not None:
            tab = [t for t in dashboard.tabs if t.id==int(tab_state['dbID'])]
            if len(tab) > 0:
                raise SecurityException('Cannot copy tab (%s) to it\'s own dashboard' % tab_state['dbID'])

        tab_state['dbID'] = None
        for b in tab_state['children']['blocks']:
            b['dbID'] = None
            for c in b['children']['columns']:
                c['dbID'] = None
                for w in c['children']['widgets']:
                    w['dbID'] = None

        # id map is not important in this case because it's about another dashboard
        AbstractStateController.save_state_id_map = {}
        self.save(dashboard, tab_state, len(dashboard.tabs))

        logger.info('copy_widget_to_tab: copied tab "%s" to dashboard "%s"' % (tab_state['objectName'], dashboard_id))
        return dashboard


class BlockStateController(AbstractStateController):
    """
    Takes care of block state logic
    """

    _instance_class = dashboard_models.DashboardBlock

    def __init__(self, session, manager):
        AbstractStateController.__init__(self, session, manager)
        self.columnStateController = ColumnStateController(session, manager)

    @transaction_decorator
    def save(self, tab, state, position):
        assert isinstance(tab, dashboard_models.DashboardTab)
        self._assert_state(state, None, dict)
        assert isinstance(position, int)
        self._assert_state(state, 'objectName', unicode, 'DashboardBlock')

        self._assert_state(state, 'children', dict)
        self._assert_state(state['children'], 'columns', list)

        # call parent which takes care of common compose logic
        block = AbstractStateController._instance_from_state(self, state)

        # reference method arguments
        block.tab = tab
        block.position = position

        # call parent save that actually writes to the db
        AbstractStateController.save(self, block, state)

        # delete any removed columns
        self._delete_children(block.columns, state['children']['columns'])
        # continue processing columns
        for i in range(len(state['children']['columns'])):
            self.columnStateController.save(block, state['children']['columns'][i], i)

        # expire tab (so it will update tab.blocks)
        self.sa_session.expire(tab)

    def load(self, id):
        block = self._instance_from_id(id)

        state = {
            "objectName": u"DashboardBlock",
            "dbID": block.id,
            "position": block.position,
            "children": {
                'columns': []
            }
        }

        for column in block.columns:
            state['children']['columns'].append(self.columnStateController.load(column.id))

        return state


class ColumnStateController(AbstractStateController):
    """
    Takes care of state handling for columns
    """

    _instance_class = dashboard_models.DashboardColumn

    def __init__(self, session, manager):
        AbstractStateController.__init__(self, session, manager)
        self.widgets = self.get_widgets()

    @transaction_decorator
    def save(self, block, state, position):
        assert isinstance(block, dashboard_models.DashboardBlock)
        self._assert_state(state, None, dict)
        assert isinstance(position, int)
        self._assert_state(state, 'objectName', unicode, 'DashboardColumn')

        self._assert_state(state, 'children', dict)
        self._assert_state(state['children'], 'widgets', list)

        # call parent which takes care of common compose logic
        column = AbstractStateController._instance_from_state(self, state)

        column.block = block
        column.width_percentage = state['width']
        column.position = position

        # call parent save that actually writes to the db
        AbstractStateController.save(self, column, state)

        # delete any removed
        self._delete_children(column.widgets)

        # process widgets
        for i in range(len(state['children']['widgets'])):
            self.__save_item_state(column, state['children']['widgets'][i], i)

        # expire
        self.sa_session.expire(block)

    def __save_item_state(self, column, state, position):
        """
        Takes care of the saving state of widgets by mapping the correspondent controller and instanciating it.
        """

        try:
            WidgetController = self.widgets[state['objectName']]
        except KeyError:
            logger.error('ColumnStateController: Ignoring item saving. No mapper found for %s' % state['objectName'])
        else:
            wc = WidgetController(self.sa_session, self.manager)
            wc.save(column, state, position)

    def load(self, id):
        column = self._instance_from_id(id)

        state = {
            "objectName": u"DashboardColumn",
            "dbID": column.id,
            "width": column.width_percentage,
            "position": column.position,
            "children": {
                'widgets': []
            }
        }

        for widget in column.widgets:
            try:
                wc = self._get_widget_controller(widget.object_name)
                state['children']['widgets'].append(wc.load(widget.id))
            except WidgetException:
                pass

        return state

    def _get_widget_controller(self, object_name):

        try:
            WidgetController = self.widgets[object_name]
        except KeyError:
            raise WidgetException('ColumnStateController: Ignoring item loading. No mapper found for %s' % object_name)

        wc = WidgetController(self.sa_session, self.manager)
        return wc

    def get_widgets(self):
        self.assert_permissions('view_dashboard')

        from django.conf import settings
        from twisted.python.reflect import namedAny

        apps = settings.INSTALLED_APPS

        dashboard_widgets = OrderedDict()
        # see if there is a sa_datamapper module in the app
        # if there is import it and get the list of the tables that will be reset
        for app in apps:
            try:
                dashboard_module = namedAny(app+'.dashboard')
            except AttributeError, e:
                # if we do not have an widgets module then the app doesn't add icons
                continue

            if not hasattr(dashboard_module, 'widgets_mapper') and not hasattr(dashboard_module, 'get_dynamic_widgets'):
                continue

            if hasattr(dashboard_module, 'widgets_mapper'):
                for widget, controller in dashboard_module.widgets_mapper.items():
                    if controller.app is None:
                        controller.app = app
                    dashboard_widgets.update([(widget, controller)])

            if hasattr(dashboard_module, 'get_dynamic_widgets'):
                ws = dashboard_module.get_dynamic_widgets(self.sa_session, self.account_id)
                for widget, controller in ws.items():
                    if controller.app is None:
                        controller.app = app
                    dashboard_widgets.update([(widget, controller)])

        return dashboard_widgets

    def copy_widget_to_tab(self, widget_state, tab_id):

        assert isinstance(widget_state, dict)
        assert isinstance(tab_id, int)

        dc = DashboardStateController(self.sa_session, self.manager)
        my_db_ids = [d.id for d in dc.get_dashboards() if not d.is_readonly]

        wc = self._get_widget_controller(widget_state['objectName'])

        tab = self.sa_session.query(DashboardTab).get(tab_id)
        if tab.dashboard.id not in my_db_ids:
            raise SecurityException('copy_widget_to_tab: no access to tab %s' % tab)

        if widget_state['dbID'] is not None:
            widget = self.sa_session.query(DashboardWidget).filter(DashboardWidget.id==widget_state['dbID']).first()
            if widget and widget.column.block.tab.dashboard.id == tab.dashboard.id:
                raise SecurityException('Cannot copy widget (%s) to it\'s own dashboard' % widget_state['dbID'])

        widget_state['dbID'] = None

        try:
            column = tab.blocks[0].columns[0]
        except IndexError:
            raise PublicUserException(_('No blocks/columns in tab: %s') % tab)

        # id map is not important in this case because it's about another dashboard
        AbstractStateController.save_state_id_map = {}
        wc.save(column, widget_state, len(column.widgets))

        logger.info('copy_widget_to_tab: copied widget "%s" to tab "%s"' % (widget_state['objectName'], tab_id))
        return tab

class DashboardWidgetStateController(AbstractStateController):
    """
    Abstract widget controller.
    """

    _instance_class = dashboard_models.DashboardWidget

    # label of the widget.
    label = ''
    # yui name of the object.
    name = ''
    # URL to an icon of choice.
    icon = None
    # the category of the widget.
    category = ''
    # the application of the widget. (used for icons and such)
    app = None
    # params
    params = None
    # permissions
    permissions = [] # default none
    # default show
    hidden = False

    @transaction_decorator
    def save(self, column, state, position, widget=None):

        assert isinstance(column, dashboard_models.DashboardColumn)
        self._assert_state(state, None, dict)
        assert isinstance(position, int)

        if widget is None:
            widget = AbstractStateController._instance_from_state(self, state)
        widget.column = column
        widget.position = position
        widget.title = state['title']
        widget.show_header = True if not state.has_key('show_header') else state['show_header']

        widget.text = None if not state.has_key('text') else state['text']
        widget.text_location = 0 if not state.has_key('text_location') else state['text_location']

        widget.object_name = state['objectName']
        widget.object_loader = state['loaderName']

        AbstractStateController.save(self, widget, state)

        # expire
        self.sa_session.expire(column)

    def load(self, id):
        widget = self._instance_from_id(id)

        return {
            "objectName": widget.object_name,
            "loaderName": widget.object_loader,
            "dbID": widget.id,
            "position": widget.position,
            "title": widget.title,
            "show_header": widget.show_header,
            "text": widget.text,
            "text_location": widget.text_location
        }

    def has_permissions(self, controller):
        return controller.has_permissions(*self.permissions)

class DashboardWidgetCQStateController(DashboardWidgetStateController):
    """
    Controller for the ProductItem
    """

    _instance_class = dashboard_models.DashboardWidgetCQ
    app = None

    @transaction_decorator
    def save(self, column, state, position):

        if state.has_key('tablesState'):
            assert isinstance(state['tablesState'], list) # cq can have multiple tables
        if state.has_key('filtersState'):
            assert isinstance(state['filtersState'], dict)

        widget = AbstractStateController._instance_from_state(self, state)
        if state.has_key('tablesState'):
            widget.tables_state = simplejson.dumps(state['tablesState'], ensure_ascii=False, cls=JsonRpcJSONEncoder)
        if state.has_key('filtersState'):
            widget.filters_state = simplejson.dumps(state['filtersState'], ensure_ascii=False, cls=JsonRpcJSONEncoder)

        DashboardWidgetStateController.save(self, column, state, position, widget)

        return widget

    def load(self, id):
        widget = self._instance_from_id(id)

        ts = []
        if widget.tables_state is not None:
            ts = simplejson.loads(widget.tables_state, object_hook=json_object_hook)
        fs = {}
        if widget.filters_state is not None:
            fs = simplejson.loads(widget.filters_state, object_hook=json_object_hook)

        return {
            "objectName": widget.object_name,
            "loaderName": widget.object_loader,
            "dbID": widget.id,
            "position": widget.position,
            "title": widget.title,
            "show_header": widget.show_header,
            "text": widget.text,
            "text_location": widget.text_location,
            "tablesState": ts,
            "filtersState": fs,
        }


class WidgetException(Exception):
    pass
