# Copyright (c) 2008 BaseSoft B.V.
# See LICENSE for details.

"""
This defines the tables for the file app.
"""

import datetime
from sqlalchemy.orm import mapper, relation
from sqlalchemy import Column, Integer, String, UnicodeText, ForeignKey, DateTime, Unicode, SmallInteger
from basesoft.sqlalchemy.table import IQTable

from basesoft.web.contrib.license.hooks import PrivilegeObjectMapper
from basesoft.web.contrib.file import models as file_models
from basesoft.web.contrib.license import models as license_models
from django.utils.translation import ugettext as _

__mapping_defined = False
__tables_defined = False
tables = {}

def reset():
    global __mapping_defined
    global __tables_defined

    __mapping_defined = False
    __tables_defined = False

def get_tables():
    """
    """
    return tables

def do_tables(metadata):
    global __tables_defined
    if not __tables_defined:

        tblFileTmp = IQTable('tblFileTmp', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('file_name', String(255), nullable=True),
            Column('file_hash', String(8), nullable=True, unique=True),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now),
        )

        tblFileCategory = IQTable('tblFileCategory', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('label', Unicode(128), nullable=False),
            Column('deleted', SmallInteger, nullable=False, default=0),
            Column('archived', SmallInteger, nullable=False, default=0),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now),
        )

        tblFileCategoryGroup = IQTable('tblFileCategoryGroup', metadata,
            Column('category_id', Integer, ForeignKey('tblFileCategory.id', ondelete="CASCADE"), primary_key=True),
            Column('group_id', Integer, ForeignKey('tblGroup.id'), primary_key=True),
            Column('privilege', Integer, nullable=False, primary_key=True, autoincrement=False),
            Column('group_has_privilege', SmallInteger, default=0),
            Column('license_has_privilege', SmallInteger, default=0),
            Column('domain_has_privilege', SmallInteger, default=0),
            Column('world_has_privilege', SmallInteger, default=0),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now),
        )

        tblFile = IQTable('tblFile', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('polymorphic_type', String(255), nullable=False),
            Column('category_id', Integer, ForeignKey('tblFileCategory.id'), nullable=False),
            Column('file_name', String(255), nullable=True),
            Column('file_hash', String(8), nullable=True, unique=True),
            Column('deleted', SmallInteger, nullable=False, default=0),
            Column('archived', SmallInteger, nullable=False, default=0),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now),
        )

        tblFileGroup = IQTable('tblFileGroup', metadata,
            Column('file_id', Integer, ForeignKey('tblFile.id', ondelete="CASCADE"), primary_key=True),
            Column('group_id', Integer, ForeignKey('tblGroup.id'), primary_key=True),
            Column('privilege', Integer, nullable=False, primary_key=True, autoincrement=False),
            Column('group_has_privilege', SmallInteger, default=0),
            Column('license_has_privilege', SmallInteger, default=0),
            Column('domain_has_privilege', SmallInteger, default=0),
            Column('world_has_privilege', SmallInteger, default=0),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now),
        )

        tblImageFile = IQTable('tblImageFile', metadata,
            Column('file_id', Integer, ForeignKey('tblFile.id', ondelete='CASCADE'), primary_key=True),
        )

        tblExcelFile = IQTable('tblExcelFile', metadata,
            Column('file_id', Integer, ForeignKey('tblFile.id', ondelete='CASCADE'), primary_key=True),
        )

        tblImportFileType = IQTable('tblImportFileType', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('name', Unicode(128), nullable=False),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now),
        )

        tblImportFile = IQTable('tblImportFile', metadata,
            Column('file_id', Integer, ForeignKey('tblFile.id', ondelete='CASCADE'), primary_key=True),
            Column('type_id', Integer, ForeignKey('tblImportFileType.id'), nullable=False),
            Column('status', SmallInteger, nullable=False, default=0),
            # serialized dictionary with import settings.
            Column('settings', UnicodeText, nullable=False),
            Column('result', UnicodeText),
            Column('failure_file_id', Integer, ForeignKey('tblFile.id'), nullable=True),
        )

        tables['tblFileTmp'] = tblFileTmp
        tables['tblFileCategory'] = tblFileCategory
        tables['tblFileCategoryGroup'] = tblFileCategoryGroup
        tables['tblFile'] = tblFile
        tables['tblFileGroup'] = tblFileGroup
        tables['tblImageFile'] = tblImageFile
        tables['tblExcelFile'] = tblExcelFile
        tables['tblImportFileType'] = tblImportFileType
        tables['tblImportFile'] = tblImportFile

        __tables_defined = True

def do_mappings():
    """
    This function is only called by the django middleware
    when it is needed
    """
    global __mapping_defined
    if not __mapping_defined:

        mapper(file_models.FileTmp, tables['tblFileTmp'])

        mapper(file_models.FileCategory, tables['tblFileCategory'],
            extension = PrivilegeObjectMapper(),
            properties = {
                'groups': relation(file_models.FileCategoryGroup, backref="category", cascade="all", passive_deletes=True)
            }
        )

        mapper(file_models.FileCategoryGroup, tables['tblFileCategoryGroup'],
            properties = {
                'group': relation(license_models.Group)
            }
        )

        mapper(file_models.File, tables['tblFile'],
            extension = PrivilegeObjectMapper(),
            properties = {
                'category': relation(file_models.FileCategory),
                'groups': relation(file_models.FileGroup, backref="file", cascade="all", passive_deletes=True)
            },
            polymorphic_on=tables['tblFile'].c.polymorphic_type,
            polymorphic_identity='File'
        )

        mapper(file_models.FileGroup, tables['tblFileGroup'],
            properties = {
                'group': relation(license_models.Group)
            }
        )

        mapper(file_models.ImageFile, tables['tblImageFile'],
            polymorphic_identity='ImageFile',
            inherits=file_models.File
        )

        mapper(file_models.ExcelFile, tables['tblExcelFile'],
            polymorphic_identity='ExcelFile',
            inherits=file_models.File
        )

        mapper(file_models.ImportFileType, tables['tblImportFileType'])

        mapper(file_models.ImportFile, tables['tblImportFile'],
            properties = {
                'type': relation(file_models.ImportFileType),
                'failure_file': relation(file_models.File, primaryjoin=get_tables()['tblFile'].c.id==tables['tblImportFile'].c.failure_file_id)
            },
            polymorphic_identity='ImportFile',
            inherits=file_models.File,
            inherit_condition=get_tables()['tblFile'].c.id==tables['tblImportFile'].c.file_id
        )

        __mapping_defined = True

def app_permissions_category():
    """
    Returns a category for all permissions defined for this app
    """
    return license_models.PermissionCategory(u'File')

def app_permissions():
    """
    The permissions used within this application. Can be sync'd with the
    `iqsyncpermissions` command.
    """
    return [
        license_models.Permission(u'file_manager', u"""
            This gives the right to use the file manager.
            """, _("Use the file manager")),
        license_models.Permission(u'view_file', u"""
            This gives the right to view a file (download right).
            Note that group, license and domain level
            access define which File the group has privileges on.
            """, _("Download files")),
        license_models.Permission(u'create_file', u"""
            This gives the right to create a file (upload).
            Note that group, license and domain level
            access define which File the group has privileges on.
            """, _("Upload files")),
        license_models.Permission(u'update_file', u"""
            This gives the right to update a file.
            Note that group, license and domain level
            access define which File the group has privileges on.
            """, _("Update files")),
        license_models.Permission(u'delete_file', u"""
            This gives the right to delete a file.
            Note that group, license and domain level
            access define which File the group has privileges on.
            """, _("Delete files")),
        ]

def app_configs():
    """
    The account configs available for this application. This can be sync'd with the
    `iqsyncpermissions` command.
    """
    from basesoft.web.contrib.account import models as account_models
    configs = []

    config = account_models.Config()
    config.name = 'file.import.failure.email.notification.address'
    config.type = 'string'
    config.default = ''
    config.description = u'The email address to which notifications about import failures should be sent.'
    configs.append(config)

    return configs

