# Copyright (c) 2011 BaseSoft B.V.
# See LICENSE for details.

"""
Graph Controller module
"""
from django.conf import settings
from django.utils import simplejson

from basesoft.web.core.util.controllers import transaction_decorator, AbstractController
from basesoft.web.contrib.graph.models import ColorSet, GraphTheme
from basesoft.web.contrib.license.models import License

from basesoft.web.core.jsonrpc.dispatcher import json_object_hook, JsonRpcJSONEncoder
from basesoft.web.contrib.license.controllers import LicenseController

class GraphController(AbstractController):

    def __init__(self, sa_session=None, locale=None, account_id=None, request=None):
        AbstractController.__init__(self, sa_session, locale, account_id, request)
        if request is not None:
            lc = LicenseController(sa_session, locale, account_id)
            self.site_licensee = lc.get_license_for_request(request)
        else:
            self.site_licensee = sa_session.query(License).filter(License.name==settings.SITE_OWNER_LICENSE_NAME).one()

    def get_color_sets(self, id=None):
        """
        Return the available color sets by id
        """

        # this code can also be executed when in a public enviornment.. like iqreport publisj
        if self.get_account() is None:
            if id is None:
                return self.sa_session.query(ColorSet).order_by(ColorSet.name)
            else:
                return self.sa_session.query(ColorSet).filter(ColorSet.id==id)
        else:
            return self.get_privileged_object_by_id(ColorSet, id, order_by=ColorSet.name)

    def get_graph_themes(self, id=None):
        """
        Return the available graph themes by id
        """
        return self.get_privileged_object_by_id(GraphTheme, id)

    @transaction_decorator
    def save_or_update_color_set(self, state):
        """
        Creates/updates a color set object based on the given state and returns the updated state
        """
        assert isinstance(state, dict)
        assert isinstance(state['name'], unicode)
        assert isinstance(state['color_config'], (list, tuple))

        # save or update
        if state['id'] is None:
            # we create the color set
            cs = ColorSet()
            cs.name = state['name']
            cs.color_config = simplejson.dumps(state['color_config'], ensure_ascii=False, cls=JsonRpcJSONEncoder),
        else:
            # need to update existing color set
            cs = self.get_privileged_object_by_id(ColorSet, state['id'])[0]
            self.assert_privileges_on_object(cs, ColorSet.PRIV_UPDATE)
            cs.name = state['name']
            cs.color_config = simplejson.dumps(state['color_config'], ensure_ascii=False, cls=JsonRpcJSONEncoder),

        if state['id'] is None:
            self.save_default_privilege_object_group(cs)

        self.sa_session.add(cs)
        self.sa_session.flush()

        return self.load_color_set(cs.id)

    def load_color_set(self, id):
        """
        Get the color set state for the supplied id.
        """
        color_set = self.get_color_sets(id).pop()
        colors = None
        if color_set.color_config is not None:
            colors = simplejson.loads(color_set.color_config, object_hook=json_object_hook)

        # return the state of this color set
        return {
            'id': color_set.id,
            'name': color_set.name,
            'color_config': colors,
        }

    def get_color_config(self, color_set):
        """
        Return the list of colors from a color_set object (which has them json encoded)
        @param color_set: a ColorSet object
        @return: a list of hex color codes
        """
        return simplejson.loads(color_set.color_config, object_hook=json_object_hook)
