# Copyright (c) 2009 BaseSoft B.V.
# See LICENSE for details.

from django.conf import settings

from django.utils import simplejson
from basesoft.web.core.jsonrpc.dispatcher import JsonRpcJSONEncoder
from basesoft.web.contrib.graph.models import ColorSet, GraphTheme
from basesoft.web.contrib.license.models import Group, License
from sqlalchemy.orm.exc import NoResultFound
from sqlalchemy.sql.expression import and_

def sync(session):

    # the following colorsets get synced with VIEW on world level

    color_sets = {
        'Default': ['#fe6403', '#4684EE', '#DC3912', '#FF9900', '#008000', '#666666',
                    '#4942CC', '#CB4AC5', '#d6ae00', '#336699', '#dd4477', '#aaaa11',
                    '#66aa00', '#888888', '#994499', '#dd5511', '#22aa99', '#999999',
                    '#705770', '#109618', '#a32929'],


        'Variant 1': ['#666633', '#009933', '#0099CC', '#9999CC', '#330000', '#996600', '#999933', '#FFCC00', '#FF6633', '#CC3366', '#990033', '#336666'],
        'Variant 2': ['#6E2400', '#FF3300', '#DEB100', '#B4773B', '#FF8C00', '#9ACD32', '#556B2F', '#8B008B', '#A9A9A9', '#87CEFA', '#3366CC'],
        'Variant 3': ['#0099CC', '#339933', '#FFCC00', '#CC3333', '#FF6600', '#CC9900', '#336666', '#000000', '#996699', '#CCCCCC', '#66CCFF', '#CCCC00'],
        'Variant 4': ['#3399cc', '#ffcc66', '#999933', '#669999', '#99ccff', '#cc9933', '#cccc99', '#006666', '#ecabff', '#a88dec', '#c2cf69', '#ffa8fe'],

        # fusion charts
        'Variant 5': ["#AFD8F8", "#F6BD0F", "#8BBA00", "#FF8E46" , "008E8E", "#D64646", "#8E468E", "#588526" , "B3AA00", "#008ED6", "#9D080D", "#A186BE" , "CC6600", "#FDC689", "#ABA000", "#F26D7D" , "FFF200", "#0054A6", "#F7941C", "#CC3300" , "006600", "#663300", "#6DCFF6"]
    }

    group = session.query(Group).join(License).filter(and_(License.name==settings.SITE_OWNER_LICENSE_NAME, Group.is_default==True)).one()
    cs_conf = []
    for cs_name in color_sets:
        cs_conf.append({
            'name': cs_name,
            'color_config': simplejson.dumps(color_sets[cs_name], ensure_ascii=False, cls=JsonRpcJSONEncoder),
            'group': group,
            'group_privileges': ColorSet.all_privileges,
            'license_privileges': ColorSet.no_privileges,
            'domain_privileges': ColorSet.no_privileges,
            'world_privileges': ColorSet.PRIV_VIEW
        })
    sync_color_sets(session, cs_conf)

    # graph themes
    sync_graph_themes(session)


def sync_graph_themes(session):

    group = session.query(Group).join(License).filter(and_(License.name==settings.SITE_OWNER_LICENSE_NAME, Group.is_default==True)).one()

    # demo themes
    summer_graphs = {
        'color_theme': 'light',
        'chart_background_top_color': '#fff000',

        # bars
        'bar_stacked': True,

        # pies
        'perspective': 40,
        'depth': 30,
        'occupy': 100,

        # lines
        'line_width': 1
    }

    winter_graphs = {
        'color_theme': 'contrast',
        'chart_background_top_color': '#e6f1f1',

        # bars
        'bar_horizontal': True,

        # pies
        'perspective': 80,
        'depth': 10,
        'occupy': 50,

        # lines
        'line_width': 4
    }

    default_graph_themes = [{
        'name': 'Default',
        'module': None,
        'css_class': None,
        'graph_config': None,
        'default_color_set': session.query(ColorSet).filter(ColorSet.name=='Default').one(),
        'group': group,
        'group_privileges': GraphTheme.all_privileges,
        'license_privileges': GraphTheme.no_privileges,
        'domain_privileges': GraphTheme.no_privileges,
        'world_privileges': GraphTheme.PRIV_VIEW
    }, {
        'name': 'Summer',
        'module': None,
        'css_class': None,
        'graph_config': simplejson.dumps(summer_graphs, ensure_ascii=False, cls=JsonRpcJSONEncoder),
        'default_color_set': session.query(ColorSet).filter(ColorSet.name=='Default').one(),
        'group': group,
        'group_privileges': GraphTheme.all_privileges,
        'license_privileges': GraphTheme.no_privileges,
        'domain_privileges': GraphTheme.no_privileges,
        'world_privileges': GraphTheme.PRIV_VIEW
    }, {
        'name': 'Winter',
        'module': None,
        'css_class': None,
        'graph_config': simplejson.dumps(winter_graphs, ensure_ascii=False, cls=JsonRpcJSONEncoder),
        'default_color_set': session.query(ColorSet).filter(ColorSet.name=='Default').one(),
        'group': group,
        'group_privileges': GraphTheme.all_privileges,
        'license_privileges': GraphTheme.no_privileges,
        'domain_privileges': GraphTheme.no_privileges,
        'world_privileges': GraphTheme.PRIV_VIEW
    }]

    current_themes = [t.name for t in session.query(GraphTheme).all()]
    for theme in default_graph_themes:

        # if we already have a theme with such a name, skip it for now.
        if theme['name'] in current_themes:
            continue

        t = GraphTheme()
        for attr in theme.keys():
            setattr(t, attr, theme[attr])
        session.add(t)
    session.flush()

def sync_color_sets(session, color_sets):
    """
    create/update the color sets
    """

    for cs in color_sets:
        try:
            ncs = session.query(ColorSet).filter(ColorSet.name==cs['name']).one()
        except NoResultFound:
            ncs = ColorSet()

        for attr in cs.keys():
            setattr(ncs, attr, cs[attr])
        session.add(ncs)

    session.flush()
