# Copyright (c) 2011 BaseSoft B.V.
# See LICENSE for details.

from basesoft.web.contrib.graph import models as graph_models
import datetime
from sqlalchemy.orm import mapper, relation
from sqlalchemy import Column, Integer, String, UnicodeText, ForeignKey, DateTime, Unicode, SmallInteger
from basesoft.sqlalchemy.table import IQTable

from basesoft.web.contrib.license.hooks import PrivilegeObjectMapper
from basesoft.web.contrib.license import models as license_models
from basesoft.web.contrib.account import models as account_models
from django.utils.translation import ugettext as _

__mapping_defined = False
__tables_defined = False
tables = {}

def reset():
    global __mapping_defined
    global __tables_defined

    __mapping_defined = False
    __tables_defined = False

def get_tables():
    """
    """
    return tables

def do_tables(metadata):
    global __tables_defined
    if not __tables_defined:

        tblColorSet = IQTable('tblColorSet', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('name', Unicode(50), nullable=False),
            Column('color_config', UnicodeText, nullable=False), # json encoded list of hex color codes

            Column('deleted', SmallInteger, nullable=False, default=0),
            Column('archived', SmallInteger, nullable=False, default=0),

            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now),
        )

        tblColorSetGroup = IQTable('tblColorSetGroup', metadata,
            Column('color_set_id', Integer, ForeignKey('tblColorSet.id', ondelete="CASCADE"), primary_key=True),
            Column('group_id', Integer, ForeignKey('tblGroup.id'), primary_key=True),

            Column('privilege', Integer, nullable=False, primary_key=True, autoincrement=False),
            Column('group_has_privilege', SmallInteger, default=0),
            Column('license_has_privilege', SmallInteger, default=0),
            Column('domain_has_privilege', SmallInteger, default=0),
            Column('world_has_privilege', SmallInteger, default=0),

            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now),
        )

        # Theme stuff
        tblGraphTheme = IQTable('tblGraphTheme', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('name', String(255), nullable=False, unique=True), # name like 'Basesoft Theme'.. selected by the user
            Column('module', String(255), nullable=True), # module name to load from the loader
            Column('css_class', String(255), nullable=True), # class name in the css file to apply to the graphs
            Column('graph_config', UnicodeText, nullable=True),
            Column('default_color_set_id', Integer, ForeignKey('tblColorSet.id')),

            Column('deleted', SmallInteger, nullable=False, default=0),
            Column('archived', SmallInteger, nullable=False, default=0),

            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        tblGraphThemeGroup = IQTable('tblGraphThemeGroup', metadata,
            Column('graph_theme_id', Integer, ForeignKey('tblGraphTheme.id', ondelete='CASCADE'), primary_key=True),
            Column('group_id', Integer, ForeignKey('tblGroup.id'), primary_key=True),
            Column('privilege', Integer, nullable=False, primary_key=True, autoincrement=False),
            Column('group_has_privilege', SmallInteger, default=0),
            Column('license_has_privilege', SmallInteger, default=0),
            Column('domain_has_privilege', SmallInteger, default=0),
            Column('world_has_privilege', SmallInteger, default=0),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        tables['tblColorSet'] = tblColorSet
        tables['tblColorSetGroup'] = tblColorSetGroup
        tables['tblGraphTheme'] = tblGraphTheme
        tables['tblGraphThemeGroup'] = tblGraphThemeGroup

        __tables_defined = True

def do_mappings():
    """
    This function is only called by the django middleware
    when it is needed
    """
    global __mapping_defined
    if not __mapping_defined:

        mapper(graph_models.ColorSet, tables['tblColorSet'],
            extension = PrivilegeObjectMapper(),
            properties = {
                'groups': relation(graph_models.ColorSetGroup, backref="color_set", cascade="all", passive_deletes=True)
            }
        )

        mapper(graph_models.ColorSetGroup, tables['tblColorSetGroup'],
            properties = {
                'group': relation(license_models.Group)
            }
        )

        mapper(graph_models.GraphTheme, tables['tblGraphTheme'],
            extension = PrivilegeObjectMapper(),
            properties = {
                'default_color_set': relation(graph_models.ColorSet),
                'groups': relation(graph_models.GraphThemeGroup, backref='graph_theme', cascade="all", passive_deletes=True)
        })

        mapper(graph_models.GraphThemeGroup, tables['tblGraphThemeGroup'], properties = {
            'group': relation(license_models.Group)
        })

        __mapping_defined = True

def app_permissions_category():
    """
    Returns a category for all permissions defined for this app
    """
    return license_models.PermissionCategory(u'Graph')

def app_permissions():
    """
    The permissions used within this application. Can be sync'd with the
    `iqsyncpermissions` command.
    """
    return [
        license_models.Permission(u'color_set_manager', u"""
            This gives the right to use the color set manager.
            """, _("Use the color set manager")),

        # theme
        license_models.Permission(u'view_graph_theme', u"""
            The permission to view graph themes
        """, _("View graph themes")),

    ]

def app_configs():
    """
    The account configs available for this application. This can be sync'd with the
    `iqsyncpermissions` command.
    """
    configs = []

    config = account_models.Config()
    config.name = 'basesoft.web.contrib.graph.default_color_set'
    config.type = 'string'
    config.default = ''
    config.description = u'A string that indicates the name of the color set to be used for charts.'
    configs.append(config)

    return configs
