# Copyright (c) 2008 BaseSoft B.V.
# See LICENSE for details.

import datetime
from django.utils.translation import ugettext as _

from sqlalchemy.orm import mapper, relation
from sqlalchemy import Column, Integer, String, ForeignKey, DateTime, Boolean, SmallInteger, UniqueConstraint
from basesoft.sqlalchemy.table import IQTable

from basesoft.web.contrib.i18n import models as i18n_models
from basesoft.web.contrib.account import models as account_models
from basesoft.web.contrib.license import models as license_models
from basesoft.web.contrib.i18n.db_sync import sync_locales, sync_countries

__mapping_defined = False
__tables_defined = False
tables = {}

def reset():
    global __mapping_defined
    global __tables_defined

    __mapping_defined = False
    __tables_defined = False

def get_tables():
    """
    """
    return tables

def do_tables(metadata):
    global __tables_defined
    if not __tables_defined:
        # locale
        tblLocale = IQTable('tblLocale', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('name', String(255), nullable=True),
            Column('locale', String(5), nullable=True, index=True),
            Column('availability', SmallInteger, default=0, nullable=False),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now),
        )

        tblDomainLocale = IQTable('tblDomainLocale', metadata,
            Column('locale_id', ForeignKey('tblLocale.id'), primary_key=True),
            Column('domain_id', ForeignKey('tblDomain.id'), primary_key=True, nullable=False),
            Column('availability', SmallInteger, default=0, nullable=False),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now),
        )

        # country
        tblCountry = IQTable('tblCountry', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('name', String(255), nullable=True),
            Column('iso_3166_a2', String(2), nullable=True),
            Column('iso_3166_a3', String(3), nullable=True),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now),
        )

        # domains can override country names
        tblDomainCountry = IQTable('tblDomainCountry', metadata,
            Column('country_id', ForeignKey('tblCountry.id'), primary_key=True),
            Column('domain_id', ForeignKey('tblDomain.id'), primary_key=True, nullable=False),
            Column('name', String(255), nullable=True),
            Column('active', Boolean, nullable=True),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now),
        )

        # language is used for mapping purposes, the iso_639_1 column is used to determine the Locale (in conjunction with Country iso_3166_a2)
        tblLanguage = IQTable('tblLanguage', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('name', String(255), nullable=False),
            Column('iso_639_1', String(2), nullable=False, unique=True),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now),
        )

        # different than the Domain Country, as these are just aliases to Languages.
        tblDomainLanguage = IQTable('tblDomainLanguage', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('language_id', ForeignKey('tblLanguage.id'), nullable=False),
            Column('domain_id', ForeignKey('tblDomain.id'), nullable=False),
            Column('name', String(255), nullable=False),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now),
            UniqueConstraint('domain_id', 'name'),
        )

        tables['tblLocale'] = tblLocale
        tables['tblDomainLocale'] = tblDomainLocale

        tables['tblCountry'] = tblCountry
        tables['tblDomainCountry'] = tblDomainCountry

        tables['tblLanguage'] = tblLanguage
        tables['tblDomainLanguage'] = tblDomainLanguage
        __tables_defined = True

def do_mappings():
    """
    This function is only called by the django middleware
    when it is needed
    """
    global __mapping_defined
    if not __mapping_defined:

        mapper(i18n_models.Locale, tables['tblLocale'])
        mapper(i18n_models.DomainLocale, tables['tblDomainLocale'], properties = {
            'locale': relation(i18n_models.Locale),
            'domain': relation(license_models.Domain),
        })
        mapper(i18n_models.Country, tables['tblCountry'])
        mapper(i18n_models.DomainCountry, tables['tblDomainCountry'], properties = {
            'country': relation(i18n_models.Country),
            'domain': relation(license_models.Domain),
        })
        mapper(i18n_models.Language, tables['tblLanguage'])
        mapper(i18n_models.DomainLanguage, tables['tblDomainLanguage'], properties = {
            'language': relation(i18n_models.Language),
            'domain': relation(license_models.Domain),
        })

        __mapping_defined = True


def fill_data(conn):
    from basesoft.web.core.util.db import DatabaseSession
    sa_session = DatabaseSession().Session

    # sync locales comes from db_sync but we run it here once too
    sync_locales(sa_session)

    # same for countries
    sync_countries(sa_session)

def app_permissions_category():
    """
    Returns a category for all permissions defined for this app
    """
    return license_models.PermissionCategory(u'Internationalization')

def app_permissions():
    """
    The permissions used within this application. Can be sync'd with the
    `iqsyncpermissions` command.
    """
    return [
        license_models.Permission(u'manage_domain_country_mapping', u"""
            This gives the right to manage Domain Country mappings.
            """, _("Manage domain country mapping")),
        license_models.Permission(u'manage_domain_language_aliases', u"""
            This gives the right to manage Domain Language aliases.
            """, _("Manage domain country aliases")),
    ]

def app_configs():
    """
    The account configs available for this application. This can be sync'd with the
    `iqsyncpermissions` command.
    """
    configs = []

    config = account_models.Config()
    config.name = 'basesoft.web.contrib.locale'
    config.type = 'string'
    config.default = '' # empty means dont change language (let django do it)
    config.description = u'The users Locale settings (Language, number formats, etc)'
    configs.append(config)

    return configs
