# Copyright (c) 2008 BaseSoft B.V.
# See LICENSE for details.

"""
License Controller module

Duplicating a license can be done and should be done only on Permission level.
It makes no sense to duplicate a license, including contact etc. information
"""
from django.utils.translation import ugettext as _
from django.forms.util import ValidationError
from django.conf import settings

from basesoft.web.core.util.controllers import transaction_decorator, get_models_of_type,\
    LicenseConfigController, StateManager
from sqlalchemy import and_
from sqlalchemy.exc import InvalidRequestError
from sqlalchemy.orm.exc import NoResultFound

from basesoft.web.contrib.license.models import Permission, UsageOption, Domain, License, Reseller, Group, \
    LicenseUsage, LicensePermission, PrivilegeObjectHook, GroupPermission, Role,\
    RoleLicenseConfig

from basesoft.web.contrib.contact.controllers import SimpleContactController
from basesoft.web.contrib.contact.models import Person, Company

from basesoft.web.core.exceptions import SecurityException, PublicSecurityException, PublicUserException

from basesoft import logging
from basesoft.web.contrib.account.models import Account, GroupAccount

from basesoft.web.core.input.validation.plugins import Validation
from basesoft.util.dict import dict_merge
from basesoft.util.bitwise import get_bits
from basesoft.util.ordered_dict import OrderedDict

logger = logging.getLogger('basesoft.license')

class LicenseController(SimpleContactController):
    """
    Contains methods for license control
    """

    def get_license_for_request(self, request):

        basesoft = self.sa_session.query(License).filter(License.name==settings.SITE_OWNER_LICENSE_NAME).one()

        # we are going to check if there is a license with this host.
        lookup = request.get_host()
        try:
            license = self.sa_session.query(License).filter(License.http_host==lookup).one()
            # apparently there is no way to determine the branding of the requested host.
        except InvalidRequestError:
            # Get the BaseSoft License as a Reseller.
            return basesoft

        # the license is self branded, this means that they have their own
        # templates for the IQPortal instance.
        if license.self_branded == 1:
            return license

        # in case it's not a self branded license we might be dealing with a
        # reseller based license.
        if license.reseller is None:
            # apparently not. Get the BaseSoft License as a Reseller.
            return basesoft

        # the reseller is the site licensee; they have a license with branding.
        return license.reseller

    def __get_license(self, id):
        """
        Private method for getting licenses (assumption that rights to the requested license are validated elsewhere)
        """
        assert isinstance(id, (int, long))
        return self.sa_session.query(License).filter(License.id==id).one()

    def get_licenses(self, id=None, polymorphic_type=None):
        """
        Return a list with all Licenses this account has view_licese permission on.
        """
        # TODO: this won't work in case user requests only 'normal' licenses (type License) - #3214
        # eventually we will separate this 'normal' licenses also

        if polymorphic_type is None:
            polymorphic_type = License
        else:
            pp = [p['model'] for p in self.get_available_license_types()]
            if polymorphic_type not in pp:
                raise SecurityException('Save license: invalid polymorphic_type supplied (%s)' % polymorphic_type)

        # if you can view all domains, you automatically can view all the licenses.
        if self.has_permissions('view_all_domains'):
            if id is None:
                return self.sa_session.query(polymorphic_type).all()
            if not isinstance(id, list):
                id = [id]
            return self.sa_session.query(polymorphic_type).filter(polymorphic_type.id.in_(id)).all()

        # if current license is a Reseller, get licenses that were created by it
        # TODO: if reseller, we should use webshop controller - #3194
        current_license = self._get_default_group().license
        if isinstance(current_license, Reseller):
            filter = Domain.reseller==current_license
            if id is not None:
                if not isinstance(id, list):
                    id = [id]
                filter = and_(filter, polymorphic_type.id.in_(id))
            return self.sa_session.query(polymorphic_type).join(
                   Domain, Domain.id==polymorphic_type.domain_id).filter(filter).all()

        # with view_domain can view all licenses from current domain
        if self.has_permissions('view_domain'):
            if id is None:
                return self.sa_session.query(polymorphic_type).filter(polymorphic_type.domain_id==current_license.domain.id).all()
            if not isinstance(id, list):
                id = [id]
            return self.sa_session.query(polymorphic_type).filter(and_(polymorphic_type.id.in_(id),
                                                              polymorphic_type.domain_id==current_license.domain.id)).all()

        # getting details for licenses should only be possible with view_license permission
        self.assert_permissions('view_license')
        account_licenses = self.get_account_licenses_with_permssions('view_license')

        # check if requested license id(s) have correct permissions for this account
        if id is not None:
            if not isinstance(id, list):
                id = [id]
            a_l_ids = [x.id for x in account_licenses]
            for i in id:
                try:
                    if int(i) not in a_l_ids:
                        raise SecurityException("Insufficient permissions for getting license id %s" % i)
                except ValueError:
                    continue
        if id is None:
            return account_licenses
        else:
            # return only requested licenses
            return [x for x in account_licenses if x.id in id]

    def get_available_license_types(self):
        """
        Return all available license types.
        """
        models = get_models_of_type(License, False, None, True)
        rv = []
        for model in models:
            # only include if user has correct permissions
            if self.has_permissions(*model['model'].REQUIRED_PERMISSIONS):
                rv.append(model)
        return rv

    def get_domain_licenses(self, domain_id=None):

        if self.get_default_group().license.controller != self.__class__:
            c = self.get_default_group().license.controller(self.sa_session, account_id=self.get_account().id, request=self.request)
            return c.get_domain_licenses(domain_id=domain_id)

        if self.has_permissions('view_all_domains'):
            if domain_id is not None:
                return self.sa_session.query(License).filter(License.domain_id==domain_id).all()
            return self.sa_session.query(License).all()

        current_license = self._get_default_group().license
        # if reseller, can only get licenses from it's domains
        # TODO: if reseller, we should use webshop controller - #3194
        if isinstance(current_license, Reseller):
            filter = Domain.reseller==current_license
            if domain_id is not None:
                filter = and_(filter, License.domain_id==domain_id)
            return self.sa_session.query(License).join(
                    Domain, Domain.id==License.domain_id).filter(filter).all()

        self.assert_permissions('view_domain', 'view_license')
        if domain_id is not None and domain_id != current_license.domain.id:
            raise SecurityException("Trying to get license for domain id %s without permissions" % domain_id)
        return self.sa_session.query(License).filter(License.domain_id==current_license.domain.id).all()

    def get_domains(self, id=None):
        """
        Returns a list with domains.
        """
        if self.has_permissions('view_all_domains'):
            if id is None:
                return self.sa_session.query(Domain).all()
            if not isinstance(id, list):
                id = [id]
            return self.sa_session.query(Domain).filter(Domain.id.in_(id)).all()

        current_license = self._get_default_group().license
        # if reseller, can only get its own domains
        # TODO: if reseller, we should use webshop controller - #3194
        if isinstance(current_license, Reseller):
            filter = Domain.reseller==current_license
            if id is not None:
                if not isinstance(id, list):
                    id = [id]
                filter = and_(filter, Domain.id.in_(id))
            return self.sa_session.query(Domain).filter(filter).all()

        self.assert_permissions('view_domain')
        id = self._get_default_group().license.domain.id
        return self.sa_session.query(Domain).filter(Domain.id==id).all()

    def get_permissions(self, id=None):
        """
        @param id: None, list or int
        @return: Permission objects

        Returns a list with all permissions in case id is None or a list of permissions with supplied id/ids.
        (a list of one permission in case a valid id is supplied)
        """
        # if user has "view_all_permissions" he can obviously view all permissions
        if self.has_permissions('view_all_permissions'):
            if id is None:
                return self.sa_session.query(Permission).all()
            if not isinstance(id, list):
                id = [id]
            return self.sa_session.query(Permission).filter(Permission.id.in_(id)).all()

        current_license = self._get_default_group().license
        # if reseller, can only get permissions from its own domains
        # TODO: if reseller, we should use webshop controller - #3194
        if isinstance(current_license, Reseller):
            filter = Domain.reseller==current_license
            if id is not None:
                if not isinstance(id, list):
                    id = [id]
                filter = and_(filter, Permission.id.in_(id))
            perms = self.sa_session.query(Permission).join(LicensePermission).join(License).join(
                    Domain, Domain.id==License.domain_id).filter(filter).all()
            ret = []
            for p in perms:
                if p not in ret:
                    ret.append(p)
            return ret

        # if user has "view_domain" he can view all domain information, which includes the domain's permissions
        if self.has_permissions('view_domain'):
            filter = License.domain==self._get_default_group().license.domain
            if id is not None:
                if not isinstance(id, list):
                    id = [id]
                filter = and_(filter, Permission.id.in_(id))
            perms = self.sa_session.query(Permission).join(LicensePermission).join(License).filter(filter).all()
            ret = []
            for p in perms:
                if p not in ret:
                    ret.append(p)
            return ret

        # getting permissions for current license should only be possible with view_license permission
        # only use self._get_default_group().license.permissions if user has correct perm

        # if user has "view_license" he can view all license information, which includes the license's permissions
        self.assert_permissions('view_license')
        if id is None:
            return self._get_default_group().license.permissions
        if not isinstance(id, list):
            id = [id]
        return [p for p in self._get_default_group().license.permissions if p.id in id]

    def get_groups_with_permissions(self, permissions):
        if not isinstance(permissions, list):
            permissions = [permissions]

        return self.sa_session.query(Group).join(
                (GroupAccount, GroupAccount.group_id==Group.id)
            ).join(
                (GroupPermission, GroupPermission.group_id==Group.id)
            ).join(
                (Permission, Permission.id==GroupPermission.permission_id)
            ).filter(
                and_(
                     GroupAccount.account_id==self.get_account().id,
                     Permission.name.in_(permissions),
                     GroupAccount.active==1 #2993
                )
            ).order_by(GroupAccount.is_default.desc()).all()

    def get_usage_options(self, id=None):
        """
        @param id: None, list or int
        @return: UsageOptions objects

        Returns a list with all usage options in case id is None or a list of usage options with supplied id/ids.
        (a list of one usage option in case a valid id is supplied)
        """
        # if user has "view_all_usage_options" he can obviously view all usage options
        if self.has_permissions('view_all_usage_options'):
            if id is None:
                return self.sa_session.query(UsageOption).all()
            if not isinstance(id, list):
                id = [id]
            return self.sa_session.query(UsageOption).filter(UsageOption.id.in_(id)).all()

        current_license = self._get_default_group().license
        # if reseller, can only get usage options from its own domains
        # TODO: if reseller, we should use webshop controller - #3194
        if isinstance(current_license, Reseller):
            filter = Domain.reseller==current_license
            if id is not None:
                if not isinstance(id, list):
                    id = [id]
                filter = and_(filter, UsageOption.id.in_(id))
            opts = self.sa_session.query(UsageOption).join(LicenseUsage).join(License).join(
                    Domain, Domain.id==License.domain_id).filter(filter).all()
            ret = []
            for p in opts:
                if p not in ret:
                    ret.append(p)
            return ret

        # if user has "view_domain" he can view all domain information, which includes the domain's usage options
        if self.has_permissions('view_domain'):
            filter = License.domain==self._get_default_group().license.domain
            if id is not None:
                if not isinstance(id, list):
                    id = [id]
                filter = and_(filter, UsageOption.id.in_(id))
            opts = self.sa_session.query(UsageOption).join(LicenseUsage).join(License).filter(filter).all()
            ret = []
            for p in opts:
                if p not in ret:
                    ret.append(p)
            return ret

        # if user has "view_license" he can view all license information, which includes the license's usage options
        self.assert_permissions('view_license')
        filter = License.id==self._get_default_group().license.id
        if id is not None:
            if not isinstance(id, list):
                id = [id]
            filter = and_(filter, UsageOption.id.in_(id))
        return self.sa_session.query(UsageOption).join(LicenseUsage).join(License).filter(filter).all()

    def get_privilege_objects(self, name=None, include_models=False):
        """
        Get all the models.py objects which are defined as being Privilege Objects.
        TODO: check for the correct permissions - #3215
        """
        from basesoft.web.core.models import PrivilegeObject
        return get_models_of_type(PrivilegeObject, True, name, include_models)

    def get_privilege_object_privileges(self, object, object_id):
        """
        Get a dictionary label,value of all the available privileges on a certain privilege object.
        """
        model = self.get_privilege_objects(object)
        #instance = self.sa_session.query(model).filter(model.id==object_id).one()
        rv = []
        # only get the privs the current user has access too.
        for p in model().get_privileges():
            priv, label = p
            #if self.has_privileges_on_object(instance, priv):
            rv.append({'value': priv, 'label': label})

        return rv


    def get_groups(self, id=None):
        """
        @param id: None, list or int
        @return: Group objects

        Returns a list with all groups in case id is None or a list of groups with supplied id/ids.
        (a list of one group in case a valid id is supplied)
        """

        if id is not None and not isinstance(id, list):
            id = [id]

        # if user has "view_all_domains" he can view all groups also
        if self.has_permissions('view_all_domains'):
            if id is None:
                return self.sa_session.query(Group).all()
            return self.sa_session.query(Group).filter(Group.id.in_(id)).all()

        current_license = self._get_default_group().license
        # if reseller, can only get groups from its own domains
        # TODO: if reseller, we should use webshop controller - #3194
        if isinstance(current_license, Reseller):
            filter = Domain.reseller==current_license
            if id is not None:
                filter = and_(filter, Group.id.in_(id))
            return self.sa_session.query(Group).join(License).join(
                    Domain, Domain.id==License.domain_id).filter(filter).all()

        # if user has "view_domain" he can view only groups from its domain
        if self.has_permissions('view_domain'):
            filter = License.domain==self._get_default_group().license.domain
            if id is not None:
                filter = and_(filter, Group.id.in_(id))
            return self.sa_session.query(Group).join(License).filter(filter).all()

        # if user has "view_license" he can view only groups from the licenses he is in
        account_licenses = self.get_account_licenses_with_permssions('view_license')
        if len(account_licenses) == 0:
            if id is None:
                return self.get_active_groups()
            return [g for g in self.get_active_groups() if g.id in id]

        license_ids = [l.id for l in account_licenses]
        filter = Group.license_id.in_(license_ids)
        if id is not None:
            filter = and_(filter, Group.id.in_(id))

        return self.sa_session.query(Group).filter(filter).all()

    def get_groups_for_license(self, license, id=None):
        """
        Returns a list of Groups that are attached to the supplied License.
        """
        # if user has "view_all_domains" he can view all groups also
        if self.has_permissions('view_all_domains'):
            filter = Group.license==license
            if id is not None:
                if not isinstance(id, list):
                    id = [id]
                filter = and_(filter, Group.id.in_(id))
            return self.sa_session.query(Group).filter(filter).all()

        # if reseller, can only get groups from its own domains
        # TODO: if reseller, we should use webshop controller - #3194
        current_license = self._get_default_group().license
        if isinstance(current_license, Reseller):
            filter = and_(Domain.reseller==current_license, Group.license==license)
            if id is not None:
                if not isinstance(id, list):
                    id = [id]
                filter = and_(filter, Group.id.in_(id))
            return self.sa_session.query(Group).join(License).join(
                    Domain, Domain.id==License.domain_id).filter(filter).all()

        # if user has "view_domain" he can view only groups from its domain
        if self.has_permissions('view_domain'):
            # if provided license is not from current domain, raise SecurityException
            if license.domain != self._get_default_group().license.domain:
                raise SecurityException("Insufficient permission to view groups for license %s" % license.name)
            filter = and_(License.domain==self._get_default_group().license.domain, Group.license==license)
            if id is not None:
                if not isinstance(id, list):
                    id = [id]
                filter = and_(filter, Group.id.in_(id))
            return self.sa_session.query(Group).join(License).filter(filter).all()

        # if user has "view_license" he can view only groups from the licenses he is in
        self.assert_permissions('view_license')
        account_licenses = self.get_account_licenses_with_permssions('view_license')
        # if provided license is not in account_licenses, raise SecurityException
        if license not in account_licenses:
            raise SecurityException("Insufficient permission to view groups for license %s" % license.name)
        filter = Group.license==license
        if id is not None:
            if not isinstance(id, list):
                id = [id]
            filter = and_(filter, Group.id.in_(id))
        return self.sa_session.query(Group).filter(filter).all()

    def get_groups_for_domain(self, domain_id=None):
        """
        Get all the groups for all the licenses in the provided domain.
        """
        if self.has_permissions('view_all_domains'):
            if domain_id is None:
                return self.sa_session.query(Group).all()
            else:
                return self.sa_session.query(Group).join(Group.license).filter(License.domain_id==domain_id).all()
        if self.has_permissions('view_domain'):
            if domain_id is None:
                domain_id = self._get_default_group().license.domain.id
            else:
                # if provided domain is not the current one, raise SecurityException
                if domain_id != self._get_default_group().license.domain.id:
                    raise SecurityException("Insufficient permission to view groups for domain %s" % domain_id)
            return self.sa_session.query(Group).join(Group.license).filter(License.domain_id==domain_id).all()
        # get all groups for all licenses the user had view_license on
        account_licenses = self.get_account_licenses_with_permssions('view_license')
        groups = []
        for l in account_licenses:
            groups = groups + self.get_groups_for_license(l)
        return groups

    def get_own_domain_groups(self):
        """
        Get all the groups for all the licenses in the domain this user is a part of
        """
        domain_id = self._get_default_group().license.domain.id
        return self.get_groups_for_domain(domain_id)

    def get_accounts_for_group(self, group):
        """
        Returns all accounts in the specified group
        """

        # check if the user has access to the group
        if self.has_any_permissions('view_all_domains', 'view_domain', 'view_license'):
            groups = self.get_groups_for_domain()
        else:
            groups = self._active_account_groups

        if group not in groups:
            return []

        return self.sa_session.query(Account).join(GroupAccount).filter(GroupAccount.group_id==group.id).all()


    def get_groups_for_privilege_object(self, object_name, id, filter_owner=True):
        """
        Get all the groups available for the provided privilege object.
        """
        # we should have the rights to do privilege delegation on this object.
        self.assert_privilege_delegation(object_name, id)
        return self.get_groups()

    def get_object_privilege_group(self, object_name, id, group_id):
        """
        """
        # we should have the rights to do privilege delegation on this object.
        self.assert_privilege_delegation(object_name, id)

        from basesoft.util.bitwise import get_bits

        model = self.get_privilege_objects(object_name)
        # Query the instance of the supplied object
        instance = self.sa_session.query(model).filter(model.id==id).one()
        for go in instance.groups:
            if go.group.id == group_id:
                return {'group_id': group_id,
                        'group_privileges': get_bits(go.group_privileges),
                        'license_privileges': get_bits(go.license_privileges)}
        raise SecurityException('Accessing non-privileged group from the privilege object')

    def get_privilege_object_settings(self, object_name, id):
        """
        Get the privilege and group setting of the object itself. This is the owner information of the object.
        """
        # we should have the rights to do privilege delegation on this object.
        self.assert_privilege_delegation(object_name, id)

        from basesoft.util.bitwise import get_bits

        model = self.get_privilege_objects(object_name)
        # Query the instance of the supplied object
        instance = self.sa_session.query(model).filter(model.id==id).one()

        return {
            'group_id': instance.group.id,
            'group_privileges': get_bits(instance.group_privileges),
            'license_privileges': get_bits(instance.license_privileges),
            'domain_privileges': get_bits(instance.domain_privileges)
        }

    def assert_privilege_delegation(self, object_name, id):
        """
        We validate if the current account is part of any group that has the rights to distribute
        privileges on the provided object.
        """

        # always ok.
        if self.has_permissions('privilege_object_administrator'):
            return
        assert id is not None

        self.assert_permissions('distribute_group_permissions')
        if id is None:
            return

        model = self.get_privilege_objects(object_name)
        try:
            instance = self.sa_session.query(model).filter(model.id==id).one()
        except NoResultFound:
            raise SecurityException('Trying to distribute privileges on a non-existing object')

        self.assert_privileges_on_object(instance, instance.PRIV_DELEGATE_PRIVILEGES)

    def has_object_group_privileges(self, object_name, id, group_id):
        """
        Checks whether or not the group is already linked to the supplied object.
        """
        model = self.get_privilege_objects(object_name)
        # Query the instance of the supplied object
        instance = self.sa_session.query(model).filter(model.id==id).one()
        if int(group_id) in [g.group.id for g in instance.groups]:
            return True
        return False

    @transaction_decorator
    def delete_group_privileges(self, object_name, id, group_priv_keys):
        """
        Removes the groups from the attached group objects
        """
        self.assert_privilege_delegation(object_name, id)

        model = self.get_privilege_objects(object_name)
        instance = self.sa_session.query(model).filter(model.id==id).one()
        group_obj, field = instance.get_group_object()

        account_group_ids = [g.id for g in self._get_active_groups()]

        for i in group_priv_keys:
            gid, privilege = i

            # cannot delete privileges for own group.
            if gid in account_group_ids:
                raise PublicSecurityException(_('Cannot delete privileges for a group you are part of.'))

            go = self.sa_session.query(group_obj).filter(
                and_(
                    getattr(group_obj, field)==instance,
                    group_obj.group_id==gid,
                    group_obj.privilege==privilege
                )
            ).one()
            self.sa_session.delete(go)

    @transaction_decorator
    def save_privilege_delegation(self, object_name, id, group_id, privileges, group_has_privilege, license_has_privilege=None, domain_has_privilege=None):
        """
        Save the set of object privileges
        """
        # we should have the rights to do privilege delegation on this object.
        self.assert_privilege_delegation(object_name, id)

        if not self.has_permissions('view_license') and license_has_privilege is not None:
            raise SecurityException('Trying to distribute license privileges on object without correct permissions on license')
        if not self.has_permissions('view_domain') and domain_has_privilege is not None:
            raise SecurityException('Trying to distribute domain privileges on object without correct permissions on domain')

        model = self.get_privilege_objects(object_name)
        # Query the instance of the supplied object
        instance = self.sa_session.query(model).filter(model.id==id).one()

        account_group_ids = [g.id for g in self._get_active_groups()]

        if not isinstance(privileges, (list, tuple)):
            privileges = [privileges]

        # cannot unset privileges for own group.
        if group_id in account_group_ids:
            #self.assert_privileges_on_object(instance, *privileges)
            if not group_has_privilege:
                raise PublicSecurityException(_('Cannot delete privileges for a group you are part of.'))

        # do we already have a Group object attached for this group & privilege
        for go in instance.groups:
            if go.privilege not in privileges:
                continue
            if go.group.id != group_id:
                continue
            privileges = [p for p in privileges if p != go.privilege]
            go.group_has_privilege = group_has_privilege
            if license_has_privilege is not None:
                go.license_has_privilege = license_has_privilege
            if domain_has_privilege is not None:
                go.domain_has_privilege = domain_has_privilege
            self.sa_session.add(go)

            # no more privileges to distribute, we're done!
            if len(privileges) == 0:
                return instance

        model, key = instance.get_group_object()
        # add the remaining privileges as new Group objects.
        for p in privileges:
            go = model()
            go.group = self.get_groups(group_id).pop()
            go.privilege = p
            go.group_has_privilege = group_has_privilege
            if license_has_privilege is not None:
                go.license_has_privilege = license_has_privilege
            if domain_has_privilege is not None:
                go.domain_has_privilege = domain_has_privilege
            setattr(go, key, instance)
            instance.groups.append(go)
            self.sa_session.add(go)

        return instance

    def set_privilege_object_query(self, cq, object_name, id):
        """
        This builds a query based on the provided privilege object and sets it as a custom query object
        """
        self.assert_privilege_delegation(object_name, id)

        from sqlalchemy.orm import object_mapper
        from sqlalchemy import select
        from basesoft.web.contrib.license.sa_datamapper import get_tables

        instance = self.get_privilege_objects(object_name)()
        group_instance = instance.get_group_object()[0]()

        objTbl = object_mapper(instance).local_table
        groupTbl = object_mapper(group_instance).local_table
        tbls = get_tables()

        cq.set_query_where(objTbl.c.id==id)

        from_obj = objTbl.join(groupTbl).join(tbls['tblGroup'], groupTbl.c.group_id==tbls['tblGroup'].c.id)
        cols = [
            tbls['tblGroup'].c.id,
            tbls['tblGroup'].c.name.label('Group'),
            groupTbl.c.group_privileges,
            groupTbl.c.license_privileges
        ]
        query = select(columns=cols, from_obj=from_obj)

        cq.set_query(query)

    @transaction_decorator
    def unset_default_group(self, group):
        """
        Unsets the default group if it is not the supplied group
        """
        groups = self.get_groups_for_license(group.license)
        for g in groups:
            if g.is_default and g is not group:
                g.is_default = False
                self.sa_session.add(g)

    def validate_default_group(self, license_id, group_id=None):
        """
        If we dont have a default group set in the groups of the license OR the default group
        is the supplied group, this will return True
        """
        license = self.get_licenses(license_id)[0]
        groups = self.get_groups_for_license(license)
        for group in groups:
            if group.id == group_id and group.is_default:
                return True
            if group.is_default:
                return False
        return True

    @transaction_decorator
    def save_or_update_usage_option(self, id, name, description):
        """
        @param name: C{str} Short name of the usage option
        @param description: C{str} Description of what the usage option means

        This method takes care of saving and updating the permissions.
        """
        if id is not None:
            usage_option = self.sa_session.query(UsageOption).get(id)
        else:
            usage_option = UsageOption()

        usage_option.name = name
        usage_option.description = description

        self.sa_session.add(usage_option)
        return usage_option


    @transaction_decorator
    def save_or_update_domain(self, id, name=None, settings=None, status=None):
        """
        @param name: C{str} Name of the Domain
        @param id: C{int} id of the Domain

        This method takes care of saving and updating the Domains.
        """
        # assert name is not empty
        if id is not None:
            self.assert_permissions("update_domain")
            domain = self.sa_session.query(Domain).get(id)
        else:
            self.assert_permissions("create_domain")
            domain = Domain()
            l = self._get_default_group()
            if l and isinstance(l.license, Reseller):
                domain.reseller = l.license

        if name is not None:
            domain.name = name
        if settings is not None:
            domain.settings = settings
        if status is not None:
            domain.status = status

        self.sa_session.add(domain)
        return domain

    @transaction_decorator
    def save_domain_group(self, group_id, domains):
        """
        Links domains to groups
        @param group_id: the id of the group to which we want to link domains
        @param domains: a list of domain ids that we want to link to the group
        @return: the group object
        """
        group = self.get_groups(group_id)[0]
        group.domains = self.get_domains(domains)
        self.sa_session.add(group)
        return group


    def update_master_data(self, id, state, auto_approve=False):
        self.assert_permissions('update_license_masterdata', license_id=id)
        license = self.__get_license(id)
        return self._update_master_data(license, state, auto_approve)

    @transaction_decorator
    def _update_master_data(self, license, state, auto_approve=False):
        if state.has_key('company_bank_account_address'):
            if not state.has_key('bank_data'):
                state['bank_data'] = {}
            state['bank_data']['address_data'] = state['company_bank_account_address']

        if license.temp_company is None:
            try:
                original = self._load_company_state(license.company)
            except NoResultFound: # Company didnt exist yet
                original = {}

            original['is_temp'] = Company.TEMPORARY_STATUS_APPLIED
            original['dbID'] = None # unset the ids
            if original.has_key('address_data'):
                original['address_data']['dbID'] = None
            if original.has_key('bank_data'):
                original['bank_data']['dbID'] = None
            if original.has_key('contact_person_data'):
                original['contact_person_data']['dbID'] = None
            if original.has_key('invoice_person_data'):
                original['invoice_person_data']['dbID'] = None
        else:
            original = self._load_company_state(license.temp_company)

        if original.has_key('invoice_person_data'):
            if original['invoice_person_data'].has_key('is_same_person_as'):
                del original['invoice_person_data']['is_same_person_as']

        state = dict_merge(original, state)
        if state.has_key('company'):
            for field, value in state['company'].iteritems():
                state[field] = value

        license.temp_company = self._save_or_update_company(original['dbID'], state)
        self.sa_session.add(license)

        if not auto_approve:
            from basesoft.web.contrib.notification.controllers import NotificationController
            nc = NotificationController(self.sa_session)

            # send a notification to the default group of this domain.
            n = nc.set_notification('Master data altered for %(company_name)s' % {'company_name': license.name}, 'The master data for "%(company_name)s" has been altered and requires a review for approval.' % {'company_name': license.name})
            nc.create_system_notification(n, self.get_default_group().license.domain.default_group)

            self.logger.info('Master data change request saved for license %s by Account %s' % (license.id, self.get_account().id))
        else:
            self.approve_master_data_change(license.id, auto_approve=auto_approve)

        return license

    @transaction_decorator
    def reject_master_data_change(self, id, reason):
        """
        Rejects the change of master data.
        """
        self.assert_permissions('license_masterdata_change_approval')

        from basesoft.web.contrib.notification.controllers import NotificationController
        nc = NotificationController(self.sa_session, account_id=self.get_account().id)
        notification = nc.set_notification('Master data change rejected', reason)

        # get all groups in license which have master data change permissions.
        recipients = self.get_groups_in_license_with_permissions('update_license_masterdata', license_id=id)
        nc.create_system_notification(notification, recipients, for_license=True)

        license = self.get_licenses(id).pop()
        license.temp_company.is_temp = Company.TEMPORARY_STATUS_REJECTED
        self.sa_session.add(license)
        return license

    @transaction_decorator
    def approve_master_data_change(self, id, auto_approve=False):
        """
        Approves the change to the master data by applying the temporary company as the new company information.
        """
        if not auto_approve:
            self.assert_permissions('license_masterdata_change_approval')
            license = self.get_licenses(id).pop()
        else:
            license = self.__get_license(id)

        original = self._load_company_state(license.company)
        state = self._load_company_state(license.temp_company)

        if original.has_key('invoice_person_data'):
            if original['invoice_person_data'].has_key('is_same_person_as'):
                del original['invoice_person_data']['is_same_person_as']

        del state['dbID']
        state['polymorphic_type'] = 'Company'
        if state.has_key('address_data'):
            del state['address_data']['dbID']
        if state.has_key('bank_data'):
            del state['bank_data']['dbID']
        if state.has_key('contact_person_data'):
            del state['contact_person_data']['dbID']
        if state.has_key('invoice_person_data'):
            del state['invoice_person_data']['dbID']

        state = dict_merge(original, state)

        if not state.has_key('dbID'):
            state['dbID'] = None
        state['is_temp'] = False
        license.company = self._save_or_update_company(state['dbID'], state)
        license.name = license.company.name
        license.temp_company = None
        self.sa_session.add(license)

        if not auto_approve:
            from basesoft.web.contrib.notification.controllers import NotificationController
            nc = NotificationController(self.sa_session, account_id=self.get_account().id)
            notification = nc.set_notification('Master data change approved', 'Your master data changes are now applied to your license.')

            # get all groups in license which have master data change permissions.
            recipients = self.get_groups_in_license_with_permissions('update_license_masterdata', license_id=id)
            nc.create_system_notification(notification, recipients, for_license=True)

        return license

    @transaction_decorator
    def save_or_update_license(self, id, name, domain, polymorphic_type, contact,
                               company, expiration_date, is_default=False, permissions=[], usage_options=[], state=None):
        """
        @param name: C{str} name of the License
        @param domain: Domain, the domain to which this License belongs.
        @param polymorphic_type: license type.
        @param contact: Person, the contact person for this License.

        This method takes care of saving and updating the Licenses.
        """
        if polymorphic_type is None:
            polymorphic_type = License
        else:
            pp = [p['model'] for p in self.get_available_license_types()]
            if polymorphic_type not in pp:
                raise SecurityException('Save license: invalid polymorphic_type supplied (%s)' % polymorphic_type)
        assert isinstance(domain, Domain)
        assert isinstance(contact, Person)
        if company is not None:
            assert isinstance(company, Company)
        elif contact is not None:
            company = Company()
            company.contact_person = contact

        if id is not None:
            self.assert_permissions("update_license")
            # a license type cannot be changed; id and polymorphic type must match otherwise
            # next query will not have any results
            license = self.sa_session.query(polymorphic_type).filter(polymorphic_type.id==id)
            if license.count() == 0:
                raise SecurityException("Save license: Supplied license id and license type do not match")
            license = license.one()
            # cannot update domain for a license
            if license.domain.id != domain.id:
                raise SecurityException("Update license: Trying to change domain for license (%s) " % unicode(name))
        else:
            self.assert_permissions("create_license")
            license = polymorphic_type()
            license.domain = domain

        license.name = unicode(name)
        #license.contact_person = contact
        if expiration_date == '':
            expiration_date = None
        license.expiration_date = expiration_date
        license.company = company

        try:
            dl = domain.default_license
        except NoResultFound:
            dl = None

        if dl is not None and is_default and dl != license:
            dl.is_default = False
            self.sa_session.add(dl)

        license.is_default = is_default

        # And save the permissions to the license
        if len(permissions) > 0:
            self.save_permissions_to_license(license, permissions, None)
        # Now save the usage_options to the license
        if len(usage_options) > 0:
            self.save_or_update_license_usage_options(license, usage_options)

        self.sa_session.add(license)
        return license

    def _create_new_license(self, name, polymorphic_type=None, contact=None, company=None, role_name=None):
        """
        Creates a new license and adds the current user into the new group

        It doesn't call save_or_update_license() here because that checks for create_license, grant etc permisisons
        This method is protected so it should be called from a place where proper permissions checks should take place
        """

        # domain is always same as user
        domain = self.get_default_group().license.domain

        # get permissions from role
        permissions = []
        if role_name is not None:
            role = self.sa_session.query(Role).filter(Role.name==role_name).one()
            permissions = role.permissions

        if contact is None:
            contact = self.get_default_group().license.company.contact_person

        if polymorphic_type is None:
            polymorphic_type = License

        license = polymorphic_type()
        license.name = name
        license.domain = domain
        license.contact_person = contact
        self.sa_session.add(license)

        group = Group()
        group.license = license
        group.is_default = True
        group.name = Group.DEFAULT_GROUP_NAME

        ga = GroupAccount()
        ga.group = group
        ga.account = self.get_account()
        ga.active = True
        self.sa_session.add(ga)

        for perm in permissions:
            group.permissions.append(perm)

        self.sa_session.add(group)

        self.logger.info('created new licence: %s' % license.id)

        return license

    @transaction_decorator
    def update_license_active_state(self, license_id, active=False):
        """
        This changes the active state of the provided license ID.
        """

        self.assert_permissions('update_license')
        license = self.get_licenses(license_id).pop()

        license.is_active = active
        self.sa_session.add(license)
        return license

    @transaction_decorator
    def save_privilege_hook_from_state(self, state):
        """
        """
        self.assert_permissions('hook_manager')
        assert state.has_key('object')
        assert state.has_key('domain_privileges')
        assert state.has_key('license_privileges')
        assert state.has_key('group_privileges')

        poh = PrivilegeObjectHook()
        if state.has_key('domain_id'):
            self.assert_permissions('view_all_domains')
            poh.domain = self.get_domains(state['domain_id']).pop()
        else:
            self.assert_permissions('view_domain', 'update_domain')
            poh.domain = self._get_default_group().license.domain

        if state.has_key('license_id') and state['license_id']:
            poh.license = self.get_licenses(state['license_id']).pop()

        if state.has_key('group_id') and state['group_id']:
            poh.group = self.get_groups(state['group_id']).pop()

        if state.has_key('target_group_id') and state['target_group_id']:
            poh.target_group = self.get_groups(state['target_group_id']).pop()

        poh.object = state['object']
        poh.domain_privileges = sum([int(i) for i in state['domain_privileges']])
        poh.license_privileges = sum([int(i) for i in state['license_privileges']])
        poh.group_privileges = sum([int(i) for i in state['group_privileges']])
        self.sa_session.add(poh)
        return poh

    @transaction_decorator
    def save_group_from_state(self, state, license_id, group_id=None):
        license = self.get_licenses(license_id)[0]
        group = self.save_or_update_group(group_id, state['name'], license, state['is_default'])
        state['permissions'] = [int(p) for p in state['permissions']]
        self.save_permissions_to_group(group, self.get_permissions(state['permissions']))
        return group

    @transaction_decorator
    def save_or_update_group(self, id, name, license, is_default=None):
        """
        @param name: C{str} The name of the group
        @param license: License, the license to which the group belongs.
        @param id: C{int} the id of the group.
        """
        if id is not None:
            group = self.sa_session.query(Group).get(id)
        else:
            group = Group()

        if name is None:
            name = Group.DEFAULT_GROUP_NAME

        group.name = name
        group.license = license
        if is_default is not None:
            group.is_default = is_default
            if group.is_default:
                self.unset_default_group(group)

        self.sa_session.add(group)
        return group

    @transaction_decorator
    def save_license_from_state(self, state):
        """
        Save the license as provided by the state dictionary
        @param state: {dict}
        """

        # assert the different entries in the state dict.
        assert isinstance(state, dict)
        assert isinstance(state['license'], dict)
        assert isinstance(state['domain'], dict)
        # check if polymorphic type is correct and user has permissions for it
        pt = self.get_available_license_types()
        polymorphic_type = None
        for p in pt:
            if state['license']['polymorphic_type'] == p['value']:
                polymorphic_type = p['model']
                break
        if polymorphic_type is None:
            raise SecurityException("Save license from state: Invalid polymorphic type (%s) " \
                                    % state['license']['polymorphic_type'])

        assert isinstance(state['contact_person'], dict)
        if state.has_key("company"):
            assert isinstance(state['company'], dict)
        if state.has_key("company_address"):
            assert isinstance(state['company_address'], dict)
        if state.has_key("company_bank_account"):
            assert isinstance(state['company_bank_account'], dict)
        assert isinstance(state['group'], dict)
        assert isinstance(state['permissions'], list)

        license_id = None
        domain_id = None
        contact_person_id = None
        invoice_person_id = None
        company_id = None
        default_group_id = None
        if state.has_key("dbID") and state["dbID"] is not None:
            # use get_license for correct permission and Reseller checks
            license = self.get_licenses(state["dbID"])[0]

            license_id = state["dbID"]
            domain_id = license.domain.id
            if license.company is not None:
                contact_person_id = license.company.contact_person_id
                invoice_person_id = license.company.invoice_person_id
            company_id = license.company_id
            default_group_id = license.default_group.id

        # extract from the state dict, for readability
        license_state = state['license']

        # get a list of permission objects
        state['permissions'] = [int(p) for p in state['permissions']]
        permissions = self.get_permissions(state['permissions'])

        # Save the domain, currently only names are stored for domains
        domain_state = state['domain']
        # see if the domain name already exists, then attach this license to that domain, otherwise create a new domain
        domain_search = self.sa_session.query(Domain).filter(Domain.name==domain_state['name'])
        if domain_search.count() > 0:
            domain = domain_search.one()
        else:
            domain = self.save_or_update_domain(domain_id, domain_state['name'])

        # Save the contact information from the contact state dict.
        contact_state = state['contact_person']
        contact = self._save_or_update_person(contact_person_id, contact_state)

        invoice = None
        if state.has_key('invoice_person'):
            ip = state['invoice_person']

            if ip.has_key('is_same_person_as') and ip['is_same_person_as']:
                invoice = contact
            else:
                invoice = self._save_or_update_person(invoice_person_id, ip)

        group_state = state['group']

        # Company information: only save in case there is a company name set or contact information.
        company = None
        if contact is not None or \
            invoice is not None or \
            (state.has_key("company") and state['company'].has_key("name") and state['company']["name"] != ""):

            company_state = state['company']
            # Attach the contact to the company state dict.
            company_state['contact_person_data'] = contact
            # Attach the invoice contact to the company state dict
            if invoice is not None:
                company_state['invoice_person_data'] = invoice
            # Save the company information
            company = self._save_or_update_company(company_id, company_state)

            # Save an address from the company address information.
            if state.has_key("company_address"):
                company_address_state = state['company_address']
                # TODO see #599 - this can also be an insert of a new address for this company
                ca_id = None
                if len(company.addresses) > 0:
                    ca_id = company.addresses[0].address_id

                if company_address_state.has_key('address'):
                    address = self._save_or_update_address(ca_id, company_address_state['address'])
                    # Remove the address information from the company address state.
                    del company_address_state['address']

                    # Save the address, attached to the newly created company.
                    if address is not None:
                        self._save_or_update_company_address(company, address, company_address_state)

            # Save bank account information
            if state.has_key("company_bank_account"):
                company_bank_account_state = state['company_bank_account']
                # TODO see #599 - this can also be an insert of a new bank account for this company
                cba_id = None
                if len(company.bank_accounts) > 0:
                    cba_id = company.bank_accounts[0].bank_account_id

                if company_bank_account_state.has_key('bank_account'):
                    bank_account = self._save_or_update_bank_account(cba_id,
                                       company_bank_account_state['bank_account'])
                    del company_bank_account_state['bank_account']

                    # Save the bank account to the company.
                    if bank_account is not None:
                        self._save_or_update_company_bank_account(company, bank_account,
                                                              company_bank_account_state)
        else:
            logger.info("License save: No company name was set, so company info not saved for license %s" % license_state['name'])

        # Finally save the license information.
        license = self.save_or_update_license(license_id,
                                              license_state['name'],
                                              domain, polymorphic_type,
                                              contact, company,
                                              license_state['expiration_date'],
                                              license_state['is_default'], state=state)

        # And create a group.
        group = self.save_or_update_group(default_group_id, group_state['name'], license, is_default=True)

        if state.has_key('append_to_group'):
            group = None
            if state['append_to_group'] is not None and state['append_to_group'] != "":
                g = self.get_groups(state['append_to_group'])
                if len(g) > 0:
                    group = g.pop()

        # And save the permissions to the license and to the group.
        self.save_permissions_to_license(license, permissions, group)
        # Now save the usage_options to the license and to the group.
        if state.has_key("usage_options"):
            assert isinstance(state['usage_options'], list)
            if len(state['usage_options']) > 0:
                self.save_or_update_license_usage_options(license, state["usage_options"])

        if state.has_key('configs'):
            lcc = LicenseConfigController(self.sa_session, license)
            lcc.set_configs(state['configs'])

        return license

    def load_group_state(self, id):
        group = self.get_groups(id)[0]
        state = {
            'license': {
                'name': group.license.name
            },
            'permissions': [p.id for p in group.permissions],
            'domains': [d.id for d in group.domains],
            'is_default': group.is_default,
            'name': group.name
        }
        return state

    def load_master_data_state(self, _id, original=False):

        if isinstance(_id, License):
            _license = _id
        else:
            _license = self.get_licenses(_id)
            if len(_license) == 0:
                return None
            _license = _license[0]

        if not self.has_permissions('license_masterdata_change_approval'):
            self.assert_permissions('update_license_masterdata', license_id=_license.id)

        return self._load_master_data_state_from_license(_license, original)

    def _load_master_data_state_from_license(self, license, original=False):

        state = {
            'id': license.id,
            'created': license.created,
            'last_updated': license.last_updated,
            'company': {},
            'contact_person_data': {},
            'invoice_person_data': {},
            'outstanding_change_request': license.temp_company is not None,
        }

        # if the company has an outstanding temporary company, show the details of that company
        c = license.temp_company
        # else show the current active company details.
        if c is None or original:
            c = license.company

        if c is None:
            return state

        company_state = self._load_company_state(c.id)

        if c.contact_person is not None:
            state['contact_person_data'] = self._load_person_state(c.contact_person.id)

        if c.invoice_person is not None:
            if c.contact_person == c.invoice_person:
                state['invoice_person_data'] = state['contact_person_data']
                state['invoice_person_data']['is_same_person_as'] = True
            else:
                state['invoice_person_data'] = self._load_person_state(c.invoice_person.id)

        state['company'] = company_state
        if company_state.has_key('address_data'):
            state['address_data'] = company_state['address_data']
        if company_state.has_key('bank_data'):
            state['bank_data'] = company_state['bank_data']

        state['company_js_module'] = self._instance_class.company_instance_class.JS_MODULE
        state['person_js_module'] = self._instance_class.person_instance_class.JS_MODULE

        return state

    def load_license_state(self, id):
        """
        Return the license state, having its id
        """
        license = self.get_licenses(id)
        if len(license) == 0:
            return {}
        license = license[0]

        person_state = {}
        invoice_state = {}
        if license.company is not None and license.company.contact_person is not None:
            person_state = self._load_person_state(license.company.contact_person.id)
            person_state['address_data'] = {}
            if license.company.contact_person.address_id is not None:
                person_state['address_data'] = self._load_address_state(license.company.contact_person.address_id)

            if license.company.invoice_person is not None:
                if license.company.invoice_person == license.company.contact_person:
                    invoice_state['is_same_person_as'] = True
                else:
                    invoice_state = self._load_person_state(license.company.invoice_person.id)


        luos = self.sa_session.query(LicenseUsage).filter(LicenseUsage.license_id==license.id).all()
        lcc = LicenseConfigController(self.sa_session, license)

        sorted_permissions = self.get_ordered_permissions_per_app(license.permissions)
        sorted_permissions = [item for sublist in sorted_permissions.values() for item in sublist]

        state = {
            'license': {
                'name': license.name,
                'expiration_date': license.expiration_date,
                'is_default': license.is_default,
                'polymorphic_type': license.polymorphic_type,
                'outstanding_change_request': license.temp_company is not None
            },
            'domain': {
                'name': license.domain.name
            },
            'group': {
                'name': license.default_group.name
            },
            'permissions': [p['value'] for p in sorted_permissions],
            'usage_options':[{'usage_option_id': uo.usage_option_id,
                              'credits': uo.credits,
                              'price_per_unit': uo.price_per_unit} for uo in luos],
            'contact_person': person_state,
            'invoice_person': invoice_state,
            'company': {},
            'company_address': {},
            'company_bank_account': {},
            'configs': lcc.get_configs()
        }

        if license.company is not None:
            state['company'] = self._load_company_state(license.company.id)
            if len(license.company.addresses) > 0:
                state['company_address'] = self._load_address_state(license.company.addresses[0].address_id)
            if len(license.company.bank_accounts) > 0:
                state['company_bank_account'] = self._load_bank_account_state(license.company.bank_accounts[0].bank_account_id)
        return state

    def get_ordered_permissions_per_app(self, permissions, sort_by_category_only=True):
        permissions_map = {}
        for permission in permissions:
            if permission.title:
                label = _(permission.title)
            else:
                label = permission.name
            if not permissions_map.has_key(permission.category.name):
                permissions_map[permission.category.name] = []
            permissions_map[permission.category.name].append({'label': label, 'value': permission.id})

        rv = []
        categories = sorted(permissions_map.keys())
        for category in categories:
            rv.append((category, permissions_map[category] if sort_by_category_only else sorted(permissions_map[category], key=lambda k: k['label'])))

        return OrderedDict(rv)

    def validate_permissions(self, requested, privileged=None):
        """
        Validates if the current account has access to the requested permissions.
        This is mostly used for sysadmin/service manager/group manager and resellers.
        """
        if privileged is None:
            privileged = self.get_permissions()

        for req in requested:
            if req not in privileged:
                raise SecurityException('No access to use permission %s' % req)

        return True

    @transaction_decorator
    def save_permissions_to_license(self, license, permissions, group=None, revoke_permissions=True):
        """
        The logic is distributed to the add and revoke methods based on
        the provided information. The user needs to have the correct rights
        to add/revoke permissions.
        """
        add = []
        for perm in permissions:
            if perm not in license.permissions:
                add.append(perm)

        revoke = []
        for perm in license.permissions:
            if perm not in permissions:
                revoke.append(perm)

        if len(add):
            self.add_permissions_to_license(license, permissions)
        if group is not None:
            if group not in self.get_groups_for_license(license):
                raise SecurityException('Unable to attach permission to group which is not part of the license')
            self.add_permissions_to_group(group, permissions)

        if revoke_permissions and len(revoke):
            self.revoke_permissions_from_license(license, revoke)

    @transaction_decorator
    def add_permissions_to_license(self, license, permissions):
        """
        This will add the supplied permissions to the license.
        Assumed privileges on the license object are previously checked.
        """
        self.assert_permissions('distribute_license_permissions')
        self._add_permissions_to_license(license, permissions)

    @transaction_decorator
    def _add_permissions_to_license(self, license, permissions):
        for perm in permissions:
            if perm not in license.permissions:
                license.permissions.append(perm)
        self.sa_session.add(license)

    @transaction_decorator
    def revoke_permissions_from_license(self, license, permissions):
        """
        @param license, The License object.
        @param permissions, C{list}, a list of Permission objects.

        This will revoke the permissions from the license. It will also revoke
        the same permissions from any of the attached groups that have them.
        """
        self.assert_permissions('distribute_license_permissions')

        perms = []
        for perm in license.permissions:
            if perm not in permissions:
                perms.append(perm)
        license.permissions = perms

        # we should also delete from all groups attached to the license
        # TODO: can we do this automatically on database level; multitable constraint?
        for group in self.get_groups_for_license(license):
            self.revoke_permissions_from_group(group, permissions)

        self.sa_session.add(license)

    @transaction_decorator
    def save_permissions_to_group(self, group, permissions):
        revoke = []
        for perm in group.permissions:
            if perm not in permissions:
                revoke.append(perm)

        if len(permissions):
            self.add_permissions_to_group(group, permissions)
        if len(revoke):
            self.revoke_permissions_from_group(group, revoke)

    @transaction_decorator
    def revoke_permissions_from_group(self, group, permissions):
        """
        @param group, the Group object
        @param permissions, C{list}, a list of Permission objects.

        This will revoke permissions from the supplied group.
        """
        self.assert_permissions('distribute_group_permissions')

        perms = []
        for perm in group.permissions:
            if perm not in permissions:
                perms.append(perm)
        group.permissions = perms
        self.sa_session.add(group)

    @transaction_decorator
    def add_permissions_to_group(self, group, permissions):
        """
        This will add the supplied permissions to the group.
        Note, a group can never have a permission that is not part of its license
        Assumed privileges on the group object are previously checked.
        """

        self.assert_permissions('distribute_group_permissions')
        self._add_permissions_to_group(group, permissions)

    @transaction_decorator
    def _add_permissions_to_group(self, group, permissions):
        for perm in permissions:
            if perm not in group.license.permissions:
                raise SecurityException('Cannot assign permissions that are not part of the license')
            if perm not in group.permissions:
                group.permissions.append(perm)
        self.sa_session.add(group)

    @transaction_decorator
    def save_or_update_license_usage_options(self, license, usage_option_defs):
        """
        This will set the supplied usage options and its credits and price to the License.
        Usage Options are only on License level and count for all groups that
        are attached to the License.
        @param license: {License}
        @param usage_option_defs: array of dict with usage options: usage_option_id, credits and price_per_unit
        """
        from sqlalchemy.sql import not_
        self.assert_permissions('distribute_license_permissions')

        uo_ids = [uo["usage_option_id"] for uo in usage_option_defs]

        # see if there's something to remove
        luos = self.sa_session.query(LicenseUsage).filter(and_(LicenseUsage.license_id==license.id,
                                                               not_(LicenseUsage.usage_option_id.in_(uo_ids)))).all()
        for luo in luos:
            self.sa_session.delete(luo)

        for usage_option_def in usage_option_defs:
            try:
                # check to see if we already have this Usage Option attached
                luo = self.sa_session.query(
                          LicenseUsage).filter(and_(
                          LicenseUsage.license_id == license.id,
                          LicenseUsage.usage_option_id == usage_option_def["usage_option_id"])).one()
                luo.credits = usage_option_def["credits"]
                luo.price_per_unit = usage_option_def["price_per_unit"]
                self.sa_session.add(luo)
            # apparently we are adding a new Usage Option to the License.
            except InvalidRequestError:
                uo = self.get_usage_options(usage_option_def["usage_option_id"])
                if len(uo) > 0:
                    luo = LicenseUsage()
                    luo.license = license
                    luo.usage_option = uo[0]
                    luo.credits = usage_option_def["credits"]
                    luo.price_per_unit = usage_option_def["price_per_unit"]
                    self.sa_session.add(luo)

    @transaction_decorator
    def add_usage_options_to_license(self, license, usage_option_defs):
        """
        This will add the supplied usage options and its credits and price to the License.
        Usage Options are only on License level and count for all groups that
        are attached to the License.
        @param license: {License}
        @param usage_option_defs: array of dict with usage options: usage_option_id, credits and price_per_unit
        """
        self.assert_permissions('distribute_license_permissions')

        for usage_option_def in usage_option_defs:
            if not isinstance(usage_option_def, dict):
                usage_option_def = {
                    'usage_option_id': usage_option_def.usage_option.id,
                    'credits': usage_option_def.credits,
                    'price_per_unit': usage_option_def.price_per_unit
                }

            try:
                # check to see if we already have this Usage Option attached
                luo = self.sa_session.query(
                          LicenseUsage).filter(and_(
                          LicenseUsage.license_id == license.id,
                          LicenseUsage.usage_option_id == usage_option_def["usage_option_id"])).one()
                luo.credits += usage_option_def["credits"]
                luo.price_per_unit = usage_option_def["price_per_unit"]
            # apparently we are adding a new Usage Option to the License.
            except InvalidRequestError:
                luo = LicenseUsage()
                luo.license = license
                luo.usage_option = self.get_usage_options(usage_option_def["usage_option_id"])[0]
                luo.credits = usage_option_def["credits"]
                luo.price_per_unit = usage_option_def["price_per_unit"]

            self.sa_session.add(luo)

    @transaction_decorator
    def save_or_update_role(self, id, role_type, name, permissions, license_configs=None, dashboard=False, dashboard_privileges=None, required_permission='manage_roles'):
        """
        Roles are used as permission presets and can be applied to licenses or groups.
        """

        if id is None and role_type != Role.ROLE_TYPE_AUTOMATIC:
            self.assert_permissions('manage_roles', 'create_roles')

        if id is not None:
            role = self.sa_session.query(Role).filter(Role.id==id).one()
            if self.get_account() is not None:
                self.assert_permissions(role.required_permission)
        else:
            if not self.is_unique_role_name(name):
                raise PublicUserException(_('Role name is already in use.'))
            if role_type == Role.ROLE_TYPE_AUTOMATIC: # Automated roles should not be added by users
                assert self.get_account() is None
            role = Role()
            role.role_type = role_type
            role.name = name
            role.required_permission = required_permission
            role.permissions = permissions

        if self.get_account() is None or role.role_type != Role.ROLE_TYPE_AUTOMATIC:
            role.name = name
            role.required_permission = required_permission
            role.permissions = permissions

            if license_configs is not None:
                # (re)create role license configs
                lcs = []
                for c, v in license_configs:
                    lc = RoleLicenseConfig()
                    lc.set_config(c, v, self.sa_session)
                    lcs.append(lc)

                for rlc in role.license_configs:
                    if rlc not in lcs:
                        self.sa_session.delete(rlc)

                role.license_configs = lcs

        if dashboard is not False:
            # it is petarded to add a dashboard to a Role which cannot view it.
            if 'view_dashboard' not in [p.name for p in role.permissions]:
                raise PublicUserException(_('Permission for View Dashboard is required in order to set a dashboard.'))

            from basesoft.web.contrib.dashboard.models import Dashboard
            if not isinstance(dashboard, Dashboard):
                from basesoft.web.contrib.dashboard.controllers import DashboardStateController
                dsc = DashboardStateController(self.sa_session, StateManager(None, self.account_id, Dashboard.MODE_COMPOSE))
                dashboard = dsc.get_dashboards(dashboard).pop()

            role.dashboard = dashboard
            # if privileges are provided the dashboard is referenced instead of cloned.
            if dashboard_privileges is not None:
                if dashboard_privileges & Dashboard.PRIV_UPDATE == Dashboard.PRIV_UPDATE:
                    if 'update_dashboard' not in [p.name for p in role.permissions]:
                        raise PublicUserException(_('Permission to Update Dashboard is required in order to set update privileges for Dashboard.'))

                # TODO: delete, archive... ?
                role.dashboard_privileges = dashboard_privileges

        elif dashboard_privileges is not None:
            raise PublicUserException(_('Cannot add privileges for Dashboard if no dashboard is attached.'))

        self.sa_session.add(role)

        return role

    def load_role(self, _id):
        role = self.get_roles(_id).pop()
        return {
            'name': role.name
        }

    def apply_role(self, role_id, domain_id, license_type, apply_to_default_group=False, revoke_permissions=False, apply_dashboard=False):
        """
        @param (int) role_id: The role we will apply.
        @param (int) domain_id: The domain where we will apply the role at.
        @param (string) license_type: The polymorphic license type to which we will apply the role.
        @param (bool) apply_to_default_group: whether or not the role permissions should bubble to the default group of the license.
        @param (bool) remove_unspecified_permissions: remove any permissions from the license which are undefined in the supplied role.
        """
        self.assert_permissions('apply_roles')

        role = self.get_roles(role_id).pop()
        domain = self.get_domains(domain_id).pop()

        lt = get_models_of_type(License, name=license_type, include_models=True)
        assert issubclass(lt, License)

        licenses = self.sa_session.query(lt).filter(lt.domain==domain).all()
        if len(licenses) == 0:
            raise PublicUserException(_('No licenses of supplied license type in domain %(name)s.' % {'name': domain.name}))

        for _license in licenses:
            group = _license.default_group if apply_to_default_group else None
            self.save_permissions_to_license(_license, role.permissions, group, revoke_permissions=revoke_permissions)

            if apply_dashboard:
                self.__apply_dashboard_from_role(role, _license, group)

        return True

    def get_roles(self, id=None):
        """
        Returns all the roles or the roles for the supplied id(s).
        """
        self.assert_permissions('manage_roles')
        if id is None:
            return self.sa_session.query(Role).all()
        if type(id) != list:
            id = [id]
        return self.sa_session.query(Role).filter(Role.id.in_(id)).all()

    def get_role_by_name(self, name):
        """
        Returns the role for the supplied name, raises a ResultNotFound if the entry does not exist.
        """
        #self.assert_permissions('manage_roles')
        return self.sa_session.query(Role).filter(Role.name==name).one()

    def grant_from_role(self, role, _license=None, group=None, add_dashboard=False):
        """
        Grant permissions to the supplied license and/or the supplied group based on the role.
        """
        if not isinstance(role, Role):
            role = self.get_role_by_name(role)

        if role.role_type != Role.ROLE_TYPE_AUTOMATIC:
            self.assert_permissions('manage_roles')

        if _license is not None:
            self._add_permissions_to_license(_license, role.permissions)
        if group is not None:
            self._add_permissions_to_group(group, role.permissions)

        if not add_dashboard:
            return

        self.__apply_dashboard_from_role(role, _license, group)

    def __apply_dashboard_from_role(self, role, _license, group=None):
        # we do not have the dashboard privileges set, so the dashboard will be a clone.
        if role.dashboard_privileges is None:
            # TODO: implement cloning of the dashboard
            return

        # this will only be applied for the license default group or the group if supplied.
        if group is None:
            group = _license.default_group

        self.grant_privilege_to(role.dashboard, get_bits(role.dashboard_privileges), group=True, target_group=group, check_own_privs=False)

    def is_unique_role_name(self, name):
        """
        Checks the uniqueness of the supplied name. This is used for form behaviour.
        """
        try:
            self.get_role_by_name(name)
            return False
        except NoResultFound:
            return True

    def save_license_configs(self, configs, values, license=None):
        if license is None:
            license = self.get_default_group().license
        if not isinstance(license, License):
            license = self.__get_license(license)

        self.assert_permissions('manage_license_configs', license_id=license.id)

        # check for the license configs individual permissions
        confs = configs[license.polymorphic_type] # TODO: merge this with default License polymorphic type?
        # this makes sure we only set the configs we have permissions for on this license
        confs = [c.get_config(self.sa_session) for c in confs if c.has_permissions(self, license_id=license.id)]

        lcc = LicenseConfigController(self.sa_session, scope=license)

        for cat in confs:
            for conf in cat['configs']:
                if not values.has_key(conf['name']):
                    continue
                lcc.set(conf['name'], values[conf['name']])

        return True

class UniqueRoleName(Validation):
    """
    Plugin for unique check of role name.
    """
    def __init__(self, *args, **kwargs):
        Validation.__init__(self, *args, **kwargs)

    def check(self, params={}):
        if not params.has_key('controller'):
            raise PublicUserException(_('Incorrect usage.'))
        c = params['controller']
        if not c.is_unique_role_name(self._value):
            raise ValidationError(_('Role name already in use.'))
