# Copyright (c) 2010 BaseSoft B.V.
# See LICENSE for details.

from django.utils.translation import ugettext as _
from sqlalchemy import func
from sqlalchemy.orm import aliased

from basesoft.web.core.exceptions import PermissionDeniedException
from basesoft.web.core.util.custom_query import CustomQuery, CustomQueryField, CustomQueryModel, DATA_TYPE_BOOLEAN

from basesoft.web.contrib.account.models import GroupAccount
from basesoft.web.contrib.license.models import License, Domain, Group, PrivilegeObjectHook, DomainGroup, Permission,\
    PermissionCategory, RolePermission, Role
from basesoft.web.contrib.contact.models import Person, Company
from basesoft.web.contrib.dashboard.models import Dashboard

class PermissionOverviewCustomQuery(CustomQuery):
    permissions = ['view_all_permissions']

    def tables(self):
        return (
            CustomQueryModel(Permission),
            CustomQueryModel(PermissionCategory),
        )

    def fields(self):
        return (
            CustomQueryField(Permission.id, 'id'),
            CustomQueryField(PermissionCategory.name, _('Category'), is_filter=True, is_group_by=True),
            CustomQueryField(Permission.name, _('Handle'), is_filter=True),
            CustomQueryField(Permission.title, _('Title'), is_filter=True),
            CustomQueryField(Permission.description, _('Description')),
        )

class RoleOverviewCustomQuery(CustomQuery):
    permissions = ['manage_roles']

    def tables(self):
        return (
            CustomQueryModel(Role),
            CustomQueryModel(RolePermission),
            CustomQueryModel(Permission),
            CustomQueryModel(Dashboard, use_outerjoin=True, priv=None, onclause=Role.dashboard_id==Dashboard.id)
        )

    def fields(self):
        return (
            CustomQueryField(Role.id, 'role_id'),
            CustomQueryField(Role.name, _('Role name'), key="role_name", is_filter=True),
            CustomQueryField(Role.role_type, _('Role type'), key="role_type", is_filter=True),
            CustomQueryField(Role.required_permission, 'Required permission', key="required_permission"),
            CustomQueryField(Dashboard.name, _('Dashboard'), key="dashboard"),
            CustomQueryField(func.group_concat(Permission.title), _('Permissions')),
        )

    def _row_processor(self, row):
        return self.has_permissions(row['required_permission'])

    group_by = (Role.id, )

TempCompany = aliased(Company)

class LicenseCustomQuery(CustomQuery):
    def validate_access(self):
        # self.is_reseller() or
        if not ( self.has_permissions('view_all_domains') \
            or self.has_permissions('view_domain') \
            or self.has_permissions('view_license')):
            raise PermissionDeniedException('You do not have access to the requested resource')

    def tables(self):
        return (
            CustomQueryModel(License),
            CustomQueryModel(Domain, onclause=License.domain_id==Domain.id),
            CustomQueryModel(Company, onclause=License.company_id==Company.id, use_outerjoin=True),
            CustomQueryModel(Person, onclause=Company.contact_person_id==Person.id),
            CustomQueryModel(TempCompany, onclause=License.temp_company_id==TempCompany.id, use_outerjoin=True),
        )

    def fields(self):
        return (
            CustomQueryField(License.id, _('ID'), key='id'),
            CustomQueryField(Domain.id, _('DomainID'), key="domain_id"),
            CustomQueryField(Domain.name, _('Domain'), key="domain_name", is_filter=True),
            CustomQueryField(Domain.status, _('Status'), key="domain_status", is_filter=True),
            CustomQueryField(License.name, _('License'), key='license_name', is_filter=True),
            CustomQueryField(License.is_default, _('Is default'), type=DATA_TYPE_BOOLEAN),
            CustomQueryField(License.is_active, _('Is active'), key='is_active', type=DATA_TYPE_BOOLEAN, is_filter=True),
            CustomQueryField(License.polymorphic_type, _('Type'), is_filter=True, filter_options=License.polymorphic_type),
            CustomQueryField(func.concat_ws(' ', Person.first_name, Person.middle_name, Person.last_name), _('Contact'), key='contact'),
            CustomQueryField(Company.name, _('Company')),
            CustomQueryField(TempCompany.is_temp, _('Master data change'), key='master_data', is_filter=True),
            CustomQueryField(License.id, _('Details')),
            CustomQueryField(License.id, _('Groups'), key='license_groups'),
            CustomQueryField(Domain.id, _('Locales'), key='locales')
        )

    def set_where(self):
        # if user has 'view_all_domains' permission, then no where clause needs to be set
        if self.has_permissions('view_all_domains'):
            return None
        current_license = self._get_default_group().license

        # if 'view_domain', user can see all license in his domain, so filter on domain
        if self.has_permissions('view_domain'):
            self.where = Domain.id==current_license.domain.id
            return

        licenses = []
        for g in self.get_active_groups():
            if self.has_permissions('view_license', group_id=g.id):
                licenses.append(g.license.id)

        if len(licenses) > 0:
            self.where = License.id.in_(licenses)

class GroupCustomQuery(CustomQuery):
    def tables(self):
        return (
            CustomQueryModel(Group),
            CustomQueryModel(GroupAccount, use_outerjoin=True),
            CustomQueryModel(DomainGroup, use_outerjoin=True),
        )

    def fields(self):
        return (
            CustomQueryField(Group.license_id, 'license', in_select=False),
            CustomQueryField(Group.id, _('ID')),
            CustomQueryField(Group.name, _('Group')),
            CustomQueryField(Group.is_default, _('Default')),
            CustomQueryField(func.count(GroupAccount.account_id.distinct()), _('Accounts'), key='no_of_accounts'),
            CustomQueryField(func.count(DomainGroup.domain_id.distinct()), _('Domains'), key='no_of_domains'),
        )

    group_by = (Group.id, )

group_alias1 = aliased(Group)
group_alias2 = aliased(Group)

class HookCustomQuery(CustomQuery):
    permissions = ['hook_manager']
    def tables(self):
        return (
            CustomQueryModel(PrivilegeObjectHook),
            CustomQueryModel(Domain, onclause=PrivilegeObjectHook.domain_id==Domain.id),
            CustomQueryModel(License, onclause=PrivilegeObjectHook.license_id==License.id, use_outerjoin=True),
            CustomQueryModel(group_alias1, onclause=PrivilegeObjectHook.group_id==group_alias1.id, use_outerjoin=True),
            CustomQueryModel(group_alias2, onclause=PrivilegeObjectHook.target_group_id==group_alias2.id, use_outerjoin=True),
        )
    def fields(self):
        return (
            CustomQueryField(PrivilegeObjectHook.id, _('ID')),
            CustomQueryField(PrivilegeObjectHook.object, _('Object'), is_group_by=True, is_filter=True),
            CustomQueryField(Domain.name, _('Domain')),
            CustomQueryField(License.name, _('License')),
        )

    def set_where(self):
        if not self.has_permissions('view_all_domains'):
            self.where = Domain.id==self.get_default_group().license.domain.id

class PrivilegeDelegationCustomQuery(CustomQuery):

    def __init__(self, sa_session, account_id, object_name, object_id):
        from basesoft.web.contrib.license.controllers import LicenseController
        self.object_name = object_name
        self.object_id = object_id
        self.license_controller = LicenseController(sa_session, account_id=account_id)
        self.model = self.license_controller.get_privilege_objects(self.object_name)
        CustomQuery.__init__(self, sa_session, account_id=account_id)

        group_obj = self.model().get_group_object()[0]
        self.group_by = (Group.id, group_obj.privilege)

    def validate_access(self):
        if not self.license_controller.has_permissions('privilege_object_administrator'):
            self.license_controller.assert_privilege_delegation(self.object_name, self.object_id)

    def tables(self):
        group_obj, attr = self.model().get_group_object()
        return (
            CustomQueryModel(self.model, auto_group_by=False, priv=None if self.license_controller.has_permissions('view_all_domains') else True),
            CustomQueryModel(group_obj, onclause=self.model.id==getattr(group_obj, '%s_id' % attr)),
            CustomQueryModel(Group, onclause=group_obj.group_id==Group.id),
            CustomQueryModel(License, onclause=Group.license_id==License.id),
            CustomQueryModel(Domain, onclause=License.domain_id==Domain.id)
        )

    def fields(self):
        group_obj = self.model().get_group_object()[0]
        return (
            CustomQueryField(Group.id, _('ID')),
            CustomQueryField(Group.name, _('Group')),
            CustomQueryField(License.name, _('License')),
            CustomQueryField(Domain.name, _('Domain')),
            CustomQueryField(group_obj.privilege, _('Privilege'), key='privilege'),
            CustomQueryField(group_obj.group_has_privilege, _('Group has privilege'), type=DATA_TYPE_BOOLEAN),
            CustomQueryField(group_obj.license_has_privilege, _('License has privilege'), type=DATA_TYPE_BOOLEAN),
            CustomQueryField(group_obj.domain_has_privilege, _('Domain has privilege'), type=DATA_TYPE_BOOLEAN),
        )

    def set_where(self):
        self.where = self.model.id==self.object_id


class PrivilegeObjectBrowserCustomQuery(CustomQuery):

    permissions = ['privilege_object_administrator'] # for now just sysadmin


    def __init__(self, sa_session, account_id, target=None, value=None):
        from basesoft.web.contrib.license.controllers import LicenseController
        from iqportal.iquest.core.models import Survey
        self.license_controller = LicenseController(sa_session, account_id=account_id)

        if target is not None and target == 'object':
            mdl = self.license_controller.get_privilege_objects(name=value, include_models=True)
            self._objects = {'model' : mdl, 'label': mdl.__name__ if not hasattr(mdl, 'NAME_I18N') else mdl.NAME_I18N}
        else:
            self._objects = {'model': Survey, 'label': 'Survey'} #self.license_controller.get_privilege_objects(include_models=True)[0]
        CustomQuery.__init__(self, sa_session, account_id=account_id)

        group_obj = self._objects['model']().get_group_object()[0]
        self.group_by = (self._objects['model'].id, Group.id, group_obj.privilege)

    def tables(self):
        obj = self._objects
        model = obj['model']
        group_obj, attr = model().get_group_object()
        return (
            CustomQueryModel(obj['model'], auto_group_by=False, priv=None),
            CustomQueryModel(group_obj, onclause=model.id==getattr(group_obj, '%s_id' % attr)),
            CustomQueryModel(Group, onclause=group_obj.group_id==Group.id),
            CustomQueryModel(License, onclause=Group.license_id==License.id),
            CustomQueryModel(Domain, onclause=License.domain_id==Domain.id)
        )

    def fields(self):
        obj = self._objects
        inst = obj['model']()
        group_obj = inst.get_group_object()[0]
        return (
            CustomQueryField(obj['model'].id, _('Object ID'), key='object_id', is_filter=True),
            CustomQueryField(obj['model'].id, _('Object'), key='object'),
            CustomQueryField(obj['model'].get_name_property(), _('Object Label'), key='object_label', is_filter=True),
            CustomQueryField(Group.id, _('Group ID'), in_select=False, is_filter=True, filter_options=Group.name),
            CustomQueryField(Group.name, _('Group')),
            CustomQueryField(License.name, _('License'), is_filter=True, is_group_by=True, filter_options=License.name),
            CustomQueryField(Domain.name, _('Domain'), is_filter=True, is_group_by=True, filter_options=Domain.name),
            CustomQueryField(group_obj.privilege, _('Privilege'), key='privilege', is_filter=True, filter_options=[{'value': p[0], 'label': p[1]} for p in inst.get_privileges()]),
            CustomQueryField(group_obj.group_has_privilege, 'G', type=DATA_TYPE_BOOLEAN),
            CustomQueryField(group_obj.license_has_privilege, 'L', type=DATA_TYPE_BOOLEAN),
            CustomQueryField(group_obj.domain_has_privilege, 'D', type=DATA_TYPE_BOOLEAN),
            CustomQueryField(group_obj.world_has_privilege, 'W', type=DATA_TYPE_BOOLEAN),
        )

    def set_value_processors(self):

        obj = self._objects
        privs = {}
        for key, value in obj['model']().get_privileges():
            privs[key] = value

        def label_proc(row, key):
            obj = self._objects
            return obj['model'].__name__

        def priv_proc(row, key):
            if privs.has_key(getattr(row, key)):
                return privs[getattr(row, key)]
            return getattr(row, key)

        self._value_processors['object'] = label_proc
        self._value_processors['privilege'] = priv_proc

custom_queries = (
    LicenseCustomQuery,
    GroupCustomQuery,
    HookCustomQuery,
    PrivilegeDelegationCustomQuery,
    PermissionOverviewCustomQuery,
    RoleOverviewCustomQuery,
    PrivilegeObjectBrowserCustomQuery,
)
