# Copyright (c) 2009 BaseSoft B.V.
# See LICENSE for details.

from django.conf import settings
from django.utils import translation
from sqlalchemy import and_
from basesoft.web.contrib.license.models import RoleLicenseConfig

def sync(session):
    """
    Synchronizes all permissions, usage options and roles.
    """
    permissions = []
    usage_options = []
    roles = []

    # make sure the current language is English, because permissions have translated titles, so we want them to get in
    # English in the database.
    current_language = translation.get_language()
    translation.activate('en') # set English while we add the permissions and usage options

    # go through the apps and see if they have permissions to sync.
    for app in settings.INSTALLED_APPS:
        try:
            sa_module = __import__(app + '.sa_datamapper', {}, {}, [''])
        except ImportError:
            # if we do not have a datamapper then just continue as there
            # are no permissions to be sync'ed
            continue

        if hasattr(sa_module, 'app_permissions') and not hasattr(sa_module, 'app_permissions_category'):
            raise Exception("%s: application has permissions defined, but no category for them" % app)
        if hasattr(sa_module, 'app_permissions'):
            category = sa_module.app_permissions_category()
            perms = sa_module.app_permissions()
            for perm in perms:
                perm.category = category
                permissions.append(perm)
        if hasattr(sa_module, 'app_usage_options'):
            usage_options += sa_module.app_usage_options()

        if hasattr(sa_module, 'app_roles'):
            roles += sa_module.app_roles()

    ### sync Permissions
    # Delete every permission that is not defined in the above permission list.
    # All permissions that are deleted are obviously out of date and no
    # longer used.
    from basesoft.web.contrib.license.models import Permission, PermissionCategory
    perms = session.query(Permission).all()
    availablenames = []
    names = {}
    for p in permissions:
        names[p.name] = p
    for perm in perms:
        if not names.has_key(perm.name):
            session.delete(perm)
            continue
        update = names[perm.name]
        print update
        availablenames.append(perm.name)
        cat = session.query(PermissionCategory).filter(PermissionCategory.name==update.category.name)
        if cat.count() > 0:
            perm.category = cat.one()
        else:
            perm.category = update.category
        perm.description = update.description
        perm.title = update.title
        session.add(perm)
    session.flush()

    # now add all the ones that are new.
    for permission in permissions:
        if permission.name not in availablenames:
            # permissions from multiple modules can be assigned to same category just by providing
            # the same category name
            if permission.category.id is None:
                cat = session.query(PermissionCategory).filter(PermissionCategory.name==permission.category.name)
                if cat.count() > 0:
                    permission.category = cat.one()
            session.add(permission)
    session.flush()

    ### sync UsageOptions
    # Delete every usage option that is not defined in the above usage option list.
    # All uo that are deleted are obviously out of date and no
    # longer used.
    from basesoft.web.contrib.license.models import UsageOption
    uos = session.query(UsageOption).all()
    availablenames = []
    names = [p.name for p in usage_options]
    for uo in uos:
        if uo.name not in names:
            session.delete(uo)
        else:
            availablenames.append(uo.name)
    session.flush()

    # now add all the ones that are new.
    for usage_option in usage_options:
        if usage_option.name not in availablenames:
            session.add(usage_option)
    session.flush()

    # set back whatever language we had before we added the permissions and usage options to the db
    translation.activate(current_language)

def post_sync(sa_session, app_name):

    if app_name != 'basesoft.web.contrib.account':
        return

    roles = []
    for app in settings.INSTALLED_APPS:
        try:
            sa_module = __import__(app + '.sa_datamapper', {}, {}, [''])
        except ImportError:
            # if we do not have a datamapper then just continue as there
            # are no permissions to be sync'ed
            continue

        if hasattr(sa_module, 'app_roles'):
            roles += sa_module.app_roles()

    """
    Now we Sync automated Role definitions.
    """
    from basesoft.web.contrib.license.models import Role, Permission
    current_roles = sa_session.query(Role).filter(Role.role_type==Role.ROLE_TYPE_AUTOMATIC).all()
    names = [r['name'] for r in roles]
    _map = {}
    for cr in current_roles:
        # delete automatic roles which are no longer used.
        if cr.name not in names:
            sa_session.delete(cr)
        else:
            _map[cr.name] = cr.id
    sa_session.flush()

    from basesoft.web.contrib.license.controllers import LicenseController
    lc = LicenseController(sa_session)

    pnames = {}
    for p in sa_session.query(Permission).all():
        pnames[p.name] = p

    for role in roles:
        id = None
        if _map.has_key(role['name']):
            id = _map[role['name']]

        lc.save_or_update_role(id, Role.ROLE_TYPE_AUTOMATIC, role['name'], [pnames[p] for p in role['permissions']], role['license_configs'], required_permission=role['required_permission'])
