# Copyright (c) 2008 BaseSoft B.V.
# See LICENSE for details.

from django.utils.translation import ugettext as _

from basesoft.web.contrib.account.decorators import login_required
from basesoft.web.core.forms.handler import FormValidator
from basesoft.web.core.jsonrpc.dispatcher import JsonRpcDispatcher
from basesoft.web.contrib.license.controllers import LicenseController

@login_required
def domain_and_license_form_rpc(request):
    """
    """
    from basesoft.web.core.input.validation import plugins
    form_handler = FormValidator({
        'domain_name': [(plugins.Required, {})],
        'license_name': [(plugins.Required, {})],
        'group_name': [(plugins.Required, {})]
        })
    dispatcher = JsonRpcDispatcher()
    dispatcher.register_instance(form_handler)
    return dispatcher.handle_request(request)

@login_required
def group_form_rpc(request):
    from basesoft.web.core.input.validation import plugins
    form_handler = FormValidator({
        'name': [(plugins.Required, {})]
        })
    dispatcher = JsonRpcDispatcher()
    dispatcher.register_instance(form_handler)
    return dispatcher.handle_request(request)

@login_required
def hook_form_rpc(request):
    from basesoft.web.core.input.validation import plugins
    form_handler = FormValidator({
        'domain': [(plugins.Required, {})],
        'object': [(plugins.Required, {})]
        })
    dispatcher = JsonRpcDispatcher()
    dispatcher.register_instance(form_handler)
    return dispatcher.handle_request(request)

@login_required
def priv_form_rpc(request):
    from basesoft.web.core.input.validation import plugins
    form_handler = FormValidator({
        'group_id': [(plugins.Required, {})],
        'privileges': [(plugins.Required, {})]
        })
    dispatcher = JsonRpcDispatcher()
    dispatcher.register_instance(form_handler)
    return dispatcher.handle_request(request)

class LicenseRpc:

    def __init__(self, session, account_id):

        self.account_id = account_id
        self.session = session

        # instanciate a compose controller
        self.controller = LicenseController(session, account_id=account_id)

    def get_domains(self):
        return [{'label': domain.name, 'value': domain.id} for domain in self.controller.get_domains()]

    def get_current_domain(self):
        domain = self.controller.get_default_group().license.domain
        return {'label': domain.name, 'value': domain.id}

    def get_own_groups(self):
        return [{'label': group.name, 'value': group.id} for group in self.controller.get_own_domain_groups()]

    def get_groups_with_permissions(self, permissions):
        return [{'label': '%s - %s' %(group.license.name, group.name), 'value': group.id}
                for group in self.controller.get_groups_with_permissions(permissions)]

    def get_licenses(self, exclude_current=False):
        if not exclude_current:
            return [{'label': license.name, 'value': license.id} for license in self.controller.get_licenses()]
        else:
            current_license = self.controller.get_default_group().license
            return [{'label': license.name, 'value': license.id}
                    for license in self.controller.get_licenses() if license.id != current_license.id]

    def get_available_license_types(self):
        ret = [{'label': t['label'], 'value': t['value'], 'default': False} for t in self.controller.get_available_license_types()]
        if len(ret) > 0:
            ret[0]['default'] = True
        return ret

    def get_domain_licenses(self, domain_id=None):
        return [{'label': license.name, 'value': license.id} for license in self.controller.get_domain_licenses(domain_id)]

    def get_reseller_licenses(self):
        return [{'label': license.name, 'value': license.id} for license in self.controller.get_reseller_licenses()]

    def get_permissions(self):
        """
        This returns the list of permissions that the group has access too.
        The label will be the translated title, but if title is not present, use the handler.
        """
        permissions = []
        for permission in self.controller.get_permissions():
            if permission.title:
                label = _(permission.title)
            else:
                label = permission.name
            permissions.append({'label': label, 'value': permission.id})
        return permissions

    def get_permissions_per_app(self):
        """
        This returns the list of permissions that the group has access too. All permissions will be grouped
        per application.
        The label will be the translated title, but if title is not present, use the handler.
        """
        return self.controller.get_ordered_permissions_per_app(self.controller.get_permissions())

    def get_license_permissions_per_app(self, license_id):
        return self.controller.get_ordered_permissions_per_app(license.permissions)

    def get_license_permissions(self, license_id):
        """
        """
        license = self.controller.get_licenses(license_id)[0]
        permissions = []
        for permission in license.permissions:
            if permission.title:
                label = _(permission.title)
            else:
                label = permission.name
            permissions.append({'label': label, 'value': permission.id})
        return permissions

    def has_permissions(self, *check_permissions):
        """
        This returns a list with the user's permissions (from the ones given as arg)
        """
        return [perm for perm in check_permissions if self.controller.has_permissions(perm)]

    def validate_default_group(self, license_id, group_id=None):
        """
        """
        return self.controller.validate_default_group(license_id, group_id)

    def get_usage_options(self):
        """
        This returns the list of usage options that the group has access too
        The label will be the translated title, but if title is not present, use the handler.
        """
        usage = []
        for usage_opt in self.controller.get_usage_options():
            if usage_opt.title:
                label = _(usage_opt.title)
            else:
                label = usage_opt.name
            usage.append({'label': label, 'value': usage_opt.id})
        return usage

    def load_group(self, group_id):
        """
        Return the group state
        """
        return self.controller.load_group_state(group_id)

    def load_license(self, license_id):
        """
        Return license state
        """
        return self.controller.load_license_state(license_id)

    def get_license_groups(self, license_id=None, domain_id=None):
        """
        This will get all the groups that are attached to license_id
        """
        if license_id is None:
            return [{'label': '%s - %s' % (group.license.name, group.name), 'value': group.id}
                    for group in self.controller.get_groups_for_domain(domain_id)]

        all = self.controller.get_licenses(license_id)
        if len(all) == 0:
            return []
        license = all.pop()

        if domain_id is not None and license.domain.id != domain_id:
            return []

        groups = self.controller.get_groups_for_license(license)
        rv = []
        for group in groups:
            rv.append({'label': group.name, 'value': group.id})
        return rv

    def save_group(self, license_id, data, group_id=None):
        """
        Creates or updates a groups name, permissions and other settings.
        """
        group = self.controller.save_group_from_state(data, license_id, group_id)
        return group.id

    def save_domain_group(self, group_id, domains):
        """
        Links domains to groups
        @param group_id: the id of the group to which we want to link domains
        @param domains: a list of domain ids that we want to link to the group
        """
        self.controller.save_domain_group(int(group_id), [int(d) for d in domains])
        return True

    def save_license(self, data):
        """
        TODO: these two methods should be wrapped in a Database transaction.
        """
        license = self.controller.save_license_from_state(data['state'])
        if data.has_key('create_account') and data['create_account']:
            from basesoft.web.contrib.account.controllers import AccountController
            ac = AccountController(self.session, account_id=self.account_id)
            ac.create_account_from_license_contact(license, data['activate_account'])
        return license.id

    def save_privilege_hook(self, state):
        """
        This will save a new privilege hook.
        """
        poh = self.controller.save_privilege_hook_from_state(state)
        return poh.id

    def has_object_group_privileges(self, object, id, group_id):
        return self.controller.has_object_group_privileges(object, id, group_id)

    def validate_privilege_delegation(self, object, id):
        """
        """
        self.controller.assert_privilege_delegation(object, id)

        return self.controller.get_privilege_object_privileges(object, id)

    def save_privilege_delegation_for_own_group(self, object, id, privileges):
        group_id = self.controller.get_default_group().id
        return self.save_privilege_delegation(object, id, group_id, privileges, True)

    def save_privilege_delegation(self, object, id, group_id, privileges, group_has_privilege, license_has_privilege=None, domain_has_privilege=None):
        privileges = [int(p) for p in privileges]
        instance = self.controller.save_privilege_delegation(object, id, int(group_id), privileges, group_has_privilege, license_has_privilege, domain_has_privilege)

    def delete_group_privileges(self, object, id, group_ids):
        return self.controller.delete_group_privileges(object, id, group_ids)

    def get_groups_for_privilege_object(self, object, id, filter_owner=True):
        return [{'label': '%s - %s' % (group.license.name, group.name), 'value': group.id}
                for group in self.controller.get_groups_for_privilege_object(object, id, filter_owner)]

    def get_object_privilege_group(self, object, id, group_id):
        return self.controller.get_object_privilege_group(object, id, group_id)

    def get_privilege_object_settings(self, object, id):
        return self.controller.get_privilege_object_settings(object, id)

    def get_privilege_objects(self):
        """
        Returns all the privilege objects for any app thats enabled in the settings file.
        """
        return self.controller.get_privilege_objects()

    def get_privilege_object_privileges(self, object, id=None):
        return self.controller.get_privilege_object_privileges(object, id)

@login_required
def rpc(request):
    """
    """
    user_id = request.session['user_id']
    sa_session = request.Session()

    dispatcher = JsonRpcDispatcher()
    dispatcher.register_instance(LicenseRpc(sa_session, user_id))
    return dispatcher.handle_request(request)

from basesoft.web.core.util.custom_query import CustomQueryRpcHandler

def custom_query(request, object_name, object_id):

    name = 'basesoft.web.contrib.license.PrivilegeDelegationCustomQuery'
    dispatcher = JsonRpcDispatcher()
    dispatcher.register_instance(CustomQueryRpcHandler(name, request.Session(), account_id=request.session['user_id'], object_name=object_name, object_id=object_id))
    return dispatcher.handle_request(request)

def object_browser(request, target, value):

    name = 'basesoft.web.contrib.license.PrivilegeObjectBrowserCustomQuery'
    dispatcher = JsonRpcDispatcher()
    dispatcher.register_instance(CustomQueryRpcHandler(name, request.Session(), account_id=request.session['user_id'], target=target, value=value))
    return dispatcher.handle_request(request)

