# Copyright (c) 2008 BaseSoft B.V.
# See LICENSE for details.

"""
These are the models used within the mailing django application.
Different applications like Survey and Report can hook into the mailing app
with a definition of their custom variable replacements
"""

from django.conf import settings

from sqlalchemy.orm.session import Session
from sqlalchemy.sql import and_
from sqlalchemy.orm.exc import NoResultFound

from django.utils.translation import ugettext

from basesoft.web.core.models import PrivilegeObject

class ModelWrapper(object):
    """
    Model wrapper is used for wiki replacement fields
    """
    def __init__(self, person, cb, context):
        self.person = person
        self.cb = cb
        self.context = context

    def __getattr__(self, name):
        return self.cb(self.person, name, self.context)

class ContextWrapper(object):
    def __init__(self, person, cb, context):
        self.wrapper = ModelWrapper(person, cb, context)

    def __getattr__(self, name):
        return self.wrapper

class MailLocale(object):
    """
    The mail can be composed in different locales.
    @DynamicAttrs
    """

    mail = None
    locale = None

    subject = None
    content = None

class Mail(PrivilegeObject):
    """
    Mapped to the mail table

    Children classes that extend this class (IQuestMail, AccountActivationMail, etc) need to have NAME_I18N defined.
    eg. NAME_I18N = _("IQuest Mail")
    This is a friendly translatable name for the mail, that will be displayed in menus, etc.
    @DynamicAttrs
    """

    # privilege to schedule a mailing
    PRIV_SCHEDULE = 32

    all_privileges = (PrivilegeObject.PRIV_VIEW, PrivilegeObject.PRIV_UPDATE, PrivilegeObject.PRIV_ARCHIVE, \
                      PrivilegeObject.PRIV_DELEGATE_PRIVILEGES, PRIV_SCHEDULE)

    mark_as_deleted_permission = 'delete_mail'
    mark_as_archived_permission = 'update_mail'

    JS_MODULE = None

    @staticmethod
    def replacement_fields():
        """
        This are the replacement fields that all mails will have and derived Mail models can extend this method for
        overriding replacement fields.
        """
        from basesoft.web.contrib.contact.models import Person
        return {
            ugettext('Person'): {
                'person.first_name': Person.cols_i18n['first_name'],
                'person.last_name': Person.cols_i18n['last_name'],
                'person.middle_name': Person.cols_i18n['middle_name'],
                'person.title': Person.cols_i18n['title'],
                'person.position': Person.cols_i18n['position'],
                'person.department': Person.cols_i18n['department'],
            }
        }

    def get_group_object(self):
        return (MailGroup, 'mail')

    def get_privileges(self):
        return PrivilegeObject.get_privileges(self) + [
            (self.PRIV_SCHEDULE, ugettext('Schedule privileges'))
        ]

    def __repr__(self):
        return '<Mail id="%s", name="%s">' % (self.id, self.name)

    # this regexp is for replacement fields
    regexp = '<img[^>]*(?:[^>]*\s+class="?replacement-field\s*"?|[^>]*\s+src="?(?:[^"]*)"?|[^>]*\s+title="?([\w_\.]+)"?){3}[^>]*/?>'

    # this regexp is for embedding images. Example: <img  src="basesoft.web.contrib.file/view/8854b1a4_TNT%20Sure%20we%20can.jpg">
    # has to be done with mail context so in schedule
    regexp_embed = '<img[^>]*src="[^"]*(basesoft.web.contrib.file/view/([\w\d]{8})_[^"]*)"[^>]*>'

    def get_html_mail(self, person, context, html=None):
        """
        """
        import re
        replacement_fields_pattern = re.compile(self.regexp)

        def replace_field(match_obj):
            """
            Scoped function gets called for every replacement field (<input>) found in the html message with as argument
            the match object found by python's regular expression .sub method. Returns the string the that the value
            needs to be replaced with.
            """
            name = match_obj.group(1)
            return self._replace_field(person, name, context)

        if html is None:
            # get the base mail content
            html = self.get_locale_mail(person.preferred_locale).content

        # we are dealing with wiki content
        if self.is_wiki:
            from basesoft.web.contrib.wiki.parser import Parser
            c = ContextWrapper(person, self._replace_field, context)
            return Parser().render_to_html(html, c)

        # process replacement fields
        html = re.sub(replacement_fields_pattern, replace_field, html)
        return html


    def _replace_field(self, person, field_name, context=None):
        """
        Method gets called for every found replacement-field. Arguments are the context (contact, mailing) and
        field to be replaced. It should return the new value for the replacement field.
        If a task implements replacement fields it probably wants to override this method.
        """

        def format(return_value):
            """
            Format the return value based on what type it has, etc.
            @param return_value: the value that needs to be returned for the replacement field. can be of any type
            @return String: the formatted value for the replacement field
            """
            # here we check for the type of the return value, and format the string accordingly if needed
            if return_value is None: return_value = ''
            # TODO: if we have a date or datetime, format it accordingly

            return "%s" % return_value # return the data as string, no matter what it might be (date, date-time, etc)

        replacement_fields = {}

        if person is not None:
            replacement_fields.update({
                'person.last_name': person.last_name,
                'person.first_name': person.first_name,
                'person.middle_name': person.middle_name,
                'person.title': person.title,
                'person.position': person.position,
                'person.department': person.department
            })

        if replacement_fields.has_key(field_name):
            return format(replacement_fields[field_name])

        # by default return an empty string if no replacement field value was found
        return ''

    def get_locale_mail(self, locale, mail_locale_class=MailLocale):
        """
        Method that returns the best MailLocale object for this mail based on the given locale object.
        """
        session = Session.object_session(self)

        # first try the given locale
        try:
            return session.query(mail_locale_class).filter(and_(mail_locale_class.mail==self, mail_locale_class.locale==locale)).one()
        except NoResultFound:
            # nuffin found.. try the fallback locale
            try:
                return session.query(mail_locale_class).filter(and_(mail_locale_class.mail==self, mail_locale_class.locale==self.fallback_locale)).one()
            except NoResultFound:
                # no fallback avaiable either.. whatever.. return the first locale or error out
                try:
                    return self.locales[0]
                except IndexError:
                    # TODO raise a proper exception here
                    raise Exception('%s: No fallback locale content available.' % self)

    def clone(self):
        raise NotImplementedError()

    def _base_url(self, context):
        assert isinstance(context, dict)

        if not context.has_key('license') or context['license'] is None:
            raise Exception('Invalid license in context: %s' % context)

        license = context['license']

        url = settings.BASE_URL

        http_host = None
        if license.http_host is not None:
            http_host = license.http_host
        elif license.domain.http_host is not None:
            http_host = license.domain.http_host

        if http_host is not None:
            if settings.BASE_URL.startswith('https://'):
                url = 'https://'
            else:
                url = 'http://'
            url = url + http_host
            if not url.endswith('/'):
                url = url + '/'

        return url

    def mail_from(self):
        if self.mail_from_text is None:
            return None
        return '"%s" <noreply@iqportalmail.com>' % self.mail_from_text

class MailGroup(object):
    """
    Model representing the link between mails and groups. Used for privileges.
    @DynamicAttrs
    """

class Mailing(object):
    """
    Mapped to the mailing table
    @DynamicAttrs
    """
    STATUS_SCHEDULED = 0                # initial state (allows editing)
    STATUS_PREPARING_FOR_SENDING = 1    # the send_data has passed but approved_for_sending is not True yet.
    STATUS_SENDING = 2                  # the scheduler is currently sending out the emails
    STATUS_SENT = 3                     # the mailing has been sent
    STATUS_ARCHIVED = 4                 # the mailing is done and archived, can't be resent.

    FILTER_NONE = 0 # this will send everyone the mail

    mail = None
    contact_lists = None
    status = None
    send_date = None

    def get_filters(self):
        return [(self.FILTER_NONE, ugettext('No filter'))]

    def approved_for_sending(self):
        return True

    def __repr__(self):
        return '<Mailing id="%s", send_date="%s">' % (self.id, self.send_date)

class MailResult(object):
    """
    A link between a mailing and a contact, containing a status
    @DynamicAttrs
    """
    STATUS_NEW = 0
    # Accepted gets set when the outgoing smtp server accepted the email for delivery. As you can never really know with
    # this doesnt mean the email is sent to the user. It could still be that the email gets bounced while it is being
    # relayed to the final destination server.
    STATUS_ACCEPTED = 1

    # Refused is set when the outgoing smtp server directly refuses the email.
    STATUS_REFUSED = 2

    # In case of server communcation errors the status gets set to 'error'.
    STATUS_ERROR = 3

    # TODO: implement bounce status, somehow, someday (status not yet implemented!)
    STATUS_BOUNCED = 4

    # if the email address was not valid or some other condition was raised that forces this mail to be ignored.
    STATUS_IGNORED = 5

    mail = None
    mailing = None

class MailingContactList(object):
    """
    @DynamicAttrs
    """
    pass

