# Copyright (c) 2008 BaseSoft B.V.
# See LICENSE for details.

"""
This file contains the objects, functions and variables to setup the Mailing app
"""

import datetime
from sqlalchemy import Column, Unicode, \
                       Integer, ForeignKey, String, \
                       DateTime, UnicodeText, SmallInteger
from basesoft.sqlalchemy.table import IQTable
from basesoft.web.contrib.license.hooks import PrivilegeObjectMapper
from basesoft.web.contrib.license import models as license_models
from django.utils.translation import ugettext as _

__mapping_defined = False
__tables_defined = False
tables = {}

def reset():
    global __mapping_defined
    global __tables_defined

    __mapping_defined = False
    __tables_defined = False

def get_tables():
    return tables

def do_tables(metadata):
    global __tables_defined
    if not __tables_defined:

        # the definition of the mail including privileges options.
        tblMail = IQTable('tblMail', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('name', String(40), nullable=False),
            Column('mail_from_text', Unicode(40), nullable=True), # can be overridden in tblMailing.
            Column('is_wiki', SmallInteger, nullable=False, default=1),
            Column('polymorphic_type', String(255), nullable=False),
            Column('deleted', SmallInteger, nullable=False, default=0),
            Column('archived', SmallInteger, nullable=False, default=0),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now),
        )

        # This is the link between a group of accounts and the mail, this will be used for access rights.
        tblMailGroup = IQTable('tblMailGroup', metadata,
            Column('mail_id', Integer, ForeignKey('tblMail.id', ondelete="CASCADE"), primary_key=True),
            Column('group_id', Integer, ForeignKey('tblGroup.id'), primary_key=True),
            Column('privilege', Integer, nullable=False, primary_key=True, autoincrement=False),
            Column('group_has_privilege', SmallInteger, default=0),
            Column('license_has_privilege', SmallInteger, default=0),
            Column('domain_has_privilege', SmallInteger, default=0),
            Column('world_has_privilege', SmallInteger, default=0),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        # the content and subject definitions in different locales.
        tblMailLocale = IQTable('tblMailLocale', metadata,
            Column('mail_id', Integer, ForeignKey('tblMail.id', ondelete='CASCADE'), primary_key=True),
            Column('locale_id', Integer, ForeignKey('tblLocale.id'), primary_key=True),
            Column('subject', String(128), nullable=False),
            Column('content', UnicodeText, nullable=False),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        # an actual scheduled mailing, for status indications see the models.
        tblMailing = IQTable('tblMailing', metadata,
            Column('id', Integer, primary_key=True),
            Column('polymorphic_type', String(255), nullable=False),
            Column('mail_id', Integer, ForeignKey('tblMail.id'), nullable=False),
            Column('mail_from_text', String(100), nullable=False),
            Column('status', Integer, nullable=False, default=0),
            Column('filter', Integer, nullable=False, default=0),
            Column('number_of_recipients', Integer, nullable=True),
            Column('send_date', DateTime(), nullable=False),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        # which contact lists' contacts should receive the mailing
        tblMailingContactList = IQTable('tblMailingContactList', metadata,
            Column('mailing_id', Integer, ForeignKey('tblMailing.id', ondelete='CASCADE'), primary_key=True),
            Column('contact_list_id', Integer, ForeignKey('tblContactList.id', ondelete='CASCADE'), primary_key=True),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        # the result of the mailing on a per mailing/per contact base
        tblMailResult = IQTable('tblMailResult', metadata,
            Column('contact_id', Integer, ForeignKey('tblContact.id', ondelete='CASCADE'), primary_key=True),
            Column('mailing_id', Integer, ForeignKey('tblMailing.id', ondelete='CASCADE'), primary_key=True),
            Column('status', Integer, nullable=False),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        tables['tblMail'] = tblMail
        tables['tblMailGroup'] = tblMailGroup
        tables['tblMailLocale'] = tblMailLocale
        tables['tblMailing'] = tblMailing
        tables['tblMailingContactList'] = tblMailingContactList
        tables['tblMailResult'] = tblMailResult

        __tables_defined = True

def do_mappings():
    """
    This function is called by Django middleware when needed.
    """
    global __mapping_defined
    if not __mapping_defined:
        from sqlalchemy.orm import mapper, relation
        from basesoft.web.contrib.mailing import models as mailing_models
        from basesoft.web.contrib.i18n import models as i18n_models
        from basesoft.web.contrib.contact import models as contact_models

        mapper(mailing_models.Mail, tables['tblMail'],
            extension = PrivilegeObjectMapper(),
            properties = {
                'groups': relation(mailing_models.MailGroup, backref='mail', cascade="all", passive_deletes=True)
                # don't add locales property here since some of it's children might have other Locale table (tnt)
            },
            polymorphic_on=tables['tblMail'].c.polymorphic_type,
            polymorphic_identity='Mail'
        )

        mapper(mailing_models.MailGroup, tables['tblMailGroup'], properties = {

            'group': relation(license_models.Group)
        })

        mapper(mailing_models.MailLocale, tables['tblMailLocale'], properties = {
           'mail': relation(mailing_models.Mail),
           'locale': relation(i18n_models.Locale)
        })

        mapper(mailing_models.Mailing, tables['tblMailing'],
            properties = {
                'mail': relation(mailing_models.Mail),
                'contact_lists': relation(contact_models.ContactList, secondary=tables['tblMailingContactList']),
                'results': relation(mailing_models.MailResult),

            },
            polymorphic_on=tables['tblMailing'].c.polymorphic_type,
            polymorphic_identity='Mailing'
        )

        mapper(mailing_models.MailResult, tables['tblMailResult'], properties = {
            'mailing': relation(mailing_models.Mailing),
            'contact': relation(contact_models.Contact)
        })

        mapper(mailing_models.MailingContactList, tables['tblMailingContactList'],
            properties = {
                'mailing': relation(mailing_models.Mailing),
                'contact_list': relation(contact_models.ContactList)
            }
        )

        __mapping_defined = True

def fill_date(conn):
    pass

def app_permissions_category():
    """
    Returns a category for all permissions defined for this app
    """
    return license_models.PermissionCategory(u'Mailing')

def app_permissions():
    """
    The permissions used within this application. Can be sync'd with the
    `iqsyncpermissions` command.
    """
    return [
        license_models.Permission(u"create_mail", u"""
            Allow the user to create a new mail. This is only a global permission
            and is not related to the different app extensions like survey mail
            or report mail.
        """, _("Create mails")),
        license_models.Permission(u"view_mail", u"""
            This gives the rights to view a composed email. This is only a global
            permission and is not related to the different app extensions like
            survey mail or report mail.
        """, _("View mails")),
        license_models.Permission(u"update_mail", u"""
            This gives the rights to update an existing email. This is only a
            global permission and is not related to the different app extensions
            like survey mail or report mail.
        """, _("Update mails")),
        license_models.Permission(u"delete_mail", u"""
            This gives the rights to delete a mail. This is only a global
            permission and is not related to the different app extensions like
            survey mail or report mail.
        """, _("Delete mails")),
        license_models.Permission(u"schedule_mail", u"""
            This gives the rights to schedule a mail. This is only a global
            permission and is not related to the different app extensions like
            survey mail or report mail.
        """, _("Schedule mails")),
        license_models.Permission(u"view_mailing_results", u"""
            This gives the rights to view mailing results. Directly related
            to view rights on mail level and also related to the extension app
            it was composed for.
        """, _("View mailing results"))
        ]

def app_configs():
    from basesoft.web.contrib.account import models as account_models
    configs = []

    # Can be monthly, yearly, (quarterly)
    config = account_models.Config()
    config.name = 'basesoft.web.contrib.mailing.default_mail_from_text'
    config.type = 'string'
    config.default = ''
    config.description = u'Default text for the "From" field in emails.'
    configs.append(config)

    return configs
