# Copyright (c) 2008 BaseSoft B.V.
# See LICENSE for details.

from basesoft.web.core.jsonrpc.dispatcher import JsonRpcDispatcher
from basesoft.web.contrib.mailing.controllers import MailBaseController
from basesoft.web.contrib.account.decorators import login_required
from basesoft.web.core.exceptions import SecurityException
from basesoft.web.contrib.branding.controllers import HeaderFooterController
from basesoft.web.contrib.license.models import Group

@login_required
def mail_manager_rpc(request):
    """
    rpc view for the mail compose panel
    """
    sa_session = request.Session()
    account_id = request.session['user_id']

    locale = request.session['config']['basesoft.web.contrib.locale']

    mail_controller = MailBaseController(sa_session, locale, account_id)

    class ManagerRpc:
        """
        """
        def delete_mail(self, id):
            """
            Deletes the mails with the given id
            @param id: an id or a list of ids of mails that need to be deleted
            @return the list of ids that were not deleted
            """
            undeleted = [] # list where we keep the id's of the mails we don't have permissions to delete
            if not isinstance(id, list): id = [id]
            for mail_id in id:
                try:
                    mail_controller.delete_mail(mail_id)
                except SecurityException:
                    undeleted.append(mail_id)

            return undeleted

        def get_mail_types(self, mail_type=None):
            """
            @param mail_type: None, string or list. the mail types that interest us
                None or empty list if we want all the ones we have access to
            Returns a list of all the mail types that the user has permissions to create.
            List contains the types: eg. ['MockMail', 'IQuestMail']
            """
            available_types = mail_controller.get_all_mail_types()

            l = [{'name': class_name.NAME_I18N, 'type': class_name.__name__, 'js_module': class_name.JS_MODULE} for class_name in available_types]

            if mail_type is not None:
                if not isinstance(mail_type, list):
                    mail_type = [mail_type]

                # if were' interested in certain mail types, filter to get only the ones we want
                if len(mail_type) > 0:
                    l = [e for e in l if e['type'] in mail_type]

            return l

        def get_mails(self, id=None, mail_type=None):
            """
            Return a list with mail objects on which the user has "view" rights.
            @param id: the id of the mail that interests us. None if we want all
            @param mail_type: the type of the mail that interests us. eg. 'IQuestMail' if we only want IQuestMails
                or None if we want all mails
            """
            mails = mail_controller.get_mails(id=id, mail_type=mail_type)
            return [{"id": m.id, "name": m.name} for m in mails]

        def duplicate_mail(self, id, mail_type=None):
            original = mail_controller.get_mails(id=id, mail_type=mail_type).pop()
            mail_controller.duplicate_mail(original)
            return True

    m = ManagerRpc()

    dispatcher = JsonRpcDispatcher()
    dispatcher.register_instance(m)
    return dispatcher.handle_request(request)

@login_required
def mail_compose_rpc(request):
    """
    rpc view for the mail compose panel
    """
    sa_session = request.Session()
    account_id = request.session['user_id']

    locale = request.session['config']['basesoft.web.contrib.locale']

    mail_controller = MailBaseController(sa_session, locale, account_id)
    hf_controller = HeaderFooterController(sa_session, locale, account_id)

    class MailComposeRpc:

        def load_by_id(self, mail_id):
            """
            Load the requested mail.
            This method is called from the MailDetailPanel on load
             or when requested
            @param mail_id: {int} the id of the mail we want to load
            @return: the state of the given mail
            """
            assert isinstance(mail_id, int)

            state = mail_controller.load_by_mail_id(mail_id)
            return state

        def save_state(self, state, group=None):
            """
            Save the mail from the state and return the id
            This method is called from the MailDetailPanel when user uses the
                save menu
            @param state: {dict} the state of the mail we want to save. if it
                has a dbID set, it will update, if it's not present, save new
            @return: the id of the saved mail
            """
            assert isinstance(state, dict)
            if group is not None:
                group = sa_session.query(Group).filter(Group.id==group).one()
            mail = mail_controller.save_state(state, group=group)
            return mail.id

        def get_replacement_fields(self, mail_type, is_wiki=False):
            """
            Returns the replacement fields for the given mail_type
            """
            return mail_controller.get_replacement_fields(mail_type, is_wiki)

        def get_locales(self, mail_type):
            return mail_controller.get_locales_state(mail_type)

        def get_mail_locales(self, mail_type, mail_id, readOnly=False):
            """
            Returns locales.
            @param readOnly: only return mail locales (so locales for which mail has translations)
            """
            if mail_id is None and readOnly:
                return []
            locales = mail_controller.get_locales(mail_type)
            if not readOnly:
                return [l.get_state() for l in locales]
            if mail_id is not None:
                mail_locales = mail_controller.get_mail_locales(mail_id, [l for l in locales])
                return [l.locale.get_state() for l in mail_locales]
            return []

        def get_extra_data(self, mail_type):
            """
            Returns the replacement fields and available locales for the given mail_type
            """
            locales_state = mail_controller.get_locales_state(mail_type)

            fallback_options = {}
            for f in locales_state:
                fallback_options[f['locale']] = f['locale']

            header_footer_count = 0
            if hf_controller.has_permissions('branding_view_header_footer'):
                header_footer_count = len(hf_controller.get_header_footers())

            return {
                'js_module': mail_controller.get_js_module(mail_type),
                'replacement_fields': mail_controller.get_replacement_fields(mail_type),
                'locales_state': locales_state,
                'fallback_options': fallback_options,
                'header_footer_count': header_footer_count,
            }

    mc = MailComposeRpc()

    dispatcher = JsonRpcDispatcher()
    dispatcher.register_instance(mc)
    return dispatcher.handle_request(request)

def mail_compose_validation_rpc(request):
    """View that handles form validation in the mail compose interface"""

    from basesoft.web.core.input.validation import plugins
    from basesoft.web.core.forms.handler import FormValidator

    form_handler = FormValidator(
        fields = {
            'name': [(plugins.Required, {})]
        }
    )
    dispatcher = JsonRpcDispatcher()
    dispatcher.register_instance(form_handler)
    return dispatcher.handle_request(request)

def mailing_validation_rpc(request):
    """View that handles form validation in the mailing form"""

    from basesoft.web.core.input.validation import plugins
    from basesoft.web.core.forms.handler import FormValidator

    form_handler = FormValidator(
        fields = {
            'mail_id': [(plugins.Required, {})],
            'mail_from_text': [(plugins.Required, {})],
            'send_date': [(plugins.Required, {})],
            'contact_lists': [(plugins.Required, {})],
        }
    )
    dispatcher = JsonRpcDispatcher()
    dispatcher.register_instance(form_handler)
    return dispatcher.handle_request(request)
