# Copyright (c) 2011 BaseSoft B.V.
# See LICENSE for details.
from sqlalchemy import and_
from sqlalchemy.orm.exc import NoResultFound

from basesoft.web.core.util.controllers import AbstractController, transaction_decorator
from basesoft.web.contrib.manual.models import ManualChapter, ManualChapterLocale
from basesoft.web.contrib.i18n.models import Locale
from basesoft.web.contrib.license.controllers import LicenseController

class ManualController(AbstractController):

    def get_chapters(self, id=None, verify=True):
        """
        Returns chapters to which the user has permissions.
        In case id is provided it returns only chapters for those ids.
        """
        if id is None:
            chapters = self.sa_session.query(ManualChapter).all()
        else:
            if type(id) != list:
                id = [id]
            chapters = self.sa_session.query(ManualChapter).filter(ManualChapter.id.in_(id)).all()
        rv = []
        for c in chapters:
            if verify and c.permissions is not None and not self.has_permissions(*[p.name for p in c.permissions]):
                continue
            rv.append(c)
        return rv

    def get_by_application_and_handle(self, app, handle):
        return self.sa_session.query(ManualChapter).filter(and_(ManualChapter.app==app, ManualChapter.handle==handle)).first()

    def get_by_parent_chapter(self, parent):
        return self.sa_session.query(ManualChapter).filter(ManualChapter.parent_chapter==parent).all()

    def get_all_chapters(self, translated=True):
        """
        Returns a list with all the chapters (per application) for which the user has permissions.
        In case translated is set to True, it only returns chapters which are translated in the users' preferred locale.
        """
        rv = {}

        q = self.sa_session.query(ManualChapter)
        where = None

        # TODO: make work for public part of IQPortal also.
        if translated and self.get_account() is not None:
            locale = self.sa_session.query(Locale).filter(Locale.locale==self.config.get('basesoft.web.contrib.locale')).first()

            if locale is None:
                pass # TODO: get the default domain locale?

            q = q.join(ManualChapterLocale)
            where = ManualChapterLocale.locale==locale

        if where is not None:
            q = q.filter(where)

        chapters = q.all()

        for c in chapters:
            if c.permissions is not None and not self.has_permissions(*[p.name for p in c.permissions]):
                continue
            if not rv.has_key(c.app):
                rv[c.app] = {}
            rv[c.app][c.handle] = c.id

        return rv

    def get_chapter_hierachy(self):
        """
        Returns the chapter hierachy for the current user. It includes only chapter translated into their
        preferred locale
        """
        rv = []

        locale = self.sa_session.query(Locale).filter(Locale.locale==self.config.get('basesoft.web.contrib.locale')).one()
        q = self.sa_session.query(ManualChapter).join(ManualChapterLocale).filter(
                ManualChapterLocale.locale==locale and ManualChapter.parent_chapter_id == None)
        chapters = q.all()

        def get_hierachy(chapter):
            chap = { 'chapter_id': chapter.id, 'handle': chapter.handle, 'app': chapter.app,
                    'subchapters': [get_hierachy(sub) for sub in chapter.subchapters
                        if (sub.permissions is None or self.has_permissions(*[p.name for p in sub.permissions]))]
            }
            return chap

        for c in chapters:
            if c.permissions is not None and not self.has_permissions(*[p.name for p in c.permissions]):
                continue
            rv.append(get_hierachy(c))
        return rv


    def get_full_chapter(self, id, locale=None):
        """
        Returns a full recursive state including translations in the provided locale
        If no locale is provided, it will use the users' preferred locale.
        """

        def render_chapter(state, locale):
            rv = {}
            ct = self.get_chapter_translation(state['chapter'], locale)
            if ct is not None:
                rv['id'] = ct.chapter.id
                rv['title'] = ct.title
                rv['description'] = ct.description

            children = []
            for i in state['subchapters']:
                children.append(render_chapter(i, locale))

            if len(children) > 0:
                rv['subchapters'] = children

            return rv

        state = self.get_chapter_state(id, True)
        return render_chapter(state, locale)

    def get_chapter_state(self, id, include_object=False, verify=True):
        """
        Returns a recursive chapter state
        """
        c = self.get_chapters(id, verify=verify).pop()
        pc = None
        if c.parent_chapter is not None:
            pc = c.parent_chapter.id
        rv = {
            'id': id,
            'handle': c.handle,
            'application': c.app,
            'permissions': [p.id for p in c.permissions],
            'parent_chapter_id': pc,
            'subchapters': [self.get_chapter_state(s.id, include_object) for s in c.subchapters]
        }
        if include_object:
            rv['chapter'] = c
        return rv

    def get_chapter_translation(self, chapter, locale):
        """
        Get the translation for the chapter in the provided locale
        """
        if not isinstance(chapter, ManualChapter):
            chapter = self.get_chapters(chapter).pop()
        if not isinstance(locale, Locale):
            if locale is None:
                return None
            locale = self.sa_session.query(Locale).filter(Locale.id==locale).one()

        try:
            return self.sa_session.query(ManualChapterLocale).filter(
                and_(ManualChapterLocale.chapter==chapter, ManualChapterLocale.locale==locale)).one()
        except NoResultFound:
            return None

    @transaction_decorator
    def save_chapter_translation(self, id, values):
        """
        Saves the translation of a chapter (and it subchapters)
        """
        locale = self.sa_session.query(Locale).filter(Locale.id==values['locale']).one()

        def save_translation(state, values, locale):
            id = state['id']
            title = values['title_%s' % id]
            description = values['description_%s' % id]
            chapter = state['chapter']

            # we cannot have null values in the db
            if title == None:
                title = ''
            if description == None:
                description = ''

            ct = self.get_chapter_translation(chapter, locale)

            # we dont store empty translations
            if title == '' and description == '' and ct is not None:
                self.sa_session.delete(ct)
            elif title != '' or description != '':
                if ct is None:
                    ct = ManualChapterLocale()
                    ct.chapter = chapter
                    ct.locale = locale
                ct.title = title
                ct.description = description
                self.sa_session.add(ct)

            if len(state['subchapters']) > 0:
                for s in state['subchapters']:
                    save_translation(s, values, locale)

        state = self.get_chapter_state(id, True)
        save_translation(state, values, locale)

    @transaction_decorator
    def save_translation(self, chapter, title=None, description=None, force=False):
        """
        Save a translation for the provided chapter.
        In case force is False only store translations for non-existing locale translations.
        """
        locale = self.sa_session.query(Locale).filter(Locale.locale==self.config.get('basesoft.web.contrib.locale')).one()
        ct = self.get_chapter_translation(chapter, locale)
        if ct is None:
            ct = ManualChapterLocale()
            ct.locale = locale
            ct.chapter = chapter
            force = True

        if force is False:
            return

        if title is not None:
            ct.title = title

        if description is not None:
            ct.description = description
        else:
            ct.description = '';
        self.sa_session.add(ct)
        return ct

    @transaction_decorator
    def generate_manual_chapters(self, application, handle, title=None, subchapters=None, overwrite_translations=False, parent_chapter_id=None):
        """
        We can generate manual chapters based on the provided data (from client-side forms for example)
        """
        chapter = self.save_or_update_manual_chapter(None, application, handle, parent_chapter_id=parent_chapter_id)
        if title is not None:
            self.save_translation(chapter, title=title, force=overwrite_translations)

        if subchapters is not None:
            for s in subchapters:
                self.generate_manual_chapters(
                    application=application,
                    overwrite_translations=overwrite_translations,
                    parent_chapter_id=chapter.id,
                    **s)

        return chapter

    @transaction_decorator
    def save_or_update_manual_chapter(self, id, application, handle, parent_chapter_id=None, permissions=None):
        """
        Save or update a chapter definition (no translations)
        """
        if id is None:
            c = self.get_by_application_and_handle(application, handle)
            if c is None:
                self.assert_permissions('create_manual_chapter')
                c = ManualChapter()
            else:
                self.assert_permissions('update_manual_chapter')
        else:
            self.assert_permissions('update_manual_chapter')
            c = self.get_chapters(id).pop()

        c.app = application
        c.handle = handle
        c.position = 1
        if parent_chapter_id is not None and parent_chapter_id != '':
            c.parent_chapter = self.get_chapters(parent_chapter_id).pop()
            c.position = len(c.parent_chapter.subchapters)
        else:
            c.position = len(self.get_by_parent_chapter(None))

        if permissions is not None:
            lc = LicenseController(self.sa_session, account_id=self.get_account().id)
            c.permissions = lc.get_permissions(permissions)

        self.sa_session.add(c)
        return c
