# Copyright (c) 2011 BaseSoft B.V.
# See LICENSE for details.

import datetime

from django.utils.translation import ugettext as _

from sqlalchemy.orm import mapper, relation, backref
from sqlalchemy import UniqueConstraint, Column, Integer, String, ForeignKey, DateTime, UnicodeText, SmallInteger

from basesoft.sqlalchemy.table import IQTable
from basesoft.web.contrib.account import models as account_models
from basesoft.web.contrib.license import models as license_models
from basesoft.web.contrib.manual import models as manual_models
from basesoft.web.contrib.i18n import models as i18n_models

__mapping_defined = False
__tables_defined = False
tables = {}

def reset():
    global __mapping_defined
    global __tables_defined

    __mapping_defined = False
    __tables_defined = False

def get_tables():
    """
    """
    return tables

def do_tables(metadata):
    global __tables_defined
    if not __tables_defined:

        tblManualChapter = IQTable('tblManualChapter', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('parent_chapter_id', Integer, ForeignKey('tblManualChapter.id', ondelete='CASCADE'), nullable=True),
            Column('position', Integer, nullable=False),

            Column('app', String(255), nullable=False),
            Column('handle', String(255), nullable=False),

            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now),

            UniqueConstraint('app', 'handle')
        )

        tblManualChapterPermission = IQTable('tblManualChapterPermission', metadata,
            Column('chapter_id', Integer, ForeignKey('tblManualChapter.id', ondelete='CASCADE'), primary_key=True),
            Column('permission_id', Integer, ForeignKey('tblPermission.id', ondelete='CASCADE'), primary_key=True),
        )

        tblManualChapterLocale = IQTable('tblManualChapterLocale', metadata,
            Column('chapter_id', Integer, ForeignKey('tblManualChapter.id', ondelete='CASCADE'), primary_key=True),
            Column('locale_id', Integer, ForeignKey('tblLocale.id', ondelete='CASCADE'), primary_key=True),

            Column('title', UnicodeText, nullable=False),
            Column('description', UnicodeText, nullable=False), # use Wiki Formatting

            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        tables['tblManualChapter'] = tblManualChapter
        tables['tblManualChapterPermission'] = tblManualChapterPermission
        tables['tblManualChapterLocale'] = tblManualChapterLocale

        __tables_defined = True

def do_mappings():
    """
    This function is only called by the django middleware
    when it is needed
    """
    global __mapping_defined
    if not __mapping_defined:

        mapper(manual_models.ManualChapter, tables['tblManualChapter'],
            properties = {
                'subchapters': relation(manual_models.ManualChapter, backref=backref(
                    'parent_chapter', remote_side=[tables['tblManualChapter'].c.id]
                ), order_by=tables['tblManualChapter'].c.position),
                'permissions': relation(license_models.Permission, secondary=tables['tblManualChapterPermission'])
            }
        )

        mapper(manual_models.ManualChapterPermission, tables['tblManualChapterPermission'],
            properties = {
                'chapter': relation(manual_models.ManualChapter),
                'permission': relation(license_models.Permission)
            }
        )

        mapper(manual_models.ManualChapterLocale, tables['tblManualChapterLocale'],
            properties = {
                'chapter': relation(manual_models.ManualChapter),
                'locale': relation(i18n_models.Locale)
            }
        )

        __mapping_defined = True

def fill_data(conn):
    pass

def app_permissions_category():
    """
    Returns a category for all permissions defined for this app
    """
    return license_models.PermissionCategory(u'Manual')

def app_permissions():
    """
    The permissions used within this application. Can be sync'd with the
    `iqsyncpermissions` command.
    """
    return [
        license_models.Permission(u'view_manual_chapter', u"""
            Gives the permission to view manual chapters.
            """, _("View manual chapter")),
        license_models.Permission(u'create_manual_chapter', u"""
            Gives the permission to create a manual chapter.
            """, _("Create manual chapter")),
        license_models.Permission(u'update_manual_chapter', u"""
            Gives the permission to update a manual chapter.
            """, _("Update manual chapter")),
        license_models.Permission(u'translate_manual_chapter', u"""
            Gives the permission to translate a manual chapter.
            """, _("Translate manual chapter")),
        ]

def app_configs():
    """
    The account configs available for this application. This can be sync'd with the
    `iqsyncpermissions` command.
    """
    configs = []

    config = account_models.Config()
    config.name = 'manual.context.sensitive.help'
    config.type = 'boolean'
    config.default = 'True'
    config.description = u'Boolean for whether or not context sensitive help is shown.'
    configs.append(config)

    config = account_models.Config()
    config.name = 'manual.translate.locale.right'
    config.type = 'int'
    config.default = None
    config.description = u'Locale id for selected primary locale in translate interface.'
    configs.append(config)

    config = account_models.Config()
    config.name = 'manual.translate.locale.left'
    config.type = 'int'
    config.default = None
    config.description = u'Locale id for selected secondary locale in translate interface.'
    configs.append(config)

    return configs
