# Copyright (c) 2010 BaseSoft B.V.
# See LICENSE for details.

from django.utils.translation import ugettext as _
from sqlalchemy import and_

from basesoft.web.core.util.controllers import AbstractController, transaction_decorator
from basesoft.web.core.exceptions import PermissionDeniedException

from basesoft.web.contrib.license.controllers import LicenseController
from basesoft.web.contrib.license.models import Group
from basesoft.web.contrib.notification.models import Notification, NotificationGroup, NotificationAccount

class NotificationController(AbstractController):

    _instanceClass = Notification

    def __init__(self, sa_session=None, locale=None, account_id=None, request=None):
        AbstractController.__init__(self, sa_session, locale, account_id, request)
        self.license_controller = LicenseController(self.sa_session, None, self.account_id)

    def get_notifications(self, id=None):
        self.assert_permissions('view_notification')
        return self.get_privileged_object_by_id(Notification, id, Notification.PRIV_VIEW)

    def get_unread_notifications_count(self):
        """
        Return the number of unread notifications
        """

        filter, fo = self.get_query_filter_on_privileges(Notification, NotificationGroup, Notification.PRIV_VIEW)
        total_count = self.sa_session.query(Notification).select_from(fo).filter(filter).count()

        filter = and_(filter, NotificationAccount.account_id==self.account_id)

        fo = fo.join(NotificationAccount, Notification.id==NotificationAccount.notification_id)
        read_count = self.sa_session.query(Notification).select_from(fo).filter(filter).count()

        return total_count - read_count

    def load_notification(self, id, mark_as_read=False):
        """
        Loads a notification for the user.
        """
        n = self.get_notifications(id)[0]
        if mark_as_read:
            self.mark_as_read(n)
        sender = _("System")
        # where can user reply? if he can...
        reply = None
        if n.sender is not None:
            group = self.license_controller.get_groups(n.sender.default_group.id)
            if len(group) == 1:
                reply = group[0].id
            sender = n.sender.name
            if n.sender.person is not None and n.sender.person.full_name is not None:
                sender = n.sender.person.full_name
        return {
            'id': n.id,
            'subject': n.subject,
            'text': n.text,
            'sender': sender,
            'date': n.created,
            'reply_to': reply
        }

    @transaction_decorator
    def mark_as_read(self, notification):
        """
        Marks a notification as read by adding an entry to NotificationAccount
        """

        na = self.sa_session.query(NotificationAccount).filter(and_(NotificationAccount.notification==notification,\
                                                              NotificationAccount.account_id==self.account_id)).first()
        if na is None:
            na = NotificationAccount()
            na.notification = notification
            na.account = self.get_account()
            self.sa_session.add(na)
        return na

    @transaction_decorator
    def mark_as_archived(self, notification):
        """
        Marks a notification as archived for current user
        """
        if not isinstance(notification, list):
            notification = [notification]
        if not isinstance(notification[0], Notification):
            # got ids
            notification = self.get_notifications(notification)

        for n in notification:
            # also unread notifications can be marked as archived
            na = self.mark_as_read(n)
            na.archived = True
            self.sa_session.add(na)

    def get_available_recipients_groups(self):
        """
        Method returns all recipients the user can send notifications to depending on its permissions.
        """
        self.assert_permissions('create_notification')

        group = self.get_account().default_group
        #get all groups this account has access to
        try:
            groups = self.license_controller.get_groups()
        except PermissionDeniedException:
            groups = [group]

        ids = [g.id for g in groups]
        if self.has_permissions('send_to_license_default_group') and group.license.default_group.id not in ids:
            groups.append(group.license.default_group)
        if self.has_permissions('send_to_domain_default_group') and group.license.domain.default_group.id not in ids:
            groups.append(group.license.domain.default_group)

        ret = []
        for g in groups:
            d = {'id': g.id, 'for_domain': False, 'for_license': False}
            if self.has_permissions("view_all_domains") or self.has_permissions("view_domain"):
                d['name'] = g.license.domain.name + ' - ' + g.license.name + ' - ' + g.name
                d['for_domain'] = True
                d['for_license'] = True
            elif self.has_permissions('view_license'):
                d['name'] = g.license.name + ' - ' + g.name
                d['for_license'] = True
            else:
                d['name'] = g.name
            ret.append(d)
        return ret

    def get_all_recipients(self):
        """
        Returns a list with everybody the user can send a notification too. Unlike get_available_recipients_groups,
        this method just returns one group and the correspondent values for for_license and for_domain in case applicable
        instead of all groups in that license/domain.
        """

        try:
            domains = self.license_controller.get_domains()
            return [{'group': [d.default_group for d in domains],
                     'for_group': False,
                     'for_license': False,
                     'for_domain': True
                     }]
        except PermissionDeniedException:
            group = self.get_account().default_group
            if self.has_permissions('view_license'):
                ret = [{'group': [group.license.default_group],
                        'for_group': False,
                        'for_license': True,
                        'for_domain': False
                        }]
                if self.has_permissions('send_to_domain_default_group'):
                    ret.append({'group': [group.license.domain.default_group],
                                'for_group': True,
                                'for_license': False,
                                'for_domain': False
                                })
            ret = [{'group': [group],
                    'for_group': True,
                    'for_license': False,
                    'for_domain': False
                    }]
            defg = []
            if self.has_permissions('send_to_license_default_group'):
                defg.append(group.license.default_group)
            if self.has_permissions('send_to_domain_default_group'):
                defg.append(group.license.domain.default_group)
            if len(defg) > 0:
                ret.append({'group': defg,
                            'for_group': True,
                            'for_license': False,
                            'for_domain': False
                            })
            return ret

    def set_notification(self, subject, text):
        notification = self._instanceClass()
        notification.subject = subject
        notification.text = text
        return notification

    @transaction_decorator
    def create_system_notification(self, notification, recipient_groups, for_group=True, for_license=False, for_domain=False):
        """
        Creates a system notification. Recipient should always be a Group (or group id) and set correspondent values for for_group,
        for_license or for_domain in case notification should be for the group, group's whole license or for its whole domain.
        When a notification should be sent to multiple groups/licenses/domains recipient_groups would be a list.

        No permissions checks are done since this is a system notification.
        """
        assert isinstance(notification, self._instanceClass)

        if not isinstance(recipient_groups, list):
            recipient_groups = [recipient_groups]

        if not isinstance(recipient_groups[0], Group):
            rgs = self.sa_session.query(Group).filter(Group.id.in_(recipient_groups)).all()
            assert len(rgs) == len(recipient_groups)
            recipient_groups = rgs

        for rg in recipient_groups:
            assert isinstance(rg, Group)
            notification = self.save_view_privileges(notification, rg, for_group, for_license, for_domain);
        self.sa_session.add(notification)
        self.sa_session.flush()
        return notification

    @transaction_decorator
    def create_notification(self, notification, recipient_groups, for_group=True, for_license=False, for_domain=False):
        """
        Creates a notification with current account as sender. All correspondent rights will be checked.
        """
        self.assert_permissions('create_notification')
        assert isinstance(notification, self._instanceClass)
        notification.sender = self.get_account()

        if not isinstance(recipient_groups, list):
            recipient_groups = [recipient_groups]

        for rg in recipient_groups:
            g = self.has_recipient_permissions(rg, for_license, for_domain)
            if g:
                notification = self.save_view_privileges(notification, g, for_group, for_license, for_domain);
        self.sa_session.add(notification)
        self.sa_session.flush()

        self.logger.info('Created notification for recipient_groups=%s' % recipient_groups)

        return notification

    def has_recipient_permissions(self, recipient, for_license, for_domain):
        """
        Checks if current user (sender) has permissions to send notifications to provided recipient.

        The following apply for sender:
         * send_to_license_default_group - can send to default group of its license
         * send_to_domain_default_group - can send to default group of its domain
         * view_all_domains - can send to everybody
         * view_domain - can send to everybody in its own domain
         * view_license - can send to everybody in its own license
         * else, can send to everybody in its own group
        """
        if isinstance(recipient, Group):
            recipient_id = recipient.id
        else:
            recipient_id = int(recipient)

        account = self.get_account()
        domain = account.default_group.license.domain
        license = account.default_group.license
        group = account.default_group

        if for_domain:
            self.assert_any_permissions('view_all_domains', 'view_domain')
        if for_license:
            self.assert_any_permissions('view_all_domains', 'view_domain', 'view_license')

        if recipient_id == group.id:
            return group

        # first check for default groups and correspondent permissions
        if (recipient_id == license.default_group.id and self.has_permissions('send_to_license_default_group')) or\
           (recipient_id == domain.default_group.id and self.has_permissions('send_to_domain_default_group')):
            return self.sa_session.query(Group).filter(Group.id==recipient_id)

        # let license controller take care of all next checks
        rgs = self.license_controller.get_groups(recipient_id)
        if len(rgs) != 1:
            raise PermissionDeniedException('Insufficient permissions')

        return rgs[0]

    def save_view_privileges(self, notification, group, for_group, for_license, for_domain):

        # lets always save for default group
        if for_license:
            group = group.license.default_group
        if for_domain:
            group = group.license.domain.default_group

        # this will prevent saving privileges for same license or domain
        go = self.sa_session.query(NotificationGroup).filter(and_(NotificationGroup.notification==notification,
                                                             NotificationGroup.group==group,
                                                             NotificationGroup.privilege==notification.PRIV_VIEW
                                                             )).first()
        if go is None:
            go = NotificationGroup()
            go.group = group
            go.privilege = notification.PRIV_VIEW
        go.group_has_privilege = for_group
        go.license_has_privilege = for_license
        go.domain_has_privilege = for_domain
        self.sa_session.add(go)
        notification.groups.append(go)

        return notification
