# Copyright (c) 2010 BaseSoft B.V.
# See LICENSE for details.

import datetime

from sqlalchemy.orm import mapper, relation
from sqlalchemy import Column, Integer, String, ForeignKey, DateTime, UnicodeText, SmallInteger
from basesoft.sqlalchemy.table import IQTable

from basesoft.web.contrib.license.hooks import PrivilegeObjectMapper
from basesoft.web.contrib.notification import models as notification_models
from basesoft.web.contrib.license import models as license_models
from basesoft.web.contrib.account import models as account_models
from django.utils.translation import ugettext as _

__mapping_defined = False
__tables_defined = False
tables = {}

def reset():
    global __mapping_defined
    global __tables_defined

    __mapping_defined = False
    __tables_defined = False

def get_tables():
    """
    """
    return tables

def do_tables(metadata):
    global __tables_defined
    if not __tables_defined:

        tblNotification = IQTable('tblNotification', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('polymorphic_type', String(255), nullable=False),
            Column('subject', String(255), nullable=False),
            Column('text', UnicodeText),
            Column('sender_id', Integer, ForeignKey('tblAccount.id', ondelete='CASCADE')),

            Column('deleted', SmallInteger, nullable=False, default=0),
            Column('archived', SmallInteger, nullable=False, default=0),

            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        tblNotificationGroup = IQTable('tblNotificationGroup', metadata,
            Column('notification_id', Integer, ForeignKey('tblNotification.id', ondelete="CASCADE"), primary_key=True),
            Column('group_id', Integer, ForeignKey('tblGroup.id'), primary_key=True),
            Column('privilege', Integer, nullable=False, primary_key=True, autoincrement=False),
            Column('group_has_privilege', SmallInteger, default=0),
            Column('license_has_privilege', SmallInteger, default=0),
            Column('domain_has_privilege', SmallInteger, default=0),
            Column('world_has_privilege', SmallInteger, default=0),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        tblNotificationAccount = IQTable('tblNotificationAccount', metadata,
            Column('notification_id', Integer, ForeignKey('tblNotification.id', ondelete='CASCADE'), primary_key=True),
            Column('account_id', Integer, ForeignKey('tblAccount.id', ondelete='CASCADE'), primary_key=True),
            Column('archived', SmallInteger, nullable=False, default=0),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        tables['tblNotification'] = tblNotification
        tables['tblNotificationGroup'] = tblNotificationGroup
        tables['tblNotificationAccount'] = tblNotificationAccount

        __tables_defined = True

def do_mappings():
    """
    This function is only called by the django middleware
    when it is needed
    """
    global __mapping_defined
    if not __mapping_defined:

        mapper(notification_models.Notification, tables['tblNotification'],
            extension = PrivilegeObjectMapper(),
            properties = {
                'groups': relation(notification_models.NotificationGroup, backref='notification', cascade="all", passive_deletes=True),
                'sender': relation(account_models.Account)
            },
            polymorphic_on=tables['tblNotification'].c.polymorphic_type,
            polymorphic_identity='Notification'
        )

        mapper(notification_models.NotificationGroup, tables['tblNotificationGroup'], properties = {
            'group': relation(license_models.Group)
        })

        mapper(notification_models.NotificationAccount, tables['tblNotificationAccount'], properties = {
            'notification': relation(notification_models.Notification),
            'account': relation(account_models.Account)
        })

        __mapping_defined = True

def fill_data(conn):
    pass

def app_permissions_category():
    """
    Returns a category for all permissions defined for this app
    """
    return license_models.PermissionCategory(u'Notification')

def app_permissions():
    """
    The permissions used within this application. Can be sync'd with the
    `iqsyncpermissions` command.
    """
    return [
        license_models.Permission(u'create_notification', u"""
            Gives the permission to create (send) notifications.
            """, _("Create notification")),
        license_models.Permission(u'view_notification', u"""
            Gives the permission to view notifications.
            """, _("View notification")),
        # update_notification will be mostly used for archiving unused notifications
        license_models.Permission(u'update_notification', u"""
            Gives the permission to update notifications.
            """, _("Update notifications")),
        license_models.Permission(u'archive_notification', u"""
            Gives the permission to archive notifications.
            """, _("Archive notifications")),
        license_models.Permission(u'delete_notification', u"""
            Gives the permission to delete notifications.
            """, _("Delete notifications")),

        license_models.Permission(u'send_to_license_default_group', u"""
            Gives the permission to send a notification to the default group of the license.
            Only available with create_notification permission.
            """, _("Send notifications to license default group")),
        license_models.Permission(u'send_to_domain_default_group', u"""
            Gives the permission to send a notification to the default group of the domain.
            Only available with create_notification permission.
            """, _("Send notifications to domain default group")),
    ]
