'''
Created on Oct 2, 2013

@author: marten
'''
import re
import hashlib

from decimal import Decimal
import datetime

from dateutil.relativedelta import relativedelta

from django.utils.translation import ugettext as _

from sqlalchemy.orm.exc import NoResultFound
from sqlalchemy.sql.expression import and_, func

from basesoft.web.core.util.controllers import AbstractController, transaction_decorator, LicenseConfigController, get_models_of_type
from basesoft.web.core.exceptions import SystemException, PublicSecurityException, PublicUserException
from basesoft.web.contrib.order.models import Order, OrderLine, TaxClass,\
    TaxClassCountry, TaxClassCountryRate, InvoiceFile, NewInvoiceMail, GLAccount,\
    Currency, PaymentMethod, OrderPaymentStatus, PaymentMethodOgone
from basesoft.web.contrib.contact.controllers import SimpleContactController
from basesoft.web.contrib.i18n.models import Country
from basesoft.util.virtual_date import today, now
from basesoft.web.contrib.file.controllers import AnonymousFileController
from basesoft.web.contrib.license.controllers import LicenseController

from django.conf import settings
from sqlalchemy.exc import IntegrityError
from basesoft.web.contrib.license.models import License
from sqlalchemy.orm.session import make_transient
from iqportal.iqube.core.controllers import IQubeAggregator


class TaxController(AbstractController):

    def get_tax_classes(self):
        return self.get_privileged_object_by_id(TaxClass, id=None, order_by=TaxClass.name)

    @transaction_decorator
    def save_tax_class_state(self, tax_class_id, state):

        self.assert_any_permissions('manage_tax_classes')

        if tax_class_id is None:

            # check for code already existing, then error
            tc = self._get_privileged_object(TaxClass, TaxClass.code == state['code'], TaxClass.PRIV_VIEW)
            if len(tc) > 0:
                raise PublicUserException(_('Tax class already exists: %s' % state['code']))

            tc = TaxClass()

        else:
            tc = self.sa_session.query(TaxClass) \
                .filter(TaxClass.id == tax_class_id) \
                .one()

        tc.code = state['code']
        tc.name = state['name']

        self.sa_session.add(tc)

        # when first time saving, grant domain view access
        if tax_class_id is None:
            self.grant_privilege_to(tc, (TaxClass.PRIV_VIEW, ), check_own_privs=False, domain=True)

        return tc

    def get_tax_class_state(self, id):

        tax_class = self.get_privileged_object_by_id(TaxClass, id)[0]
        return {
            'tax_class_id': tax_class.id,
            'name': tax_class.name,
            'code': tax_class.code,
        }

    def get_tax_class_country(self, tax_class_country_id):
        tcc = self.sa_session.query(TaxClassCountry).filter(TaxClassCountry.id == tax_class_country_id).one()
        self.assert_privileges_on_object(tcc.tax_class, TaxClass.PRIV_VIEW)
        return tcc

    def get_tax_class_country_state(self, tax_class_country_id):
        tcc = self.get_tax_class_country(tax_class_country_id)

        rates = []
        for rate in tcc.rates:
            rates.append({
                'start_date': rate.start_date,
                'rate': rate.rate,
            })

        return {
            'tax_class_country_id': tcc.id,
            'country_id': tcc.country.id,
            'country_name': tcc.country.name,
            'code': tcc.tax_class.code,
            'tax_class_id': tcc.tax_class.id,
            'tax_rates': rates,
        }


    @transaction_decorator
    def save_country_tax_state(self, tax_class_country_id, state):

        self.assert_any_permissions('manage_country_tax')

        self.logger.info('save_country_tax_state: %s' % (state))

        country = self.sa_session.query(Country).filter(Country.id == state['country_id']).one()
        tax_class = self.sa_session.query(TaxClass).filter(TaxClass.id == state['tax_class_id']).one()

        # check if unique
        sls = self.sa_session.query(TaxClassCountry) \
            .filter(TaxClassCountry.id != tax_class_country_id, TaxClassCountry.country_id == state['country_id'], TaxClassCountry.tax_class_id == state['tax_class_id']) \
            .all()
        if len(sls) > 0:
            raise PublicUserException(_('This Country / Tax class combination already exists'))

        if tax_class_country_id is None:
            tcc = TaxClassCountry()
            tcc.country = country
            tcc.tax_class = tax_class
        else:
            tcc = self.sa_session.query(TaxClassCountry) \
                .filter(TaxClassCountry.id == tax_class_country_id) \
                .one()

        self.sa_session.add(tcc)

        # allow today on first save, else must be tomorrow
        min_start_date = today() if tcc.id is None else today() + relativedelta(days=1)

        for i in range(len(state['tax_rates'])):

            line = state['tax_rates'][i]

            assert isinstance(line['start_date'], (datetime.date, datetime.datetime))
            assert isinstance(line['rate'], Decimal)

            # js sends datetimes, convert to date
            if isinstance(line['start_date'], datetime.datetime):
                line['start_date'] = line['start_date'].date()

            try:
                db_rate = tcc.rates[i]
            except IndexError:
                db_rate = TaxClassCountryRate()
            db_rate.tax_class_country = tcc

            # check if the user wants to change the fee
            if db_rate.start_date != line['start_date'] or db_rate.rate != line['rate']:

                # error if the user tries to change an existing fee from the past
                if db_rate.start_date is not None and db_rate.start_date <= today():
                    raise PublicUserException(_('Cannot change fee in the past: %s') %  db_rate.start_date)

                if tax_class_country_id is not None and line['start_date'] < min_start_date:
                    raise PublicUserException(_('Cannot set start date in the past'))

                db_rate.start_date = line['start_date']
                db_rate.rate = line['rate']

                self.logger.info('updated country-tax-rate country-tax=%s, fee=%s, start_date: %s' % (tcc.id, db_rate.rate, db_rate.start_date))
                self.sa_session.add(db_rate)

        # remove rates that have been removed
        for i in range( len(state['tax_rates']), len(tcc.rates) ):
            db_rate = tcc.contribution_fees[i]

            if db_rate.start_date <= today():
                raise PublicUserException(_('Cannot remove rate from the past: %s') %  db_rate.start_date)

            self.logger.info('removed country-tax-rate country-tax=%s, fee=%s, start_date: %s' % (tcc.id, db_rate.rate, db_rate.start_date))
            self.sa_session.delete(db_rate)


        return tcc

    @transaction_decorator
    def delete_country_taxes(self, id):

        self.assert_any_permissions('manage_country_tax')

        sl = self.get_tax_class_country(id)

        # rely on the database to check dependencies
        self.sa_session.delete(sl)

        self.logger.info('removed country-tax: %s' % id)

    def get_current_rate(self, country, tax_class_code, group=None):

        if group is None:
            group = self.get_default_group()

        # get tax_class for current domain, should be only one
        #tc = self._get_privileged_object(TaxClass, TaxClass.code == tax_class_code, TaxClass.PRIV_VIEW)
        tc = self.get_object_by_priv_in_hierarchy(group, TaxClass, TaxClass.PRIV_VIEW, TaxClass.code == tax_class_code)

        if tc is None:
            raise PublicUserException(_('No Tax class defined for code "%s"') % tax_class_code)

        try:
            tcc = self.sa_session.query(TaxClassCountry).filter(TaxClassCountry.tax_class_id == tc.id, TaxClassCountry.country_id == country.id).one()
        except NoResultFound:
            raise PublicUserException(_('No tax rate set for taxclass %(tax_class_code)s and country %(country_name)s' % ({'tax_class_code': tax_class_code, 'country_name': country.name} ) ))

        return tcc.get_current_rate()



class GLAccountController(AbstractController):

    def get_gl_licenses(self, license_id=None):
        """
        returns a list of licenses to which a GL account can be attached
        """

        rv = []
        controller = self.get_default_group().license.controller(self.sa_session, account_id=self.get_account().id, request=self.request)

        for license_ in controller.get_domain_licenses():
            if license_id is None or license_.id == license_id:
                rv.append(license_)
        return rv

    @transaction_decorator
    def save_gl_account_state(self, gl_account_id, state):

        self.assert_any_permissions('manage_gl_accounts')

        if gl_account_id is None:
            ga = GLAccount()
        else:
            ga = self.sa_session.query(GLAccount) \
                .filter(GLAccount.id == gl_account_id) \
                .one()

        merchant = self.get_gl_licenses(int(state['merchant_id']))[0]

        ga.merchant = merchant
        ga.code = state['code']
        ga.account_number = state['account_number']
        self.sa_session.add(ga)

        try:
            self.sa_session.flush()
        except IntegrityError:
            raise PublicUserException(_('The GL Account already exists for this merchant/code'))


        self.logger.info('updated gl account: %s, merchant=%s, code=%s, account_number=%s' % (gl_account_id, merchant, state['code'], state['account_number']) )

        return ga

    def get_gl_account_state(self, ga_id):

        self.assert_any_permissions('manage_gl_accounts')
        ga = self.sa_session.query(GLAccount).filter(GLAccount.id == ga_id).one()

        assert ga.merchant in self.get_gl_licenses()

        state = {
            'gl_account_id': ga.id,
            'merchant_id': ga.merchant.id,
            'code': ga.code,
            'account_number': ga.account_number,
        }

        return state

    @transaction_decorator
    def delete_country_taxes(self, id):

        self.assert_any_permissions('manage_gl_accounts')

        ga = self.get_privileged_object_by_id(GLAccount, id)[0]

        # rely on the database to check dependencies
        self.sa_session.delete(ga)

        self.logger.info('removed gl account: %s' % id)

def get_available_order_types(pc):
    models = get_models_of_type(Order, exclude_parent=True)
    wp = []
    for model in models:
        m = model['model']
        if m.any_permissions:
            if pc.has_any_permissions(*m.permissions):
                wp.append(m)
        else:
            if pc.has_permissions(*m.permissions):
                wp.append(m)
    return wp

class AbstractOrderController(AbstractController):
    """
    Abstract Order controller which needs to be extended by the individual controller.

    The extension classes should do the correct permission checks for saving and updating.
    """
    _instance_class = None
    _additional_order_fields = None
    _additional_order_line_fields = None

    def __init__(self, *args, **kwargs):
        super(AbstractOrderController, self).__init__(*args, **kwargs)
        self.set_additional_fields()

    def set_additional_fields(self):
        """
        Override to set the allowed additional fields for this order and order line
        """
        self._set_additional_fields({}, {})

    def _get_controller(self, _id):
        o = self.get_privileged_object_by_id(Order, id=_id).pop()
        return o.get_order_controller()(self.sa_session, account_id=self.get_account().id, request=self.request)

    def _set_additional_fields(self, order_fields, order_line_fields):
        assert type(order_fields) == dict
        assert type(order_line_fields) == dict

        self._additional_order_fields = order_fields
        self._additional_order_line_fields = order_line_fields

    def get(self, _id=None, priv=Order.PRIV_VIEW):
        assert self._instance_class is not None
        return self.get_privileged_object_by_id(self._instance_class, id=_id, priv=priv)

    def get_order(self, order_id, priv=Order.PRIV_VIEW):
        assert order_id is not None
        oc = self._get_controller(order_id)
        return oc.get(order_id, priv=priv)


    def get_open_orders(self, priv=Order.PRIV_VIEW):
        """
        Returns new and pending orders (both are not ready for export yet).
        """

        _where = self._instance_class.status.in_([self._instance_class.STATUS_NEW, self._instance_class.STATUS_PENDING])
        _where = and_(_where, self._instance_class.type!=self._instance_class.TYPE_CREDIT)

        return self._get_privileged_object(self._instance_class, _where, priv=priv)

    def load_order_state(self, _id):
        oc = self._get_controller(_id)
        return oc.load_state(_id)

    def load_state(self, _id):
        self.assert_permissions(*self._instance_class.permissions)

        if isinstance(_id, self._instance_class):
            order = _id
        else:
            order = self.get(_id).pop()

        scc = SimpleContactController(self.sa_session, account_id=self.get_account().id)

        rv = {
            'id': order.id,
            'client': scc.load_company_state(order.client.company),
            'merchant': scc.load_company_state(order.merchant.company),
            'lines': self._load_lines_state(order.lines),
            'total_amount_excluding_vat': order.total_amount_excluding_vat,
            'total_vat_amount': order.total_amount_including_vat-order.total_amount_excluding_vat,
            'total_amount_including_vat': order.total_amount_including_vat,
            'invoiced_amount': order.invoiced_amount,
            'status': order.status,
            'amount_calculation_field': None,
            'type': order.type,
            'currency_code': order.currency.code,
            'currency_symbol': order.currency.symbol,
            'display_date': order.display_date,
        }

        return rv

    def _load_state_for_export(self, order):

        payment_method_id = None
        payment_method_label = None
        payment_status = None
        payment_date = None

        # try to get last payment_status and set vars
        try:
            ps = self.sa_session.query(OrderPaymentStatus) \
                .filter(OrderPaymentStatus.order==order) \
                .order_by(OrderPaymentStatus.last_updated.desc()).first()
            if ps is not None:
                payment_method_id = ps.payment_method.id
                payment_method_label = ps.payment_method.label
                payment_status = order.get_current_payment_status()
                payment_date = ps.payment_date

        except NoResultFound:
            pass


        rv = {
            'id': order.id,
            'client_id': order.client.id,
            'merchant_id': order.merchant.id,
            'lines': self._load_lines_state(order.lines, True),
            'currency_code': order.currency.code,
            'total_amount_excluding_vat': order.total_amount_excluding_vat,
            'total_vat_amount': order.total_amount_including_vat-order.total_amount_excluding_vat,
            'total_amount_including_vat': order.total_amount_including_vat,
            'invoiced_amount': order.invoiced_amount,
            'current_payment_method_id': payment_method_id,
            'current_payment_method_label': payment_method_label,
            'current_payment_status': payment_status,
            'current_payment_date': payment_date,
            'created': order.created,
            'last_updated': order.last_updated,
            'export_date': order.export_date,
            'type': order.type,
            'polymorphic_type': order.polymorphic_type,
            'properties': [],
        }

        return rv

    def _load_lines_state(self, lines, for_export=False):
        rv = []
        for line in lines:
            rv.append({
                'index': line.index,
                'polymorphic_type': line.polymorphic_type,
                'description': line.description,
                'price_per_unit': line.price_per_unit,
                'quantity': line.quantity,
                'vat_rate': line.vat_rate,
                'gl_account': line.gl_account_number,
                'line_amount_excluding_vat': line.line_amount_excluding_vat,
                'line_amount_including_vat': line.line_amount_including_vat,
                'properties': [],
            })
        return rv

    def _set_order_from_state(self):
        pass

    def _assert_ready_for_approval(self, order):
        pass

    def _assert_valid_line_removal(self, order, line):
        if order.status in (Order.STATUS_SUBMITTED, Order.STATUS_CREDITED, ):
            raise PublicUserException('Cannot remove an order line from an order which is already submitted or credited.')

    @transaction_decorator
    def submit_order(self, _id):

        if isinstance(_id, self._instance_class):
            order = _id
        else:
            oc = self._get_controller(_id)
            orders = oc.get(_id, priv=self._instance_class.PRIV_SUBMIT)
            if len(orders) < 1:
                raise PublicSecurityException(_('Cannot approve Order #%s' % _id))
            order = orders.pop()

        self._assert_ready_for_approval(order)

        # if this is the approval of a resubmitted ORDER, we also submit the linked credit.
        if order.parent_order is not None and order.parent_order.type==self._instance_class.TYPE_CREDIT:
            if order.parent_order.status in [self._instance_class.STATUS_NEW, self._instance_class.STATUS_PENDING]:
                order.parent_order.status = self._instance_class.STATUS_SUBMITTED

        order.status = self._instance_class.STATUS_SUBMITTED
        if self.get_account() is None:
            order.auto_submitted = self._instance_class.SUBMIT_DONE_BY_SYSTEM
        else:
            order.auto_submitted = self._instance_class.SUBMIT_DONE_BY_USER

        self.sa_session.add(order)

        self.logger.info('submit_order: order [%s] submitted' % order.id)

        return order

    def submit_payment(self, order_id, payment_method_id):

        if payment_method_id is None: # we can submit the payment without a method if total amount is 0
            c = self._get_controller(order_id)
            order = c.get(order_id, Order.PRIV_PAY).pop()
            if order.total_amount_including_vat != 0:
                raise PublicUserException(_('Cannot submit order without having a payment method selected.'))
            return c.submit_order(order)

        pc = PaymentController(self.sa_session, account_id=self.account_id, request=self.request)
        payment_method = pc.get_payment_methods(payment_method_id=payment_method_id)[0]
        pc = pc.get_payment_controller(payment_method_id)(self.sa_session, account_id=self.account_id, request=self.request, payment_method=payment_method)

        self.logger.info('submit_payment: order [%s] payment marked as submitted' % order_id)

        return pc.submit_payment(order_id, _('Payment marked as submitted by user'))

    @transaction_decorator
    def _create_child_order(self, order, is_credit=False, exclude_line_type=None):
        assert isinstance(order, self._instance_class)

        kwargs = {}
        parent_order = order
        lines = []
        if self._additional_order_fields.has_key(self._instance_class):
            for field in self._additional_order_fields[self._instance_class]:
                if not hasattr(order, field):
                    continue
                kwargs[field] = getattr(order, field)

        for line in order.lines:
            if exclude_line_type is not None and isinstance(line, exclude_line_type):
                continue
            quantity = 0 # TODO: None?
            if is_credit:
                quantity = 0-line.quantity
            newline = {
                'description': line.description,
                'quantity': quantity,
                'price': line.price_per_unit,
                'polymorphic_type': line.__class__
            }

            if self._additional_order_line_fields.has_key(line.__class__):
                for field in self._additional_order_line_fields[line.__class__]:
                    if not hasattr(line, field):
                        continue
                    newline[field] = getattr(line, field)

            lines.append(newline)

        order = self._save_or_update_order(None, lines, order.merchant, order.client.default_group, parent_order, **kwargs)
        if is_credit:
            order.type = self._instance_class.TYPE_CREDIT
        return order

    @transaction_decorator
    def _credit_order(self, order, submit=False):
        credit = self._create_child_order(order, True)
        order.status = self._instance_class.STATUS_CREDITED

        # this should be supplied if the Merchant credits an order (the goods are returned or something)
        if submit:
            credit.status = self._instance_class.STATUS_SUBMITTED
        else:
            credit.status = self._instance_class.STATUS_PENDING

        self.sa_session.add(credit)
        return credit

    def has_different_quantities(self, order, quantities):
        """
        This can be used to determine whether or not a Credit should be created.
        """
        if len(order.lines) != len(quantities.values()): # different sizes means different quantities.
            return True

        for line in order.lines:
            q = int(quantities['line_%s' % line.index])
            if line.quantity != q:
                return True

        return False

    @transaction_decorator
    def _save_or_update_order(self, _id, lines, merchant=None, scope=None, parent_order=None, display_date=None, **kwargs):
        """
        This assumes the right permission checks have already been made by the caller method.
        """
        assert len(lines) > 0

        if scope is None:
            scope = self.get_default_group()

        if merchant is None:
            merchant = scope.license.domain.reseller

        if parent_order is not None:
            assert isinstance(parent_order, self._instance_class) # we can only have a order trail of the same order types.

        if len(kwargs) > 0:
            if self._additional_order_fields.has_key(self._instance_class):
                assert len([k for k in kwargs.keys() if k in self._additional_order_fields[self._instance_class]]) == len(kwargs)
            else:
                raise SystemException('Incorrect key-worded arguments supplied for order %s' % self._instance_class)

        client = scope.license

        if _id is None:
            order = self._instance_class()
            order.merchant = merchant
            order.client = scope.license
            # only save a parent order on first insert!
            order.parent_order = parent_order

        else:
            order = self.sa_session.query(self._instance_class).filter(self._instance_class.id==_id).one()
            assert order.merchant == merchant
            assert order.client == scope.license

        # display date
        order.display_date = display_date

        for key, value in kwargs.iteritems():
            setattr(order, key, value)

        # currency
        try:
            country = merchant.company.addresses[0].address.country
        except (IndexError, AttributeError):
            raise PublicUserException(_('Cannot get country from merchant %s: no address/country set') % merchant.name)
        try:
            client_country = client.company.addresses[0].address.country
        except (IndexError, AttributeError):
            raise PublicUserException(_('Cannot get country from client %s: no address/country set') % client.name)

        currency_id = self.sa_session.query(func.iqGetCurrencyIdForCountry(country.id, today() )).one()[0]
        currency = self.sa_session.query(Currency).filter(Currency.id == currency_id).one()
        order.currency = currency

        # pad the lines
        while len(lines) < len(order.lines):
            line = order.lines.pop()
            self.sa_session.delete(line)
            self.sa_session.flush()

        # set lines.
        index = 0
        for line in lines:
            line['apply_vat'] = client_country == country
            line['order'] = order
            line['index'] = index
            line['merchant'] = merchant
            self._save_or_update_order_line(**line)
            index = index + 1

        order.total_amount_excluding_vat = sum([l.line_amount_excluding_vat for l in order.lines])
        order.total_amount_including_vat = sum([l.line_amount_including_vat for l in order.lines])
        order.invoiced_amount = self._get_invoiced_amount(order)

        if _id is None:
            self.save_default_privilege_object_group(order, target_group=scope)
            if self._instance_class.auto_submit:
                self.submit_order(order)

        self.sa_session.add(order)

        return order

    def _get_invoiced_amount(self, order):
        lcc = LicenseConfigController(self.sa_session, scope=order.merchant)
        min_amount = lcc.get('basesoft.web.contrib.order.minimum_invoiceable_amount')
        if min_amount == 0:
            return order.total_amount_including_vat

        # negative or positive amount which is smaller than threshold is never invoiced
        if abs(order.total_amount_excluding_vat) < min_amount: # check on EXCLUDING VAT
            return 0

        return order.total_amount_including_vat

    def _get_vat_rate(self, apply_vat, polymorphic_type, merchant, price):
        tax_rate = None
        vat_fee = 0

        if not apply_vat or polymorphic_type.tax_class_code is None:
            tax_rate = 0
            vat_fee = 0
        else:

            try:
                country = merchant.company.addresses[0].address.country
            except (IndexError, AttributeError):
                raise PublicUserException(_('Cannot get country from merchant %s: no address/country set') % merchant.name)

            tax_controller = TaxController(self.sa_session, request=self.request)
            tax_rate = tax_controller.get_current_rate(country, polymorphic_type.tax_class_code, merchant.default_group)

            if tax_rate is None:
                raise PublicUserException(_('No current tax rate set for %(country_name)s/%(tax_class_code)s') % ({'country_name': country.name, 'tax_class_code': polymorphic_type.tax_class_code}) )
            vat_fee = (tax_rate / 100) * price
        return tax_rate, vat_fee

    def _save_or_update_order_line(self, order, index, merchant, description, quantity, price, polymorphic_type=OrderLine, apply_vat=True, **kwargs):

        # FIXME: this should be properly fixed, but sys.maxint on 64bit systems vs 32bit database is problem here.
        if int(quantity) > 2147483647 or int(quantity) < -2147483647:
            raise PublicUserException(_('Quantity out of range.'))

        if polymorphic_type.MAX_LINE_QUANTITY is not None and abs(int(quantity)) > polymorphic_type.MAX_LINE_QUANTITY:
            raise PublicUserException(_('Quantity should be smaller than or equal to %(max_quantity)s.' % {'max_quantity': polymorphic_type.MAX_LINE_QUANTITY}))

        if len(kwargs) > 0:
            if self._additional_order_line_fields.has_key(polymorphic_type):
                assert len([k for k in kwargs.keys() if k in self._additional_order_line_fields[polymorphic_type]]) == len(kwargs)
            else:
                raise SystemException('Incorrect key-worded arguments supplied for order line %s' % polymorphic_type)

        line = None
        if order.id is not None:
            try:
                line = self.sa_session.query(polymorphic_type).filter(and_(polymorphic_type.order==order, polymorphic_type.index==index)).one()
            except NoResultFound:
                line = self.sa_session.query(OrderLine).with_polymorphic('*').filter(and_(OrderLine.order==order, OrderLine.index==index)).first()
                if line is not None:
                    self.logger.warn('Found different polymorphic order line on index. Removing... %s' % line.id)
                    raise SystemException('Mixed line types is not yet supported.')
                line = None

        if line is None:
            line = polymorphic_type()
            line.index = index

        # retrieve the line's vat_rate based on the line object tax_class and merchants country
        tax_rate, vat_fee = self._get_vat_rate(apply_vat, polymorphic_type, merchant, price)

        assert tax_rate is not None
        assert vat_fee is not None

        # get the gl account number based on the orderline gl_account_code
        gl_account_number = None
        if polymorphic_type.gl_account_code is not None:

            try:
                gl_account = self.sa_session.query(GLAccount) \
                    .filter(GLAccount.code == polymorphic_type.gl_account_code,
                            GLAccount.merchant == merchant).one()
            except NoResultFound:
                raise PublicUserException(_('No GL Account available for merchant %(merchant)s and code %(code)s') % {'merchant': merchant.name, 'code': polymorphic_type.gl_account_code})

            gl_account_number = gl_account.account_number


        unit = self._get_line_unit(quantity, order, **kwargs)

        line.order = order
        line.description = description
        line.quantity = quantity
        line.price_per_unit = price
        line.vat_rate = tax_rate
        line.gl_account_number = gl_account_number

        line.line_amount_excluding_vat = unit*price
        line.line_amount_including_vat = line.line_amount_excluding_vat+(unit*vat_fee)

        for key, value in kwargs.iteritems():
            setattr(line, key, value)

        self.sa_session.add(line)

    def _get_line_unit(self, quantity, order, **kwargs):
        return quantity

    @transaction_decorator
    def remove_line(self, _id, line_id):
        if isinstance(_id, self._instance_class):
            order = _id
        else:
            oc = self._get_controller(_id)
            orders = oc.get(_id, priv=self._instance_class.PRIV_UPDATE)
            if len(orders) < 1:
                raise PublicSecurityException(_('Cannot update Order #%s' % _id))
            order = orders.pop()

        lines = [l for l in order.lines if l.id == line_id]
        keep = [l for l in order.lines if l.id != line_id]
        if len(lines) == 0:
            raise PublicUserException(_('Cannot find line within order.'))

        line = lines[0]
        self._assert_valid_line_removal(order, line)

        self.logger.warn('Removing line from order [%s]' % order.id)

        self.sa_session.delete(line)
        self.sa_session.expire(order)
        # we should now update the indexes of the remaining lines and the new total

        lines = self.sa_session.query(OrderLine).filter(OrderLine.order==order).order_by(OrderLine.index).all()

        index = 0
        for line in lines:
            line.index = index
            index += 1
            self.sa_session.add(line)

        order.total_amount_excluding_vat = sum([l.line_amount_excluding_vat for l in order.lines])
        order.total_amount_including_vat = sum([l.line_amount_including_vat for l in order.lines])
        order.invoiced_amount = self._get_invoiced_amount(order)
        self.sa_session.add(order)
        return order

    @transaction_decorator
    def attach_invoice_to_order(self, _id, file_name, data, is_credit, overwrite=False):
        oc = self._get_controller(_id)
        orders = oc.get(_id, Order.PRIV_ADD_INVOICE)
        if len(orders) == 0:
            raise PublicSecurityException(_('Order #%s not found.' % _id))

        order = orders.pop()
        # we can only attach invoices to previously exported orders.
        if order.export_date is None:
            raise PublicSecurityException(_('Unable to attach Invoice to unexported Order #%s.' % _id))

        if not overwrite and order.invoice is not None:
            raise PublicSecurityException(_('Order #%s already has an invoice/credit note attached.' % _id))

        if is_credit:
            if order.total_amount_excluding_vat > 0:
                raise PublicUserException(_('Trying to attach a Credit note to Order #%s which has a positive total amount.' % _id))
            category = 'Credit notes'
            props = {'is_credit': InvoiceFile.IS_CREDIT}
        else:
            if order.total_amount_excluding_vat < 0:
                raise PublicUserException(_('Trying to attach an Invoice note to Order #%s which has a negative total amount.' % _id))
            category = 'Invoice notes'
            props = {'is_credit': InvoiceFile.IS_INVOICE}

        afc = AnonymousFileController(self.sa_session)
        order.invoice = afc.save_file_for_target_group(data, file_name, category, order.client.default_group, InvoiceFile, props, privileges=[InvoiceFile.PRIV_VIEW])
        order.invoice_stage = self._instance_class.INVOICE_STAGE_NEW
        self.sa_session.add(order)

        self.logger.info('attach_invoice_to_order: attached invoice to order %s' % (order.id))

        return True

    def get_new_invoice_mail(self, order):
        group = order.merchant.default_group
        return self.get_object_by_priv_in_hierarchy(group, NewInvoiceMail, NewInvoiceMail.PRIV_USE_FOR_SENDING)

    def _get_all_controllers(self):
        rv = []

        for ot in get_models_of_type(Order, exclude_parent=True):
            if self.has_permissions(*ot['model'].permissions):
                rv.append(ot['model']().get_order_controller())

        return rv

    @transaction_decorator
    def get_unexported_orders_for_merchant(self, merchant_id, all_types=False):
        """
        This returns the state for all unexported Orders for the provided Merchant and marks them as exported.
        """
        if all_types:
            rv = []
            ctrls = self._get_all_controllers()
            for ctrl in ctrls:
                _instance = ctrl(self.sa_session, account_id=self.get_account().id)
                rv += _instance.get_unexported_orders_for_merchant(merchant_id, False)
            return rv

        self.assert_permissions(*self._instance_class.permissions)
        # SUBMITTED AND CREDITED Orders which were not previously exported and don't have 0 as invoiced_amount should be exported.
        where = and_(
            self._instance_class.status.in_([self._instance_class.STATUS_SUBMITTED, self._instance_class.STATUS_CREDITED]),
            self._instance_class.merchant_id==merchant_id,
            self._instance_class.export_date==None,
            self._instance_class.invoiced_amount!=0,
        )

        rv = []
        for order in self._get_privileged_object(self._instance_class, where, priv=Order.PRIV_EXPORT):
            order.export_date = today()
            rv.append(self._load_state_for_export(order))
            self.sa_session.add(order)

        return rv

    def get_exported_orders_by_id(self, _id):
        o = self.get_privileged_object_by_id(Order, id=_id)
        if len(o) == 0:
            raise PublicSecurityException(_('Order #%s not found.' % _id))
        ctrl = self._get_controller(_id)
        return ctrl.get_exported_instance_class_orders_by_id(_id)

    def get_exported_instance_class_orders_by_id(self, _id):
        """
        Returns a previously exported Order for the supplied ID.
        """
        self.assert_permissions(*self._instance_class.permissions)
        orders = self.get_privileged_object_by_id(self._instance_class, _id, priv=self._instance_class.PRIV_EXPORT)
        if len(orders) == 0:
            raise PublicSecurityException(_('Order #%s not found.' % _id))

        order = orders.pop()
        # we can only re-export previously exported orders.
        if order.export_date is None:
            raise PublicSecurityException(_('Order #%s is not available for export.' % _id))

        return self._load_state_for_export(order)

    def get_exported_orders_by_ids(self, order_ids):
        assert isinstance(order_ids, list)
        rv = []
        for order_id in order_ids:
            rv.append(self.get_exported_orders_by_id(order_id))
        return rv

    def get_orders_by_export_date(self, merchant_id, export_date, all_types=False):
        """
        This returns all the orders that were exported on the supplied date for the supplied merchant.
        """
        if all_types:
            rv = []
            ctrls = self._get_all_controllers()
            for ctrl in ctrls:
                _instance = ctrl(self.sa_session, account_id=self.get_account().id)
                rv += _instance.get_orders_by_export_date(merchant_id, export_date, False)
            return rv

        self.assert_permissions(*self._instance_class.permissions)
        where = and_(
            func.date(self._instance_class.export_date)==export_date,
            self._instance_class.merchant_id==merchant_id
        )

        rv = []
        for order in self._get_privileged_object(self._instance_class, where, priv=Order.PRIV_EXPORT):
            rv.append(self._load_state_for_export(order))

        return rv

    @transaction_decorator
    def remove_order_invoice(self, order_id):

        self.assert_permissions('remove_order_invoice')

        oc = self._get_controller(order_id)
        order = oc.get(_id=order_id, priv=Order.PRIV_REMOVE_INVOICE)[0]

        order.invoice = None
        order.invoice_stage = None

        self.sa_session.add(order)

        self.logger.info('remove_order_invoice: removed invoice for order %s' % (order_id))

    def payment_status_change_cb(self, order):
        """
        Called by the PaymentController when a new payment status is added for the order.
        """
        pass


class PaymentController(AbstractController):
    """
    Controls orders payment stuff. Can be used to get available payment methods or retrieve a specific
    controller for a payment_method.
    """

    _payment_method = None

    def __init__(self, sa_session, payment_method=None, *args, **kwargs):
        super(PaymentController, self).__init__(sa_session, *args, **kwargs)

        if payment_method is not None:
            assert isinstance(payment_method, PaymentMethod), payment_method
            self._payment_method = payment_method

    def get_licenses(self, license_id=None):
        """
        returns a list of licenses to which a GL account can be attached
        """

        rv = []
        controller = self.get_default_group().license.controller(self.sa_session, account_id=self.get_account().id, request=self.request)

        for license_ in controller.get_domain_licenses():
            if license_id is None or license_.id == license_id:
                rv.append(license_)
        return rv

    @transaction_decorator
    def save_or_update_payment_method_for_merchant(self, polymorphic_type, merchant=None, label=None, **kwargs):

        self.assert_permissions('manage_payment_methods') # people who can complete orders can set payment methods?

        if type(polymorphic_type) in [str, unicode]:
            methods = self.get_available_payment_methods(True)
            pt = None
            for m in methods:
                if m['value'] == polymorphic_type:
                    pt = m['model']
                    break

            if pt is None:
                raise PublicSecurityException(_('No access to provided Payment Method'))
            polymorphic_type = pt

        if merchant is None:
            merchant = self.get_default_group().license

        if not isinstance(merchant, License):
            merchant = self.get_licenses(int(merchant)).pop()

        if merchant is None:
            raise PublicSecurityException(_('No access to provided Merchant'))

        pm = self.sa_session.query(polymorphic_type).filter(polymorphic_type.merchant==merchant).first()
        if pm is None:
            pm = polymorphic_type()
            pm.merchant = merchant

        pm.label = label

        if kwargs is not None:
            for setting, value in kwargs.iteritems():
                setattr(pm, setting, value)

        self.sa_session.add(pm)
        return pm

    def get_available_payment_methods(self, include_model=False):
        rv = []
        for m in get_models_of_type(PaymentMethod, True, None, True):
            if not self.has_permissions(*m['model'].permissions):
                continue
            obj = {
                'value': m['value'],
                'label': m['model'].name_i18n,
                'js_module': m['model'].js_module
            }
            if include_model:
                obj['model'] = m['model']
            rv.append(obj)
        return rv

    def get_state(self, payment_method):
        return {}

    def get_payment_method_state(self, payment_method_id):
        payment_method = self.get_payment_methods(payment_method_id=payment_method_id)[0]
        c = payment_method.get_payment_controller()
        rv = c(self.sa_session, account_id=self.get_account().id, request=self.request).get_state(payment_method)
        rv['merchant'] = payment_method.merchant.id
        rv['label'] = payment_method.label
        return rv

    def get_payment_methods(self, merchant=None, payment_method_id=None, payment_method_polymorphic_type=None):
        """
        returns the payment methods available for the current account
        """
        assert merchant is not None or payment_method_id is not None or payment_method_polymorphic_type is not None
        where = None

        if merchant is not None:
            where = PaymentMethod.merchant==merchant

        if payment_method_id is not None:
            _where = PaymentMethod.id == payment_method_id
            where = _where if where is None else and_(where, _where)

        if payment_method_polymorphic_type is not None:
            _where = PaymentMethod.polymorphic_type == payment_method_polymorphic_type
            where = _where if where is None else and_(where, _where)

        return self.sa_session.query(PaymentMethod).filter(where).order_by(PaymentMethod.priority).all()

    def get_payment_controller(self, payment_method_id):
        payment_method = self.get_payment_methods(payment_method_id=payment_method_id)[0]
        return payment_method.get_payment_controller()

    def get_payment_vars(self, order_id):
        """
        returns payment-method specific information for the given order-id. This should happen in the payment subclass
        """

        # the PaymentController must have been instanciated with a payment_method
        assert self._payment_method is not None

        assert isinstance (order_id, (int, long))
        return {}

    @transaction_decorator
    def _add_payment_status(self, order_id, payment_status, description, payment_date=None, order_priv=Order.PRIV_PAY):
        assert isinstance(order_id, (int, long))
        assert isinstance(payment_status, (int, long) )
        assert isinstance(description, (str, unicode) )

        # get the order for the given priv (the way depends on the payment processor)
        order = self._get_order(order_id, order_priv)

        # if we come here, the PaymentController must have been instanciated with a payment_method
        # except for credits
        if order.type == Order.TYPE_DEBIT:
            assert self._payment_method is not None

            # check that the current payment_method is avaialble for the order's merchant
            if len(self.get_payment_methods(merchant=order.merchant, payment_method_id=self._payment_method.id)) != 1:
                raise PublicUserException('Payment method not available for Merchant')

        # else: order is a credit. expect not to have a payment method set
        else:
            assert self._payment_method is None

        # get last payment_method for this order if available
        # this is used to check if an order completes that it uses the same payment method
        last_payment_method = None
        if len(order.payment_statuses) > 0:
            last_payment_method = order.payment_statuses[-1].payment_method

        # verify if current pament_status is allowed to change to the new status
        current_pament_status = order.get_current_payment_status()
        if current_pament_status == OrderPaymentStatus.PAYMENT_PENDING:

            assert payment_status in (
                OrderPaymentStatus.PAYMENT_PENDING, # pending can go to pending in case of user cancels payment
                OrderPaymentStatus.PAYMENT_SUBMITTED,
                OrderPaymentStatus.PAYMENT_COMPLETED,
                OrderPaymentStatus.PAYMENT_REJECTED
            ), 'Invalid new status for order %s, %s' % (order, payment_status)

        elif current_pament_status == OrderPaymentStatus.PAYMENT_SUBMITTED:

            if payment_status == OrderPaymentStatus.PAYMENT_SUBMITTED:
                raise PublicUserException(_('The order already has status submitted'))

            assert payment_status in (
                OrderPaymentStatus.PAYMENT_COMPLETED,
                OrderPaymentStatus.PAYMENT_REJECTED,
            ), 'Invalid new status for order %s, %s' % (order, payment_status)

        elif current_pament_status == OrderPaymentStatus.PAYMENT_COMPLETED:
            raise PublicUserException(_('Cannot update order %s: status is completed' % order.id))

        elif current_pament_status == OrderPaymentStatus.PAYMENT_REJECTED:
            assert payment_status in (
                OrderPaymentStatus.PAYMENT_PENDING,
                OrderPaymentStatus.PAYMENT_SUBMITTED,
                OrderPaymentStatus.PAYMENT_COMPLETED,
            ), 'Invalid new status for order %s, %s' % (order.id, payment_status)

        else:
            raise Exception('Invalid current status for order %s, %s' % (order, current_pament_status))

        # check if the current order allows changs for its status
        if order.status == Order.STATUS_REJECTED:
            raise PublicUserException('Order [%s] does not allow payment changes' % order.id)

        account_id = None
        account = self.get_account()
        if account is not None:
            account_id = account.id
        oc = order.get_order_controller()(self.sa_session, account_id=account_id, request=self.request)

        # if the order has status new or pending, auto submit it
        if order.status in (Order.STATUS_NEW, Order.STATUS_PENDING):
            self.logger.info("Auto submitting order %s" % order.id)
            oc.submit_order(order)

        # an order cannot change payment method unless it has status rejected
        if (last_payment_method is not None):
            if current_pament_status != Order.STATUS_REJECTED:
                if last_payment_method != self._payment_method:
                    raise PublicUserException(_('Order has different payment method set'))

        if payment_date is None:
            payment_date = now()

        ps = OrderPaymentStatus()
        ps.order = order
        ps.payment_method = self._payment_method
        ps.status = payment_status
        ps.description = description
        ps.payment_date = payment_date
        self.sa_session.add(ps)

        self.logger.info("Added OrderPaymentStatus: order=%s, payment_method=%s, payment_status=%s, description=%s" % (order, self._payment_method, payment_status, description) )

        # notify the order's controller that the payment status has changed

        oc.payment_status_change_cb(order)


    def _get_order(self, order_id, order_priv):
        """
        default logic to get the order checks for the apropriate permissions. other payment processors may check it different
        """

        # check if user has 'pay' priv on this order
        try:
            return self._get_privileged_object(Order, and_(Order.id == order_id), priv=order_priv)[0]
        except IndexError:
            raise PublicUserException(_("Permission denied to order %s" % order_id))

    def submit_payment(self, order_id, description, payment_date=None):
        """
        Changes an order from pending to submitted.
        Normally doesnt have to be overrided by specific payment methods since users can submit their orders themselfs if they have pay_orders permission.
        """
        self.assert_any_permissions('pay_orders')
        self._add_payment_status(order_id, OrderPaymentStatus.PAYMENT_SUBMITTED, description, payment_date)

    def complete_payment(self, order_id, description, payment_date=None):
        """
        Default complete_payment method, asserts for apropriate permissions.
        Can be overriden in a custom payment method that checks differently
        """
        self.assert_any_permissions('complete_orders')
        self._add_payment_status(order_id, OrderPaymentStatus.PAYMENT_COMPLETED, description, payment_date, order_priv=Order.PRIV_COMPLETE)

    def reject_payment(self, order_id, description, payment_date=None):
        """
        see complete_payment()
        """
        self.assert_any_permissions('complete_orders')
        self._add_payment_status(order_id, OrderPaymentStatus.PAYMENT_REJECTED, description, payment_date, order_priv=Order.PRIV_COMPLETE)



class BankTransferPaymentController(PaymentController):
    pass


class OgonePaymentController(PaymentController):

    # ogne payment statusses
    # see Ogone_e-Com-ADV_EN.pdf and param_cookbook.pdf

    # 'requested' seems to mean status accepted according to Ogone_e-Com-ADV_EN.pdf chapter 14
    STATUS_PAYMENT_REQUESTED = 9

    # status uncertain
    STATUS_PAYMENT_UNCERTAIN = 92

    def __init__(self, sa_session, psp_id=None, *args, **kwargs):
        super(OgonePaymentController, self).__init__(sa_session, *args, **kwargs)

        self._psp_id = psp_id

    def get_state(self, payment_method):
        return {
            'psp_id': payment_method.psp_id,
            'sha_in': payment_method.sha_in,
            'sha_out': payment_method.sha_out,
        }

    @staticmethod
    def ogone_hash(fields, sha_key):
        """
        builds an ogone shasign based on the given fields and sha_key
        the 'fields' param is modified if fields are empty, then they are deleted
        """
        assert isinstance(fields, dict)

        # change all keys to upperkees
        for key in fields.keys():
            if key != key.upper():
                fields[key.upper()] = fields[key]
                del fields[key]

        # build the hash
        # hash = <key1>=<value1><sha_in><key2>=<value=2><sha_in>...
        hash_this = ''
        keys = sorted(fields.keys())
        sha_in = sha_key

        assert 'SHASIGN' not in keys

        for key in keys:

            # require keys to be uppercase already
            assert key == key.upper()

            value = fields[key]

            # ogone doesnt seem to like empty fields
            if value is None or value == '':
                del fields[key]
                continue

            hash_this += '%s=%s%s' % (key, value, sha_in)

        #print 'hash_this: %s' % hash_this
        return hashlib.sha1(hash_this.encode('utf8')).hexdigest().upper()

    def _get_psp_settings(self, order_id):

        psp_id = self._psp_id
        if self._payment_method is not None:
            psp_id = self._payment_method.psp_id
            sha_in = self._payment_method.sha_in
            sha_out = self._payment_method.sha_out
        else:

            assert psp_id is not None
            assert order_id is not None

            merchant = self.sa_session.query(Order).filter(Order.id == order_id).one().merchant
            pm = self.sa_session.query(PaymentMethodOgone) \
                .filter(PaymentMethodOgone.psp_id == psp_id,
                        PaymentMethodOgone.merchant == merchant
                        ).one()
            sha_in = pm.sha_in
            sha_out = pm.sha_out

        assert sha_in is not None
        assert sha_out is not None

        return {
            'post_action': settings.ORDER_OGONE_POST,
            'SHA_IN': sha_in,
            'SHA_OUT': sha_out,
        }

    def get_payment_vars(self, order_id):
        """
        Returns the form information needed to POST a payment request from iqportal to ogone.

        Implementation from Ogone_e-Com-BAS_EN.pdf
        http://trac.office.basesoft.com/pv-cycle/ticket/124
        """

        rv = super(OgonePaymentController, self).get_payment_vars(order_id)

        settings_vars = self._get_psp_settings(order_id=order_id)
        rv['post_action'] = settings_vars['post_action']

        aoc = AbstractOrderController(self.sa_session, account_id=self.get_account().id, request=self.request)
        order = aoc.load_order_state(order_id)

        if order['client'].has_key('invoice_person_data'):
            invoice_person = order['client']['invoice_person_data']
        else:
            try:
                invoice_person = order['client']['contact_person_data']
            except KeyError:
                raise PublicUserException(_('No company invoice person set'))

        try:
            invoice_address = order['client']['address_data']
        except KeyError:
            raise PublicUserException(_('No company invoice address set'))

        # build invoice person name
        person_name = ''
        if invoice_person['first_name']:
            person_name += invoice_person['first_name']+' '
        if invoice_person['middle_name']:
            person_name += invoice_person['middle_name']+' '
        person_name += invoice_person['last_name']

        # try to get locale, fallback to english
        try:
            locale_str = invoice_person['preferred_locale_str']
        except KeyError:
            locale_str = 'en_GB'

        # street
        street = invoice_address['street']
        if invoice_address['street_2'] is not None:
            street += " " + invoice_address['street_2']

        # amount
        amount = int(round(order['total_amount_including_vat'] * 100))
        assert amount > 0

        # urls
        assert self.request is not None
        matches = re.match("^(?P<url>https?://[^/]+)/?.*$", self.request.META['HTTP_REFERER'])
        base_url = matches.group('url')
        base_url += '/basesoft.web.contrib.order/payment_method/ogone/%s/' % self._payment_method.psp_id
        accept_url = base_url + 'accept/'
        decline_url = base_url + 'decline/'
        exception_url = base_url + 'exception/'
        cancel_url = base_url + 'cancel/'

        rv['fields'] = {
            'PSPID': self._payment_method.psp_id,
            'ORDERID': order_id,
            'AMOUNT': amount,
            'CURRENCY': order['currency_code'],
            'LANGUAGE': locale_str,
            'CN': person_name,
            'EMAIL': invoice_person['email'],
            'OWNERADDRESS': street,
            'OWNERZIP': invoice_address['postcode'],
            'OWNERTOWN': invoice_address['city'],
            'OWNERCTY': invoice_address['country_name'],
            'OWNERTELNO': invoice_person['phone'],
            'TITLE': _('PV Cycle Payment Gateway'),
            #'LOGO': 'http://ws1.basesoft.com/~martijn/logo_PVcycle.jpg', # FIXME: needs to be on the ogone server
            'ACCEPTURL': accept_url,
            'DECLINEURL': decline_url,
            'EXCEPTIONURL': exception_url,
            'CANCELURL': cancel_url,
        }

        # append sha1 signature
        rv['fields']['SHASIGN'] = OgonePaymentController.ogone_hash(rv['fields'], settings_vars['SHA_IN'])

        return rv

    def _init_ogone_reply(self, params_orig):
        """
        verifies if the reply from ogone is valid and returns the params uppercase
        """
        assert isinstance(params_orig, dict)
        self.logger.info('OgonePaymentController._init_ogone_reply: psp_id:%s -> params:%s' % (self._payment_method, params_orig))

        # convert all params to uppercase and strip out the SHASIGN
        params = {}
        shasign = None
        for k,v in params_orig.iteritems():
            if k.upper() == 'SHASIGN':
                shasign = v
                continue
            params[k.upper()] = v

        assert shasign is not None

        # check if the signed request matches its given params for the current payment_method
        settings_vars = self._get_psp_settings(params['ORDERID'])
        generated_sign = OgonePaymentController.ogone_hash(params, settings_vars['SHA_OUT'])
        if shasign != generated_sign:
            raise Exception('SHASIGN mismatch. %s != %s' % (shasign, generated_sign) )

        # now that we know that the post was signed, set the payment_method from the order
        order = self._get_order(params['ORDERID'], None)
        payment_method = self.sa_session.query(PaymentMethodOgone) \
            .filter(PaymentMethodOgone.merchant == order.merchant,
                    PaymentMethodOgone.psp_id == self._psp_id).one()
        self._payment_method = payment_method

        return params

    def _get_order(self, order_id, order_priv):
        """
        overrides the default priv based order retrieval. here we know the hash is checked so just get it
        """
        return self.sa_session.query(Order).filter(Order.id == order_id).one()

    def payment_process(self, params):
        """
        called by ogone reply view. update the payment status based on the returned status
        """
        params = self._init_ogone_reply(params)

        description = ''
        order_id = int(params['ORDERID'])
        status = int(params['STATUS'])

        if status == self.STATUS_PAYMENT_REQUESTED:
            description = _('Ogone payment accepted. payid: %s') % params['PAYID']
            self._add_payment_status(order_id, OrderPaymentStatus.PAYMENT_COMPLETED, description)

        elif status == self.STATUS_PAYMENT_UNCERTAIN:
            description = _('Ogone payment uncrertain. payid: %s') % params['PAYID']
            self._add_payment_status(order_id, OrderPaymentStatus.PAYMENT_SUBMITTED, description)

        else:
            description = _('Recieved unknown status: %s' % status)
            self.logger.error('unknown status: %s, marking order as submitted anyway' % status)
            self._add_payment_status(order_id, OrderPaymentStatus.PAYMENT_SUBMITTED, description)


    def payment_cancel(self, params):
        """
        called from view payment_method_ogone_direct_cancel .
        """
        params = self._init_ogone_reply(params)
        description = 'Ogone payment canceled. payid: %s' % params['PAYID']
        self._add_payment_status(int(params['ORDERID']), OrderPaymentStatus.PAYMENT_PENDING, description)

class OrderAggregator(IQubeAggregator):
    _table = "aggOrder"
    _affected_cubes = ({'datasource': 'Order', 'cube': 'Order'}, )
    _columns = [
        'order_id',
        'order_line_id',
        'order_type',
        'order_status',
        'invoice_type',
        'merchant_id',
        'client_id',
        'submitted_time_id',
        'payment_status',
        'payment_method',
        'payment_time_id',
        'gl_account',
        'quantity',
        'line_amount_excluding_vat',
        'line_amount_including_vat',
        'price_per_unit',
        'last_updated'
    ]

    _select_sql = """

        SELECT
            o.id as order_id,
            ol.id as order_line_id,
            o.polymorphic_type as order_type,
            o.status as order_status,
            IF(o.type=0, 'Debit', 'Credit') as invoice_type,
            o.merchant_id as merchant_id,
            o.client_id as client_id,
            IF(ISNULL(o.display_date), NULL, DATE_FORMAT(o.display_date, '%%Y%%m%%d')) as submitted_time_id,
            iqGetCurrentOrderPaymentStatus(o.id) as payment_status,
            iqGetCurrentOrderPaymentMethod(o.id) as payment_method,
            IF(ISNULL(iqGetCurrentOrderPaymentDate(o.id)), NULL, DATE_FORMAT(o.display_date, '%%Y%%m%%d')) as payment_time_id,
            ol.gl_account_number as gl_account,
            ol.quantity as quantity,
            ol.line_amount_excluding_vat as line_amount_excluding_vat,
            ol.line_amount_including_vat as line_amount_including_vat,
            ol.price_per_unit as price_per_unit,
            o.last_updated as last_updated
        FROM tblOrder as o
        INNER JOIN tblOrderLine as ol
            ON o.id = ol.order_id
        WHERE o.last_updated > %(max_last_updated)s
        AND o.deleted = 0
        ORDER BY o.id
    """

    def run(self, loop_validator=None):

        max_last_updated = 0
        rows = self.db_engine_ro.execute("""
            SELECT IFNULL(MAX(a.last_updated), 0)
            FROM %s as a""" % self._table)

        row = rows.fetchone()
        rows.close()

        if row is not None:
            max_last_updated = row[0]

        self.db_engine.execute("""
            DELETE FROM %(tbl)s WHERE order_id IN (
                SELECT id
                FROM tblOrder
                WHERE last_updated > '%(max_last_updated)s'
            )""" % {
            'tbl': self._table,
            'max_last_updated': max_last_updated
        })

        affected_rows = 0

        affected_rows = affected_rows + self._work({
            'max_last_updated': max_last_updated,
        }, loop_validator)

        return affected_rows

