'''
Created on Oct 9, 2013

@author: marten
'''
from django.utils.translation import ugettext_lazy, ugettext as _
from sqlalchemy.orm.query import aliased
from sqlalchemy import func

from basesoft.web.core.util.custom_query import CustomQuery, CustomQueryModel, CustomQueryField,\
    DATA_TYPE_DECIMAL
from basesoft.web.contrib.order.models import Order, TaxClassCountry, TaxClass, InvoiceFile,\
    GLAccount, OrderPaymentStatus, PaymentMethod, Currency
from basesoft.web.contrib.order.controllers import get_available_order_types,\
    GLAccountController, PaymentController
from basesoft.web.contrib.license.models import License
from basesoft.web.contrib.i18n.models import Country
from basesoft.util.virtual_date import today
from basesoft.web.core.exceptions import PublicUserException
from basesoft.web.core.util.controllers import get_models_of_type

Merchant = aliased(License)

class TaxClassCustomQuery(CustomQuery):

    permissions = ['manage_tax_classes']

    def tables(self):

        return [
            CustomQueryModel(TaxClass),
        ]

    def fields(self):
        return [
            CustomQueryField(TaxClass.id, _('ID'), key='tax_class_id'),
            CustomQueryField(TaxClass.name, _('Name'), key='tax_name'),
            CustomQueryField(TaxClass.code, _('Code'), key='tax_code'),
        ]

class TaxClassCountryCustomQuery(CustomQuery):

    permissions = ['view_country_tax']

    def tables(self):

        return [
            CustomQueryModel(TaxClass),
            CustomQueryModel(TaxClassCountry, onclause=TaxClassCountry.tax_class_id==TaxClass.id),
            CustomQueryModel(Country, onclause=TaxClassCountry.country_id==Country.id),
        ]

    def fields(self):
        return [
            CustomQueryField(TaxClassCountry.id, _('ID'), key='tax_class_country_id'),
            CustomQueryField(Country.name, _('Country name'), key='country_name'),
            CustomQueryField(TaxClass.name, _('Name'), key='tax_name'),
            CustomQueryField(TaxClass.code, _('Code'), key='tax_code'),
            CustomQueryField(func.iqGetCurrentTaxCountryRate(TaxClass.id, Country.id, today()), _('Current VAT rate %'), key='current_rate', type=DATA_TYPE_DECIMAL),
        ]
    group_by = (TaxClassCountry.id, )

class GLAccountCustomQuery(CustomQuery):

    group_by = (GLAccount.id, )

    permissions = ['manage_gl_accounts']

    def tables(self):
        return [
            CustomQueryModel(GLAccount),
            CustomQueryModel(Merchant, onclause=GLAccount.merchant_id==Merchant.id),
        ]

    def fields(self):
        return [
            CustomQueryField(GLAccount.id, _('ID'), key='gl_account_id'),
            CustomQueryField(GLAccount.code, _('Code'), key='gl_code', is_filter=True),
            CustomQueryField(Merchant.name, _('Merchant'), key='merchant', is_filter=True),
            CustomQueryField(GLAccount.account_number, _('Account Number'), key='gl_account_number'),
        ]

    def set_where(self):
        c = GLAccountController(self.sa_session, account_id=self.get_account().id, request=self.request)
        ids = [x.id for x in c.get_gl_licenses()]
        if len(ids) == 0:
            raise PublicUserException(_('No licenses available to assign GL Accounts to.'))
        self.where = License.id.in_( ids )

class AbstractOrderCustomQuery(CustomQuery):
    I18N_CHECKOUT_LABEL = ugettext_lazy('Checkout')
    _order_types = {} # on model level so we only load it once

    def _tables(self, wp=None):

        if wp is None:
            wp = get_available_order_types(self)

        return [
            CustomQueryModel(Order, with_polymorphic=wp),
            CustomQueryModel(License, onclause=Order.client_id==License.id),
            CustomQueryModel(Merchant, onclause=Order.merchant_id==Merchant.id),
            CustomQueryModel(Currency, onclause=Order.currency_id==Currency.id),
            CustomQueryModel(InvoiceFile, onclause=Order.invoice_id==InvoiceFile.file_id, use_outerjoin=True, priv=None),
        ]

    def _fields(self):

        self._load_order_types()

        return [
            CustomQueryField(Order.id, 'Order ID', key='id', is_filter=True),
            CustomQueryField(Order.polymorphic_type, 'Polymorphic type', key='polymorphic_type'),
            CustomQueryField(Order.id, _('Details'), key='details'),
            CustomQueryField(Order.status, _('Status'), key='status', is_filter=True, filter_options=[{
                'value': Order.STATUS_NEW,
                'label': _("Incomplete"),
            },{
                'value': Order.STATUS_PENDING,
                'label': _("Pending"),
            },{
                'value': Order.STATUS_SUBMITTED,
                'label': _("Submitted"),
            },{
                'value': Order.STATUS_REJECTED,
                'label': _("Rejected"),
            },{
                'value': Order.STATUS_CREDITED,
                'label': _("Credited"),
            }]),
            CustomQueryField(License.name, _('Client'), key='client', is_filter=True),
            CustomQueryField(Merchant.name, _('Merchant'), key='merchant', is_filter=True, is_group_by=True),
            CustomQueryField(func.iqGetCurrentOrderPaymentStatus(Order.id), _('Payment status'), key='payment_status', is_filter=True, filter_options=[{
                'value': OrderPaymentStatus.PAYMENT_PENDING,
                'label': _("Pending"),
            },{
                'value': OrderPaymentStatus.PAYMENT_SUBMITTED,
                'label': _("Submitted"),
            },{
                'value': OrderPaymentStatus.PAYMENT_PARTIALLY_RECEIVED,
                'label': _("Partially received"),
            },{
                'value': OrderPaymentStatus.PAYMENT_COMPLETED,
                'label': _("Completed"),
            },{
                'value': OrderPaymentStatus.PAYMENT_REJECTED,
                'label': _("Rejected"),
            }]),
            CustomQueryField(func.ifnull(Currency.symbol, Currency.code), _('Currency'), 'currency'),
            CustomQueryField(Order.total_amount_excluding_vat, _('Total amount (ex VAT)'), 'total_amount'),
            CustomQueryField(Order.total_amount_including_vat, _('Total amount (incl VAT)'), 'total_amount_incl_vat'),
            CustomQueryField(Order.id, self.I18N_CHECKOUT_LABEL, key='payment_checkout'),
            CustomQueryField(Order.export_date, _('Export date'), key='export_date'),
            CustomQueryField(InvoiceFile.file_hash, _('Invoice'), 'invoice'),
            CustomQueryField(Order.type, _('Type'), key='type'),
            CustomQueryField(Order.polymorphic_type, _('Order type'), key='order_type', is_filter=True, filter_options=[ {'value': key, 'label': label} for key, label in self._order_types.iteritems()  ] ),
            CustomQueryField(Order.created, _('Created'), key='created', is_filter=True),

            CustomQueryField(func.iqGetCurrentOrderPaymentMethod(Order.id), _('Payment method'), key='payment_method', is_filter=True),

        ]

    def set_value_processors(self):
        self._value_processors['order_type'] = self._get_order_type

    def _load_order_types(self):
        self._order_types = {}
        models = get_models_of_type(Order, exclude_parent=True)
        for m in models:
            self._order_types[m['value']] = m['model']().get_name_i18n()

    def _get_order_type(self, row, key):
        if len(self._order_types) == 0:
            self._load_order_types()
        value = getattr(row, key)
        if self._order_types.has_key(value):
            return self._order_types[value]
        return value

class DefaultOrderCustomQuery(AbstractOrderCustomQuery):
    def tables(self):
        return self._tables()

    def fields(self):
        return self._fields()

    group_by = (Order.id, )

class PaymentStatusCustomQuery(CustomQuery):

    order_by = (OrderPaymentStatus.last_updated.desc(), )

    def tables(self):

        return [
            CustomQueryModel(Order, auto_group_by=False),
            CustomQueryModel(OrderPaymentStatus),
            CustomQueryModel(PaymentMethod, use_outerjoin=True),
        ]

    def fields(self):
        return [
            CustomQueryField(Order.id, 'Order ID', key='order_id', is_filter=True, in_select=False),
            CustomQueryField(OrderPaymentStatus.status, _('Status'), key='payment_status'),
            CustomQueryField(PaymentMethod.label, _('Payment method'), key='payment_method'),
            CustomQueryField(OrderPaymentStatus.description, _('Description'), key='description'),
            CustomQueryField(OrderPaymentStatus.last_updated, _('Last updated'), key='last_updated'),
        ]

class AbstractPaymentMethodCustomQuery(CustomQuery):

    permissions = ['manage_payment_methods']

    def tables(self):
        return (
            CustomQueryModel(PaymentMethod, with_polymorphic='*'),
            CustomQueryModel(License)
        )

    def fields(self):
        return (
            CustomQueryField(PaymentMethod.id, _('ID'), key='id'),
            CustomQueryField(PaymentMethod.polymorphic_type, _('Method'), key='polymorphic_type', is_filter=True),
            CustomQueryField(PaymentMethod.label, _('Label'), key='label', is_filter=True),
            CustomQueryField(License.name, _('Merchant'), key='merchant', is_filter=True),
            CustomQueryField(PaymentMethod.id, _('Details'), key='details'),
        )

custom_queries = (DefaultOrderCustomQuery, TaxClassCustomQuery, TaxClassCountryCustomQuery, GLAccountCustomQuery, PaymentStatusCustomQuery)

