# Copyright (c) 2013 BaseSoft B.V.
# See LICENSE for details.
from basesoft.web.contrib.account.decorators import login_required
from basesoft.web.core.util.controllers import get_models_of_type
from basesoft.web.contrib.order.models import Order, PaymentMethod,\
    OrderPaymentStatus
from basesoft.web.contrib.order.controllers import AbstractOrderController,\
    TaxController, GLAccountController, PaymentController
from basesoft.web.core.forms.handler import FormValidator
from basesoft.web.core.input.validation import plugins
from django.utils.translation import ugettext as _
import datetime

from django.conf import settings
from iqportal.iqube.core.controllers import ConfigurableWidgetController,\
    DataSourceController
from django.utils import simplejson

class TaxHandler:

    @login_required
    def __init__(self, request, **kwargs):
        self.sa_session = request.Session()
        self.account_id = request.session['user_id']

        self._controller = TaxController(self.sa_session, account_id = self.account_id, request=request)

    def get_tax_classes(self):
        return [{'label': tax_class.name, 'value': tax_class.id} for tax_class in self._controller.get_tax_classes()]

    def validate_tax_class(self, args):

        form_handler = FormValidator({
            'name': [(plugins.Required, {})],
            'code': [(plugins.Required, {})],
        })
        return form_handler.validate(args)

    def save_tax_class(self, id, state):
        return self._controller.save_tax_class_state(id, state).id

    def load_tax_class(self, id):
        return self._controller.get_tax_class_state(id)

    def validate_save_country_tax(self, args):

        form_handler = FormValidator({
            'country_id': [(plugins.Required, {})],
            'tax_class_id': [(plugins.Required, {})],
            'start_date': [(plugins.Required, {})],
            'rate': [(plugins.Required, {}), (plugins.Number, {})],

        })
        return form_handler.validate(args)

    def load_country_tax(self, id):
        return self._controller.get_tax_class_country_state(id)

    def save_country_tax(self, id, state):
        return self._controller.save_country_tax_state(id, state).id

    def delete_country_taxes(self, ids):
        for id in ids:
            self._controller.delete_country_taxes(id)

class GLAccountHandler:

    @login_required
    def __init__(self, request, **kwargs):
        self.sa_session = request.Session()
        self.account_id = request.session['user_id']

        self._controller = GLAccountController(self.sa_session, account_id = self.account_id, request=request)

    def validate_gl_account(self, args):

        form_handler = FormValidator({
            'licenses': [(plugins.Required, {})],
            'code': [(plugins.Required, {})],
            'account_number': [(plugins.Required, {})],
        })
        return form_handler.validate(args)

    def save_gl_account(self, id, state):
        return self._controller.save_gl_account_state(id, state).id

    def load_gl_account(self, id):
        return self._controller.get_gl_account_state(id)

    def delete_gl_accounts(self, ids):
        for id in ids:
            self._controller.delete_gl_accounts(id)

    def get_gl_licenses(self):
        return [{'label': license_.name, 'value': license_.id}
            for license_ in self._controller.get_gl_licenses()]


class OrderHandler:

    @login_required
    def __init__(self, request, **kwargs):
        self.sa_session = request.Session()
        self.account_id = request.session['user_id']
        self.request = request

        self.aoc = AbstractOrderController(self.sa_session, account_id=self.account_id, request=request)
        self.pc = PaymentController(self.sa_session, account_id=self.account_id, request=request)

    def get_js_module(self, polymorphic_type):
        model = get_models_of_type(Order, exclude_parent=True, name=polymorphic_type)
        return model.js_details_module

    def is_credit(self, order_id):
        assert order_id is not None
        order = self.aoc.get_order(order_id).pop()
        return order.type == Order.TYPE_CREDIT

    def _get_payment_controller_instance(self, payment_method_id):
        payment_method = self.pc.get_payment_methods(payment_method_id=payment_method_id)[0]
        return self.pc.get_payment_controller(payment_method_id)(self.sa_session, account_id=self.account_id, request=self.request, payment_method=payment_method)

    def load_order(self, order_id):
        return self.aoc.load_order_state(order_id)

    def remove_invoices(self, order_ids):
        for order_id in order_ids:
            return self.aoc.remove_order_invoice(order_id)

    def get_available_payment_methods(self):
        return self.pc.get_available_payment_methods()

    def get_payment_method_js_module(self, polymorphic_type):
        model = get_models_of_type(PaymentMethod, exclude_parent=True, name=polymorphic_type)
        return model.js_module

    def get_payment_methods(self, order_id):
        assert order_id is not None
        order = self.aoc.get_order(order_id).pop()
        assert order is not None

        rv = []
        for pm in self.pc.get_payment_methods(order.merchant):
            rv.append({
                'id': pm.id,
                'name': pm.label,
                'js_module': pm.js_module
            })

        return rv

    def get_payment_method_state(self, payment_method_id):
        return self.pc.get_payment_method_state(payment_method_id)

    def get_payment_details(self, order_id, payment_method_id):
        order = self.load_order(order_id)
        pc = self._get_payment_controller_instance(payment_method_id)

        return {
            'order': order,
            'payment_vars': pc.get_payment_vars(order_id)
        }

    def submit_payment(self, order_id, payment_method_id):
        self.aoc.submit_payment(order_id, payment_method_id)
        return True

    def save_payment_method_for_merchant(self, polymorphic_type, values):
        pm = self.pc.save_or_update_payment_method_for_merchant(polymorphic_type, **values)
        return pm.id

    def save_payment_status(self, payment_status_id, form_values):

        order_id = int(form_values['order_id'])
        payment_method_id = None if not form_values.has_key('payment_method') else int(form_values['payment_method'])
        payment_status = int(form_values['payment_status'])
        description = form_values['description']

        if payment_method_id is None:
            pc = PaymentController(self.sa_session, account_id=self.account_id, request=self.request)
        else:
            pc = self._get_payment_controller_instance(payment_method_id)

        if payment_status == OrderPaymentStatus.PAYMENT_COMPLETED:
            pc.complete_payment(order_id, description)
        elif payment_status == OrderPaymentStatus.PAYMENT_REJECTED:
            pc.reject_payment(order_id, description)
        else:
            raise Exception('Not implemented payment_status: %s' % payment_status)

handlers = (OrderHandler, TaxHandler, GLAccountHandler)
