# Copyright (c) 2010 BaseSoft B.V.
# See LICENSE for details.

"""
Session Controller module
"""
from basesoft.web.contrib.session.models import HTTPSession
from datetime import datetime, timedelta
from django.conf import settings
from sqlalchemy.exc import InvalidRequestError

class SessionController(object):
    """
    Contains methods for working with HTTPSessions
    """

    def __init__(self, sa_session):
        """
        Initialize the controller and specify the sqlalchemy session so we can query for sessions, etc.
        """
        self.sa_session = sa_session

    def cleanup_sessions(self):
        """
        This will remove all Sessions which expired x time ago
        """
        max_removal_date = datetime.now()-timedelta(seconds=settings.EXPIRED_SESSION_REMOVAL_AGE)

        sessions = self.sa_session.query(HTTPSession).filter(HTTPSession.expire_date<max_removal_date).all()
        for s in sessions:
            self.sa_session.delete(s)

        self.sa_session.flush()

    def get_sessions(self, account_id=None, non_expired=False):
        """
        Return a list with HTTPSession.
        @param account_id: int or list for the id's that we want the sessions for. if None, get sessions for all users
        @param non_expired: a boolean that indicates if the sessions we want to return are not expired at the moment
            we request them. so if True, get only sessions that have an expire_date in the future
        """
        q = self.sa_session.query(HTTPSession)

        if account_id is not None:
            if not isinstance(account_id, list):
                account_id = [account_id]
            q = q.filter(HTTPSession.account_id.in_(account_id))

        if non_expired:
            q = q.filter(HTTPSession.expire_date > datetime.now())

        return q.all()

    def check_active(self, account_id):
        """
        Check if the given account has any active sessions, and if so, return True, which means we're logged already
        somewhere else. If there are no active sessions, return False and also delete any sessions that are inactive
        (even if they are not expired yet - that happens when users close the tab and keepalive doesn't ping to refresh
        the sessions). For determining if an unexpired session is still active, we use the settings.ACTIVITIY_TIMEOUT
        setting.

        @param account_id: the account for which we are trying to determine if there are active sessions or not
        @return: boolean that indicates if the user is active already
        """
        sessions = self.get_sessions(account_id, True)

        if len(sessions) == 0:
            return False # not active anywhere else

        # can't use the transaction decorator here, because this is not an AbstractController
        if self.sa_session.is_active:
            self.sa_session.begin_nested()
        else:
            self.sa_session.begin()

        deleted = False # a flag that indicates if any sessions were deleted, to know if a flush is needed
        ret = False
        for s in sessions:
            if datetime.now() - s.last_updated > timedelta(minutes=settings.ACTIVITIY_TIMEOUT):
                # we have an active session that hasn't been updated in the last ACTIVITIY_TIMEOUT minutes, so we can
                # delete that and log in the new place
                self.sa_session.delete(s)
                deleted = True
            else:
                # we have an active session that has not expired, so we can't log from another place
                ret=True
                break

        if deleted:
            self.sa_session.flush()

        self.sa_session.commit()
        return ret

    def kill_sessions(self, session_key):
        """
        Kill the HTTPSession with the given session_key.
        @param session_key: a session key or a list of keys that need to be killed
        """
        if not isinstance(session_key, list):
            session_key = [session_key]

        sessions = self.sa_session.query(HTTPSession).filter(HTTPSession.session_key.in_(session_key))

        for s in sessions:
            try:
                self.sa_session.delete(s)
            except InvalidRequestError:
                pass
        self.sa_session.flush()
