# Copyright (c) 2010 BaseSoft B.V.
# See LICENSE for details.

import string

import xmlrpclib
from django.conf import settings
from django.utils.translation import ugettext as _

from basesoft.web.core.util.controllers import AbstractController, transaction_decorator, LicenseConfigController,\
    AccountConfigController
from basesoft.web.contrib.ticketing.models import Ticket
from basesoft.web.contrib.license.models import Group
from basesoft.web.core.exceptions import PermissionDeniedException
from basesoft.web.contrib.wiki.parser import Parser
from basesoft.web.contrib.wiki.rpc import PreviewContext

class TicketController(AbstractController):

    def __init__(self, sa_session=None, account_id=None, server=None):
        """
        Initialise the controller.
        @param server: a xmlrpclib.ServerProxy or None. If None (default), it will be created using the TRAC_CONNSTR
            connection string defined in settings
        """
        AbstractController.__init__(self, sa_session=sa_session, account_id=account_id)
        if server is None:
            self.server = xmlrpclib.ServerProxy(settings.TRAC_CONNSTR)
        else:
            self.server = server

    def get_tickets(self, id=None):
        self.assert_permissions('view_ticket')
        return self.get_privileged_object_by_id(Ticket, id)

    def get_tickets_by_trac_id(self, trac_ticket_id):
        self.assert_permissions('view_ticket')
        return self._get_privileged_object(Ticket, where=Ticket.trac_ticket_id==trac_ticket_id)

    def get_non_privileged_tickets(self):
        """
        Return a list of tickets that are not yet in IQPortal as privileged object.
        """
        self.assert_permissions('ticket_admin')
        privileged_tickets = [t.trac_ticket_id for t in self.sa_session.query(Ticket).all()]

        tickets = [t for t in self.server.ticket.query() if t not in privileged_tickets]

        rv = []

        multicall = xmlrpclib.MultiCall(self.server)
        for ticket in tickets:
            multicall.ticket.get(ticket)

        for t in multicall():
            rv.append({'value': t[0], 'label': t[3]['summary']})

        return rv

    @transaction_decorator
    def save_ticket_to_privileged_object(self, tickets, groups, privileges, group_has_privilege=True, license_has_privilege=False, domain_has_privilege=False):
        self.assert_permissions('ticket_admin')

        groups = self.sa_session.query(Group).filter(Group.id.in_(groups)).all()

        for id in tickets:
            t = Ticket()
            t.trac_ticket_id = id
            for g in groups:
                self.grant_privilege_to(
                    t,
                    privileges,
                    group=group_has_privilege,
                    license=license_has_privilege,
                    domain=domain_has_privilege,
                    target_group=g,
                    check_own_privs=False
                )

    def _get_version(self, state):
        """
        Strip any unimportant information from the version (betaX, pX, etc)
        """
        if state.has_key('version') and state['version'] is not None:
            return state['version']
        import re
        return re.sub(r'^(.*)(_.*)$', '\\1', settings.VERSION)

    def _get_keywords(self, state):
        """
        If no keywords were provided with the state, return the IQPortal version (acceptance, stable, etc)
        """
        if state.has_key('keywords') and state['keywords'] is not None:
            return state['keywords']
        rv = None if settings.TRAC_IQPORTAL_VERSION is None else settings.TRAC_IQPORTAL_VERSION
        if rv is None:
            return self._get_version(state)

        return '%s (%s)' % (rv, self._get_version(state))

    @transaction_decorator
    def save_or_update_ticket(self, state):
        """
        Creates/updates a ticket object based on the given state and returns the updated state
        """
        assert isinstance(state, dict)
        assert isinstance(state['summary'], unicode)
        assert isinstance(state['description'], unicode)
        assert isinstance(state['type'], unicode)

        # save or update
        if state['id'] is not None:
            self.assert_permissions('update_ticket')
        else:
            self.assert_permissions('create_ticket')

        attributes = {
            'type': state['type'],
            'iqportal_reporter': self.get_account().name,
            'keywords': self._get_keywords(state)
        }

        cc = []
        if self.get_account().person is not None and self.get_account().person.email is not None:
            cc.append(self.get_account().person.email)

        lcc = LicenseConfigController(self.sa_session, scope=self.get_default_group().license)
        # license config for always CC. #3223
        always_cc = lcc.get('basesoft.web.contrib.ticketing.always.cc')
        if always_cc != '':
            cc.append(always_cc)

        if len(cc) > 0:
            attributes['cc'] = ','.join(cc)

        if state['id'] is None:
            # we create the ticket
            t = Ticket()
            summary = '[%s] %s' % (self.get_default_group().license.domain.name, state['summary'])
            t.trac_ticket_id = int(self.server.ticket.create(summary, state['description'], attributes, True))
            attributes['escalate'] = 'none'
        else:
            # need to update existing ticket
            t = self.get_privileged_object_by_id(Ticket, state['id'])[0]
            if self.has_privileges_on_object(t, Ticket.PRIV_UPDATE):
                attributes['action'] = 'leave' # required to support workflow
                attributes['summary'] = state['summary']
                attributes['description'] = string.replace(state['description'], '\r', '') # remove any \r for trac
            else:
                attributes = {}

            comment = ''
            # check if we can add comments
            if self.has_permissions('create_ticket_comment') and self.has_privileges_on_object(t, Ticket.PRIV_ADD_COMMENT):
                assert isinstance(state['comment'], unicode)
                comment = '%s\n----\nReported by: %s' % (state['comment'], self.get_account().name)

            self.server.ticket.update(t.trac_ticket_id, comment, attributes, True) # update ticket on trac also

        if state.has_key('attachment') and state['attachment'] not in [None, '']:
            # trac expects base64 encoded file content
            from basesoft.web.contrib.file.controllers import UploadFileController
            ufc = UploadFileController(self.sa_session)
            f = ufc.get_hash_temp_files(state['attachment']).pop()
            content = xmlrpclib.Binary(ufc.get_file_content(f.file_hash)['content'])
            self.server.ticket.putAttachment(t.trac_ticket_id, f.file_name, self.get_account().name, content, False) # last argument is "replace", always set to false.

        t.account_id = self._account.id

        if state['id'] is None:
            self.save_default_privilege_object_group(t)

        self.sa_session.add(t)
        self.sa_session.flush()

        return self.load_ticket_state(t.id)


    def load_ticket_state(self, id, id_already_from_trac=False, is_trac_id=False):
        """
        Get the ticket state for the supplied id.
        """
        if id_already_from_trac:
            self.assert_permissions('ticket_admin')
            t = self.server.ticket.get(id)
            privileged_id = None
            privileges = None
        else:
            if is_trac_id:
                ticket = self.get_tickets_by_trac_id(id)
            else:
                ticket = self.get_tickets(id)
            if len(ticket) == 0:
                raise PermissionDeniedException(_('You do not have access to this ticket'))
            ticket = ticket.pop()
            t = self.server.ticket.get(ticket.trac_ticket_id)
            privileged_id = ticket.id
            privileges = {
                'update': self.has_privileges_on_object(ticket, Ticket.PRIV_UPDATE),
                'comment': self.has_privileges_on_object(ticket, Ticket.PRIV_ADD_COMMENT)
            }

        id = t[0]
        attributes = t[3]

        comments = []
        attachments = []

        p = Parser()
        p.set_context(PreviewContext())

        if self.has_permissions('view_ticket_comment'):
            changelog = self.server.ticket.changeLog(id)
            for c in changelog:
                field = c[2]
                newvalue = c[4]
                # changelog is a line that contains example of what has changed. eg:
                # [<DateTime '20101021T11:12:10' at 9d5318c>, 'Mircea Uifalean', 'comment', '1', 'some comment', 1]
                # [<DateTime '20101021T11:12:10' at 9d531cc>, 'Mircea Uifalean', 'component', '', 'Core', 1]
                if field == 'comment':
                    if newvalue != '':
                        comments.append(p.render_to_html(newvalue)) # wiki view

            a = self.server.ticket.listAttachments(id)
            for i in a:
                attachments.append([i[0], i[1], i[2], i[4]])

        # return the state of this ticket
        return {
            'id': privileged_id,
            'summary': attributes['summary'],
            'status': attributes['status'],
            'resolution': attributes['resolution'],
            'type': attributes['type'],
            'description': string.replace(attributes['description'], '\r', ''),
            'trac_ticket_id': id,
            'comments': comments,
            'privileges': privileges,
            'attachments': attachments
        }

    def view_attachement(self, id, file_name):
        ticket = self.get_tickets(id).pop()
        self.assert_permissions('view_ticket_comment')

        b = self.server.ticket.getAttachment(ticket.trac_ticket_id, file_name)

        return {
            'content': b.data,
            'filename': file_name
        }

    def get_ticket_types(self):
        """"
        Return the available ticket types from trac
        """
        ticket_types = self.server.ticket.type.getAll()
        return ticket_types

    def get_ticket_info(self):
        """
        Return the ticket info for the tickets the current user has access to (summary, status, resolution)
        This will be a dictionary that has the trac_ticket_id as key and a dictionary containing summary, status and
        resolution as values. eg: {40: {'status': 'closed', 'summary': 'Ticket about stuff', 'resolution': 'fixed'}}
        """
        tickets = self.get_tickets()

        multicall = xmlrpclib.MultiCall(self.server)
        for ticket in tickets:
            multicall.ticket.get(ticket.trac_ticket_id)

        info = {}
        for t in multicall():
            # t is a list like we get when calling get(id)
            id = t[0]
            attributes = t[3]
            info[id] = {
                'summary': attributes['summary'],
                'status': attributes['status'],
                'resolution': attributes['resolution']
            }

        return info

    def _connect_direct(self):

        from sqlalchemy import create_engine
        from sqlalchemy.orm import scoped_session, sessionmaker

        db_engine = create_engine(settings.TRAC_DB_CONNSTR, echo=False, pool_recycle=2)
        return scoped_session(sessionmaker(bind=db_engine, autoflush=True, autocommit=True))

    def get_new_count(self):
        if settings.TRAC_DB_CONNSTR is None:
            return None

        tickets = self.get_tickets()
        if len(tickets) == 0:
            return None

        acc = AccountConfigController(self.sa_session, account_id=self.get_account().id)
        last_ticket_retrieval = acc.get('basesoft.web.contrib.ticketing.last.ticket.retrieval')

        sql = """

            SELECT COUNT( DISTINCT `t`.`id` ) as cnt
            FROM `ticket` as `t`
            WHERE `t`.`changetime` > %(last_ticket_retrieval)s
                AND `t`.`id` IN (%(tickets)s)
        """ % {
            'last_ticket_retrieval': last_ticket_retrieval,
            'tickets': ', '.join([str(t.trac_ticket_id) for t in tickets])
        }
        sess = self._connect_direct()
        result = sess.execute(sql).fetchone()
        if result is None:
            return None
        return result['cnt'] if int(result['cnt']) > 0 else None

