# Copyright (c) 2010 BaseSoft B.V.
# See LICENSE for details.

"""
This defines the tables for the ticketing app.
"""

import datetime

from sqlalchemy.orm import mapper, relation
from sqlalchemy import Column, Integer, ForeignKey, DateTime, SmallInteger
from basesoft.sqlalchemy.table import IQTable

from basesoft.web.contrib.license import models as license_models
from basesoft.web.contrib.ticketing import models as ticket_models
from django.utils.translation import ugettext as _

__mapping_defined = False
__tables_defined = False
tables = {}

def reset():
    global __mapping_defined
    global __tables_defined

    __mapping_defined = False
    __tables_defined = False

def get_tables():
    """
    """
    return tables

def do_tables(metadata):
    global __tables_defined
    if not __tables_defined:
        # tickets

        tblTicket = IQTable('tblTicket', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('trac_ticket_id', Integer, nullable=False),
            Column('account_id', Integer, ForeignKey('tblAccount.id')), # account that created the ticket

            Column('deleted', SmallInteger, nullable=False, default=0),
            Column('archived', SmallInteger, nullable=False, default=0),

            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        tblTicketGroup = IQTable('tblTicketGroup', metadata,
            Column('ticket_id', Integer, ForeignKey('tblTicket.id', ondelete="CASCADE"), primary_key=True),
            Column('group_id', Integer, ForeignKey('tblGroup.id'), primary_key=True),
            Column('privilege', Integer, nullable=False, primary_key=True, autoincrement=False),
            Column('group_has_privilege', SmallInteger, default=0),
            Column('license_has_privilege', SmallInteger, default=0),
            Column('domain_has_privilege', SmallInteger, default=0),
            Column('world_has_privilege', SmallInteger, default=0),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        tables['tblTicket'] = tblTicket
        tables['tblTicketGroup'] = tblTicketGroup
        __tables_defined = True

def do_mappings():
    """
    This function is only called by the django middleware
    when it is needed
    """
    global __mapping_defined
    if not __mapping_defined:

        mapper(ticket_models.Ticket, tables['tblTicket'],
            properties = {
                'groups': relation(ticket_models.TicketGroup, backref='ticket', cascade="all", passive_deletes=True),
            }
        )

        mapper(ticket_models.TicketGroup, tables['tblTicketGroup'],
            properties = {
                'group': relation(license_models.Group)
            }
        )

        __mapping_defined = True

def app_permissions_category():
    """
    Returns a category for all permissions defined for this app
    """
    return license_models.PermissionCategory(u'Ticketing')

def app_permissions():
    """
    The permissions used within this application. Can be sync'd with the
    `iqsyncpermissions` command.
    """
    return [
        license_models.Permission(u'ticket_admin', u"""
            This gives the right to administer ALL tickets.
            """, _("Ticket admin")),

        license_models.Permission(u'view_ticket', u"""
            This gives the right to view a ticket.
            """, _("View tickets")),
        license_models.Permission(u'create_ticket', u"""
            This gives the right to create a ticket.
            """, _("Create tickets")),
        license_models.Permission(u'update_ticket', u"""
            This gives the right to update a ticket.
            """, _("Update tickets")),
        license_models.Permission(u'delete_ticket', u"""
            This gives the right to delete a ticket.
            """, _("Delete tickets")),

        license_models.Permission(u'view_ticket_comment', u"""
            This gives the right to view a ticket comment.
            """, _("View ticket comments")),
        license_models.Permission(u'create_ticket_comment', u"""
            This gives the right to create a ticket comment.
            """, _("Create ticket comments")),
        ]

def app_configs():
    from basesoft.web.contrib.account import models as account_models
    configs = []

    config = account_models.Config()
    config.name = 'basesoft.web.contrib.ticketing.always.cc'
    config.type = 'string'
    config.default = ''
    config.description = u'On ticket creation and new comments a CC will be send to this address.'
    configs.append(config)

    config = account_models.Config()
    config.name = 'basesoft.web.contrib.ticketing.last.ticket.retrieval'
    config.type = 'integer'
    config.default = 0
    config.description = u'Last timestamp (since epoch) of ticket retrieval, used to inform user of new ticket changes.'
    configs.append(config)

    return configs
