# Copyright (c) 2013 BaseSoft B.V.
# See LICENSE for details.

"""
Based on Trac wiki
"""
import locale

import re
import os
import datetime
from basesoft.web.contrib.wiki.provider import WikiProvider
from basesoft.web.core.exceptions import PublicUserException
from django.utils.translation import ugettext as _
from django.utils.dateformat import DateFormat
from django.utils.formats import get_format

class Formatter:
    """
    """
    def __init__(self, macros=None, providers=None, context=None):
        """
        """

        if providers is None:
            providers = self._get_providers()
        self.providers = providers
        self.macros = macros
        self.context = context
        self.reset()

    def _get_providers(self):
        """
        """
        return [WikiProvider()]


class HTMLFormatter(Formatter):
    """
    """

    def reset(self):
        """
        """
        self._open_tags = []
        self._anchors = {}
        self.paragraph_open = False
        self.in_list_item = False
        self.in_code_block = 0
        self.in_center = 0
        self.in_table = 0
        self.in_table_row = 0
        self.in_table_cell = 0
        self._list_stack = []

    def close(self, items):
        """
        """
        rv = ''
        for item in items:
            close = getattr(self,'close_%s' % item)
            rv += close()
        return rv

    def open_paragraph(self):
        if not self.paragraph_open:
            self.paragraph_open = True
            return '<p>' + os.linesep
        return ''

    def close_paragraph(self):
        """
        """
        if self.paragraph_open:
            rv = ''
            while self._open_tags != []:
                rv += self._open_tags.pop()[1]
            self.paragraph_open = False
            return rv + '</p>' + os.linesep
        return ''

    def replace(self, fullmatch):
        """
        """
        for itype, match in fullmatch.groupdict().items():
            if match is None:
                continue
            if match[0] == '!':
                continue
            if itype in ['hdepth', 'lns', 'ltgt', 'label','macroname', 'inline', 'inline2','ldepth', 'property', 'definition']:
                continue
            if not hasattr(self, '_%s_formatter' % itype):
                raise PublicUserException(_('Incorrect usage of the wiki-editor. (%s)' % itype))
            handler = getattr(self, '_%s_formatter' % itype)
            return handler(match, fullmatch)

    def _lowerthan_formatter(self, match, fullmatch):
        return '&lt;'

    def _bolditalic_formatter(self, match, fullmatch):
        italic = ('<em>', '</em>')
        italic_open = self.has_open_tag(italic)
        tmp = ''
        if italic_open:
            tmp += italic[1]
            self.close_tag(italic[1])
        tmp += self._bold_formatter(match, fullmatch)
        if not italic_open:
            tmp += italic[0]
            self.open_tag(*italic)
        return tmp

    def _italic_formatter(self, match, fullmatch):
        return self.simple_tag_handler(match, '<em>', '</em>')

    def _bold_formatter(self, match, fullmatch):
        return self.simple_tag_handler(match, '<strong>', '</strong>')

    def _underline_formatter(self, match, fullmatch):
        return self.simple_tag_handler(match, '<u>', '</u>')

    def _strike_formatter(self, match, fullmatch):
        return self.simple_tag_handler(match, '<del>', '</del>')

    def _center_formatter(self, match, fullmatch):
        if self.in_center:
            self.in_center = 0
            return '</center>'

        self.in_center = 1

        return '<center>'

    def _subscript_formatter(self, match, fullmatch):
        return self.simple_tag_handler(match, '<sub>', '</sub>')

    def _superscript_formatter(self, match, fullmatch):
        return self.simple_tag_handler(match, '<sup>', '</sup>')

    def _pagebreak_formatter(self, match, fullmatch):
        return '<div style="page-break-after:always;" class="print-page-break"></div><div class="page"></div>'

    def _inlinecode_formatter(self, match, fullmatch):
        return '<tt>%s</tt>' % fullmatch.group('inline')

    def _inlinecode2_formatter(self, match, fullmatch):
        return '<tt>%s</tt>' % fullmatch.group('inline2')

    def _feed_formatter(self, match, fullmatch):

        link = '<a href="/%s">%s</a>' % (match, match)
        return '<span class="at-name">@%s</span>' % link

    def _hashtag_formatter(self, match, fullmatch):

        link = '<a href="/search?q=%s">%s</a>' % (match, match)
        return '<span class="hashtag">#%s</span>' % link

    def _lhref_formatter(self, match, fullmatch):
        """
        """
        link = fullmatch.group('ltgt')
        label = fullmatch.group('label') or 'http:%s' % link
        format = fullmatch.group('lns')
        if format == 'wiki':
            return '<a href="/wiki/%s">%s</a>' % (link, label)
        elif format == 'http':
            return '<a href="http:%s" target="_blank">%s</a>' % (link, label)

    def _rel_formatter(self, match, fullmatch):
        link = fullmatch.group('ltgt')
        label = fullmatch.group('label') or 'http:%s' % link
        return '<a href="%s">%s</a>' % (match, label)

    def _parse_heading(self, match, fullmatch, shorten):
        match = match.strip()

        depth = min(len(fullmatch.group('hdepth')), 5)
        anchor = fullmatch.group('hanchor') or ''
        heading_text = match[depth+1:-depth-1-len(anchor)]

        rules = self.providers[0].get_replacement_rules()
        heading_text = re.sub(rules, self.replace, heading_text)

        heading = heading_text

        if anchor:
            anchor = anchor[1:]

        i = 1
        anchor_base = anchor
        while anchor in self._anchors:
            anchor = anchor_base + str(i)
            i += 1
        self._anchors[anchor] = True

        return (depth, heading, anchor)

    def _heading_formatter(self, match, fullmatch):
        self.close_paragraph()
        depth, heading, anchor = self._parse_heading(match, fullmatch, False)
        return '<h%d id="%s">%s</h%d>' % (depth, anchor, heading, depth)

    def _macro_formatter(self, match, fullmatch):
        """
        """
        name = fullmatch.group('macroname')
        if name.lower() == 'br':
            if self.in_table and not self.in_table_cell:
                return ''
            return '<br />'

        args = fullmatch.group('macroargs') or ''

        if not self.macros.has_key(name):
            return '<div class="wiki-error">Unknown Macro %s</div>' % name

        m = self.macros[name]()
        return m.execute(args)

    def _replacement_formatter(self, match, fullmatch):
        if self.context is None:
            return '<span class="wiki-error">No context provided</span>'

        assert issubclass(self.context.__class__, object)

        _object = fullmatch.group('replacement')
        _property = fullmatch.group('property') or None

        if hasattr(self.context, _object):
            obj = getattr(self.context, _object)
            if _property is None:
                return '%s' % obj
            if hasattr(obj, _property):
                rv = getattr(obj, _property)

                if isinstance(rv, datetime.date):
                    df = DateFormat(rv)
                    rv = df.format(get_format('SHORT_DATE_FORMAT'))

                if rv is None:
                    return ''
                return '%s' % rv
        return '<span class="wiki-error">Unsupported replacement field used `%s`.</span>' % _property

    def _list_formatter(self, match, fullmatch):
        """
        """
        ldepth = len(fullmatch.group('ldepth'))
        listid = match[ldepth]
        self.in_list_item = True
        class_ = start = None
        if listid in '-*':
            type_ = 'ul'
        else:
            type_ = 'ol'
            idx = '01iI'.find(listid)
            if idx >= 0:
                class_ = ('arabiczero', None, 'lowerroman', 'upperroman')[idx]
            elif listid.isdigit():
                start = match[ldepth:match.find('.')]
            elif listid.islower():
                class_ = 'loweralpha'
            elif listid.isupper():
                class_ = 'upperalpha'
        return self._set_list_depth(ldepth, type_, class_, start)

    def _get_list_depth(self):
        """Return the space offset associated to the deepest opened list."""
        return self._list_stack and self._list_stack[-1][1] or 0

    def _set_list_depth(self, depth, new_type, list_class, start):
        def open_list():
            self.close_paragraph()
            self._list_stack.append((new_type, depth))
            class_attr = (list_class and ' class="%s"' % list_class) or ''
            start_attr = (start and ' start="%s"' % start) or ''
            return '<'+new_type+class_attr+start_attr+'><li>'
        def close_list(tp):
            self._list_stack.pop()
            return '</li></%s>' % tp

        rv = ''
        # depending on the indent/dedent, open or close lists
        if depth > self._get_list_depth():
            rv += open_list()
        else:
            while self._list_stack:
                deepest_type, deepest_offset = self._list_stack[-1]
                if depth >= deepest_offset:
                    break
                rv += close_list(deepest_type)
            if depth > 0:
                if self._list_stack:
                    old_type, old_offset = self._list_stack[-1]
                    if new_type and old_type != new_type:
                        rv += close_list(old_type)
                        rv += open_list()
                    else:
                        if old_offset != depth: # adjust last depth
                            self._list_stack[-1] = (old_type, depth)
                        rv += '</li><li>'
                else:
                    rv += open_list()
        return rv

    def close_list(self):
        return self._set_list_depth(0, None, None, None)

    def simple_tag_handler(self, match, open_tag, close_tag):
        """
        """
        if self.has_open_tag((open_tag, close_tag)):
            return self.close_tag(close_tag)
        else:
            self.open_tag(open_tag, close_tag)
        return open_tag

    def has_open_tag(self, tag):
        """
        """
        return tag in self._open_tags

    def open_tag(self, open, close):
        """
        """
        self._open_tags.append((open, close))

    def close_tag(self, tag):
        """
        """
        tmp =  ''
        for i in xrange(len(self._open_tags)-1, -1, -1):
            tmp += self._open_tags[i][1]
            if self._open_tags[i][1] == tag:
                del self._open_tags[i]
                for j in xrange(i, len(self._open_tags)):
                    tmp += self._open_tags[j][0]
                break
        return tmp

    def handle_code_block(self, line):
        if line.strip() == WikiProvider.STARTBLOCK:
            self.in_code_block += 1
            if self.in_code_block == 1:
                self.code_processor = None
                self.code_buf = []
            else:
                self.code_buf.append(line)
                if not self.code_processor:
                    self.code_processor = WikiProcessor()
        elif line.strip() == WikiProvider.ENDBLOCK:
            self.in_code_block -= 1
            if self.in_code_block == 0 and self.code_processor:
                self.close_table()
                self.close_paragraph()
                if self.code_buf:
                    self.code_buf.append('')
                code_text = os.linesep.join(self.code_buf)
                processed = self.code_processor.process(code_text)
                return processed
            else:
                self.code_buf.append(line)
        elif not self.code_processor:
                self.code_buf.append(line)
                self.code_processor = WikiProcessor()
        else:
            self.code_buf.append(line)
        return ''

    def close_code_blocks(self):
        rv = ''
        while self.in_code_block > 0:
            rv += self.handle_code_block(WikiProvider.ENDBLOCK)
        return rv

    def format(self, text):
        """
        """
        for provider in self.providers:
            rules = provider.rules
        rv = ''
        for line in text.splitlines():
            if self.in_code_block or line.strip() == WikiProvider.STARTBLOCK:
                rv += self.handle_code_block(line)
                continue
            elif line[0:4] == '----':
                rv += self.close(['table','paragraph','list'])
                rv += '<hr class="wiki" />'+os.linesep
                continue
            elif line == '':
                # new paragraph
                rv += self.close(['table','paragraph', 'list'])
                continue

            line = line.replace('\t', ' '*8)

            if line.strip()[0:2] != '||':
                line += ' [[BR]]'

            skip_paragraph = False

            if self.in_center:
                skip_paragraph = True

            self.in_list_item = False

            result = re.sub(rules, self.replace, line)


            if self.in_center or not skip_paragraph:
                skip_paragraph = False

            #if not self.in_list_item and not self.in_table:
            #    rv += self.close_list()+'<br />'+os.linesep

            if self.in_table and line.strip()[0:2] != '||':
                rv += self.close_table()

#            self.in_list_item = False
#            result = re.sub(rules, self.replace, line)

#            if not self.in_list_item:
#                rv += self.close_list()

            if len(result) and not self.in_list_item and not self.in_center and not skip_paragraph:
                rv += self.open_paragraph()

            rv += result
            rv += self.close_table_row()
            rv += os.linesep
        rv += self.close(['table', 'paragraph', 'list', 'code_blocks'])
        return rv

    # Table

    def _last_table_cell_formatter(self, match, fullmatch):
        return ''

    def _table_cell_formatter(self, match, fullmatch):
        rv = self.open_table()
        rv += self.open_table_row()
        if self.in_table_cell:
            return rv + '</td><td>'
        else:
            self.in_table_cell = 1
            return rv + '<td>'

    def open_table(self):
        rv = ''
        if not self.in_table:
            rv += self.close_paragraph()
            rv += self.close_list()
            self.in_table = 1
            return '<table class="wiki">' + os.linesep
        return ''

    def open_table_row(self):
        if not self.in_table_row:
            rv = self.open_table()
            self.in_table_row = 1
            return rv + '<tr>'
        return ''

    def close_table_row(self):
        rv = ''
        if self.in_table_row:
            self.in_table_row = 0
            if self.in_table_cell:
                self.in_table_cell = 0
                rv += '</td>'

            rv += '</tr>'
        return rv

    def close_table(self):
        if self.in_table:
            rv = self.close_table_row()
            self.in_table = 0
            return rv + '</table>' + os.linesep
        return ''

class WikiProcessor(object):

    def __init__(self):
        """
        """

    def process(self, text):
        return '<pre class="wiki">%s</pre>' % text
