# Copyright (c) 2013 BaseSoft B.V.
# See LICENSE for details.
from django.utils.translation import ugettext as _

from basesoft.web.contrib.wiki.parser import Parser
from basesoft.web.core.forms.handler import FormValidator
from basesoft.web.core.input.validation import plugins
from basesoft.web.contrib.wiki.controllers import WikiController
from basesoft.web.contrib.i18n.controllers import I18nController
from basesoft.web.contrib.i18n.models import Locale
from basesoft.web.core.exceptions import PublicUserException

class PreviewObject(object):
    def __init__(self, prefix, available_replacements=None):
        self.prefix = prefix.capitalize()
        self.available_replacements = available_replacements

    def __getattr__(self, name):
        if self.available_replacements is not None:
            if name not in self.available_replacements:
                raise PublicUserException(_('Unsupported replacement field used: %s' % name))
        return '%s %s' % (self.prefix, name.capitalize())

class PreviewContext(object):

    def __init__(self, available_replacements=None):
        self.available_replacements = available_replacements

    def __getattr__(self, name):
        if self.available_replacements is not None:
            if not self.available_replacements.has_key(name):
                raise PublicUserException(_('Unsupported replacement object used: %s.' % name))
            return PreviewObject(name, self.available_replacements[name])
        return PreviewObject(name)

class WikiHandler:

    def __init__(self, request, **kwargs):
        self.request = request
        self.sa_session = request.Session()

        self.account_id = None
        if request.session.has_key('user_id'):
            self.account_id = request.session['user_id']

        self._i18n_controller = I18nController(self.sa_session, account_id=self.account_id, request=request)


    def wiki2html(self, wiki_str, available_replacements=None):
        ar = {}
        if available_replacements is not None:
            for rf in available_replacements:
                ar[rf['key']] = []
                if not rf.has_key('submenu'):
                    continue
                ar[rf['key']] = [x['key'] for x in rf['submenu']]

        p = Parser()
        p.set_context(PreviewContext(ar))
        return p.render_to_html(wiki_str)



class AbstractWikiHandler(object):
    """
    Helper class for loading/saving wiki objects.
    Child class must set self._controller which is an extension of the WikiController
    """

    def __init__(self):
        raise Exception('Abstract class')

    def get_locales(self):
        """
        returns all wiki editable locales for the current user
        """
        i18n_controller = I18nController(self.sa_session, account_id=self.account_id)
        return [{
            'label': locale.name,
            'value': locale.id,
            'id': locale.id
        } for locale in i18n_controller.get_domain_locales(availability=Locale.AVAILABLE_FOR_COMPOSE)]

    def validate_wiki(self, args):
        """
        validates wiki form in wiki_plugin-debug.js
        """
        form_handler = FormValidator({
            'fallback_locale_id': [(plugins.Required, {})],
            'locale_id': [(plugins.Required, {})],
            'title': [(plugins.Required, {})],
            'content': [(plugins.Required, {})],
        })

        return form_handler.validate(args)

    def load_wiki(self, instance_id, locale_id):
        """
        loads raw wiki content for the given locale
        """
        assert isinstance(self._controller, WikiController)
        return self._controller.load_wiki(instance_id, locale_id)

    def load_wiki_with_fallback(self, instance_id):
        """
        loads parsed wiki content and takes care of fallback handling
        """
        assert isinstance(self._controller, WikiController)
        locale = self.sa_session.query(Locale).filter(Locale.locale==self.request.session['config']['basesoft.web.contrib.locale']).one()
        return self._controller.load_wiki_with_fallback(instance_id, locale)

    def validate_wiki_translate(self, args):
        """
        validator for the wiki translate panel
        """
        form_handler = FormValidator({
            'locale_id': [(plugins.Required, {})],
            'title': [(plugins.Required, {})],
            'content': [(plugins.Required, {})],
        })

        return form_handler.validate(args)

    def load_wiki_translate(self, instance_id):
        """
        loads wiki for translate interface
        """
        assert isinstance(self._controller, WikiController)
        return self._controller.load_wiki_translate(instance_id)

    def save_wiki_revision(self, instance_id, state):
        """
        saves wiki text (only the title/content)
        """
        assert isinstance(self._controller, WikiController)
        self._controller.save_wiki_revision(instance_id, state)

handlers = (WikiHandler,)

