# Copyright (c) 2009 BaseSoft B.V.
# See LICENSE for details.

# classes used for storing the localization settings for all the language codes we use

import sys

from django.utils.translation import ugettext_lazy as _
from django.utils import simplejson as json

from basesoft.web.core.jsonrpc.dispatcher import JsonRpcJSONEncoder

class default:
    """
    List of all the constants.
    See also: http://developer.yahoo.com/yui/examples/calendar/germany.html
    Note: all of the constants that are upper case will automatically be added in the environment,
    in BASESOFT.env.localization
    """

    # ---------------- settings that YUI requires (for calendar for example) ---------------- #

    # The setting that determines which length of month labels should be used. Possible values are 'short' and 'long'
    LOCALE_MONTHS = 'short'
    # The setting that determines which length of weekday labels should be used. Possible values are '1char', 'short', 'medium', and 'long'
    LOCALE_WEEKDAYS = 'short'
    # ==================== HACK ======================= #
    # we use a different gettext value for the short months and day, not the english values, because they might conflict
    # with other handlers;
    # eg: 'May' is also the full month and the medium month.
    # The short month labels for the current locale
    MONTHS_SHORT = [_('Jan_January_months_short'),
                    _('Feb_February_months_short'),
                    _('Mar_March_months_short'),
                    _('Apr_April_months_short'),
                    _('May_May_months_short'),
                    _('Jun_June_months_short'),
                    _('Jul_July_months_short'),
                    _('Aug_August_months_short'),
                    _('Sep_September_months_short'),
                    _('Oct_October_months_short'),
                    _('Nov_November_months_short'),
                    _('Dec_December_months_short')]
    # The long month labels for the current locale
    MONTHS_LONG = [_('January_months_long'),
                   _('February_months_long'),
                   _('March_months_long'),
                   _('April_months_long'),
                   _('May_months_long'),
                   _('June_months_long'),
                   _('July_months_long'),
                   _('August_months_long'),
                   _('September_months_long'),
                   _('October_months_long'),
                   _('November_months_long'),
                   _('December_months_long')]
    # The 1-character weekday labels for the current locale
    WEEKDAYS_1CHAR = [_('S_Sunday_weekdays_1char'),
                      _('M_Monday_weekdays_1char'),
                      _('T_Tuesday_weekdays_1char'),
                      _('W_Wednesday_weekdays_1char'),
                      _('T_Thursday_weekdays_1char'),
                      _('F_Friday_weekdays_1char'),
                      _('S_Saturday_weekdays_1char')]
    # The short weekday labels for the current locale
    WEEKDAYS_SHORT = [_('Su_Sunday_weekdays_short'),
                      _('Mo_Monday_weekdays_short'),
                      _('Tu_Tuesday_weekdays_short'),
                      _('We_Wednesday_weekdays_short'),
                      _('Th_Thursday_weekdays_short'),
                      _('Fr_Friday_weekdays_short'),
                      _('Sa_Saturday_weekdays_short')]
    # The medium weekday labels for the current locale
    WEEKDAYS_MEDIUM = [_('Sun_Sunday_weekdays_medium'),
                       _('Mon_Monday_weekdays_medium'),
                       _('Tue_Tuesday_weekdays_medium'),
                       _('Wed_Wednesday_weekdays_medium'),
                       _('Thu_Thursday_weekdays_medium'),
                       _('Fri_Friday_weekdays_medium'),
                       _('Sat_Saturday_weekdays_medium')]
    # The long weekday labels for the current locale
    WEEKDAYS_LONG = [_('Sunday'), _('Monday'), _('Tuesday'), _('Wednesday'), _('Thursday'), _('Friday'), _('Saturday')]
    # The value used to delimit individual dates in a date string passed to various Calendar functions
    DATE_DELIMITER = ','
    # The value used to delimit date fields in a date string passed to various Calendar functions
    DATE_FIELD_DELIMITER = '/'
    # The value used to delimit date ranges in a date string passed to various Calendar functions
    DATE_RANGE_DELIMITER = '-'
    # The position of the month in a month/day/year date string. Default: 1
    MDY_MONTH_POSITION = 1
    # The position of the day in a month/day/year date string. Default: 2
    MDY_DAY_POSITION = 2
    # The position of the year in a month/day/year date string. Default: 3
    MDY_YEAR_POSITION = 3
    # The position of the month in a month/day date string. Default: 1
    MD_MONTH_POSITION = 1
    # The position of the day in a month/day date string. Default: 2
    MD_DAY_POSITION = 2
    # The position of the month in a month/year date string. Default: 1
    MY_MONTH_POSITION = 1
    # The position of the year in a month/year date string. Default: 2
    MY_YEAR_POSITION = 2
    # what's the first day of the week 0-Sunday, 1-Monday, etc.
    START_WEEKDAY = 0

    # -------------------- additional constants -------------------

    # if true, show dates like 01/05/2009, if false, 1/5/2009
    SHOW_DATE_LEADING_ZERO = True

    # Decimal separator
    DECIMAL_SEPARATOR = '.'
    # Thousand separator
    THOUSAND_SEPARATOR = ','

    def get_settings(self):
        """
        Return the settings as a dictionary with settings as keys and their values
        Eg. {'START_WEEKDAY':1, ...}
        """
        result = {}
        # we check each item from our class
        for p in dir(self):
            # check if it's one of our upper case constants that needs to be added in the env
            if p == p.upper():
                result[p] = getattr(self, p)

        return result

class en(default):
    """
    Base English localization settings.
    Everything that's different from default settings should be overriden here
    """
    pass

class en_gb(en):
    """
    British English localization settings.
    Everything that's different from the base English settings should be overriden here
    """
    MDY_MONTH_POSITION = 2
    MDY_DAY_POSITION = 1
    MDY_YEAR_POSITION = 3

class nl(default):
    """
    Base Dutch localization settings.
    Everything that's different from the default settings should be overriden here
    """
    DATE_FIELD_DELIMITER = '-'
    MDY_MONTH_POSITION = 2
    MDY_DAY_POSITION = 1
    MDY_YEAR_POSITION = 3
    START_WEEKDAY = 1 # Monday is the first day of the week for nl
    DECIMAL_SEPARATOR = ','
    THOUSAND_SEPARATOR = '.'

class ro(default):
    """
    Base Romanian localization settings.
    Everything that's different from the default settings should be overriden here
    """
    DATE_FIELD_DELIMITER = '.'
    MDY_MONTH_POSITION = 2
    MDY_DAY_POSITION = 1
    MDY_YEAR_POSITION = 3
    START_WEEKDAY = 1 # Monday is the first day of the week for ro
    DECIMAL_SEPARATOR = ','
    THOUSAND_SEPARATOR = ''

class de(default):
    """
    Base German localization settings.
    Everything that's different from the default settings should be overriden here
    """
    DATE_FIELD_DELIMITER = '.'
    MDY_MONTH_POSITION = 2
    MDY_DAY_POSITION = 1
    MDY_YEAR_POSITION = 3
    START_WEEKDAY = 1 # Monday is the first day of the week for de
    DECIMAL_SEPARATOR = ','
    THOUSAND_SEPARATOR = '.'

class fr(default):
    """
    Base French localization settings.
    """
    MDY_MONTH_POSITION = 2
    MDY_DAY_POSITION = 1
    MDY_YEAR_POSITION = 3
    START_WEEKDAY = 1 # Monday is the first day of the week for fr
    DECIMAL_SEPARATOR = ','
    THOUSAND_SEPARATOR = ' '

class it(default):
    """
    Base Italian localization settings.
    """
    MDY_MONTH_POSITION = 2
    MDY_DAY_POSITION = 1
    MDY_YEAR_POSITION = 3
    START_WEEKDAY = 1 # Monday is the first day of the week for it
    DECIMAL_SEPARATOR = ','
    THOUSAND_SEPARATOR = '.'

def get(language_code=None):
    """
    Return the class that contains the localization settings that we want based on the language_code.
    If we don't have defined the class for the given language_code, try to get the 'base' one (in case we provide
    'en-us' for example, and we only have 'en' defined), or if we don't have that one, get the default settings
    @param language_code: the language code we want the settings for. can be in format 'en', 'en-gb', 'en_GB', etc
        If language_code is null, get the default settings
    @return: the class that contains our settings
    """

    current_module = sys.modules[__name__]
    class_to_find = language_code.lower().replace('-', '_') # normalize the class name

    # check if our module has the class we're looking for
    cl = getattr(current_module, class_to_find, None)
    if cl is None:
        # we didn't have the class we're looking for, so check for the base class
        split_locale = class_to_find.split('_')
        # look for the base, if not found, return the default class
        cl = getattr(current_module, split_locale[0], default)

    return cl

def get_json(language_code=None):
    """
    Return the settings for the current language_code, json encoded
    @param language_code: the language code we want the settings for. can be in format 'en', 'en-gb', 'en_GB', etc
        If language_code is null, get the default settings
    @return: the dictionary with the settings, json encoded
    """
    return json.dumps(get(language_code)().get_settings(), ensure_ascii=False, cls=JsonRpcJSONEncoder)
