# Copyright (c) 2010 BaseSoft B.V.
# See LICENSE for details.

from django.utils.translation import ugettext as _

from basesoft.web.core.util.custom_query import CustomQuery, CustomQueryField, CustomQueryModel
from basesoft.web.contrib.account.models import Account, GroupAccount
from basesoft.web.contrib.contact.models import Person
from basesoft.web.contrib.license.models import License, Domain, Group
from basesoft.web.contrib.session.models import HTTPSession

from basesoft.web.core.models import Log
from sqlalchemy import func, case

class LogCustomQuery(CustomQuery):
    permissions = ['view_log']

    def tables(self):
        return (
            CustomQueryModel(Log),
            CustomQueryModel(Account),
            CustomQueryModel(Person, onclause=Account.person_id==Person.id, use_outerjoin=True),
        )

    def fields(self):
        return (
            CustomQueryField(Log.id, _('ID'), key='log_id'),
            CustomQueryField(Account.name, _('User'), is_filter=True, key='account_name'),
            CustomQueryField(func.concat(case([(Person.first_name!=None, Person.first_name)], else_=""),
                " ",
                case([(Person.middle_name!=None, Person.middle_name)], else_=""),
                " ",
                case([(Person.last_name!=None, Person.last_name)], else_=""),
                ), _('Name'), key='full_name'),
            CustomQueryField(Log.message, _('Message'), key='log_message'),
            CustomQueryField(Account.last_login, _('Last login'), key='last_login'),
            CustomQueryField(Log.ip, _('IP'), key='log_ip'),
            CustomQueryField(Log.level, _('Level'), key='log_level'),
            CustomQueryField(Log.logger_name, _('Logger'), key='logger_name'),
            CustomQueryField(Log.created, _('Created')),
        )

    def set_where(self):
        # TODO: ticket #1496: if user has permission view_unbound_log, the log entries that have no group also need
        # to be shown

        #filter, fo = self.get_query_filter_on_privileges(Log, LogGroup, Log.PRIV_VIEW)
        #if self.has_permissions('view_unbound_log'):
        #    filter = ... ?
        #self.where = filter
        pass


class LoginOverviewCustomQuery(CustomQuery):
    permission = ['view_login_overview']

    def validate_access(self):
        self.assert_any_permissions('view_domain', 'view_license', 'view_all_domains')

    def tables(self):
        return (
            CustomQueryModel(Account),
            CustomQueryModel(GroupAccount, onclause=GroupAccount.account_id==Account.id),
            CustomQueryModel(Group, onclause=GroupAccount.group_id==Group.id),
            CustomQueryModel(License, onclause=Group.license_id==License.id),
            CustomQueryModel(Domain, onclause=License.domain_id==Domain.id),
            CustomQueryModel(HTTPSession, onclause=Account.id==HTTPSession.account_id, use_outerjoin=True),
            CustomQueryModel(Person, onclause=Account.person_id==Person.id, use_outerjoin=True),
        )

    def fields(self):
        return (
            CustomQueryField(Account.name, _('User'), is_filter=True, key='account_name'),
            CustomQueryField(func.concat(case([(Person.first_name!=None, Person.first_name)], else_=""),
                " ",
                case([(Person.middle_name!=None, Person.middle_name)], else_=""),
                " ",
                case([(Person.last_name!=None, Person.last_name)], else_=""),
                ), _('Name'), key='full_name'),
            CustomQueryField(HTTPSession.logged_in, _('Login date'), key='login_date'),
            CustomQueryField(HTTPSession.logged_out, _('Logout date'), key='logout_date'),
            CustomQueryField(HTTPSession.last_updated, _('Duration'), key='duration'),
            CustomQueryField(HTTPSession.expire_date, _('Expire date'), key='expire_date'),
            CustomQueryField(HTTPSession.session_key, _('Status'), key='status'),
            CustomQueryField(Group.id, _('Group'), in_select=False, is_filter=True, filter_options=Group.name),
            CustomQueryField(Group.name, _('Group')),
            CustomQueryField(License.name, _('License'), is_group_by=True),
            CustomQueryField(HTTPSession.ip, _('IP'), key='log_ip'),
        )

    def set_where(self):
        if self.has_permissions('view_all_domains'):
            return None
        dg = self.get_default_group()
        if self.has_permissions('view_domain'):
            # filter on current domain
            self.where = Domain.id==dg.license.domain.id

        licenses = []
        for g in self.get_active_groups():
            if self.has_permissions('view_license', group_id=g.id):
                licenses.append(g.license.id)

        if len(licenses) > 0:
            self.where = License.id.in_(licenses)

custom_queries = (LogCustomQuery, LoginOverviewCustomQuery)
