# Copyright (c) 2008 BaseSoft B.V.
# See LICENSE for details.

"""
Module for creating form behaviours that can be used for validating different
types of input, including form fields
"""

from django.utils.translation import ugettext as _
from django.forms.util import ValidationError
import re
from basesoft.web.core.input.validation.share.iban import iban_validator
from basesoft.web.core.input.validation.share.vat import check_vies, check_vat

class IncorrectPluginConfiguration(Exception):
    """The plugin received incorrect parameters"""
    pass

class Validation:

    debug = False

    def __init__(self, value):
        self._handles = {}
        self._value = value

    def check(self):
        pass

class Required(Validation):
    """
    Check if the field has a value or not
    """

    def __init__(self, *args, **kwargs):
        Validation.__init__(self, *args, **kwargs)
        self._handles['required_value'] = _('A value is required')

    def check(self, params={}):
        if self._value == 0:
            return
        if not self._value: raise ValidationError(self._handles['required_value'])

class RequiredCheckbox(Validation):
    def __init__(self, *args, **kwargs):
        Validation.__init__(self, *args, **kwargs)
        self._handles['required_value'] = _('A value is required')

    def check(self, params={}):
        if not self._value: raise ValidationError(self._handles['required_value'])

class RequiredEULA(RequiredCheckbox):
    def __init__(self, *args, **kwargs):
        Validation.__init__(self, *args, **kwargs)
        self._handles['required_value'] = _('You need to accept the EULA before you can register.')

class RestrictedLength(Validation):
    """
    Check the value and make sure they have the specified length
    """

    def __init__(self, *args, **kwargs):
        Validation.__init__(self, *args, **kwargs)
        self._handles['too_short'] = _('Value too short')
        self._handles['too_long'] =  _('Value too long')

    def check(self, params={}):
        if not self._value: return

        if params.has_key('min') and not type(params['min']) == int:
            raise IncorrectPluginConfiguration('The min parameter needs to be an integer')

        if params.has_key('max') and not type(params['max']) == int:
            raise IncorrectPluginConfiguration('The max parameter needs to be an integer')

        # check against the minimum length only if it exists
        if params.has_key('min') and (len(self._value) < params['min']):
            raise ValidationError(self._handles['too_short'])

        # check against the maximum length only if it exists
        if params.has_key('max') and (len(self._value) > params['max']):
            raise ValidationError(self._handles['too_long'])

class Regex(Validation):
    """
    Check strings against a given regular expression.

    The compiled_regex class variable needs to be defined in the subclasses that
    extend this class, otherwise an IncorrectPluginConfiguration is raised.
    """

    # need to define the compiled_regex in the subclasses
    compiled_regex = None

    def __init__(self, *args, **kwargs):
        Validation.__init__(self, *args, **kwargs)
        self._handles['no_match'] = _('Value doesn\'t match the regular expression')

    def check(self, params={}):
        if not self._value: return
        if type(self._value) != unicode:
            self._value = unicode(self._value, 'utf-8')

        try:
            if not self.compiled_regex.match(self._value):
                raise ValidationError(self._handles['no_match'])
        except AttributeError:
            # raised if compiled_regex is None
            raise IncorrectPluginConfiguration('The plugin doesn\'t have the correct regular expression configured')

class LimitedRegexString(Regex, RestrictedLength):
    """
    Generic class for checking strings.
    Can be extended if other rules need to be enforced.

    compiled_regex needs to be defined in the subclasses, otherwise an
    IncorrectPluginConfiguration is raised
    """

    def __init__(self, *args, **kwargs):
        RestrictedLength.__init__(self, *args, **kwargs)
        self._handles['no_match'] = _('Not a valid value')

    def check(self, params={}):
        if not self._value: return

        Regex.check(self, params)
        RestrictedLength.check(self, params)

class Alphanumeric(Regex):
    """Plugin used for validating values that should only contain alphanumeric and _"""

    # \w If UNICODE is set, this will match the characters [0-9_] plus whatever is classified as alphanumeric in the
    # Unicode character properties database.
    compiled_regex = re.compile(r"^[\w]+$", re.UNICODE)

    def __init__(self, *args, **kwargs):
        Regex.__init__(self, *args, **kwargs)
        self._handles['no_match'] =  _('Value can only contain letters, numbers and underscores')

class IQPortalName(LimitedRegexString):
    """Plugin used for validating person names"""

    # \w If UNICODE is set, this will match the characters [0-9_] plus whatever is classified as alphanumeric in the
    # Unicode character properties database. On top of that we allow some more chars including apostrophe.
    compiled_regex = re.compile(ur'^[\w \.\(\)\-\/\'\u2019]+$', re.UNICODE)

    def __init__(self, *args, **kwargs):
        LimitedRegexString.__init__(self, *args, **kwargs)
        self._handles['no_match'] =  _('Not a valid name')
        self._handles['too_short'] = _('Name too short')
        self._handles['too_long'] =  _('Name too long')

class Username(LimitedRegexString):
    """Plugin used for validating usernames"""

    compiled_regex = re.compile(r"^[a-zA-Z0-9_]+$")

    def __init__(self, *args, **kwargs):
        LimitedRegexString.__init__(self, *args, **kwargs)
        self._handles['no_match'] =  _('Your account name can only contain letters, numbers and underscores')
        self._handles['too_short'] = _('The provided account name is too short')
        self._handles['too_long'] =  _('The provided account name is too long')

class Email(Regex):
    """Plugin used for validating an email address"""

    compiled_regex = re.compile(r"^[a-zA-Z0-9'._+-]+@[a-zA-Z0-9.-]+\.[a-zA-Z]{2,4}$")

    def __init__(self, *args, **kwargs):
        Regex.__init__(self, *args, **kwargs)
        self._handles['no_match'] =  _('Not a valid email address')

class URL(Regex):

    compiled_regex = re.compile(r"^http://.+")

    def __init__(self, *args, **kwargs):
        Regex.__init__(self, *args, **kwargs)
        self._handles['no_match'] =  _('Not a valid URL (must start with http://)')


class Date(Regex):
    """
    Plugin for the date fields
    """

    compiled_regex = re.compile(r"^(19|20)\d\d-(0[1-9]|1[012])-(0[1-9]|[12][0-9]|3[01])$")

    def __init__(self, *args, **kwargs):
        Regex.__init__(self, *args, **kwargs)
        self._handles['no_match'] = _('Not a valid date format')

class Number(Validation):
    """
    Plugin for number fields
    """
    def __init__(self, *args, **kwargs):
        Validation.__init__(self, *args, **kwargs)
        self._handles['no_match'] = _('Not a valid number')

    def check(self, params={}):
        if self._value == '' or self._value is None: # we don't raise anything if we have an empty value. we have required plugins for that
            return

        try:
            v = float(self._value)
        except ValueError:
            raise ValidationError(self._handles['no_match'])

        inclusive = False # if this value is True we allow MIN and MAX values
        if params.has_key('inclusive'):
            inclusive = params['inclusive']

        if not inclusive:

            if params.has_key('min_value'):
                if v <= params['min_value']:
                    raise ValidationError(_('Value needs to be larger than %(min_value)s.' % {'min_value': params['min_value']}))
            if params.has_key('max_value'):
                if v >= params['max_value']:
                    raise ValidationError(_('Value needs to be lower than %(max_value)s.' % {'max_value': params['max_value']}))
            return

        if params.has_key('min_value'):
            if v < params['min_value']:
                raise ValidationError(_('Value needs to be larger than %(min_value)s.' % {'min_value': params['min_value']}))

        if params.has_key('max_value'):
            if v > params['max_value']:
                raise ValidationError(_('Value needs to be lower than %(max_value)s.' % {'max_value': params['max_value']}))


class Integer(Validation):
    """
    Plugin for integer fields
    """
    def __init__(self, *args, **kwargs):
        Validation.__init__(self, *args, **kwargs)
        self._handles['no_match'] = _('Not a valid integer')
        self._handles['not_positive'] = _('Not a postive integer')

    def check(self, params={'unsigned': False}):
        if self._value == '' or self._value is None: # we don't raise anything if we have an empty value. we have required plugins for that
            return

        try:
            v = int(self._value)
        except ValueError:
            raise ValidationError(self._handles['no_match'])

        if str(v) != str(self._value):
            raise ValidationError(self._handles['no_match'])

        unsigned = False
        if params.has_key('unsigned'):
            unsigned = params['unsigned']

        allow_zero = True
        if params.has_key('allow_zero'):
            allow_zero = params['allow_zero']

        _min = 0
        if not allow_zero:
            _min = 1

        if unsigned and v < _min:
            raise ValidationError(self._handles['not_positive'])

        if params.has_key('max_value'):
            if v > params['max_value']:
                raise ValidationError(_('Value should be smaller than or equal to %(max_value)s' % {'max_value': params['max_value']}))

class Country(Validation):
    """
    Plugin for country fields. Checks in db for all countries.
    The check method takes as param the country value format:
    name, iso_3166_a2, iso_3166_a3 or db id (default is name)
    """
    def __init__(self, *args, **kwargs):
        Validation.__init__(self, *args, **kwargs)
        self._handles['no_match'] = _('Not a valid country')

    def check(self, params={"format": "name", "db_session": None, 'domain': None}):
        """
        Checks the db for valid country
        """

        assert params["format"] in ["name", "iso_3166_a2", "iso_3166_a3", "id"]

        db_session = params["db_session"]

        from basesoft.web.contrib.i18n.models import Country, DomainCountry
        country = db_session.query(Country).filter(getattr(Country, params["format"])==self._value).first()

        domain = None
        if params.has_key('domain'):
            domain = params['domain']
        if country is None and params["format"] == 'name' and domain is not None:
            from sqlalchemy.sql import and_
            country = db_session.query(DomainCountry).filter(and_(DomainCountry.name==self._value,
                    DomainCountry.domain==domain, DomainCountry.active==1)).first()

        if country is None:
            raise ValidationError(self._handles['no_match'])

class Gender(Validation):
    """
    Plugin for gender fields.
    """
    def __init__(self, *args, **kwargs):
        Validation.__init__(self, *args, **kwargs)
        self._handles['no_match'] = _('Not a valid gender')

    def check(self, params={}):
        if self._value not in [None, '']:
            from basesoft.web.contrib.contact.models import Person

            gender_ok = False
            for gender in Person.gender_options:
                if self._value.lower() in Person.gender_map[gender]:
                    gender_ok = True
                    break
            if not gender_ok:
                raise ValidationError(self._handles['no_match'])

class StrongPassword(Validation):
    """
    Plugin for validating a strong password using python's cracklib.
    """

    def __init__(self, *args, **kwargs):
        Validation.__init__(self, *args, **kwargs)

    def check(self, params={}):
        import cracklib
        try:
            cracklib.FascistCheck(self._value)
        except ValueError, e:
            raise ValidationError(_(e.message))


class IBAN(Validation):
    def check(self, params={}):
        if self._value in [None, '']:
            return
        iban_validator(self._value)

class SWIFTBIC(Validation):
    """
    Original: https://github.com/benkonrath/django-iban
    """
    def check(self, params={}):
        """ Validation for ISO 9362:2009 (SWIFT-BIC). """
        if self._value in [None, '']:
            return
        import string
        value = self._value

        # Length is 8 or 11.
        swift_bic_length = len(value)
        if swift_bic_length != 8 and swift_bic_length != 11:
            raise ValidationError(_(u"A SWIFT-BIC is either 8 or 11 characters long."))

        # First 4 letters are A - Z.
        institution_code = value[:4]
        for x in institution_code:
            if x not in string.uppercase:
                raise ValidationError(_(u"%s is not a valid SWIFT-BIC Institution Code.") % institution_code)

        from basesoft.web.contrib.i18n.models import Country
        db_session = params["db_session"]
        OFFICIAL_COUNTRIES = [country.iso_3166_a2 for country in db_session.query(Country).all()]

        # Letters 5 and 6 consist of an ISO 3166-1 alpha-2 country code.
        country_code = value[4:6]
        if country_code not in OFFICIAL_COUNTRIES:
            raise ValidationError(_(u"%s is not a valid SWIFT-BIC Country Code.") % country_code)

class VAT(Validation):
    def check(self, params={}):
        if self._value in [None, '']:
            return
        if not re.match('^[\w]+$', self._value):
            raise ValidationError(_(u"The VAT number can only alphanumeric characters (no spaces)"))
        if not check_vies(self._value):
            raise ValidationError(_(u"%s is not a valid VAT number.") % self._value)

class VATOffline(Validation):
    def check(self, params={}):

        if self._value in [None, '']:
            return
        if not re.match('^[\w]+$', self._value):
            raise ValidationError(_(u"The VAT number can only alphanumeric characters (no spaces)"))
        if not check_vat(self._value):
            raise ValidationError(_(u"%s is not a valid VAT number.") % self._value)

