# Copyright (c) 2010 BaseSoft B.V.
# See LICENSE for details.

from django.utils.translation import ugettext
from sqlalchemy.orm.collections import InstrumentedList

class PrivilegeObject(object):
    """
    Identifying object for basesoft Privilege Objects.
    @DynamicAttrs
    """
    PRIV_VIEW = 1
    PRIV_UPDATE = 2
    PRIV_DELETE = 4 # mark deleted.
    PRIV_ARCHIVE = 8 # mark archived.
    PRIV_DELEGATE_PRIVILEGES = 16

    DELETION_GRACE_PERIOD = 30 # in days

    all_privileges = (PRIV_VIEW, PRIV_UPDATE, PRIV_ARCHIVE, PRIV_DELETE, PRIV_DELEGATE_PRIVILEGES)
    no_privileges = 0

    mark_as_deleted_permission = 'mark_as_deleted'
    mark_as_archived_permission = 'mark_as_archived'

    archived = None
    deleted = None

    _name_property = 'id' # fallback to ID as all priv objects have that.

    def on_mark_as_deleted(self):
        pass

    def on_mark_as_archived(self):
        pass

    def get_privileges(self):
        return [
            (self.PRIV_VIEW, ugettext('View privileges')),
            (self.PRIV_UPDATE, ugettext('Update privileges')),
            (self.PRIV_DELETE, ugettext('Mark as deleted privileges')),
            (self.PRIV_ARCHIVE, ugettext('Mark as archived privileges')),
            (self.PRIV_DELEGATE_PRIVILEGES, ugettext('Privilege delegation'))
        ]

    @classmethod
    def get_name_property(cls):
        return getattr(cls, cls._name_property)

    def cleanup_garbage(self):
        """
        Custom cleanup method for Privileged objects.
        """
        pass

    def clone(self, clone_children=None, reference_children=None, exclude=None):
        """
        This will clone a Privileged Object.
        """
        # exclude some attributes we never want to clone.
        default_exclude = ['_sa_instance_state', 'id', 'deleted', 'archived', 'created', 'last_updated', 'groups']
        if exclude is None:
            exclude = []
        exclude = exclude + default_exclude
        if clone_children is None:
            clone_children = {}
        if reference_children is None:
            reference_children = []

        cls = self.__class__
        new = cls()
        for attr, value in self.__dict__.iteritems():
            if attr in exclude:
                continue
            if not isinstance(value, InstrumentedList):
                setattr(new, attr, value)
                continue

        # if we have any children we want to reference.
        for name in reference_children:
            setattr(new, name, getattr(self, name))

        for name, config in clone_children.iteritems():
            if config.has_key('exclude'):
                exc = exclude + config['exclude']
            else:
                exc = exclude
            for original_child in getattr(self, name):
                # clone children of Privilege Object type.
                if issubclass(original_child.__class__, PrivilegeObject):
                    new_child = original_child.clone(exclude=exc)
                    getattr(new, name).append(new_child)
                    continue
                new_child = original_child.__class__()
                # TODO: deep copy #3232
                for a, v in original_child.__dict__.iteritems():
                    if a in exc:
                        continue
                    setattr(new_child, a, v)
                getattr(new, name).append(new_child)

        go, field = self.get_group_object()
        if not isinstance(self.groups[0], go):
            return new
        # clone the group objects for privileges.
        for original_go in self.groups:
            new_go = go()
            new_go.group = original_go.group
            for a, v in original_go.__dict__.iteritems():
                if a in exclude:
                    continue
                setattr(new_go, a, v)
            setattr(new_go, field, new)

        return new

    @staticmethod
    def get_privilege_filter(filter, fo, account, group_obj):
        """
        fixme: doc
        """
        return filter, fo

class ComposePrivilegeObject(PrivilegeObject):
    MODE_COMPOSE = 1

    # update when a new mode is added here
    MAX_MODE_BIT = 1

class Log(PrivilegeObject):
    """
    @DynamicAttrs
    """

    mark_as_deleted_permission = 'delete_log'
    mark_as_archived_permission = 'update_log'

    def get_group_object(self):
        return (LogGroup, 'log')

    def __repr__(self):
        return '<Log id="%s">' % self.id

class LogGroup(object):
    """
    Model representing the link between log entries and groups. Used for permissions.
    @DynamicAttrs
    """
    def __repr__(self):
        return '<LogGroup>'

class SchedulerTask(object):
    """
    @DynamicAttrs
    """
