# Copyright (c) 2010 BaseSoft B.V.
# See LICENSE for details.
import os

from django.utils.translation import get_language
from django.conf import settings
from twisted.python.reflect import namedAny

from basesoft import logging
from basesoft.web.core import L10n_settings
from basesoft.web.contrib.session.controllers import SessionController
from basesoft.web.core.util.controllers import PermissionController,\
    LicenseConfigController
from basesoft.web.contrib.iconbar.controllers import IconBarController
from basesoft import logging
from basesoft.web.contrib.manual.controllers import ManualController
from basesoft.web.contrib.branding.controllers import SiteController,\
    LicenseThemeController
from basesoft.util.virtual_date import today
from basesoft.web.core.forms.handler import FormValidator
from basesoft.web.core.input.validation import plugins
from basesoft.web.contrib.i18n.models import Country

class LoginOverviewRPCHandler:
    def __init__(self, request, **kwargs):
        self.sa_session = request.Session()
        self.session_controller = SessionController(self.sa_session)
        self.permission_controller = PermissionController(self.sa_session, request.session['user_id'])

    def kill_session(self, session_key):
        """
        Kill the session with the given key
        """
        # can't assert permissions in the session controller, because it doesn't inherit the abstract controller
        self.permission_controller.assert_permissions('kill_session')
        self.session_controller.kill_sessions(session_key)

def _get_user_notifications(request):
    """
    Returns a list of configs used for displaying growls on the client side if needed.
    eg. if a user needs to flush a dataset
    """
    growls = []

    # don't check anything for guests
    if not request.session.has_key('user_id'):
        return growls

    # the list of applications that we check for growls modules
    apps = settings.INSTALLED_APPS

    # see if there is a growls module in the app
    for app in apps:
        try:
            growls_module = namedAny(app + '.growls')
        except AttributeError:
            # if we do not have an growls module then the app doesn't need to check for any notifications
            continue

        if hasattr(growls_module, 'growlers'):
            # this is the list of classes that the app uses to check for problems the user needs to be notified of
            growlers = growls_module.growlers
            for growler_class in growlers:
                g = growler_class(request)
                growls.extend(g.get_growls())

    return growls

def _get_files(path, prefix=''):
    files = []
    for f in os.listdir(path):
        if os.path.isfile(os.path.join(path, f)):
            files.append(os.path.join(prefix, f))
            continue
        files.extend(_get_files(os.path.join(path, f), os.path.join(prefix, f)))
    return files

def _get_theme_icons(theme):
    rv = {}

    media_root = settings.STATIC_MEDIA_ROOT

    # icons should be placed in the appropriate app under /media/img/$theme/(16x16|22x22|32x32|75x75)/$name
    apps = settings.INSTALLED_APPS
    for app in apps:
        rv[app] = {}
        if settings.SERVE_STATIC_MEDIA:
            module = __import__(app, {}, {}, [''])
            media_path = os.path.join(module.__path__[0], 'media')
        else:
            media_path = os.path.join(media_root, app)

        media_path = os.path.join(media_path, 'img')

        if(os.path.exists(media_path)):
            f = {}
            for i in _get_files(media_path):
                f[i] = i
            rv[app]['default'] = f

        theme_path = os.path.join(media_path, theme)
        if(os.path.exists(theme_path)):
            f = {}
            for i in _get_files(theme_path):
                f[i] = i
            rv[app][theme] = f
    #print rv
    return rv

class EnvironmentRpcHandler:

    def __init__(self, request, **kwargs):
        self.sa_session = request.Session()
        self.request = request

    def get_env(self):

        request = self.request
        account_id = None
        sa_session = request.Session()
        if request.session.has_key('user_id'):
            account_id = request.session['user_id']

        # manual entries
        manual_controller = ManualController(sa_session, account_id=account_id)
        chapters = manual_controller.get_all_chapters()
        if account_id is not None:
            icontheme = manual_controller.config.get('basesoft.web.core.media.theme.icons')
        else:
            icontheme = 'basesoft'

        # default
        perms = []
        account_configs = {}
        credits = {}
        license_configs = {}
        icon_theme = {'icons': _get_theme_icons(icontheme), 'name': icontheme, 'cache': {}}

        if account_id is not None:

            # account permissions
            permissions_controller = PermissionController(sa_session, account_id=account_id)
            perms = {}
            for p in permissions_controller.get_account_permissions():
                perms[p.name] = True

            # account configs
            account_configs = permissions_controller.config.get_configs()

            # license configs
            lcc = LicenseConfigController(self.sa_session, permissions_controller.get_default_group().license)
            license_configs = lcc.get_configs()

            # get license credits
            credits = {}
            for lu in permissions_controller.get_license_usage():
                credits[lu.usage_option.name] = lu.credits


        server = {'host': request.get_host(), 'lang': get_language(), 'debug': settings.DEBUG}

        # site / theme
        sc = SiteController(self.sa_session, request=self.request)
        ltc = LicenseThemeController(self.sa_session, account_id=account_id, request=self.request)
        current_theme = ltc.get_current_theme()
        current_site = sc.get_current_site()
        company_logo = sc.get_site_logo(current_site)
        if company_logo is not None:
            company_logo = company_logo.file_hash

        site = {
            'home_app': current_site.home_app,
            'company_logo': company_logo,
            'theme': {
                'module_name': current_theme.module_name,
                'graph_theme': {
                    'name': current_theme.default_graph_theme.name,
                    'color_set': {
                        'name': current_theme.default_graph_theme.default_color_set.name
                    }
                }
            }
        }

        # pass username when logged in
        username = None
        if request.session.has_key('username'):
            username = request.session['username']

        # virtual date is None if not enabled
        virtual_date = None
        if settings.VIRTUAL_DATE and 'basesoft.web.contrib.uat' in settings.INSTALLED_APPS:
            virtual_date = today()

        # loop all apps and see of any js_modules.py has a env_modules list.
        # These are js modules that will be forced to load every time reloadEnv()
        # is called. This makes it possible for apps to hook onto the reload env without
        # hacking this view or the bootloader

        # env modules
        env_modules = []
        for app in settings.INSTALLED_APPS:
            try:
                js_modules = namedAny(app+'.js_modules')
            except AttributeError:
                continue

            if not hasattr(js_modules, 'env_modules'):
                continue

            env_modules += getattr(js_modules, 'env_modules')

        country_map = {}
        for c in self.sa_session.query(Country).all():
            country_map[c.iso_3166_a2] = c.id

        return {
            "account_id": account_id,
            "permissions": perms,
            "manual_chapters": chapters,
            "configs": account_configs,
            "license_configs": license_configs,
            "credits": credits,
            "server": server,
            "L10n_settings": L10n_settings.get(request.LANGUAGE_CODE)().get_settings(),
            "iconbarIcons": self._env_icons(request),
            'user_notifications': _get_user_notifications(request),
            "username": username,
            'icon_theme': icon_theme, # FIXME: is this still applicable?
            "site": site,
            'virtual_date': virtual_date,
            'env_modules': env_modules,
            'country_map': country_map
        }

    def _env_icons(self, request):
        """
        returns a set if icons for the given request and json encodes it (decoded clientside), see env.html
        """
        account_id = None
        if request.session.has_key('user_id'):
            account_id = request.session['user_id']

        iconbar = IconBarController(request.Session(), account_id=account_id, request=request)
        return iconbar.get_icons()

class GlobalFormManagerHandler:
    def __init__(self, request, **kwargs):
        self.sa_session = request.Session()
        self.request = request

    def validate_all_required(self, args):
        required = {}
        for key in args.keys():
            required[key] = [(plugins.Required, {})]

        form_handler = FormValidator(required)
        return form_handler.validate(args)

class UserNotificationsHandler:
    def __init__(self, request, **kwargs):
        self.sa_session = request.Session()
        self.request = request

    def get_user_notifications(self):
        return _get_user_notifications(self.request)

class ExcelRPCHandler:

    def __init__(self, request, **kwargs):
        self.sa_session = request.Session()

    def write_excel(self, data, headers):
        """
        Writes data to excel file. User for Customdatatable data export.
        """
        import xlwt
        import random
        from hashlib import sha256 as sha

        workbook = xlwt.Workbook(encoding="UTF-8")
        sheet = workbook.add_sheet('data')

        # add headers to the sheet
        for i, col in enumerate(headers):
            sheet.write(0, i, col)

        # now data
        for row_index, row_data in enumerate(data):
            for column_index, column in enumerate(headers):
                sheet.write(row_index+1, column_index, row_data[column])

        hash = sha(str(random.random())).hexdigest()[:8]
        workbook.save(os.path.join(settings.TMP_FILE_REPOSITORY, 'data_%s.xls' % hash))

        return hash

class LogRpcHandler:

    def __init__(self, request, **kwargs):
        self.request = request

    def console(self, messages):
        logger = logging.getLogger("basesoft.core", request=self.request)

        for type, message in messages:

            message = 'LogRpcHandler: %s: %s' % (type, message[:-2])

            if type == 'debug':
                logger.debug(message)

            if type == 'info':
                logger.info(message)

            if type == 'warn':
                logger.warn(message)

            if type == 'error':
                logger.error(message)

class PasswordCheckHandler:

    def __init__(self, request, **kwargs):
        self.request = request

    def get_password_strength(self, pwd):
        import cracklib, re

        score = 0

        try:
            cracklib.FascistCheck(pwd)
        except ValueError:
            return score

        if len(pwd) >= 8:
            score += 1
        if len(pwd) >= 12:
            score += 1
        if re.search('\d+',pwd):
            score += 1
        if re.search('[a-z]',pwd) and re.search('[A-Z]',pwd):
            score += 1
        if re.search('.[!,@,#,$,%,^,&,*,?,_,~,-,(,)]',pwd):
            score += 1

        return score

handlers = (LoginOverviewRPCHandler, EnvironmentRpcHandler, ExcelRPCHandler, UserNotificationsHandler, LogRpcHandler, PasswordCheckHandler, GlobalFormManagerHandler, )

