# Copyright (c) 2008 BaseSoft B.V.
# See LICENSE for details.

"""
abstract controller module
"""

from django.utils.translation import ugettext as _

from sqlalchemy import case, and_, select

from django.conf import settings
from decimal import Decimal, InvalidOperation

from basesoft.web.core.exceptions import SecurityException, PermissionDeniedException, SystemException,\
    PublicInactiveAccountException
from basesoft.web.core import exceptions as basesoft_exceptions

from basesoft.web.contrib.account.models import Account, Config, AccountConfig
from basesoft.web.contrib.license.models import Permission, Reseller, DomainGroup, Group, LicensePermission,\
    LicenseConfig, GroupPermission, Domain, License
from basesoft.web.core.models import PrivilegeObject

from basesoft import logging
from basesoft.web.contrib.compose.models import ComposePublication,\
    ComposePublicationLocale
from types import NoneType
from basesoft.web.contrib.i18n.models import Locale
from sqlalchemy.orm.exc import NoResultFound
from basesoft.web.core.jsonrpc.dispatcher import JsonRpcJSONEncoder

from basesoft.web.contrib.wiki.parser import Parser
logger = logging.getLogger("iqportal.util.controller")

from django.utils import simplejson


def transaction_decorator(method):
    """
    Decorates every public method of the applications' controllers
    Takes care of transaction begin and commit or rollback
    """
    def new_method(*args, **kwargs):
        assert isinstance(args[0], AbstractController)
        instance = args[0]
        if instance.sa_session.is_active:
            instance.sa_session.begin_nested()
        else:
            instance.sa_session.begin()
        try:
            rv = method(*args, **kwargs)
            instance.sa_session.commit()
            return rv
        except Exception, e:
            instance.sa_session.rollback()
            logger.warning('Rolling back for method "%s" with exception message "%s"' % (method, e.message))
            raise
    return new_method


def get_models_of_type(parent_model, exclude_parent=False, name=None, include_models=True):
    """
    This returns all models in all available apps of a certain type.
    """
    import inspect

    rv = []
    used = []

    # loop all the apps
    for app in settings.INSTALLED_APPS:
        try:
            # get all the models
            models = __import__(app + '.models', {}, {}, [''])
        except ImportError:
            continue
        # loop the models
        for obj in dir(models):
            model = getattr(models, obj)
            # check if we are dealing with a class.
            if not inspect.isclass(model):
                continue
            # check if they are a subclass of the supplied parent model.
            if not issubclass(model, parent_model):
                continue
            if obj in used:
                continue
            if exclude_parent and obj == parent_model.__name__:
                continue
            if name is not None and name==obj:
                return model

            used.append(obj)

            entry = {'label': obj, 'value': obj, 'location': app + '.models'}
            if hasattr(model, 'NAME_I18N'):
                entry['label'] = model.NAME_I18N
            if include_models:
                entry['model'] = model
            rv.append(entry)
    return sorted(rv, key=lambda k: k['value'])

class PermissionController(object):
    """
    The permission controller takes care of Permission and Privileges checking.

    Permission checking is done on a global level and depends on the Permissions
    that are attached to any of the Accounts' Groups.

    Privilege checking is done on object level. The objects that can be used
    are mapped objects to specific SQLAlchemy defined tables. These objects
    require a certain setup to work.

    See also: http://trac.office.basesoft.com/iqportal/wiki/IQPortalPermissions
    """

    # every controller that extends this controller needs to override it's logger_name if it does any log calls,
    # so that we can see in each log call where it came from
    logger_name = 'default'

    CHECK_ACTIVE_ACCOUNT = True

    def __init__(self, sa_session=None, account_id=None, request=None, check_active=True):
        """
        @param sa_session: None or SQLAlchemy session.
                           Gets created if not supplied
        @param account_id: None or id of an account within the system.
                           Required for permission/privilege checking.
        @param request:    None or a request object used for logging calls
        This initializes a Permission Controller.
        """
        self.sa_session = sa_session

        # boolean that allows to use the controller for inactive accounts
        self.check_active = check_active

        # we do not have a session.
        if self.sa_session is None:
            from basesoft.web.core.util.db import Session
            self.sa_session = Session()

        # set some private properties for the permission/privilege systems.
        self._account = None
        self._account_default_group = None
        self._active_account_groups = []
        # an account_id is supplied, so lets get some information necessary.
        if account_id is not None:
            if isinstance(account_id, Account):
                self._account = account_id
            else:
                # get the account object based on the id.
                self._account = self.sa_session.query(Account).filter(Account.id==account_id).one()
            # loop the groups attached to the account
            for ag in self._account.groups:
                # check if the account is active at the group and if the license is active too
                if self.check_active and (not ag.active or not ag.group.license.is_active):
                    continue
                self._active_account_groups.append(ag.group)
                # this is the default group for this account?
                if ag.is_default:
                    self._account_default_group = ag.group

            if self.CHECK_ACTIVE_ACCOUNT and len(self._active_account_groups) == 0:
                raise PublicInactiveAccountException(_('This account is not active.'))

        # get an instance of an AccountController
        self.config = AccountConfigController(sa_session, account_id=account_id)

        # get an instance of the logging class
        self.logger = logging.Logger("%s.%s" % (self.__module__, self.__class__.__name__), request)

    def _get_account(self):
        """
        Protected method to get the account object that instantiated
        this controller.
        """
        return self._account

    get_account = _get_account

    def _get_default_group(self):
        """
        Protected method to get the default group of the account that
        instantiated this controller
        """
        return self._account_default_group

    get_default_group = _get_default_group

    def _get_active_groups(self):
        """
        Protected method to get all the active groups of the account that
        instantiated this controller
        """
        return self._active_account_groups

    get_active_groups = _get_active_groups

    def is_reseller(self):
        """
        This will return True if the current user is a reseller or False otherwise
        @return: Boolean
        """
        return isinstance(self._get_default_group().license, Reseller)

    def assert_any_permissions(self, *requires):
        """
        This will check whether or not the Account has at least one of the provided
        global permissions: 'contact_manager', 'create_survey'
        """
        assert isinstance(self._account, Account)

        # get all the permission objects
        permissions = self.sa_session.query(Permission).filter(Permission.name.in_(requires)).all()
        # if the number of requires is not the same as the number of
        # queried permissions, something must be off.
        if len(requires) is not len(permissions):
            raise PermissionDeniedException('Checking for Permissions that are undefined: %s' %\
                                    [i for i in requires])

        # check if the account has at least one of the permissions
        for group in self._active_account_groups:
            for permission in permissions:
                if permission in group.permissions:
                    return
        raise PermissionDeniedException('Insufficient Permissions: none of the following %s ' % [i.name for i in permissions])

    def has_any_permissions(self, *requires):
        """
        Checks if the account has at least one of the required permissions.
        @return bool
        """
        try:
            self.assert_any_permissions(*requires)
        except PermissionDeniedException:
            return False
        except AssertionError:
            return False
        return True

    def assert_permissions(self, *requires, **kwargs):
        """
        This will check whether or not the Account has all of the provided
        global permissions: 'contact_manager', 'create_survey'
        """
        if kwargs.has_key('anonymous') and kwargs['anonymous']:
            assert self._account is None
        else:
            assert isinstance(self._account, Account)

        scope_group_id = None

        if kwargs.has_key('group_id') and kwargs['group_id'] is not None:
            scope_group_id = [int(kwargs['group_id'])]

        # check for specific license rights:
        if kwargs.has_key('license_id') and kwargs['license_id'] is not None:
            scope_group_id = []
            for group in self._active_account_groups:
                if group.license.id == int(kwargs['license_id']):
                    scope_group_id.append(group.id)

        # Caution, this overrides the groups to be checked. Should be used by non-user invoked methods.
        if kwargs.has_key('groups'):
            groups = kwargs['groups']
        else:
            groups = self._active_account_groups

        # get all the permission objects
        permissions = self.sa_session.query(Permission).filter(Permission.name.in_(requires)).all()

        # if the number of requires is not the same as the number of
        # queried permissions, something must be off.
        if len(requires) is not len(permissions):
            raise PermissionDeniedException('Checking for Permissions that are undefined: %s' %\
                                    [i for i in requires])

        # a list with all the validated permissions.
        validated = []

        # if one of the groups to which the account belongs has the permission
        # the account can continue.
        for group in groups:
            if scope_group_id is not None and group.id not in scope_group_id:
                continue
            for permission in permissions:
                if permission in validated:
                    continue
                if permission in group.permissions:
                    validated.append(permission)

        # The total number of permissions required is not the same as the
        # total number of validated permissions. Raise!
        if(len(permissions) is not len(validated)):
            raise PermissionDeniedException('Insufficient Permissions: %s' % ','.join([_(i.title) for i in permissions if i not in validated]))

    def has_permissions(self, *requires, **kwargs):
        """
        Checks if the account has the required permissions or not.
        @return bool
        """
        try:
            self.assert_permissions(*requires, **kwargs)
        except PermissionDeniedException:
            return False
        except AssertionError:
            return False
        return True

    def get_account_permissions(self, group=None):
        """
        Returns a list with permission objects for the account.
        In case no group is set, it checks in all the groups the account belongs to, else just returns the
        permissions for the specified group.

        If group is set and it is not one of the groups the account belongs too (or it is not active),
        a SecurityException will be raised.
        """
        if self._account is None:
            return []

        if group is not None:
            if group not in self._active_account_groups:
                raise SecurityException('Invalid group %s (id %s) for account %s (id %s)' % \
                                    (group.name, group.id, self._account.name, self._account.id))
            else:
                return group.permissions

        perms = []
        for group in self._active_account_groups:
            [perms.append(p) for p in group.permissions if p not in perms]
        return perms

    def get_license_usage(self, usage_option_name=None):
        """
        Checks if the license has the specified usage option set. If no usage option is provided,
        a list with all license usage options is returned.
        """
        from basesoft.web.contrib.license.models import LicenseUsage, UsageOption

        license = self._get_default_group().license

        if usage_option_name is None:
            lu = self.sa_session.query(LicenseUsage).filter(LicenseUsage.license == license)
            if lu.count() == 0:
                return []
            return lu.all()

        uo = self.sa_session.query(UsageOption).filter(UsageOption.name==usage_option_name)
        if uo.count() == 0:
            raise PermissionDeniedException("Invalid usage option name %s" % usage_option_name)
        uo = uo.one()

        lu = self.sa_session.query(LicenseUsage).filter(and_(LicenseUsage.license == license,
                                                        LicenseUsage.usage_option==uo))
        if lu.count() == 0:
            return None
        return lu.one()

    def assert_credits(self, usage_option, credits=1):
        """
        This will check if there are enough credits left for a certain usage
        option within the License.

        This check should be used on saving of certain objects (Survey,
        Contact, etc.).

        To make it more convenient for the end user in, say, the web
        interfacing, we should also add this check to the views that
        allow for the creation of new objects.
        """
        assert isinstance(self._account, Account)
        lu = self.get_license_usage(usage_option)
        if lu is not None and lu.credits < credits:
            # apparently the License does not have enough credits left for this usage_option
            raise PermissionDeniedException('Insufficient no. of Credits (%s) on %s' % (credits, usage_option))

    @transaction_decorator
    def substract_usage_credits(self, usage_option, credits):
        """
        Decreases the number of credits for the specified usage_option.
        @param usage_option: string
        @param credits: integer
        """
        assert isinstance(self._account, Account)
        lu = self.get_license_usage(usage_option)
        if lu is not None:
            # this can be negative - > debits
            lu.credits = lu.credits - credits
            self.sa_session.add(lu)

    def assert_privileges_on_object(self, obj, *privileges):
        """
        This will check if the current account has certain privileges on the
        supplied object. This object requires to have a link with the Group
        table and has to have license and domain permission fields on it.
        """
        assert isinstance(self._account, Account)
        assert isinstance(obj, PrivilegeObject)

        licenses = []
        for g in self._active_account_groups:
            # check for domain admin / service manager
            for obj_group in obj.groups:
                if obj_group.group.license.domain in g.domains:
                    return
            licenses.append(g.license)

        # check the groups of the account first.
        for obj_group in obj.groups:

            if obj_group.privilege not in privileges:
                continue

            # check for world privileges.
            if obj_group.world_has_privilege:
                privileges = [p for p in privileges if p != obj_group.privilege]

            # check if the group is one of the users groups.
            if obj_group.group_has_privilege and obj_group.group in self._active_account_groups:
                privileges = [p for p in privileges if p != obj_group.privilege]

            # check if the license of the group, is in one of the active group licenses.
            if obj_group.license_has_privilege and obj_group.group.license in licenses:
                privileges = [p for p in privileges if p != obj_group.privilege]

            # check if the domain of the group is in one of the active group domains
            if obj_group.domain_has_privilege and obj_group.group.license.domain == self._get_default_group().license.domain:
                privileges = [p for p in privileges if p != obj_group.privilege]

            # no more privileges required.
            if len(privileges) == 0:
                return

        privs = obj.get_privileges()
        descriptions = '(%s)' % ', '.join([p[1] for p in privs if p[0] in privileges])

        # Well, the account has no access whatsoever to this object, raise!
        raise PermissionDeniedException('Insufficient Privileges %s on object %s for account %s' % (descriptions, obj, self._account))

    def has_privileges_on_object(self, obj, *privileges):
        try:
            self.assert_privileges_on_object(obj, *privileges)
            return True
        except PermissionDeniedException, e:
            logger.warning('Has privilege check failed with exception message: %s' % e.message)
            return False

    def get_query_filter_on_privileges(self, obj, group_obj, priv, fo=None, obj_group=None, group_obj_group=None, license=None, groups=None):
        """
        @param obj: An object mapped to a Database Table.
        @param group_obj: An object mapped to a Database Table that defines
                          the link between a Group and the supplied obj.
        @param priv: int that represents the required privileges.
        @param fo: sqlalchemy from_obj

        An SQLAlchemy 'where' statement. Can be attached to queries that are
        using Privilege based objects and need specific privileges checking.
        """
        assert issubclass(obj, PrivilegeObject)

        from sqlalchemy.sql import or_
        from sqlalchemy.orm import aliased, join
        from basesoft.web.contrib.license.models import License, Group

        if groups is None:
            groups = self._active_account_groups
            domain_id = self._get_default_group().license.domain.id
        else:
            domain_id = groups[0].license.domain.id

        account_groups = [g.id for g in groups]
        licenses = [g.license.id for g in groups]
        serviced_domains = [dg.domain_id for dg in self.sa_session.query(DomainGroup).filter(DomainGroup.group_id.in_(account_groups)).all()]

        if obj_group is None:
            obj_group = aliased(Group)
        if group_obj_group is None: # TODO: remove me?
            group_obj_group = aliased(Group)
        if license is None:
            license = aliased(License)

        if fo is None:
            fo = join(obj, group_obj)

        fo = fo.outerjoin(obj_group, group_obj.group_id==obj_group.id)
        fo = fo.outerjoin(license, obj_group.license_id==license.id)

        domain_filter = None
        if len(serviced_domains) > 0:
            domain_filter = license.domain_id.in_(serviced_domains)

        filter = and_(
            group_obj.privilege==priv,
            or_(
                group_obj.world_has_privilege==True,
                and_(
                    or_(
                        group_obj.group_has_privilege==True,
                        group_obj.license_has_privilege==True,
                        group_obj.domain_has_privilege==True
                    ),
                    group_obj.group_id.in_(account_groups)
                ),
                and_(
                    or_(
                        group_obj.license_has_privilege==True,
                        group_obj.domain_has_privilege==True
                    ),
                    obj_group.license_id.in_(licenses)
                ),
                and_(
                    group_obj.domain_has_privilege==True,
                    license.domain_id==domain_id
                ),
                domain_filter
            )
        )

        filter, fo = obj.get_privilege_filter(filter, fo, self.get_account(), group_obj)

        return filter, fo

    def get_licenses_with_permissions(self, *permissions):

        lps = self.sa_session.query(LicensePermission).join(Permission).filter(Permission.name.in_(permissions)).all()
        return [lp.license for lp in lps]

    def get_accounts_with_privilege_on_instance(self, instance, privilege, permissions=None):
        groups = []
        for go in instance.groups:
            if go.privilege != privilege:
                continue
            if go.group_has_privilege:
                groups.append(go.group)
            if go.license_has_privilege:
                groups += go.group.license.groups
            if go.domain_has_privilege:
                for l in go.group.domain.licenses:
                    groups += l.groups

        if permissions is not None:
            permissions = self.sa_session.query(Permission).filter(Permission.name.in_(permissions)).all()

        accounts = []
        for group in groups:
            if permissions is not None:
                has = True
                for permission in permissions:
                    if not permission in group.permissions:
                        has = False
                        break
                if not has:
                    continue

            accounts += group.accounts

        return list(set(accounts)) # unique accounts only


    def get_account_licenses_with_permssions(self, *permissions):
        """
        Returns all licenses for which this account's active groups have the provided permissions
        """
        licenses = []
        for g in self._active_account_groups:
            # if this group license is already ok, then don't check again
            if g.license in licenses:
                continue
            gp = self.sa_session.query(GroupPermission).join(Permission).filter(
                 and_(GroupPermission.group_id==g.id, Permission.name.in_(permissions))).all()
            # there must be the same number of group permissions as required permissions
            if len(gp) != len(permissions):
                continue
            licenses.append(g.license)
        return licenses

    def get_groups_in_license_with_permissions(self, *permissions, **kwargs):
        if kwargs.has_key('license_id'):
            license = self.sa_session.query(License).filter(License.id==kwargs['license_id']).one()
        else:
            license = self.get_default_group().license

        anonymous = False
        if kwargs.has_key('anonymous'): # can only be used if instantiated without an account. (Tasks)
            anonymous = kwargs['anonymous']

        rv = []
        for g in license.groups:
            if self.has_permissions(*permissions, groups=[g], anonymous=anonymous):
                rv.append(g)
        return rv

    def get_groups_in_domain_with_permissions(self, *permissions, **kwargs):
        """
        This returns all the groups within the specified domain which have the supplied permissions.
        Can be used to send a notification to every group who has rights to specific interfaces.
        """
        anonymous = False
        if kwargs.has_key('anonymous'): # can only be used if instantiated without an account. (Tasks)
            anonymous = kwargs['anonymous']

        if kwargs.has_key('domain_id'):
            domain = self.sa_session.query(Domain).filter(Domain.id==kwargs['domain_id']).one()
        else:
            domain = self.get_default_group().license.domain

        rv = []
        for l in domain.licenses:
            for g in l.groups:
                if self.has_permissions(*permissions, groups=[g], anonymous=anonymous):
                    rv.append(g)
        return rv

    def get_object_by_priv_in_hierarchy(self, group, obj, priv, _where=None):
        """
         * Check if obj is available on the license level
         * Check if obj is available on the license domain level
         * Check if obj is available on reseller level (if license.reseller)
         * Get the obj on BaseSoft level (!)
        """
        _license = group.license
        domain = _license.domain
        reseller = domain.reseller
        site_owner = self.sa_session.query(License).filter(License.name==settings.SITE_OWNER_LICENSE_NAME).one()

        domain_mail = None
        reseller_mail = None
        site_owner_mail = None

        go, bitbucket = obj().get_group_object()

        _filter, fo = self.get_query_filter_on_privileges(obj, go, priv=priv, groups=[group])
        if _where is not None:
            _filter = and_(_filter, _where)
        mails = self.sa_session.query(obj).select_from(fo).filter(_filter).all()
        for mail in mails:
            for g in mail.groups:
                if g.privilege!=priv:
                    continue
                if g.group.license == _license:
                    return mail
                if g.group.license.domain == domain and domain_mail is None:
                    domain_mail = mail
                    continue
                if g.group.license == reseller and reseller_mail is None:
                    reseller_mail = mail
                    continue
                if g.group.license == site_owner and site_owner_mail is None:
                    site_owner_mail = mail

        if domain_mail is not None:
            return domain_mail
        if reseller_mail is not None:
            return reseller_mail
        if site_owner_mail is not None:
            return site_owner_mail

        return None

class AbstractConfigController(object):
    def __init__(self, sa_session=None, scope=None):
        self.sa_session = sa_session
        if self.sa_session is None:
            from basesoft.web.core.util.db import Session
            self.sa_session = Session()

        self.scope = scope

    def get_configs(self, config_name=None):
        raise SystemException('Implement on child object')

    def _get_configs(self, join_table, where_field, config_name=None):
        if config_name is not None:
            assert isinstance(config_name, list)
            # check if config_names supplied are all correct
            for cn in config_name:
                q = self.sa_session.query(Config).filter(Config.name==cn)
                if q.count() == 0:
                    raise basesoft_exceptions.PublicSystemException(
                             "Trying to get value for an invalid config: '%s' (Scope id %s)"
                             % (cn, self.scope.id))

        from basesoft.web.contrib.account.sa_datamapper import get_tables as tables

        c = tables()['tblConfig']

        columns = [c.c.id, c.c.name, c.c.type,
                   case([(join_table.c.value==None,  c.c.default)],  else_=join_table.c.value).label("value")
                  ]
        scope_id = None
        if self.scope is not None:
            scope_id = self.scope.id
        fo = c.outerjoin(join_table, and_(c.c.id == join_table.c.config_id, getattr(join_table.c, where_field) == scope_id))
        q = select(columns=columns, from_obj=fo)

        if config_name is not None:
            q = q.where(c.c.name.in_(config_name))

        result = self.sa_session.connection().execute(q)
        ret = {}
        for row in result:
            ret[row.name] = self._check_config_value_type(row.name, row.value, row.type)

        return ret

    def get(self, config_name):
        """
        Return the value of the supplied config name
        @param config_name: string - the name of the config
        """
        return self.get_configs([config_name])[config_name]

    def _check_config_value_type(self, config_name, value, type_str):
        """
        Method used for checking config values having the type as str.
        @return:  the value converted to the specified type.
        """

        message = "Config '%s': Wrong value '%s' for type '%s' (Scope id %s)" % (config_name, value,
                                                                                   type_str, self.scope)
        # none is always allowed
        if value is None:
            return value

        if type_str == 'string':
            if not isinstance(value, (str, unicode)):
                raise basesoft_exceptions.PublicSystemException(message)

        if type_str == 'integer':
            try:
                value = int(value)
            except ValueError:
                raise basesoft_exceptions.PublicSystemException(message)

        if type_str == 'decimal':
            try:
                value = Decimal(value)
            except TypeError:
                raise basesoft_exceptions.PublicSystemException(message)
            except InvalidOperation:
                raise basesoft_exceptions.PublicSystemException(message)

        if type_str == 'boolean' and not isinstance(value, bool):
            if value.lower() == "true":
                value = True
            elif value.lower() == "false":
                value = False
            else:
                raise basesoft_exceptions.PublicSystemException(message)
        return value

    def set(self, config_name, config_value):
        """
        @param config_name: string - the name of the config
        @param config_value: the value to set
        """
        self.set_configs({config_name: config_value})

class LicenseConfigController(AbstractConfigController):
    def get_configs(self, config_name=None):
        from basesoft.web.contrib.license.sa_datamapper import get_tables as tables
        return self._get_configs(tables()['tblLicenseConfig'], 'license_id', config_name)

    def set_configs(self, configs):
        if self.scope is None:
            raise basesoft_exceptions.PublicSystemException("Trying to set configs for an invalid license")

        for config_name, value in configs.iteritems():
            try:
                default = self.sa_session.query(Config).filter(Config.name==config_name).one()
            except NoResultFound:
                raise basesoft_exceptions.PublicSystemException(
                    "Trying to set value for an invalid config: '%s' (License id %s)" % (config_name, self.scope.id))


            current_lc = self.sa_session.query(LicenseConfig).filter(and_(LicenseConfig.license_id==self.scope.id,
                                                            LicenseConfig.config_id==default.id)).first()

            if default.default is not None and default.default.lower() == str(value).lower():
                if current_lc is not None:
                    self.sa_session.delete(current_lc)
                continue

            if value == '':
                value = None

            if default.default is None and value is None:
                if current_lc is not None:
                    self.sa_session.delete(current_lc)
                continue

            if value is None:
                if current_lc is not None:
                    self.sa_session.delete(current_lc)
                continue

            config_value = self._check_config_value_type(config_name, value, default.type)

            if current_lc is None:
                current_lc = LicenseConfig()
                current_lc.license = self.scope
                current_lc.config = default

            if config_value is None:
                current_lc.value = None
            else:
                current_lc.value = unicode(config_value)

            self.sa_session.add(current_lc)
        self.sa_session.flush()

class AccountConfigController(AbstractConfigController):
    """
    The account config controller takes care of account configs get/set.
    """

    def __init__(self, sa_session=None, account_id=None, scope_account_id=None):
        """
        @param sa_session: None or SQLAlchemy session.
        @param account_id: None or id of an account within the system.
        @param scope_account_id: None or id of the account that gets/sets configs for account_id;
                                'config_manager' permission is required for this account.
        This initializes a AccountConfigController.
        """

        self.sa_session = sa_session
        # we do not have a session.
        if self.sa_session is None:
            from basesoft.web.core.util.db import Session
            self.sa_session = Session()

        if account_id == scope_account_id:
            scope_account_id = None

        account = None
        if account_id is not None:
            if isinstance(account_id, Account):
                account = account_id
            else:
                # get the account object based on the id.
                account = self.sa_session.query(Account).filter(
                                 Account.id==account_id).one()

        self._scope_account = None
        if scope_account_id is not None:
            if isinstance(scope_account_id, Account):
                self._scope_account = scope_account_id
            else:
                # get the account object based on the id.
                self._scope_account = self.sa_session.query(Account).filter(
                                 Account.id==scope_account_id).one()

        AbstractConfigController.__init__(self, self.sa_session, account)

    def get_configs(self, config_name=None):
        """
        Return the values for the supplied config(s)
        @param config_name: None: method returns a dictionary with all config_names and values pairs
                            array of strings(names of config_names): method returns a dictionary with
                                    supplied config_names and values pairs
        """
        if self.scope is None:
            raise basesoft_exceptions.PublicSystemException("Trying to get config for an invalid account")

        # check for perms if the request is not for the current account
        if self._scope_account is not None:
            self.__check_scope_account_perms()

        from basesoft.web.contrib.account.sa_datamapper import get_tables as tables
        return self._get_configs(tables()['tblAccountConfig'], 'account_id', config_name)

    def set_configs(self, account_config):
        """
        TODO: move to AbstractConfigController and make abstract
        Updates provided configs for account.
        @param account_config: dictionary with config_name and value pairs
        """
        if self.scope is None:
            raise basesoft_exceptions.PublicSystemException("Trying to set config for an invalid account")

        # check for perms if the request is not for the current account
        if self._scope_account is not None:
            self.__check_scope_account_perms()

        for config_name in account_config:

            # basesoft.web.contrib.locale could be a locale str but also a language code,
            # so we need to convert that to a locale str
            if config_name == "basesoft.web.contrib.locale":
                from basesoft.web.contrib.i18n.controllers import I18nController
                locale_controller = I18nController(self.sa_session, None, self.scope.id)
                if account_config[config_name] != '':
                    account_config[config_name] = locale_controller.to_locale(account_config[config_name])

            default = self.sa_session.query(Config).filter(Config.name==config_name)
            if default.count() == 0:
                raise basesoft_exceptions.PublicSystemException(
                            "Trying to set value for an invalid config: '%s' (Account id %s)"
                            % (config_name, self.scope.id))
            default = default.one()

            config_value = account_config[config_name]
            q = self.sa_session.query(AccountConfig).filter(and_(AccountConfig.account_id==self.scope.id,
                                                            AccountConfig.config_id==default.id))
            has_ac = q.count()
            # check if value is the same as default
            if not default.default is None and default.default.lower() == str(config_value).lower():
                # if there is no entry for AccountConfig, then just ignore this value
                # but if there is an entry in AccountConfig remove it
                # cuz this config was set to default and no point in keeping this entry
                if has_ac > 0:
                    self.sa_session.delete(q.one())
                continue
            if default.default is None and config_value is None:
                if has_ac > 0:
                    self.sa_session.delete(q.one())
                continue

            # check if value has correct type
            config_value = self._check_config_value_type(config_name, config_value, default.type)

            if has_ac > 0:
                c = q.one()
                if config_value is None:
                    c.value = None
                else:
                    c.value = str(config_value)
            else:
                c = AccountConfig()
                c.account = self.scope
                c.config = default
                c.value = str(config_value)
            self.sa_session.add(c)
        self.sa_session.flush()

    def __check_scope_account_perms(self):
        """
        Checks if the scope account has the right permissions using the permission controller.
        """
        pc = PermissionController(self.sa_session, self._scope_account)
        pc.assert_permissions("account_config_manager")


class AbstractController(PermissionController):
    """
    AbstractController - parent class for every application's controllers
    """
    # in some cases we need to skip checking permissions and privileges
    check_privileges = True

    def __init__(self, sa_session=None, locale=None, account_id=None, request=None, check_active=True):
        """
        @param sa_session: valid sqlalchemy session object or, if None, one will be created
        @param locale: String
        @param account_id: the id of the account (optional)
        @param request: used by the logger of this controller (optional). If we intend to do any log calls, we should
            provide request, as it contains info needed for a more detailed logging
        """
        PermissionController.__init__(self, sa_session, account_id, request, check_active=check_active)
        self.locale = locale
        self.account_id = account_id
        self.request = request

    def _get_privileged_object(self, obj, where=None, priv=PrivilegeObject.PRIV_VIEW, filter_marks=True, order_by=None, with_polymorphic=None):

        group_obj = obj().get_group_object()[0]

        filter, fo = self.get_query_filter_on_privileges(obj, group_obj, priv)

        if filter_marks:
            filter = and_(filter, and_(obj.deleted==False, obj.archived==False))
        if where is not None:
            filter = and_(filter, where)

        query = self.sa_session.query(obj).select_from(fo).filter(filter)

        if order_by is not None:
            query = query.order_by(order_by)

        return query.all()

    get_privileged_object = _get_privileged_object

    def get_privileged_object_by_id(self, obj, id, priv=PrivilegeObject.PRIV_VIEW, **kwargs):
        where = None
        if id is not None:
            if not isinstance(id, list):
                where = obj.id==id
            else:
                where = obj.id.in_(id)

        return self._get_privileged_object(obj, where, priv, **kwargs)

    @transaction_decorator
    def _set_mark(self, obj, ids, mark, priv, value):
        """
        Sets the flag for 'mark' to 'value' for 'obj' with 'ids' for which user has 'priv'
        """

        assert mark in ['deleted', 'archived']

        if self.has_permissions('privilege_object_administrator'):
            if not isinstance(ids, list):
                ids = [ids]
            _filter = obj.id.in_(ids)
            if value:
                _filter = and_(_filter, and_(obj.deleted==False, obj.archived==False))
            instances = self.sa_session.query(obj).filter(_filter).all()
        else:
            instances = self.get_privileged_object_by_id(obj, ids, priv, filter_marks=value)
        if len(instances) == 0:
            pr = 'DELETE'
            if mark == 'archived':
                pr = 'ARCHIVE'
            logger.warning('%s: _set_mark: no instance found while trying to set mark `%s` to %s for `%s` with ids: %s. %s'
                           % (self, mark, value, obj, ids, 'Probably no %s privilege on the object.' % pr))
            return ids

        if not isinstance(ids, list):
            ids = [ids]

        for instance in instances:
            setattr(instance, mark, value)
            self.sa_session.add(instance)
            ids = [id for id in ids if not id==instance.id]
            getattr(instance, 'on_mark_as_%s' % mark)()

        if len(ids) > 0:
            logger.warning('%s: _set_mark: no ids while trying to set mark `%s` to %s for `%s` with ids: %s' % (self, mark, value, obj, ids))

        return ids

    def mark_as_deleted(self, obj, ids):
        if type(obj) in [str, unicode]:
            obj = get_models_of_type(PrivilegeObject, True, obj, False)
        if not self.has_permissions('privilege_object_administrator'): # administrator have this right always. TODO: fix for resellers
            self.assert_permissions(obj.mark_as_deleted_permission)
        return self._set_mark(obj, ids, 'deleted', obj.PRIV_DELETE, True)

    def mark_as_archived(self, obj, ids):
        if type(obj) in [str, unicode]:
            obj = get_models_of_type(PrivilegeObject, True, obj, False)
        if not self.has_permissions('privilege_object_administrator'): # administrator have this right always. TODO: fix for resellers
            self.assert_permissions(obj.mark_as_archived_permission)
        return self._set_mark(obj, ids, 'archived', obj.PRIV_ARCHIVE, True)

    def unmark_as_deleted(self, obj, ids):
        if type(obj) in [str, unicode]:
            obj = get_models_of_type(PrivilegeObject, True, obj, False)
        if not self.has_permissions('privilege_object_administrator'): # administrator have this right always. TODO: fix for resellers
            self.assert_permissions(obj.mark_as_deleted_permission)
        return self._set_mark(obj, ids, 'deleted', obj.PRIV_DELETE, False)

    def unmark_as_archived(self, obj, ids):
        if type(obj) in [str, unicode]:
            obj = get_models_of_type(PrivilegeObject, True, obj, False)
        if not self.has_permissions('privilege_object_administrator'): # administrator have this right always. TODO: fix for resellers
            self.assert_permissions(obj.mark_as_archived_permission)
        return self._set_mark(obj, ids, 'archived', obj.PRIV_ARCHIVE, False)

    @transaction_decorator
    def save_default_privilege_object_group(self, instance, target_group=None, privileges=None):
        """
        Saves default privileges for current default group to provided Privilege Object.
        """
        assert isinstance(instance, PrivilegeObject)
        # can only use this on new.
        assert len(instance.groups)==0

        if target_group is None:
            target_group = self.get_default_group()
        if not isinstance(target_group, Group):
            target_group = self.sa_session.query(Group).filter(Group.id==target_group).one()

        if privileges is None:
            privileges = instance.all_privileges

        object = instance.get_group_object()[0]
        for privilege in privileges:
            go = object()
            go.group = target_group
            go.privilege = privilege
            go.group_has_privilege = True
            self.sa_session.add(go)
            instance.groups.append(go)

    def get_group_instance_by_privilege(self, instance, priv, group=None):
        object, field = instance.get_group_object()
        if group is None:
            group = self.get_default_group()
        return self.sa_session.query(object).filter(and_(getattr(object, field)==instance, object.group==group,
                                                         object.privilege==priv)).first()

    @transaction_decorator
    def _grant_or_revoke(self, instance, privileges, grant, group=False, license=False, domain=False, target_group=None,
                         check_own_privs=True, world=False):
        if not (isinstance(privileges, list) or isinstance(privileges, tuple)):
            privileges = [privileges]

        if check_own_privs:
            # can only grant/revoke if they have the rights to delegate privileges AND the priv they want to grant/revoke
            privs_to_check = [instance.PRIV_DELEGATE_PRIVILEGES]
            for p in privileges:
                privs_to_check.append(p)
            self.assert_privileges_on_object(instance, *privs_to_check)

        if target_group is None:
            target_group = self.get_default_group()

        object, field = instance.get_group_object()

        for privilege in privileges:
            remove = True
            go = self.get_group_instance_by_privilege(instance, privilege, target_group)
            if not go:
                go = object()
                go.group = target_group
                go.privilege = privilege
                setattr(go, field, instance)
                remove = False
            if group:
                go.group_has_privilege = grant
            if license:
                go.license_has_privilege = grant
            if domain:
                go.domain_has_privilege = grant
            if world:
                go.world_has_privilege = grant
            if go.group_has_privilege or go.license_has_privilege or go.domain_has_privilege or go.world_has_privilege:
                self.sa_session.add(go)
            elif remove: # we dont want to keep useless group objects.
                logger.warning('Removed group object from instance [%s] for privilege: %s' % (instance, privilege))
                self.sa_session.delete(go)
        #self.sa_session.expire(instance)

    def grant_privilege_to(self, instance, privileges, group=False, license=False, domain=False, target_group=None, check_own_privs=True, world=False):
        logger.info('Granting privileges [%s] on instance [%s] for group: %s, license: %s, domain: %s on target_group: %s' % (privileges, instance, group, license, domain, target_group))
        self._grant_or_revoke(instance, privileges, True, group, license, domain, target_group, check_own_privs, world)

    def revoke_privilege_from(self, instance, privileges, group=False, license=False, domain=False, target_group=None, check_own_privs=True, world=False):
        logger.info('Revoking privileges [%s] from instance [%s] for group: %s, license: %s, domain: %s on target_group: %s' % (privileges, instance, group, license, domain, target_group))
        self._grant_or_revoke(instance, privileges, False, group, license, domain, target_group, check_own_privs, world)

class StateManager:

    STRICT_MODE_IS_EQUAL = 1
    STRICT_MODE_HAS_ANY = 2
    STRICT_MODE_HAS_ALL = 3

    def __init__(self, locale, account_id, mode):
        self.locale = locale
        self.account_id = account_id
        self.mode = mode

    def has_mode(self, strict, *modes):

        sum = 0
        for m in modes:
            sum = sum + m

        # STRICT_MODE_IS_EQUAL means that the sum of the supplied modes should equal the current mode.
        if strict == self.STRICT_MODE_IS_EQUAL:
            return self.mode == sum;

        # STRICT_MODE_HAS_ALL means that every supplied mode should be in the current mode.
        if strict == self.STRICT_MODE_HAS_ALL:
            for m in modes:
                if self.mode & m != m:
                    return False
            return True

        # STRICT_MODE_HAS_ANY means that at least one of the modes provided is in the current mode.
        for m in modes:
            if self.mode & m == m:
                return True
        return False

class AbstractStateController(AbstractController):
    """
    Takes care of common logic for all compose modules. One day this will be moved to iqportal compose.
    @abstract
    """

    # global id_map is used to return a saved state id map back to the client.
    # It will be filled like a dictionary of {domID: dbID} for all items in the survey. This way the client can map
    # the (new) id's for the saved survey
    save_state_id_map = {}

    # delete pool is a list of instances marked for deletion that is filled by _delete_childs() and popped by save().
    # This will actually be deleted from the DB by the commit_delete_pool() method which should be called when the full
    # state has been processed.
    delete_pool = []

    # list of instances with their controller which get for post processing
    post_process_map = []

    @staticmethod
    def reset_state_map():
        """
        Resets the state map
        """

        AbstractStateController.save_state_id_map = {}
        AbstractStateController.delete_pool = []
        AbstractStateController.post_process_map = []

    @staticmethod
    def commit_delete_pool(sa_session):
        """
        Static function that uses the AbstractStateController.delete_pool and deletes all instances in it. This list got filled
        and unfilled by the _delete_children() method and save()
        """
        for instance in AbstractStateController.delete_pool:
            sa_session.delete(instance)
            sa_session.expire(instance) # http://trac.office.basesoft.com/iqportal/ticket/3716
            logger.info('AbstractStateController.commit_delete_pool: deleted %s' % instance )

        AbstractStateController.delete_pool = []
        sa_session.flush()

    # model class reference for which the controller is working for (like Survey)
    _instance_class = None

    # list of locale attributes beloning to this class.. like 'html', 'title', ..
    _locale_attributes = []

    # string representing the column for the locale name in the locale table
    _locale_name_column = 'locale'

    def __init__(self, session, manager, *args, **kwargs):
        """
        extends abstractcontrollers' init so it can create a locale controller
        """
        AbstractController.__init__(self, session, manager.locale, manager.account_id, *args, **kwargs)

        from basesoft.web.contrib.i18n.controllers import I18nController
        self.locale_controller = I18nController(session, manager.locale, manager.account_id)

        self.manager = manager

    def _assert_state(self, state, key, type, assert_value=None):
        """
        Handy assert method for verifying state.
        @param state: dict with state to assert against
        @param key: str key in state to assert against. use None for the state itself
        @param type: type to check. can be a type or a tuple of types. if type
            is a tuple of types, then the asserted value must be of one of this types
        @param assert_value: optional value to equal check state[key] for

        @raise PublicSystemException: when the sate is asserted as incorrect
        """

        if key is None:
            value = state
        else:
            try:
                value = state[key]
            except KeyError:
                raise basesoft_exceptions.PublicSystemException("Missing key '%s' in state %s" % (key, state))

        try:

            if type == unicode:
                type = (unicode, str)

            if type == int:
                type = (int, long)

            # make sure we always have a tuple
            if not isinstance(type, tuple): type=(type,)

            found = False
            for t in type:
                try:
                    if t is None:
                        assert value is None
                    else:
                        assert isinstance(value, t)
                    found = True
                    break
                except AssertionError:
                    pass
            if not found: raise AssertionError()
        except AssertionError:
            raise basesoft_exceptions.PublicSystemException("Wrong type for key '%s' in state '%s'." % (key, state))

        if assert_value is not None:
            try:
                assert value == assert_value
            except AssertionError:
                raise basesoft_exceptions.PublicSystemException("Expected value '%s' for key '%s' in state '%s'" % (assert_value, key, state))

    def _instance_from_state(self, state):
        """
        Abstract method that takes care of retrieving the common compose objects based on a state

        @access protected

        # model_class needs to be referened to a model that correspondents with the inherited controller class. For
        # example the SurveyStateController needs to reference iquest_models.Survey.
        """

        assert self._instance_class is not None

        # Get the object from the db if the id is passed else create a new these are objects like Survey, Block,
        # Question, etc
        if state['dbID'] != None:
            instance = self.sa_session.query(self._instance_class).get(state['dbID'])
        else:
            instance = self._instance_class()

        if instance is None:
            raise Exception('No instance for state: %s' % state)

        # set id mapping (wont be saved to db)
        instance.domID = state['domID']

        return instance

    def _instance_from_id(self, id):
        """
        This returns the instance for the given id based on the current _instance_class
        """
        obj = self.sa_session.query(self._instance_class).get(id)
        if not obj:
            raise basesoft_exceptions.PublicSystemException("Object not found: %s" % id)
        return obj

    _modules = []

    @transaction_decorator
    def save(self, instance, state):
        """
        Takes care of saving and filling the id_map
        """
        self._assert_state(state, 'domID', unicode)

        self.sa_session.add(instance)

        # explicit flush so we have an id available
        self.sa_session.flush()

        # a domID can only be supplied once
        if AbstractStateController.save_state_id_map.has_key(state['domID']):
            raise Exception('id_map already has %s' % state['domID'])

        # update the id_map
        AbstractStateController.save_state_id_map[state['domID']] = instance.id

        # If we got here it means the object is still in the state and therefor shouldnt be deleted so pop it from the delete_pool.
        if instance in AbstractStateController.delete_pool:
            AbstractStateController.delete_pool.remove(instance)

        #self._modules.append(instance)

        AbstractStateController.post_process_map.append( (self, instance) )

    def _canvas_post_save(self):

        for controller, instance in AbstractStateController.post_process_map:

            if hasattr(controller, 'canvas_post_save'):
                controller.canvas_post_save(instance)

        # reset the map, it shouldnt be needed anymore
        AbstractStateController.post_process_map = []

        self.sa_session.flush()


    @transaction_decorator
    def _save_locale(self, parent_instance, locale_model_class, state):
        """
        This takes care of common locale state saving in the compose application.

        @param parent_instance: sqlalchemy object instance representing the parent table of the locales. (like Survey())
        @param locale_model_class: sqlalchemy class reference where locales are saved (like iquest_models.SurveyLocale)
        @param locale_attributes: list of strings representing state and locale_model_class attributes. (like ['html', 'description', 'title']
        @param state: state containing the locale texts. Like: { locales: {'en_GB': {'title': 'english title', 'description': '..'}, 'nl_NL': {..}} }
        """

        # verify state
        # maybe do this a bit more
        self._assert_state(state, 'locales', dict)
        assert isinstance(parent_instance.locales, list)

        is_wiki = hasattr(parent_instance, 'is_wiki') and parent_instance.is_wiki

        # Loop instance.locales and update them for whats found in the state.
        # InstLocale is an instance of a model like SurveyLocale that contains texts per locale
        # Now set the texts from the state and delete it from state so we know whats new
        new_locales = []

        for instLocale in parent_instance.locales:
            locale_str = getattr(instLocale.locale, self._locale_name_column)

            keep = False

            # check if the state even has a entry for this locale_str (en_GB, nl_NL)
            if state['locales'].has_key(locale_str):

                for key in self._locale_attributes:

                    state_key = key+'_wiki' if is_wiki else key

                    if hasattr(instLocale, key):

                        # we will unset the value for this attribute if it isn't present in the state
                        # see also ticket #1002
                        if state['locales'][locale_str].has_key(state_key):
                            setattr(instLocale, key, state['locales'][locale_str][state_key])
                            del state['locales'][locale_str][state_key]

                            keep = True
                        else:
                            setattr(instLocale, key, None)

                    else:
                        raise Exception("_save_locale: Expected to find key '%s' in instLocale '%s'" % (key, instLocale) )

                if keep:
                    new_locales.append(instLocale)
                else:
                    self.sa_session.delete(instLocale)
            else:
                # locale was once set but not anymore in the state.. delete it
                self.sa_session.delete(instLocale)

        parent_instance.locales = new_locales
        self.sa_session.flush()

        # whats left in state is new
        for locale_str in state['locales']:
            locale = self.locale_controller.get_locale_by_str(locale_str)

            # instanciate the locale class (like SurveyLocale) and set the locale
            append = False
            localeInst = locale_model_class()
            localeInst.locale = locale

            for locale_attribute in self._locale_attributes:

                state_key = locale_attribute+'_wiki' if is_wiki else locale_attribute

                texts = state['locales'][locale_str]
                if texts.has_key(state_key):

                    text = texts[state_key]
                    if text is not None:
                        setattr(localeInst, locale_attribute, unicode(text))
                    else:
                        setattr(localeInst, locale_attribute, None)
                    append = True

            # Append it to the instance locales. Notice that we dont have to set a back reference (like normally
            # on SurveyLocale we need to set the survey instance). This is taken care of by sqlalchemy
            if append:
                parent_instance.locales.append(localeInst)

    @transaction_decorator
    def _save_available_locales(self, parent_instance, locale_model_class, backref, state):
        """
        This saves all available locales for a certain instance in the compose application.
        The provided instance must have the available_locales property.

        @param parent_instance: sqlalchemy object instance representing the parent table of the locales. (like Survey())
        @param locale_model_class: sqlalchemy class reference where locales are saved (like iquest_models.SurveyLocale)
        @param backref: string representing the locale_model_class property name to reference parent_instance (like 'survey')
        @param state: state containing the available_locales list. Like: { available_locales: ['en_GB', 'nl_NL'], ...}
        """
        self._assert_state(state, 'availableLocales', list)
        assert isinstance(parent_instance.available_locales, list)
        locales = state['availableLocales']

        current = []
        # delete from db what was removed
        for locale in parent_instance.available_locales:
            if getattr(locale, self._locale_name_column) not in locales:
                obj_locale = self.sa_session.query(locale_model_class) \
                    .filter(and_(locale_model_class.locale == locale,
                        getattr(locale_model_class, backref) == parent_instance)).one()
                self.sa_session.delete(obj_locale)
                self.sa_session.flush()
                continue
            current.append(getattr(locale, self._locale_name_column))
        # add only the new
        for locale_str in locales:
            if locale_str not in current:
                sl = locale_model_class()
                setattr(sl, backref, parent_instance)
                sl.locale = self.locale_controller.get_locale_by_str(locale_str)
                self.sa_session.add(sl)

    def _load_locales(self, instance, extra_attributes=None):
        """
        Returns a state locales dictionary for the given instance
        """
        rv = {}
        for locale_inst in instance.locales:
            locale_str = getattr(locale_inst.locale, self._locale_name_column)
            if not rv.has_key(locale_str):
                rv[locale_str] = {}

            for attr in self._locale_attributes:

                if hasattr(instance, 'is_wiki') and instance.is_wiki:
                    rv[locale_str][attr+'_wiki'] = getattr(locale_inst, attr) # copy original text
                    rv[locale_str][attr+'_html'] = Parser().render_to_html(getattr(locale_inst, attr))
                else:
                    rv[locale_str][attr] = getattr(locale_inst, attr)

            if extra_attributes is not None:
                for attr in extra_attributes:
                    rv[locale_str][attr] = getattr(locale_inst, attr)

        return rv


    def _save_publication_state(self, publication, state, locale):
        """
        the state arriving here should already be stripped from other locales
        """
        assert issubclass(publication.__class__, ComposePublication)
        assert type(locale) in (NoneType, Locale)
        assert type(state) in (NoneType, dict)

        # if state is None remove any cached state from the db
        if state is None:

            q = self.sa_session.query(ComposePublicationLocale).filter(ComposePublicationLocale.publication==publication)
            if  locale is not None:
                q = q.filter(ComposePublicationLocale.locale==locale)
            objs = q.all()

            for cl in objs:
                self.sa_session.delete(cl)
        else:

            try:
                cl = self.sa_session.query(ComposePublicationLocale) \
                    .filter(ComposePublicationLocale.publication==publication, ComposePublicationLocale.locale==locale).one()
            except NoResultFound:
                cl = ComposePublicationLocale()
                cl.publication = publication
                cl.locale = locale

            cl.state = simplejson.dumps(state, ensure_ascii=False, cls=JsonRpcJSONEncoder)
            self.sa_session.add(cl)

        self.sa_session.add(publication)
        self.sa_session.flush()


    def _locale_strip_state(self, state, locale, fallback_locale):
        """
        recursive strips all other locales from the given state
        tested by: trial iqportal.iquest.test.unit.test_response.ResponseTest.test_locales
        """

        assert isinstance(state, dict)
        assert isinstance(locale, Locale )
        assert isinstance(fallback_locale, (str, unicode) )

        l_str = locale.locale


        if state.has_key('locales'):

            # state['locales'] looks something like:
            # {'nl_NL': {'trailing_label': None, 'label': u'Second question, second answer Dutch'},
            #  'en_GB': {'trailing_label': None, 'label': u'Second question, second answer English'}}

            if state['locales'].has_key(l_str):

                # new state locales will be only for this locale
                new_locales = {l_str: {}}

                # loop keys (label, trailing label, ..)
                for k in state['locales'][l_str]:

                    # check if current locale has a valid string.. else fallback to fallback locale if set
                    if isinstance(state['locales'][l_str][k], (str, unicode)) and state['locales'][l_str][k].strip() != '':
                        new_locales[l_str][k] = state['locales'][l_str][k]
                    elif state['locales'].has_key(fallback_locale) and isinstance(state['locales'][fallback_locale][k], (str, unicode)) and state['locales'][fallback_locale][k].strip() != '':
                        new_locales[l_str][k] = state['locales'][fallback_locale][k]
                    else:
                        new_locales[l_str][k] = ''

                # set new locales
                state['locales'] = new_locales

            else:
                # if we come here there is no locale key at all.. use fallback directly if available
                if state['locales'].has_key(fallback_locale):
                    state['locales'] = {l_str: state['locales'][fallback_locale] }


        # recursive process children
        if state.has_key('children'):
            for i in state['children']:
                for j in range(len(state['children'][i])):
                    state['children'][i][j] = self._locale_strip_state(state['children'][i][j], locale, fallback_locale)

        return state

    def _delete_children(self, children, state=None):
        """
        We dont actually delete yet just yet. Instead we mark all children as deleted. After this function is called
        the save() method on all children will then pop them out of the delete_pool.

        @param children: collection of childs (like survey.blocks)
        @param state: state object to compare against (deprecated)
        """

        for child in children:
            assert child not in AbstractStateController.delete_pool
            AbstractStateController.delete_pool.append(child)
