# Copyright (c) 2010 BaseSoft B.V.
# See LICENSE for details.
import os

from decimal import Decimal

from django.conf import settings
from django.utils.translation import ugettext as _

from sqlalchemy.types import Numeric
from sqlalchemy import and_, not_, or_, desc, func, Integer, String, Date, DateTime, Boolean, DECIMAL
from sqlalchemy.orm import join, aliased, outerjoin
from sqlalchemy.orm.exc import NoResultFound
from sqlalchemy.orm.util import AliasedClass

from basesoft.web.core.models import PrivilegeObject
from basesoft.web.core.util.controllers import AbstractController
from basesoft.web.contrib.license.models import Group, License
from basesoft.util.ordered_dict import OrderedDict
from basesoft.web.core.exceptions import PublicSystemException

_custom_queries = {}
_cqs_loaded = False

DATA_TYPE_STRING = 1
DATA_TYPE_INT = 2
DATA_TYPE_DATE = 3
DATA_TYPE_BOOLEAN = 4
DATA_TYPE_DECIMAL = 5

FILTER_CONTAINS = 1
FILTER_NOT_CONTAINS = 2
FILTER_EQUAL = 3
FILTER_NOT_EQUAL = 4
FILTER_STARTS_WITH = 5
FILTER_ENDS_WITH = 6
FILTER_LARGER_THAN = 7
FILTER_SMALLER_THAN = 8
FILTER_LARGER_THAN_EQUAL = 9
FILTER_SMALLER_THAN_EQUAL = 10
FILTER_BETWEEN = 11

from threading import Lock
load_cqs_lock = Lock()

def _load_cqs():
    global _cqs_loaded, _custom_queries
    load_cqs_lock.acquire()

    if not _cqs_loaded:

        for app in settings.INSTALLED_APPS:
            try:
                # TODO: check if file exists and error on import
                module = __import__(app + '.custom_query', {}, {}, [''])
            except ImportError:
                continue

            if not hasattr(module, 'custom_queries'):
                continue

            for cq in module.custom_queries:
                _custom_queries['%s.%s' % (app, cq.__name__)] = cq

        _cqs_loaded = True

    load_cqs_lock.release()

def make_key(field):
    # TODO: mkpretty #3230
    key = str(field.label('')).replace('`', '').replace('"', '').replace('.', '_').replace('tbl', '').replace('(', '_').replace(')', '').lower()
    return field.label(key), key

class CustomQueryModel:
    priv_group_obj = None
    priv_group_obj_group = None
    priv_license_obj = None

    def __init__(self, model, onclause=None, priv=True, use_outerjoin=False, auto_group_by=True, show_archived=False, use_in_count=True, with_polymorphic=None):
        self.model = model
        self.onclause = onclause
        self.priv = priv
        self.use_outerjoin = use_outerjoin
        self.auto_group_by = auto_group_by
        self.show_archived = show_archived
        self.use_in_count = use_in_count

        self.with_polymorphic = with_polymorphic # this can be used to query only models with a specific polymorphic type

        if issubclass(model, PrivilegeObject) and self.auto_group_by is True:
            self.priv_group_obj = aliased(Group)
            self.priv_group_obj_group = aliased(Group)
            self.priv_license_obj = aliased(License)

    def set_license_object(self, model):
        """
        Can be used if you want to pass this as the license object to get_query_filter_on_privileges
        """
        self.priv_license_obj = model

    def set_group_object(self, model):
        """
        Can be used if you want to pass this as the group object to get_query_filter_on_privileges
        """
        self.priv_group_obj = model

    def set_group_object_group(self, model):
        """
        Can be used if you want to pass this as the group-group object to get_query_filter_on_privileges
        """
        self.priv_group_obj_group = model


    def process(self, custom_query, fo, f, initial):

        model = self.model

        if fo is None:
            if issubclass(model, PrivilegeObject) and self.priv is not None:
                filter, fo = custom_query._get_query_filter_on_privileges(self)
                if f is None:
                    f = filter
                else:
                    f = and_(f, filter)
                return fo, f, initial

            fo = initial = model
            return fo, f, initial

        if fo == initial:
            if self.use_outerjoin:
                fo = outerjoin(fo, model, self.onclause)
            else:
                fo = join(fo, model, self.onclause)
        else:
            if self.use_outerjoin:
                fo = fo.outerjoin(model, self.onclause)
            else:
                fo = fo.join(model, self.onclause)

        if issubclass(model, PrivilegeObject) and self.priv is not None:
            filter, fo = custom_query._get_query_filter_on_privileges(self, fo)
            if f is None:
                f = filter
            else:
                f = and_(f, filter)

        if self.with_polymorphic is not None:
            fo.with_polymorphic(self.with_polymorphic)

        return fo, f, initial



class CustomQueryField:

    def __init__(self, field, label, key=None, sortable=True, is_sorted=False, sorted_asc=True, in_select=True,  \
                 is_filter=False, filter_options=None, is_group_by=False, type=None, filter_is_unique=False):
        if key is None:
            self.field, self.key = make_key(field)
        else:
            self.field = field.label(key)
            self.key = key
        self.label = label

        if not sortable and is_sorted:
            raise Exception("Trying to sort an non-sortable field: '%s'" % self.key)

        if not is_filter and filter_options is not None:
            raise Exception("Trying to set filter options for a field which is not set as filter: field %s" % self.key)

        self.sortable = sortable
        self.is_sorted = is_sorted
        self.sorted_asc = sorted_asc
        self.in_select = in_select
        self.is_filter = is_filter
        self.filter_options = filter_options
        self.is_group_by = is_group_by
        self.filter_is_unique = filter_is_unique

        if type is not None:
            self.type = type
        elif hasattr(field, 'property'):
            ftype = field.property.columns[0].type
            if isinstance(ftype, Integer):
                self.type = DATA_TYPE_INT
            elif isinstance(ftype, String):
                self.type = DATA_TYPE_STRING
            elif isinstance(ftype, (Date, DateTime,)):
                self.type = DATA_TYPE_DATE
            elif isinstance(ftype, Boolean):
                self.type = DATA_TYPE_BOOLEAN
            elif isinstance(ftype, (Decimal, DECIMAL, Numeric)):
                self.type = DATA_TYPE_DECIMAL
            else:
                raise Exception('Unknown type: %s' % ftype)
        else:
            self.type = DATA_TYPE_STRING

    def get_dataset_def(self):
        return {
            'key': self.key,
            'label': self.label, # gettext translated header for yui datatable
            'sortable': self.sortable,
            'is_sorted': self.is_sorted,
            'sorted_asc': self.sorted_asc,
            'type': self.type
        }

    def get_filter_def(self):
        return {
            'label': self.label,
            'type': self.type,
            'unique': self.filter_is_unique
        }

    def get_filter_option_fields(self):

        return [self.field.label('value'), self.filter_options.label('label')]

def get_operator_map():
    return {
        FILTER_CONTAINS: ContainsFilter,
        FILTER_NOT_CONTAINS: NotcontainsFilter,
        FILTER_EQUAL: EqualFilter,
        FILTER_NOT_EQUAL: NotequalFilter,
        FILTER_STARTS_WITH: StartswithFilter,
        FILTER_ENDS_WITH: EndswithFilter,
        FILTER_LARGER_THAN: LargerthanFilter,
        FILTER_SMALLER_THAN: SmallerthanFilter,
        FILTER_LARGER_THAN_EQUAL: LargerthanequalFilter,
        FILTER_SMALLER_THAN_EQUAL: SmallerthanequalFilter,
        FILTER_BETWEEN: BetweenFilter
    }

class CustomQuery(AbstractController):

    group_by = None
    where = None
    order_by = None # if specified, overrides the sort and asc params that are usually used in get_data
    permissions = None

    def __init__(self, sa_session, account_id=None, **kwargs):
        AbstractController.__init__(self, sa_session, account_id=account_id)
        self.operator_map = get_operator_map()

        self._value_processors = {}
        self.set_value_processors()

        self.cq_tables = [t for t in self.tables()]
        self.cq_fields = [f for f in self.fields()]

        self._set_privilege_object_fields()

        self.validate_access()
        self.set_field_map()
        self.set_where()

    def validate_access(self):
        if self.permissions is not None:
            self.assert_permissions(*self.permissions)

    def tables(self):
        raise Exception('Not implemented') # TODO: not implemented error

    def fields(self):
        raise Exception('Not implemented') # TODO: not implemented error

    def set_where(self):
        pass

    def set_value_processors(self):
        pass

    def _get_query_filter_on_privileges(self, table, fo=None):
        model = model_or_alias = table.model
        if isinstance(model,  AliasedClass):
            model = model_or_alias.__getstate__()['mapper'].class_

        if table.priv is True:
            priv = model.PRIV_VIEW
        elif table.priv is None:
            return None, fo
        else:
            priv = table.priv

        group_obj = aliased(model().get_group_object()[0])
        if fo is None:
            fo = outerjoin(model_or_alias, group_obj)
        else:
            fo = fo.outerjoin(group_obj)
        filter, fo = self.get_query_filter_on_privileges(model_or_alias, group_obj, priv, fo,
                                                   obj_group=table.priv_group_obj,
                                                   group_obj_group=table.priv_group_obj_group,
                                                   license=table.priv_license_obj)
        if table.use_outerjoin:
            filter = or_(filter, model_or_alias.id==None)
        return filter, fo

    def _make_query(self, split_by, select, count=False):
        fo = None
        initial = None
        f = None

        if self.where is not None:
            f = self.where

        if split_by is not None:
            field, where = split_by
            field = self.get_group_by_object()[field]
            if f is None:
                f = field.field==where
            else:
                f = and_(f, field.field==where)

        for table in self.cq_tables:

            #if count and not table.use_in_count:
            #    continue

            fo, f, initial = table.process(self, fo, f, initial)

        q = self.sa_session.query(*select).execution_options(stream_results=True).select_from(fo).filter(f)
        if self.group_by is not None:
            q = q.group_by(*self.group_by)
        return q

    def _get_data(self, split_by, sort, asc, offset, limit):

        if self.order_by is not None:
            sort = self.order_by
        else:
            if sort is not None:
                for f in self.cq_fields:
                    if f.key == sort and not f.sortable:
                        raise Exception("Requested sort on a non-sortable field: '%s'" % f.key)
                    else:
                        break
                if not asc:
                    sort = desc(sort)
            sort = (sort, )

        q = self._make_query(split_by, [i.field for i in self.cq_fields if i.in_select]).order_by(*sort).limit(limit).offset(offset)

        rv = []
        for row in q.all():
            data = {}
            for key in row.keys():
                data[key] = self._value_processor(row, key)
            rv.append(data)
        return rv

    def _value_processor(self, row, key):
        if not self._value_processors.has_key(key):
            return getattr(row, key)
        return self._value_processors[key](row, key)

    def _get_row_count(self, split_by):

        try:
            if self.group_by is None:
                return self._make_query(split_by, [func.count('*').label('count')], count=True).one().count
            else:
                return len(self._make_query(split_by, [func.count('*').label('count')], count=True).all())

        except NoResultFound:
            return 0

    def _get_filter_options(self, filter):

        if isinstance(filter.filter_options, list):
            return filter.filter_options

        rv = []
        for row in self._make_query(None, filter.get_filter_option_fields()).group_by(filter.field).order_by(filter.filter_options).distinct().all():
            data = {}
            for key in row.keys():
                data[key] = getattr(row, key)
            rv.append(data)

        return rv

    def _apply_default_filters(self, filters):
        """
        This method applies 'deleted' and 'archived' filters for all privileged objects,
        in case none of them are already set by 'filters'.

        Child classes should not add filters for deleted and archived with set_where
        """
        filters_fields = [str(self.field_map[f].field) for f in filters]
        default_f = None
        for table in self.cq_tables:
            model = table.model
            if issubclass(model, PrivilegeObject):
                dfld = make_key(model.deleted)[0]
                afld = make_key(model.archived)[0]

                if str(dfld) in filters_fields or str(afld) in filters_fields:
                    continue

                if table.show_archived:
                    da_f = model.deleted==False
                else:
                    da_f = and_(model.deleted==False, model.archived==False)

                # if table has an onclause, we add the filters for deleted and archived to that,
                # else we add it to self.where
                if table.onclause is not None:
                    table.onclause = and_(table.onclause, da_f)
                else:
                    if default_f is None:
                        default_f = da_f
                    else:
                        default_f = and_(default_f, da_f)

        if default_f is not None:
            if self.where is None:
                self.where = default_f
            else:
                self.where = and_(self.where, default_f)

    def _apply_filters(self, filters, static=False):
        for filter_name in filters:
            field = self.field_map[filter_name]
            if not static:
                assert field.is_filter
            operator = filters[filter_name]['operator']
            values = filters[filter_name]['values']

            filter_field = field.field
            filter = self.operator_map[operator](filter_field, values)
            if self.where is None:
                self.where = filter.execute()
            else:
                self.where = and_(self.where, filter.execute())

    def get_definitions(self, group_by, static_filters, applied_filters, no_empty_tables=True):

        # no_empty_tables: if true, definitions for empty tables will not be included
        # (this applies only in case group_by is set)

        has_sortable = False
        for f in self.cq_fields:
            if f.sortable:
                has_sortable = True
                break;
        if not has_sortable:
            raise Exception('There is no sortable field defined')

        self._apply_filters(static_filters, True)
        self._apply_default_filters(static_filters)
        self._apply_filters(applied_filters)

        if group_by is None:
            return [{
                'title': None,
                'group_by_label': None,
                'column_defs': [c.get_dataset_def() for c in self.cq_fields if c.in_select],
            }]

        rv = []

        group_by = self.get_group_by_object()[group_by]

        s = [group_by.field]
        group_by_values = {}
        group_by_key = None
        if isinstance(group_by.filter_options, list):
            for f in group_by.filter_options:
                group_by_values[f['value']] = f['label']
        else:
            pass
            if group_by.filter_options is not None:
                s.append(group_by.filter_options.label('templabel'))
                group_by_key = 'templabel'

        order_by = s
        for dataset_labels in self._make_query(None, s).group_by(group_by.field).order_by(*order_by).distinct().all():
            val = getattr(dataset_labels, group_by.key)
            if group_by_values.has_key(val):
                val_label = group_by_values[val]
            else:
                if group_by_key is not None:
                    val_label = getattr(dataset_labels, group_by_key)
                else:
                    val_label = val
            if no_empty_tables and self._get_row_count([group_by.key, val]) == 0:
                continue;
            rv.append({
                'title': val,
                'group_by_label': val_label,
                'column_defs': [c.get_dataset_def() for c in self.cq_fields if c.in_select]
            })

        return rv

    def get_data(self, split_by, sort, asc, offset, limit, applied_filters, static_filters, row_count=True):

        self._apply_filters(static_filters, True)
        self._apply_filters(applied_filters)

        all_filters = {}
        for k in static_filters:
            all_filters[k] = static_filters[k]
        for k in applied_filters:
            all_filters[k] = applied_filters[k]

        self._apply_default_filters(all_filters)

        data = self._get_data(split_by, sort, asc, offset, limit)
        if row_count:
            return {
                'data': data,
                'totalRecords': self._get_row_count(split_by)
            }
        else:
            return data

    def set_field_map(self):
        self.field_map = {}
        for f in self.cq_fields:
            self.field_map[f.key] = f

    def get_group_by_object(self):
        group_by = {}
        for f in self.cq_fields:
            if f.is_group_by:
                group_by[f.key] = f

        return group_by

    def _set_privilege_object_fields(self):
        """
        If 'primary table' is a privileged object, include:
            * deleted and archived
            * group, license and domain filters/group by's
        """
        tbl = None
        for table in self.cq_tables:
            if table.onclause is None and issubclass(table.model, PrivilegeObject):
                tbl = table
                break

        if tbl is None:
            return

        if self.group_by is None and tbl.auto_group_by:
            self.group_by = (tbl.model.id, )

        if tbl.model.deleted is None or tbl.model.archived is None:
            raise Exception('model %s is missing deleted/archived' % tbl.model)

        # check if there isn't already filters set for deleted and archived, else add them
        delf = make_key(tbl.model.deleted)[0]
        archf = make_key(tbl.model.archived)[0]
        add_del = True
        add_arch = True
        for f in self.cq_fields:

            if str(f.field) == str(delf):
                f.is_filter = True
                add_del = False

            if str(f.field) == str(archf):
                f.is_filter = True
                add_arch = False

        from django.utils.translation import ugettext as _
        if add_del:
            self.cq_fields.append(CustomQueryField(tbl.model.deleted, _('Deleted'), key="deleted",
                                                   is_filter=True, type=DATA_TYPE_BOOLEAN))
        if add_arch:
            self.cq_fields.append(CustomQueryField(tbl.model.archived, _('Archived'), key="archived",
                                                   is_filter=True, type=DATA_TYPE_BOOLEAN))

        if self.has_permissions('view_all_domains'):
            # TODO: add domain filter for Service Managers, Resellers, etc. #3229
            pass

        if self.has_permissions('view_domain') and tbl.priv_license_obj is not None:
            self.cq_fields.append(CustomQueryField(tbl.priv_license_obj.id, _('License'),
                                                   key='filter_license', is_filter=True, is_group_by=True,
                                                   in_select=False, filter_options=tbl.priv_license_obj.name))
        if self.has_permissions('view_license') and tbl.priv_group_obj is not None:
            self.cq_fields.append(CustomQueryField(tbl.priv_group_obj.id, _('Group'),
                                                   key='filter_group', is_filter=True, is_group_by=True,
                                                   in_select=False, filter_options=tbl.priv_group_obj.name))

    def get_filters_and_group_by(self, static_filters):

        filters = {}
        group_by = {}
        self._apply_filters(static_filters, True)
        self._apply_default_filters(static_filters)

        _order = {}
        for f in self.cq_fields:

            _order[f.label] = f.key
            if f.is_filter:
                filters[f.key] = f.get_filter_def()
                if f.filter_options is not None:
                    filters[f.key]['options'] = self._get_filter_options(f)

            if f.is_group_by:
                group_by[f.key] = f.label

        # now we do some sorting
        mysort = sorted(_order.keys(), key=lambda s: s.lower())
        x = []
        y = []
        for l in mysort:
            if filters.has_key(_order[l]):
                x.append((_order[l], filters[_order[l]]))
            if group_by.has_key(_order[l]):
                y.append((_order[l], group_by[_order[l]]))

        return {
            'filters':  OrderedDict(x),
            'group_by': OrderedDict(y)
        }

class SQLCustomQueryTable:
    def __init__(self, tbl, alias=None, onclause=None, use_outerjoin=False):
        self.tbl = tbl
        self.alias = alias if alias is not None else tbl
        self.onclause = onclause
        self.use_outerjoin = use_outerjoin

    def process(self, custom_query, fo):
        pass

class SQLCustomQueryField:
    def __init__(self, tbl_alias, field, label, _type=None, key=None, is_filter=False, filter_is_unique=False, in_select=True):
        self.tbl_alias = tbl_alias
        self.field = field
        self.label = label
        self.key = key if key is not None else '%s_%s' % (tbl_alias, field)
        self.is_filter = is_filter
        self.filter_options = None
        self.is_group_by = False
        self.type = _type if _type is not None else DATA_TYPE_STRING
        self.filter_is_unique = filter_is_unique
        self.in_select = in_select
        self.sortable = True

    def get_dataset_def(self):
        return {
            'key': self.key,
            'label': self.label, # gettext translated header for yui datatable
            'sortable': self.sortable,
            #'is_sorted': self.is_sorted,
            #'sorted_asc': self.sorted_asc,
            'type': self.type
        }

    def get_filter_def(self):
        return {
            'label': self.label,
            'type': self.type,
            'unique': self.filter_is_unique
        }

class SQLCustomQuery(AbstractController):

    connstr = None
    group_by = None

    def __init__(self, sa_session, account_id=None, **kwargs):
        self._tables = {}
        self._fields = {}
        self.operator_map = get_operator_map()

        self._value_processors = {}

        AbstractController.__init__(self, sa_session, account_id=account_id)

        self.set_value_processors()

        self.set_scope_session()
        self.where = []
        self._params = []

        self.set_where()
        self.set_field_map()

    def set_where(self):
        pass

    def add_where(self, statement, params=None):
        self.where.append(statement)
        if params is None:
            return
        if type(params) != list:
            params = [params]
        self._params += params

    def _unique_param(self):
        return len(self._params)

    def set_value_processors(self):
        pass

    def set_field_map(self):
        self.field_map = {}
        for f in self.fields():
            self.field_map[f.key] = f

    def set_scope_session(self):
        self._scope_session = self.sa_session
        if self.connstr is None:
            return

        from sqlalchemy import create_engine
        from sqlalchemy.orm import scoped_session, sessionmaker

        db_engine = create_engine(self.connstr, echo=False, pool_recycle=2)
        self._scope_session = scoped_session(sessionmaker(bind=db_engine, autoflush=True, autocommit=True))

    def _get_field(self, f, use_as=True):
        if f.tbl_alias is not None:
            rv = "`%s`.`%s`" % (f.tbl_alias, f.field)
        else:
            rv = f.field
        if not use_as:
            return rv

        return '%s as `%s`' % (rv, f.key)

    def _build_query(self, split_by, sort=None, asc=True, offset=None, limit=None, count=False):
        select = []
        from_obj = []

        if count:
            # GROUP BY's are executed after the count...
            if self.group_by is not None and len(self.group_by) == 1:
                select.append('COUNT(%s) as `cnt`' % ', '.join(['DISTINCT `%s`.`%s`' % (t, a) for t, a in self.group_by]))
            else:
                select.append('COUNT(*) as `cnt`')
        else:
            for f in self.fields():
                select.append(self._get_field(f))

        for t in self.tables():
            if len(from_obj) == 0:
                from_obj.append('`%s` as `%s`' % (t.tbl, t.alias))
                continue
            onclause = []
            for clause in t.onclause:
                if len(clause) == 4:
                    onclause.append('`%s`.`%s` = `%s`.`%s`' % clause)
                else:
                    onclause.append('`%s`.`%s` = "%s"' % clause)
            from_obj.append('%(join)s JOIN `%(tbl)s` as `%(alias)s` ON %(onclause)s' % {
                'join': 'INNER' if not t.use_outerjoin else 'LEFT OUTER',
                'tbl': t.tbl,
                'alias': t.alias,
                'onclause': ' AND '.join(onclause)
            })

        # TODO: split by
        sql = 'SELECT %s \nFROM %s' % (', '.join(select), '\n'.join(from_obj))
        if len(self.where) > 0:
            sql += '\nWHERE %s' % ' AND '.join(self.where)

        if not count and self.group_by is not None:
            sql += '\nGROUP BY %s' % ', '.join(['`%s`.`%s`' % (t, a) for t, a in self.group_by])

        if sort is not None:
            f = self.field_map[sort]
            sql += '\nORDER BY %s %s' % (self._get_field(f, False), 'ASC' if asc else 'DESC')

        if offset is not None:
            sql += '\nLIMIT %d, %d' % (int(offset), int(limit))

        return sql

    def _get_data(self, split_by, sort, asc, offset, limit):

        sql = self._build_query(split_by, sort, asc, offset, limit)
        rv = []
        rows = self._scope_session.execute(sql, self._get_params()).fetchall()
        for r in rows:
            # if the row processor returns False, the whole row is skipped.
            if not self._row_processor(r):
                continue
            res = {}
            for f in self.fields():
                res[f.key] = self._value_processor(r, f.key)
            rv.append(res)

        return rv

    def _get_params(self):
        rv = {}
        for i in range(len(self._params)):
            rv['param%d' % i] = self._params[i]
        return rv

    def _row_processor(self, row):
        return True

    def _value_processor(self, row, key):
        if not self._value_processors.has_key(key):
            return row[key]
        return self._value_processors[key](row, key)

    def _get_row_count(self, split_by):
        sql = self._build_query(split_by, count=True)
        result = self._scope_session.execute(sql, self._get_params()).fetchone()
        return 0 if result is None else result['cnt']

    def get_data(self, split_by, sort, asc, offset, limit, applied_filters, static_filters, row_count=True):

        self._apply_filters(static_filters, True)
        self._apply_filters(applied_filters)

        data = self._get_data(split_by, sort, asc, offset, limit)
        if row_count:
            return {
                'data': data,
                'totalRecords': self._get_row_count(split_by)
            }
        else:
            return data

    def _apply_filters(self, filters, static=False):

        for filter_name in filters:
            field = self.field_map[filter_name]
            if not static:
                assert field.is_filter
            operator = filters[filter_name]['operator']
            values = filters[filter_name]['values']

            filter_field = '`%s`.`%s`' % (field.tbl_alias, field.field)
            _filter = self.operator_map[operator](filter_field, values, field.type)
            self.add_where(*_filter.execute_sql(self._unique_param()))

    def get_definitions(self, group_by, static_filters, applied_filters, no_empty_tables=True):
        return [{
            'title': None,
            'group_by_label': None,
            'column_defs': [c.get_dataset_def() for c in self.fields() if c.in_select],
        }]

    def get_filters_and_group_by(self, static_filters):
        filters = {}
        group_by = {}
        #self._apply_filters(static_filters, True)

        for f in self.fields():

            if f.is_filter:
                filters[f.key] = f.get_filter_def()
                if f.filter_options is not None:
                    filters[f.key]['options'] = self._get_filter_options(f)

            if f.is_group_by:
                group_by[f.key] = f.label

        return {
            'filters':  filters,
            'group_by': group_by
        }

class Filter:
    """
    Filter object, this should be extended by all the applicable filters.
    """
    def __init__(self, field, values, _type=None):
        """
        Get the sqlalchemy column object of the field and initialize empty values.
        """
        self.field = field
        self.type = _type
        self.values = [self._process_value(v) for v in values]

    def add_values(self, values):
        """
        Add the values to be compared
        """
        if type(values) != list:
            values = [values]
        self.values += [self._process_value(v) for v in values]

    def _process_value(self, value):
        if self.type is None:
            return value

        if self.type == DATA_TYPE_BOOLEAN:
            return '1' if value is True else '0'

        return value

    def execute(self):
        """
        This function needs to be overridden by the child classes.
        """
        raise NotImplementedError('Please override this function')

    def execute_sql(self):
        """
        This function needs to be overridden by the child classes.
        """
        raise NotImplementedError('Please override this function')

    def _get_params(self, i, x):
        rv = []
        for j in range(x):
            rv.append(':param%d' % i)
            i += 1
        return rv

class EqualFilter(Filter):
    """
    Equal to, if multiple values are supplied the field should be equal to either
    of them.
    """
    def execute(self):
        values = self.values
        rv = self.field.in_(values)
        if None in values:
            rv = or_(rv, self.field==None)
        return rv

    def execute_sql(self, param_i):
        rv = '%s IN (%s)' % (self.field, ', '.join(self._get_params(param_i, len(self.values))))
        if None in self.values:
            rv = '(%s OR ISNULL(%s))' % (rv, self.field)
        return rv, self.values

class NotequalFilter(Filter):
    """
    Same as Equal filter with a `not` around it.
    """
    def execute(self):
        return not_(self.field.in_(self.values))

    def execute_sql(self, param_i):
        return '%s NOT IN (%s)' % (self.field, ', '.join(self._get_params(param_i, len(self.values)))), self.values

class ContainsFilter(Filter):
    """
    This checks whether or not a certain stringfield contains values.
    """
    def execute(self):
        """
        """
        fields = []
        for like in self.values:
            fields.append(self.field.like('%'+like+'%'))
        return or_(*fields)

    def execute_sql(self, param_i):
        where = []
        for value in self.values:
            where.append('%s LIKE "%%:param%d%%"' % (self.field, param_i))
            param_i += 1

        return '(%s)' % ' OR '.join(where), self.values

class NotcontainsFilter(Filter):
    """
    This checks whether or not a certain stringfield not contains values
    """
    def execute(self):
        """
        This uses `and` for obvious reasons
        """
        fields = []
        for like in self.values:
            fields.append(self.field.like('%'+like+'%'))
        return not_(or_(*fields))

    def execute_sql(self, param_i):
        where = []
        for value in self.values:
            where.append('%s NOT LIKE "%%:param%d%%"' % (self.field, param_i))
            param_i += 1

        return '(%s)' % ' AND '.join(where), self.values

class StartswithFilter(Filter):
    """
    Does the field start with certain values?
    """
    def execute(self):
        """
        This uses `or` cause a string can only start with one value, not more.
        """
        fields = []
        for like in self.values:
            fields.append(self.field.like(like+'%'))
        return or_(*fields)

    def execute_sql(self, param_i):
        where = []
        for value in self.values:
            where.append('%s LIKE ":param%d%%"' % (self.field, param_i))
            param_i += 1

        return '(%s)' % ' OR '.join(where), self.values

class EndswithFilter(Filter):
    """
    Does the field end with certain values?
    """
    def execute(self):
        """
        This uses `or` cause a string can only end with one value, not more.
        """
        fields = []
        for like in self.values:
            fields.append(self.field.like('%'+like))
        return or_(*fields)

    def execute_sql(self, param_i):
        where = []
        for value in self.values:
            where.append('%s LIKE "%%:param%d"' % (self.field, param_i))
            param_i += 1

        return '(%s)' % ' OR '.join(where), self.values

class LargerthanFilter(Filter):
    """
    Is the value of the field larger than a certain value?
    """
    def execute(self):
        """
        Assert that we only have one value, multiple values do not make sense.
        """
        assert len(self.values) == 1
        return self.field>self.values[0]

class LargerthanequalFilter(Filter):
    """
    Is the value of the field larger than or equal to a certain value?
    """
    def execute(self):
        """
        Assert that we only have one value, multiple values do not make sense.
        """
        assert len(self.values) == 1
        return self.field>=self.values[0]

class SmallerthanFilter(Filter):
    """
    Is the value of the field smaller than a certain value?
    """
    def execute(self):
        """
        Assert that we only have one value, multiple values do not make sense.
        """
        assert len(self.values) == 1
        return self.field<self.values[0]

class SmallerthanequalFilter(Filter):
    """
    Is the value of the field smaller than or equal to a certain value?
    """
    def execute(self):
        """
        Assert that we only have one value, multiple values do not make sense.
        """
        assert len(self.values) == 1
        return self.field<=self.values[0]

class BetweenFilter(Filter):
    """
    Is the value of the field anywhere between those values?
    """
    def execute(self):
        """
        """
        # assert that we have an even number of values
        fields = []
        for first, last in self.values:
            fields.append(self.field.between(first,last))
        return or_(*fields)

class CustomQueryRpcHandler:

    def __init__(self, name, sa_session, account_id=None, **kwargs):
        _load_cqs()

        try:
            CustomQueryClass = _custom_queries[name]
        except IndexError:
            raise Exception('CustomQuery not found: %s' % name)

        self.cq = CustomQueryClass(sa_session, account_id=account_id, **kwargs)

    def get_filters_and_group_by(self, static_filters):
        return self.cq.get_filters_and_group_by(static_filters)

    def get_definitions(self, group_by, static_filters, applied_filters, no_empty_tables=True):
        if group_by == '':
            group_by = None
        return self.cq.get_definitions(group_by, static_filters, applied_filters, no_empty_tables)

    def get_data(self, split_by, params, applied_filters, static_filters):
        asc = params['dir'] != 'desc'
        return self.cq.get_data(split_by, params['sort'], asc, params['startIndex'], params['results'], applied_filters, static_filters)

    def write_excel(self, headers, order_by_params, applied_filters, static_filters):
        """
        Writes cq data into xls, returns hash.
        """
        asc = order_by_params['dir'] != 'desc'
        data = self.cq.get_data(None, order_by_params['sort'], asc, None, None, applied_filters, static_filters, False)

        import xlwt
        import random
        from hashlib import sha256 as sha

        workbook = xlwt.Workbook(encoding="UTF-8")
        sheet = workbook.add_sheet('data')

        # add headers to the seeet
        for i, col in enumerate(headers):
            sheet.write(0, i, col['label'])

        # now data
        for row_index, row_data in enumerate(data):
            for column_index, column in enumerate(headers):
                sheet.write(row_index+1, column_index, row_data[column['key']] )

        hash = sha(str(random.random())).hexdigest()[:8]
        workbook.save(os.path.join(settings.TMP_FILE_REPOSITORY, 'data_%s.xls' % hash))

        return hash

    def custom_call(self, method, split_by, params, applied_filters, static_filters, *args):
        if not hasattr(self.cq, method):
            raise PublicSystemException(_('Incorrect method provided'))

        m = getattr(self.cq, method)
        return m(split_by, params, applied_filters, static_filters, *args)
