# Copyright (c) 2008 BaseSoft B.V.
# See LICENSE for details.

from sqlalchemy import create_engine, MetaData
from django.conf import settings
from sqlalchemy.orm import scoped_session, sessionmaker

from basesoft import logging
import MySQLdb.cursors
logger = logging.getLogger('basesoft.core')

metadata = MetaData(create_engine(settings.DB_CONNSTR, echo=False))

# ignore some sqlalchemy warnings
import warnings
from sqlalchemy.exc import SAWarning

warnings.filterwarnings("ignore", 'Unicode type received non-unicode bind param value', SAWarning)

# full apache cpu?
#warnings.filterwarnings("error", 'Data truncated .*', Warning)


# see if there is a sa_datamapper module in the app
# if there is import it and do model mapping
for app in settings.INSTALLED_APPS:
    try:
        sa_module = __import__(app + '.sa_datamapper', {}, {}, [''])
    except ImportError:
        continue

    if hasattr(sa_module, 'do_tables'):
        sa_module.do_tables(metadata)

    if hasattr(sa_module, 'do_mappings'):
        sa_module.do_mappings()


from threading import Lock
make_db_session_lock = Lock()

class AbstractDatabaseSession:
    """
    Abstract singleton database session
    """

    conn_str = None
    Session = None

    isolation_level = 'REPEATABLE READ'

    def __init__(self):

        if self.__class__.Session is None:

            # when the session isnt initialised yet then multiple threads are active
            # so make a lock that creates the session and releases when done
            # then the other threads will check for session again and use that
            make_db_session_lock.acquire()

            # second thread checks for session again
            if self.__class__.Session is not None:
                make_db_session_lock.release()
                return

            # first thread creates the Session
            db_engine = create_engine(self.__class__.conn_str, echo=False, pool_recycle=3600, max_overflow=50, isolation_level=self.isolation_level, connect_args=self.get_connect_args())
            self.__class__.Session = scoped_session(sessionmaker(bind=db_engine, autoflush=True, autocommit=True))

            # first thread release
            make_db_session_lock.release()

    def get_connect_args(self):
        return {}

class DatabaseSession(AbstractDatabaseSession):
    conn_str = settings.DB_CONNSTR

class ReadOnlyDatabaseSession(AbstractDatabaseSession):
    conn_str = settings.DB_CONNSTR_READONLY
    isolation_level = 'READ COMMITTED'

    def get_connect_args(self):
        return {'cursorclass': MySQLdb.cursors.SSCursor}

class TaskDatabaseSession(AbstractDatabaseSession):
    conn_str = settings.DB_CONNSTR

def Session():
    return DatabaseSession().Session

def SessionReadOnly():
    return ReadOnlyDatabaseSession().Session

def TaskSession():
    return TaskDatabaseSession().Session

from sqlalchemy.ext import compiler
from sqlalchemy.sql import ColumnElement

class group_concat(ColumnElement):
    def __init__(self, col1, col2):
        self.col1 = col1
        self.col2 = col2
        self.type = col1.type

@compiler.compiles(group_concat, 'mysql')
def compile_group_concat(element, compiler, **kw):
    return "GROUP_CONCAT(%s ORDER BY %s)" % (
                compiler.process(element.col1),
                compiler.process(element.col2)
           )


