# Copyright (c) 2008 BaseSoft B.V.
# See LICENSE for details.

from threading import Lock
from datetime import date, datetime

from django.conf import settings
from django.http import HttpResponse, HttpResponsePermanentRedirect
from django.views.decorators.cache import cache_page
from django.utils.translation import get_language
from twisted.python.reflect import namedAny

from basesoft.web.core.jsonrpc.dispatcher import JsonRpcDispatcher
from basesoft.web.core.util.controllers import AbstractController
from basesoft.web.contrib.account.decorators import login_required
from basesoft.web.core.util.parse import CodeSquasher


from basesoft import logging
from basesoft.web.core.exceptions import PublicSystemException

logger = logging.getLogger("basesoft.core")

from hashlib import md5, sha1

import random
import subprocess

from pyPdf import PdfFileReader

def blank(request):
    return HttpResponse('')

def bootstrap(request, version, type, extra_requires):
    """
    View for type=js|css that returns the initial JS code that is executed at first when the page loads.
    This includes all global objects included in all applications.
    """

    # parse extra_requires
    if extra_requires == '':
        extra_requires = []
    else:
        extra_requires = extra_requires.split(',')

    # setup requires
    requires = [
        'iq_core',
        'iq_loader',
        'iqportal_init',
        'iq_growl',
    ] + extra_requires

    return loader_get(request, version, type, '', ','.join(requires))


# list used in the loader_get() view which caches all md5 filenames from
# files that have already been served so we know they exist
_cached_loaders = []

@cache_page
def loader_get(request, version, type, exclude_modules, requires):

    """
    Loader url using a GET request to ask for the source for the given requested requires.
    @param type: string either 'css' or 'js'
    @param exclude_modules: string of comma separated modules string that already have been loaded clientside
    @param requires: string of comma seperated modules strings that are requested to be loaded
    """

    if exclude_modules == '':
        exclude_modules = []
    else:
        exclude_modules = exclude_modules.split(',')

    if requires == '':
        requires = []
    else:
        requires = requires.split(',')

    # a theme must be set in session, else fallback
    if not request.session.has_key('theme_module_name'):
        request.session['theme_module_name'] = 'basesoft_theme_default'
        logger.info('session has no theme set, falling back to basesoft_theme_default')

    # add base requires before any other requires
    # this is special sass logic for variables and themeing and shouldnt include real css code
    base_requires = ['default_variables_scss']
    base_requires += ['%s_scss' % request.session['theme_module_name']]
    requires = base_requires + requires

    to_hash = ''.join(requires + exclude_modules)
    if type == 'js': # we need to cache per language for JS files.
        to_hash = '%s_%s' % (to_hash, get_language())

    # make hashes unique per version to avoid bad caching
    to_hash += settings.VERSION

    # create md5 filename from requires
    file_name = '%s.%s' % (md5(to_hash).hexdigest(), type)
    abs_file = os.path.join(settings.STATIC_SQUASHED_ROOT, file_name)

    # create file if its not already known and when doesnt exist yet
    # but when debug is on always create a new file
    if settings.YUI_DEBUG_FILES or (not file_name in _cached_loaders and not os.path.exists(abs_file)):

        c = CodeSquasher(requires, exclude_modules, request, locale=get_language())

        if type == 'js':
            code = c.js_code_as_string()
        elif type == 'css':
            try:
                code = c.css_code_as_string()
            except Exception, e:

                raise Exception('Failed to compile: %s: %s' % (unicode(requires), e.message ))
        else:
            raise Exception('Invalid bootstrap type: %s' % type)

        f = open(abs_file, 'w')
        f.write(code.encode('utf-8'))
        f.close()

        _cached_loaders.append(file_name)


    if settings.LOADER_REDIRECT:
        return HttpResponsePermanentRedirect('/static/%s' % file_name)
    else:

        contents = open(abs_file).read()

        mimetype = 'text/javascript'
        if type == 'css':
            mimetype = 'text/css'

        return HttpResponse(contents, mimetype=mimetype)



import os
__bootstrap_registry_defined = False
bootstrap_registry = ''

def js_module_boot_register(request):
    """
    Loops all apps and squashes the bootstrap_register.js files. This is loaded at startup of the page so that the
    bootloader knows which applications are enabled.
    """
    global __bootstrap_registry_defined
    global bootstrap_registry
    if not __bootstrap_registry_defined:
        apps = settings.INSTALLED_APPS

        for app in apps:

            init_module = namedAny(app+'.__init__')
            path = os.path.dirname(init_module.__file__) + '/media'

            if os.path.exists(path):

                file = path + '/boot_register-min.js'
                if settings.YUI_DEBUG_FILES:
                    file = file.replace('-min.', '-debug.')

                if os.path.exists(file):
                    f = open(file, 'r')
                    bootstrap_registry += f.read()

        __bootstrap_registry_defined = True

    return bootstrap_registry

from basesoft.web.core.util.custom_query import CustomQueryRpcHandler

def custom_query(request, name):
    dispatcher = JsonRpcDispatcher()
    dispatcher.register_instance(CustomQueryRpcHandler(name, request.SessionReadOnly(), account_id=request.session['user_id']))
    return dispatcher.handle_request(request)

_handlers = {}
_handlers_loaded = False


# load_handlers_lock protects _load_handlers to be called simultaniously in different threads
load_handlers_lock = Lock()
def _load_handlers():

    load_handlers_lock.acquire()

    global _handlers_loaded, _handlers
    if _handlers_loaded:
        load_handlers_lock.release()
        return

    for app in settings.INSTALLED_APPS:
        init_module = namedAny(app+'.__init__')
        path = os.path.dirname(init_module.__file__) + '/rpc.py'

        if not os.path.exists(path):
            continue
        module = __import__(app + '.rpc', {}, {}, [''])

        if not hasattr(module, 'handlers'):
            raise Exception('%s - rpc module has no attribute handlers' % app)

        assert isinstance(module.handlers, (list, tuple))
        for handler in module.handlers:
            _handlers['%s.%s' % (app, handler.__name__)] = handler

    _handlers_loaded = True

    load_handlers_lock.release()

def rpc(request, name):

    _load_handlers()
    dispatcher = JsonRpcDispatcher()
    try:
        HandlerClass = _handlers[name]
    except IndexError:
        raise Exception('View not found: %s' % name)

    dispatcher.register_instance(HandlerClass(request))
    return dispatcher.handle_request(request)

def objects_rpc(request):
    controller = AbstractController(request.Session(), account_id=request.session['user_id'])

    class Handler:

        def mark_as_deleted(self, object_name, ids):
            return controller.mark_as_deleted(object_name, ids)

        def mark_as_archived(self, object_name, ids):
            return controller.mark_as_archived(object_name, ids)

        def unmark_as_deleted(self, object_name, ids):
            return controller.unmark_as_deleted(object_name, ids)

        def unmark_as_archived(self, object_name, ids):
            return controller.unmark_as_archived(object_name, ids)

    dispatcher = JsonRpcDispatcher()
    dispatcher.register_instance(Handler())
    return dispatcher.handle_request(request)

# ------------------------------------------------

@login_required
def download_xls(request, hash):
    """
    Download xls exported for customdatatables
    """

    path = os.path.join(settings.TMP_FILE_REPOSITORY, 'data_%s.xls' % hash)
    file = open(path, 'r')

    response = HttpResponse(file, mimetype='application/excel')
    response['Content-Disposition'] = 'attachment; filename=data_%s.xls' % date.strftime(datetime.now(), '%d-%m-%y_%H-%I')

    # remove the file
    os.unlink(path)
    return response


def download_pdf(request, url_path):
    """
    Creates a pdf document for the given url_path. This could be a compose runner in print mode
    """

    # fix for #3080: in apache environment double slashes get stripped so the url_path will arrive without first slash
    if url_path[0] != '/':
        url_path = '/' + url_path

    # build url hased on url_path. if we need external urls in the future we can add a check for this.
    protocol = 'http://'
    if request.is_secure():
        protocol = 'https://'
    url = ''.join((protocol, request.META['HTTP_HOST'], url_path))

    temp_file = os.path.join(settings.TMP_FILE_REPOSITORY, 'download_%s.pdf' % sha1(str(random.random())).hexdigest())

    footer_html = os.path.abspath(os.path.dirname(__file__)) + '/templates/pdf_footer.html'

    # Prepare the pdf convert command..
    #
    # Important: Javascript delay is set to 2 mins which should never be reached. It gives the document time to render
    # images n stuff. When the document is done it should call window.print() which tells the pdf printer to convert to pdf
    # All documents should call window.print so also headers and footers
    #

    cmd = [settings.WKHTMLTOPDF_BIN,
           "--print-media-type",
           "--javascript-delay", "120000", # 2 mins
           "--footer-right", "[page]/[toPage]",
           "--cookie","sessionid", request.session.session_key,
           "--debug-javascript",
           "--footer-html", footer_html,
           "--load-error-handling", "ignore",
           url,
           temp_file
    ]

    logger.info('download_pdf: cmd: %s' % " ".join(cmd))

    # spawn the converter process. very important is here not to use shell=True for command injection
    proc = subprocess.Popen(cmd)

    # wait till program is done
    # we dont check for error code because that seems to be unreliable. If the html contains a non existing image (404) or
    # any error code then the wkhtmltopdf returns an error code. Instead just check if it generated any pdf file.
    proc.wait()

    if not os.path.exists(temp_file):
        raise Exception('Failed to create pdf')

    # use a pdf reader to extract filename from created pdf file (its taken from html title)
    pdf_reader = PdfFileReader(open(temp_file, "r"))
    filename = "%s.pdf" % pdf_reader.getDocumentInfo().title

    # present as download
    response = HttpResponse(open(temp_file, 'r'), mimetype='application/pdf')
    response['Content-Disposition'] = 'attachment; filename="%s"' % filename # date.strftime(datetime.now(), '%d-%m-%y_%H-%I')

    return response

def download_screenshot(request, url_path):

    if settings.WKHTMLTOIMAGE_BIN is None:
        raise PublicSystemException('Cannot generate screenshot')

    # fix for #3080: in apache environment double slashes get stripped so the url_path will arrive without first slash
    if url_path[0] != '/':
        url_path = '/' + url_path

    # build url hased on url_path. if we need external urls in the future we can add a check for this.
    protocol = 'http://'
    if request.is_secure():
        protocol = 'https://'
    url = ''.join((protocol, request.META['HTTP_HOST'], url_path))

    temp_file = os.path.join(settings.TMP_FILE_REPOSITORY, 'download_%s.png' % sha1(str(random.random())).hexdigest())

    cmd = [settings.WKHTMLTOIMAGE_BIN,
           "--javascript-delay", "30000", # 1 min
           "--cookie","sessionid", request.session.session_key,
           "--debug-javascript",
           url,
           temp_file
    ]

    # spawn the converter process. very important is here not to use shell=True for command injection
    proc = subprocess.Popen(cmd)

    # wait till program is done
    # we dont check for error code because that seems to be unreliable. If the html contains a non existing image (404) or
    # any error code then the wkhtmltopdf returns an error code. Instead just check if it generated any pdf file.
    proc.wait()

    if not os.path.exists(temp_file):
        raise Exception('Failed to create screenshot')

    response = HttpResponse(open(temp_file, 'r'), mimetype='image/png')
    response['Content-Disposition'] = 'attachment; filename="screenshot.png"'

    return response
