# Copyright (c) 2009 BaseSoft B.V.
# See LICENSE for details.

"""
This defines the tables for the file app.
"""

import datetime
from sqlalchemy.orm import mapper, relation
from sqlalchemy import Column, Integer, String, ForeignKey, DateTime, UnicodeText, SmallInteger
from basesoft.sqlalchemy.table import IQTable

from basesoft.web.dev.compose import models as compose_models
from basesoft.web.contrib.account import models as account_models
from basesoft.web.contrib.license import models as license_models
from django.utils.translation import ugettext as _

from basesoft.web.contrib.license.hooks import PrivilegeObjectMapper

__mapping_defined = False
__tables_defined = False
tables = {}

def reset():
    global __mapping_defined
    global __tables_defined

    __mapping_defined = False
    __tables_defined = False

def get_tables():
    """
    """
    return tables

def do_tables(metadata):
    global __tables_defined
    if not __tables_defined:

        tblMockupCategory = IQTable('tblMockupCategory', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('label', String(255), nullable=True),
            Column('deleted', SmallInteger, nullable=False, default=0),
            Column('archived', SmallInteger, nullable=False, default=0),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )
        tblMockupCategoryGroup = IQTable('tblMockupCategoryGroup', metadata,
            Column('category_id', Integer, ForeignKey('tblMockupCategory.id', ondelete="CASCADE"), primary_key=True),
            Column('group_id', Integer, ForeignKey('tblGroup.id'), primary_key=True),
            Column('privilege', Integer, nullable=False, primary_key=True, autoincrement=False),
            Column('group_has_privilege', SmallInteger, default=0),
            Column('license_has_privilege', SmallInteger, default=0),
            Column('domain_has_privilege', SmallInteger, default=0),
            Column('world_has_privilege', SmallInteger, default=0),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        tblMockup = IQTable('tblMockup', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('name', String(255), nullable=True),
            Column('category_id', Integer, ForeignKey('tblMockupCategory.id'), nullable=False),
            Column('state', UnicodeText, nullable=True),
            Column('deleted', SmallInteger, nullable=False, default=0),
            Column('archived', SmallInteger, nullable=False, default=0),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )
        tblMockupGroup = IQTable('tblMockupGroup', metadata,
            Column('mockup_id', Integer, ForeignKey('tblMockup.id', ondelete="CASCADE"), primary_key=True),
            Column('group_id', Integer, ForeignKey('tblGroup.id'), primary_key=True),
            Column('privilege', Integer, nullable=False, primary_key=True, autoincrement=False),
            Column('group_has_privilege', SmallInteger, default=0),
            Column('license_has_privilege', SmallInteger, default=0),
            Column('domain_has_privilege', SmallInteger, default=0),
            Column('world_has_privilege', SmallInteger, default=0),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        tables['tblMockupCategory'] = tblMockupCategory
        tables['tblMockupCategoryGroup'] = tblMockupCategoryGroup
        tables['tblMockup'] = tblMockup
        tables['tblMockupGroup'] = tblMockupGroup

        __tables_defined = True

def do_mappings():
    """
    This function is only called by the django middleware
    when it is needed
    """
    global __mapping_defined
    if not __mapping_defined:

        mapper(compose_models.MockupCategory, tables['tblMockupCategory'],
            extension = PrivilegeObjectMapper(),
            properties = {
                'groups': relation(compose_models.MockupCategoryGroup, backref='category', cascade="all", passive_deletes=True)
        })
        mapper(compose_models.MockupCategoryGroup, tables['tblMockupCategoryGroup'],
            properties = {
                'group': relation(license_models.Group)
            }
        )
        mapper(compose_models.Mockup, tables['tblMockup'],
            extension = PrivilegeObjectMapper(),
            properties = {
                'category': relation(compose_models.MockupCategory),
                'groups': relation(compose_models.MockupGroup, backref='mockup', cascade="all", passive_deletes=True)
        })
        mapper(compose_models.MockupGroup, tables['tblMockupGroup'],
            properties = {
                'group': relation(license_models.Group)
            }
        )

        __mapping_defined = True

def app_permissions_category():
    """
    Returns a category for all permissions defined for this app
    """
    return license_models.PermissionCategory(u'Mockup')

def app_permissions():
    """
    The permissions used within this application. Can be sync'd with the
    `iqsyncpermissions` command.
    """
    return [
        license_models.Permission(u'develop_tools', u"""
            This gives the right to use the developer tools.
            """, _("Developer tools access")),]

def app_configs():
    """
    The account configs available for this application. This can be sync'd with the
    `iqsyncpermissions` command.
    """
    configs = []

    config = account_models.Config()
    config.name = 'basesoft.web.dev.compose.palette.hidden'
    config.type = 'boolean'
    config.default = 'False'
    config.description = u'Boolean for whether or not the Basesoft mockup compose Palette is hidden.'
    configs.append(config)

    config = account_models.Config()
    config.name = 'basesoft.web.dev.compose.action.hidden'
    config.type = 'boolean'
    config.default = 'True'
    config.description = u'Boolean for whether or not the Basesoft mockup compose Action palette is hidden.'
    configs.append(config)

    config = account_models.Config()
    config.name = 'basesoft.web.dev.compose.categories.hidden'
    config.type = 'boolean'
    config.default = 'False'
    config.description = u'Boolean for whether or not the Basesoft mockup compose Categories palette is hidden.'
    configs.append(config)

    config = account_models.Config()
    config.name = 'basesoft.web.dev.compose.right_layout_unit.width'
    config.type = 'integer'
    config.default = '250'
    config.description = u'Integer used to determine the default width (in px) of the Basesoft form compose right panel.'
    configs.append(config)

    config = account_models.Config()
    config.name = 'basesoft.web.dev.compose.right_layout_unit.collapsed'
    config.type = 'boolean'
    config.default = 'False'
    config.description = u'Boolean used to determine if the Basesoft form compose right panel should be collapsed by default.'
    configs.append(config)

    config = account_models.Config()
    config.name = 'basesoft.web.dev.compose.remove.confirmation'
    config.type = 'boolean'
    config.default = 'True'
    config.description = u'Boolean used to determine if confirmation should be requested when a module is removed from the canvas.'
    configs.append(config)

    return configs
