# Copyright (c) 2012 BaseSoft B.V.
# See LICENSE for details.

from sqlalchemy import and_
from basesoft.web.contrib.license.models import License
from basesoft.web.contrib.contact.controllers import SimpleContactController, ImportContactController
from basesoft.web.contrib.contact.models import Contact, ContactList, ContactListGroup
from iqportal.client.core.models import FMECompany, FMEPerson, FMEContact
from django.utils.translation import ugettext as _
from basesoft.web.core.exceptions import SecurityException
from basesoft.web.core.util.controllers import transaction_decorator

class FMEContactController(SimpleContactController):
    """
    Controller for the polymorphic type FMEContact
    """
    _instance_class = FMEContact

    def _load_person_state(self, id):
        person = self.get_person(id)
        person_data = SimpleContactController._load_person_state(self, person)
        person_data['person_number'] = person.person_number
        return person_data

    def set_import_person_state(self, person_data):
        person = SimpleContactController.set_import_person_state(self, person_data)
        if person_data.has_key('person_number'):
            person['person_number'] = person_data['person_number']
        return person

    def _get_import_person_object_filter(self, person_data, unique_fields):
        if len(unique_fields) == 0:
            return None, None, person_data
        has_filter, filter, qfrom, person_data = SimpleContactController._get_import_person_object_filter(self, person_data, unique_fields)

        for uf in unique_fields:
            if uf in FMEPerson.cols_i18n.keys() and person_data.has_key(uf):
                filter = and_(filter, getattr(FMEPerson, uf)==person_data[uf])
                has_filter = True

        return has_filter, filter, qfrom, person_data

    def _set_company(self, id, company_data, instance_class=None):
        company = SimpleContactController._set_company(self, id, company_data)
        for key in ["company_number", "no_of_employees", "department_number", "busrelaccount", "CAO"]:
            if company_data.has_key(key):
                if company_data[key] == '':
                    company_data[key] = None
                setattr(company, key, company_data[key])
        return company

    def _load_company_state(self, id):
        company = self.get_company(id)
        company_data = SimpleContactController._load_company_state(self, company)
        company_data['company_number'] = company.company_number
        return company_data

    def get_company_cols(self):
        """
        Used for contact import. Returns a list with this company type columns.
        """
        company = SimpleContactController.get_company_cols(self)
        for key, v in FMECompany.cols_i18n.items():
            company.append({"value": key, "label": v})
        return company

    def company_required_columns(self):
        return ['company_number', 'person_number', 'contact_list_name']

    def set_import_company_state(self, company_data):
        company = SimpleContactController.set_import_company_state(self, company_data)
        if company_data.has_key('company_number'):
            company['company_number'] = company_data['company_number']
        return company

    def _get_import_company_object_filter(self, company_data, unique_fields):
        if len(unique_fields) == 0:
            return None, None, company_data
        has_filter, filter, qfrom, company_data = SimpleContactController._get_import_company_object_filter(self, company_data, unique_fields)

        for uf in unique_fields:
            if uf in FMECompany.cols_i18n.keys() and company_data.has_key(uf):
                filter = and_(filter, getattr(FMECompany, uf)==company_data[uf])
                has_filter = True

        return has_filter, filter, qfrom, company_data

    def get_field_plugins(self):
        """
        Used for contact import
        """
        from basesoft.web.core.input.validation import plugins
        fp, gp = SimpleContactController.get_field_plugins(self)
        fp['company_number'] = [(plugins.Number, {})]
        fp['person_number'] = [(plugins.Number, {})]
        return fp, gp

    def validate_state(self, state):
        """
        Used for contact import: person number, company number and contact list are always required for an FME import.
        """
        returns = SimpleContactController.validate_state(self, state)

        if not isinstance(state, list):
            states = [state]
        else:
            states = state

        i = 0
        for state in states:
            ret = returns[i]
            i = i + 1
            if not state.has_key('company'):
                ret[''] = _("No company information was provided.")
                continue
            if not state['company'].has_key('company_number'):
                ret[''] = _("FME Company Number was not provided")
                continue
            if not state.has_key('person_number'):
                ret[''] = _("FME Person Number was not provided")
                continue
            if not state.has_key('contact_list_name'):
                ret[''] = _("Contact list is required.")
                continue
            if state['company']['company_number'] in ['', None]:
                ret['company_number'] = _("Invalid FME Company Number.")
            if state['person_number'] in ['', None]:
                ret['person_number'] = _("Invalid FME Person Number.")
            if state['contact_list_name'] in ['', None]:
                ret['contact_list_name'] = _("Invalid Contact list.")
            license, cl_name = self.check_contact_list_license(state['contact_list_name'])
            if not isinstance(license, License):
                ret['contact_list_name'] = license

        return returns

    def check_contact_list_license(self, contact_list):
        # split contact list
        pos = contact_list.find(":")
        if pos < 0:
            return (_("Invalid Contact list format (<license name>: <contact list name>)"), None)

        license_name = contact_list[:pos].strip()
        contact_list_name = contact_list[pos+1:].strip()

        #search for the license
        license = self.sa_session.query(License).filter(and_(License.domain==self._get_default_group().license.domain,
                                                        License.name==license_name)).first()
        if license is None:
            return (_('Invalid license name: %s' % license_name), None)

        # we need to check if the found license is in this account's licenses
        for group in self.get_active_groups():
            if license.id == group.license.id:
                return (license, contact_list_name)
        return (_('Invalid license name: %s' % license_name), None)


class FMEImportContactController(ImportContactController):
    """
    Overwrite contact controller in order to split contact lists from <license name>: <contact list name> and save per license.
    """
    privilege_save_handler = False

    def get_simple_contact_controller(self, objectName):
        """
        Overwrite so it doesn't search all apps again
        """
        return FMEContactController

    @transaction_decorator
    def save_or_update_contact(self, id, person_data, company_data=None, group_id=None, permissions_checks=True, save_contact_privs=True, contact_data=None):
        """
        Make sure it doesn't save contact privs.
        """
        # call parent with save_privs false so it doesn't save privs for current group - FME has contact list required
        # in the file, so saving contacts privs will be taken care of in the method for linking contacts to contact lists
        contact = ImportContactController.save_or_update_contact(self, id, person_data, company_data, group_id, permissions_checks, False, contact_data=contact_data)
        # save update priv on domain level because on each it needs to be found when setting unique fields
        self.grant_privilege_to(contact, Contact.PRIV_UPDATE, domain=True,
                                target_group=self.get_default_group(), check_own_privs=False)
        return contact

    @transaction_decorator
    def link_contacts(self, contacts, contact_list, group_id=None, check_contacts_privs=True):
        """
        Overwrite in order to split contact list name, save it for the correct license and then update contacts privs
        for that license
        """
        assert isinstance(contacts, list)
        self.assert_permissions('link_to_contact_list')
        assert isinstance(contact_list, unicode) or isinstance(contact_list, str)

        license, contact_list_name = self.simple_contact_controller.check_contact_list_license(contact_list)
        # if we get a str, that's an exception
        if not isinstance(license, License):
            raise SecurityException(license)

        # get contact list for that specific license
        filter, fo = self.get_query_filter_on_privileges(ContactList, ContactListGroup, ContactList.PRIV_LINK_CONTACTS,
                                                         groups=[license.default_group])
        filter = and_(ContactList.name==contact_list_name, filter)

        cl = self.sa_session.query(ContactList).select_from(fo).filter(filter).first()
        if cl is None:
            self.assert_permissions('create_contact_list')
            cl = ContactList()
            cl.name = contact_list_name
            # grant contact list privs on license level for the found license
            self.grant_privilege_to(cl, ContactList.all_privileges, license=True,
                                    target_group=license.default_group, check_own_privs=False)

        contact_list = cl

        #only add the ones that are not already linked to the contact list
        distinct_contacts = contact_list.contacts
        has_new = False
        for c in contacts:
            assert isinstance(c, Contact)
            # grant contact privs on license level for the found license
            self.grant_privilege_to(c, Contact.all_privileges, license=True,
                                    target_group=license.default_group, check_own_privs=False)
            try:
                distinct_contacts.index(c)
            except ValueError:
                distinct_contacts.append(c)
                has_new = True

        contact_list.contacts = distinct_contacts
        self.sa_session.add(contact_list)

        if has_new:
            connection = self.sa_session.connection()
            # set correct is_processed for this contact list for all online publications
            # so new invitees will be created
            connection.execute("""UPDATE tblComposePublicationContactList pcl
                INNER JOIN tblComposePublication p ON p.id=pcl.publication_id
                SET is_processed=3
                WHERE is_processed=2 AND p.status=2 AND p.archived=0 AND
                p.deleted=0 AND pcl.contact_list_id=%s""" % contact_list.id)
        return contact_list

contact_controllers = {'FMEContact': FMEContactController}
import_contact_controllers = {'FMEContact': FMEImportContactController}
