# Copyright (c) 2010 BaseSoft B.V.
# See LICENSE for details.

from django.utils.translation import ugettext as _
from sqlalchemy import case, func, and_

from basesoft.web.core.util.custom_query import CustomQuery, CustomQueryField, CustomQueryModel, DATA_TYPE_BOOLEAN

from iqportal.iqreport.core.models import Report, ReportPublication
from iqportal.iqreport.display.models import ReportViewer
from basesoft.web.contrib.compose.models import ComposeInvitee, ComposePublicationContactList
from basesoft.web.contrib.contact.models import ContactListContact, ContactList, Contact, Person, Company

class ReportCustomQuery(CustomQuery):

    def tables(self):
        return (
            CustomQueryModel(Report),
        )

    def fields(self):
        return (
            CustomQueryField(Report.id, _('ID'), key='id'),
            CustomQueryField(Report.name, _('Name'), key='name', is_filter=True),
            CustomQueryField(Report.created, _('Created'), key='created'),
            CustomQueryField(Report.last_updated, _('Last updated'), key='last_updated')
        )

class ReportPublicationCustomQuery(CustomQuery):

    def tables(self):
        return (
            CustomQueryModel(ReportPublication),
            CustomQueryModel(Report, onclause=Report.id==ReportPublication.report_id, priv=None),
            CustomQueryModel(ComposePublicationContactList,
                             onclause=ReportPublication.id==ComposePublicationContactList.publication_id,
                             use_outerjoin=True),
            CustomQueryModel(ContactList,
                             onclause=ContactList.id==ComposePublicationContactList.contact_list_id,
                             use_outerjoin=True, priv=None),
            CustomQueryModel(ContactListContact,
                             onclause=ContactList.id==ContactListContact.contact_list_id,
                             use_outerjoin=True),
            CustomQueryModel(Contact,
                             onclause=ContactListContact.contact_id==Contact.id,
                             use_outerjoin=True, priv=None),
            CustomQueryModel(ComposeInvitee,
                             onclause=ReportPublication.id==ComposeInvitee.publication_id,
                             use_outerjoin=True),
            CustomQueryModel(ReportViewer,
                             onclause=ComposeInvitee.id==ReportViewer.invitee_id, use_outerjoin=True)
        )

    def fields(self):
        return (
            CustomQueryField(ReportPublication.id, _('ID'), key="id"),
            CustomQueryField(Report.name, _('Report'), key="report", is_filter=True, is_group_by=True),
            CustomQueryField(Report.id, _('Report ID'), key="report_id"),
            CustomQueryField(ReportPublication.label, _('Label'), key="label", is_filter=True),
            CustomQueryField(ReportPublication.polymorphic_type, _('Type'), key="type"),
            CustomQueryField(ReportPublication.begin_date, _('Begin date'), key="begin_date", is_filter=True),
            CustomQueryField(ReportPublication.end_date, _('End date'), key="end_date", is_filter=True),

            # if publication is new or scheduled, the invitees list will be all the contacts from the
            # publication's contact lists

            # if publication is online or finished, we get its invitees list from the invitees table;
            # after a publication is online, even if the contacts from its contact lists change, the invitees
            # will not change, they will still be just the ones from tblComposeInvitee created when publication
            # went online
            CustomQueryField(case([(ReportPublication.status.in_([ReportPublication.STATUS_NEW,
                                                                  ReportPublication.STATUS_SCHEDULED]),
                             func.count(ContactListContact.contact_id.distinct()))],
                             else_=func.count(ComposeInvitee.contact_id.distinct())),
                             _("Invitees"), key="invitees"),
            CustomQueryField(func.count(ReportViewer.id.distinct()), _("Viewers"), key="viewers"),
            CustomQueryField(ReportPublication.status, _('Status'), key="status", is_filter=True),
            CustomQueryField(ReportPublication.cache_status, _('Cache status'), key="cache_status"),
            CustomQueryField(ReportPublication.anonymous, _('Anonymous'), key="anonymous", is_filter=True, type=DATA_TYPE_BOOLEAN),
            CustomQueryField(ReportPublication.anonymous_hash, _('Report link'), key="report_link"),

            CustomQueryField(Report.id, _('Preview'), key="preview"),
        )

    group_by = (ReportPublication.id, )

class PublicationInviteesCustomQuery(CustomQuery):

    def tables(self):
        return (
            CustomQueryModel(Report),
            CustomQueryModel(ReportPublication,
                             onclause=Report.id==ReportPublication.report_id),
            CustomQueryModel(ComposeInvitee,
                             onclause=ComposeInvitee.publication_id==ReportPublication.id),
            CustomQueryModel(ReportViewer,
                             onclause=ComposeInvitee.id==ReportViewer.invitee_id, use_outerjoin=True),
            CustomQueryModel(Contact, onclause=ComposeInvitee.contact_id==Contact.id, use_outerjoin=True, show_archived=True),
            CustomQueryModel(Person, onclause=Contact.person_id==Person.id, use_outerjoin=True),
            CustomQueryModel(Company, onclause=Contact.company_id==Company.id, use_outerjoin=True),
        )

    def fields(self):
        return (
            CustomQueryField(ComposeInvitee.id, _('Invitee ID'), key='invitee_id'),
            CustomQueryField(func.concat(case([(Person.first_name!=None, Person.first_name)], else_=""),
                                        " ",
                                        case([(Person.middle_name!=None, Person.middle_name)], else_=""),
                                        " ",
                                        case([(Person.last_name!=None, Person.last_name)], else_=""),
                                        ), _('Invitee'), key='invitee'),
            CustomQueryField(Company.name, _('Company'), key='company'),
            CustomQueryField(Person.email, _('Email'), key='email'),
            CustomQueryField(ReportViewer.number_of_views, _('No. of views'), key='views'),
            CustomQueryField(ComposeInvitee.invitee_hash, _('Invitee hash'), key='invitee_hash'),
            CustomQueryField(ReportPublication.id, _('Publication ID'), key='publication_id'),
        )

    def set_where(self):
        self.where = and_(ComposeInvitee.contact_id!=None, Report.deleted==False, ReportPublication.deleted==False)

    group_by = (ComposeInvitee.id, )

custom_queries = (ReportCustomQuery, ReportPublicationCustomQuery, PublicationInviteesCustomQuery, )