
from django.utils import simplejson as json, simplejson
from django.utils.translation import ugettext as _

from basesoft.web.core.jsonrpc.dispatcher import json_object_hook

from basesoft.web.contrib.i18n.controllers import I18nController
from basesoft.web.contrib.i18n.models import Locale
from basesoft.web.contrib.license.controllers import LicenseController

from basesoft.web.contrib.account.decorators import assert_user_logged_in
from basesoft.web.core.jsonrpc.dispatcher import JsonRpcDispatcher
from basesoft.web.contrib.compose.views import compose_runner

from iqportal.iqreport.core.models import Report
from iqportal.iqreport.compose.controllers import ComposeController
from basesoft.util.ordered_dict import OrderedDict
from iqportal.iqube.core.controllers import ConfigurableWidgetController
from iqportal.iqube.core.models import DataSourceWidget
from iqportal.iqube.core.dashboard import ConfigurableWidget
from basesoft.web.core.exceptions import PublicSecurityException


class ComposeRPCHandler:
    """
    Handles rpc calls from the survey compose application. Required is to be logged in
    url: /iqportal.iquest.compose/rpc/
    """

    def __init__(self, session, locale, account_id, group_id):

        self.account_id = account_id
        self.session = session
        self.group_id = group_id
        self.locale = locale

        # instanciate a compose controller
        self.controller = ComposeController(session, locale, account_id)

        # a license controller we'll use to get the group
        self.lc = LicenseController(session, locale, account_id)

    def get_compose_vars(self):
        """
        called before the manager is instanciated and returns all information that is needed for that..
        like localeLists, themes, last saved survey etc
        """

        i18n_controller = I18nController(self.session, self.locale, account_id=self.account_id)

        # get last report_id
        last_report_id = None
        list = self.controller.get_reports(order_by=Report.last_updated)
        if len(list) > 0:
            last_report_id = list[len(list)-1].id

        tl = [];
        for t in self.controller.get_themes():
            gc = None
            graph_theme_module = None
            graph_theme_css_class = None
            if t.graph_theme is not None:
                graph_theme_module = t.graph_theme.module
                graph_theme_css_class = t.graph_theme.css_class
                if t.graph_theme.graph_config is not None:
                    gc = json.loads(t.graph_theme.graph_config, object_hook=json_object_hook)
            tl.append({
                'name': t.name,
                'module': t.module,
                'cssClass': t.css_class,
                'graph_theme_module': graph_theme_module,
                'graph_theme_cssClass': graph_theme_css_class,
                'graph_config': gc,
            })

        return {
            'lastID': last_report_id,
            'localeList': [{'name': locale.name, 'locale': locale.locale}
                           for locale in i18n_controller.get_domain_locales(availability=Locale.AVAILABLE_FOR_COMPOSE)],
            'themeList': tl
        }

    def get_reports(self):
        """
        Returns a list of reports that the current group has access to. Used for selections in forms.
        """
        return [{'label': report.name, 'value': report.id} for report in self.controller.get_reports()]

    def get_last_modified_reports(self, limit=5):
        return [{'name': report.name, 'id': report.id} for report in self.controller.get_reports(order_by=Report.last_updated.desc())[:limit]]

    def save(self, state, group_id=None):
        """
        Receives a state object and saves the report using the controller
        It returns a dict of domId's with new/updated id's back to the application.
        """
        return self.controller.save_state(state, group_id)

    def mark_as_deleted(self, report_id):
        """
        Mark the supplied survey as deleted.
        """
        return self.controller.mark_as_deleted(report_id)

    def load(self, report_id, publication_id=None):
        """
        Receives a suvey_id and loads the survey using the controller.
        It returns a state of the survey
        """
        return self.controller.load_state(report_id, publication_id)

    def get_datasources(self):
        """
        returns a list of datasources of which the current user can select to create his report from
        """
        from iqportal.iqube.core.controllers import DataSourceController
        dsc = DataSourceController(self.session, account_id=self.account_id)
        return [ {'id': ds.id, 'name': ds.name } for ds in dsc.get_datasources() ]

    def load_dynamic_items(self):
        """
        Dynamic items can be Configurable Widgets.
        """
        items = OrderedDict()
        cwc = ConfigurableWidgetController(self.session, account_id=self.account_id)
        if cwc.has_permissions('view_datasource_widget'):
            for widget in cwc.get_configurable_widgets(availability=DataSourceWidget.AVAILABLE_AS_REPORT_ITEM):
                # make a dynamic class (extend PivotWidget) for each mdx result
                name = 'ReportConfigurableWidget%s' % widget.id
                config, applied_filters = cwc.get_state(widget)
                requires = []
                for i in config['dimensions']:
                    if i.has_key('fancy_select_module') and i['fancy_select_module'] not in requires:
                        requires.append(i['fancy_select_module'])

                if config.has_key('icon'):
                    icon = config['icon']
                else:
                    icon = ConfigurableWidget.icon

                cls = {
                    'name': 'configurable_report_item',
                    'label': widget.title,
                    'icon': icon,
                    'app': widget.application,
                    'category': widget.category,
                    'requires': requires,
                    'params': {
                        'datasource_widget_id': widget.id,
                        'config': config,
                        'appliedConfig': applied_filters,
                        'graphConfig': config.has_key('graph_config') and config['graph_config'] or {},
                        'tableConfig': {'isTransposed': False, 'slicersExpanded': False}
                    }
                }
                items.update([(name, cls)])
        return items


    def create_pool_item(self, result_id, title):
        return




@assert_user_logged_in
def rpc(request):
    """View that handles all rpc calls"""

    dispatcher = JsonRpcDispatcher()

    locale = request.session['config']['basesoft.web.contrib.locale']

    if request.session.has_key('user_id'):
        account_id = request.session['user_id']
    else:
        account_id = None

    if request.session.has_key('group_id'):
        group_id = request.session['group_id']
    else:
        group_id = None

    dispatcher.register_instance(ComposeRPCHandler(request.Session(), locale, account_id, group_id))

    return dispatcher.handle_request(request)

def preview_report(request, params_str):
    """
    called by the iqreport preview panel in an iframe
    """

    vars_ = simplejson.loads(params_str)

    locale = request.session['config']['basesoft.web.contrib.locale']
    account_id = request.session['user_id']

    c = ComposeController(request.Session(), locale, account_id)
    report = c.get_reports(id=vars_['runner_id'])

    if len(report) == 0:
        raise PublicSecurityException(_('No access to provided report'))

    report = report.pop()

    if vars_.has_key('publication_id') and vars_['publication_id'] is not None:
        # get the publication so we can correctly load the cache for the report.
        p = c.get_report_publication(vars_['publication_id']) # this validates view privs on the publication
        vars_['publication_label'] = p.label
        request.session['report_publication_id'] = p.id
    elif request.session.has_key('report_publication_id'):
        del request.session['report_publication_id']

    if report.groups[0].group.license.self_branded:
        licensee = report.groups[0].group.license.http_host
    else:

        lc = LicenseController(request.Session())
        site_licensee = lc.get_license_for_request(request)
        licensee = site_licensee.http_host

    requires = ('iqreport_init', )
    title = 'IQReport - %s' % report.name
    vars_['compose_name'] = report.name

    # if an invite previously opened its own report, the viewer's id was stored in session
    # and now on preview that one will be used without any privs checks
    # so make sure a viewer id is not used when opening a report preview
    if request.session.has_key('report_viewer_id'):
        del request.session['report_viewer_id']

    return compose_runner(request, requires, vars_, title, licensee)

