# Copyright (c) 2009 BaseSoft B.V.
# See LICENSE for details.

"""
This file contains the objects, functions and variables to setup the
IQReport core application.
"""

import datetime
from sqlalchemy import Column, Integer, ForeignKey, String, DateTime, Unicode, UnicodeText, SmallInteger
from sqlalchemy.dialects.mysql import MEDIUMTEXT
from basesoft.sqlalchemy.table import IQTable

from basesoft.web.contrib.license.hooks import PrivilegeObjectMapper
from basesoft.web.contrib.license import models as license_models
from django.utils.translation import ugettext as _
from sqlalchemy.orm import backref
from basesoft.web.contrib.i18n.models import Locale
from sqlalchemy.types import Boolean

__mapping_defined = False
__tables_defined = False
tables = {}

def reset():
    global __mapping_defined
    global __tables_defined

    __mapping_defined = False
    __tables_defined = False

def get_tables():
    return tables

def do_tables(metadata):
    global __tables_defined
    if not __tables_defined:

        tblIQReportMail = IQTable('tblIQReportMail', metadata,
            Column('mail_id', Integer, ForeignKey('tblMail.id', ondelete='CASCADE'), primary_key=True),
            Column('fallback_locale_id', Integer, ForeignKey('tblLocale.id'), nullable=False),
        )

        # Contains reports
        tblReport = IQTable('tblReport', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),

            # filename of the report, this should become unique per customer or something
            Column('name', String(255), nullable=False),

            # theme gets mapped in the state controller againt a dict to get its css_class and name
            Column('report_theme_id', Integer, ForeignKey('tblReportTheme.id'), nullable=False),

            # when a user opens a report the report can be presented in his locale
            # if there are no translations it can fallback to the fallback_locale
            Column('fallback_locale_id', Integer, ForeignKey('tblLocale.id'), nullable=False),

            # mark as deleted
            Column('deleted', SmallInteger, nullable=False, default=0),
            Column('archived', SmallInteger, nullable=False, default=0),

            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
            )

        # This is the link between a group of accounts and the report, this will be used for access rights.
        tblReportGroup = IQTable('tblReportGroup', metadata,
            Column('report_id', Integer, ForeignKey('tblReport.id', ondelete="CASCADE"), primary_key=True),
            Column('group_id', Integer, ForeignKey('tblGroup.id'), primary_key=True),
            Column('privilege', Integer, nullable=False, primary_key=True, autoincrement=False),
            Column('group_has_privilege', SmallInteger, default=0),
            Column('license_has_privilege', SmallInteger, default=0),
            Column('domain_has_privilege', SmallInteger, default=0),
            Column('world_has_privilege', SmallInteger, default=0),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
            )

        # Holds the Locale texts for Report
        tblReportLocale = IQTable('tblReportLocale', metadata,
            Column('report_id', Integer, ForeignKey('tblReport.id', ondelete='CASCADE'), primary_key=True),
            Column('locale_id', Integer, ForeignKey('tblLocale.id'), primary_key=True),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
            )

        # Blocks go into a Report
        tblReportBlock = IQTable('tblReportBlock', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('report_id', Integer, ForeignKey('tblReport.id', ondelete='CASCADE'), nullable=False),
            Column('position', Integer, nullable=False),
            Column('page_end', SmallInteger, nullable=False, default=False),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
            )

        # Columns go in blocks
        tblReportBlockColumn = IQTable('tblReportBlockColumn', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('block_id', Integer, ForeignKey('tblReportBlock.id', ondelete='CASCADE'), nullable=False),
            Column('position', Integer, nullable=False),
            Column('width_percentage', SmallInteger, nullable=True),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
            )

        # Items go in blocks. Items are abstract and need to be inherited (like Html or QA)
        tblReportItem = IQTable('tblReportItem', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('column_id', Integer, ForeignKey('tblReportBlockColumn.id', ondelete='CASCADE'), nullable=False),
            Column('polymorphic_type', String(255), nullable=False),
            Column('position', Integer, nullable=False),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
            )

        # Html item contains just html text
        tblReportItemHtml = IQTable('tblReportItemHtml', metadata,
            Column('item_id', Integer, ForeignKey('tblReportItem.id', ondelete='CASCADE'), primary_key=True),
            Column('is_wiki', SmallInteger, nullable=False, default=1),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
            )

        # Locale texts for HtmlItem
        tblReportItemHtmlLocale = IQTable('tblReportItemHtmlLocale', metadata,
            Column('item_html_id', Integer, ForeignKey('tblReportItemHtml.item_id', ondelete='CASCADE'), primary_key=True),
            Column('locale_id', Integer, ForeignKey('tblLocale.id'), primary_key=True),
            Column('html', UnicodeText, nullable=False),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        tblReportItemDynamicTable = IQTable('tblReportItemDynamicTable', metadata,
            Column('item_id', Integer, ForeignKey('tblReportItem.id', ondelete='CASCADE'), primary_key=True),
            Column('data', UnicodeText, nullable=False),
            Column('title', UnicodeText, nullable=True),
            Column('show_title', SmallInteger, nullable=False, default=1),
            Column('show_table', SmallInteger, nullable=False, default=1),
            Column('show_graph', SmallInteger, nullable=False, default=0),
            Column('graph_config', UnicodeText, nullable=True),
        )

        # IQube Configurable Widget report entry
        tblReportItemDataSourceWidget = IQTable('tblReportItemDataSourceWidget', metadata,
            Column('item_id', Integer, ForeignKey('tblReportItem.id', ondelete='CASCADE'), primary_key=True),
            Column('datasource_widget_id', Integer, ForeignKey('tblDataSourceWidget.id', ondelete="CASCADE"), nullable=True),
            Column('datasource_cube_id', Integer, ForeignKey('tblDataSourceCube.id', ondelete="CASCADE"), nullable=True),
            Column('title', UnicodeText, nullable=True),
            Column('show_title', SmallInteger, nullable=False, default=1),
            Column('applied_config', MEDIUMTEXT, nullable=False),
            Column('graph_config', UnicodeText, nullable=True),
            Column('table_config', UnicodeText, nullable=True),
            Column('linked_items', UnicodeText, nullable=True),
        )

        tblIQReportGraphType = IQTable('tblIQReportGraphType', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('module', String(255), nullable=False),
            Column('name', String(255), nullable=False),
            Column('label', Unicode(255), nullable=False),
            Column('is_always_available', SmallInteger, nullable=False, default=0),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        tblIQReportGraphTypeDatasource = IQTable('tblIQReportGraphTypeDatasource', metadata,
            Column('graph_type_id', ForeignKey('tblIQReportGraphType.id', ondelete='CASCADE'), primary_key=True),
            Column('datasource_id', Integer, ForeignKey('tblDataSource.id'), primary_key=True),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        # Theme stuff
        tblReportTheme = IQTable('tblReportTheme', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('name', String(255), nullable=False, unique=True), # name like 'Basesoft Theme'.. selected by the user
            Column('module', String(255), nullable=True), # module name to load from the loader
            Column('css_class', String(255), nullable=True), # class name in the css file to apply to the survey
            Column('graph_theme_id', ForeignKey('tblGraphTheme.id'), nullable=True), # GraphTheme that has informations about how the graphs look

            Column('deleted', SmallInteger, nullable=False, default=0),
            Column('archived', SmallInteger, nullable=False, default=0),

            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        tblReportThemeGroup = IQTable('tblReportThemeGroup', metadata,
            Column('report_theme_id', Integer, ForeignKey('tblReportTheme.id', ondelete='CASCADE'), primary_key=True),
            Column('group_id', Integer, ForeignKey('tblGroup.id'), primary_key=True),
            Column('privilege', Integer, nullable=False, primary_key=True, autoincrement=False),
            Column('group_has_privilege', SmallInteger, default=0),
            Column('license_has_privilege', SmallInteger, default=0),
            Column('domain_has_privilege', SmallInteger, default=0),
            Column('world_has_privilege', SmallInteger, default=0),

            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        tblIQReportDashboardDynamicTable = IQTable('tblIQReportDashboardDynamicTable', metadata,
            Column('widget_id', Integer, ForeignKey('tblDashboardWidget.id', ondelete='CASCADE'), primary_key=True),
            Column('data', UnicodeText, nullable=False),
            Column('show_table', SmallInteger, nullable=False, default=1),
            Column('show_graph', SmallInteger, nullable=False, default=0),
            Column('graph_config', UnicodeText, nullable=True),
        )

        tblReportPublication = IQTable('tblReportPublication', metadata,
            Column('publication_id', Integer, ForeignKey('tblComposePublication.id'), primary_key=True),
            Column('report_id', Integer, ForeignKey('tblReport.id'), nullable=False),
            Column('cache_status', SmallInteger, nullable=False, default=0),
            Column('cache_role_name', String(255), nullable=True)
        )

        tblIQReportGraphPreset = IQTable('tblIQReportGraphPreset', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('name', String(255), nullable=False),
            Column('graph_lib', SmallInteger, nullable=False), # rgraph, reportlab, matplot, etc.
            Column('config', UnicodeText, nullable=True),

            Column('deleted', SmallInteger, nullable=False, default=0),
            Column('archived', SmallInteger, nullable=False, default=0),

            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )
        tblIQReportGraphPresetGroup = IQTable('tblIQReportGraphPresetGroup', metadata,
            Column('graph_preset_id', Integer, ForeignKey('tblIQReportGraphPreset.id', ondelete='CASCADE'), primary_key=True),
            Column('group_id', Integer, ForeignKey('tblGroup.id'), primary_key=True),
            Column('privilege', Integer, nullable=False, primary_key=True, autoincrement=False),
            Column('group_has_privilege', SmallInteger, default=0),
            Column('license_has_privilege', SmallInteger, default=0),
            Column('domain_has_privilege', SmallInteger, default=0),
            Column('world_has_privilege', SmallInteger, default=0),

            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )


        tables['tblIQReportMail'] = tblIQReportMail

        tables['tblReport'] = tblReport
        tables['tblReportGroup'] = tblReportGroup
        tables['tblReportLocale'] = tblReportLocale
        tables['tblReportBlock'] = tblReportBlock
        tables['tblReportBlockColumn'] = tblReportBlockColumn
        tables['tblReportItem'] = tblReportItem
        tables['tblReportItemHtml'] = tblReportItemHtml
        tables['tblReportItemHtmlLocale'] = tblReportItemHtmlLocale
        tables['tblReportItemDynamicTable'] = tblReportItemDynamicTable
        tables['tblReportItemDataSourceWidget'] = tblReportItemDataSourceWidget
        tables['tblReportTheme'] = tblReportTheme
        tables['tblReportThemeGroup'] = tblReportThemeGroup
        tables['tblReportPublication'] = tblReportPublication

        tables['tblIQReportGraphType'] = tblIQReportGraphType
        tables['tblIQReportGraphTypeDatasource'] = tblIQReportGraphTypeDatasource

        tables['tblIQReportDashboardDynamicTable'] = tblIQReportDashboardDynamicTable

        tables['tblIQReportGraphPreset'] = tblIQReportGraphPreset
        tables['tblIQReportGraphPresetGroup'] = tblIQReportGraphPresetGroup

        __tables_defined = True

def do_mappings():
    """
    This function is called by Django middleware when needed.
    """
    global __mapping_defined
    if not __mapping_defined:
        from sqlalchemy.orm import mapper, relation
        from iqportal.iqreport.core import models as core_models
        from basesoft.web.contrib.mailing import models as mailing_models
        from basesoft.web.contrib.mailing.sa_datamapper import get_tables as mail_tables
        from basesoft.web.contrib.compose import models as compose_models
        from iqportal.iqube.core import models as iqube_models
        from basesoft.web.contrib.i18n import models as i18n_models
        from basesoft.web.contrib.graph import models as graph_models
        from basesoft.web.contrib.dashboard.models import DashboardWidget

        from basesoft.web.contrib.dashboard.sa_datamapper import get_tables as dashboard_get_tables
        dashboard_tables = dashboard_get_tables()

        mapper(core_models.IQReportMail, tables['tblIQReportMail'],
            properties = {
                'fallback_locale': relation(i18n_models.Locale),
                'locales': relation(mailing_models.MailLocale, order_by=mail_tables()['tblMailLocale'].c.locale_id, passive_deletes=True)
            },
            polymorphic_identity='IQReportMail',
            inherits=mailing_models.Mail
        )

        mapper(core_models.Report, tables['tblReport'],
            extension = PrivilegeObjectMapper(),
            properties = {
                'report_theme': relation(core_models.ReportTheme),
                'fallback_locale': relation(i18n_models.Locale),
                'available_locales': relation(Locale, secondary=tables['tblReportLocale']),
                'locales': relation(core_models.ReportLocale, order_by=tables['tblReportLocale'].c.locale_id),
                'blocks': relation(core_models.ReportBlock, order_by=tables['tblReportBlock'].c.position),
                'groups': relation(core_models.ReportGroup, backref='report', cascade="all", passive_deletes=True),
        })

        mapper(core_models.ReportGroup, tables['tblReportGroup'], properties = {
            'group': relation(license_models.Group)
        })

        mapper(core_models.ReportLocale, tables['tblReportLocale'], properties = {
            'report': relation(core_models.Report),
            'locale': relation(i18n_models.Locale)
        })

        mapper(core_models.ReportBlock, tables['tblReportBlock'], properties = {
            'report': relation(core_models.Report),
            'columns': relation(core_models.ReportBlockColumn, passive_deletes=True, order_by=tables['tblReportBlockColumn'].c.position)
        })

        mapper(core_models.ReportBlockColumn, tables['tblReportBlockColumn'], properties = {
            'block': relation(core_models.ReportBlock)
        })

        mapper(core_models.ReportItem, tables['tblReportItem'],
            properties = {
                'column': relation(core_models.ReportBlockColumn)
            },
            polymorphic_on=tables['tblReportItem'].c.polymorphic_type,
            polymorphic_identity='Item'
        )

        mapper(core_models.ReportItemHtml, tables['tblReportItemHtml'],
            properties = {
                'locales': relation(
                    core_models.ReportItemHtmlLocale,
                    order_by=tables['tblReportItemHtmlLocale'].c.locale_id,
                    passive_deletes=True
                ),
            },
            polymorphic_identity='ItemHtml',
            inherits=core_models.ReportItem
        )

        mapper(core_models.ReportItemHtmlLocale, tables['tblReportItemHtmlLocale'],
            properties = {
                'item_html': relation(core_models.ReportItemHtml),
                'locale': relation(i18n_models.Locale)
            }
        )

        mapper(core_models.ReportItemDynamicTable, tables['tblReportItemDynamicTable'],
            polymorphic_identity='ItemDynamicTable',
            inherits=core_models.ReportItem
        )

        mapper(core_models.ReportItemDataSourceWidget, tables['tblReportItemDataSourceWidget'],
            properties = {
                'datasource_widget': relation(iqube_models.DataSourceWidget),
                'datasource_cube': relation(iqube_models.DataSourceCube)
            },
            polymorphic_identity='ReportItemDataSourceWidget',
            inherits=core_models.ReportItem
        )

        mapper(core_models.IQReportGraphType, tables['tblIQReportGraphType'],
            properties = {}
        )

        mapper(core_models.IQReportGraphTypeDatasource, tables['tblIQReportGraphTypeDatasource'],
            properties = {
                'graph_type': relation(core_models.IQReportGraphType),
                'datasource': relation(iqube_models.DataSource),
            }
        )

        mapper(core_models.ReportTheme, tables['tblReportTheme'],
            extension = PrivilegeObjectMapper(),
            properties = {
                'graph_theme': relation(graph_models.GraphTheme),
                'groups': relation(core_models.ReportThemeGroup, backref='report_theme', cascade="all", passive_deletes=True)
        })

        mapper(core_models.ReportThemeGroup, tables['tblReportThemeGroup'], properties = {
            'group': relation(license_models.Group)
        })

        mapper(core_models.IQReportDashboardDynamicTable, tables['tblIQReportDashboardDynamicTable'],
            polymorphic_identity='IQReportDashboardDynamicTable',
            inherits=DashboardWidget
        )

        mapper(core_models.ReportPublication, tables['tblReportPublication'],
            properties = {
                'report': relation(core_models.Report)
                },
            polymorphic_identity='ReportPublication',
            inherits=compose_models.ComposePublication
        )

        mapper(core_models.IQReportGraphPreset, tables['tblIQReportGraphPreset'],
            extension = PrivilegeObjectMapper(),
            properties = {
                'groups': relation(core_models.IQReportGraphPresetGroup, backref='graph_preset', cascade="all", passive_deletes=True)
        })

        mapper(core_models.IQReportGraphPresetGroup, tables['tblIQReportGraphPresetGroup'], properties = {
            'group': relation(license_models.Group)
        })

        __mapping_defined = True


def fill_data(conn):
    pass

def app_permissions_category():
    """
    Returns a category for all permissions defined for this app
    """
    return license_models.PermissionCategory(u'IQReport')

def app_permissions():
    """
    The permissions used within this application. Can be sync'd with the
    `iqsyncpermissions` command.
    """
    return [

        # report
        license_models.Permission(u'create_report', u"""
            This gives the right to create a report.
            """, _("Create reports")),
        license_models.Permission(u'view_report', u"""
            This gives the right to view a report.
            """, _("View reports")),
        license_models.Permission(u'update_report', u"""
            This gives the right to update a report.
            """, _("Update reports")),
        license_models.Permission(u'delete_report', u"""
            This gives the right to delete a report.
            """, _("Delete reports")),
        license_models.Permission(u'schedule_report', u"""
            This gives the right to schedule a report.
            """, _("Schedule reports")),

        # report publication
        license_models.Permission(u'create_report_publication', u"""
            This gives the right to create a report publication.
            """, _("Create report publications")),
        license_models.Permission(u'view_report_publication', u"""
            This gives the right to view a report publication.
            """, _("View report publications")),
        license_models.Permission(u'update_report_publication', u"""
            This gives the right to update a report publication.
            """, _("Update report publications")),
        license_models.Permission(u'delete_report_publication', u"""
            This gives the right to delete a report publication.
            """, _("Delete report publications")),
        license_models.Permission(u'schedule_report_publication', u"""
            This gives the right to schedule a report publication.
            """, _("Schedule report publications")),
        license_models.Permission(u'use_iqreport_report_publication', u"""
            This gives the right to use an IQReport report publication.
            """, _("Use iqreport report publications")),

        license_models.Permission(u'use_dynamic_table_in_report', u"""
            This gives the right to use the Dynamic Table Report Item.
            """, _('Use Dynamic Table in Report')),

        license_models.Permission(u'use_dynamic_table_on_dashboard', u"""
            This gives the right to use the Dynamic Table Dashboard Item.
            """, _('Use Dynamic Table on Dashboard')),

        license_models.Permission(u'create_mdx', u"""
            This gives the right to create MDX items.
            """, _('Create MDX items')),
        license_models.Permission(u'view_mdx', u"""
            This gives the right to view MDX items.
            """, _('View MDX items')),
        license_models.Permission(u'update_mdx', u"""
            This gives the right to update MDX items.
            """, _('Update MDX items')),
        license_models.Permission(u'delete_mdx', u"""
            This gives the right to delete MDX items.
            """, _('Delete MDX items')),
        license_models.Permission(u'use_mdx_on_dashboard', u"""
            This gives the right to use MDX items as dashboard widgets.
            """, _('Use MDX items as Dashboard widgets')),

        license_models.Permission(u'item_result', u"""
            This gives the right to use the Results widget on dashboard.
            """, _('Use Results widget on dashboard')),

        license_models.Permission(u'use_raw_mdx', u"""
            This gives the right to see and create raw MDX in the pivot table.
            """, _('Use raw MDX')),

        license_models.Permission(u'create_mdx_result', u"""
            This gives the right to create MDX result items.
            """, _('Create MDX Result items')),
        license_models.Permission(u'view_mdx_result', u"""
            This gives the right to view MDX result items.
            """, _('View MDX Result items')),
        license_models.Permission(u'update_mdx_result', u"""
            This gives the right to update MDX result items.
            """, _('Update MDX Result items')),
        license_models.Permission(u'delete_mdx_result', u"""
            This gives the right to delete MDX result items.
            """, _('Delete MDX Result items')),
        license_models.Permission(u'use_mdx_result_on_dashboard', u"""
            This gives the right to use MDX result items as dashboard widgets.
            """, _('Use MDX Result items as Dashboard widgets')),

        license_models.Permission(u'view_report_theme', u"""
            The permission to view report themes
            """, _("View report themes")),

        # for report mail stuff see mailing application
        license_models.Permission(u"create_iqreport_mail", u"""
            This gives the right to create new iqreport mails.
            """, _("Create IQReport mails")),
        license_models.Permission(u"view_iqreport_mail", u"""
            This gives the right to view iqreport mails.
            """, _("View IQReport mails")),
        license_models.Permission(u"update_iqreport_mail", u"""
            This gives the right to update existing iqreport mails.
            """, _("Update IQReport mails")),
        license_models.Permission(u"delete_iqreport_mail", u"""
            This gives the right to delete iqreport mails.
            """, _("Delete IQReport mails")),
        # this permission is used when attaching a mail to a mailing (so to be sent)
        license_models.Permission(u"schedule_iqreport_mail", u"""
            The right to use the iqreport mail in a publication mailing.
            """, _("Schedule IQReport mails")),

        license_models.Permission(u'use_advanced_graphs', u"""
            This gives the right to use the advanced graph types.
            """, _("Use advanced graphs")),

        # this permission is currently used on the dashboard, but should also be available in IQReport as a mode.
        license_models.Permission(u'link_result_widgets', u"""
            This gives the right to link result widgets together to push dimensions/measures to other widgets.
            """, _("Link result widgets")),

        license_models.Permission(u'create_graph_preset', u"""
            This gives the right to create a graph preset.
            """, _("Create graph preset")),
        license_models.Permission(u'view_graph_preset', u"""
            This gives the right to view graph presets.
            """, _("View graph preset")),
        license_models.Permission(u'update_graph_preset', u"""
            This gives the right to update a graph preset.
            """, _("Update graph preset")),
        license_models.Permission(u'delete_graph_preset', u"""
            This gives the right to mark a graph preset as deleted.
            """, _("Delete graph preset")),
        license_models.Permission(u'archive_graph_preset', u"""
            This gives the right to mark a graph preset as archived.
            """, _("Archive graph preset")),
    ]

def app_roles():
    return [{
        'name': 'IQ Report file import (basic)',
        'required_permission': 'manage_iqportal_roles',
        'permissions': [
            'view_dashboard', 'save_dashboard',
            'view_notification', 'delete_notification',
            'file_manager', 'view_file', 'create_file', 'update_file',
            'view_datasource', 'create_datasource', 'update_datasource', 'archive_datasource',
            'datasource_manager', 'file_based_datasource_access',
            'datasource_view_cube', 'datasource_create_cube', 'datasource_delete_cube', 'datasource_delete_table',
            'view_datasource_widget',
            'view_report', 'create_report', 'update_report',
        ],
        'license_configs': [],
    }, {
        'name': 'IQ Report file import (advanced)',
        'required_permission': 'manage_iqportal_roles',
        'permissions': [
            'view_dashboard', 'save_dashboard',
            'view_notification', 'delete_notification',
            'file_manager', 'view_file', 'create_file', 'update_file',
            'view_datasource', 'create_datasource', 'update_datasource', 'archive_datasource',
            'datasource_manager', 'file_based_datasource_access',
            'datasource_view_cube', 'datasource_create_cube', 'datasource_delete_cube', 'datasource_delete_table',
            'view_datasource_widget', 'pivot_table',
            'view_report', 'create_report', 'update_report',
            'view_report_theme', 'use_advanced_graphs',
            'view_graph_preset', 'create_graph_preset', 'update_graph_preset', 'delete_graph_preset',
            'view_report_publication', 'create_report_publication', 'update_report_publication', 'schedule_report_publication', 'use_iqreport_report_publication',
            'view_mail', 'create_mail', 'update_mail',
            'view_iqreport_mail', 'create_iqreport_mail', 'update_iqreport_mail', 'schedule_iqreport_mail',
        ],
        'license_configs': [],
    }]

def app_configs():
    """
    The account configs available for this application. This can be sync'd with the
    `iqsyncpermissions` command.
    """
    from basesoft.web.contrib.account import models as account_models
    configs = []

    config = account_models.Config()
    config.name = 'iqreport.compose.palette.hidden'
    config.type = 'boolean'
    config.default = 'False'
    config.description = u'Boolean for whether or not the IQReport compose Palette is hidden.'
    configs.append(config)

    config = account_models.Config()
    config.name = 'iqreport.compose.action.hidden'
    config.type = 'boolean'
    config.default = 'True'
    config.description = u'Boolean for whether or not the IQReport compose Action palette is hidden.'
    configs.append(config)

    config = account_models.Config()
    config.name = 'iqreport.compose.history.hidden'
    config.type = 'boolean'
    config.default = 'False'
    config.description = u'Boolean for whether or not the IQreport compose History palette is hidden.'
    configs.append(config)

    config = account_models.Config()
    config.name = 'iqreport.compose.right_layout_unit.width'
    config.type = 'integer'
    config.default = '250'
    config.description = u'Integer used to determine the default width (in px) of the IQReport compose right panel.'
    configs.append(config)

    config = account_models.Config()
    config.name = 'iqreport.compose.right_layout_unit.collapsed'
    config.type = 'boolean'
    config.default = 'False'
    config.description = u'Boolean used to determine if the IQReport compose right panel should be collapsed by default.'
    configs.append(config)

    config = account_models.Config()
    config.name = 'iqreport.compose.top_layout_unit.height'
    config.type = 'integer'
    config.default = '63'
    config.description = u'Integer used to determine the default height (in px) of the IQReport top compose panel.'
    configs.append(config)

    config = account_models.Config()
    config.name = 'iqreport.compose.top_layout_unit.collapsed'
    config.type = 'boolean'
    config.default = 'False'
    config.description = u'Boolean used to determine if the IQReport top compose panel should be collapsed by default.'
    configs.append(config)

    config = account_models.Config()
    config.name = 'iqreport.compose.remove.confirmation'
    config.type = 'boolean'
    config.default = 'True'
    config.description = u'Boolean used to determine if confirmation should be requested when a module is removed from the IQReport canvas.'
    configs.append(config)

    config = account_models.Config()
    config.name = 'iqreport.core.graph.animate'
    config.type = 'boolean'
    config.default = 'False'
    config.description = u'Boolean used to determine to show animations.'
    configs.append(config)

    config = account_models.Config()
    config.name = 'iqreport.core.graph.prefer_flash'
    config.type = 'boolean'
    config.default = 'False'
    config.description = u'Boolean used to determine to use Adobe Flash if available.'
    configs.append(config)

    config = account_models.Config()
    config.name = 'iqreport.core.item.show_pivot'
    config.type = 'boolean'
    config.default = 'False'
    config.description = u'Show pivot in report items'
    configs.append(config)

    return configs

