# Copyright (c) 2012 BaseSoft B.V.
# See LICENSE for details.

from django.utils.translation import ugettext as _
from sqlalchemy.orm.exc import NoResultFound

from basesoft import logging
from basesoft.web.core.util.controllers import AbstractController, StateManager
from basesoft.web.contrib.compose.models import ComposeInvitee,\
    ComposePublication
from basesoft.web.core.exceptions import PublicUserException

from iqportal.iqreport.display.models import ReportViewer
from iqportal.iqreport.compose.controllers import ReportStateController
from iqportal.iqreport.core.models import Report
from iqportal.iqube.xmla.client import XMLACacheController

logger = logging.getLogger("iqportal.iqreport.display")

class IQReportXMLACacheController(XMLACacheController):

    def set_context(self, publication, contact):
        self.publication = publication
        self.contact = contact
        if self.publication is not None:
            self.role_name = self.publication.cache_role_name

class ReportDisplayController(AbstractController):
    _report_viewer = None

    def __init__(self, sa_session, locale, invitee_hash=None, report_viewer_id=None):

        logger.info('ReportDisplayController: invitee_hash: %s, report_viewer_id: %s' % (invitee_hash, report_viewer_id) )

        if invitee_hash: assert isinstance(invitee_hash, (unicode, str))
        if invitee_hash: assert len(invitee_hash) == 8

        AbstractController.__init__(self, sa_session, locale)

        if report_viewer_id is not None:
            self._report_viewer = self.sa_session.query(ReportViewer).filter(ReportViewer.id==report_viewer_id).one()

            # different inviteee hash than the report viewer is part of, reset.
            if invitee_hash is not None and self._report_viewer.invitee.invitee_hash != invitee_hash:
                logger.info('Invitee hash difference for report viewer (%s, %s)' % (invitee_hash, self._report_viewer))
                self._report_viewer = None

        if self._report_viewer is None:
            try:
                invitee = self.sa_session.query(ComposeInvitee).filter(
                    ComposeInvitee.invitee_hash == invitee_hash).one()
            except NoResultFound:
                raise PublicUserException('Invalid hash supplied: %s' % invitee_hash)

            self._report_viewer = self._init_report_viewer(invitee)

    def _init_report_viewer(self, invitee):
        if invitee.publication.status != ComposePublication.STATUS_ONLINE:
            raise PublicUserException(_('This report is currently not available'))

        # we are dealing with an Anonymous Report, always make a new report_viewer
        if invitee.contact is None:
            report_viewer = None
        else:
            try:
                report_viewer = self.sa_session.query(ReportViewer).filter(ReportViewer.invitee_id==invitee.id).one()
            except NoResultFound: # they were not here before
                report_viewer = None

        if report_viewer is None:
            report_viewer = ReportViewer()
            report_viewer.number_of_views = 0
            report_viewer.invitee = invitee

        report_viewer.number_of_views = report_viewer.number_of_views + 1

        # flush the the report_viewer gets an ID which is needed in session
        self.sa_session.add(report_viewer)
        self.sa_session.flush()

        return report_viewer

    def get_report_viewer(self):
        return self._report_viewer

    def get_publication(self):
        return self._report_viewer.invitee.publication

    def load_state(self):
        manager = StateManager(self.locale, self.account_id, Report.MODE_VIEW)
        controller = ReportStateController(self.sa_session, manager)
        state = controller.load_by_invitee(self._report_viewer.invitee)

        state['report']['show_page_nrs'] = self._report_viewer.invitee.publication.show_page_nrs
        state['report']['viewer_print'] = self._report_viewer.invitee.publication.viewer_print & ComposePublication.VIEWER_PRINT_PRINTER == ComposePublication.VIEWER_PRINT_PRINTER
        state['report']['viewer_pdf'] = self._report_viewer.invitee.publication.viewer_print & ComposePublication.VIEWER_PRINT_PDF == ComposePublication.VIEWER_PRINT_PDF
        state['report']['exit_url'] = self._report_viewer.invitee.publication.exit_url

        return state



