# Copyright (c) 2014 BaseSoft B.V.
# See LICENSE for details.

import datetime
from sqlalchemy.orm import mapper, relation
from sqlalchemy import UnicodeText, Column, Integer, DateTime, ForeignKey, SmallInteger, Unicode

from basesoft.sqlalchemy.table import IQTable
from basesoft.web.contrib.license import models as license_models
from basesoft.web.contrib.license.hooks import PrivilegeObjectMapper

__mapping_defined = False
__tables_defined = False
tables = {}

def reset():
    global __mapping_defined
    global __tables_defined

    __mapping_defined = False
    __tables_defined = False

def get_tables():
    return tables

def do_tables(metadata):
    """
    This function is called by Django middleware when needed.
    """
    global __tables_defined
    if __tables_defined:
        return

    tblMdxFormatString = IQTable('tblMdxFormatString', metadata,
        Column('id', Integer, primary_key=True, autoincrement=True),
        Column('name', Unicode(255), nullable=False),
        Column('format_string', UnicodeText, nullable=True),
        Column('deleted', SmallInteger, nullable=False, default=0),
        Column('archived', SmallInteger, nullable=False, default=0),
        Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
        Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now),
    )

    tblMdxFormatStringGroup = IQTable('tblMdxFormatStringGroup', metadata,
        Column('format_string_id', Integer, ForeignKey('tblMdxFormatString.id', ondelete="CASCADE"), primary_key=True),
        Column('group_id', Integer, ForeignKey('tblGroup.id'), primary_key=True),
        Column('privilege', Integer, nullable=False, primary_key=True, autoincrement=False),
        Column('group_has_privilege', SmallInteger, default=0),
        Column('license_has_privilege', SmallInteger, default=0),
        Column('domain_has_privilege', SmallInteger, default=0),
        Column('world_has_privilege', SmallInteger, default=0),
        Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
        Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
    )

    tables['tblMdxFormatString'] = tblMdxFormatString
    tables['tblMdxFormatStringGroup'] = tblMdxFormatStringGroup

    __tables_defined = True

def do_mappings():
    """
    This function is called by Django middleware when needed.
    """
    global __mapping_defined
    if __mapping_defined:
        return

    from iqportal.iqube.compose import models as compose_models

    mapper(compose_models.MdxFormatString, tables['tblMdxFormatString'],
        extension = PrivilegeObjectMapper(),
        properties = {
            'groups': relation(compose_models.MdxFormatStringGroup, backref='format_string', cascade="all", passive_deletes=True)
    })

    mapper(compose_models.MdxFormatStringGroup, tables['tblMdxFormatStringGroup'],
        properties = {
            'group': relation(license_models.Group)
    })

    __mapping_defined = True

def fill_data(conn):
    pass

def app_permissions_category():
    """
    Returns a category for all permissions defined for this app
    """
    return license_models.PermissionCategory(u'MDX calculated members')

def app_permissions():
    from django.utils.translation import ugettext as _
    return [
        license_models.Permission(u'manage_format_strings', u"""
            This gives the right to manage format strings (creation/update/delete).
            """, _("Manage format strings")),
    ]

def app_configs():
    from basesoft.web.contrib.account import models as account_models
    configs = []

    config = account_models.Config()
    config.name = 'iqube.compose.calculated.member.id'
    config.type = 'integer'
    config.default = 0
    config.description = u'Calculated Member ID which will be loaded when getting to the compose app.'
    configs.append(config)

    config = account_models.Config()
    config.name = 'iqube.compose.palette.hidden'
    config.type = 'boolean'
    config.default = 'False'
    config.description = u'Boolean for whether or not the IQube Palette is hidden.'
    configs.append(config)

    config = account_models.Config()
    config.name = 'iqube.compose.legend.hidden'
    config.type = 'boolean'
    config.default = 'True'
    config.description = u'Boolean for whether or not the IQube Legend panel is hidden.'
    configs.append(config)

    config = account_models.Config()
    config.name = 'iqube.compose.action.hidden'
    config.type = 'boolean'
    config.default = 'True'
    config.description = u'Boolean for whether or not the IQube Action panel is hidden.'
    configs.append(config)

    config = account_models.Config()
    config.name = 'iqube.compose.right_layout_unit.width'
    config.type = 'integer'
    config.default = '250'
    config.description = u'Integer used to determine the default width (in px) of the IQube right compose panel.'
    configs.append(config)

    config = account_models.Config()
    config.name = 'iqube.compose.right_layout_unit.collapsed'
    config.type = 'boolean'
    config.default = 'False'
    config.description = u'Boolean used to determine if the IQube right compose panel should be collapsed by default.'
    configs.append(config)

    config = account_models.Config()
    config.name = 'iqube.compose.top_layout_unit.height'
    config.type = 'integer'
    config.default = '88'
    config.description = u'Integer used to determine the default height (in px) of the IQube top compose panel.'
    configs.append(config)

    config = account_models.Config()
    config.name = 'iqube.compose.top_layout_unit.collapsed'
    config.type = 'boolean'
    config.default = 'False'
    config.description = u'Boolean used to determine if the IQube top compose panel should be collapsed by default.'
    configs.append(config)

    config = account_models.Config()
    config.name = 'iqube.compose.remove.confirmation'
    config.type = 'boolean'
    config.default = 'True'
    config.description = u'Boolean used to determine if confirmation should be requested when a module is removed from the IQube compose.'
    configs.append(config)

    return configs
