# Copyright (c) 2011 BaseSoft B.V.
# See LICENSE for details.

configurable_widget_class_definition = u"""
from iqportal.iqube.core.db_sync import AbstractConfigurableWidget

class %(class_name)sWidget(AbstractConfigurableWidget):
    category = '%(category)s'
    cube = '%(cube)s'
    datasource = '%(datasource)s'

    def config(self):
        return %(config)s

    def applied_filters(self):
        return %(applied_filters)s

configurable_widgets = (%(class_name)sWidget, )
"""

class AbstractMdxDefinition:
    """
    Descriptions should come from translations
    """
    name = None
    label = None
    return_type = None
    fields = None
    category = None
    description = None
    example_description = None
    example_mdx = None
    available = True

    def get_db_tuple(self):
        return (self.label, self.return_type, self.fields, self.category, )

class MdxMethodFilter(AbstractMdxDefinition):
    name = 'Filter'
    label = 'Filter'
    return_type = 'set'
    fields = (('set', True), ('logical', True), )
    category = 'Set functions'
    description = """
        Returns the set that results from filtering a specified set based on a search condition.
    """
    example_description = """
        The following example returns only the `Promotion Media` where `Unit Sales` is greater than 5000.
    """
    example_mdx = """
        WITH SET [High Sales Only] AS 'Filter([Promotion Media].Children, [Measures].[Unit Sales]>5000)'
        SELECT {[Measures].[Unit Sales]} ON COLUMNS,
            {[High Sales Only]} ON ROWS
        FROM [Sales]
    """

class MdxMethodSum(AbstractMdxDefinition):
    name = 'Sum'
    label = 'Sum'
    return_type = 'numeric'
    fields = (('set', True), ('numeric', False), )
    category = 'Numeric functions'
    description = """
        Returns the sum of a numeric expression evaluated over a specified set.
    """

class MdxMethodIif(AbstractMdxDefinition):
    name = 'Iif'
    label = 'Iif'
    return_type = 'predefined'
    fields = (('logical', True), ('predefined', True), ('predefined', True))
    category = 'Generic functions'
    description = """
        Evaluates different branch expressions depending on whether a Boolean condition is true or false.
    """
    example_description = """
        The following example shows ''High sales'' for each `Promotion Media` with over 5000 `Unit Sales`.
    """
    example_mdx = """
        WITH MEMBER [Measures].[Human Readable] AS 'IIF([Measures].[Unit Sales]>5000, "High sales", "Low sales")'
        SELECT {[Measures].[Unit Sales], [Measures].[Human Readable]} ON COLUMNS,
            {[Promotion Media].Children} ON ROWS
        FROM [Sales]
    """

class MdxMethodAxis(AbstractMdxDefinition):
    # TODO: Please note that the Axis method is not implemented in Mondrian but it is implemented in IQPortal.
    name = 'Axis'
    label = 'Axis'
    return_type = 'set'
    fields = None
    category = 'Set functions'
    description ="""
        Returns the set of tuples on a specified axis.
    """

class MdxMethodTopCount(AbstractMdxDefinition):
    name = 'TopCount'
    label = 'Top count'
    return_type = 'set'
    fields = (('set', True), ('count', True), ('numeric', False))
    category = 'Set functions'
    description = """
        Sorts a set in descending order and returns the specified number of elements in the specified set with the highest values.
    """
    example_mdx = """
        WITH SET [Calc] AS 'TopCount({[Store Type].[Store Type].Members}, 5, [Measures].[Unit Sales])'
        SELECT {[Measures].[Unit Sales]} ON COLUMNS,
            NON EMPTY [Calc] ON ROWS
        FROM [Sales]
    """

class MdxMethodBottomCount(AbstractMdxDefinition):
    name = 'BottomCount'
    label = 'Bottom count'
    return_type = 'set'
    fields = (('set', True), ('count', True), ('numeric', False))
    category = 'Set functions'
    description = """
        Sorts a set in ascending order, and returns the specified number of elements in the specified set with the lowest values.
    """
    example_description = """
        The following example returns the bottom 5 `Promotion Media Types` by `Unit Sales`.
    """
    example_mdx =  """
        WITH SET [Calc] AS 'BottomCount({[Promotion Media].[Media Type].Members}, 5, [Measures].[Unit Sales])'
        SELECT {[Measures].[Unit Sales]} ON COLUMNS,
            NON EMPTY [Calc] ON ROWS
        FROM [Sales]
    """

class MdxMethodAverage(AbstractMdxDefinition):
    name = 'Avg'
    label = 'Average'
    return_type = 'numeric'
    fields = (('set', True), ('numeric', False))
    category = 'Numeric functions'
    description = """
        Evaluates a set and returns the average of the non empty values of the cells in the set, averaged over the measures in the set or over a specified measure.
    """

class MdxMethodGenerate(AbstractMdxDefinition):
    name = 'Generate'
    label = 'Generate'
    return_type = 'set'
    fields = (('set', True), ('set', True), ('string', False))
    category = 'Set functions'
    description = """
        Applies a set to each member of another set, and then joins the resulting sets by union. Alternatively, this function returns a concatenated string created by evaluating a string expression over a set.

        The most common practical use of Generate is to evaluate a complex set expression, such as TopCount, over a set of members.
    """

class MdxMethodCrossjoin(AbstractMdxDefinition):
    name = 'Crossjoin'
    label = 'Crossjoin'
    return_type = 'set'
    fields = (('set', True), ('set', True))
    category = 'Set functions'
    description = """
        The Crossjoin function returns the cross product of two or more specified sets. The order of tuples in the resulting set depends on the order of the sets to be joined and the order of their members.
    """
    example_description = """
        The following example shows a simple use of the Crossjoin function on the Rows axis of a query with the product of `Store Type` and `Gender`.
    """
    example_mdx = """
        WITH SET [Calc] AS 'Crossjoin({[Store Type].[Store Type].Members}, {[Gender].Children})'
        SELECT {[Measures].[Unit Sales]} ON COLUMNS,
            NON EMPTY [Calc] ON ROWS
        FROM [Sales]
    """

class MdxMethodExcept(AbstractMdxDefinition):
    name = 'Except'
    label = 'Except'
    return_type = 'set'
    fields = (('set', True), ('set', True), ('predefined', False))
    category = 'Set functions'
    description = """
        Evaluates two sets and removes those tuples in the first set that also exist in the second set, optionally retaining duplicates.
    """
    example_description = """
        The following example shows all `Promotion Media` with the exception of the `No Media` member.
    """
    example_mdx = """
        WITH SET [Actual Media] AS 'Except([Promotion Media].Children, {[Promotion Media].[No Media]})'
        SELECT {[Measures].[Unit Sales]} ON COLUMNS,
            {[Actual Media]} ON ROWS
        FROM [Sales]
    """

class MdxMethodAggregate(AbstractMdxDefinition):
    name = 'Aggregate'
    label = 'Aggregate'
    return_type = 'numeric'
    fields = (('set', True), )
    category = 'Numeric functions'
    description = """
        Returns a number that is calculated by aggregating over the cells returned by the set expression. If a numeric expression is not provided, this function aggregates each measure within the current query context by using the default aggregation operator that is specified for each measure. If a numeric expression is provided, this function first evaluates, and then sums, the numeric expression for each cell in the specified set.
    """
    example_description = """
        The following example returns the sum of the `Unit Sales` member, aggregated over the `Media Types` from the `Promotion Media` dimension per `Store Type`.
    """
    example_mdx = """
        WITH MEMBER [Promotion Media].[Total] AS 'AGGREGATE({[Promotion Media].[Media Type].Members})'
        SELECT {[Promotion Media].[Total]} ON COLUMNS,
            {[Store Type].[Store Type].Members} ON ROWS
        FROM [Sales]
        WHERE [Measures].[Unit Sales]
    """

class MdxMethodYTD(AbstractMdxDefinition):
    name = 'YTD'
    label = 'Year to date'
    return_type = 'set'
    fields = (('time_member', False), )
    category = 'Time functions'
    description = """
       Returns a set of sibling members from the same level as a given member, starting with the first sibling and ending with the given member, as constrained by the Year level in the Time dimension.
    """

class MdxMethodQTD(AbstractMdxDefinition):
    name = 'QTD'
    label = 'Quarter to date'
    return_type = 'set'
    fields = (('time_member', False), )
    category = 'Time functions'
    description = """
       Returns a set of sibling members from the same level as a given member, starting with the first sibling and ending with the given member, as constrained by the Quarter level in the Time dimension.
    """

class MdxMethodMTD(AbstractMdxDefinition):
    name = 'MTD'
    label = 'Month to date'
    return_type = 'set'
    fields = (('time_member', False), )
    category = 'Time functions'
    description = """
       Returns a set of sibling members from the same level as a given member, starting with the first sibling and ending with the given member, as constrained by the Month level in the Time dimension.
    """

class MdxMethodWTD(AbstractMdxDefinition):
    name = 'WTD'
    label = 'Week to date'
    return_type = 'set'
    fields = (('time_member', False), )
    category = 'Time functions'
    description = """
       Returns a set of sibling members from the same level as a given member, starting with the first sibling and ending with the given member, as constrained by the Week level in the Time dimension.
    """

class MdxMethodClosingPeriod(AbstractMdxDefinition):
    name = 'ClosingPeriod'
    label = 'Closing period'
    return_type = 'member'
    fields = (('time_level', False), ('time_member', False), )
    category = 'Time functions'
    description = """
        Returns the member that is the last sibling among the descendants of a specified member at a specified level.
    """

class MdxMethodLastPeriods(AbstractMdxDefinition):
    name = 'LastPeriods'
    label = 'Last periods'
    return_type = 'set'
    fields = (('count', True), ('time_member', False), )
    category = 'Time functions'
    description = """
        Returns a set of members up to and including a specified member.
    """

class MdxMethodOpeningPeriod(AbstractMdxDefinition):
    name = 'OpeningPeriod'
    label = 'Opening period'
    return_type = 'member'
    fields = (('time_level', False), ('time_member', False), )
    category = 'Time functions'
    description = """
        Returns the first sibling among the descendants of a specified level, optionally at a specified member.
    """

class MdxMethodParallelPeriod(AbstractMdxDefinition):
    name = 'ParallelPeriod'
    label = 'Parallel period'
    return_type = 'member'
    fields = (('time_level', False),  ('count', False), ('time_member', False), )
    category = 'Time functions'
    description = """
        Returns a member from a prior period in the same relative position as a specified member.
    """

MDX_METHODS = {
    MdxMethodAxis.name: MdxMethodAxis,
    MdxMethodFilter.name: MdxMethodFilter,
    MdxMethodSum.name: MdxMethodSum,
    MdxMethodIif.name: MdxMethodIif,
    MdxMethodTopCount.name: MdxMethodTopCount,
    MdxMethodBottomCount.name: MdxMethodBottomCount,
    MdxMethodAverage.name: MdxMethodAverage,
    MdxMethodGenerate.name: MdxMethodGenerate,
    MdxMethodCrossjoin.name: MdxMethodCrossjoin,
    MdxMethodExcept.name: MdxMethodExcept,
    MdxMethodAggregate.name: MdxMethodAggregate,
    MdxMethodYTD.name: MdxMethodYTD,
    MdxMethodQTD.name: MdxMethodQTD,
    MdxMethodMTD.name: MdxMethodMTD,
    MdxMethodWTD.name: MdxMethodWTD,
    MdxMethodClosingPeriod.name: MdxMethodClosingPeriod,
    MdxMethodLastPeriods.name: MdxMethodLastPeriods,
    MdxMethodOpeningPeriod.name: MdxMethodOpeningPeriod,
    MdxMethodParallelPeriod.name: MdxMethodParallelPeriod,
}

class AbstractMdxSetProperty(AbstractMdxDefinition):
    category = 'set'

class MdxSetPropertyCurrent(AbstractMdxSetProperty):
    name = 'Current'
    label = 'Current'
    return_type = 'member' # FIXME: this actual returns a tuple, not a member
    description = 'Returns the current tuple from a set during iteration.'

class MdxSetPropertyCurrentOrdinal(AbstractMdxSetProperty):
    name = 'CurrentOrdinal'
    label = 'Current ordinal'
    return_type = 'numeric'
    description = 'Returns the current iteration number within a set during iteration.'
    example_mdx = """
        WITH SET [MySet] AS '[Promotion Media].[Media Type].Members'
            MEMBER [Measures].[Current Ordinal Demo] AS '[MySet].CurrentOrdinal'
        SELECT {[Measures].[Unit Sales], [Measures].[Current Ordinal Demo]} ON COLUMNS,
            [MySet] ON ROWS
        FROM [Sales]
    """

class MdxSetPropertyCount(AbstractMdxSetProperty):
    name = 'Count'
    label = 'Count'
    return_type = 'numeric'
    description = 'Returns the number of cells in a set.'

class MdxSetPropertyItem(AbstractMdxSetProperty):
    name = 'Item'
    label = 'Item'
    return_type = 'numeric' # FIXME: this actual returns a tuple, not a member
    fields = (( 'numeric', True), )
    description = 'Returns a tuple from a set.'

class AbstractMdxMemberProperty(AbstractMdxDefinition):
    category = 'member'

class MdxMemberPropertyDimension(AbstractMdxMemberProperty):
    # TODO: IMPLEMENT HIERARCHY RERTURN TYPE BEFORE ENABLING THIS
    available = False
    name = 'Dimension'
    label = 'Dimension'
    return_type = 'hierarchy'
    description = 'Returns the hierarchy that contains a specified member.'

class MdxMemberPropertyFirstChild(AbstractMdxMemberProperty):
    name = 'FirstChild'
    label = 'First child'
    return_type = 'member'
    description = 'Returns the first child of a specified member.'

class MdxMemberPropertyFirstSibling(AbstractMdxMemberProperty):
    name = 'FirstSibling'
    label = 'First sibling'
    return_type = 'member'
    description = 'Returns the first child of the parent of a member.'

class MdxMemberPropertyHierarchy(AbstractMdxMemberProperty):
    # TODO: IMPLEMENT HIERARCHY RERTURN TYPE BEFORE ENABLING THIS
    available = False
    name = 'Hierarchy'
    label = 'Hierarchy'
    return_type = 'hierarchy',
    description = 'Returns the hierarchy that contains a specified member.'

class MdxMemberPropertyLastChild(AbstractMdxMemberProperty):
    name = 'LastChild'
    label = 'Last child'
    return_type = 'member',
    description = 'Returns the last child of a specified member.'

class MdxMemberPropertyLastSibling(AbstractMdxMemberProperty):
    name = 'LastSibling'
    label = 'Last sibling'
    return_type = 'member'
    description = 'Returns the last child of the parent of a specified member.'

class MdxMemberPropertyLag(AbstractMdxMemberProperty):
    name = 'Lag'
    label = 'Lag'
    return_type = 'member'
    fields = (('count', True), )
    description = 'Returns the member that is a specified number of positions before a specified member at the member\'s level.'

class MdxMemberPropertyLead(AbstractMdxMemberProperty):
    name = 'Lead'
    label = 'Lead'
    return_type = 'member'
    fields = (('count', True), )
    description = 'Returns the member that is a specified number of positions following a specified member along the member\'s level. '

class MdxMemberPropertyLevel(AbstractMdxMemberProperty):
    name = 'Level'
    label = 'Level'
    return_type = 'level',
    description = 'Returns the level of a member.'

class MdxMemberPropertyName(AbstractMdxMemberProperty):
    name = 'Name'
    label = 'Name'
    return_type = 'string'
    description = 'Returns the name of a member.'

class MdxMemberPropertyNextMember(AbstractMdxMemberProperty):
    name = 'NextMember'
    label = 'Next member'
    return_type = 'member'
    description = 'Returns the next member in the level that contains a specified member.'

class MdxMemberPropertyParent(AbstractMdxMemberProperty):
    name = 'Parent'
    label = 'Parent'
    return_type = 'member'
    description = 'Returns the parent of a member.'

class MdxMemberPropertyPrevMember(AbstractMdxMemberProperty):
    name = 'PrevMember'
    label = 'Previous member'
    return_type = 'member'
    description = 'Returns the previous member in the level that contains a specified member.'

class MdxMemberPropertyProperties(AbstractMdxMemberProperty):
    name = 'Properties'
    label = 'Properties'
    return_type = 'string'
    fields = (('string', True), )
    description = 'Returns a string, or a strongly-typed value, that contains a member property value.'

class MdxMemberPropertySiblings(AbstractMdxMemberProperty):
    name = 'Siblings'
    label = 'Siblings'
    return_type = 'set'
    description = 'Returns the siblings of a specified member, including the member itself.'

class MdxMemberPropertyUniqueName(AbstractMdxMemberProperty):
    name = 'UniqueName'
    label = 'Unique name'
    return_type = 'string'
    description = 'Returns the unique name of a specified member.'

class MdxMemberPropertyValue(AbstractMdxMemberProperty):
    name = 'Value'
    label = 'Value'
    return_type = 'string'
    description = 'Returns the value of the current member of the Measures dimension that intersects with the current member of the attribute hierarchies in the context of the query.'

class AbstractMdxLevelProperty(AbstractMdxDefinition):
    category = 'level'

class MdxLevelPropertyAllMembers(AbstractMdxLevelProperty):
    name = 'AllMembers'
    label = 'All members'
    return_type = 'set'
    description = 'Evaluates either a level expression and returns a set that contains all members of the specified level, which includes all calculated members in the level.'

class MdxLevelPropertyDimension(AbstractMdxLevelProperty):
    # TODO: IMPLEMENT HIERARCHY RERTURN TYPE BEFORE ENABLING THIS
    available = False
    name = 'Dimension'
    label = 'Dimension'
    return_type = 'hierarchy'
    description = 'Returns the hierarchy that contains a specified level.'

class MdxLevelPropertyHierarchy(AbstractMdxLevelProperty):
    # TODO: IMPLEMENT HIERARCHY RERTURN TYPE BEFORE ENABLING THIS
    available = False
    name = 'Hierarchy'
    label = 'Hierarchy'
    return_type = 'hierarchy'
    description = 'Returns the hierarchy that contains a specified level.'

class MdxLevelPropertyMembers(AbstractMdxLevelProperty):
    name = 'Members'
    label = 'Members'
    return_type = 'set'
    description = 'Returns the set of members in a level.'

class MdxLevelPropertyName(AbstractMdxLevelProperty):
    name = 'Name'
    label = 'Name'
    return_type = 'string'
    description = 'Returns the name of a level.'

class MdxLevelPropertyOrdinal(AbstractMdxLevelProperty):
    name = 'Ordinal'
    label = 'Ordinal'
    return_type = 'numeric'
    description = 'Returns the zero-based ordinal value associated with a level.'

class MdxLevelPropertyUniqueName(AbstractMdxLevelProperty):
    name = 'UniqueName'
    label = 'Unique name'
    return_type = 'string'
    description = 'Returns the unique name of a specified level.'

MDX_PROPERTIES = []
__properties_defined = False
def get_properties():
    global __properties_defined
    if __properties_defined:
        return MDX_PROPERTIES

    import sys, inspect
    clsmembers = inspect.getmembers(sys.modules[__name__], inspect.isclass)
    for name, cls in clsmembers:
        if name in ('AbstractMdxSetProperty', 'AbstractMdxMemberProperty', 'AbstractMdxLevelProperty'):
            continue
        if issubclass(cls, (AbstractMdxSetProperty, AbstractMdxMemberProperty, AbstractMdxLevelProperty)):
            if cls.available:
                MDX_PROPERTIES.append(cls)

    __properties_defined = True
    return MDX_PROPERTIES

__properties_map_defined = False
MDX_PROPERTIES_MAP = {}

def get_properties_map():
    global __properties_map_defined
    if __properties_map_defined:
        return MDX_PROPERTIES_MAP

    for p in get_properties():
        if not MDX_PROPERTIES_MAP.has_key(p.category):
            MDX_PROPERTIES_MAP[p.category] = {}
        MDX_PROPERTIES_MAP[p.category][p.name] = p

    __properties_map_defined = True
    return MDX_PROPERTIES_MAP

