# Copyright (c) 2011 BaseSoft B.V.
# See LICENSE for details.

"""
The DB Sync file for IQube manages predefined Configurable Widgets.

For example the widgets for TNT or XBRL

"""

from twisted.python.reflect import namedAny

from datetime import date, timedelta

from django.conf import settings
from django.utils import simplejson

from sqlalchemy import create_engine, and_

from basesoft.web.core.jsonrpc.dispatcher import JsonRpcJSONEncoder
from basesoft.web.contrib.license.models import Domain, Group, License
from basesoft.web.core.util.controllers import AbstractController

from iqportal.iqube.core import MDX_METHODS
from iqportal.iqube.core.models import DataSource, DataSourceWidget, MdxMethod, MdxMethodField

synced_widgets = []
synced_apps = []

from basesoft import logging
logger = logging.getLogger('iqportal.iqube.core')

def post_sync(session, app_name):

    # sync time tables
    if app_name == 'iqportal.iqube.core':

        db_engine = create_engine(settings.DB_CONNSTR, echo=False)
        _sync_time_tables(db_engine)

        mdx_method_sync(session)

    # sync config widgets..

    # we loop the apps, look for db_sync and see if there are any configurable_widgets defined.
    global synced_apps, synced_widgets
    synced_apps.append(app_name)
    for app in settings.INSTALLED_APPS:
        try:
            sync_module = namedAny(app+'.db_sync')
        except AttributeError, e:
            continue

        if not hasattr(sync_module, 'configurable_widgets'):
            continue

        for cw in getattr(sync_module, 'configurable_widgets'):

            #logger.info("starting sync widget %s" % cw.label)
            cw_key = app + '.'+cw.__name__

            if cw_key in synced_widgets:
                #print 'skipping widget because label already synced: %s' % cw.label
                continue

            if cw.requires_app_sync is not None and cw.requires_app_sync not in synced_apps:
                #print 'skipping widget requires app_sync: %s' % cw.label
                continue

            # trying to sync widget. it might fail for  the rest dbsync hasnt
            # completed yet. the loyalty widget does some xmla calls to get members
            # which will fail if the view doesnt exist yet
            widget = cw(session)
            try:
                logger.info('trying to sync widget "%s" for app %s' % (widget, app))
                widget.sync(app)
            except Exception, e:
                import traceback
                logger.error('ERROR: Failed to sync widget: %s' % e.message)
                logger.error(traceback.format_exc())
            else:
                synced_widgets.append(cw_key)


def _sync_time_tables(db_engine):


    begin_date = date(1970,1,1)
    end_date = date(2050,12,31)
    days = (end_date-begin_date).days

    class Buff(object):
        table = ''
        cols = None
        buffer = None

        do_fill = False

        def __init__(self):

            r = db_engine.execute("SELECT * FROM %s" % self.table)
            rows = r.fetchall()
            self.do_fill = len(rows) == 0

            if not self.do_fill:
                logger.info('_sync_time_tables: not filling for table %s' % self.table)

            self.buffer = []

        def add_day(self, mydate, id):
            pass

    class QuarterBuff(Buff):
        table = 'aggTimeQuarter'

        def __init__(self):
            self.cols = ['id', 'date', 'year', 'quarter', 'month', 'day']
            super(QuarterBuff, self).__init__()

        def add_day(self, mydate, id):

            self.buffer.append((
                int(id),
                 "'%s'" % unicode(mydate),
                 mydate.year,
                 (mydate.month-1)//3+1, # quarter
                 mydate.month,
                 mydate.day,
            ))

    class MySQLWeekBuff(Buff):

        # http://dev.mysql.com/doc/refman/5.5/en/date-and-time-functions.html#function_week
        week_mode = None

        def __init__(self):
            self.cols = ['id', 'date', 'year', 'week', 'day']
            super(MySQLWeekBuff, self).__init__()

        def add_day(self, mydate, id):

            self.buffer.append((
                int(id),
                 "'%s'" % unicode(mydate),
                 mydate.year,
                 "WEEK('%s', %s)" % (unicode(mydate), self.week_mode),
                 mydate.day,
            ))

    class WeekUSBuff(MySQLWeekBuff):
        table = 'aggTimeWeekUS'
        week_mode = 0

    class WeekISO8601Buff(MySQLWeekBuff):
        table = 'aggTimeWeekISO8601'
        week_mode = 1

    buffers = [QuarterBuff(), WeekUSBuff(), WeekISO8601Buff()]

    def flush():

        for buff in buffers:

            if len(buff.buffer) == 0 or not buff.do_fill:
                continue

            cols = (("`%s`," * len(buff.cols))[:-1]) % tuple(buff.cols)
            sql = "INSERT INTO %s (%s) VALUES " % (buff.table, cols)

            for row in buff.buffer:
                values = ("%s," * len(row))[:-1] % row
                sql += "(%s)," % values
            sql = sql[:-1]

            db_engine.execute(sql)
            logger.info('_sync_time_tables: flushed for %s' % buff.table)

            buff.buffer = []

    start_day = 0

    while start_day <= days:
        mydate = begin_date+timedelta(days=start_day)
        id = mydate.strftime('%Y%m%d')

        for buff in buffers:
            buff.add_day(mydate, id)

        if len(buff.buffer) >= 1000:
            flush()
        start_day = start_day + 1
    flush()


class AbstractConfigurableWidget:
    datasource = None
    cube = None
    category = 'Configurable Widgets'
    availability = DataSourceWidget.AVAILABLE_AS_DASHBOARD_WIDGET

    requires_app_sync = None

    _ds = None

    def __label(self):
        return self.config()['label']
    label = property(__label)

    def __init__(self, sa_session, datasource=None, cube=None):
        self.sa_session = sa_session
        if datasource is not None:
            self._ds = datasource
        if cube is not None:
            self.cube = cube
        self.client = None

    def get_group(self):
        group = self.sa_session.query(Domain).filter(Domain.name=='Resellers').one().default_group
        return group

    def modify_filters(self, applied_filters, client, cube):
        self.client = client
        self.cube = cube
        return self.modify_applied_filters(applied_filters)

    def get_modified_filter(self, filter):

        assert self.cube is not None

        x = filter
        if x[:4] == 'SET ' or x[:7] == 'MEMBER ':
            return x

        if self.client is None:
            ds = self.get_datasource()
            if ds is None:
                raise Exception('Unable to connect to datasource: %s' % self._ds)
            self.client = ds.connect()
            self.client.set_role_name()

        append = ''
        unique_name = "MEMBER_UNIQUE_NAME"
        use_unique_name = 'member_unique_name'
        member_name = None

        member = []

        if x[-9:] == '.Children':
            append = '.Children'
            x = x[:-9]
            if x.count('[') == 1:
                unique_name = 'HIERARCHY_UNIQUE_NAME'

            member_name = self.client.get_member_tree_op(self.cube, x, self.client.MDTREEOP_SELF, unique_name)[0]['member_name']

            member = self.client.get_level_info(self.cube, x)

        if x[-8:] == '.Members':
            append = '.Members'
            x = x[:-8]

            unique_name = "LEVEL_UNIQUE_NAME"
            use_unique_name = 'level_unique_name'

            member = self.client.get_level_info(self.cube, x)
            member_name = self.client.get_member_tree_op(self.cube, x, self.client.MDTREEOP_SELF, unique_name)[0]['member_name']
        else:
            member = self.client.get_member_tree_op(self.cube, x, self.client.MDTREEOP_SELF, unique_name)

        if len(member) == 0:
            logger.warn('WARNING: member "%s" not found in cube %s' % (x, self.cube))
            return None

        member = member[0]
        if member_name is not None:
            member['member_name'] = member_name

        if append != '':
            member['member_unique_name'] = '%s%s' % (member[use_unique_name], append)

        keep = ['member_unique_name', 'hierarchy_unique_name', 'depth', 'member_name', 'level_unique_name', 'dimension_unique_name']
        n = {}
        for i in keep:
            if member.has_key(i):
                n[i] = member[i]

        return n

    def modify_applied_filters(self, applied_filters):
        """
        This is used to get the full information of members used in the applied filters for widgets.
        """

        if self.client is None:
            ds = self.get_datasource()
            if ds is None:
                raise Exception('Unable to connect to datasource: %s' % self._ds)
            self.client = ds.connect()
            self.client.set_role_name()

        if len(applied_filters['column_order']) == 0:
            raise Exception("Column_order cant be empty")

        for index in applied_filters['dimensions'].values():
            d = applied_filters['dimension_%s' % index]
            if len(d) == 0:
                continue

            rv = []
            for x in d:

                if type(x) == dict:
                    rv.append(x)
                    continue

                member = self.get_modified_filter(x)
                if member is None:
                    continue
                rv.append(member)

            applied_filters['dimension_%s' % index] = [simplejson.dumps(x, ensure_ascii=False) for x in rv]

        return applied_filters

    def sync(self, app):
        """
        New Configurable Widgets are assigned to BaseSoft.
        """
        if not self.validate_requirements():
            logger.info('IQube::ConfigurableWidget(%s:%s): Validation failed' % (self.datasource, self.label))
            return

        logger.info('IQube::ConfigurableWidget(%s:%s): synchronizing...' % (self.datasource, self.label))
        # see if it already exists. LABEL + DATASOURCE is unique
        cw = self.sa_session.query(DataSourceWidget).filter(
            and_(
                DataSourceWidget.label==self.label,
                DataSourceWidget.datasource==self.get_datasource()
            )).first()

        if cw is None:
            cw = DataSourceWidget()
            cw.datasource = self.get_datasource()

        cw.cube = self.cube
        cw.label = self.label
        cw.category = self.category
        cw.application = app
        cw.availability = self.availability
        cw.config = simplejson.dumps(self.config(), ensure_ascii=False, cls=JsonRpcJSONEncoder)
        cw.applied_filters = simplejson.dumps(self.modify_applied_filters(self.applied_filters()), ensure_ascii=False, cls=JsonRpcJSONEncoder)

        if cw.id is None:
            object = cw.get_group_object()[0]
            for privilege in cw.all_privileges:
                go = object()
                go.group = self.get_group()
                go.privilege = privilege
                go.group_has_privilege = True
                go.license_has_privilege = True
                cw.groups.append(go)

        self.sa_session.add(cw)
        self.sa_session.flush()


    def get_datasource(self):
        if self._ds is not None:
            return self._ds

        self._ds = self.sa_session.query(DataSource).filter(DataSource.name=='%s.%s'% (settings.SETTINGS_MODULE, self.datasource)).first()
        return self._ds

    def validate_requirements(self):
        return True

def mdx_method_sync(sa_session):

    group = sa_session.query(Group).join(License).filter(and_(License.name=='BaseSoft Solutions B.V.', Group.is_default==True)).one()

    for m in MDX_METHODS.values():
        m = m()
        label, return_type, available_fields, category = m.get_db_tuple()

        method = sa_session.query(MdxMethod).filter(MdxMethod.name == m.name).first()

        if method is None:
            method = MdxMethod()
            method.name = m.name
            method.return_type = return_type
        method.label = label
        method.category = category

        i = 0
        if available_fields is not None:
            #loop the tuples representing the method fields
            for field in available_fields:
                field_type, required_field = field

                fields = sa_session.query(MdxMethodField).filter(and_(MdxMethodField.order == i, MdxMethodField.method == method)).first()

                if fields is None:
                    fields = MdxMethodField()
                    fields.method = method
                fields.order = i
                fields.field_type = field_type
                fields.required_field = required_field
                sa_session.add(fields)
                i += 1

        sa_session.query(MdxMethodField).filter(and_(MdxMethodField.order >= i, MdxMethodField.method == method)).delete()

        sa_session.add(method)
        c = AbstractController(sa_session=sa_session, account_id=None)
        c.grant_privilege_to(method, privileges=MdxMethod.all_privileges, group=True, target_group=group, check_own_privs=False)
        sa_session.flush()

