# Copyright (c) 2010 BaseSoft B.V.
# See LICENSE for details.

import re
from django.utils.translation import ugettext

from basesoft.web.core.models import PrivilegeObject
from basesoft.web.contrib.dashboard.models import DashboardWidget

from django.utils.translation import ugettext_lazy as _


class IncorrectCalculatedMember(Exception):
    pass

class CalculatedMember:
    """
    This is still used for the creation of inline CM from Configurable Widgets.
    @DynamicAttrs
    """

    def __init__(self, dimension=None, name=None, formula=None, member_string=None, type=None):
        if type is None:
            type = MdxCalculatedMember.CALCULATED_MEMBER

        self.type = type
        if member_string is not None:
            dimension, name, formula = self._from_string(member_string)

        self.dimension = dimension
        self.name = name
        self.formula = formula
        self.format_expression = None
        self.format_string = None

    def _set_from_string(self, member_string):
        regexp = "SET \[([^\]]+)\] AS '(.*)'"
        pattern = re.compile(regexp, re.MULTILINE|re.DOTALL)
        match = re.match(pattern, member_string)
        if match is None:
            raise IncorrectCalculatedMember()

        self.type = MdxCalculatedMember.NAMED_SET

        return (False, match.group(1), match.group(2), )

    def _from_string(self, member_string):
        if type(member_string) not in [str, unicode]:
            raise IncorrectCalculatedMember()
        regexp = "MEMBER ([\[\w\s\-\.\]]+)\.\[([^\]]+)\] AS '(.*)'"
        pattern = re.compile(regexp, re.MULTILINE|re.DOTALL)
        match = re.match(pattern, member_string)
        if match is None:
            return self._set_from_string(member_string)

        return (match.group(1), match.group(2), match.group(3), )

    def __repr__(self):
        if self.type == MdxCalculatedMember.CALCULATED_MEMBER:
            rv = "MEMBER %s.[%s] AS '%s'" % (self.dimension, self.name, self.formula)
            if self.format_expression is not None:
                return "%s, FORMAT_STRING='%s'" % (rv, self.format_expression)
            if self.format_string is not None:
                return "%s, FORMAT_STRING='%s'" % (rv, self.format_string)
            return rv
        else:
            return "SET [%s] AS '%s'" % (self.name, self.formula)

class DataSource(PrivilegeObject):
    """
    Model for the DataSource
    @DynamicAttrs
    """

    # Constants for the role_level field on this model. When role is 'none', the client
    # will not pass a role_name to the mondrian server. When its set to account the account id
    # will be passed, etc
    ROLE_LEVEL_NONE = 0
    ROLE_LEVEL_ACCOUNT = 1
    ROLE_LEVEL_DB = 2

    # privilege grants access to create cubes on this datasource
    PRIV_CREATE_CUBE = 32
    # and delete them
    PRIV_DELETE_CUBE = 64
    # result widget access
    PRIV_USE_IN_RESULT = 128

    all_privileges = (PrivilegeObject.PRIV_VIEW, PrivilegeObject.PRIV_UPDATE, PrivilegeObject.PRIV_ARCHIVE, PrivilegeObject.PRIV_DELETE, \
                      PrivilegeObject.PRIV_DELEGATE_PRIVILEGES, PRIV_CREATE_CUBE, PRIV_DELETE_CUBE, PRIV_USE_IN_RESULT)

    xmla_client = None

    mark_as_deleted_permission = 'delete_datasource'
    mark_as_archived_permission = 'archive_datasource'

    DATASOURCE_TYPE_DATABASE = 1
    DATASOURCE_TYPE_FILE = 2
    DATASOURCE_TYPE_OPTIQ = 3

    type_i18n = {
        DATASOURCE_TYPE_DATABASE: _('Database'),

        # FIXME: make a datasource_optiq_file
        DATASOURCE_TYPE_FILE: _('Database'),
        DATASOURCE_TYPE_OPTIQ: _('File (Optiq)'),
    }

    def get_group_object(self):
        return (DataSourceGroup, 'datasource')

    def get_privileges(self):
        return PrivilegeObject.get_privileges(self) + [
            (self.PRIV_CREATE_CUBE, ugettext('Create cube privileges')),
            (self.PRIV_DELETE_CUBE, ugettext('Delete cube privileges'))
        ]

    def connect(self, sa_session=None, account_id=None):
        """
        Makes a connection to the IQube server and switches to the correct project.
        """

        from django.conf import settings
        from iqportal.iqube.xmla.client import get_client

        if self.role_level is None or self.role_level == self.ROLE_LEVEL_NONE:
            role_name = None
        elif self.role_level in [self.ROLE_LEVEL_ACCOUNT, self.ROLE_LEVEL_DB]:
            role_name = 'iqportal_account_%s' % account_id
        else:
            raise Exception('role_level not implemented: ' + str(self.role_level))

        if self.xmla_client is not None:
            self.xmla_client.set_role_name(role_name)
            return self.xmla_client

        host, port = settings.MONDRIAN_XMLA_SERVER
        self.xmla_client = get_client(host, port, settings.MONDRIAN_XMLA_PATH, self.mondrian_project, self.client_module, role_name, account_id)

        return self.xmla_client

    def __repr__(self):
        return '<DataSource id="%s", name="%s">' % (self.id, self.name)

class DataSourceGroup(object):
    """
    Model representing the link between datasources and groups.
    @DynamicAttrs
    """
    pass

class DataSourceCube(PrivilegeObject):
    """
    @DynamicAttrs
    """
    mark_as_deleted_permission = 'delete_datasource'
    mark_as_archived_permission = 'archive_datasource'

    def get_group_object(self):
        return (DataSourceCubeGroup, 'datasource_cube')

class DataSourceCubeGroup(object):
    """
    @DynamicAttrs
    """
    pass

class DataSourceWidget(PrivilegeObject):
    """
    @DynamicAttrs
    """
    AVAILABLE_AS_DASHBOARD_WIDGET = 1
    AVAILABLE_AS_REPORT_ITEM = 2

    # the filters prevent the dimension from loading it's children
    FILTERS_CHILDREN_NONE = 0
    # the filters load only direct children of the filter options
    FILTERS_CHILDREN_DIRECT_DESCENDANTS = 1
    # the filters load all descendants from the filter options
    FILTERS_CHILDREN_ALL_DESCENDANTS = 2

    _name_property = 'label'

    def is_report_item(self):
        return self._is(DataSourceWidget.AVAILABLE_AS_REPORT_ITEM)

    def is_dashboard_widget(self):
        return self._is(DataSourceWidget.AVAILABLE_AS_DASHBOARD_WIDGET)

    def is_available(self, availability):
        return ((self.availability & availability) == availability)

    _is = is_available

    def get_group_object(self):
        return (DataSourceWidgetGroup, 'datasource_widget')

    def __repr__(self):
        return "<DataSourceWidget id=%s>" % self.id

    def __title(self):
        """
        the widget title is the translated label
        """
        return _(self.label)
    title = property(__title)

class DataSourceWidgetGroup(object):
    """
    @DynamicAttrs
    """
    pass

class DataSourceDashboardWidget(DashboardWidget):
    """
    @DynamicAttrs
    """
    def __repr__(self):
        return '<DataSourceDashboardWidget id=%s>' % self.id

class DataSourceCalculatedMember(PrivilegeObject):
    """
    @DynamicAttrs
    TODO, remove any references to this and then remove this model (1.1, ticket #4366)
    """
    CALCULATED_MEMBER = 0
    CALCULATED_SET = 1

    def validate(self, account_id):
        return True

    def get_group_object(self):
        return (DataSourceCalculatedMemberGroup, 'calculated_member')

    def __repr__(self):
        name = "[%s]" % self.name
        type = 'SET'
        if self.type == self.CALCULATED_MEMBER:
            name = '%s.[%s]' % (self.dimension, self.name)
            type = 'MEMBER'

        return "%s %s AS '%s'" % (type, name, self.formula)

class DataSourceCalculatedMemberGroup(object):
    """
    @DynamicAttrs
    """
    pass

class MdxMember(object):
    """
    @DynamicAttrs
    @deprecated: this is no longer used
    """
    pass

class MdxMemberMember(object):
    """
    @DynamicAttrs
    @deprecated: this is not longer used
    """
    pass

class MdxMethod(PrivilegeObject):
    """
    @DynamicAttrs
    Model for Mdx Methods
    """

    def get_group_object(self):
        return (MdxMethodGroup, 'mdx_method')


class MdxMethodField(object):
    """
    @DynamicAttrs
    """
    pass

class MdxMethodGroup(object):
    """
    @DynamicAttrs
    """
    pass

class MdxCalculatedMember(PrivilegeObject):
    """
    @DynamicAttrs
    """
    CALCULATED_MEMBER = 0
    NAMED_SET = 1

    def get_group_object(self):
        return (MdxCalculatedMemberGroup, 'calculated_member')

class MdxCalculatedMemberGroup(object):
    """
    @DynamicAttrs
    """
    pass

class MdxDataSourceCalculatedMember(MdxCalculatedMember):
    """
    @DynamicAttrs
    """
    pass

