# Copyright (c) 2011 BaseSoft B.V.
# See LICENSE for details.

from django.conf import settings
from django.utils import simplejson
from django.utils.translation import ugettext as _

from basesoft.web.core.util.controllers import AbstractController
from basesoft.web.core.exceptions import PublicUserException, PublicSecurityException
from basesoft.util.ordered_dict import OrderedDict

from iqportal.iqube.core.models import CalculatedMember, DataSource, IncorrectCalculatedMember
from iqportal.iqube.compose.controllers import MdxCalculatedMemberController

from basesoft import logging
logger = logging.getLogger("iqportal.iqube.core.reporters")

_reporters = {}
_reporters_loaded = False

def _load_reporters():
    global _reporters_loaded, _reporters
    if _reporters_loaded:
        return
    _reporters_loaded = True

    for app in settings.INSTALLED_APPS:
        try:
            # TODO: check if file exists and error on import
            module = __import__(app + '.reporters', {}, {}, [''])
        except ImportError:
            continue

        if not hasattr(module, 'reporters'):
            continue

        for reporter in module.reporters:
            _reporters['%s.%s' % (app, reporter.__name__)] = reporter

class ReporterRpcHandler:

    def __init__(self, sa_session, datasource, config, account_id=None, **kwargs):
        _load_reporters()
        self.config = config

        try:
            ReporterClass = _reporters[config['reporter']]
        except IndexError:
            raise Exception('Reporter not found: %s' % config['reporter'])

        self.reporter = ReporterClass(sa_session, datasource, account_id=account_id, config=self.config, **kwargs)

    def get_mdx_config(self, applied_config):
        return self.reporter.get_mdx_config(self.config, self.reporter.deserialize_all(applied_config))

class DefaultCellClickReporter:
    def __init__(self, sa_session, selection, datasource, cube, account_id=None):

        self.sa_session = sa_session
        self.selection = selection
        self.account_id = account_id
        self.cube = cube

        if not isinstance(datasource, DataSource):
            raise Exception('Not a valid DataSource: %s' % datasource)
        self._datasource = datasource

    def _get_mdx(self, column_members, row_members=None):
        # everything in self.selection needs to go on slicers ?
        slicers = []
        for member in self.selection:
            slicers.append({
                 'member_unique_name': member['uname'],
                 'hierarchy_unique_name': member['hierarchy'],
                 'depth': member['lnum']
            })

        client = self._datasource.connect(account_id=self.account_id)
        mdx = client.build_select_statement(column_members, row_members, self.cube, slicers, True, allow_empty_rows=True)
        return client.execute_mdx(mdx)

class DefaultIQubeReporter(AbstractController):
    """
    Default reporter.
    """

    def __init__(self, sa_session, datasource, account_id=None):
        AbstractController.__init__(self, sa_session, account_id=account_id)

        if not isinstance(datasource, DataSource):
            raise Exception('Not a valid DataSource: %s' % datasource)
        self._datasource = datasource

    def deserialize_all(self, applied_config):
        return applied_config

    def _make_mdx(self, name, dimension=None, depth=None):
        if type(name) == dict:
            if name.has_key('depth'):
                depth = name['depth']
            name = name['member_unique_name']

        if dimension is None:
            dimension = name[0:name.find(']')+1]
        elif dimension is False:
            dimension = None # we supply None as dimension so the MDX builder will place the sets in crossjoins
            name = '[%s]' % name
        else:
            name = '%s.[%s]' % (dimension, name)

        if depth is None:
            depth = name.count(']') - 2

        return {
            "member_unique_name":"%s" % name,
            "hierarchy_unique_name": dimension,
            "depth": "%s" % depth
        }
    def _make_mdx_from_cm(self, cm):
        cm = CalculatedMember(member_string=cm)
        return self._make_mdx(cm.name, cm.dimension)

    def get_mdx_config(self, config=None):
        rv = self.get_default_config()
        rv['datasource_id'] = self._datasource.id
        return rv

    def get_default_config(self):
        return {}


class ConfigurableWidgetReporter(DefaultIQubeReporter):
    """
    TODO: validate applied config with possible config. #3255
    """
    _extra_with = None
    all_measures = None
    show_measures = None

    def __init__(self, sa_session, datasource, account_id=None, config=None):
        self._config = config
        DefaultIQubeReporter.__init__(self, sa_session, datasource, account_id)

    def deserialize_all(self, applied_config):
        if type(applied_config) != dict:
            return None

        for index in applied_config['dimensions'].values():
            new = []
            params = applied_config['dimension_%s' % index]
            if type(params) != list:
                params = [params]
            for x in params:
                x = self.deserialize(x)
                if type(x) == list:
                    new = new + x
                else:
                    new.append(x)
            applied_config['dimension_%s' % index] = new

        return applied_config

    def deserialize(self, x):
        try:
            return simplejson.loads(x)
        except ValueError:
            return x
        #except simplejson.decoder.JSONDecodeError:
        #    return x

    def validate(self, config, applied_config):
        return True

    def modify_applied_config(self, config, applied_config):
        for dim in config['dimensions']:
            if dim.has_key('axis_depends_on') and dim['axis_depends_on'] is not None:
                index = applied_config['dimensions'][dim['caption']]
                if not applied_config['dimensions'].has_key(dim['axis_depends_on']):
                    continue
                target = applied_config['dimensions'][dim['axis_depends_on']]
                axis = applied_config['axes_%s' % target]
                applied_config['axes_%s' % index] = axis
                if axis == 'rows':
                    applied_config['row_order'].append(dim['caption'])
                elif axis == 'columns':
                    applied_config['column_order'].append(dim['caption'])
                dim['axis_depends_on'] = None
        return applied_config

    def _get_applied_config(self, config, applied_config):
        rv = {'columns': [], 'rows': [], 'slicers': []}
        mdx_with = []
        split = []
        result_headers = []
        def apply(conf, index, rv):
            # we have no axis set for the current dimension, no reason to continue
            if applied_config['axes_%s' % index] == '':
                logger.warn('ConfigurableWidgetReporter: Axis not set for index %s' % index)
                return
            if applied_config.has_key('result_full_headers_%s' % index) and applied_config['result_full_headers_%s' % index]:
                [result_headers.append(x.get('hierarchy_unique_name')) for x in applied_config['dimension_%s' % index]
                    if x.get('hierarchy_unique_name') not in result_headers]

            for c in conf:
                if type(c) == dict:
                    if c.has_key('calculated_member_id'):
                        c['settings']['axis'] = applied_config['axes_%s' % index]
                        self._apply_mdx_function(applied_config, c, rv)
                        continue
                    rv[applied_config['axes_%s' % index]].append(self._make_mdx(c))
                    continue

                try:
                    _id = int(c)
                    self._auto_configure_mdx_function(applied_config, _id, rv, applied_config['axes_%s' % index])
                    continue
                except ValueError:
                    pass
                try:
                    rv[applied_config['axes_%s' % index]].append(self._make_mdx_from_cm(c))
                    mdx_with.append(str(c))
                except IncorrectCalculatedMember:
                    rv[applied_config['axes_%s' % index]].append(self._make_mdx(c))

        applied_config = self.modify_applied_config(config, applied_config)

        for index in applied_config['dimensions'].values():
            if applied_config.has_key('split_by_%s' % index) and applied_config['split_by_%s' % index]:
                split.append(index)
                continue
            if applied_config['axes_%s' % index] == 'slicers':
                apply(applied_config['dimension_%s' % index], index, rv)

        for i in ['row_order', 'column_order']:

            for d in applied_config[i]:
                if d is None:
                    continue

                try:
                    index = applied_config['dimensions'][d]
                    apply(applied_config['dimension_%s' % index], index, rv)
                    continue
                except KeyError:
                    if d[:8] != 'formula_': # FIXME we should change this to named_set
                        continue
                #get the configured MDX string
                try:
                    formula_conf = applied_config['named_sets'][int(d[8:])]
                    self._apply_mdx_function(applied_config, self.deserialize(formula_conf), rv)
                except IndexError:
                    continue

        if self._extra_with is not None:
            ew = self._extra_with
            if not isinstance(self._extra_with, list):
                ew = [self._extra_with]
            for e in ew:
                mdx_with.append(e)
        return (rv['columns'], rv['rows'], rv['slicers'], mdx_with, split, result_headers, )

    def _auto_configure_mdx_function(self, applied_config, calculated_member_id, rv, axis):
        """
        We try to auto configure calculated members, this can be done in case they have no required inputs.
        """

        mcmc = MdxCalculatedMemberController(self.sa_session, account_id=self.get_account().id)
        formula_config = mcmc.auto_configure_mdx_function(calculated_member_id)
        formula_config['settings']['axis'] = axis
        self._apply_mdx_function(applied_config, formula_config, rv)

    def _apply_mdx_function(self, applied_config, formula_config, rv):

        mcmc = MdxCalculatedMemberController(self.sa_session, account_id=self.get_account().id)
        # retrieve the correct MDX formula
        formula, _type, dimension = mcmc.get_parsed(formula_config['calculated_member_id'], formula_config['replacements'], True)

        if self._extra_with is not None:
            if not isinstance(self._extra_with, list):
                self._extra_with = [self._extra_with]
        else:
            self._extra_with = []

        settings = formula_config['settings']

        name = '[CM %s]' % len(self._extra_with)
        if settings.has_key('label') and settings['label']:
            name = '[%s]' % settings['label']

        name, _with, dimension = mcmc.get_with_statement(name, _type, formula, dimension, settings)
        self._extra_with += _with

        if not rv:
            return {'hierarchy_unique_name': dimension, 'member_unique_name': name}
        rv[settings['axis']].append({'hierarchy_unique_name': dimension, 'member_unique_name': name})

    def _get_mdx_definition(self, cube, columns, rows, slicers, mdx_with, nonempty, axis=None, c=None, hierarchize=True, result_headers= None):

        context = None
        if self._config.has_key('context'):
            context = self._config['context']

        cm_controller = MdxCalculatedMemberController(self.sa_session, account_id=self.account_id)
        rv = DefaultIQubeReporter.get_mdx_config(self)
        rv['mdx_config'] = {
            'columns': [i for i in columns],
            'rows': [i for i in rows],
            'cube': cube,
            'slicers': [i for i in slicers],
            'nonempty': nonempty,
            'mdx_with': [i for i in mdx_with],
            'hierarchize': hierarchize,
            'context': context,
            'result_full_headers': result_headers
        }

        name = None
        # TODO: is this still used?
        if axis is not None:
            if type(c) == dict:
                m = self._make_mdx(c)
                if m not in rv['mdx_config'][axis]:
                    rv['mdx_config'][axis].append(m)
                name = c
            else:
                try:
                    _id = int(c)
                    cm = self.cm_controller.get_calculated_member(_id)[0]
                    name = '[%s]' % cm.label
                    formula, _type, dimension = self.cm_controller.get_parsed(cm.id, [], True)
                    name, _with, dimension = self.cm_controller.get_with_statement(name, cm.type, formula, 'Position', None)
                    rv['mdx_config'][axis].append({'hierarchy_unique_name': dimension, 'member_unique_name': name})
                    rv['mdx_config']['mdx_with'] += _with
                except ValueError:
                    # only add if not already selected for this axis
                    m = self._make_mdx(c)
                    if m not in rv['mdx_config'][axis]:
                        rv['mdx_config'][axis].append(m)
                name = c
        return (name, rv)

    def get_mdx_config(self, config, applied_config):

        if not self.validate(config, applied_config):
            raise PublicUserException('Incorrect usage of widget configurations')

        hierarchize = True
        if applied_config.has_key('hierarchize'):
            hierarchize = applied_config['hierarchize']

        columns, rows, slicers, mdx_with, split, result_headers = self._get_applied_config(config, applied_config)

        nonempty = True
        if applied_config.has_key('hide_empty_fields'):
            nonempty = applied_config['hide_empty_fields']

        rv = []
        if len(split) == 0:
            rv.append(self._get_mdx_definition(config['cube'], columns, rows, slicers, mdx_with, nonempty, hierarchize=hierarchize, result_headers=result_headers))
        else:
            for index in split:
                for c in applied_config['dimension_%s' % index]:
                    rv.append(self._get_mdx_definition(config['cube'], columns, rows, slicers, mdx_with, nonempty,
                                                       applied_config['axes_%s' % index], c, hierarchize=hierarchize, result_headers=result_headers))
        #print "reporter config: %s" % applied_config
        #print rv
        return OrderedDict(rv)

    def __process_measure(self, applied_config, field, settings, use_measure):
        """
        Processes a measure to set its label and format string.
        """
        fmt_string = ''
        measures, label = settings

        decimal_field = '%s_decimals' % field
        label_field = '%s_label' % field
        format_string_field = '%s_format_string' % field

        # if there is a decimal field set for the current measure, apply a format string.
        # This is here for backwards compatibility, the correct field to use is the Format String field.
        if applied_config.has_key(decimal_field):
            try:
                dec = int(applied_config[decimal_field])
                fmt_string = ", FORMAT_STRING='#,##0.%s'" % ('0' * dec)
            except ValueError:
                pass

        # if the label is overridden by the user, set it.
        if applied_config.has_key(label_field) and applied_config[label_field]:
            label = applied_config[label_field]

        measures_member = '[Measures].[%s]' % label
        if use_measure == field:
            self._use_measure = measures_member

        if applied_config.has_key(format_string_field) and applied_config[format_string_field]:
            if applied_config[format_string_field][0] == '#':
                fmt_string = ", FORMAT_STRING='%s'" % applied_config[format_string_field]
            else:
                fmt_string = ", FORMAT_STRING=%s" % applied_config[format_string_field]
                fmt_string = fmt_string.replace('{value}', measures_member)

        # if we have a special processing method for this field, use that instead of the default processing.
        if hasattr(self, 'process_%s' % field):
            getattr(self, 'process_%s' % field)(applied_config, field, measures_member, fmt_string)
            return

        # merge all the measures from the settings into a new measure
        if type(measures) in [str, unicode]:
            members = measures
        else:
            members = None
            for m in measures:
                m = '[Measures].[%s]' % m
                if members is None:
                    members = m
                    continue
                members = members + '+' + m

        # Only do this in case the original Measure is different from the calculated measure.
        if measures_member != members:
            self._extra_with.append("MEMBER %s as '%s'%s" % (measures_member, members, fmt_string))
        # always add the measure to the "all_measures" list.
        self.all_measures.append(measures_member)
        # if this measure has to be shown, add it to the list.
        if applied_config.has_key('show_%s_measure' % field) and applied_config['show_%s_measure' % field]:
            self.show_measures[field] = measures_member

    def process_measures(self, applied_config, fields, options, use_measure):
        """
        Processes all measures supplied, this will set variable labels and format strings.
        @param applied_config: the dictionary with applicable configuration settings
        @param fields: a list with measure handles used in the javascript configurable widget forms.
        @param options: dictionary with settings per field
        """
        self.all_measures = []
        self.show_measures = {}

        for field in fields:
            self.__process_measure(applied_config, field, options[field], use_measure)

        if applied_config.has_key('calculated_measures'):
            for cm in applied_config['calculated_measures']:
                if type(cm) in (str, unicode, ):
                    cm = self.deserialize(cm)
                if type(cm) != dict:
                    continue

                if cm.has_key('calculated_member_id'):
                    result = self._apply_mdx_function(applied_config, cm, rv=False)
                    self.show_measures[cm['settings']['label']] = result
                    continue
                result = self._make_mdx(cm)
                self.show_measures[result['member_unique_name']] = result

        measures_index = applied_config['dimensions']['Measures']
        # In case measures is set to slicers and there is more than one selected, this might give weird results,
        # move it to the columns Axis.
        if len(self.show_measures) > 1 and applied_config['axes_%s' % measures_index] == 'slicers':
            applied_config['axes_%s' % measures_index] = 'columns'
            applied_config['column_order'].append('Measures')

        ordered = [f for f in fields if self.show_measures.has_key(f)]
        for f in self.show_measures.keys():
            if f not in ordered:
                ordered.append(f)

        # set the order we want to show
        if applied_config.has_key('measure_order') and applied_config['measure_order']:
            ordered = [f for f in applied_config['measure_order'] if self.show_measures.has_key(f)]

        applied_config['dimension_%s' % measures_index] = []
        for i in ordered:
            if type(self.show_measures[i]) == list:
                for m in self.show_measures[i]:
                    applied_config['dimension_%s' % measures_index].append(m)
                continue
            applied_config['dimension_%s' % measures_index].append(self.show_measures[i])

class AbstractRankConfigurableWidgetReporter(ConfigurableWidgetReporter):
    """
    Abstract class used for Ranking widgets (League Tables etc.)
    """
    RANK_MEMBER_NAME = None # tuple (hierarchy, label) for calculated member name
    RANK_MEMBER_VALUE = None # tuple (hierarchy, label) for calculated member value
    RANK_HIERARCHY = None # hierarchy name
    RANK_MEMBERS = None # tuple of members (.children, ...)
    RANK_AXIS = None # columns or rows
    RANK_ORDER = 'BDESC'

    def get_mdx_config(self, config, applied_config):

        hierarchy, label = self.RANK_MEMBER_NAME
        value_hierarchy, value_label = self.RANK_MEMBER_VALUE

        rank = """MEMBER %(rankmembername)s AS 'Rank(
                %(rankhierarchy)s.CurrentMember,
                Order(
                    {%(rankmembers)s},
                    %(rankmembervalue)s,
                    %(rankorder)s
                ),
                %(rankmembervalue)s
            )'""" % {
                'rankmembername': '[%s].[%s]' % (hierarchy, label),
                'rankmembervalue': '[%s].[%s]' % (value_hierarchy, value_label),
                'rankhierarchy': '[%s]' % self.RANK_HIERARCHY,
                'rankmembers': self.RANK_MEMBERS,
                'rankorder': self.RANK_ORDER
            }

        index = applied_config['dimensions'][hierarchy]
        if applied_config['axes_%s' % index] == "":
            applied_config['axes_%s' % index] = self.RANK_AXIS

        applied_config['dimension_%s' % index].append(rank)

        return ConfigurableWidgetReporter.get_mdx_config(self, config, applied_config)


class ReportResultIQubeReporter(DefaultIQubeReporter):

    def __init__(self, sa_session, datasource, account_id=None, config=None):
        self._config = config
        DefaultIQubeReporter.__init__(self, sa_session, datasource, account_id)

    def get_mdx_config(self, config, applied_config):

        rv = {
            'columns': applied_config['columns'],
            'rows': applied_config['rows'],
            'cube': config['cube'],
            'slicers': applied_config['slicers'],
            'nonempty': applied_config['hide_empty_fields'],
        }
        return rv


reporters = (ConfigurableWidgetReporter, ReportResultIQubeReporter, )

