# Copyright (c) 2010 BaseSoft B.V.
# See LICENSE for details.

"""
This file contains the objects, functions and variables to setup the
IQube core application.
"""
import datetime

from sqlalchemy import Column, Integer, ForeignKey, String, DateTime, SmallInteger, UnicodeText, Unicode, UniqueConstraint, Date, Boolean
from sqlalchemy.dialects.mysql import MEDIUMTEXT

from django.utils.translation import ugettext as _

from basesoft.sqlalchemy.table import IQTable
from basesoft.web.contrib.license.hooks import PrivilegeObjectMapper
from basesoft.web.contrib.license import models as license_models
from basesoft.web.contrib.dashboard.models import DashboardWidget
from basesoft.logging import getLogger
logger = getLogger('iqportal.iqube.core')

__mapping_defined = False
__tables_defined = False
tables = {}

def reset():
    global __mapping_defined
    global __tables_defined

    __mapping_defined = False
    __tables_defined = False

def get_tables():
    return tables

def do_tables(metadata):
    global __tables_defined
    if not __tables_defined:

        # The DataSource
        tblDataSource = IQTable('tblDataSource', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            # this is the name the user deals with
            Column('name', String(255), nullable=False),
            # datasource type: Database or File
            Column('type', SmallInteger, nullable=False),
            # This is the project name on the IQube server.
            Column('mondrian_project', String(255), nullable=False, unique=True),
            # the client module to be loaded for mdx creation
            Column('client_module', String(255), nullable=True),

            Column('role_level', SmallInteger, nullable=True),
            # flag that indicates if the datasource needs a flush
            Column('has_stale_definitions', SmallInteger, nullable=False, default=0),

            # time in minutes that expire xmla_cache results by the XmlaCacheCleanup task
            # set to 0 to disable caching (realtime datasource) (to be implemented)
            # set to null to never expire cache (a special task should take care of it then)
            Column('xmla_cache_expiry', Integer, nullable=True, default=3600),

            Column('deleted', SmallInteger, nullable=False, default=0),
            Column('archived', SmallInteger, nullable=False, default=0),

            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        # This is the link between a group of accounts and the datasource, this will be used for access rights.
        tblDataSourceGroup = IQTable('tblDataSourceGroup', metadata,
            Column('datasource_id', Integer, ForeignKey('tblDataSource.id', ondelete="CASCADE"), primary_key=True),
            Column('group_id', Integer, ForeignKey('tblGroup.id'), primary_key=True),
            Column('privilege', Integer, nullable=False, primary_key=True, autoincrement=False),
            Column('group_has_privilege', SmallInteger, default=0),
            Column('license_has_privilege', SmallInteger, default=0),
            Column('domain_has_privilege', SmallInteger, default=0),
            Column('world_has_privilege', SmallInteger, default=0),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        # Cubes for datasources
        tblDataSourceCube = IQTable('tblDataSourceCube', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('datasource_id', Integer, ForeignKey('tblDataSource.id', ondelete="CASCADE"), nullable=False),
            Column('name', Unicode(255), nullable=False),
            Column('default_config', MEDIUMTEXT, nullable=True),
            Column('deleted', SmallInteger, nullable=False, default=0),
            Column('archived', SmallInteger, nullable=False, default=0),
            Column('last_aggregation_update', DateTime(), nullable=True, default=None),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now),
            UniqueConstraint('datasource_id', 'name')
        )

        tblDataSourceCubeGroup = IQTable('tblDataSourceCubeGroup', metadata,
            Column('datasource_cube_id', Integer, ForeignKey('tblDataSourceCube.id', ondelete="CASCADE"), primary_key=True),
            Column('group_id', Integer, ForeignKey('tblGroup.id'), primary_key=True),
            Column('privilege', Integer, nullable=False, primary_key=True, autoincrement=False),
            Column('group_has_privilege', SmallInteger, default=0),
            Column('license_has_privilege', SmallInteger, default=0),
            Column('domain_has_privilege', SmallInteger, default=0),
            Column('world_has_privilege', SmallInteger, default=0),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        # Calculated Members bound to a datasource # FIXME merge this with tblMdxCalculatedMember
        tblDataSourceCalculatedMember = IQTable('tblDataSourceCalculatedMember', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('datasource_id', Integer, ForeignKey('tblDataSource.id', ondelete="CASCADE"), nullable=False),
            Column('cube', UnicodeText, nullable=True), # can be cube specific CM
            Column('name', Unicode(255), nullable=False),
            Column('caption', Unicode(255), nullable=False),
            Column('polymorphic_type', String(255), nullable=False),

            Column('dimension', UnicodeText, nullable=False, default=u'Measures'),
            Column('formula', UnicodeText, nullable=False),

            Column('type', SmallInteger, nullable=False, default=0),
            Column('format_string', Unicode(255), nullable=True),
            Column('format_expression', Unicode(255), nullable=True),

            Column('deleted', SmallInteger, nullable=False, default=0),
            Column('archived', SmallInteger, nullable=False, default=0),

            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now),

            UniqueConstraint('datasource_id', 'name')
        )

        tblDataSourceCalculatedMemberGroup = IQTable('tblDataSourceCalculatedMemberGroup', metadata,
            Column('calculated_member_id', Integer, ForeignKey('tblDataSourceCalculatedMember.id', ondelete="CASCADE"), primary_key=True),
            Column('group_id', Integer, ForeignKey('tblGroup.id'), primary_key=True),
            Column('privilege', Integer, nullable=False, primary_key=True, autoincrement=False),
            Column('group_has_privilege', SmallInteger, default=0),
            Column('license_has_privilege', SmallInteger, default=0),
            Column('domain_has_privilege', SmallInteger, default=0),
            Column('world_has_privilege', SmallInteger, default=0),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        tblDataSourceWidget = IQTable('tblDataSourceWidget', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('datasource_id', Integer, ForeignKey('tblDataSource.id', ondelete="CASCADE"), nullable=False),

            Column('label', Unicode(255), nullable=False),
            Column('category', Unicode(255), nullable=False),
            Column('application', Unicode(255), nullable=True),

            Column('cube', UnicodeText, nullable=False),
            Column('config', UnicodeText, nullable=False),
            Column('applied_filters', MEDIUMTEXT, nullable=True),
            Column('availability', Integer, nullable=False, default=1),

            Column('deleted', SmallInteger, nullable=False, default=0),
            Column('archived', SmallInteger, nullable=False, default=0),

            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now),

            UniqueConstraint('datasource_id', 'label')
        )

        tblDataSourceWidgetGroup = IQTable('tblDataSourceWidgetGroup', metadata,
            Column('datasource_widget_id', Integer, ForeignKey('tblDataSourceWidget.id', ondelete="CASCADE"), primary_key=True),
            Column('group_id', Integer, ForeignKey('tblGroup.id'), primary_key=True),
            Column('privilege', Integer, nullable=False, primary_key=True, autoincrement=False),
            Column('group_has_privilege', SmallInteger, default=0),
            Column('license_has_privilege', SmallInteger, default=0),
            Column('domain_has_privilege', SmallInteger, default=0),
            Column('world_has_privilege', SmallInteger, default=0),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        # IQube Configurable Widget dashboard entry
        tblDataSourceDashboardWidget = IQTable('tblDataSourceDashboardWidget', metadata,
            Column('widget_id', Integer, ForeignKey('tblDashboardWidget.id', ondelete='CASCADE'), primary_key=True),
            Column('datasource_widget_id', Integer, ForeignKey('tblDataSourceWidget.id', ondelete="CASCADE"), nullable=True),
            Column('datasource_cube_id', Integer, ForeignKey('tblDataSourceCube.id', ondelete="CASCADE"), nullable=True),
            Column('applied_config', MEDIUMTEXT, nullable=False),
            Column('graph_config', UnicodeText, nullable=True),
            Column('table_config', UnicodeText, nullable=True),
            Column('linked_items', UnicodeText, nullable=True),
        )

        tblMdxMember = IQTable('tblMdxMember', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('datasource_id', Integer, ForeignKey('tblDataSource.id', ondelete="CASCADE"), nullable=False),
            Column('cube', UnicodeText, nullable=False),

            Column('definition', UnicodeText, nullable=False),

            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        tblMdxMemberMember = IQTable('tblMdxMemberMember', metadata,
            Column('member_id', Integer, ForeignKey('tblMdxMember.id', ondelete="CASCADE"), nullable=False, primary_key=True),
            Column('grouped_member_id', Integer, ForeignKey('tblMdxMember.id', ondelete="CASCADE"), nullable=False, primary_key=True),
        )

        tblMdxMethod = IQTable('tblMdxMethod', metadata,
            Column('id', Integer, primary_key=True, autoincrement = True),
            Column('label', Unicode(50), nullable = False),
            Column('name', Unicode(50), nullable = False),
            Column('category', UnicodeText, nullable=True, default=None),

            Column('return_type', Unicode(50), nullable = False),
            Column('example', UnicodeText, nullable = True),

            Column('deleted', SmallInteger, nullable=False, default=0),
            Column('archived', SmallInteger, nullable=False, default=0),

            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now),


        )

        tblMdxMethodField = IQTable('tblMdxMethodField', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('order', Integer, nullable=False),
            Column('method_id', Integer, ForeignKey('tblMdxMethod.id', ondelete="CASCADE")),
            Column('field_type', Unicode(50), nullable=False),
            Column('required_field', Boolean, nullable=False)
        )

        tblMdxMethodGroup = IQTable('tblMdxMethodGroup', metadata,
            Column('mdx_method_id', Integer, ForeignKey('tblMdxMethod.id', ondelete="CASCADE"), primary_key=True),
            Column('group_id', Integer, ForeignKey('tblGroup.id'), primary_key=True),
            Column('privilege', Integer, nullable=False, primary_key=True),
            Column('group_has_privilege', SmallInteger, default=0),
            Column('license_has_privilege', SmallInteger, default=0),
            Column('domain_has_privilege', SmallInteger, default=0),
            Column('world_has_privilege', SmallInteger, default=0),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        # merge of tblMdxSavedMethod and tblDataSourceCalculatedMember tables.
        tblMdxCalculatedMember = IQTable('tblMdxCalculatedMember', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('polymorphic_type', String(255), nullable=False),

            Column('label', UnicodeText, nullable=False),

            Column('formula', UnicodeText, nullable=False), # json string
            Column('type', SmallInteger, nullable=False, default=0), # calculated member vs. named set
            Column('input_count', Integer, nullable=False, default=0), # the number of unlinked inputs
            Column('dimension', UnicodeText, nullable=True, default=None),

            Column('format_string', UnicodeText, nullable=True, default=None),

            Column('deleted', SmallInteger, nullable=False, default=0),
            Column('archived', SmallInteger, nullable=False, default=0),

            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now),

        )

        tblMdxCalculatedMemberGroup = IQTable('tblMdxCalculatedMemberGroup', metadata,
            Column('calculated_member_id', Integer, ForeignKey('tblMdxCalculatedMember.id', ondelete="CASCADE"), primary_key=True),
            Column('group_id', Integer, ForeignKey('tblGroup.id'), primary_key=True),
            Column('privilege', Integer, nullable=False, autoincrement=False, primary_key=True),
            Column('group_has_privilege', SmallInteger, default=0),
            Column('license_has_privilege', SmallInteger, default=0),
            Column('domain_has_privilege', SmallInteger, default=0),
            Column('world_has_privilege', SmallInteger, default=0),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        tblMdxDataSourceCalculatedMember = IQTable('tblMdxDataSourceCalculatedMember', metadata,
            Column('calculated_member_id', Integer, ForeignKey('tblMdxCalculatedMember.id', ondelete='CASCADE'), primary_key=True),
            Column('datasource_id', Integer, ForeignKey('tblDataSource.id', ondelete="CASCADE"), nullable=False),
            Column('cube', UnicodeText, nullable=True),
            Column('constraints', UnicodeText, nullable=True),
        )

        # common time dimension tables
        aggTimeQuarter = IQTable('aggTimeQuarter', metadata,
            Column('id', Integer, primary_key=True),
            Column('date', Date),
            Column('year', SmallInteger),
            Column('quarter', SmallInteger),
            Column('month', SmallInteger),
            Column('day', SmallInteger),
        )


        aggTimeWeekUS = IQTable('aggTimeWeekUS', metadata,
            Column('id', Integer, primary_key=True),
            Column('date', Date),
            Column('year', SmallInteger),
            Column('week', SmallInteger),
            Column('day', SmallInteger),
        )

        aggTimeWeekISO8601 = IQTable('aggTimeWeekISO8601', metadata,
            Column('id', Integer, primary_key=True),
            Column('date', Date),
            Column('year', SmallInteger),
            Column('week', SmallInteger),
            Column('day', SmallInteger),
        )

        tables['tblDataSource'] = tblDataSource
        tables['tblDataSourceGroup'] = tblDataSourceGroup

        tables['tblDataSourceCube'] = tblDataSourceCube
        tables['tblDataSourceCubeGroup'] = tblDataSourceCubeGroup

        tables['tblMdxMember'] = tblMdxMember
        tables['tblMdxMemberMember'] = tblMdxMemberMember

        tables['tblDataSourceWidget'] = tblDataSourceWidget
        tables['tblDataSourceWidgetGroup'] = tblDataSourceWidgetGroup
        tables['tblDataSourceDashboardWidget'] = tblDataSourceDashboardWidget

        tables['tblDataSourceCalculatedMember'] = tblDataSourceCalculatedMember # FIXME
        tables['tblDataSourceCalculatedMemberGroup'] = tblDataSourceCalculatedMemberGroup

        tables['tblMdxMethod'] = tblMdxMethod
        tables['tblMdxMethodField'] = tblMdxMethodField
        tables['tblMdxMethodGroup'] = tblMdxMethodGroup

        tables['tblMdxCalculatedMember'] = tblMdxCalculatedMember
        tables['tblMdxCalculatedMemberGroup'] = tblMdxCalculatedMemberGroup
        tables['tblMdxDataSourceCalculatedMember'] = tblMdxDataSourceCalculatedMember

        tables['aggTimeQuarter'] = aggTimeQuarter
        tables['aggTimeWeekUS'] = aggTimeWeekUS
        tables['aggTimeWeekISO8601'] = aggTimeWeekISO8601

        __tables_defined = True

def do_mappings():
    """
    This function is called by Django middleware when needed.
    """
    global __mapping_defined
    if not __mapping_defined:
        from sqlalchemy.orm import mapper, relation
        from iqportal.iqube.core import models as core_models
        from basesoft.web.contrib.account import models as account_models

        mapper(core_models.DataSource, tables['tblDataSource'],
            extension = PrivilegeObjectMapper(),
            properties = {
                'groups': relation(core_models.DataSourceGroup, backref='datasource', cascade="all", passive_deletes=True)
        })

        mapper(core_models.DataSourceGroup, tables['tblDataSourceGroup'], properties = {
            'group': relation(license_models.Group)
        })

        mapper(core_models.DataSourceCube, tables['tblDataSourceCube'],
            extension = PrivilegeObjectMapper(),
            properties = {
                'datasource': relation(core_models.DataSource),
                'groups': relation(core_models.DataSourceCubeGroup, backref='datasource_cube', cascade="all", passive_deletes=True)
        })

        mapper(core_models.DataSourceCubeGroup, tables['tblDataSourceCubeGroup'], properties = {
            'group': relation(license_models.Group)
        })

        mapper(core_models.DataSourceCalculatedMember, tables['tblDataSourceCalculatedMember'],
            extension = PrivilegeObjectMapper(),
            properties = {
                'datasource': relation(core_models.DataSource),
                'groups': relation(core_models.DataSourceCalculatedMemberGroup, backref='calculated_member', cascade="all", passive_deletes=True)
            },
            polymorphic_on=tables['tblDataSourceCalculatedMember'].c.polymorphic_type,
            polymorphic_identity='CalculatedMember',
            with_polymorphic='*'
        )

        mapper(core_models.DataSourceCalculatedMemberGroup, tables['tblDataSourceCalculatedMemberGroup'], properties = {
            'group': relation(license_models.Group)
        })

        mapper(core_models.DataSourceWidget, tables['tblDataSourceWidget'],
            extension = PrivilegeObjectMapper(),
            properties = {
                'datasource': relation(core_models.DataSource),
                'groups': relation(core_models.DataSourceWidgetGroup, backref='datasource_widget', cascade="all", passive_deletes=True)
        })

        mapper(core_models.DataSourceWidgetGroup, tables['tblDataSourceWidgetGroup'], properties = {
            'group': relation(license_models.Group)
        })

        mapper(core_models.DataSourceDashboardWidget, tables['tblDataSourceDashboardWidget'],
            properties = {
                'datasource_widget': relation(core_models.DataSourceWidget),
                'datasource_cube': relation(core_models.DataSourceCube),
            },
            polymorphic_identity='DataSourceDashboardWidget',
            inherits=DashboardWidget
        )

        mapper(core_models.MdxMember, tables['tblMdxMember'], properties = {
            'datasource': relation(core_models.DataSource),
            'members': relation(core_models.MdxMember, secondary=tables['tblMdxMemberMember'], passive_deletes=True,
                    primaryjoin=tables['tblMdxMember'].c.id==tables['tblMdxMemberMember'].c.grouped_member_id,
                    secondaryjoin=tables['tblMdxMemberMember'].c.member_id==tables['tblMdxMember'].c.id,
                    backref="grouped_member")
        })

        mapper(core_models.MdxMemberMember, tables['tblMdxMemberMember'], properties = {
        })

        mapper(core_models.MdxMethod, tables['tblMdxMethod'],
            extension = PrivilegeObjectMapper(),
            properties = {
                'groups': relation(core_models.MdxMethodGroup, backref='mdx_method', cascade="all"),
        })

        mapper(core_models.MdxMethodField, tables['tblMdxMethodField'],
            properties = {
                'method': relation(core_models.MdxMethod, backref='fields')
        })

        mapper(core_models.MdxMethodGroup, tables['tblMdxMethodGroup'],
            properties = {
                'group': relation(license_models.Group),
        })

        mapper(core_models.MdxCalculatedMember, tables['tblMdxCalculatedMember'],
            extension = PrivilegeObjectMapper(),
            properties = {
                'groups': relation(core_models.MdxCalculatedMemberGroup, backref='calculated_member', cascade='all')
            },
            polymorphic_on=tables['tblMdxCalculatedMember'].c.polymorphic_type,
            polymorphic_identity='CalculatedMember',
            with_polymorphic='*'
        )

        mapper(core_models.MdxCalculatedMemberGroup, tables['tblMdxCalculatedMemberGroup'],
            properties = {
                'group': relation(license_models.Group)
        })

        mapper(core_models.MdxDataSourceCalculatedMember, tables['tblMdxDataSourceCalculatedMember'],
            polymorphic_identity='DataSourceCalculatedMember',
            inherits=core_models.MdxCalculatedMember,
            properties = {
                'datasource': relation(core_models.DataSource),
            }
        )

        __mapping_defined = True


def fill_data(conn):
    pass

def app_permissions_category():
    """
    Returns a category for all permissions defined for this app
    """
    return license_models.PermissionCategory(u'IQube')

def app_permissions():
    """
    The permissions used within this application. Can be sync'd with the
    `iqsyncpermissions` command.
    """
    return [

        # Datasource
        license_models.Permission(u'create_datasource', u"""
            This gives the right to create a datasource.
            """, _("Create datasources")),
        license_models.Permission(u'view_datasource', u"""
            This gives the right to view a datasource.
            """, _("View datasources")),
        license_models.Permission(u'update_datasource', u"""
            This gives the right to update a datasource.
            """, _("Update datasources")),
        license_models.Permission(u'delete_datasource', u"""
            This gives the right to mark a datasource as deleted.
            """, _("Delete datasources")),
        license_models.Permission(u'archive_datasource', u"""
            This gives the right to mark a datasource as archived.
            """, _("Archive datasources")),
        license_models.Permission(u'datasource_manager', u"""
            This gives the right to access the datasource manager.
            """, _("Datasource manager access")),
        license_models.Permission(u'datasource_view_cube', u"""
            This gives the right to view cubes for a datasource.
            """, _("View cubes for datasources")),
        license_models.Permission(u'datasource_create_cube', u"""
            This gives the right to create a cube on a datasource.
            """, _("Create cubes on datasources")),
        license_models.Permission(u'datasource_delete_cube', u"""
            This gives the right to delete a cube from a datasource.
            """, _("Delete cubes from datasources")),
        license_models.Permission(u'datasource_delete_table', u"""
            This gives the right to delete a table from a file based datasource.
            """, _("Delete tables from datasources")),
        license_models.Permission(u'file_based_datasource_access', u"""
            This gives the right to access File based datasource.
            """, _("File based datasource access")),
        license_models.Permission(u'pivot_table', u"""
            This gives the right to be able to use the pivot table in results
            """, _("Can use Pivot Table")),

        # Calculated Members
        license_models.Permission(u'create_calculated_member', u"""
            This gives the right to create a calculated member.
            """, _("Create calculated member")),
        license_models.Permission(u'view_calculated_member', u"""
            This gives the right to view a calculated member.
            """, _("View calculated member")),
        license_models.Permission(u'update_calculated_member', u"""
            This gives the right to update a calculated member.
            """, _("Update calculated member")),
        license_models.Permission(u'delete_calculated_member', u"""
            This gives the right to delete a calculated member.
            """, _("Delete calculated member")),

        # Calculated Members
        license_models.Permission(u'create_datasource_widget', u"""
            This gives the right to create a datasource configurable widget.
            """, _("Create configurable widget")),
        license_models.Permission(u'view_datasource_widget', u"""
            This gives the right to view datasource configurable widgets.
            """, _("View configurable widget")),
        license_models.Permission(u'update_datasource_widget', u"""
            This gives the right to update a datasource configurable widget.
            """, _("Update configurable widget")),
        license_models.Permission(u'delete_datasource_widget', u"""
            This gives the right to delete a datasource configurable widget.self.group
            """, _("Delete configurable widget")),

        # Mdx Methods
        license_models.Permission(u'manage_mdx_methods', u"""
            This gives the right to manage MDX Methods.
            """, _("Manage MDX Methods")),

    ]


def app_configs():
    """
    The account configs available for this application. This can be sync'd with the
    `iqsyncpermissions` command.
    """
    from basesoft.web.contrib.account import models as account_models
    configs = []

    config = account_models.Config()
    config.name = 'iqube.core.widget_isdocked'
    config.type = 'boolean'
    config.default = 'False'
    config.description = u'Boolean used to determine if result properties panels are docked.'
    configs.append(config)

    return configs
