# Copyright (c) 2009 BaseSoft B.V.
# See LICENSE for details.

import re

from django.shortcuts import render_to_response
from django.http import HttpResponse
from django.utils import simplejson as json, simplejson
from django.conf import settings
from django.template.loader import render_to_string
from django.template import RequestContext

from basesoft.web.core.jsonrpc.dispatcher import JsonRpcDispatcher
from basesoft.web.core.jsonrpc.dispatcher import JsonRpcJSONEncoder
from basesoft.web.contrib.license.controllers import LicenseController
from basesoft.web.core.util.db import Session
from basesoft.web.core.exceptions import SecurityException

from iqportal.iqube.core.controllers import DataSourceController, ProjectController

def js_module_iqube_env(request):
    """
    """

    iqube_env = {}
    iqube_env['LUCIDDB_SCHEMA_PREFIX'] = settings.LUCIDDB_SCHEMA_PREFIX.upper()

    iqube_env_str = json.dumps(iqube_env, ensure_ascii=False, cls=JsonRpcJSONEncoder)
    return render_to_string("iqube_env.html", {"iqube_env": iqube_env_str}, context_instance=RequestContext(request))


def json_rpc(request):
    """
    """
    aid = request.session['user_id']
    dsc = DataSourceController(request.Session(), account_id=request.session['user_id'])

    class Handler:

        def save_datasource(self, id, args):
            """
            Save the data source and create IQube stuff.
            """
            ds = dsc.save_or_update_datasource(id, args['name'], args['description'], args['name'], args['protocol'], \
                                               args['user'], args['password'], args['host'], args['port'], args['database'])
            return ds.id

        def load_datasource(self, id):
            """
            Load an existing datasource.
            """
            return dsc.load_datasource_state(id)

        def load_datasource_objects(self, id):
            """
            Returns 'objects' related to a datasource
            Different datasources will have different objects returned
            """
            return dsc.get_datasource_objects(id)

        def get_datasource_cubes(self, datasource_id):
            """
            Returns the total number of iqubes for a datasource.
            """
            if datasource_id is None:
                return []

            ds = dsc.get_datasources(datasource_id).pop()

            cubes = [c.name for c in self.dsc.get_cubes(ds)]

            client = ds.connect(account_id=aid)
            rv = [{'name': c['cube_name'], 'description': c['description'], 'type': c['cube_type']}
                    for c in client.get_cubes() if c['cube_name'] in cubes]
            if len(rv):
                rv[0]['default'] = True
            return rv

        def get_datasource_tables(self, id):
            """
            Returns the data tables for a datasource.
            """
            return dsc.get_datasource_tables(id)

        def get_datasource_table_definition(self, id, table_name):
            """
            """
            return dsc.get_datasource_table_definition(id, table_name)

        def get_root_members(self, datasource, cube):
            ds = dsc.get_datasources(datasource).pop()
            client = ds.connect(account_id=aid)
            return client.get_hierarchies(cube)

        def get_hierarchy_levels(self, datasource, cube, hierarchy_unique_name):
            ds = dsc.get_datasources(datasource).pop()
            client = ds.connect(account_id=aid)
            return client.get_hierarchy_levels(cube, hierarchy_unique_name)

        def get_hierarchy_members(self, datasource, cube, hierarchy_unique_name):
            ds = dsc.get_datasources(datasource).pop()
            client = ds.connect(account_id=aid)
            return client.get_member_tree_op(cube, hierarchy_unique_name, client.MDTREEOP_CHILDREN, 'HIERARCHY_UNIQUE_NAME')

        def get_level_members(self, datasource, cube, level_unique_name):
            ds = dsc.get_datasources(datasource).pop()
            client = ds.connect(account_id=aid)
            return client.get_level_members(cube, level_unique_name)

        def get_named_sets(self, datasource, cube):
            ds = dsc.get_datasources(datasource).pop()
            client = ds.connect(account_id=aid)
            return client.get_named_sets(cube)

        def get_member_children(self, datasource, cube, member_unique_name):
            ds = dsc.get_datasources(datasource).pop()
            client = ds.connect(account_id=aid)
            return client.get_member_tree_op(cube, member_unique_name, client.MDTREEOP_CHILDREN)

        def test_connection(self, args):
            """
            Test a connection by using SQLAlchemy.
            """
            from sqlalchemy.exc import OperationalError
            try:
                dsc.test_connection(args['protocol'],args['user'], args['password'], args['host'], args['port'], args['database'])
            except OperationalError, e:
                return e.message
            return None

        def get_datasources(self):
            """
            Returns dictionary with id and name for all datasources the user has rights on (both view and create cube).
            """
            return [{'value': d.id, 'label': d.name} for d in dsc.get_datasource_with_cube_privs()]

        def get_cubes_by_local_table(self, datasource_id, table):
            """
            Gets cubes linked to local luciddb tables for datasource_id
            """
            datasource = dsc.get_datasources(datasource_id)[0]
            pc = ProjectController(request.Session(), account_id=request.session['user_id'])
            return pc.get_cubes_by_local_table(datasource.mondrian_project, table)

        def check_create_cube_privilege(self, datasource_id):
            """
            Checks if the user has the right to create a cube on datasource_id
            """
            ds = dsc.get_datasource_with_cube_privs(datasource_id)
            if len(ds) == 0:
                return False
            return True

        def check_delete_cube_privilege(self, datasource_id):
            """
            Checks if the user has the right to delete a cube on datasource_id
            """
            ds = dsc.get_datasource_with_delete_cube_privs(datasource_id)
            if len(ds) == 0:
                return False
            return True

        def delete_local_table(self, datasource_id, table_name):
            """
            Drop a local table from a file based datasource
            """
            datasource = dsc.get_datasources(datasource_id)[0]
            pc = ProjectController(request.Session(), account_id=request.session['user_id'])
            pc.delete_local_table(datasource, table_name)
            return True


        def delete_cube(self, datasource_id, cube_name):
            datasource = dsc.get_datasources(datasource_id)[0]
            pc = ProjectController(request.Session(), account_id=request.session['user_id'])
            pc.delete_mondrian_cube(datasource_id, datasource.mondrian_project, cube_name)
            return True

        def flush_schema(self, datasource_id):
            """
            flushes cache for the provided datasource's mondrian schema
            """
            pc = ProjectController(request.Session(), account_id=request.session['user_id'])
            return pc.flush_schema(datasource_id)

        def clear_cache(self, datasource_id):
            """
            flushes cache for the provided datasource's mondrian schema
            """
            pc = ProjectController(request.Session(), account_id=request.session['user_id'])
            pc.clear_cache(datasource_id)

    dispatcher = JsonRpcDispatcher()
    dispatcher.register_instance(Handler())
    return dispatcher.handle_request(request)

def datasource_validation_rpc(request):
    """
    Form behaviour for the invoice creation stuff.
    """
    from basesoft.web.core.input.validation import plugins
    from basesoft.web.core.forms.handler import FormValidator
    form_handler = FormValidator({
        'name': [(plugins.Required, {}), (plugins.Alphanumeric, {})],
        'protocol': [(plugins.Required, {})],
        'host': [(plugins.Required, {})],
        'port': [(plugins.Required, {})],
        'database': [(plugins.Required, {})],
        'user': [(plugins.Required, {})],
        'password': [(plugins.Required, {})]
    })
    dispatcher = JsonRpcDispatcher()
    dispatcher.register_instance(form_handler)
    return dispatcher.handle_request(request)


def xmla(request):

    lc = LicenseController(Session(), account_id=request.session['user_id'])
    lc.assert_permissions('use_raw_mdx')

    _vars = {
        'js_requires': 'xmla_init',
        'css_requires': 'iqube_cube_table_scss'
    }

    return render_to_response('xmla.html', _vars, context_instance=RequestContext(request))


def get_iqube_role(request, datasource_id, account_id):
    """
    view called by iqube to a json string with role information for the given datasource_id and account_id
    """

    # only iqube can connect to this view which is on the LAN
    x_forwarded_for = request.META.get('HTTP_X_FORWARDED_FOR')
    if x_forwarded_for:
        ip = x_forwarded_for.split(',')[0]
    else:
        ip = request.META.get('REMOTE_ADDR')

    if not re.match('^(127\.|192\.168\.|10\.)', ip):
        raise SecurityException('ip not allowed: %s' % ip)

    dsc = DataSourceController(request.Session(), account_id=None)
    role_list = dsc.get_iqube_role_json(datasource_id, account_id)

    return HttpResponse(simplejson.dumps({'roles': role_list}))



