# Copyright (c) 2014 BaseSoft B.V.
# See LICENSE for details.

import os
import cStringIO
import csv
import codecs
import re
import datetime

from django.conf import settings
from django.utils import simplejson

from basesoft.web.core.util.controllers import AbstractController, \
    transaction_decorator
from iqportal.iqube.core.controllers import ProjectController
from iqportal.iqube.core.models import DataSource
from basesoft import logging
from iqportal.iqube.optiq.importers import AbstractImporter
from iqportal.iqube.optiq.exceptions import OptiqImportValueError
from django.utils.translation import ugettext as _

if not os.path.exists(settings.WRITABLE_DIRECTORY):
    os.mkdir(settings.WRITABLE_DIRECTORY)

OPTIQ_WRITABLE = os.path.join(settings.WRITABLE_DIRECTORY, 'optiq')

if not os.path.exists(OPTIQ_WRITABLE):
    os.mkdir(OPTIQ_WRITABLE)

logger = logging.getLogger("iqportal.iqube.optiq")

# http://dev.mysql.com/doc/refman/5.0/en/numeric-types.html#integer-types
INT_TYPES = (
    # small int types are disabled for type checking.. see #3479
    #('TINYINT', 1, ),
    #('SMALLINT', 2, ),
    ('INTEGER', 4, ),
    ('BIGINT', 8, ),
)


def make_valid_name(x):
    """
    make sure luciddb wont error if any of the following table/column names is used
    """
    name = re.sub(r'[^\w|\d|_]', '_', x.strip()).upper()

    # cant start with number
    if re.match(r'^\d+', name):
        name = u't_' + name

    # TODO: prevent ending with __FK (time dimension)

    return name


class AbstractOptiqController(AbstractController):

    def get_model_json_path(self, datasource_id):
        ds_path = os.path.join(OPTIQ_WRITABLE, str(datasource_id))
        if not os.path.exists(ds_path):
            os.mkdir(ds_path)
        filename = os.path.join(ds_path, 'model.json')
        return filename

    @transaction_decorator
    def create_datasource(self, name):

        def _write_model_json(datasource):
            """
            callback executed after the ds is saved to the db and before mondrian is called
            """

            filename = self.get_model_json_path(datasource.id)

            contents = self._get_model_json(datasource.id)
            with open(filename, 'w') as f:
                f.write(contents)
            logger.info('saved model.json to %s' % filename)

            return filename

        if not os.path.exists(OPTIQ_WRITABLE):
            os.mkdir(OPTIQ_WRITABLE)

        pc = ProjectController(self.sa_session, account_id=self.get_account().id, request=self.request)
        datasource = pc.save_or_update_datasource(None, name, 'optiq', name,
                                            protocol='optiq',
                                            user='admin',
                                            password='admin',
                                            host='',
                                            port='',
                                            database=_write_model_json,
                                            ds_type=DataSource.DATASOURCE_TYPE_OPTIQ)

        self._init_time_dimension(datasource)

        return datasource


    def _get_model_json(self, datasource_id):
        """
        must be implemented by child classes to return an optiq model.json
        """
        raise Exception('abstract')

    def _init_time_dimension(self, datasource):
        """
        must be implemented by child classes to create a time dimension table
        """
        raise Exception('abstract')


    def get_metadata(self, datasource_id):
        """
        data types ints: http://alvinalexander.com/java/edu/pj/jdbc/recipes/ResultSet-ColumnType.shtml
        """

        pc = ProjectController(self.sa_session, account_id=self.get_account().id, request=self.request)
        datasource = pc.get_datasource(datasource_id) # for permission check

        filename = self.get_model_json_path(datasource_id)
        return pc.iqube_controller.get_optiq_metadata(filename)


    def get_tables_state(self, datasource_id):
        metadata = self.get_metadata(datasource_id)

        rv = []
        cur_table = None
        cur_table_columns = None

        def _add_columns(table_name, table_columns):
            rv.append({
                'name': table_name,
                'columns': table_columns
            })

        for row in metadata:
            if row['TABLE_NAME'] != cur_table:
                if cur_table_columns is not None:
                    _add_columns(cur_table, cur_table_columns)
                cur_table = row['TABLE_NAME']
                cur_table_columns = []
            cur_table_columns.append(row)

        _add_columns(cur_table, cur_table_columns)

        return rv



class OptiqCSVController(AbstractOptiqController):

    def _get_model_json(self, datasource_id):
        """
        returns json
        """

        ds_path = os.path.join(OPTIQ_WRITABLE, str(datasource_id), 'csv')

        return simplejson.dumps({
            'version': '1.0',
            'defaultSchema': 'default',
            'schemas': [{
                'name': 'default',
                'type': 'custom',
                'factory': 'net.hydromatic.optiq.impl.csv.CsvSchemaFactory',
                'operand': {
                    'directory': ds_path
                }
            }]
        })


    def save_csv(self, datasource, table_name, content):
        """
        saves content for the given table_name. overwrites if already exists
        """

        f = self._open_csv_file(datasource, table_name)
        f.write(content)
        f.close()

    def _open_csv_file(self, datasource, table_name):

        pc = ProjectController(self.sa_session, account_id=self.get_account().id, request=self.request)
        datasource = pc.get_datasource(datasource.id) # for permission check

        ds_path = os.path.join(OPTIQ_WRITABLE, str(datasource.id))
        assert os.path.exists(ds_path)

        csv_path = os.path.join(ds_path, 'csv')

        if not os.path.exists(csv_path):
            os.mkdir(csv_path)

        table_path = os.path.join(csv_path, table_name + '.csv')

        logger.info("_open_csv_file: %s" % table_path)
        return open(table_path, 'w')


    type_map = {
        'VARCHAR': 'string',
        'INTEGER': 'int',
        'BIGINT': 'long',
        'DECIMAL': 'double',
        'DATE': 'date',
        'TIME': 'timestamp',
    }

    def do_import(self, datasource, table_name, importer, columns):
        assert isinstance(datasource, DataSource)
        assert isinstance(importer, AbstractImporter)
        assert isinstance(columns, list)
        assert isinstance(table_name, unicode)

        logger.info('OptiqCSVController: do_import: %s, %s' % (datasource, table_name))

        # returns a file pointer ready for writing
        self._csv_file = self._open_csv_file(datasource, table_name)
        self._csv_queue = cStringIO.StringIO()
        self._csv_encoder = codecs.getincrementalencoder('utf-8')()
        self._csv_writer = csv.writer(self._csv_queue, delimiter=",", quotechar='"', quoting=csv.QUOTE_NONNUMERIC)

        # write header
        self._write_csv_row(['%s:%s' % (c['COLUMN_NAME'], self.type_map[c['DATA_TYPE']]) for c in columns])

        first_row = True
        for row in importer._read_row():
            if first_row:
                first_row = False
                continue

            self._insert_row(row, columns)

        self._csv_file.close()

    def _write_csv_row(self, row):

        self._csv_writer.writerow(['' if s is None else unicode(s).encode("utf-8") for s in row])

        data = self._csv_queue.getvalue()
        data = data.decode("utf-8")
        data = self._csv_encoder.encode(data)
        self._csv_file.write(data)
        self._csv_queue.truncate(0)



    def _insert_row(self, values, columns):
        """
        """

        int_types_dict = dict(INT_TYPES)

        # these columns should convert a '' value to None
        empty_to_none = dict(INT_TYPES).keys() + ['DECIMAL', '']

        value_i = -1
        while True:
            value_i += 1

            if value_i > len(columns)-1:
                break

            column = columns[value_i]
            value = values[value_i]

            if column['DATA_TYPE'] in empty_to_none and type(value) in (str, unicode) and value.strip() == '':
                values[value_i] = None
                continue

            try:

                # Check for time data
                # this is the case if the current destination column has a time column too
                time_column = column['COLUMN_NAME'] + '__TIME'

                #if self._table_columns.has_key(time_column):
                if column['DATA_TYPE'] == 'TIME':

                    # FIXME: explain time FK

                    # assert the next column is a __TIME
                    assert columns[value_i+1]['COLUMN_NAME'] == column['COLUMN_NAME'] + '__TIME', columns[value_i+1]['COLUMN_NAME']

                    # the varchar column just gets the current value as string
                    values[value_i] = unicode(value)

                    # convert time data to seconds..

                    # empty values are 0 seconds
                    if unicode(value).strip() == '':
                        #values[value_i + 1] = 0
                        value_i += 1
                        values.insert(value_i, 0)
                        continue

                    d1 = None

                    # time only notation
                    if re.match(r'^(\d{2}):(\d{2}):(\d{2})$', unicode(value)):
                        d1 = datetime.datetime.strptime(value, "%H:%M:%S")

                    # full date time notation
                    elif re.match(r'^(\d{4})-(\d{2})-(\d{2}) (\d{2}):(\d{2}):(\d{2})$', unicode(value)):
                        d1 = datetime.datetime.strptime(value, "%Y-%m-%d %H:%M:%S")
                    else:
                        raise OptiqImportValueError(_('Invalid time format: %(value)s for column %(column)s' % {'value': value, 'column': column['COLUMN_NAME']} ))


                    # time data is based on 1900. calculate the difference to get to seconds
                    d2 = datetime.datetime(1900, 1, 1)
                    delta = (d1-d2)
                    seconds = (delta.days * (24 * 3600)) + delta.seconds
                    #values[value] = seconds

                    # shift the column index and set value
                    value_i += 1
                    values.insert(value_i, seconds)

                # integer columns
                elif column['DATA_TYPE'] in int_types_dict.keys():

                    # first convert to float then to int because if the value is "5265.0" (unicode) then
                    # casting to int directly raises exception. this is the case for xlsx files that always gives int values as float
                    values[value_i] = int(float(value))

                    # check integer in range
                    max_int_v = float(pow(2, 8 *  int_types_dict[column['DATA_TYPE']] ) / 2)
                    if not (values[value_i] >= (max_int_v * -1) and values[value_i] < max_int_v):
                        raise OptiqImportValueError(_('Integer out of range: %(value)s for column %(column)s' % {'value': values[value_i], 'column': column['COLUMN_NAME']} ))

                elif column['DATA_TYPE'] == 'DECIMAL':

                    # check if convert to float goes ok. this is also required for sql insert
                    values[value_i] = float(value)

                    # check if dec_digits and precision matches column specs
                    max_a_len = column['PRECISION'] - column['DEC_DIGITS']
                    max_digits = column['DEC_DIGITS']
                    a_len, s_len = [len(x) for x in unicode(values[value_i]).replace('-', '').split('.')]
                    if a_len > max_a_len or s_len > max_digits:
                        raise OptiqImportValueError(_('Decimal out of range: %(value)s for column %(column)s' % {'value': values[value_i], 'column': column['COLUMN_NAME']} ))

                elif column['DATA_TYPE'] == 'VARCHAR':

                    if len(values[value_i]) > column['PRECISION']:
                        raise OptiqImportValueError(_('Varchar longer than %(precision)s characters: "%(value)s" for column "%(column)s"' % {'precision': column['PRECISION'], 'value': values[value_i], 'column': column['COLUMN_NAME']} ))

                elif column['DATA_TYPE'] == 'DATE':

                    # assert the next column is a date __FK
                    assert columns[value_i+1]['COLUMN_NAME'] == column['COLUMN_NAME'] + '__FK'

                    # if no date is supplied dont bother
                    if unicode(value).strip() == '':
                        values[value_i] = None
                        #values[value_i + 1] = 0

                        value_i += 1
                        values.insert(value_i, 0)

                        continue

                    # values arrive as string, we support two date formats
                    try:
                        dt = datetime.datetime.strptime(values[value_i], "%Y-%m-%d 00:00:00")
                    except ValueError:
                        dt = datetime.datetime.strptime(values[value_i], "%Y-%m-%d")

                    # create a native date type and set that as value
                    d = datetime.datetime.date(dt)
                    values[value_i] = d

                    # insert into the shadow datetime column as an integer for mondrian time dimension
                    #values[column_name + '__FK'] = d.strftime('%Y%m%d')

                    value_i += 1
                    values.insert(value_i, d.strftime('%Y%m%d'))


                else:
                    raise Exception('unknown column type: %s' % column['DATA_TYPE'])

            except ValueError:
                raise OptiqImportValueError(_('Error converting value "%(value)s" for "%(column)s" to "%(datatype)s"' % {'value': value, 'column': column['COLUMN_NAME'], 'datatype': column['DATA_TYPE']} ))


        self._write_csv_row(values)


    def _init_time_dimension(self, datasource):

        csv_file = self._open_csv_file(datasource, 'date_dimension')
        csv_writer = csv.writer(csv_file, delimiter=',', quotechar='"', quoting=csv.QUOTE_MINIMAL)

        csv_writer.writerow([
            'ID:int',
            'CDATE:date',
            'CYEAR:int',
            'QUARTER:int',
            'CMONTH:int',
            'WEEK_OF_YEAR:int',
            'DAY_OF_MONTH:int',
        ])


        begin_date = datetime.date(1970,1,1)
        end_date = datetime.date(2050,12,31)
        d = end_date-begin_date
        d = d.days

        s = 0
        while s <= d:
            mydate = begin_date+datetime.timedelta(days=s)
            id_ = mydate.strftime('%Y%m%d')

            csv_writer.writerow([
                int(id_),
                 mydate,
                 mydate.year,# year
                 (mydate.month-1)//3+1, # quarter
                 mydate.month, # month
                 mydate.isocalendar()[1], # week of year
                 mydate.day,
            ])

            s = s + 1

        csv_file.close()





