# Copyright (c) 2014 BaseSoft B.V.
# See LICENSE for details.

import re
import os

from basesoft.util.ordered_dict import OrderedDict
from basesoft.logging import getLogger
from django.utils.translation import ugettext as _
from xlrd.xldate import xldate_as_tuple
from xlrd.sheet import Cell, ctype_text
from basesoft.web.contrib.file.models import FileTmp
from basesoft.web.core.exceptions import PublicUserException

from django.conf import settings
from iqportal.iqube.optiq.exceptions import OptiqInvalidFile,\
    OptiqImportValueError
import itertools

logger = getLogger('iqportal.iqube.optiq')

import csv, codecs

import datetime


class AbstractImporter(object):

    _row_i = None

    @staticmethod
    def get_importer(tmp_file):
        """
        static method to return the right importer for the given file
        """

        assert isinstance(tmp_file, FileTmp), "type: %s" % (type(tmp_file))

        file_path = os.path.join(settings.TMP_FILE_REPOSITORY, tmp_file.file_hash)
        assert os.path.exists(file_path), 'file not found: %s' % file_path

        base_filename, file_extension = os.path.splitext(tmp_file.file_name)

        importer = None
        if file_extension in ('.csv', '.txt'):
            importer = CsvImporter(file_path)

        elif file_extension == '.xlsx':
            importer = XlsxImporter(file_path)

        elif file_extension == '.xls':
            importer = XlsImporter(file_path)

        else:
            raise PublicUserException(_('Cannot import for extension: %s') % file_extension)

        return importer


    def __init__(self, file_path):
        self._file_path = file_path

    def get_file_columns(self, file_path, import_options=None):
        """
        should return a columns list based on given filename
        """
        raise Exception('abstract')

    def _guess_datatype(self, values):
        """
        guesses a datatype from a list of values
        """

        from iqportal.iqube.optiq.controllers import INT_TYPES


        is_nullable = True

        # test int values we check for float because excel files deliver all
        # values as floats. this works well for real ints, varchars and floats representing ints
        def test_int(value):
            if float(value) % 1.0 != 0:
                raise ValueError

        def test_date(value):

            # for speedup do a quick regex first
            if not re.match(r'^(\d{4})-(\d{2})-(\d{2})$', value) and \
                not re.match(r'^(\d{4})-(\d{2})-(\d{2}) (\d{2}):(\d{2}):(\d{2})$', unicode(value)):
                raise ValueError

            # first check if its a valid date short format
            try:
                datetime.datetime.strptime(value, "%Y-%m-%d")
                return
            except ValueError:
                pass

            # else try full datetime format with 0 time
            datetime.datetime.strptime(value, "%Y-%m-%d 00:00:00")

        def test_time(value):

            # quick test
            if not re.match(r'^(\d{2}):(\d{2}):(\d{2})$', value) and \
                not re.match(r'^(\d{4})-(\d{2})-(\d{2}) (\d{2}):(\d{2}):(\d{2})$', unicode(value)):
                raise ValueError

            # first try HH:MM:SS notation
            try:
                date = datetime.datetime.strptime(value, "%H:%M:%S")
            except ValueError:

                # try a full time notation but only if the year is 1900
                #
                # in excel there is no difference between date and time so they are
                # delivered as a full datetime notification. Detect a time column
                # when the year is 1900
                date = datetime.datetime.strptime(value, "%Y-%m-%d %H:%M:%S")
                if date.year > 1900:
                    raise ValueError


        tests = [
            ('INTEGER', test_int ),
            ('DECIMAL', float),
            ('TIME', test_time),
            ('DATE', test_date),
            ('VARCHAR', lambda value: True)
        ]

        datatype = None
        for test_datatype, test_func in tests:

            # stop testing if datatype is found
            if datatype is not None:
                continue

            # all_good can either be None (unknown),
            # False (value is not current test type) or True (so far, so good)
            all_good = None

            values, values1 = itertools.tee(values)
            for v in values1:

                # stop testing current type if previous test resulted negative
                if all_good == False:
                    continue

                # always ignore empty values for type detection
                if v.strip() == '':
                    continue

                # execute the test function. if it didnt error then value must
                # be the current datatype
                try:
                    test_func(v)
                    all_good = True
                except (ValueError, TypeError) as e:
                    all_good = False

            # if all values have tested ok then set the current datatype
            if all_good:
                datatype = test_datatype

        # datatype is None when all values are ''
        # then lets just make it a 'VARCHAR
        if datatype is None:
            datatype = 'VARCHAR'

        # now that we know the datatype, check precision and dec_digits
        precision = None
        dec_digits = None

        if datatype == 'VARCHAR':
            precision = 1

            values, values1 = itertools.tee(values)
            for v in values1:
                v = unicode(v)
                if len(v) > precision:
                    # give precision 25% margin
                    precision = len(v) + (len(v) / 4)

        elif datatype in ('INTEGER'):

            # checks for the min and max values in the list
            # and depending on that chooses the best integer type

            datatype = None
            values, values1 = itertools.tee(values)
            values1 = [float(v) for v in values1]

            if (len(values1) == 0):
                values1 = [0]

            min_v = min(values1)
            max_v = max(values1)

            for int_type in INT_TYPES:

                # compare using floats cuz min() / max() return floats for big values
                max_int_v = float(pow(2, 8 * int_type[1]) / 2)

                if (min_v >= max_int_v * -1) and (max_v <= (max_int_v-1)):
                    datatype = int_type[0]
                    break

            # for now error if the integer is too big, else we should look at aprox datatypes
            if datatype is None:
                raise OptiqImportValueError(_('Integer out of range: %(min_v)s / %(max_v)s' % (min_v, max_v)))


        elif datatype in ('DECIMAL'):

            # Decimal type has to specify a precision and dec_digits.
            # Loop all the values and keep track of max the max digits before and after the dot.
            # The precision is the sum of both.

            abs_digits = 0
            dec_digits = 0

            values, values1 = itertools.tee(values)
            for v in values1:
                v = float(v)
                a_len, s_len = [len(x) for x in unicode(v).replace('-', '').split('.')]
                if a_len > abs_digits:
                    abs_digits = a_len
                if s_len > dec_digits:
                    dec_digits = s_len

            precision = abs_digits + dec_digits

            # for now hard set max precision. maybe in the future we can introduce a aprox type
            # if the user values are bigger than this he will get an error on import
            if precision > 19:
                precision = 19

        return datatype, precision, dec_digits, is_nullable


class LineImporter(AbstractImporter):
    """
    common class for importers that import line by line with headers on the first line: csv, xls, xlsx etc
    """

    _row_i = None
    _column_mapping = None

    def get_file_columns(self, import_options=None):
        """
        returns a columns list based on given filename
        """

        from iqportal.iqube.optiq.controllers import make_valid_name

        file_path = self._file_path

        if isinstance(file_path, FileTmp):
            file_path = os.path.join(settings.TMP_FILE_REPOSITORY, file_path.file_hash)

        # make a in-memory copy of the complete file
        # this seems to work much faster than using iterators on the imports
        rows = []
        for row in self._read_row():
            rows.append(row)

        # read first row
        row_0 = rows[0]

        # returns an iterator for all values of the given index, skipping first row
        def field_iterator(index):
            first_row = True
            for row in rows:

                if first_row:
                    first_row = False
                    continue

                if row[index].strip() == '':
                    continue

                yield row[index]

        rv = []
        for col_i in range(len(row_0)):
            column_name = row_0[col_i]

            datatype, precision, dec_digits, is_nullable = self._guess_datatype( field_iterator(col_i) )

            column_name = make_valid_name(column_name)
            if column_name == '':
                raise OptiqInvalidFile('Column %s has a blank title' % col_i)

            rv.append({
                'COLUMN_NAME': column_name,
                'DATA_TYPE': datatype,
                'PRECISION': precision,
                'DEC_DIGITS': dec_digits,
                'IS_NULLABLE': is_nullable,
            })

            # create an extra date shadow column for fk time dimension table
            if datatype == 'DATE':
                rv.append({
                    'COLUMN_NAME': column_name + '__FK',
                    'DATA_TYPE': 'INTEGER',
                    'PRECISION': '',
                    'DEC_DIGITS': '',
                    'IS_NULLABLE': is_nullable,
                })

            # create an extra time shadow column
            if datatype == 'TIME':
                rv.append({
                    'COLUMN_NAME': column_name + '__TIME',
                    'DATA_TYPE': 'INTEGER',
                    'PRECISION': '',
                    'DEC_DIGITS': '',
                    'IS_NULLABLE': is_nullable,
                })

        # check for duplicate columns
        c_list = [y['COLUMN_NAME'] for y in rv]
        for c in c_list:
            if c_list.count(c) > 1:
                raise OptiqInvalidFile(_('Duplicate colum: %s') % c)

        return rv

class CsvImporter(LineImporter):
    """
    LineImporter for CSV
    """

    class UTF8Recoder:
        def __init__(self, f, encoding):
            self.reader = codecs.getreader(encoding)(f)

        def __iter__(self):
            return self

        def next(self):
            return self.reader.next().encode("utf-8")

    class UnicodeCsvReader:

        def __init__(self, f, dialect=csv.excel, encoding="utf-8", **kwds):
            f = CsvImporter.UTF8Recoder(f, encoding)
            self.reader = csv.reader(f, dialect=dialect, **kwds)

        def next(self):
            row = self.reader.next()
            return [unicode(s, "utf-8") for s in row]

        def __iter__(self):
            return self

    def _read_row(self):

        csvfile = open(self._file_path, "rb")

        # sniff 10MB. it goes wrong if a value has a comma in it which can be way up ahead the file
        dialect = csv.Sniffer().sniff(csvfile.read(1024 * 1024 * 10))
        csvfile.seek(0)

        reader = CsvImporter.UnicodeCsvReader(csvfile, dialect=dialect)

        for row in reader:
            yield row

class XlsxImporter(LineImporter):
    """
    Importer for XLSX
    """

    def _read_row(self):

        from openpyxl.reader.excel import load_workbook

        wb = load_workbook(filename = self._file_path, use_iterators = True)
        ws = wb.get_active_sheet()

        # even though excel does provide some kind of value type information,
        # we still convert all values to unicode because
        #  * integer types are passed as floats which is basically incorrect
        #  * detection of valid types is happens anyway before sending to the server
        #  * this is inline with csv importer (no differences)

        for row in ws.iter_rows():
            values = []

            for cell in row:

                value = cell.internal_value
                if  value is None:
                    value = ''
                elif isinstance(value, datetime.datetime):

                    # its wierd but for 'time' data we need to add 1 day
                    if value.year == 1900 and value.month < 3:
                        value = value + datetime.timedelta(days=1)

                values.append(unicode(value))

            yield values


class XlsImporter(LineImporter):
    """
    Importer for XLS
    """


    def _read_row(self):

        # xls file is in old format
        # http://scienceoss.com/readexcelfilesfrompython/

        import xlrd
        wb = xlrd.open_workbook(self._file_path)
        sh = wb.sheet_by_index(0)

        for rownum in range(sh.nrows):

            values = sh.row_values(rownum)
            newrow = []
            for colnum in range(len(values)):

                c = None
                try:
                    c = sh.row(rownum)[colnum]
                except IndexError:
                    newrow.append('')

                if c is not None:

                    # code below converts an xls datetime to a python datetime object
                    # see docs of xldate_as_tuple() for wierdness
                    if ctype_text[c.ctype] == 'xldate':

                        value = c.value
                        days = int(value)

                        count_days = False
                        if days < 61:
                            count_days = True
                            value = value - days

                        t = xldate_as_tuple(value, wb.datemode)

                        year = 1900
                        month = 1
                        day = 1

                        if t[0] != 0:
                            year = t[0]
                            month = t[1]
                            day = t[2]

                        d1 = datetime.datetime(year, month, day, t[3], t[4], t[5])

                        if count_days:
                            d1 = d1 + datetime.timedelta(days=days)

                        newrow.append(d1.strftime("%Y-%m-%d %H:%M:%S")) # '1900-01-01 12:10:15' or '2013-12-13 00:00:00'

                    else:
                        newrow.append(unicode(c.value))

            yield newrow

