# Copyright (c) 2010 BaseSoft B.V.
# See LICENSE for details.

import time

import httplib
import socket
import hashlib
import os

from lxml import etree, sax

from django.conf import settings
from iqportal.iqube.xmla.handler import DiscoverHandler, ExecuteHandler

from basesoft import logging
from basesoft.web.core.exceptions import PublicUserException
logger = logging.getLogger("iqportal.iqube.xmla")

from xml.sax.saxutils import escape

def get_client(url, port, path=None, project=None, client_module=None, role_name=None, account_id=None):
    if client_module is None:
        return XMLAClient(url, port, path, project, role_name, account_id)

    from twisted.python.reflect import namedAny
    try:
        module = namedAny(client_module)
    except AttributeError, e:
        return XMLAClient(url, port, path, project, role_name, account_id)

    if not hasattr(module, 'XMLAClient'):
        return XMLAClient(url, port, path, project, role_name, account_id)

    return module.XMLAClient(url, port, path, project, role_name, account_id)

class XMLACacheController:

    # if force from cache is True we NEED to load from cache else error.
    force_from_cache = False

    def __init__(self, context, client):
        logger.debug("%s:__init__(): cache controller available" % self)
        self._client = client
        self.set_context(**context)

    def set_context(self, project, role):
        self.role_name = role
        self.project = project
        self.set_cache_path()

    def set_cache_path(self, *subdirs):
        self._cache_path = os.path.join(settings.CACHE_DIR, 'xmla', self.project, *subdirs)

    def load(self, message):
        xmla_response = None
        # we don't have caching enabled
        if not settings.MONDRIAN_XMLA_CACHE_ENABLED:
            return xmla_response

        # cache based on the message content and role_name
        role_name = self.role_name
        if role_name is None:
            role_name = ''
        md5_content = message + role_name

        message_md5 = hashlib.md5(md5_content.encode('utf-8')).hexdigest()
        self._hash_path = message_md5[:2]
        self._cache_file = os.path.join(self._cache_path, self._hash_path, message_md5)

        if os.path.exists(self._cache_file):
            logger.debug("%s:load(): loading xmla response from cache: %s" % (self, self._cache_file))
            xmla_response = open(self._cache_file).read()
        elif self.force_from_cache:
            logger.warn("%s:load(): force loading from cache failed: %s" % (self, self._cache_file))
            raise PublicUserException('Cache currently not available, unable to load data. Try again later.')

        return xmla_response

    def save(self, xmla_response):
        # we dont have caching enabled
        if not settings.MONDRIAN_XMLA_CACHE_ENABLED:
            return

        if xmla_response.find('<SOAP-ENV:Fault>') != -1:
            logger.warn("%s:save(): not caching faulty response" % self)
            return

        path = os.path.join(self._cache_path, self._hash_path)
        if not os.path.exists(path):
            try:
                os.makedirs(path)
            except OSError, e:
                print 'failed to mkdir: %s' % e.message

        if not os.path.exists(self._cache_file):
            f = open(self._cache_file, 'w' )
            f.write(xmla_response)
            f.close()
            logger.debug("%s:save(): wrote xmla response to cache: %s" % (self, self._cache_file))

    def __repr__(self):
        return '<XMLACacheController>'

class XMLAClient:
    """
    Python based XMLA client. For more info see: http://xmla.org

    TODO: build the XML for the calls properly with an object instead of strings... #3246
    """
    provider = 'Mondrian'
    _xmla_cache_controller = XMLACacheController

    # Bitwise constants used for the TREE_OP attribute in the MDSCHEMA_MEMBERS call.
    MDTREEOP_CHILDREN = 1
    MDTREEOP_SIBLINGS = 2
    MDTREEOP_PARENT = 4
    MDTREEOP_SELF = 8
    MDTREEOP_DESCENDANTS = 16
    MDTREEOP_ANCESTORS = 32

    MD_DIMTYPE_UNKNOWN = 0
    MD_DIMTYPE_TIME = 1
    MD_DIMTYPE_MEASURE = 2
    MD_DIMTYPE_OTHER = 3


    def __init__(self, url, port, path=None, project=None, role_name=None, account_id=None):
        self.webservice = httplib.HTTP(url, port)
        self.set_path(path)
        self.set_project(project)
        self.set_mdx_builder()
        self.set_role_name(role_name)
        self.account_id = account_id
        self.set_context()

    def set_mdx_builder(self, builder=None):
        from iqportal.iqube.xmla.mdx import MdxBuilder
        if builder is None:
            self._mdx_builder = MdxBuilder
            return
        self._mdx_builder = builder

    def set_context(self, context=None):
        # can be used to store per xmla cache per context (publication of reports, etc)
        if context is None:
            context = {}
        self._context = context
        self._context['project'] = self.project
        self._context['role'] = self.role_name
        self.cache_controller = self._xmla_cache_controller(self._context, self)

    def set_path(self, path):
        """
        The path where the webservice is posting to.
        """
        self.path = path

    def set_project(self, project):
        """
        The project we are working with.
        """
        self.project = project

    def set_role_name(self, role_name=None):
        self.role_name = role_name

    def _send(self, message, handler, raw=False):
        """
        This creates a SOAP call to XMLA provider.
        """

        # TODO: if we ever need realtime analasis (< 1 minute) then we have to check
        # the datasource cache expiry for '0' and then dont read/write to cache #3247

        # xmla caching: return result directly if found
        xmla_response = self.cache_controller.load(message)

        # no response available, send request to mondrian
        if xmla_response is None:

            message = u"""<?xml version="1.0" encoding="UTF-8"?>
            <SOAP-ENV:Envelope SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/">
                <SOAP-ENV:Body>
                    %s
                </SOAP-ENV:Body>
            </SOAP-ENV:Envelope>
            """ % message

            msg_utf8 = message.encode('utf-8')

            self.webservice.putrequest("POST", self.path)
            self.webservice.putheader("User-Agent", "iqportal.iqube.xmla.client")
            self.webservice.putheader("Content-type", "text/xml; charset=\"UTF-8\"")
            self.webservice.putheader("Content-length", "%d" % len(msg_utf8))

            if self.role_name is not None:
                self.webservice.putheader("Mondrian-rolename", self.role_name)

            self.webservice.endheaders()

            # TODO: notify when query takes more than x seconds #3248
            time_a = time.time()


            self.webservice.send(msg_utf8)
            status, reason, msg = self.webservice.getreply()
            if status in range(400, 500): # we do not accept 400 status codes
                logger.error("Unexpected IQube response: %s, %s, %s" % (status, reason, msg))
                raise Exception('Unexpected IQube result status: %s' % status)
            time_b = time.time()
            logger.debug('mondrian query took %s seconds' % (time_b - time_a) )

            xmla_response = self.webservice.getfile().read()

            # xmla caching: write result to file
            self.cache_controller.save(xmla_response)

        if raw:
            return xmla_response

        try:
            tree = etree.fromstring(xmla_response)
        except Exception, e:
            logger.error("Failed to parse response: %s" % xmla_response)
            raise e

        # instanciate the provided handler to parse the response.
        h = handler()
        sax.saxify(tree, h)

        result = h.get_result()

        return result

    def _discover(self, request_type, properties=None, restrictions=None):
        """
        Discover request
        """
        props = u'<DataSourceInfo>basesoft</DataSourceInfo><Catalog>%s</Catalog>\n' % (self.project)
        if properties is not None:
            for prop, value in properties.iteritems():
                props = u'%s<%s>%s</%s>\n' % (props, prop, escape(value), prop)

        restrict = ''
        if restrictions is not None:
            for prop, value in restrictions.iteritems():

                if not isinstance(value, unicode):
                    value = unicode(value)

                restrict = u'%s<%s>%s</%s>\n' % (restrict, prop, escape(value), prop)

        message = u"""
        <Discover xmlns="urn:schemas-microsoft-com:xml-analysis">
            <RequestType>%s</RequestType>
            <Restrictions>
                <RestrictionList>
                %s
                </RestrictionList>
            </Restrictions>
            <Properties>
                <PropertyList>
                %s
                </PropertyList>
            </Properties>
        </Discover>
        """ % (request_type, restrict, props)

        return self._send(message, DiscoverHandler)

    def test_connection(self):
        try:
            self.get_datasources()
            return True
        except socket.error, e:
            print 'Exception: %s' % e.message
            return False

    def get_datasources(self):
        """
        Returns a list with all datasources defined in the Mondrian application.
        """
        return self._discover('DISCOVER_DATASOURCES')

    def get_cubes(self):
        """
        Get all the cubes (normal and virtual) for the current project.
        """
        return self._discover('MDSCHEMA_CUBES', restrictions={
            'CATALOG_NAME': self.project
        })

    def get_dimensions(self, cube_name):
        """
        Get all the dimensions for the provided cube
        """
        return self._discover('MDSCHEMA_DIMENSIONS', restrictions={
            'CATALOG_NAME': self.project,
            'CUBE_NAME': cube_name
        })

    def get_dimensions_by_type(self, cube_name, dimension_type):
        """
        Get a specific dimension by type.
        Currently Mondrian does not support retrieval of a single dimension type
        so we use the get_dimensions method and we filter the response.
        This needs to be change when Mondrian supports restrictions on DIMENSION_TYPE.
        """
        return [d for d in self.get_dimensions(cube_name) if int(d['dimension_type']) == dimension_type]

    def get_dimension_info(self, cube_name, dimension_name):
        """
        Get all the information for the provided dimension
        """
        return self._discover('MDSCHEMA_DIMENSIONS', restrictions={
            'CATALOG_NAME': self.project,
            'CUBE_NAME': cube_name,
            'DIMENSION_NAME': dimension_name
        })

    def get_hierarchies(self, cube_name):
        """
        Get all the hierarchies defined in the cube.
        """
        return self._discover('MDSCHEMA_HIERARCHIES', restrictions={
            'CATALOG_NAME': self.project,
            'CUBE_NAME': cube_name
        })

    def get_dimension_hierarchies(self, cube_name, dimension_unique_name):
        """
        Get all the hierarchies in the cube for the provided dimension.
        """
        return self._discover('MDSCHEMA_HIERARCHIES', restrictions={
            'CATALOG_NAME': self.project,
            'CUBE_NAME': cube_name,
            'DIMENSION_UNIQUE_NAME': dimension_unique_name
        })

    def get_dimension_levels(self, cube_name, dimension_unique_name):
        """
        Get all levels within the provided dimension.
        """
        return self._discover('MDSCHEMA_LEVELS', restrictions={
            'CATALOG_NAME': self.project,
            'CUBE_NAME': cube_name,
            'DIMENSION_UNIQUE_NAME': dimension_unique_name
        })

    def get_hierarchy_levels(self, cube_name, hierarchy_unique_name):
        """
        Get all levels within the provided dimension.
        """
        return self._discover('MDSCHEMA_LEVELS', restrictions={
            'CATALOG_NAME': self.project,
            'CUBE_NAME': cube_name,
            'HIERARCHY_UNIQUE_NAME': hierarchy_unique_name
        })

    def get_level_info(self, cube_name, level_unique_name):
        """
        Get all the information for the provided level
        """
        return self._discover('MDSCHEMA_LEVELS', restrictions={
            'CATALOG_NAME': self.project,
            'CUBE_NAME': cube_name,
            'LEVEL_UNIQUE_NAME': level_unique_name
        })

    def get_measures(self, cube_name):
        """
        Get all measures for the provided cube.
        """
        return self._discover('MDSCHEMA_MEASURES', restrictions={
            'CATALOG_NAME': self.project,
            'CUBE_NAME': cube_name
        })

    def get_calculated_members(self, cube_name, dimension_unique_name=None):
        members = self._discover('MDSCHEMA_MEMBERS', restrictions={
            'CATALOG_NAME': self.project,
            'CUBE_NAME': cube_name,
            'LEVEL_UNIQUE_NAME': '[Measures].[MeasuresLevel]'
        })
        return [i for i in members if dimension_unique_name is not None and i['dimension_unique_name']=='[%s]' % dimension_unique_name]

    def get_named_sets(self, cube_name):
        return self._discover('MDSCHEMA_SETS', restrictions={
            'CATALOG_NAME': self.project,
            'CUBE_NAME': cube_name
        })

    def get_level_members(self, cube_name, level_unique_name):
        """
        Get all the members within the provided level.
        """
        return self._discover('MDSCHEMA_MEMBERS', restrictions={
            'CATALOG_NAME': self.project,
            'CUBE_NAME': cube_name,
            'LEVEL_UNIQUE_NAME': level_unique_name
        })

    def get_hierarchy_members(self, cube_name, hierarchy_unique_name):
        """
        Get all the members within the provided hierarchy.
        """
        return self._discover('MDSCHEMA_MEMBERS', restrictions={
            'CATALOG_NAME': self.project,
            'CUBE_NAME': cube_name,
            'HIERARCHY_UNIQUE_NAME': hierarchy_unique_name
        })

    def get_member_tree_op(self, cube_name, unique_name, tree_op, unique_name_handle='MEMBER_UNIQUE_NAME'):
        """
        Get all the members with the provided bitwise tree_op
        """

        return self._discover('MDSCHEMA_MEMBERS', restrictions={
            'CATALOG_NAME': self.project,
            'CUBE_NAME': cube_name,
            'TREE_OP': tree_op,
            unique_name_handle: unique_name
        })

    def _execute(self, mdx, **kwargs):
        """
        Execute request, used for MDX Queries only
        """
        props = '<DataSourceInfo>basesoft</DataSourceInfo>'
        raw = False

        if not kwargs.has_key('Catalog'):
            props = '%s<Catalog>%s</Catalog>\n' % (props, self.project)

        if kwargs.has_key('raw'):
            raw = True
            del kwargs['raw']

        for prop, value in kwargs.iteritems():
            props = '%s<%s>%s</%s>\n' % (props, prop, value, prop)

        message = u"""
        <Execute xmlns="urn:schemas-microsoft-com:xml-analysis">
            <Command>
                <Statement>
                    %s
                </Statement>
            </Command>
            <Properties>
                <PropertyList>
                    %s
                    <Format>Multidimensional</Format>
                    <AxisFormat>TupleFormat</AxisFormat>
                </PropertyList>
            </Properties>
        </Execute>
        """ % (escape(mdx), props)

        return self._send(message, ExecuteHandler, raw)

    def execute_mdx(self, mdx, **kwargs):
        logger.debug('%s: executing mdx: %s' % (self, mdx) )
        return self._execute(mdx, **kwargs)

    def build_select_statement(self, columns, rows, cube, slicers, nonempty, mdx_with=None, is_graph=False, allow_empty_rows=False, hierarchize=True):
        """
        Builds a MDX query from provided member information.
        TODO: should this be here? Maybe cuz of permissions/data filters later on. #3249
        """
        builder = self._mdx_builder(cube, columns, rows, slicers, nonempty, is_graph=is_graph, account_id=self.account_id, hierarchize=hierarchize)
        builder.allow_empty_rows = allow_empty_rows
        builder.set_with(mdx_with)
        mdx = builder.get_result()
        return mdx

    def __repr__(self):
        return '<XMLAClient project=%s>' % self.project
