# Copyright (c) 2008 BaseSoft Solutions B.V.
# See LICENSE for details.

from pyparsing import \
    alphas, alphanums, delimitedList, Group, CaselessLiteral, Combine, \
    Literal, Keyword, Word, Optional, StringEnd, nums, Forward, ZeroOrMore, ParseException

scalar = Word(alphanums+"_"+" "+"-"+")"+"(")

lpar  = Literal( "(" ).suppress()
rpar  = Literal( ")" ).suppress()

leftSqBr = Literal("[").suppress()
rightSqBr = Literal("]").suppress()

point = Literal('.')

def operator_grammar(expr, g):
    """
    This parses any math operator related stuff for example:

        * [Measures].[Unit Sales]/[Measures].[Customer Count]
        * ([Survey].[1].[1]/[Survey].[1].[2])*100
    """

    pusher = g.push_first

    # the pusher method adds the result to a termorary stack.

    plus  = Literal( "+" )
    minus = Literal( "-" )
    mult  = Literal( "*" )
    div   = Literal( "/" )
    addop  = plus | minus
    multop = mult | div
    expop = Literal( "^" )

    measure_def = Group(\
        delimitedList(leftSqBr + scalar + rightSqBr,
                      ".", combine=False).setResultsName("level" ,True)
        )

    e = CaselessLiteral('E')
    number = Word(nums)
    integer = Combine( Optional(addop) + number )
    floatnumber = Combine( integer +
                           Optional( point + Optional(number) ) +
                           Optional( e + integer ))

    atoms = (e | floatnumber | integer | measure_def ).setParseAction(pusher) \
                | (lpar + expr.suppress() + rpar)

    factor = Forward()
    # for calculations we can have multiple operators
    factor << ( atoms ) + ZeroOrMore( ( expop + factor ).setParseAction(pusher) )
    term = factor + ZeroOrMore( ( multop + factor ).setParseAction(pusher) )
    expr << term + ZeroOrMore( ( addop + term ).setParseAction(pusher) )

    return expr

class CalculationGrammar:
    """
    This tokenizes the Calculation statements and adds them to an expression
    stack to be used later in either a predefined Calculated Member or a
    calculated member defined using the WITH-preprocessor
    """
    def __init__(self, func_def):
        """
        Initialize some vars
        """
        self.stack = {}
        self.stack_name = ''
        self.func_def = func_def

    def mdx(self, func):
        """
        This will do the grammar parsing
        """
        asToken = Keyword("as", caseless=True).suppress()

        scalar = Word(alphanums+"_"+" "+"-"+")"+"(")
        quote = Literal('"').suppress()
        apostrophe = Literal("'").suppress()

        self.exprStack = []
        self.funcStack = []
        self.levelStack = []
        # measure: [A].[B].[C]
        measure_def = Group(\
            delimitedList(leftSqBr + scalar + rightSqBr,
                          ".", combine=False)
            ).setResultsName("level" ,True)

        set_def = Group(leftSqBr + scalar + rightSqBr).setResultsName("set", True)

        point = Literal('.')

        set_or_member = measure_def | set_def

        calc = Group(func)
        # [Member].[Name] AS 'calc'
        return Optional(quote) + set_or_member.setParseAction( self.set_stack_name ).setResultsName('name') \
                + Optional(quote) + asToken + Optional(apostrophe) \
                + calc.setParseAction( self.create_stack ).setResultsName('asContent') \
                + Optional(apostrophe)

    def set_stack_name(self, str, loc, toks):
        """
        Set the stack name, but only if it is empty, this is the name of the SET or CALCULATED MEMBER
        """
        if self.stack_name == '':
            self.stack_name = toks[0]

    def add_func(self, str, loc, toks):
        """
        This pushes all the tokens of an MDX method to the front of the stack, as the grammar parses inside out.
        """
        newstack = []
        while len(toks) > 0:
            newstack.append(toks.pop(0))

        self.funcStack = newstack + self.funcStack


    def push_first(self, str, loc, toks):
        """
        Push the next calculation in line on the stack, this can be a member,
        a value, or an operator.
        @param toks: tokens from pyparsing
        """
        if len(toks.level) > 0 and len(toks) == 1:
            # if we are dealing with a level (Member, Set) push the whole match of tokens
            self.exprStack.append(toks)
        else:
            # if anything else we only use the last of the tokens.
            self.exprStack.append(toks.pop(0))

    def push_number(self, str, loc, toks):
        # pushes a numerical value on the stack, this can be a number user for a Count (Example TopCount)
        self.funcStack.append(toks.pop(0))

    def parse_full_method_or_operator(self, str, loc, toks):
        """
        This gets called when we are done matching a complete method or operator calculation
        """
        if len(self.exprStack) == 0:
            # There were no expressions on the stack
            return

        # we push the remaining expressing to the func stack and emtpy it.
        self.funcStack.append(self.exprStack)
        self.exprStack = []

    def create_stack(self, str, loc, toks):
        """
        This gets done one time per calculation
        """
        if len(self.exprStack) > 0 or len(self.funcStack) > 0:
            self.stack[self.stack_name] = (self.exprStack,self.funcStack)
        self.exprStack = []
        self.funcStack = []
        self.stack_name = ''

    def get_stack(self, s):
        """
        This gets the stack created by the parser, it adds MemberCalculation
        objects for each grouped calculation for easy querying.
        """
        import pyparsing
        if len(s) == 0:
            return None
        op = s.pop()
        if type(op) not in [str, unicode]:
            # list, pyparsing result or other.
            if type(op) == pyparsing.ParseResults:
                # we check for specific parse names
                if len(op.func) == 1:
                    # if we have just a single function on the match, create a new Function.
                    return FunctionCalculation(op, [self.get_stack([o]) for o in op[0]])
                if len(op.func) > 1:
                    # if we have more than one function, we create a Set.
                    return SetCalculation([self.get_stack([o]) for o in op.func])
                if len(op.level) == 1:
                    return StringCalculation([i for i in op.level[0]])
                if len(op.level) > 1:
                    # if we are dealing with multiple levels
                    return SetCalculation([StringCalculation([i for i in l]) for l in op.level])
                # anything else, try to push into a Set.
                return SetCalculation([self.get_stack([i]) for i in op])

            if type(op) == list and len(op) > 0:
                return self.get_stack(op)
            return op

        if op in "+-*/^":
            # one of the operators, the next on the stack is right, the first op the stack in left.
            op2 = self.get_stack(s)
            op1 = self.get_stack(s)
            return MemberCalculation(op1, op, op2)
        return StringCalculation(op)

    def __str__(self):
        return "<CalculationGrammar %s>" % self.stack

class CalculatedMemberGrammar:

    def get_properties(self):
        properties = [
            'CurrentMember',
            'Children',
            'Members',
            'FirstChild',
            'LastChild',
            'Count'
        ]
        rv = None
        for prop in properties:
            if not rv:
                rv = Keyword('.%s' % prop)
                continue
            rv = rv | Keyword('.%s' % prop)
        return rv

    def get_functions(self):
        functions = [
            'avg',
            'rank',
            'iif',
            'aggregate',
            'topcount',
            'bottomcount',
            'crossjoin',
            'except',
            'generate'
        ] # TODO, these should come from MdxFormula table?
        rv = None
        for prop in functions:
            if not rv:
                rv = Keyword('%s' % prop, caseless=True)
                continue
            rv = rv | Keyword('%s' % prop, caseless=True)
        return rv

    def get_methods(self):
        methods = [
            'Item'
        ]
        rv = None
        for prop in methods:
            if not rv:
                rv = Keyword('.%s' % prop)
                continue
            rv = rv | Keyword('.%s' % prop)
        return rv

    def get_mdx(self):

        scalar = Word(alphanums+"_"+" "+"-"+")"+"("+".")

        leftBr = Literal("(").suppress()
        rightBr = Literal(")").suppress()
        leftSqBr = Literal("[").suppress()
        rightSqBr = Literal("]").suppress()
        leftCurlBr = Literal("{").suppress()
        rightCurlBr = Literal("}").suppress()

        level = delimitedList(leftSqBr + scalar + rightSqBr, ".", combine=False)
        props = self.get_properties()

        if props:
            level = level + Optional(props)

        g = CalculationGrammar(None)

        level = Group(level).setResultsName("level" ,True)


        functions = self.get_functions()



        number = Word(nums)

        if functions:

            expr = Forward() # Forward the parser, this creates recursive grammar.
            op = operator_grammar(Forward(), g) # create a forwarder for operator (math calculations)

            # fill the function epxr, list of functions
            # Mathematical operator expr or List of function followed by:
            #  * a number; or
            #  * a recursive self; or
            #  * a level/member name; or
            #  * a set; or
            #  * a mathematical operator expr.
            expr << ((functions + leftBr \
                  + delimitedList(
                                number.setParseAction(g.push_number)
                                | expr.setParseAction(g.add_func).setResultsName('func')
                                | Group(leftCurlBr + delimitedList(level,',') + rightCurlBr)
                                | delimitedList(level,','),
                                ',') + rightBr) | op).setParseAction(g.parse_full_method_or_operator)


        # this is needed for the WITH-preprocessor for calculated members
        self.calculator = g
        return self.calculator.mdx(expr)

class AbstractCalculation:
    def get_value(self, x, client, cube):
        if type(x) == list:
            post_label = None
            if x[len(x)-1][0] == '.': # FIXME: HACK This should come from properties.
                post_label = x.pop()[1:]

            member = '[%s]' % '].['.join(x)
            desc = member

            if client and not post_label:
                desc = client.get_member_tree_op(cube, member, client.MDTREEOP_SELF).pop()
            elif client: # HACKHACK
                desc = client.get_level_info(cube, member).pop()

            if post_label is not None:
                member = '%s.%s' % (member, post_label)
                if client:
                    desc['member_caption'] = '%s (%s)' % (desc['level_caption'], post_label)
                else:
                    desc = '%s (%s)' % (desc, post_label)

            return {'name': member, 'description': desc}
        if isinstance(x, MemberCalculation) or isinstance(x, FunctionCalculation) or isinstance(x, SetCalculation) or isinstance(x, StringCalculation):
            return x.get_dict(client, cube)
        return {'name': x, 'description': x}

class StringCalculation(AbstractCalculation):
    def __init__(self, _str):
        self._str = _str

    def get_dict(self, client=None, cube=None):
        return self.get_value(self._str, client, cube)

class SetCalculation(AbstractCalculation):
    def __init__(self, _set):
        self._set = _set

    def get_dict(self, client=None, cube=None):
        return [self.get_value(x, client, cube) for x in self._set]

class FunctionCalculation(AbstractCalculation):

    def __init__(self, result, args):
        self._func = result.func[0]
        self._content = args

    def get_dict(self, client=None, cube=None):

        args = []
        for x in self._content:
            if type(x) == list:
                if type(x[0]) == list:
                    args.append([self.get_value(a, client, cube) for a in x])
                    continue
            args.append(self.get_value(x, client, cube))

        return {'function_': self._func, 'arguments_': args}

    def __repr__(self):
        return '<Function : %s arguments: %s>' % (self._func, self._content)

class MemberCalculation(AbstractCalculation):
    """
    Container object for execution of the Member calculations
    """
    def __init__(self, left_value, operator=None, right_value=None):
        """
        @param left_value: the left value of the calculation
        @param operator: the operator of the calculation
        @param right_value: the right value of the calculation
        """
        self.left_value = left_value
        self.operator = operator
        self.right_value = right_value

    def __eq__(self, other):
        """
        See if this MemberCalculation is equal to another one
        """
        if not isinstance(other,MemberCalculation):
            return False
        return self.left_value == other.left_value \
           and self.operator == other.operator \
           and self.right_value == other.right_value

    def __ne__(self, other):
        """
        See if this MemberCalculation is not equal to another one
        """
        if not isinstance(other,MemberCalculation):
            return True
        return self.left_value != other.left_value \
            or self.operator != other.operator \
            or self.right_value != other.right_value

    def __str__(self):
        """
        String output of the MemberCalculation object
        """
        return "<MemberCalculation: %s %s %s>" \
            % (self.left_value, self.operator, self.right_value)

    def get_dict(self, client=None, cube=None):

        l = self.get_value(self.left_value, client, cube)
        r = self.get_value(self.right_value, client, cube)

        return {'left': l, 'operator': self.operator, 'right': r}

    __repr__ = __str__

