# Copyright (c) 2010 BaseSoft B.V.
# See LICENSE for details.

from django.utils.translation import ugettext as _

from sqlalchemy import func, case, and_
from sqlalchemy.orm import aliased

from basesoft.web.core.util.custom_query import CustomQuery, CustomQueryField, CustomQueryModel, DATA_TYPE_BOOLEAN

from iqportal.iquest.core.models import Survey, SurveyPublication, SurveyVariableSet, IQuestItemPoolCategory, IQuestItemPoolItem
from basesoft.web.contrib.compose.models import ComposeInvitee, ComposePublicationContactList
from iqportal.iquest.response.models import SurveyParticipant, SurveyQAResponse
from basesoft.web.contrib.contact.models import ContactListContact, ContactList, Contact

class SurveyCustomQuery(CustomQuery):
    permissions = ('view_survey', )

    def tables(self):
        return (
            CustomQueryModel(Survey),
            CustomQueryModel(SurveyPublication, onclause=Survey.id==SurveyPublication.survey_id, use_outerjoin=True, priv=None),
        )

    def fields(self):
        return (
            CustomQueryField(Survey.id, _('ID')),
            CustomQueryField(Survey.name, _('Name'), is_filter=True),
            CustomQueryField(Survey.id, _('Settings'), key='settings'),
            CustomQueryField(func.count(SurveyPublication.id), _('Publications'), key='publications'),
            CustomQueryField(Survey.created, _('Created')),
            CustomQueryField(Survey.last_updated, _('Last updated')),
        )

    group_by = [Survey.id, ]

class PublicationCustomQuery(CustomQuery):
    permissions = ('view_survey_publication', )

    def tables(self):
        return (
            CustomQueryModel(SurveyPublication),
            CustomQueryModel(Survey, onclause=Survey.id==SurveyPublication.survey_id, priv=None),
            CustomQueryModel(ComposePublicationContactList,
                             onclause=SurveyPublication.id==ComposePublicationContactList.publication_id,
                             use_outerjoin=True),
            CustomQueryModel(ContactList,
                             onclause=ContactList.id==ComposePublicationContactList.contact_list_id,
                             use_outerjoin=True, priv=None),
            CustomQueryModel(ContactListContact,
                             onclause=ContactList.id==ContactListContact.contact_list_id,
                             use_outerjoin=True),
            CustomQueryModel(Contact,
                             onclause=ContactListContact.contact_id==Contact.id,
                             use_outerjoin=True, priv=None),
            CustomQueryModel(SurveyVariableSet,
                             onclause=SurveyPublication.variable_set_id==SurveyVariableSet.id,
                             use_outerjoin=True),
        )

    def fields(self):
        return (
            CustomQueryField(SurveyPublication.id, _('ID'), key="id"),
            CustomQueryField(Survey.name, _('Survey'), key="survey", is_filter=True, is_group_by=True),
            CustomQueryField(Survey.id, _('Survey ID'), key="survey_id"),
            CustomQueryField(SurveyPublication.label, _('Label'), key="label", is_filter=True),
            CustomQueryField(SurveyPublication.begin_date, _('Begin date'), key="begin_date", is_filter=True),
            CustomQueryField(SurveyPublication.end_date, _('End date'), key="end_date", is_filter=True),

            # if publication is new or scheduled, the invitees list will be all the contacts from the
            # publication's contact lists

            # if publication is online or finished, we get its invitees list from the invitees table;
            # after a publication is online, even if the contacts from its contact lists change, the invitees
            # will not change, they will still be just the ones from tblComposeInvitee created when publication
            # went online
            CustomQueryField(case([(SurveyPublication.status.in_([SurveyPublication.STATUS_NEW,
                                                                  SurveyPublication.STATUS_SCHEDULED]),
                             func.count(ContactListContact.contact_id.distinct()))],
                             else_=SurveyPublication.number_of_invitees),
                             _("Invitees"), key="invitees"),
            CustomQueryField(SurveyPublication.number_of_participants,
                             _('Participants'), key="participants"),
            CustomQueryField(SurveyPublication.number_of_respondents, _('Respondents'), key="respondents"),
            CustomQueryField(SurveyPublication.id, _('Response'), key="response"),
            CustomQueryField(SurveyPublication.status, _('Status'), key="status", is_filter=True),
            CustomQueryField(SurveyVariableSet.name, _('Variable Set'), key="variable_set", is_filter=True),
            CustomQueryField(SurveyPublication.anonymous, _('Anonymous'), key="anonymous", is_filter=True, type=DATA_TYPE_BOOLEAN),
            CustomQueryField(SurveyPublication.anonymous_hash, _('Survey link'), key="survey_link"),
            CustomQueryField(SurveyPublication.id, _('Export'), key="export"),

            CustomQueryField(SurveyPublication.id, _('Preview'), key="preview"),
            CustomQueryField(SurveyPublication.id, _('Report'), key="report"),
       )

    group_by = (SurveyPublication.id, )

class DashboardPublicationCustomQuery(CustomQuery):
    permissions = ('view_survey_publication', 'view_survey', )

    def tables(self):
        return (
            CustomQueryModel(SurveyPublication),
            CustomQueryModel(Survey, onclause=Survey.id==SurveyPublication.survey_id),
        )

    def fields(self):
        return (
            CustomQueryField(Survey.id, _('Survey ID'), key="survey_id"),
            CustomQueryField(SurveyPublication.id, _('ID'), key="id"),
            CustomQueryField(Survey.name, _('Survey'), key="survey", is_group_by=True),
            CustomQueryField(SurveyPublication.label, _('Label'), key="label", is_filter=True),
            CustomQueryField(SurveyPublication.begin_date, _('Begin date'), key="begin_date", is_filter=True),
            CustomQueryField(SurveyPublication.end_date, _('End date'), key="end_date", is_filter=True),
            CustomQueryField(SurveyPublication.id, _('Report'), key="report"),
            CustomQueryField(SurveyPublication.id, _('Response'), key="response"),
            CustomQueryField(SurveyPublication.status, _('Status'), key="status", is_filter=True),
            CustomQueryField(SurveyPublication.id, _('Preview'), key="preview"),
        )

class VariableSetCustomQuery(CustomQuery):
    permissions = ('view_survey_variable_set', )

    def tables(self):
        return (
            CustomQueryModel(SurveyVariableSet),
            CustomQueryModel(Survey, onclause=Survey.id==SurveyVariableSet.survey_id),
        )

    def fields(self):
        return (
            CustomQueryField(Survey.id, 'survey_id', in_select=False),
            CustomQueryField(SurveyVariableSet.id, _('ID'), key='id'),
            CustomQueryField(SurveyVariableSet.name, _('Name'), key='name', is_filter=True),
            CustomQueryField(SurveyVariableSet.created, _('Created')),
            CustomQueryField(SurveyVariableSet.last_updated, _('Last updated')),
        )

class PoolCustomQuery(CustomQuery):
    permissions = ('view_iquest_pool_category', )

    def tables(self):
        return (
            CustomQueryModel(IQuestItemPoolCategory),
            CustomQueryModel(IQuestItemPoolItem, onclause=IQuestItemPoolCategory.id==IQuestItemPoolItem.pool_category_id,
                             use_outerjoin=True),
        )

    def fields(self):
        return (
            CustomQueryField(IQuestItemPoolCategory.id, _('ID'), key='id'),
            CustomQueryField(IQuestItemPoolCategory.name, _('Name'), key = 'name', is_filter=True),
            CustomQueryField(func.count(IQuestItemPoolItem.id), _('Items'), key='items'),
            CustomQueryField(IQuestItemPoolCategory.created, _('Created')),
            CustomQueryField(IQuestItemPoolCategory.last_updated, _('Last updated'))
        )

    group_by = [IQuestItemPoolCategory.id, ]

class PoolItemCustomQuery(CustomQuery):
    permissions = ('view_iquest_pool_category', )

    def tables(self):
        return (
            CustomQueryModel(IQuestItemPoolItem),
            CustomQueryModel(IQuestItemPoolCategory, onclause=IQuestItemPoolCategory.id==IQuestItemPoolItem.pool_category_id),
        )

    def fields(self):
        return (
            CustomQueryField(IQuestItemPoolItem.id, _('ID'), key='id'),
            CustomQueryField(IQuestItemPoolItem.name, _('Name'), key='name', is_filter=True),
            CustomQueryField(IQuestItemPoolItem.id, _('Display'), key='display'),
            CustomQueryField(IQuestItemPoolItem.polymorphic_type, _('Type'), key='type', is_filter=True),
            CustomQueryField(IQuestItemPoolCategory.name, _('Category name'), key = 'category_name', is_filter=True),
            CustomQueryField(IQuestItemPoolCategory.id, _('Category'), key = 'category_id', is_group_by=True),
            CustomQueryField(IQuestItemPoolItem.created, _('Created')),
            CustomQueryField(IQuestItemPoolItem.last_updated, _('Last updated'))
        )


custom_queries = (SurveyCustomQuery, PublicationCustomQuery, DashboardPublicationCustomQuery, \
                  VariableSetCustomQuery, PoolCustomQuery, PoolItemCustomQuery)

