# Copyright (c) 2008 BaseSoft B.V.
# See LICENSE for details.

from sqlalchemy.sql import and_

from sqlalchemy.orm.session import Session
from django.utils.translation import ugettext, ugettext_lazy as _

from basesoft.web.contrib.compose.models import ComposePublication, ComposeInvitee, ComposeMail
from basesoft.web.core.models import PrivilegeObject, ComposePrivilegeObject

"""
These are the Models that are mapped to the Database Tables for the compose
part of IQuest.
"""

# ----- GENERAL IQUEST MODELS -----

class IQuestMail(ComposeMail):
    """
    Model for a survey mail, this inherits from the mailing app Mail
    @DynamicAttrs
    """

    NAME_I18N = _("IQuest e-mail")

    def __repr__(self):
        return '<IQuestMail id=%s>' % self.id

    @staticmethod
    def replacement_fields():
        x = ComposeMail.replacement_fields()
        x.update({
            ugettext('Survey'): {
                'iquest.survey.link': ugettext('Survey link'),
                'iquest.survey.name': ugettext('Survey name'),
                'iquest.survey.publication_label': ugettext('Publication label')
            }
        })
        return x

    def _replace_field(self, person, field_name, context):
        """
        Method called to replace every given replacement field with the current context. here we deal with specific
        IQuest replacement fields.. contact stuff is handled upstream.
        """
        from sqlalchemy.orm.exc import NoResultFound

        session = Session.object_session(self)

        # context is set in the MailingTask
        if context.has_key('publication'):
            publication = context['publication']
        else:
            assert context.has_key('mailing')
            publication = context['mailing'].publication
        contact = context['contact']

        if field_name == 'iquest.survey.name':
            return publication.survey.name

        if field_name == 'iquest.survey.publication_label':
            return publication.label

        # Process the link to survey.. this depends if the survey is anonymous or not
        # if the survey hasn't been published we just throw an exception which should abort the task (untested)
        if field_name == 'iquest.survey.link':

            # first we check if we can find the contact as part of the publication.
            try:
                invitee = session.query(ComposeInvitee).filter(
                    and_(ComposeInvitee.contact==contact,
                         ComposeInvitee.publication==publication)).one()
                invitee_hash = invitee.invitee_hash
            except NoResultFound:
                # apparently this contact was not included as a non-anonymous member of the publication
                # Lets see if this publication also is anonymous.
                if publication.anonymous:
                    try:
                        invitee_hash = publication.invitees[0].invitee_hash
                    except IndexError:
                        raise Exception('Publication %s, Contact: %s: Cannot setup invite email because survey hasnt been published yet.' %
                                        (publication.id, contact.id) )
                else:
                    raise Exception('Publication %s, Contact: %s: This contact is not part of this non-anonymous publication.' %
                                    (publication.id, contact.id) )

            # return the invite hash url in a <a>
            link = self._base_url(context) + 'iqportal.iquest.response/display/%s/' % invitee_hash
            return '<a href="%s">%s</a>' % (link, link)

        return ComposeMail._replace_field(self, person, field_name, context)

class IQuestTheme(PrivilegeObject):
    """
    @DynamicAttrs
    """

    name = None
    module = None
    cssClass = None

    def get_group_object(self):
        return (IQuestThemeGroup, 'theme')

    def __repr__(self):
        return '<IQuestTheme name=%s>' % self.name

class IQuestThemeGroup(object):
    """
    @DynamicAttrs
    """
    pass

class IQuestQAFilter(PrivilegeObject):
    """
    Model for the ItemQA Filter.
    @DynamicAttrs
    """

    id = None
    name = None

    mark_as_deleted_permission = 'delete_itemqa_filter'
    mark_as_archived_permission = 'update_itemqa_filter'

    def get_group_object(self):
        return (IQuestQAFilterGroup, 'filter')

    def __repr__(self):
        return '<IQuestQAFilter id=%s>' % self.id

class IQuestQAFilterGroup(object):
    """
    Model representing the link between itemqa filters and groups.
    @DynamicAttrs
    """

# ----- IQUEST POOL MODELS -----
class IQuestItemPoolCategory(PrivilegeObject):
    """
    @DynamicAttrs
    """
    DEFAULT_CATEGORY_NAME = 'Core Items'

    mark_as_deleted_permission = 'delete_iquest_pool_category'
    mark_as_archived_permission = 'update_iquest_pool_category'

    def get_group_object(self):
        return (IQuestItemPoolCategoryGroup, 'pool_category')

    def __repr__(self):
        return '<IQuestItemPoolCategory>'

class IQuestItemPoolCategoryGroup(object):
    """
    @DynamicAttrs
    """
    pass

class IQuestItemPoolItem(object):
    """
    @DynamicAttrs
    """
    def __repr__(self):
        return '<IQuestItemPoolItem id="%s">' % self.id

class IQuestItemPoolItemHtml(IQuestItemPoolItem):
    """
    @DynamicAttrs
    """
    def __repr__(self):
        return '<IQuestItemPoolItemHtml>'

class IQuestItemPoolItemHtmlLocale(object):
    """
    @DynamicAttrs
    """
    def __repr__(self):
        return '<IQuestItemPoolItemHtmlLocale>'

class IQuestItemPoolItemQA(IQuestItemPoolItem):
    """
    @DynamicAttrs
    """

    # Alignment for alignAnswers or alignQuestions
    ALIGN_TOP = 1
    ALIGN_RIGHT = 2
    ALIGN_BOTTOM = 3
    ALIGN_LEFT = 4

    # These define the how the QA item should look like and behave in the controller
    TYPE_MULTIPLE_CHOICE = 1
    TYPE_SINGLE_CHOICE = 2
    TYPE_SINGLE_TEXT_INPUT = 3

    # possible values for a QA objectName. This basically specifies a js handler for the QA. Except for the NPS QA
    # which needs special checks (can't add/remove answers), all QA's are handled by the default ItemQA.
    # all types specified here need mappers js side and server side (controller).
    # be sure to add them also to the SurveyItemQA model if applicable.
    available_object_names = [u'SurveyItemQA', u'ItemQANPS']

    def __questions(self):
        """
        Easy access to all the ItemQA's questions.
        """
        session = Session.object_session(self)
        return session.query(IQuestItemPoolItemQAElement).filter(and_(IQuestItemPoolItemQAElement.pool_item_id==self.id,
                                                                      IQuestItemPoolItemQAElement.is_question==True
                                                            )).order_by(IQuestItemPoolItemQAElement.position).all()
    questions = property(__questions)

    def __answers(self):
        """
        Easy access to all the ItemQA's answers.
        """
        session = Session.object_session(self)
        return session.query(IQuestItemPoolItemQAElement).filter(and_(IQuestItemPoolItemQAElement.pool_item_id==self.id,
                                                                      IQuestItemPoolItemQAElement.is_question==False
                                                            )).order_by(IQuestItemPoolItemQAElement.position).all()
    answers = property(__answers)

    def __repr__(self):
        return '<IQuestItemPoolItemQA id="%s">' % self.id

class IQuestItemPoolItemQALocale(object):
    """
    @DynamicAttrs
    """
    def __repr__(self):
        return '<IQuestItemPoolItemQALocale>'

class IQuestItemPoolItemQALayoutAttribute(object):
    """
    @DynamicAttrs
    """
    pass

class IQuestItemPoolItemQAElement(object):
    """
    @DynamicAttrs
    """
    def __repr__(self):
        return '<IQuestItemPoolItemQAElement id=%s>' % self.id

class IQuestItemPoolItemQAElementLocale(object):
    """
    @DynamicAttrs
    """
    def __repr__(self):
        return '<IQuestItemPoolItemQALabelLocale>'

class IQuestItemPoolItemQAFilter(object):
    """
    Model for representing the link between filters associated with items and what attributes and values they have.
    @DynamicAttrs
    """

    def __repr__(self):
        return '<IQuestItemPoolItemQAFilter id=%s>' % self.id

class IQuestItemPoolItemQAFilterAttribute(object):
    """
    @DynamicAttrs
    """
    def __repr__(self):
        return '<IQuestItemPoolItemQAFilterAttribute>'

# ----- IQUEST SURVEY SPECIFIC TABLES -----
class Survey(ComposePrivilegeObject):
    """
    Model for the Survey
    @DynamicAttrs
    """
    MODE_RESPONSE = ComposePrivilegeObject.MAX_MODE_BIT*2

    # privilege to update a survey which has publications
    PRIV_UPDATE_PUBLISHED_SURVEY = 32

    id = None
    name = None
    last_updated = None

    _name_property = 'name'

    mark_as_deleted_permission = 'delete_survey'
    mark_as_archived_permission = 'update_survey'

    def get_privileges(self):
        return PrivilegeObject.get_privileges(self) + [
            (self.PRIV_UPDATE_PUBLISHED_SURVEY, ugettext('Update published survey privileges')),
        ]

    def get_group_object(self):
        return (SurveyGroup, 'survey')

    def __get_item_QAs(self):
        """
        Easy access to all the ItemQA's in the survey when you dont care in which block/column they are.
        """
        session = Session.object_session(self)
        return session.query(SurveyItemQA) \
            .join(SurveyBlockColumn) \
            .join(SurveyBlock) \
            .join(Survey) \
            .filter(Survey.id==self.id) \
            .order_by(SurveyBlock.position, SurveyBlockColumn.position, SurveyItemQA.position) \
            .all()

    itemQAs = property(__get_item_QAs)

    def __publications(self):
        """
        Easy access to all the publications.
        """
        session = Session.object_session(self)
        return session.query(SurveyPublication).filter(and_(SurveyPublication.deleted==0, SurveyPublication.archived==0,
                                                            SurveyPublication.survey_id==self.id
                                                            )).order_by(SurveyPublication.begin_date).all()
    publications = property(__publications)

    def __repr__(self):
        return '<Survey id=%s>' % self.id


class SurveyVariableSet(PrivilegeObject):
    """
    holds variables for a survey with a name
    @DynamicAttrs
    """
    survey = None
    name = None

    mark_as_deleted_permission = 'delete_survey_variable_set'
    mark_as_archived_permission = 'update_survey_variable_set'

    def get_group_object(self):
        return (SurveyVariableSetGroup, 'variable_set')

    def __repr__(self):
        return '<SurveyVariableSet id=%s>' % self.id


class SurveyVariableSetGroup(object):
    """
    @DynamicAttrs
    """
    pass

class SurveyVariableLocale(object):
    """
    one variable per locale and handle with text
    @DynamicAttrs
    """
    variable_set = None
    locale = None
    variable = None
    text = None

class SurveyPublication(ComposePublication):
    """
    Model representing periods in which the survey is available.
    @DynamicAttrs
    """

    mark_as_deleted_permission = 'delete_survey_publication'
    mark_as_archived_permission = 'update_survey_publication'

    # privilege to view the response (including reports)
    PRIV_VIEW_RESPONSE = 64
    # privilege to view the response (including reports)
    PRIV_CREATE_RESPONSE = 128
    # privilege to update the response (including reports)
    PRIV_UPDATE_RESPONSE = 256
    # privilege to delete the response (including reports)
    PRIV_DELETE_RESPONSE = 512

    # if given it allows download to mobile
    PRIV_OFFLINE = 1024

    all_privileges = (PrivilegeObject.PRIV_VIEW, PrivilegeObject.PRIV_UPDATE, PrivilegeObject.PRIV_DELEGATE_PRIVILEGES,\
                      PrivilegeObject.PRIV_ARCHIVE, PrivilegeObject.PRIV_DELETE, ComposePublication.PRIV_SCHEDULE,\
                      PRIV_VIEW_RESPONSE, PRIV_CREATE_RESPONSE, PRIV_UPDATE_RESPONSE, PRIV_DELETE_RESPONSE)

    # All answers can be changed by the participant.
    MULTI_SESSION_FREE_FOR_ALL = 1
    # On reopening of the Survey, all previous given answers are reset.
    MULTI_SESSION_CLEAR_PREVIOUS_ANSWERS = 2
    # On reopening of the Survey or navigating back the previous answers are marked as read-only
    MULTI_SESSION_PREVIOUS_ANSWERS_READ_ONLY = 3
    # The participant gets one chance to fill out the survey, on reopening they get a message that they already filed responses.
    # No navigating back possible.
    MULTI_SESSION_ONE_SHOT = 4

    _name_property = 'label'

    NAME_I18N = _('Survey publication')

    permissions = ('create_survey_publication', )

    js_module = 'iquest_publication'

    def get_controller_instance(self):
        from iqportal.iquest.compose.controllers import PublicationStateController
        return PublicationStateController

    def get_privileges(self):
        return ComposePublication.get_privileges(self) + [
            (self.PRIV_VIEW_RESPONSE, ugettext('View response privileges')),
            (self.PRIV_CREATE_RESPONSE, ugettext('Create response privileges')),
            (self.PRIV_UPDATE_RESPONSE, ugettext('Update response privileges')),
            (self.PRIV_DELETE_RESPONSE, ugettext('Delete response privileges')),
            (self.PRIV_OFFLINE, ugettext('Available offline')),
        ]

    @staticmethod
    def get_multi_session_options():
        return [
            (SurveyPublication.MULTI_SESSION_FREE_FOR_ALL, ugettext('Allow changes of previous given responses')), # default
            #(SurveyPublication.MULTI_SESSION_CLEAR_PREVIOUS_ANSWERS, ugettext('Clear previous given responses')),
            #(SurveyPublication.MULTI_SESSION_PREVIOUS_ANSWERS_READ_ONLY, ugettext('Previous given responses are read-only')),
            (SurveyPublication.MULTI_SESSION_ONE_SHOT, ugettext('One-shot: no backwards navigation or changes possible.')),
        ]

    def __repr__(self):
        return '<SurveyPublication id=%s>' % (self.id)

class SurveyGroup(object):
    """
    Model representing the link between surveys and groups.
    @DynamicAttrs
    """

class SurveyLocale(object):
    """
    Model for the Survey Locale, this is the list of available locales for the survey.
    @DynamicAttrs
    """

class SurveyBlock(object):
    """
    Model for a Block within a survey.
    A block can contain items of various types.
    @DynamicAttrs
    """

    def __repr__(self):
        return '<SurveyBlock id=%s>' % self.id

class SurveyBlockColumn(object):
    """
    Block contains Columns
    @DynamicAttrs
    """
    def _get_items(self):
        """
        This creates a property .items on the SurveyBlock object. When .items is requested the query below is executed
        and returns a list of all (*) extensions of SurveyItem.
        Also see http://www.sqlalchemy.org/docs/04/mappers.html#advdatamapping_mapper_inheritance
        """
        session = Session.object_session(self)
        return session.query(SurveyItem).with_polymorphic('*').filter_by(column_id=self.id).order_by('position').all()

    items = property(_get_items)

    def __repr__(self):
        return '<SurveyBlockColumn id=%s>' % self.id

class SurveyItem(object):
    """
    Model for the Survey Item.
    @DynamicAttrs
    """

    def __repr__(self):
        return '<SurveyItem id=%s>' % self.id

class SurveyItemHtml(SurveyItem):
    """
    @DynamicAttrs
    """

    def __repr__(self):
        return '<SurveyItemHtml id=%s>' % self.id

class SurveyItemHtmlLocale(object):
    """
    @DynamicAttrs
    """

    def __repr__(self):
        return '<SurveyItemHtmlLocale id=%s>' % self.item_html_id

class SurveyItemQA(SurveyItem):
    """
    Model for the Survey Question Item Group.
    @DynamicAttrs
    """

    # Alignment for alignAnswers or alignQuestions
    ALIGN_TOP = 1
    ALIGN_RIGHT = 2
    ALIGN_BOTTOM = 3
    ALIGN_LEFT = 4

    # These define the how the QA item should look like and behave in the controller
    TYPE_MULTIPLE_CHOICE = 1
    TYPE_SINGLE_CHOICE = 2
    TYPE_SINGLE_TEXT_INPUT = 3

    # possible values for a QA objectName. This basically specifies a js handler for the QA. Except for the NPS QA
    # which needs special checks (can't add/remove answers), all QA's are handled by the default ItemQA.
    # all types specified here need mappers js side and server side (controller).
    # be sure to add them also to the IQuestItemPoolItemQA model if applicable.
    available_object_names = [u'SurveyItemQA', u'ItemQANPS']

    def __elements(self):
        """
        Easy access to all the ItemQA's elements (questions and answers).
        """
        session = Session.object_session(self)
        return session.query(SurveyItemQAElement).filter(SurveyItemQAElement.item_id==self.id).order_by(
                                                        SurveyItemQAElement.position).all()
    elements = property(__elements)

    def __questions(self):
        """
        Easy access to all the ItemQA's questions.
        """
        session = Session.object_session(self)
        return session.query(SurveyItemQAElement).filter(and_(SurveyItemQAElement.item_id==self.id,
                                                              SurveyItemQAElement.is_question==True
                                                              )).order_by(SurveyItemQAElement.position).all()
    questions = property(__questions)

    def __answers(self):
        """
        Easy access to all the ItemQA's answers.
        """
        session = Session.object_session(self)
        return session.query(SurveyItemQAElement).filter(and_(SurveyItemQAElement.item_id==self.id,
                                                              SurveyItemQAElement.is_question==False
                                                              )).order_by(SurveyItemQAElement.position).all()
    answers = property(__answers)


    def __repr__(self):
        return '<SurveyItemQA id=%s>' % self.id

class SurveyItemQALayoutAttribute(object):
    """
    @DynamicAttrs
    """
    pass

class SurveyItemQALocale(object):
    """
    @DynamicAttrs
    """
    pass

class SurveyItemQAElement(object):
    """
    Model for the Survey Question or Answer Item.
    @DynamicAttrs
    """

    def __repr__(self):
        return '<SurveyItemQAElement id=%s>' % self.id

class SurveyItemQAElementLocale(object):
    """
    Model for the Survey Element Item titles and help texts per locale.
    @DynamicAttrs
    """

    def __repr__(self):
        return '<SurveyQuestionLocale %s>' % self.locale.locale

class SurveyItemQAWeightFactor(object):
    """
    @DynamicAttrs
    """
    def __repr__(self):
        return '<SurveyItemQAWeightFactor id=%s>' % self.id

class SurveyItemQAFilter(object):
    """
    Model for representing the link between filters associated with items and what attributes and values they have.
    @DynamicAttrs
    """
    def is_numerical(self):
        return self.filter.name == 'Numeric'

    def __repr__(self):
        return '<SurveyItemQAFilter id=%s>' % self.id

class SurveyItemQAFilterAttribute(object):
    """
    @DynamicAttrs
    """
    def __repr__(self):
        return '<SurveyItemQAFilterAttribute>'

class SurveyItemQABranch(object):
    """
    @DynamicAttrs
    """
    def __repr__(self):
        return '<SurveyItemQABranch id=%s>' % self.id
