# Copyright (c) 2009 BaseSoft B.V.
# See LICENSE for details.

"""
This contains the controllers for item filters/validation
"""

from sqlalchemy import and_

from basesoft.web.core.util.controllers import AbstractController, transaction_decorator
from basesoft.web.contrib.license.models import Group

from iqportal.iquest.core.models import Survey, IQuestQAFilter,\
    SurveyItemQA, SurveyItemQAFilter, SurveyItemQAFilterAttribute,\
    IQuestItemPoolItemQA, IQuestItemPoolItemQAFilter, IQuestItemPoolItemQAFilterAttribute

from iqportal.iquest.core.qa_filters import attributes as valid_attributes

class IQuestQAFilterController(AbstractController):
    """
    This controller deals with filters used for ItemQAs
    This should be called from a view
    """

    def get_item_qa(self, id):
        """
        Return the itemqa objects that the user has access to. This is useful for methods that update the filters of
        a given itemQA object.

        @param id: int or list of ints. the id(s) of the itemqa objects we want to return
        @return: a list of SurveyItemQA objects
        """
        self.assert_permissions('view_survey')

        if not isinstance(id, list):
            id = [id]

        filter = SurveyItemQA.id.in_(id)

        items = self.sa_session.query(SurveyItemQA).filter(filter).all()
        # check if the user has view privileges for the survey of which this item belongs to
        for i in items:
            survey_object = i.item.column.block.survey
            self.assert_privileges_on_object(survey_object, Survey.PRIV_VIEW)

        return items

    def existing_filters(self):
        """
        Return all existing filters (Text, Numeric, etc) as objects.
        This is used for adding the filters in the environment for example. This works even if the user is not logged
        in, because it gives us all filters, we don't need to filter them with privileges, etc.

        @return: a list of IQuestQAFilter objects.
        """
        return self.sa_session.query(IQuestQAFilter).all()

    def get_filters(self, id=None, order_by=None):
        """
        Return the filters the user has access to

        @param id: None, int or list of ints. the id(s) of the filters we want to return or None if we want all
        @param order_by: the column after which we want to make the ordering, or None we order by name
        @return: a list of IQuestQAFilter objects
        """
        if order_by is None:
            order_by = IQuestQAFilter.name

        return self.get_privileged_object_by_id(IQuestQAFilter, id, priv=IQuestQAFilter.PRIV_VIEW, order_by=order_by)

    @transaction_decorator
    def save_or_update_filter(self, name, id=None, extended_filter=None, group_id=None):
        """
        Create a new filter or update an existing one
        @param id: If id is not None, update the filter with the provided id. If id is None, insert the filter
        @param name: the name of the filter. Eg. Text, Date, VAT, etc.
        @param extended_filter: {IQuestQAFilter} or None; if None, this is a base filter, that doesn't inherit anything
        @return: {IQuestQAFilter}
        """

        if id is None:
            self.assert_permissions('create_itemqa_filter', group_id=group_id)
            filter = IQuestQAFilter()
        else:
            self.assert_permissions('update_itemqa_filter')
            filter = self.sa_session.query(IQuestQAFilter).filter(IQuestQAFilter.id==id).one()
            self.assert_privileges_on_object(filter, IQuestQAFilter.PRIV_UPDATE)

        filter.name = name
        if extended_filter is not None:
            filter.extended_filter = extended_filter

        if id is None:
            self.save_default_privilege_object_group(filter, target_group=group_id)

        self.sa_session.add(filter)
        return filter

    @transaction_decorator
    def delete_filters(self, filters):
        """
        Delete filter(s).
        If the filter is already used for any item the delete action is restricted.
        @param filters: {IQuestQAFilter} or {list} with IQuestQAFilter objects
        """
        self.assert_permissions('delete_itemqa_filter')
        if not isinstance(filters, list):
            filters = [filters]

        for f in filters:
            assert isinstance(f, IQuestQAFilter)
            self.mark_as_deleted(IQuestQAFilter, f.id)


class IQuestItemQAFilterController(AbstractController):
    """
    This controller deals with filters used for iquest pool items
    """
    _instance_class = IQuestItemPoolItemQAFilter
    _instance_attribute_class = IQuestItemPoolItemQAFilterAttribute
    _item_column = 'pool_item_id'
    _item_class = IQuestItemPoolItemQA

    def __init__(self, session, locale, account_id, manager):
        AbstractController.__init__(self, session, locale, account_id)
        self.qaFilterController = IQuestQAFilterController(session, locale, account_id)
        self.manager = manager

    def get_item_filters(self, item_id, filter_id=None, **kwargs):
        """
        Return the filters/configuration (attributes and values for filters) for the given item
        @param item_id: None, int or list of ints. the id(s) of the items for which we want to return the filters or None for all items
        @param filter_id: int or list of ints. get only item filters that have as filter one of the filters identified by id(s)

        !! Permissions and privileges are checked only in compose mode (see manager)

        **kwargs
        This params are not mandatory, but they can be provided. If they're not provided, they're not taken into account
        when querying. This is usefull if we want to get for example all the entries associated with a certain itemQA:

        @kwparam question_id: None, int or list of ints. the id(s) of the question ids for which we want to return the filters.
            Can be none too if we have an item filter that's for a whole answer.
        @kwparam answer_id: None, int or list of ints. the id(s) of the answers for which we want to return the filters.
            Can be none too if we have an item filter that's for a whole question.

        @return: a list of SurveyItemQAFilter objects
        """
        # pool items don't have a mode
        if self.manager.mode is None or self.manager.has_mode(self.manager.STRICT_MODE_IS_EQUAL, Survey.MODE_COMPOSE):
            self.assert_permissions('view_itemqa_filter')
        if not isinstance(item_id, list):
            item_id = [item_id]
        filter = getattr(self._instance_class, self._item_column).in_(item_id)

        if filter_id is not None:
            if not isinstance(filter_id, list):
                filter_id = [filter_id]

            # make sure the user has view privileges for the requested filters
            if self.manager.mode is None or self.manager.has_mode(self.manager.STRICT_MODE_IS_EQUAL, Survey.MODE_COMPOSE):
                filter_objects = self.qaFilterController.get_filters(filter_id)
                for f in filter_objects:
                    self.assert_privileges_on_object(f, f.PRIV_VIEW)
            filter = and_(self._instance_class.filter_id.in_(filter_id))

        # only check the question_id in the query if it was provided in the param list, otherwise it means we don't care
        # about the question id
        if kwargs.has_key('question_id'):
            question_id = kwargs['question_id']
            if question_id is None:
                filter = and_(filter, self._instance_class.question_id==None)
            else:
                if not isinstance(question_id, list):
                    question_id = [question_id]
                filter = and_(filter, self._instance_class.question_id.in_(question_id))

        # only check the answer_id in the query if it was provided in the param list, otherwise it means we don't care
        # about the answer id
        if kwargs.has_key('answer_id'):
            answer_id = kwargs['answer_id']
            if answer_id is None:
                filter = and_(filter, self._instance_class.answer_id==None)
            else:
                if not isinstance(answer_id, list):
                    answer_id = [answer_id]
                filter = and_(filter, self._instance_class.answer_id.in_(answer_id))

        return self.sa_session.query(self._instance_class).filter(filter).all()

    @transaction_decorator
    def save_or_update_item_filter(self, id, filter_object, item_object, attributes=None, question_id=None, answer_id=None):
        """
        Attach a filter to an itemQA or modify an existing one
        @param id: If id is not None, update the item filter with the provided id. If id is None, insert the item filter
        @param filter_object: the filter object that's attached to the item
        @param item_object: the item object to which we attach the filter
        @param attributes: a dictionary of attribute_name=>value
        @param question_id: the question id to which we attach the filter (can be None)
        @param answer_id: the question id to which we attach the filter (can be None)
        @return: {_instance_class}

        Note: if question_id and answer_id are both None, then we have a general filter, that applies to the whole item.
            if question_id is None and answer_id is not None, then we have a filter that applies to the whole answer.
            if answer_id is None and question_id is not None, then we have a filter that applies to the whole question.
            if both answer_id and question_id are not None, the filter applies to a single cell from the matrix
        """
        if id is None:
            sif = self._instance_class()
        else:
            sif = self.sa_session.query(self._instance_class).filter(self._instance_class.id==id).one()
            # delete attributes
            for attribute in sif.attributes:
                self.sa_session.delete(attribute)
            self.sa_session.flush()

        assert isinstance(filter_object, IQuestQAFilter) # we must have a proper filter object that we use for this item
        assert isinstance(attributes, dict) or attributes is None
        assert isinstance(item_object, self._item_class) # we must have an _item_class object
        assert isinstance(question_id, (int, long)) or question_id is None # make sure question_id is None or int
        assert isinstance(answer_id, (int, long)) or answer_id is None # make sure answer_id is None or int

        sif.filter = filter_object
        if hasattr(sif, 'item'):
            sif.item = item_object
        else:
            sif.pool_item = item_object
        sif.question_id = question_id
        sif.answer_id = answer_id

        if attributes is not None:
            for attr, value in attributes.iteritems():
                if value is None or value == '':
                    continue
                assert valid_attributes.has_key(attr)
                attribute = self._instance_attribute_class()
                attribute.attribute = attr
                attribute.value = unicode(value)
                self.sa_session.add(attribute)
                sif.attributes.append(attribute)

        self.sa_session.add(sif)
        return sif

    @transaction_decorator
    def delete_item_filters(self, item_filters):
        """
        Delete item filter association(s).

        @param item_filters: {SurveyItemQAFilter} or {list} with SurveyItemQAFilter objects
        """
        if not isinstance(item_filters, list):
            item_filters = [item_filters]

        for f in item_filters:
            assert isinstance(f, self._instance_class)
            self.sa_session.delete(f)

        self.sa_session.flush()

class SurveyItemQAFilterController(IQuestItemQAFilterController):
    """
    This controller deals with filters used for survey items
    """
    _instance_class = SurveyItemQAFilter
    _instance_attribute_class = SurveyItemQAFilterAttribute
    _item_column = 'item_id'
    _item_class = SurveyItemQA
