# Copyright (c) 2009 BaseSoft B.V.
# See LICENSE for details.

"""
The IQuest qa_filters are used for filtering and validating the data provided by the participants
of the surveys.

For more information see http://trac.office.basesoft.com/iqportal/wiki/IQuestItemQAValidation
"""
import re

from django.forms.util import ValidationError
from django.utils.translation import ungettext, ugettext as _
from decimal import Decimal



class Filter:
    """
    Overridable object for QA filters.
    """
    ALWAYS_CHECK = False
    
    def __init__(self, values):
        """
        @param values: list with dictionaries with the following keys:
            'a': answer_id; optional
            'q': question_id; optional
            'v': value; required
        """
        if not isinstance(values, list):
            values = [values]
        self.values = values

    def validate(self, maxAnswersSupplied=None, minAnswersRequired=None, should_validate=True):

        if not should_validate:
            return True

        len_v = len( [v['v'] for v in self.values if v['v'] is not None] )

        if maxAnswersSupplied is not None:
            maxAnswersSupplied = int(maxAnswersSupplied)
            if len_v > maxAnswersSupplied:
                raise ValidationError(_('The number of given answers must be less than %s.') % maxAnswersSupplied)

        if minAnswersRequired is not None:
            minAnswersRequired = int(minAnswersRequired)
            if len_v < minAnswersRequired:
                raise ValidationError(_('The number of given answers must be more than %s.') % minAnswersRequired)

        return True



class Text(Filter):
    """
    Allow all characters, can filter with regular expressions, can also filter on length.
    """
    regexp_exception = _('Value "%s" does not match the required pattern')

    def validate(self, minLength=None, maxLength=None, regexp=None, **kwargs):
        """
        Validate the text with the supplied keyworded arguments.
        """
        if minLength is not None:
            minLength = int(minLength)

        if maxLength is not None:
            maxLength = int(maxLength)

        for val in self.values:
            value = val['v']
            if value is None:
                value = ''

            l = len(value)
            if regexp is not None and not re.match(regexp, value):
                raise ValidationError(self.regexp_exception % value)
            if minLength is not None and l < minLength:
                raise ValidationError('Value "%s" is shorter than "%s" characters' % (value, minLength))
            if maxLength is not None and l > maxLength:
                raise ValidationError('Value "%s" is longer than "%s" characters' % (value, maxLength))

        return Filter.validate(self, **kwargs)

class AlphaNumeric(Text):
    """
    Only allows Letters and Numbers.
    """
    regexp = re.compile('^[\w\d]+$')
    regexp_exception = _('Value "%s" is not AlphaNumeric')

    def validate(self, minLength=None, maxLength=None, **kwargs):
        return Text.validate(self, minLength, maxLength, self.regexp, **kwargs)

class Email(Text):
    """
    Only allow email addresses
    """
    # note: '+' is an allowed character in Gmail user names.
    regexp = re.compile('^[a-zA-Z0-9+_\-\.]+@[0-9a-zA-Z]+[.-0-9a-zA-Z]*.[a-zA-Z]+$')
    regexp_exception = _('Value "%s" is not a valid email address')

    def validate(self, minLength=None, maxLength=None, **kwargs):
        return Text.validate(self, minLength, maxLength, self.regexp, **kwargs)

class URL(Text):
    regexp = re.compile('^http[s]?://.*$')
    regexp_exception = _('Value "%s" is not a valid URL (format: http(s)://www.domain.com/')

    def validate(self, minLength=None, maxLength=None, **kwargs):
        return Text.validate(self, minLength, maxLength, self.regexp, **kwargs)


class Date(Text):

    regexp = re.compile('^\d\d-\d\d-\d\d\d\d$ (format dd-mm-yyyy)')
    regexp_exception = _('Value "%s" is not a valid date')

    def validate(self, minLength=None, maxLength=None, minDate=None, maxDate=None):
        return Text.validate(self, minLength, maxLength, self.regexp)

class DateTime(Text):

    regexp = re.compile('^\d\d-\d\d-\d\d\d\d \d\d:\d\d$')
    regexp_exception = _('Value "%s" is not a valid date time (format dd-mm-yyyy hh:ss)')

    def validate(self, minLength=None, maxLength=None, minDate=None, maxDate=None):
        return Text.validate(self, minLength, maxLength, self.regexp)

class Numeric(Filter):
    ALWAYS_CHECK = True

    i18n_domain = {
        'float': _('floating point'),
        'int': _('whole')
    }

    def _isFloat(self, s):
        """
        Simple check to see if the provided value is a float.1
        """
        try: float(s)
        except (ValueError, TypeError): return False
        else: return True

    def _isInt(self, s):
        """
        Simple check to see if the provided value is an integer
        """
        try: return float(s).is_integer()
        except (ValueError, TypeError): return False

    def _isNumeric(self, value, domain=None):
        """
        Check to see whether or not the provided value is numerical and within the provided domain (int, float)
        """
        if value is None: return True # it isn't obliged.

        if domain is not None:
            if domain == 'float':
                if isinstance(value, float): return True
                if self._isFloat(value): return True
                return False
            if domain == 'int':
                if isinstance(value, int): return True
                if self._isInt(value): return True
                return False
            return False

        if isinstance(value, int): return True
        if isinstance(value, float): return True
        if value.isdigit(): return True
        if self._isFloat(value): return True

    def validate(self, minValue=None, maxValue=None, domain=None, sumValue=None, **kwargs):
        """
        Validate the provided values to be numerical
        """

        def set_value(value, domain=None):
            if value is None:
                return None
            # if we have a domain set, cast to the correct type
            if domain and domain == 'float':
                return Decimal(str(value))
            if domain:
                return int(float(value))
            if isinstance(value, (int, float)):
                return value
            if value.isdigit():
                return int(value)
            if self._isFloat(value):
                return float(value)
            return value

        minValue = set_value(minValue, domain)
        maxValue = set_value(maxValue, domain)
        sumValue = set_value(sumValue, domain)

        messages = []
        valid = True

        for val in self.values:
            value = val['v']
            if not self._isNumeric(value, domain):
                messages.append(ungettext('The value needs to be a %(numeric_type)s number',
                                          'All values need to be %(numeric_type)s numbers',
                                          len(self.values)) % {'numeric_type': Numeric.i18n_domain[domain]})
                valid = False
                continue
            value = set_value(value, domain)
            if value is None:
                continue
            val['v'] = value
            if minValue is not None and value < minValue:
                messages.append(ungettext('Value needs to be more than %(minValue)s',
                                          'All values need to be more than "%(minValue)s"',
                                          len(self.values)) % {'minValue': minValue})
                valid = False
                continue
            if maxValue is not None and value > maxValue:
                messages.append(ungettext('The value needs to be less than "%(maxValue)s"',
                                          'All values need to be less than "%(maxValue)s"',
                                          len(self.values)) % {'maxValue': maxValue})
                valid = False
                continue

        if not valid:
            raise ValidationError([i for i in set(messages)])

        if sumValue is not None:
            v = [val['v'] for val in self.values if val['v'] is not None]
            if len(v) > 0 and (sumValue != sum(v)):
                raise ValidationError(_('The sum of all values needs be "%s"') % sumValue)


        return Filter.validate(self, **kwargs)
