# Copyright (c) 2008 BaseSoft B.V.
# See LICENSE for details.

"""
This file contains the objects, functions and variables to setup the compose
part of the IQuest application.
"""
import datetime
from sqlalchemy import and_, Column, Integer, ForeignKey, String, \
                       DateTime, UnicodeText, Unicode, SmallInteger, UniqueConstraint
from sqlalchemy.dialects.mysql import MEDIUMTEXT
from basesoft.sqlalchemy.table import IQTable

from basesoft.web.contrib.license.hooks import PrivilegeObjectMapper
from basesoft.web.contrib.license import models as license_models
from basesoft.web.contrib.account import models as account_models
from django.utils.translation import ugettext as _
from iqportal.iquest.core.models import SurveyPublication
from basesoft.web.contrib.i18n.models import Locale
from sqlalchemy.types import Boolean

__mapping_defined = False
__tables_defined = False
tables = {}

def reset():
    global __mapping_defined
    global __tables_defined

    __mapping_defined = False
    __tables_defined = False

def get_tables():
    return tables

def do_tables(metadata):
    global __tables_defined
    if not __tables_defined:

        # ----- GENERAL IQUEST TABLES -----
        tblIQuestMail = IQTable('tblIQuestMail', metadata,
            Column('mail_id', Integer, ForeignKey('tblMail.id', ondelete='CASCADE'), primary_key=True),
            Column('fallback_locale_id', Integer, ForeignKey('tblLocale.id'), nullable=False),
        )

        # Theme stuff
        tblIQuestTheme = IQTable('tblIQuestTheme', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('name', String(255), nullable=False, unique=True), # name like 'Basesoft Theme'.. selected by the user
            Column('module', String(255), nullable=False, unique=True), # module name to load from the loader
            Column('css_class', String(255), nullable=False, unique=True), # class name in the css file to apply to the survey

            Column('deleted', SmallInteger, nullable=False, default=0),
            Column('archived', SmallInteger, nullable=False, default=0),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        tblIQuestThemeGroup = IQTable('tblIQuestThemeGroup', metadata,
            Column('theme_id', Integer, ForeignKey('tblIQuestTheme.id', ondelete='CASCADE'), primary_key=True),
            Column('group_id', Integer, ForeignKey('tblGroup.id'), primary_key=True),
            Column('privilege', Integer, nullable=False, primary_key=True, autoincrement=False),
            Column('group_has_privilege', SmallInteger, default=0),
            Column('license_has_privilege', SmallInteger, default=0),
            Column('domain_has_privilege', SmallInteger, default=0),
            Column('world_has_privilege', SmallInteger, default=0),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
            )

        # item filter stuff
        tblIQuestQAFilter = IQTable('tblIQuestQAFilter', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            # we need to know what this filter extends (if necessary), because the user might define his own filters,
            # so the validation part needs to know what to use for validating (the rules of the filter this extends)
            Column('extended_filter_id', Integer, ForeignKey('tblIQuestQAFilter.id'), nullable=True),
            Column('name', UnicodeText, nullable=False),

            Column('deleted', SmallInteger, nullable=False, default=0),
            Column('archived', SmallInteger, nullable=False, default=0),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        # This is the link between a group of accounts and the filters, this will be used for access rights.
        tblIQuestQAFilterGroup = IQTable('tblIQuestQAFilterGroup', metadata,
            Column('filter_id', Integer, ForeignKey('tblIQuestQAFilter.id', ondelete='CASCADE'), primary_key=True),
            Column('group_id', Integer, ForeignKey('tblGroup.id', ondelete='CASCADE'), primary_key=True),
            Column('privilege', Integer, nullable=False, primary_key=True, autoincrement=False),
            Column('group_has_privilege', SmallInteger, default=0),
            Column('license_has_privilege', SmallInteger, default=0),
            Column('domain_has_privilege', SmallInteger, default=0),
            Column('world_has_privilege', SmallInteger, default=0),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )


        # ----- IQUEST POOL TABLES -----
        tblIQuestItemPoolCategory = IQTable('tblIQuestItemPoolCategory', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('name', Unicode(255), nullable=False),
            Column('deleted', SmallInteger, nullable=False, default=0),
            Column('archived', SmallInteger, nullable=False, default=0),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        tblIQuestItemPoolCategoryGroup = IQTable('tblIQuestItemPoolCategoryGroup', metadata,
            Column('pool_category_id', Integer, ForeignKey('tblIQuestItemPoolCategory.id', ondelete="CASCADE"), primary_key=True),
            Column('group_id', Integer, ForeignKey('tblGroup.id'), primary_key=True),
            Column('privilege', Integer, nullable=False, primary_key=True, autoincrement=False),
            Column('group_has_privilege', SmallInteger, default=0),
            Column('license_has_privilege', SmallInteger, default=0),
            Column('domain_has_privilege', SmallInteger, default=0),
            Column('world_has_privilege', SmallInteger, default=0),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        tblIQuestItemPoolItem = IQTable('tblIQuestItemPoolItem', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('name', Unicode(255), nullable=True),
            Column('polymorphic_type', String(255), nullable=False),
            Column('pool_category_id', Integer, ForeignKey('tblIQuestItemPoolCategory.id'), nullable=False),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        tblIQuestItemPoolItemHtml = IQTable('tblIQuestItemPoolItemHtml', metadata,
            Column('pool_item_id', Integer, ForeignKey('tblIQuestItemPoolItem.id', ondelete="CASCADE"), primary_key=True),
            Column('is_wiki', SmallInteger, nullable=False, default=1),
        )

        tblIQuestItemPoolItemHtmlLocale = IQTable('tblIQuestItemPoolItemHtmlLocale', metadata,
            Column('pool_item_id', Integer, ForeignKey('tblIQuestItemPoolItemHtml.pool_item_id', ondelete='CASCADE'), primary_key=True),
            Column('locale_id', Integer, ForeignKey('tblLocale.id'), primary_key=True),
            Column('html', UnicodeText, nullable=False),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        tblIQuestItemPoolItemQA = IQTable('tblIQuestItemPoolItemQA', metadata,
            Column('pool_item_id', Integer, ForeignKey('tblIQuestItemPoolItem.id', ondelete="CASCADE"), primary_key=True),

            # see constants in SurveyItemQA.TYPE_*
            Column('type', Integer, nullable=False),
            Column('object_name', String(170), nullable=False),
            Column('layout', Integer, nullable=True),

            Column('required', SmallInteger, nullable=False, default=False),

            Column('trailing_questions', SmallInteger, nullable=False),
            Column('show_title', SmallInteger, nullable=False),
            Column('sum_questions', SmallInteger, nullable=False, default=0),
            Column('sum_answers', SmallInteger, nullable=False, default=0),
            Column('always_show_question_labels', SmallInteger, nullable=False, default=0),
            Column('always_show_answer_labels', SmallInteger, nullable=False, default=0),

            Column('validate_market_share', SmallInteger, nullable=False, default=0),

            # see constants in SurveyItemQA.ALIGN_*
            Column('questions_align', Integer, nullable=False),
            Column('answers_align', Integer, nullable=False)
        )

        tblIQuestItemPoolItemQALocale = IQTable('tblIQuestItemPoolItemQALocale', metadata,
            Column('pool_item_id', Integer, ForeignKey('tblIQuestItemPoolItemQA.pool_item_id', ondelete='CASCADE'), primary_key=True),
            Column('locale_id', Integer, ForeignKey('tblLocale.id'), primary_key=True),
            Column('label', UnicodeText, nullable=False),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        # writeme
        tblIQuestItemPoolItemQALayoutAttribute = IQTable('tblIQuestItemPoolItemQALayoutAttribute', metadata,
            Column('pool_item_id', Integer, ForeignKey('tblIQuestItemPoolItemQA.pool_item_id', ondelete='CASCADE'), primary_key=True),
            Column('key_id', Integer, primary_key=True, autoincrement=False),
            Column('value', UnicodeText, nullable=False),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        # Questions or Answers
        tblIQuestItemPoolItemQAElement = IQTable('tblIQuestItemPoolItemQAElement', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('pool_item_id', Integer, ForeignKey('tblIQuestItemPoolItemQA.pool_item_id', ondelete='CASCADE')),
            Column('file_id', Integer, ForeignKey('tblFile.id'), nullable=True),
            Column('is_question', SmallInteger, nullable=False, default=0),
            Column('position', Integer, nullable=False),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        tblIQuestItemPoolItemQAElementLocale = IQTable('tblIQuestItemPoolItemQAElementLocale', metadata,
            Column('pool_item_element_id', Integer, ForeignKey('tblIQuestItemPoolItemQAElement.id', ondelete='CASCADE'), primary_key=True),
            Column('locale_id', Integer, ForeignKey('tblLocale.id'), primary_key=True),
            Column('label', UnicodeText, nullable=False),
            Column('help', UnicodeText, nullable=True),
            Column('trailing_label', UnicodeText, nullable=True),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        tblIQuestItemPoolItemQAFilter = IQTable('tblIQuestItemPoolItemQAFilter', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('filter_id', Integer, ForeignKey('tblIQuestQAFilter.id', ondelete='RESTRICT'), nullable=False),
            Column('pool_item_id', Integer, ForeignKey('tblIQuestItemPoolItemQA.pool_item_id', ondelete='CASCADE'), nullable=False),

            Column('question_id', Integer, ForeignKey('tblIQuestItemPoolItemQAElement.id', ondelete='CASCADE'), nullable=True),
            Column('answer_id', Integer, ForeignKey('tblIQuestItemPoolItemQAElement.id', ondelete='CASCADE'), nullable=True),

            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now),
            UniqueConstraint('pool_item_id', 'filter_id', 'question_id', 'answer_id')
        )

        tblIQuestItemPoolItemQAFilterAttribute = IQTable('tblIQuestItemPoolItemQAFilterAttribute', metadata,
            Column('qa_filter_id', Integer, ForeignKey('tblIQuestItemPoolItemQAFilter.id', ondelete='CASCADE'), primary_key=True),
            # we make this a string to provide as keyworded argument to the Filter object.
            # Checks are made if the attribute is allowed.
            Column('attribute', String(64), primary_key=True),
            Column('value', UnicodeText, nullable=True),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        # ----- IQUEST SURVEY SPECIFIC TABLES -----
        tblSurvey = IQTable('tblSurvey', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),

            # filename of the survey, this should become unique per customer or something
            Column('name', String(255), nullable=False),

            # when a respondent opens a survey the survey can be presented in his locale
            # if there are no translations it can fallback to the fallback_locale
            Column('fallback_locale_id', Integer, ForeignKey('tblLocale.id'), nullable=False),

            # theme currently gets mapped in the state controller against a dict to get its css_class and name
            Column('theme_id', Integer, ForeignKey('tblIQuestTheme.id'), nullable=False),
            # flag set when a change was made for an online or finished survey (so related to translations only).
            # based on this, locale aggregators will populate agg tables.
            Column('stale', SmallInteger, nullable=False, default=0),

            Column('deleted', SmallInteger, nullable=False, default=0),
            Column('archived', SmallInteger, nullable=False, default=0),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
            )

        # This is the link between a group of accounts and the survey, this will be used for access rights.
        tblSurveyGroup = IQTable('tblSurveyGroup', metadata,
            Column('survey_id', Integer, ForeignKey('tblSurvey.id', ondelete="CASCADE"), primary_key=True),
            Column('group_id', Integer, ForeignKey('tblGroup.id'), primary_key=True),
            Column('privilege', Integer, nullable=False, primary_key=True, autoincrement=False),
            Column('group_has_privilege', SmallInteger, default=0),
            Column('license_has_privilege', SmallInteger, default=0),
            Column('domain_has_privilege', SmallInteger, default=0),
            Column('world_has_privilege', SmallInteger, default=0),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
            )

        # Holds the available Locales for Survey
        tblSurveyLocale = IQTable('tblSurveyLocale', metadata,
            Column('survey_id', Integer, ForeignKey('tblSurvey.id', ondelete='CASCADE'), primary_key=True),
            Column('locale_id', Integer, ForeignKey('tblLocale.id'), primary_key=True),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
            )

        # variable set holds survey variables
        tblSurveyVariableSet = IQTable('tblSurveyVariableSet', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('survey_id', Integer, ForeignKey('tblSurvey.id'), nullable=False),
            Column('name', String(255), nullable=False),

            Column('deleted', SmallInteger, nullable=False, default=0),
            Column('archived', SmallInteger, nullable=False, default=0),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now),
        )

        # This is the link between a group of accounts and the publication, this will be used for access rights.
        tblSurveyVariableSetGroup = IQTable('tblSurveyVariableSetGroup', metadata,
            Column('variable_set_id', Integer, ForeignKey('tblSurveyVariableSet.id', ondelete='CASCADE'), primary_key=True),
            Column('group_id', Integer, ForeignKey('tblGroup.id'), primary_key=True),
            Column('privilege', Integer, nullable=False, primary_key=True, autoincrement=False),
            Column('group_has_privilege', SmallInteger, default=0),
            Column('license_has_privilege', SmallInteger, default=0),
            Column('domain_has_privilege', SmallInteger, default=0),
            Column('world_has_privilege', SmallInteger, default=0),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
            )

        # variables for a set
        tblSurveyVariableLocale = IQTable('tblSurveyVariableLocale', metadata,
            Column('variable_set_id', Integer, ForeignKey('tblSurveyVariableSet.id', ondelete='CASCADE'), primary_key=True),
            Column('locale_id', Integer, ForeignKey('tblLocale.id'), primary_key=True),
            Column('variable', String(64), nullable=False, primary_key=True), # like 'product_name'
            Column('text', String(255), nullable=False), # like 'apples'
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        # A Survey can have multiple SurveyPublications. When the Survey is SCHEDULED these periods define in which the
        # survey will be available online.
        # These SurveyPublications should be seen as instances of a Survey. This way a client can publish his survey
        # multiple times and do reporting against one or more of them.
        tblSurveyPublication = IQTable('tblSurveyPublication', metadata,
            Column('publication_id', Integer, ForeignKey('tblComposePublication.id'), primary_key=True),
            Column('survey_id', Integer, ForeignKey('tblSurvey.id'), nullable=False),
            Column('variable_set_id', Integer, ForeignKey('tblSurveyVariableSet.id')),
            Column('branch_cache', UnicodeText, nullable=True),
            Column('minimum_respondents', SmallInteger, nullable=True),
            Column('maximum_market_share', SmallInteger, nullable=True),
            Column('multi_session_setting', SmallInteger, nullable=False, default=SurveyPublication.MULTI_SESSION_FREE_FOR_ALL),
            Column('minimum_responses_for_respondent', SmallInteger, nullable=False, default=1),
            Column('number_of_participants', Integer, nullable=False, default=0),
            Column('number_of_respondents', Integer, nullable=False, default=0),
            Column('show_end_message', SmallInteger, nullable=False, default=1),
            )

        # Blocks go into a surveey
        tblSurveyBlock = IQTable('tblSurveyBlock', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('survey_id', Integer, ForeignKey('tblSurvey.id', ondelete='CASCADE'), nullable=False),
            Column('position', Integer, nullable=False),
            Column('page_end', SmallInteger, nullable=False, default=False),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
            )

        # Columns go in blocks
        tblSurveyBlockColumn = IQTable('tblSurveyBlockColumn', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('block_id', Integer, ForeignKey('tblSurveyBlock.id', ondelete='CASCADE'), nullable=False),
            Column('position', Integer, nullable=False),
            Column('width_percentage', SmallInteger, nullable=True),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
            )

        # Items go in columns. Items are abstract and need to be inherited (like Html or QA)
        tblSurveyItem = IQTable('tblSurveyItem', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('pool_item_id', Integer, ForeignKey('tblIQuestItemPoolItem.id'), nullable=True),
            Column('column_id', Integer, ForeignKey('tblSurveyBlockColumn.id', ondelete='CASCADE'), nullable=False),
            Column('polymorphic_type', String(255), nullable=False),
            Column('position', Integer, nullable=False),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
            )

        # Html item contains just html text
        tblSurveyItemHtml = IQTable('tblSurveyItemHtml', metadata,
            Column('item_id', Integer, ForeignKey('tblSurveyItem.id', ondelete='CASCADE'), primary_key=True),
            Column('is_wiki', SmallInteger, nullable=False, default=1),
        )

        # Locale texts for HtmlItem
        tblSurveyItemHtmlLocale = IQTable('tblSurveyItemHtmlLocale', metadata,
            Column('item_html_id', Integer, ForeignKey('tblSurveyItemHtml.item_id', ondelete='CASCADE'), primary_key=True),
            Column('locale_id', Integer, ForeignKey('tblLocale.id'), primary_key=True),
            Column('html', UnicodeText, nullable=False),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        # ItemQA is a container for Questions and Answers
        tblSurveyItemQA = IQTable('tblSurveyItemQA', metadata,
            Column('item_id', Integer, ForeignKey('tblSurveyItem.id', ondelete='CASCADE'), primary_key=True),

            # see constants in SurveyItemQA.TYPE_*
            Column('type', Integer, nullable=False),
            Column('object_name', String(170), nullable=False),
            Column('layout', Integer, nullable=True),

            Column('required', SmallInteger, nullable=False, default=False),

            Column('trailing_questions', SmallInteger, nullable=False),
            Column('show_title', SmallInteger, nullable=False),
            Column('sum_questions', SmallInteger, nullable=False, default=0),
            Column('sum_answers', SmallInteger, nullable=False, default=0),
            Column('always_show_question_labels', SmallInteger, nullable=False, default=0),
            Column('always_show_answer_labels', SmallInteger, nullable=False, default=0),

            Column('validate_market_share', SmallInteger, nullable=False, default=0),

            # see constants in SurveyItemQA.ALIGN_*
            Column('questions_align', Integer, nullable=False),
            Column('answers_align', Integer, nullable=False)
        )

        # writeme
        tblSurveyItemQALayoutAttribute = IQTable('tblSurveyItemQALayoutAttribute', metadata,
            Column('item_id', Integer, ForeignKey('tblSurveyItemQA.item_id', ondelete='CASCADE'), primary_key=True),
            Column('key_id', Integer, primary_key=True, autoincrement=False),
            Column('value', UnicodeText, nullable=False),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        # ItemQA titles
        tblSurveyItemQALocale = IQTable('tblSurveyItemQALocale', metadata,
            Column('item_id', Integer, ForeignKey('tblSurveyItemQA.item_id', ondelete='CASCADE'), primary_key=True),
            Column('locale_id', Integer, ForeignKey('tblLocale.id'), primary_key=True),
            Column('label', UnicodeText, nullable=False),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
            )

        # Questions or Answers
        tblSurveyItemQAElement = IQTable('tblSurveyItemQAElement', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('pool_element_id', ForeignKey('tblIQuestItemPoolItemQAElement.id'), nullable=True),
            Column('item_id', Integer, ForeignKey('tblSurveyItemQA.item_id', ondelete='CASCADE')),
            Column('file_id', Integer, ForeignKey('tblFile.id'), nullable=True),
            Column('is_question', SmallInteger, nullable=False, default=0),
            Column('position', Integer, nullable=False),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        tblSurveyItemQAElementLocale = IQTable('tblSurveyItemQAElementLocale', metadata,
            Column('item_element_id', Integer, ForeignKey('tblSurveyItemQAElement.id', ondelete='CASCADE'), primary_key=True),
            Column('locale_id', Integer, ForeignKey('tblLocale.id'), primary_key=True),
            Column('label', UnicodeText, nullable=False),
            Column('help', UnicodeText, nullable=True),
            Column('trailing_label', UnicodeText, nullable=True),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        tblSurveyItemQAWeightFactor = IQTable('tblSurveyItemQAWeightFactor', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('item_id', Integer, ForeignKey('tblSurveyItemQA.item_id', ondelete='CASCADE'), nullable=False),
            Column('question_id', Integer, ForeignKey('tblSurveyItemQAElement.id', ondelete='CASCADE'), nullable=True),
            Column('answer_id', Integer, ForeignKey('tblSurveyItemQAElement.id', ondelete='CASCADE'), nullable=True),
            Column('weight', Integer, default=1, nullable=False),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now),
            UniqueConstraint('item_id', 'question_id', 'answer_id')
        )

        tblSurveyItemQAFilter = IQTable('tblSurveyItemQAFilter', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('filter_id', Integer, ForeignKey('tblIQuestQAFilter.id', ondelete='RESTRICT'), nullable=False),
            Column('item_id', Integer, ForeignKey('tblSurveyItemQA.item_id', ondelete='CASCADE'), nullable=False),
            Column('question_id', Integer, ForeignKey('tblSurveyItemQAElement.id', ondelete='CASCADE'), nullable=True),
            Column('answer_id', Integer, ForeignKey('tblSurveyItemQAElement.id', ondelete='CASCADE'), nullable=True),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now),
            UniqueConstraint('item_id', 'filter_id', 'question_id', 'answer_id')
        )

        tblSurveyItemQAFilterAttribute = IQTable('tblSurveyItemQAFilterAttribute', metadata,
            Column('qa_filter_id', Integer, ForeignKey('tblSurveyItemQAFilter.id', ondelete='CASCADE'), primary_key=True),
            # we make this a string to provide as keyworded argument to the Filter object.
            # Checks are made if the attribute is allowed.
            Column('attribute', String(64), primary_key=True),
            Column('value', UnicodeText, nullable=True),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        tblSurveyItemQABranch = IQTable('tblSurveyItemQABranch', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('item_id', Integer, ForeignKey('tblSurveyItemQA.item_id', ondelete='CASCADE'), nullable=False),
            Column('question_id', Integer, ForeignKey('tblSurveyItemQAElement.id', ondelete='CASCADE'), nullable=True),
            Column('answer_id', Integer, ForeignKey('tblSurveyItemQAElement.id', ondelete='CASCADE'), nullable=True),
            Column('branch_block_id', Integer, ForeignKey('tblSurveyBlock.id', ondelete='CASCADE'), nullable=False),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now),
            UniqueConstraint('item_id', 'question_id', 'answer_id')
        )

        # theme, filters and mail tables
        tables['tblIQuestTheme'] = tblIQuestTheme
        tables['tblIQuestThemeGroup'] = tblIQuestThemeGroup
        tables['tblIQuestMail'] = tblIQuestMail
        tables['tblIQuestQAFilter'] = tblIQuestQAFilter
        tables['tblIQuestQAFilterGroup'] = tblIQuestQAFilterGroup

        # Item pool tables
        tables['tblIQuestItemPoolCategory'] = tblIQuestItemPoolCategory
        tables['tblIQuestItemPoolCategoryGroup'] = tblIQuestItemPoolCategoryGroup
        tables['tblIQuestItemPoolItem'] = tblIQuestItemPoolItem
        tables['tblIQuestItemPoolItemHtml'] = tblIQuestItemPoolItemHtml
        tables['tblIQuestItemPoolItemHtmlLocale'] = tblIQuestItemPoolItemHtmlLocale
        tables['tblIQuestItemPoolItemQA'] = tblIQuestItemPoolItemQA
        tables['tblIQuestItemPoolItemQALocale'] = tblIQuestItemPoolItemQALocale
        tables['tblIQuestItemPoolItemQALayoutAttribute'] = tblIQuestItemPoolItemQALayoutAttribute
        tables['tblIQuestItemPoolItemQAElement'] = tblIQuestItemPoolItemQAElement
        tables['tblIQuestItemPoolItemQAElementLocale'] = tblIQuestItemPoolItemQAElementLocale
        tables['tblIQuestItemPoolItemQAFilter'] = tblIQuestItemPoolItemQAFilter
        tables['tblIQuestItemPoolItemQAFilterAttribute'] = tblIQuestItemPoolItemQAFilterAttribute

        # survey definition tables
        tables['tblSurvey'] = tblSurvey
        tables['tblSurveyVariableSet'] = tblSurveyVariableSet
        tables['tblSurveyVariableLocale'] = tblSurveyVariableLocale
        tables['tblSurveyVariableSetGroup'] = tblSurveyVariableSetGroup
        tables['tblSurveyPublication'] = tblSurveyPublication
        tables['tblSurveyGroup'] = tblSurveyGroup
        tables['tblSurveyLocale'] = tblSurveyLocale
        tables['tblSurveyBlock'] = tblSurveyBlock
        tables['tblSurveyBlockColumn'] = tblSurveyBlockColumn
        tables['tblSurveyItem'] = tblSurveyItem
        tables['tblSurveyItemHtml'] = tblSurveyItemHtml
        tables['tblSurveyItemHtmlLocale'] = tblSurveyItemHtmlLocale
        tables['tblSurveyItemQA'] = tblSurveyItemQA
        tables['tblSurveyItemQALayoutAttribute'] = tblSurveyItemQALayoutAttribute
        tables['tblSurveyItemQALocale'] = tblSurveyItemQALocale
        tables['tblSurveyItemQAElement'] = tblSurveyItemQAElement
        tables['tblSurveyItemQAElementLocale'] = tblSurveyItemQAElementLocale
        tables['tblSurveyItemQAWeightFactor'] = tblSurveyItemQAWeightFactor
        tables['tblSurveyItemQAFilter'] = tblSurveyItemQAFilter
        tables['tblSurveyItemQAFilterAttribute'] = tblSurveyItemQAFilterAttribute
        tables['tblSurveyItemQABranch'] = tblSurveyItemQABranch

        __tables_defined = True

def do_mappings():
    """
    This function is called by Django middleware when needed.
    """
    global __mapping_defined
    if not __mapping_defined:
        from sqlalchemy.orm import mapper, relation
        from iqportal.iquest.core import models as core_models
        from basesoft.web.contrib.compose import models as compose_models
        from basesoft.web.contrib.i18n import models as i18n_models
        from basesoft.web.contrib.mailing import models as mailing_models
        from basesoft.web.contrib.mailing.sa_datamapper import get_tables as mail_tables
        from basesoft.web.contrib.file import models as file_models

        # ----- GENERAL IQUEST TABLES -----
        mapper(core_models.IQuestMail, tables['tblIQuestMail'],
            properties = {
                'fallback_locale': relation(i18n_models.Locale),
                'locales': relation(mailing_models.MailLocale, order_by=mail_tables()['tblMailLocale'].c.locale_id, passive_deletes=True)
            },
            polymorphic_identity='IQuestMail',
            inherits=mailing_models.Mail
        )

        mapper(core_models.IQuestTheme, tables['tblIQuestTheme'],
            extension = PrivilegeObjectMapper(),
            properties = {
                'groups': relation(core_models.IQuestThemeGroup, backref='theme', cascade="all", passive_deletes=True)
        })
        mapper(core_models.IQuestThemeGroup, tables['tblIQuestThemeGroup'], properties = {
            'group': relation(license_models.Group)
        })

        mapper(core_models.IQuestQAFilter, tables['tblIQuestQAFilter'],
            extension = PrivilegeObjectMapper(),
            properties = {
                'extended_filter': relation(core_models.IQuestQAFilter, passive_deletes=True),
                'groups': relation(core_models.IQuestQAFilterGroup, backref='filter', cascade="all", passive_deletes=True)
        })

        mapper(core_models.IQuestQAFilterGroup, tables['tblIQuestQAFilterGroup'], properties = {
            'group': relation(license_models.Group)
        })

        # ----- IQUEST POOL TABLES -----
        mapper(core_models.IQuestItemPoolCategory, tables['tblIQuestItemPoolCategory'],
            extension = PrivilegeObjectMapper(),
            properties = {
                'groups': relation(core_models.IQuestItemPoolCategoryGroup, backref='pool_category', cascade="all", passive_deletes=True)
        })

        mapper(core_models.IQuestItemPoolCategoryGroup, tables['tblIQuestItemPoolCategoryGroup'],
            properties = {
                'group': relation(license_models.Group)
        })

        mapper(core_models.IQuestItemPoolItem, tables['tblIQuestItemPoolItem'],
            properties = {
                'pool_category': relation(core_models.IQuestItemPoolCategory)
            },
            polymorphic_on=tables['tblIQuestItemPoolItem'].c.polymorphic_type,
            polymorphic_identity='IQuestItemPoolItem'
        )

        mapper(core_models.IQuestItemPoolItemHtml, tables['tblIQuestItemPoolItemHtml'],
            properties = {
                'locales': relation(
                    core_models.IQuestItemPoolItemHtmlLocale,
                    order_by=tables['tblIQuestItemPoolItemHtmlLocale'].c.locale_id,
                    passive_deletes=True
                ),
            },
            polymorphic_identity='IQuestItemPoolItemHtml',
            inherits=core_models.IQuestItemPoolItem
        )

        mapper(core_models.IQuestItemPoolItemHtmlLocale, tables['tblIQuestItemPoolItemHtmlLocale'],
            properties = {
                'pool_item_html': relation(core_models.IQuestItemPoolItemHtml),
                'locale': relation(i18n_models.Locale)
            }
        )

        mapper(core_models.IQuestItemPoolItemQA, tables['tblIQuestItemPoolItemQA'],
            properties = {
                'locales': relation(core_models.IQuestItemPoolItemQALocale, passive_deletes=True),
                'layout_attributes': relation(core_models.IQuestItemPoolItemQALayoutAttribute, passive_deletes=True),
            },
            polymorphic_identity='IQuestItemPoolItemQA',
            inherits=core_models.IQuestItemPoolItem
        )

        mapper(core_models.IQuestItemPoolItemQALocale, tables['tblIQuestItemPoolItemQALocale'],
            properties = {
                'pool_item': relation(core_models.IQuestItemPoolItemQA),
                'locale': relation(i18n_models.Locale)
            }
        )

        mapper(core_models.IQuestItemPoolItemQALayoutAttribute, tables['tblIQuestItemPoolItemQALayoutAttribute'],
            properties = {
                'pool_item': relation(core_models.IQuestItemPoolItemQA),
            }
        )

        mapper(core_models.IQuestItemPoolItemQAElement, tables['tblIQuestItemPoolItemQAElement'],
            properties = {
                'pool_item': relation(core_models.IQuestItemPoolItemQA),
                'file': relation(file_models.File),
                'locales': relation(core_models.IQuestItemPoolItemQAElementLocale, passive_deletes=True)
            },
        )

        mapper(core_models.IQuestItemPoolItemQAElementLocale, tables['tblIQuestItemPoolItemQAElementLocale'],
            properties = {
                'element': relation(core_models.IQuestItemPoolItemQAElement),
                'locale': relation(i18n_models.Locale)
            }
        )

        mapper(core_models.IQuestItemPoolItemQAFilter, tables['tblIQuestItemPoolItemQAFilter'], properties = {
            'filter': relation(core_models.IQuestQAFilter),
            'pool_item': relation(core_models.IQuestItemPoolItemQA),
            'question': relation(core_models.IQuestItemPoolItemQAElement,
                                 primaryjoin=tables['tblIQuestItemPoolItemQAFilter'].c.question_id==tables['tblIQuestItemPoolItemQAElement'].c.id),
            'answer': relation(core_models.IQuestItemPoolItemQAElement,
                               primaryjoin=tables['tblIQuestItemPoolItemQAFilter'].c.answer_id==tables['tblIQuestItemPoolItemQAElement'].c.id),
            'attributes': relation(core_models.IQuestItemPoolItemQAFilterAttribute, cascade="delete, delete-orphan")
        })

        mapper(core_models.IQuestItemPoolItemQAFilterAttribute, tables['tblIQuestItemPoolItemQAFilterAttribute'], properties = {
            'qa_filter' : relation(core_models.IQuestItemPoolItemQAFilter)
        })


        # ----- IQUEST SURVEY SPECIFIC TABLES -----
        mapper(core_models.Survey, tables['tblSurvey'],
            extension = PrivilegeObjectMapper(),
            properties = {
                'theme': relation(core_models.IQuestTheme),
                'fallback_locale': relation(i18n_models.Locale),
                'available_locales': relation(Locale, secondary=tables['tblSurveyLocale']),
                'blocks': relation(core_models.SurveyBlock, order_by=tables['tblSurveyBlock'].c.position),
                'variable_sets': relation(core_models.SurveyVariableSet, order_by=tables['tblSurveyVariableSet'].c.name),
                'groups': relation(core_models.SurveyGroup, backref='survey', cascade="all", passive_deletes=True)
        })

        mapper(core_models.SurveyGroup, tables['tblSurveyGroup'], properties = {
            'group': relation(license_models.Group)
        })

        mapper(core_models.SurveyVariableSet, tables['tblSurveyVariableSet'],
            extension = PrivilegeObjectMapper(),
            properties = {
                'survey': relation(core_models.Survey),
                'variables': relation(core_models.SurveyVariableLocale, passive_deletes=True),
                'groups': relation(core_models.SurveyVariableSetGroup, backref='variable_set', cascade="all", passive_deletes=True),
        })

        mapper(core_models.SurveyVariableSetGroup, tables['tblSurveyVariableSetGroup'], properties = {
            'group': relation(license_models.Group)
        })

        mapper(core_models.SurveyVariableLocale, tables['tblSurveyVariableLocale'], properties = {
            'variable_set': relation(core_models.SurveyVariableSet),
            'locale': relation(i18n_models.Locale)
        })

        mapper(core_models.SurveyPublication, tables['tblSurveyPublication'],
            properties = {
                'survey': relation(core_models.Survey),
                'variable_set': relation(core_models.SurveyVariableSet)
                },
            polymorphic_identity='SurveyPublication',
            inherits=compose_models.ComposePublication
        )

        mapper(core_models.SurveyLocale, tables['tblSurveyLocale'], properties = {
            'survey': relation(core_models.Survey),
            'locale': relation(i18n_models.Locale)
        })

        mapper(core_models.SurveyBlock, tables['tblSurveyBlock'], properties = {
            'survey': relation(core_models.Survey),
            'columns': relation(core_models.SurveyBlockColumn, passive_deletes=True, order_by=tables['tblSurveyBlockColumn'].c.position),
            'branches': relation(core_models.SurveyItemQABranch, passive_deletes=True)
        })

        mapper(core_models.SurveyBlockColumn, tables['tblSurveyBlockColumn'], properties = {
            'block': relation(core_models.SurveyBlock)
        })

        mapper(core_models.SurveyItem, tables['tblSurveyItem'],
            properties = {
                'column': relation(core_models.SurveyBlockColumn),
                'pool_item': relation(core_models.IQuestItemPoolItem)
            },
            polymorphic_on=tables['tblSurveyItem'].c.polymorphic_type,
            polymorphic_identity='Item'
        )

        mapper(core_models.SurveyItemHtml, tables['tblSurveyItemHtml'],
            properties = {
                'locales': relation(
                    core_models.SurveyItemHtmlLocale,
                    order_by=tables['tblSurveyItemHtmlLocale'].c.locale_id,
                    passive_deletes=True
                ),
            },
            polymorphic_identity='SurveyItemHtml',
            inherits=core_models.SurveyItem
        )

        mapper(core_models.SurveyItemHtmlLocale, tables['tblSurveyItemHtmlLocale'],
            properties = {
                'item_html': relation(core_models.SurveyItemHtml),
                'locale': relation(i18n_models.Locale)
            }
        )

        mapper(core_models.SurveyItemQA, tables['tblSurveyItemQA'],
            properties = {
                'locales': relation(core_models.SurveyItemQALocale, order_by=tables['tblSurveyItemQALocale'].c.locale_id, passive_deletes=True),
                'layout_attributes': relation(core_models.SurveyItemQALayoutAttribute, passive_deletes=True),
            },
            polymorphic_identity='SurveyItemQA',
            inherits=core_models.SurveyItem
        )

        mapper(core_models.SurveyItemQALayoutAttribute, tables['tblSurveyItemQALayoutAttribute'],
            properties = {
                'item': relation(core_models.SurveyItemQA),
            }
        )

        mapper(core_models.SurveyItemQALocale, tables['tblSurveyItemQALocale'],
            properties = {
                'item': relation(core_models.SurveyItemQA),
                'locale': relation(i18n_models.Locale)
            }
        )

        mapper(core_models.SurveyItemQAElement, tables['tblSurveyItemQAElement'],
            properties = {
                'item': relation(core_models.SurveyItemQA),
                'file': relation(file_models.File),
                'pool_element': relation(core_models.IQuestItemPoolItemQAElement),
                'locales': relation(core_models.SurveyItemQAElementLocale, passive_deletes=True)
            },
        )

        mapper(core_models.SurveyItemQAElementLocale, tables['tblSurveyItemQAElementLocale'],
            properties = {
                'element': relation(core_models.SurveyItemQAElement),
                'locale': relation(i18n_models.Locale)
            }
        )

        mapper(core_models.SurveyItemQAWeightFactor, tables['tblSurveyItemQAWeightFactor'],
            properties = {
                'item': relation(core_models.SurveyItemQA),
                'question': relation(core_models.SurveyItemQAElement,
                                     primaryjoin=tables['tblSurveyItemQAWeightFactor'].c.question_id==tables['tblSurveyItemQAElement'].c.id),
                'answer': relation(core_models.SurveyItemQAElement,
                                   primaryjoin=tables['tblSurveyItemQAWeightFactor'].c.answer_id==tables['tblSurveyItemQAElement'].c.id),
            }
        )

        mapper(core_models.SurveyItemQAFilter, tables['tblSurveyItemQAFilter'], properties = {
            'filter': relation(core_models.IQuestQAFilter),
            'item': relation(core_models.SurveyItemQA),
            'question': relation(core_models.SurveyItemQAElement,
                                 primaryjoin=tables['tblSurveyItemQAFilter'].c.question_id==tables['tblSurveyItemQAElement'].c.id),
            'answer': relation(core_models.SurveyItemQAElement,
                               primaryjoin=tables['tblSurveyItemQAFilter'].c.answer_id==tables['tblSurveyItemQAElement'].c.id),
            'attributes': relation(core_models.SurveyItemQAFilterAttribute, cascade="delete, delete-orphan")
        })

        mapper(core_models.SurveyItemQAFilterAttribute, tables['tblSurveyItemQAFilterAttribute'], properties = {
            'qa_filter' : relation(core_models.SurveyItemQAFilter)
        })

        mapper(core_models.SurveyItemQABranch, tables['tblSurveyItemQABranch'], properties = {
            'item': relation(core_models.SurveyItemQA),
            'question': relation(core_models.SurveyItemQAElement,
                                 primaryjoin=tables['tblSurveyItemQABranch'].c.question_id==tables['tblSurveyItemQAElement'].c.id),
            'answer': relation(core_models.SurveyItemQAElement,
                               primaryjoin=tables['tblSurveyItemQABranch'].c.answer_id==tables['tblSurveyItemQAElement'].c.id),
            'branch_block': relation(core_models.SurveyBlock)
        })

        __mapping_defined = True


def fill_data(conn):
    pass

def app_permissions_category():
    """
    Returns a category for all permissions defined for this app
    """
    return license_models.PermissionCategory(u'IQuest')

def app_permissions():
    """
    The permissions used within this application. Can be sync'd with the
    `iqsyncpermissions` command.
    """
    return [
        license_models.Permission(u'create_survey', u"""
            This gives the right to create a survey.
            """, _("Create surveys")),
        license_models.Permission(u'view_survey', u"""
            This gives the right to view a survey.
            """, _("View surveys")),
        license_models.Permission(u'update_survey', u"""
            This gives the right to update a survey.
            """, _("Update surveys")),
        license_models.Permission(u'delete_survey', u"""
            This gives the right to delete a survey.
            """, _("Delete surveys")),

        license_models.Permission(u'update_published_survey', u"""
            This gives the right to update published surveys.
            """, _("Update published surveys")),
        license_models.Permission(u'survey_publication_multi_sess_options', u"""
            This gives the right to set multi-session options for survey publications.
            """, _("Has survey publication multi session options")),


        # make a separate permission for the NPS QA
        license_models.Permission(u'iquest_nps_access', u"""
            This gives the right to a NPS QA.
            """, _("NPS QA Access")),

        # item pool category
        license_models.Permission(u'create_iquest_pool_category', u"""
            This gives the right to create an iquest pool category.
            """, _("Create iquest pool categories")),
        license_models.Permission(u'view_iquest_pool_category', u"""
            This gives the right to view an iquest pool category.
            """, _("View iquest pool categories")),
        license_models.Permission(u'update_iquest_pool_category', u"""
            This gives the right to update an iquest pool category.
            """, _("Update iquest pool categories")),
        license_models.Permission(u'delete_iquest_pool_category', u"""
            This gives the right to delete an iquest pool category.
            """, _("Delete iquest pool categories")),

        # variable set permissions
        license_models.Permission(u"create_survey_variable_set", u"""
            This gives the right to create survey variable sets.
            """, _("Create survey variable sets")),
        license_models.Permission(u"view_survey_variable_set", u"""
            This gives the right to view survey variable sets.
            """, _("View survey variable sets")),
        license_models.Permission(u"update_survey_variable_set", u"""
            This gives the right to update existing survey variable sets.
            """, _("Update survey variable sets")),
        license_models.Permission(u"delete_survey_variable_set", u"""
            This gives the right to delete survey variable sets.
            """, _("Delete survey variable sets")),

        # survey publication perms
        license_models.Permission(u'create_survey_publication', u"""
            This gives the right to create a survey publication.
            """, _("Create survey publications")),
        license_models.Permission(u'view_survey_publication', u"""
            This gives the right to view a survey publication.
            """, _("View survey publications")),
        license_models.Permission(u'update_survey_publication', u"""
            This gives the right to update a survey publication.
            """, _("Update survey publications")),
        license_models.Permission(u'delete_survey_publication', u"""
            This gives the right to delete a survey publication.
            """, _("Delete survey publications")),
        license_models.Permission(u'schedule_survey_publication', u"""
            This gives the right to schedule a survey publication.
            """, _("Schedule survey publications")),

        # for survey mail stuff see mailing applicationR
        license_models.Permission(u"create_iquest_mail", u"""
            This gives the right to create new iquest mails.
            """, _("Create IQuest mails")),
        license_models.Permission(u"view_iquest_mail", u"""
            This gives the right to view iquest mails.
            """, _("View IQuest mails")),
        license_models.Permission(u"update_iquest_mail", u"""
            This gives the right to update existing iquest mails.
            """, _("Update IQuest mails")),
        license_models.Permission(u"delete_iquest_mail", u"""
            This gives the right to delete iquest mails.
            """, _("Delete IQuest mails")),

        # this permission is used when attaching a mail to a mailing (so to be sent)
        license_models.Permission(u"schedule_iquest_mail", u"""
            The right to use the iquest mail in a publication mailing.
            """, _("Schedule IQuest mails")),

        # theme
        license_models.Permission(u'view_iquest_theme', u"""
            The permission to view themes
        """, _("View themes")),

        # permission to add/update/delete weight factors
        license_models.Permission(u'manage_itemqa_weight_factor', u"""
            This gives the right to manage item QA weight factors
        """, _('Manage question/answer weight factor')),

        # permissions for working with itemQA filters - survey or pool items
        license_models.Permission(u"create_itemqa_filter", u"""
            This gives the right to create new itemqa filters.
            """, _("Create question/answer filters")),
        license_models.Permission(u"view_itemqa_filter", u"""
            This gives the right to view and use itemqa filters.
            """, _("View question/answer filters")),
        license_models.Permission(u"update_itemqa_filter", u"""
            This gives the right to update existing itemqa filters.
            """, _("Update question/answer filters")),
        license_models.Permission(u"delete_itemqa_filter", u"""
            This gives the right to delete itemqa filters.
            """, _("Delete question/answer filters")),

        # permissions for Conditional Branching
        license_models.Permission(u"create_conditional_branch", u"""
            This gives the right to create new conditional branches.
            """, _('Create conditional branches')),
        license_models.Permission(u"view_conditional_branch", u"""
            This gives the right to view conditional branches.
            """, _('View conditional branches')),
        license_models.Permission(u"update_conditional_branch", u"""
            This gives the right to update existing conditional branches.
            """, _('Update conditional branches')),
        license_models.Permission(u"delete_conditional_branch", u"""
            This gives the right to delete conditional branches.
            """, _('Create conditional branches')),

        # export
        license_models.Permission(u"iquest_export_excel", u"""
            This gives the right to export to excel.
            """, _('Export to Excel')),
        license_models.Permission(u"iquest_export_spss", u"""
            This gives the right to export to SPSS.
            """, _('Export to SPSS')),
        ]

def app_roles():
    return [{
        'name': 'IQuest (basic)',
        'required_permission': 'manage_iqportal_roles',
        'permissions': [
            'view_dashboard', 'save_dashboard',
            'view_notification', 'delete_notification',
            'view_survey', 'create_survey', 'update_survey',
            'view_survey_publication', 'create_survey_publication', 'update_survey_publication', 'schedule_survey_publication',
            'view_survey_publication_response'
        ],
        'license_configs': [],
    }, {
        'name': 'IQuest (default)',
        'required_permission': 'manage_iqportal_roles',
        'permissions': [
            'view_dashboard', 'save_dashboard',
            'view_notification', 'delete_notification',
            'file_manager', 'view_file', 'create_file', 'update_file',
            'view_survey', 'create_survey', 'update_survey',
            'iquest_export_excel',
            'view_survey_publication', 'create_survey_publication', 'update_survey_publication', 'schedule_survey_publication',
            'view_contact', 'create_contact', 'update_contact', 'archive_contact', 'import_contacts',
            'view_contact_list', 'create_contact_list', 'update_contact_list',
            'link_to_contact_list', 'unlink_from_contact_list',
            'view_mail', 'create_mail', 'update_mail', 'schedule_mail',
            'view_iquest_mail', 'create_iquest_mail', 'update_iquest_mail', 'schedule_iquest_mail',
            'view_survey_publication_response', 'update_survey_publication_response',
        ],
        'license_configs': [],
    }, {
        'name': 'IQuest (advanced)',
        'required_permission': 'manage_iqportal_roles',
        'permissions': [
            'view_dashboard', 'save_dashboard',
            'view_notification', 'delete_notification',
            'file_manager', 'view_file', 'create_file', 'update_file',
            'view_survey', 'create_survey', 'update_survey',
            'iquest_export_excel', 'iquest_export_spss', 'iquest_nps_access', 'view_iquest_theme',
            'view_iquest_pool_category', 'create_iquest_pool_category', 'update_iquest_pool_category',
            'manage_itemqa_weight_factor',
            'view_itemqa_filter', 'create_itemqa_filter', 'update_itemqa_filter', 'delete_itemqa_filter',
            'view_conditional_branch', 'create_conditional_branch', 'update_conditional_branch', 'delete_conditional_branch',
            'view_survey_publication', 'create_survey_publication', 'update_survey_publication', 'schedule_survey_publication',
            'send_separate_mail_to_invites',
            'survey_publication_multi_sess_options',
            'view_contact', 'create_contact', 'update_contact', 'archive_contact', 'import_contacts',
            'view_contact_list', 'create_contact_list', 'update_contact_list',
            'link_to_contact_list', 'unlink_from_contact_list',
            'view_mail', 'create_mail', 'update_mail', 'schedule_mail',
            'view_iquest_mail', 'create_iquest_mail', 'update_iquest_mail', 'schedule_iquest_mail',
            'view_survey_publication_response', 'create_survey_publication_response','update_survey_publication_response',
        ],
        'license_configs': [],
    }]


def app_usage_options():
    """
    The usage options used within this application. Can be sync'd with the
    `iqsyncpermissions` command.
    """
    return [
        license_models.UsageOption(u'max_number_of_surveys', u"""
            This defines the survey usage option; use this to limit on number of surveys.
            """, _("Maximum number of surveys")),
        license_models.UsageOption(u'max_number_of_survey_publications', u"""
            This defines the survey publications usage option; use this to limit on number of total publications
            (so for all surveys).
            """, _("Maximum number of publications"))
    ]

def app_configs():
    """
    The account configs available for this application. This can be sync'd with the
    `iqsyncpermissions` command.
    """
    configs = []

    config = account_models.Config()
    config.name = 'iquest.compose.survey.id'
    config.type = 'integer'
    config.default = 0
    config.description = u'Survey ID which will be loaded when getting to the compose app.'
    configs.append(config)

    config = account_models.Config()
    config.name = 'iquest.compose.palette.hidden'
    config.type = 'boolean'
    config.default = 'False'
    config.description = u'Boolean for whether or not the IQuest Palette is hidden.'
    configs.append(config)

    config = account_models.Config()
    config.name = 'iquest.compose.history.hidden'
    config.type = 'boolean'
    config.default = 'True'
    config.description = u'Boolean for whether or not the IQuest History panel is hidden.'
    configs.append(config)

    config = account_models.Config()
    config.name = 'iquest.compose.snapshots.hidden'
    config.type = 'boolean'
    config.default = 'True'
    config.description = u'Boolean for whether or not the IQuest Snapshots panel is hidden.'
    configs.append(config)

    config = account_models.Config()
    config.name = 'iquest.compose.action.hidden'
    config.type = 'boolean'
    config.default = 'True'
    config.description = u'Boolean for whether or not the IQuest Action panel is hidden.'
    configs.append(config)

    config = account_models.Config()
    config.name = 'iquest.compose.right_layout_unit.width'
    config.type = 'integer'
    config.default = '250'
    config.description = u'Integer used to determine the default width (in px) of the IQuest right compose panel.'
    configs.append(config)

    config = account_models.Config()
    config.name = 'iquest.compose.right_layout_unit.collapsed'
    config.type = 'boolean'
    config.default = 'False'
    config.description = u'Boolean used to determine if the IQuest right compose panel should be collapsed by default.'
    configs.append(config)

    config = account_models.Config()
    config.name = 'iquest.compose.top_layout_unit.height'
    config.type = 'integer'
    config.default = '63'
    config.description = u'Integer used to determine the default height (in px) of the IQuest top compose panel.'
    configs.append(config)

    config = account_models.Config()
    config.name = 'iquest.compose.top_layout_unit.collapsed'
    config.type = 'boolean'
    config.default = 'False'
    config.description = u'Boolean used to determine if the IQuest top compose panel should be collapsed by default.'
    configs.append(config)

    config = account_models.Config()
    config.name = 'iquest.compose.remove.confirmation'
    config.type = 'boolean'
    config.default = 'True'
    config.description = u'Boolean used to determine if confirmation should be requested when a module is removed from the IQuest compose.'
    configs.append(config)

    config = account_models.Config()
    config.name = 'iquest.publication.minimum_respondents'
    config.type = 'integer'
    config.default = 0
    config.description = u'The minimum number of respondents required for the result to be displayed in Filtered reports.'
    configs.append(config)

    config = account_models.Config()
    config.name = 'iquest.publication.maximum_market_share'
    config.type = 'integer'
    config.default = 101
    config.description = u'The maximum market share for a numerical value for the result to be displayed in Filtered reports.'
    configs.append(config)

    return configs

