'''
Created on 1 Dec 2011

@author: martijn
'''
import os
import csv
import hashlib
import random
import cStringIO
import codecs

from django.conf import settings

from basesoft.web.core.util.controllers import AbstractController
from iqportal.iquest.response.models import SurveyParticipant, SurveyQAResponse
from iqportal.iquest.core.models import SurveyPublication,\
    SurveyItemQAElement, Survey, SurveyBlock, SurveyBlockColumn, SurveyItem,\
    SurveyItemQA, SurveyItemQALocale, SurveyItemQAElementLocale
from basesoft.web.contrib.compose.models import ComposeInvitee
from sqlalchemy.sql.expression import select, and_
from sqlalchemy.orm import join
from basesoft.web.contrib.contact.models import Contact, Person, Company
from basesoft.util.ordered_dict import OrderedDict

from django.utils.translation import ugettext as _
from zipfile import ZipFile

class Exporter(AbstractController):
    """
    Abstract iquest exporter class that provides an interface for exporting with context
    """

    def __init__(self, publication_id, *args, **kwargs):
        AbstractController.__init__(self, *args, **kwargs)

        self._publication = self.sa_session.query(SurveyPublication).get(publication_id)
        self.assert_privileges_on_object(self._publication, SurveyPublication.PRIV_VIEW_RESPONSE)

        from iqportal.iquest.core.sa_datamapper import get_tables as survey_tables
        from iqportal.iquest.response.sa_datamapper import get_tables as response_tables
        from basesoft.web.contrib.contact.sa_datamapper import get_tables as contact_tables
        from basesoft.web.contrib.compose.sa_datamapper import get_tables as compose_tables

        # fill the public column definition in a structured way so that rpc calls can use the same fields for selection

        # TODO: implement person/company filtering if provided tru constructor #3266

        self.columns_def = OrderedDict([
            ('publication', (
                ('participant_id', response_tables()['tblSurveyParticipant'].c.id, _('Participant ID')),
                ('publication_label', compose_tables()['tblComposePublication'].c.label, _('Publication Label')),
            )),

            ('person', (
                ('person_first_name', contact_tables()['tblPerson'].c.first_name, _('First Name')),
                ('person_middle_name', contact_tables()['tblPerson'].c.middle_name, _('Middle Name')),
                ('person_last_name', contact_tables()['tblPerson'].c.last_name, _('Last Name')),
                ('person_email', contact_tables()['tblPerson'].c.email, _('Email')),
                ('person_position', contact_tables()['tblPerson'].c.position, _('Position')),
                ('person_department', contact_tables()['tblPerson'].c.department, _('Department')),
                ('person_phone', contact_tables()['tblPerson'].c.phone, _('Phone')),
                ('person_mobile_phone', contact_tables()['tblPerson'].c.mobile_phone, _('Mobile Phone')),
                ('person_fax', contact_tables()['tblPerson'].c.fax, _('Fax')),
                ('person_birth_date', contact_tables()['tblPerson'].c.birth_date, _('Birth Date')),
            )),

            ('contact', (
                ('contact_email', contact_tables()['tblContact'].c.email, _('Contact Email')),
            )),

            # company
            # todo: company has a contact person.. we want that too?
            ('company', (
                ('company_name', contact_tables()['tblCompany'].c.name, _('Company name')),
                ('company_vat_no', contact_tables()['tblCompany'].c.vat_no, _('VAT No.')),
                ('company_website', contact_tables()['tblCompany'].c.website, _('Website')),
            )),

            # response data
            ('response', (
                ('itemqa_id', survey_tables()['tblSurveyItemQAElement'].c.item_id),
                ('question_id', response_tables()['tblSurveyQAResponse'].c.question_id),
                ('answer_id', response_tables()['tblSurveyQAResponse'].c.answer_id),
                ('response_text', response_tables()['tblSurveyQAResponse'].c.response_text),
                ('response_int', response_tables()['tblSurveyQAResponse'].c.response_int),
                ('response_float', response_tables()['tblSurveyQAResponse'].c.response_float),
            ))
        ])


    def get_qa_headers(self):
        """
        returns a set of item_qa headers. Used by exporters implementation or rpc for item_qa filtering
        """

        from iqportal.iquest.core.sa_datamapper import get_tables as survey_tables

        rv = OrderedDict()

        publication = self.sa_session.query(SurveyPublication).filter(SurveyPublication.id==self._publication.id).one()
        locale_id = publication.survey.fallback_locale.id

        columns = (
            survey_tables()['tblSurveyItemQA'].c.item_id,
            survey_tables()['tblSurveyItemQALocale'].c.label,
            survey_tables()['tblSurveyItemQAElement'].c.id,
            survey_tables()['tblSurveyItemQAElement'].c.is_question,
            survey_tables()['tblSurveyItemQAElementLocale'].c.label,
            survey_tables()['tblSurveyItemQA'].c.type,
        )

        fo = join(SurveyPublication, Survey, SurveyPublication.survey_id == Survey.id) \
            .join(SurveyBlock, SurveyBlock.survey_id==Survey.id) \
            .join(SurveyBlockColumn, SurveyBlockColumn.block_id==SurveyBlock.id) \
            .join(SurveyItemQA, SurveyItemQA.column_id==SurveyBlockColumn.id) \
            .outerjoin(SurveyItemQALocale, and_(SurveyItemQALocale.item_id==SurveyItemQA.item_id, SurveyItemQALocale.locale_id==locale_id)) \
            .join(SurveyItemQAElement, SurveyItemQAElement.item_id==SurveyItemQA.item_id) \
            .outerjoin(SurveyItemQAElementLocale, and_(SurveyItemQAElementLocale.item_element_id==SurveyItemQAElement.id, SurveyItemQAElementLocale.locale_id==locale_id))

        filter_ = and_(SurveyPublication.id==self._publication.id)

        order_by = (survey_tables()['tblSurveyBlock'].c.position, survey_tables()['tblSurveyBlockColumn'].c.position, survey_tables()['tblSurveyItem'].c.position, survey_tables()['tblSurveyItemQAElement'].c.position)

        q = self.sa_session.connection().execute(
            select(
                columns = columns,
                whereclause = filter_,
                from_obj = fo,
                order_by = order_by
            )
        )

        while True:
            row = q.fetchone()
            if not row:
                break

            qa_id, qa_label, el_id, el_is_question, el_label, qa_type = row

            if not rv.has_key(qa_id):
                rv[qa_id] = {'label': qa_label, 'questions': [], 'answers': [], 'type': qa_type}

            if el_is_question:
                rv[qa_id]['questions'].append( (el_id, el_label) )
            else:
                rv[qa_id]['answers'].append( (el_id, el_label) )

        return rv


    def export(self):
        """
        queries contact info and responses for the current publication. calls write_participant() for each participant with context and info
        """

        # data query
        columns = []
        for key in self.columns_def.keys():
            columns += [ col[1] for col in self.columns_def[key] ]

        fo = join(SurveyQAResponse, SurveyParticipant, SurveyQAResponse.participant_id == SurveyParticipant.id) \
            .join(ComposeInvitee, SurveyParticipant.invitee_id==ComposeInvitee.id) \
            .join(SurveyPublication, ComposeInvitee.publication_id==SurveyPublication.id) \
            .outerjoin(Contact, ComposeInvitee.contact_id == Contact.id) \
            .outerjoin(Person, Contact.person_id == Person.id) \
            .outerjoin(Company, Contact.company_id == Company.id) \
            .outerjoin(SurveyItemQAElement, SurveyQAResponse.answer_id==SurveyItemQAElement.id)

        filter_ = and_(SurveyPublication.id==self._publication.id)

        q = self.sa_session.connection().execute(
            select(
                columns = columns,
                whereclause = filter_,
                from_obj = fo,
            )
        )

        participant_id = None
        participant_data = None
        answers = None

        itemqa_id_index = len(self.columns_def['publication']) + len(self.columns_def['person']) +len(self.columns_def['contact']) + len(self.columns_def['company'])

        while True:
            row = q.fetchone()
            if not row:
                break

            cur_participant_id = row[0]

            if participant_id != cur_participant_id:

                # write previous participant (if any)
                if participant_id is not None:
                    self.write_participant(participant_data)

                # set new participant
                participant_id = cur_participant_id
                participant_data = []
                answers = {}
                #print 'working on participant %s' % participant_id

                # fill basic information once per participant
                columns = []
                for key in ('publication', 'person', 'contact', 'company'):
                    columns += [ col[0] for col in self.columns_def[key] ]
                for i in range(len(columns)):
                    participant_data.append(row[i])

                # append answers dict that gets modified below (by reference)
                participant_data.append(answers)

            # process questions
            qa_id = row[itemqa_id_index]
            q_id = row[itemqa_id_index+1]
            a_id = row[itemqa_id_index+2]
            r_text = row[itemqa_id_index+3]
            r_int = row[itemqa_id_index+4]
            r_float = row[itemqa_id_index+5]

            # todo: depends on qa type?
            value = r_text
            if value is None:
                value = r_int
            if value is None:
                value = r_float

            if not answers.has_key(qa_id):
                answers[qa_id] = {}

            if q_id is not None:

                if not answers[qa_id].has_key(q_id):
                    answers[qa_id][q_id] = {}

                if not answers[qa_id][q_id].has_key(a_id):
                    answers[qa_id][q_id][a_id] = value
            else:
                answers[qa_id][a_id] = value

        # write laste participant
        self.write_participant(participant_data)

    def write_participant(self, data):
        """
        must be implemented by child classes to write one participant at the time
        """
        raise Exception('abstract');

class ArrayExporter(Exporter):
    """
    simple array export class (used for test assering above class)
    """
    def export(self):
        self._result = []
        Exporter.export(self)
        return self._result

    def write_participant(self, data):
        self._result.append(data)

class SpssExporter(Exporter):
    """
    """

    def __init__(self, *args, **kwargs):
        super(SpssExporter, self).__init__(*args, **kwargs)
        self.assert_permissions('iquest_export_spss')

    def export(self):

        # holds the data file lines
        self._variable_labels = OrderedDict()
        self._value_labels = OrderedDict()

        self._qa_headers = self.get_qa_headers()

        self._csv_file_name = os.path.join(settings.TMP_FILE_REPOSITORY, 'iquest_export_%s.csv' % hashlib.sha1(str(random.random())).hexdigest()[:8])
        file_ = open(self._csv_file_name, 'wb')
        self._queue = cStringIO.StringIO()
        self._writer = csv.writer(self._queue, delimiter="\t", quotechar='"', quoting=csv.QUOTE_ALL)
        self._stream = file_
        self._encoder = codecs.getincrementalencoder('utf-8')()

        # write first line without data, that writes the headers
        self.write_line(data=None)

        # run the exporter which will write to data csv and fill labels dicts
        Exporter.export(self)

        file_.close()

        self._write_meta_file()

        # finally combine the two files into 1 zipfile
        zip_file_name = os.path.join(settings.TMP_FILE_REPOSITORY, 'iquest_export_%s.zip' % hashlib.sha1(str(random.random())).hexdigest()[:8])
        myzip = ZipFile(zip_file_name, 'w')
        myzip.write(self._csv_file_name, 'data.csv')
        myzip.write(self._meta_file_name, 'labels.sps')

        return zip_file_name

    def _escape(self, x):
        return x.replace("'", "\'")

    def _write_meta_file(self):
        """
        writes the spss meta file to a format like this:
        http://trac.office.basesoft.com/iqportal/attachment/ticket/2511/Labels%20AZN.txt
        """

        meta_file_name = os.path.join(settings.TMP_FILE_REPOSITORY, 'iquest_export_%s.txt' % hashlib.sha1(str(random.random())).hexdigest()[:8])

        f = open(meta_file_name, "w")
        f.write("VARIABLE LABELS\n")
        for x,y in self._variable_labels.iteritems():
            line = u"%s '%s'\n" % (x, self._escape(y))
            f.write(line.encode('utf-8'))

        f.write(".\n")

        for x,y in self._value_labels.iteritems():
            f.write("VALUE LABELS %s" % x)

            for i in range(len(y)):
                line = u" %s '%s'" % (i+1, self._escape(y[i]))
                f.write(line.encode('utf-8'))

            f.write(".\n")

        f.close()

        self._meta_file_name = meta_file_name

    def write_participant(self, data):
        self.write_line(data)

    def write_line(self, data=None):
        """
        """
        row = []

        qa_headers = self._qa_headers

        i = -1 # column index
        for key in ('publication', 'person', 'contact', 'company'):
            for col in self.columns_def[key]:
                i = i+1

                if data is None:
                    row += [col[2]] # write header
                else:
                    row += [data[i]] # write data

        fill_labels = True
        q_i = 0
        for qa_id in qa_headers:
            qa_title = qa_headers[qa_id]['label']
            qa_type = qa_headers[qa_id]['type']

            #print 'qa_title (%s): %s' % (qa_id, qa_title)

            if (len(qa_headers[qa_id]['questions'])):
                for q_id, q_title in qa_headers[qa_id]['questions']:

                    # spss question-id
                    q_i = q_i + 1
                    q_variable_title = 'Q%s' % q_i # TODO: make Q001

                    if fill_labels:
                        # save qa_title + q_title to the _variable_labels used in the meta file
                        _title = ''
                        if qa_title is not None:
                            _title = qa_title
                            if q_title is not None:
                                _title += ' / '
                        if q_title is not None:
                            _title += q_title

                        self._variable_labels[q_variable_title] = _title
                        self._value_labels[q_variable_title] = []

                    # process data
                    if data is None:
                        row.append(q_variable_title)
                    else:

                        a_i = 0 # answer index
                        cell_data = []
                        for a_id, a_title in qa_headers[qa_id]['answers']:
                            a_i = a_i + 1

                            # get answer value from dataset
                            answer_value = ''
                            try:
                                answer_value = data[len(data)-1][qa_id][q_id][a_id]
                            except KeyError:
                                pass

                            # append answer to value_labels
                            if fill_labels:
                                self._value_labels[q_variable_title].append(a_title)

                            # the question types below set data as the answer index but only if its checked or selected
                            if qa_type in (SurveyItemQA.TYPE_SINGLE_CHOICE, SurveyItemQA.TYPE_MULTIPLE_CHOICE, ):
                                if answer_value:
                                    cell_data.append(a_i)

                            # other types (text, ints, etc) save the value itself
                            else:
                                if answer_value is None:
                                    answer_value = ''
                                cell_data.append(answer_value)

                        row.append( ",".join(unicode(x) for x in cell_data) )

        fill_labels = False

        # write the row to the data csv file
        self._writer.writerow(['' if s is None else unicode(s).encode("utf-8") for s in row])
        data = self._queue.getvalue()
        data = data.decode("utf-8")
        data = self._encoder.encode(data)
        self._stream.write(data)
        self._queue.truncate(0)





class TwoDimExporter(Exporter):
    """
    abstract configurable two dimensional exported (xls, csv)
    """

    HEADERS_MODE_SQUASH = 1 # all headers squashed on the first row (qa_title / q_title / a_title) (default)
    HEADERS_MODE_3ROWS = 2 # qa_title first, q_title second, a_title on third and data on 4th+

    # can be changed before calling export()
    headers_mode = HEADERS_MODE_SQUASH

    def export(self):
        """
        writes the headers and then calls super export() for writing data
        """

        self._qa_headers = self.get_qa_headers()

        if self.headers_mode == self.HEADERS_MODE_SQUASH:
            self.write_line(row_i=0)
        elif self.headers_mode == self.HEADERS_MODE_3ROWS:
            for i in range(3):
                self.write_line(row_i=i)

        Exporter.export(self)

    def write_row(self, data):
        """
        gets called for each row that should be written (should be implemented by child class)
        """
        raise Exception('abstract')

    def write_participant(self, data):
        """
        see write_line()
        """
        self.write_line(data)

    __headers_length = 0
    __prev_qa_id = None
    __prev_q_id = None

    def write_line(self, data=None, row_i=None):
        """
        Gets called to write the headers and data.
        Depending on current headers_mode it write either 1 on 3 header rows
        """

        row = []

        qa_headers = self._qa_headers
        if self.headers_mode == self.HEADERS_MODE_SQUASH:

            i = -1 # column index
            for key in ('publication', 'person', 'contact', 'company'):
                for col in self.columns_def[key]:
                    i = i+1

                    if data is None:
                        row += [col[2]] # write header
                    else:
                        row += [data[i]] # write data


            for qa_id in qa_headers:
                qa_title = qa_headers[qa_id]['label']
                #print 'qa_title (%s): %s' % (qa_id, qa_title)

                if (len(qa_headers[qa_id]['questions'])):
                    for q_id, q_title in qa_headers[qa_id]['questions']:
                        #print ' - q_title (%s): %s' % (q_id, q_title)

                        for a_id, a_title in qa_headers[qa_id]['answers']:
                            #print '   - a_title (%s): %s' % (a_id, a_title)

                            if data is None:
                                if q_title is not None:
                                    row.append('%s / %s / %s' % (qa_title, q_title, a_title) )
                                else:
                                    row.append('%s / %s' % (qa_title, a_title) )
                            else:
                                try:
                                    row.append( data[len(data)-1][qa_id][q_id][a_id] )
                                except KeyError:
                                    row.append('')

            self._write_row(row)

        if self.headers_mode == self.HEADERS_MODE_3ROWS:

            i = -1 # column index
            for key in ('publication', 'person', 'contact', 'company'):
                for col in self.columns_def[key]:
                    i = i+1

                    if row_i == 0:
                        row += [col[2]] # write header

                    if data is not None:
                        row += [data[i]] # write data

            self.__headers_length = i+1

            if (row_i in (1, 2)):
                for i in range(self.__headers_length):
                    row.append('')

            for qa_id in qa_headers:
                qa_title = qa_headers[qa_id]['label']
                #print 'qa_title (%s): %s' % (qa_id, qa_title)

                if qa_title is None:
                    qa_title = '-'

                if row_i == 0:
                    row.append(qa_title)

                if (len(qa_headers[qa_id]['questions'])):
                    for q_id, q_title in qa_headers[qa_id]['questions']:
                        #print ' - q_title (%s): %s' % (q_id, q_title)

                        if q_title is None:
                            q_title = '-'

                        if row_i == 1:
                            row.append(q_title)

                        for a_id, a_title in qa_headers[qa_id]['answers']:
                            #print '   - a_title (%s): %s' % (a_id, a_title)

                            if data is None:

                                if row_i == 0:
                                    if self.__prev_qa_id == qa_id:
                                        row.append('')
                                    else:
                                        self.__prev_qa_id = qa_id

                                if row_i == 1:
                                    if self.__prev_q_id == q_id or q_title is None:
                                        row.append('')
                                    else:
                                        self.__prev_q_id = q_id

                                if row_i == 2:
                                    row.append(a_title)
                            else:
                                try:
                                    row.append( data[len(data)-1][qa_id][q_id][a_id] )
                                except KeyError:
                                    row.append('')

            self._write_row(row)

    def _write_row(self, data):
        self._result.append(data)

class ArrayTwoDimExporter(TwoDimExporter):
    """
    simple array export class (used for test assering above class)
    """

    _result = []

    def export(self):
        self._result = []
        TwoDimExporter.export(self)
        return self._result

    def _write_row(self, data):
        self._result.append(data)

class CsvExporter(TwoDimExporter):
    """
    csv export controller

    see UnicodeWriter from http://docs.python.org/library/csv.html
    """

    _writer = None

    def export(self):

        hash_ = hashlib.sha1(str(random.random())).hexdigest()[:8]
        file_name = os.path.join(settings.TMP_FILE_REPOSITORY, 'iquest_export_%s.csv' % hash_)
        file_ = open(file_name, 'wb')

        self._queue = cStringIO.StringIO()
        self._writer = csv.writer(self._queue, delimiter="\t", quotechar='"', quoting=csv.QUOTE_ALL)
        self._stream = file_
        self._encoder = codecs.getincrementalencoder('utf-8')()

        TwoDimExporter.export(self)

        return file_name

    def _write_row(self, row):
        self._writer.writerow(['' if s is None else unicode(s).encode("utf-8") for s in row])
        data = self._queue.getvalue()
        data = data.decode("utf-8")
        data = self._encoder.encode(data)
        self._stream.write(data)
        self._queue.truncate(0)

class XlsExporter(TwoDimExporter):
    """
    xls export controller
    """

    _writer = None
    _row_i = 0
    _sheet = None

    def __init__(self, *args, **kwargs):
        super(XlsExporter, self).__init__(*args, **kwargs)
        self.assert_permissions('iquest_export_excel')


    def export(self):

        from hashlib import sha256 as sha
        from openpyxl.workbook import Workbook

        wb = Workbook(encoding="UTF-8")
        self._ws = wb.get_active_sheet()

        TwoDimExporter.export(self)

        hash_ = sha(str(random.random())).hexdigest()[:8]
        file_name = os.path.join(settings.TMP_FILE_REPOSITORY,  'iquest_export_%s.xlsx' % hash_)
        wb.save(file_name)

        return file_name

    def _write_row(self, data):

        print data

        for col_i in range(len(data)):

            value = data[col_i]
            if value is not None and unicode(value).strip() != '':
                cell = self._ws.cell(row = self._row_i, column = col_i)
                cell.value = value

        self._row_i = self._row_i + 1





