# Copyright (c) 2009 BaseSoft B.V.
# See LICENSE for details.

import hashlib
from threading import Lock

cache_locks = {}
get_cache_lock = Lock()


from sqlalchemy import and_
from sqlalchemy.orm.exc import NoResultFound
from sqlalchemy.orm.session import Session

from basesoft.web.core.exceptions import SecurityException, PublicUserException
from basesoft.web.core.util.controllers import AbstractController, transaction_decorator

from iqportal.iqube.core.controllers import IQubeAggregator
from iqportal.iqreport.core.controllers import ReportPublicationStateController
from iqportal.iqreport.display.controllers import IQReportXMLACacheController
from iqportal.iquest.report.models import ReportIQuestPublication, ReportIQuestPublicationXMLACache
from iqportal.iquest.core.models import SurveyPublication

from basesoft import logging
logger = logging.getLogger('iqportal.iquest.report')

class IQuestIQReportXMLACacheController(IQReportXMLACacheController):

    def set_context(self, project, role, publication=None, contact=None, force_cache=False, force_recache=False):
        self.project = project
        self.role_name = role
        self.publication = publication
        self.contact = contact
        self._cache = None
        # we are not in publication mode
        if self.publication is None:
            self.set_cache_path()
            return

        self.sa_session = Session.object_session(self.publication)

        # set the role to the publication cache_role_name
        self.role_name = 'iqportal_account_%s' % self.publication.cache_role_name
        self._client.set_role_name(self.role_name)
        # we are dealing with a published report, force cache loading
        self.force_from_cache = force_cache
        # if this is triggered by the task, that means we should re-cache
        self.force_recache = force_recache

    def load(self, message):
        if self.publication is None:
            return IQReportXMLACacheController.load(self, message)

        xmla_response = None
        # cache based on the message content and role_name
        role_name = self.role_name
        if role_name is None:
            role_name = ''
        md5_content = message + role_name

        self._hash = hashlib.md5(md5_content.encode('utf-8')).hexdigest()

        company = None
        if self.contact is not None:
            company = self.contact.company

        try:
            self._cache = self.sa_session.query(ReportIQuestPublicationXMLACache).filter(
                and_(ReportIQuestPublicationXMLACache.hash==self._hash,
                     ReportIQuestPublicationXMLACache.report_publication==self.publication,
                     ReportIQuestPublicationXMLACache.company==company)).one()
            if self.force_recache:
                return None
            return self._cache.xmla_response.encode('utf-8')
        except NoResultFound:
            if self.force_from_cache:
                logger.warn("%s:load(): force loading from cache failed: %s" % (self, self._hash))
                raise PublicUserException('Cache currently not available, unable to load data. Try again later.')
        return xmla_response

    def save(self, xmla_response):
        # we dont have a publication, that means preview mode, just use normal caching
        if self.publication is None:
            return IQReportXMLACacheController.save(self, xmla_response)

        # not caching
        if xmla_response.find('<SOAP-ENV:Fault>') != -1:
            logger.warn("%s:save(): not caching faulty response" % self)
            return

        # we already have this cached, no reason to do it again.
        if self._cache is not None and not self.force_recache:
            return

        company = None
        if self.contact is not None:
            company = self.contact.company

        lock_key = unicode(self._hash) + unicode(self.publication) + unicode(company)
        with get_cache_lock:
            if lock_key not in cache_locks:
                cache_locks[lock_key] = Lock()
            cache_locks[lock_key].acquire()

        try:

            q_len = len(self.sa_session.query(ReportIQuestPublicationXMLACache).filter(
                and_(ReportIQuestPublicationXMLACache.hash==self._hash,
                     ReportIQuestPublicationXMLACache.report_publication==self.publication,
                     ReportIQuestPublicationXMLACache.company==company)).all())

            if q_len > 1:
                raise Exception('duplicate cache entry for %s/%s/%s' % (self._hash, self.publication.id, company) )

            logger.info('%s:save(), saving cache (%s)' % (self, self._hash))

            company = None
            if self.contact is not None:
                company = self.contact.company
            if self._cache:
                c = self._cache
            else:
                c = ReportIQuestPublicationXMLACache()
            c.report_publication = self.publication
            c.company = company
            c.hash = self._hash
            c.xmla_response = xmla_response
            self.sa_session.add(c)

            # cant use transaction_decorator here (no abstract object), so manually flushing
            self.sa_session.flush()

        finally:
            cache_locks[lock_key].release()
            del cache_locks[lock_key]
            pass

    def __repr__(self):
        return '<IQuestIQReportXMLACacheController>'

class ReportController(AbstractController):
    """
    This is the controller for the report app within IQuest.
    """
    def get_default_report_state(self, publication):
        """
        Get the default report state for the provided publication.
        """
        from iqportal.iquest.report.reporters import DefaultReporter

        #self.assert_permissions('view_iquest_default_report')
        self.assert_privileges_on_object(publication, SurveyPublication.PRIV_VIEW_RESPONSE)

        return DefaultReporter(self.sa_session, self._get_account(), publication).get_state()


class IQuestPublicationStatisticsAggregator(IQubeAggregator):
    _table = "aggSurveyPublicationStatistics"
    _columns = [
        'invitee_id',
        'publication_id',
        'is_participant',
        'is_respondent',
        'participant_time_id',
        'company_id',
        'person_id',
        'last_updated'
    ]

    _select_sql = """

        SELECT
            i.id as invitee_id,
            i.publication_id as publication_id,
            IF(ISNULL(p.id), 0, 1) as is_participant,
            IF(p.is_respondent=1, 1, 0) as is_respondent,
            DATE_FORMAT(IFNULL(p.last_updated, i.last_updated), '%%Y%%m%%d') as participant_time_id,
            c.company_id,
            c.person_id,
            IF(ISNULL(p.last_updated), i.last_updated, p.last_updated) as last_updated
        FROM tblSurveyPublication as pub
        INNER JOIN tblComposeInvitee as i ON pub.publication_id = i.publication_id
        INNER JOIN tblComposePublication cpub ON pub.publication_id = cpub.id
        INNER JOIN tblSurvey s ON pub.survey_id = s.id
        LEFT JOIN tblSurveyParticipant as p ON i.id = p.invitee_id
        LEFT JOIN tblContact as c ON i.contact_id = c.id
        WHERE (i.last_updated > %(max_last_updated)s OR (NOT ISNULL(p.last_updated) AND p.last_updated > %(max_last_updated)s))
        AND cpub.deleted = 0
        AND s.deleted = 0
        AND cpub.id = %(publication_id)s
        ORDER BY i.id
    """

    def _get_publications(self):
        return self.db_engine_ro.execute("""
            SELECT IFNULL(MAX(a.last_updated), 0), p.publication_id
            FROM tblSurveyPublication as p
            LEFT JOIN %s as a
                ON p.publication_id = a.publication_id
            GROUP BY p.publication_id""" % self._table)

    def run(self, loop_validator=None):

        affected_rows = 0

        rows = self._get_publications()
        for max_last_updated, publication_id in rows:
            affected_rows = affected_rows + self._work({
                'max_last_updated': max_last_updated,
                'publication_id': publication_id,
            }, loop_validator)

        rows.close()

        return affected_rows

class IQuestAbstractResponseAggregator(IQubeAggregator):
    _columns = [
        'id',
        'publication_id',
        'participant_id',
        'participant_time_id',
        'company_id',
        'person_id',
        'question_id',
        'answer_id',
        'item_id',
        'participant_session_id',
        'response_int',
        'response_float',
        'weighted_int',
        'weighted_float',
        'created',
        'last_updated'
    ]

    _select_sql = """
        SELECT
            r.id,
            i.publication_id,
            r.participant_id,
            DATE_FORMAT(p.last_updated, '%%Y%%m%%d') as participant_time_id,
            c.company_id,
            c.person_id,
            r.question_id,
            r.answer_id,
            eqa.item_id,
            r.participant_session_id,
            r.response_int,
            r.response_float,
            IF(ISNULL(w.answer_id), IF(ISNULL(w2.answer_id), IF(ISNULL(w3.question_id), IF(ISNULL(w4.item_id), 1, w4.weight), w3.weight), w2.weight), w.weight)*r.response_int as weighted_int,
            IF(ISNULL(w.answer_id), IF(ISNULL(w2.answer_id), IF(ISNULL(w3.question_id), IF(ISNULL(w4.item_id), 1, w4.weight), w3.weight), w2.weight), w.weight)*r.response_float as weighted_float,
            r.created,
            r.last_updated
        FROM tblSurveyQAResponse r
        INNER JOIN tblSurveyParticipant p ON p.id=r.participant_id
        INNER JOIN tblComposeInvitee i ON i.id=p.invitee_id
        INNER JOIN tblSurveyPublication pub ON i.publication_id = pub.publication_id
        INNER JOIN tblComposePublication cpub ON pub.publication_id = cpub.id
        INNER JOIN tblSurvey s ON pub.survey_id = s.id
        INNER JOIN tblSurveyItemQAElement eqa ON eqa.id=r.question_id
        LEFT JOIN tblContact as c ON i.contact_id = c.id
        LEFT JOIN tblSurveyItemQAWeightFactor as w
            ON r.question_id = w.question_id
                AND r.answer_id = w.answer_id
        LEFT JOIN tblSurveyItemQAWeightFactor as w2
            ON ISNULL(w2.question_id)
                AND r.answer_id = w2.answer_id
        LEFT JOIN tblSurveyItemQAWeightFactor as w3
            ON r.question_id = w3.question_id
                AND ISNULL(w3.answer_id)
        LEFT JOIN tblSurveyItemQAWeightFactor as w4
            ON eqa.item_id = w4.item_id
                AND ISNULL(w4.question_id)
                AND ISNULL(w4.answer_id)
        WHERE (r.last_updated > %(max_last_updated)s OR (NOT ISNULL(c.last_updated) AND c.last_updated > %(max_last_updated)s))
        AND cpub.deleted = 0
        AND s.deleted = 0
        AND ISNULL(r.response_text)
        AND cpub.id = %(publication_id)s
        ORDER BY r.id
    """

    def _get_publications(self):
        return self.db_engine_ro.execute("""
            SELECT IFNULL(MAX(a.last_updated), 0), p.publication_id
            FROM tblSurveyPublication as p
            LEFT JOIN %s as a
                ON p.publication_id = a.publication_id
            GROUP BY p.publication_id""" % self._table)

    def run(self, loop_validator=None):

        # delete all responses that were removed by participants
        res = self.db_engine.execute("""DELETE FROM %s WHERE id NOT IN (
            SELECT r.id
            FROM tblSurveyQAResponse r
            INNER JOIN tblSurveyParticipant p ON p.id=r.participant_id
            INNER JOIN tblComposeInvitee i ON i.id=p.invitee_id
            INNER JOIN tblSurveyPublication pub ON i.publication_id = pub.publication_id
            INNER JOIN tblComposePublication cpub ON pub.publication_id = cpub.id
            INNER JOIN tblSurvey s ON pub.survey_id = s.id
            WHERE cpub.deleted = 0
            AND s.deleted = 0
        )""" % self._table)

        affected_rows = res.rowcount

        rows = self._get_publications()
        for max_last_updated, publication_id in rows:
            affected_rows = affected_rows + self._work({
                'max_last_updated': max_last_updated,
                'publication_id': publication_id,
            }, loop_validator)

        return affected_rows

class IQuestResponseFullAggregator(IQuestAbstractResponseAggregator):
    _table = "aggFullIQuestResponse"

class IQuestResponseFilteredAggregator(IQuestAbstractResponseAggregator):
    _table = "aggFilteredIQuestResponse"

    def _get_publications(self):
        # We should get all publications which have their responses changed since the last aggregate run.
        rows = self.db_engine_ro.execute("""
            SELECT IFNULL(MAX(a.last_updated), 0), p.publication_id
            FROM tblSurveyPublication as p
            LEFT JOIN %s as a
                ON p.publication_id = a.publication_id
            GROUP BY p.publication_id""" % self._table)

        rv = []

        q = """
            SELECT r.id
            FROM tblSurveyQAResponse r
            INNER JOIN tblSurveyParticipant p ON p.id=r.participant_id
            INNER JOIN tblComposeInvitee i ON i.id=p.invitee_id
            INNER JOIN tblSurveyPublication pub ON i.publication_id = pub.publication_id
            INNER JOIN tblComposePublication cpub ON pub.publication_id = cpub.id
            INNER JOIN tblSurvey s ON pub.survey_id = s.id
            LEFT JOIN tblContact as c ON i.contact_id = c.id
            WHERE (r.last_updated > %(max_last_updated)s OR (NOT ISNULL(c.last_updated) AND c.last_updated > %(max_last_updated)s))
            AND cpub.deleted = 0
            AND s.deleted = 0
            AND ISNULL(r.response_text)
            AND cpub.id = %(publication_id)s
        """

        for max_last_updated, publication_id in rows:
            where_args = {
                'max_last_updated': max_last_updated,
                'publication_id': publication_id,
            }
            # (Re)process ALL responses for the publication in case ANY response has changed
            res = self.db_engine_ro.execute(q, where_args)
            if res.rowcount > 0:
                rv.append((0, publication_id))
            res.close()

        rows.close()

        return rv

    def run(self, loop_validator=None):
        affected_rows = IQuestAbstractResponseAggregator.run(self, loop_validator=loop_validator)

        logger.info('%s: run() checking to remove filtered responses on %s' % (self, self._table))

        # check for minimum number of respondents for results to be shown. (#2215)
        to_delete = """
        SELECT question_id, answer_id, a.publication_id, minimum_respondents FROM aggFilteredIQuestResponse as a
        INNER JOIN tblSurveyPublication as p
            ON p.publication_id = a.publication_id
        GROUP BY question_id, answer_id, a.publication_id
        HAVING COUNT(*) < minimum_respondents
        """

        sql = """
        DELETE FROM aggFilteredIQuestResponse WHERE question_id = %s AND answer_id = %s AND publication_id = %s
        """

        result = self.db_engine.execute(to_delete)
        for question_id, answer_id, publication_id, bitbucket in result:
            logger.info(' - removing for question (%s) answer (%s) and publication (%s) due to low response # (%s)' % (question_id, answer_id, publication_id, bitbucket))
            r = self.db_engine.execute(sql % (question_id, answer_id, publication_id))
            affected_rows = affected_rows + r.rowcount

        # check for maximum market share, in case above set number, hide. (#2947)
        # this is done only for items that have the validate_market_share set to True.
        to_delete = """
        SELECT SUM(a.%(field)s) as total, MAX(a.%(field)s), a.question_id, a.answer_id, a.publication_id,
            p.maximum_market_share
        FROM aggFilteredIQuestResponse as a
        INNER JOIN tblSurveyPublication as p
            ON a.publication_id = p.publication_id
        INNER JOIN tblSurveyItemQA as i
            ON a.item_id = i.item_id
        WHERE NOT ISNULL(a.%(field)s)
            AND NOT ISNULL(p.maximum_market_share)
            AND i.validate_market_share = 1
        GROUP BY a.question_id, a.answer_id, a.publication_id
        HAVING ((MAX(a.%(field)s)/total)*100) > p.maximum_market_share
        """

        # we can have a market share for floats or int.
        for field in ['response_int', 'response_float']:
            logger.info(' - checking %s for market share...' % field)
            result = self.db_engine.execute(to_delete % {'field': field})
            # we have results, so we need to see if they are indeed a numerical QA result.
            for total, response, question_id, answer_id, publication_id, max_market_share in result:
                # delete because the market share was exceeding the given max (on survey publication level)
                logger.info(' - removing for question (%s) answer (%s) and publication (%s) due to exceeding market share (%s from %s > (%s%%))' %
                            (question_id, answer_id, publication_id, response, total, max_market_share))
                r = self.db_engine.execute(sql % (question_id, answer_id, publication_id))
                affected_rows = affected_rows + r.rowcount

        return affected_rows

class SurveyItemQALocaleAggregator(IQubeAggregator):

    _table = "aggSurveyItemQALocale"
    _columns = [
        'item_id',
        'locale_id',
        'label',
        'position',
        'last_updated'
    ]

    _select_sql = """
            SELECT IF(@current_survey=survey_id, @current_position := @current_position+1, @current_position := 0),
                item_id, locale_id, label,
                @current_position as `position`, last_updated,
                (@current_survey := survey_id) as survey_id FROM
            (
                SELECT  item_id, locale_id, label, last_updated, bpos, cpos, ipos, survey_id FROM
                (
                    -- try to get locale from element (only for stale surveys)
                    SELECT  e.item_id,  el.locale_id,  el.label, el.last_updated,
                        b.position as bpos, c.position as cpos, i.position as ipos, s.id as survey_id
                    FROM `tblSurveyItemQA` e
                    INNER JOIN `tblSurveyItem` i ON i.id=e.item_id
                    INNER JOIN `tblSurveyBlockColumn` c ON i.column_id = c.id
                    INNER JOIN `tblSurveyBlock` b ON c.block_id = b.id
                    INNER JOIN `tblSurvey` s ON b.survey_id = s.id  and s.stale = 1
                    INNER JOIN `tblSurveyItemQALocale` el ON e.item_id = el.item_id and el.label != ''

                    UNION

                    -- fallback
                    SELECT  e.item_id,  l.locale_id AS locale_id,
                        IF(el.label IS NULL, '-', el.label),
                        NOW() AS last_updated,
                        b.position as bpos, c.position as cpos, i.position as ipos, s.id as survey_id
                    FROM `tblSurveyItemQA` e
                    INNER JOIN `tblSurveyItem` i ON e.item_id = i.id
                    INNER JOIN `tblSurveyBlockColumn` c ON i.column_id = c.id
                    INNER JOIN `tblSurveyBlock` b ON c.block_id = b.id
                    INNER JOIN `tblSurvey` s ON b.survey_id = s.id and s.stale = 1
                    INNER JOIN tblSurveyLocale l ON l.survey_id=s.id
                    LEFT JOIN `tblSurveyItemQALocale` el ON e.item_id = el.item_id AND el.locale_id = s.fallback_locale_id
                    GROUP BY e.item_id, l.survey_id, l.locale_id
                ) AS t
                GROUP BY item_id, locale_id
                ORDER BY survey_id, bpos, cpos, ipos
            ) as t2
    """

    def run(self, loop_validator=None):

        # before running the aggregator remove all exiting entries
        # this makes sure that questions / items that have been removed are not in the aggregate tables anymore (fixes #3466)
        self.db_engine.execute("""
            DELETE FROM aggSurveyItemQALocale WHERE item_id IN (
                SELECT i.id
                FROM `tblSurveyItem` i
                INNER JOIN `tblSurveyItemQA` iqa ON i.id=iqa.item_id
                INNER JOIN `tblSurveyBlockColumn` c ON i.column_id = c.id
                INNER JOIN `tblSurveyBlock` b ON c.block_id = b.id
                INNER JOIN `tblSurvey` s ON b.survey_id = s.id and s.stale = 1
            )
        """)

        # set the variables used by the worker:
        self.db_engine_ro.execute("""SET @current_survey := NULL""")
        self.db_engine_ro.execute("""SET @current_position := 0""")

        return self._work({}, loop_validator)

class SurveyItemQAElementLocaleAggregator(IQubeAggregator):

    _table = "aggSurveyItemQAElementLocale"
    _columns = [
        'item_id',
        'item_element_id',
        'locale_id',
        'label',
        'is_question',
        'position',
        'last_updated'
    ]

    _select_sql = """
            SELECT  item_id, item_element_id, locale_id, label, is_question, `position`, last_updated FROM
            (
                -- try to get locale from element (only for stale surveys)
                SELECT  e.item_id, e.id AS item_element_id,  el.locale_id,  el.label, e.is_question, e.position, el.last_updated
                FROM `tblSurveyItemQAElement` e
                INNER JOIN `tblSurveyItem` i ON i.id=e.item_id
                INNER JOIN `tblSurveyItemQA` iqa ON i.id=iqa.item_id
                INNER JOIN `tblSurveyBlockColumn` c ON i.column_id = c.id
                INNER JOIN `tblSurveyBlock` b ON c.block_id = b.id
                INNER JOIN `tblSurvey` s ON b.survey_id = s.id and s.stale = 1
                INNER JOIN `tblSurveyItemQAElementLocale` el ON e.id = el.item_element_id and el.label != ''
                WHERE IF(e.is_question, TRUE, iqa.object_name = 'SurveyItemQA')

                UNION

                -- fallback
                SELECT  e.item_id, e.id AS item_element_id,  l.locale_id AS locale_id,
                IF(el.label IS NULL, '-', el.label), e.is_question, e.position,
                # this is using the translations of the fallback, but don't also use last_updated for fallback, because in
                # case there's a save for this locale, it won't see it because last_updated
                NOW() AS last_updated
                FROM `tblSurveyItemQAElement` e
                INNER JOIN `tblSurveyItem` i ON e.item_id = i.id
                INNER JOIN `tblSurveyItemQA` iqa ON i.id=iqa.item_id
                INNER JOIN `tblSurveyBlockColumn` c ON i.column_id = c.id
                INNER JOIN `tblSurveyBlock` b ON c.block_id = b.id
                INNER JOIN `tblSurvey` s ON b.survey_id = s.id and s.stale = 1
                INNER JOIN tblSurveyLocale l ON l.survey_id=s.id
                LEFT JOIN `tblSurveyItemQAElementLocale` el ON e.id = el.item_element_id AND el.locale_id = s.fallback_locale_id
                WHERE IF(e.is_question, TRUE, iqa.object_name = 'SurveyItemQA')
                GROUP BY e.id, l.survey_id, l.locale_id

                UNION

                -- this is specific for NPS items answers (untranslated answers won't get fallback, but position)
                SELECT  e.item_id, e.id AS item_element_id,  l.locale_id,
                    IF(el.label IS NULL, e.position, el.label),
                    e.is_question, e.position,
                    IF(el.last_updated IS NULL, NOW(), el.last_updated)
                FROM `tblSurveyItemQAElement` e
                INNER JOIN `tblSurveyItem` i ON i.id=e.item_id
                INNER JOIN `tblSurveyItemQA` iqa ON i.id=iqa.item_id
                INNER JOIN `tblSurveyBlockColumn` c ON i.column_id = c.id
                INNER JOIN `tblSurveyBlock` b ON c.block_id = b.id
                INNER JOIN `tblSurvey` s ON b.survey_id = s.id and s.stale = 1
                INNER JOIN tblSurveyLocale l ON l.survey_id=s.id
                LEFT JOIN `tblSurveyItemQAElementLocale` el ON e.id = el.item_element_id and el.locale_id = l.locale_id
                WHERE IF(e.is_question, FALSE, iqa.object_name = 'ItemQANPS')
            ) AS t
            GROUP BY item_id, item_element_id, locale_id
    """

    def run(self, loop_validator=None):

        # before running the aggregator remove all exiting entries
        # this makes sure that questions / items that have been removed are not in the aggregate tables anymore (fixes #3466)
        self.db_engine.execute("""
            DELETE FROM aggSurveyItemQAElementLocale WHERE item_id IN (
                SELECT i.id
                FROM `tblSurveyItem` i
                INNER JOIN `tblSurveyItemQA` iqa ON i.id=iqa.item_id
                INNER JOIN `tblSurveyBlockColumn` c ON i.column_id = c.id
                INNER JOIN `tblSurveyBlock` b ON c.block_id = b.id
                INNER JOIN `tblSurvey` s ON b.survey_id = s.id and s.stale = 1
            )
        """)

        # let the aggregator fill the table again
        return self._work({}, loop_validator)

class IQuestAggregator:
    """
    Repopulate all iquest aggregate tables that check for stale flag on surveys.
    """
    def run(self, loop_validator=None):
        self._aggregators = []
        affected_rows = 0

        # item QAs
        c = SurveyItemQALocaleAggregator()
        self._aggregators.append(c)
        affected_rows = affected_rows + c.run(loop_validator)

        # item QA elements
        c = SurveyItemQAElementLocaleAggregator()
        self._aggregators.append(c)
        affected_rows = affected_rows + c.run(loop_validator)

        # all surveys were processed
        c.db_engine.execute("""UPDATE tblSurvey SET stale=0""")

        return affected_rows

    def release(self):
        for a in self._aggregators:
            a.release()

class IQuestReportPublicationStateController(ReportPublicationStateController):
    """
    FIXME: end_date can only be changed if the publication is active or in the future
    """

    _instance_class = ReportIQuestPublication

    @transaction_decorator
    def save(self, state, group_id=None):

        publication = self.set_publication(state, group_id)

        if state.has_key('survey_publication_id') and state['survey_publication_id'] not in ['', None]:
            survey_publication = self.get_publications(id=state['survey_publication_id'], type=SurveyPublication)
            if len(survey_publication) == 0:
                raise SecurityException('Invalid survey publication id %s' % state['survey_publication_id'])
            publication.survey_publication = survey_publication[0]

        if state['dbID'] is None:
            self.save_default_privilege_object_group(publication, target_group=group_id)

        self.sa_session.add(self.report)
        self.sa_session.flush()

        # expire report so it will update the publications
        self.sa_session.expire(self.report)
        return publication

    def load(self, id):
        """
        Loads and returns the state for the given publication_id
        """

        state = ReportPublicationStateController.load(self, id)
        state["survey_publication_id"] = self.publication.survey_publication_id

        return state
