# Copyright (c) 2010 BaseSoft B.V.
# See LICENSE for details.

from django.conf import settings
from basesoft.web.core.management.commands.iqubesync import ProjectBase, CubeBase, RoleBase
from iqportal.iqube.core.models import DataSource
from basesoft.web.core.util.controllers import AbstractController
from iqportal.iquest.core.models import Survey, SurveyPublication
from iqportal.iqube.core.controllers import IQubeRoleController, IQubeRole
from basesoft.web.core.exceptions import PublicSecurityException

class IQuestProject(ProjectBase):
    project_name = 'IQuest'
    world_privileges = DataSource.PRIV_VIEW
    description = 'IQuest project'
    additional_graphs = ('iquest_numerical_table', 'iquest_table', )
    client_module = 'iqportal.iquest.report.client'
    calculated_member_hook_js_module = 'iquest_calculated_member_hook'
    role_level = DataSource.ROLE_LEVEL_DB

    def __init__(self, sa_session, controller):
        ProjectBase.__init__(self, sa_session, controller)
        self._db_from_connstr(settings.DB_CONNSTR_READONLY)

    def get_cubes(self):
        return (PublicationStatisticsCube, ResponseCube, FilteredResponseCube, VirtualResponseCube, VirtualFilteredResponseCube, )

    def get_roles(self):
        return (ResponseCubeRole, )

class PublicationStatisticsCube(CubeBase):
    name = 'PublicationStatistics'
    def _xml(self):
        return """
    <Cube name="%(name)s" defaultMeasure="Respondents">

        <Table name="aggSurveyPublicationStatistics" />

        <Dimension name="Survey" foreignKey="publication_id" >
            <Hierarchy hasAll="true" allMemberName="All Surveys" primaryKey="id" primaryKeyTable="tblComposePublication">
                <Join leftKey="id" rightKey="publication_id">
                    <Table name="tblComposePublication" >
                        <SQL dialect="generic">
                            tblComposePublication.deleted = 0
                            AND tblComposePublication.status > 1
                        </SQL>
                    </Table>
                    <Join leftKey="survey_id" rightKey="id">
                        <Table name="tblSurveyPublication" />
                        <Table name="tblSurvey" >
                            <SQL dialect="generic">
                                tblSurvey.deleted = 0
                            </SQL>
                        </Table>
                    </Join>
                </Join>

                <Level name="Survey" table="tblSurvey" column="id" captionColumn="name" type="Integer" />
                <Level name="Publication" table="tblComposePublication" column="id"  captionColumn="label" type="Integer" ordinalColumn="begin_date" />
            </Hierarchy>
        </Dimension>

        <Dimension name="Company" foreignKey="company_id">
            <Hierarchy hasAll="true" allMemberName="All Companies" primaryKey="id" primaryKeyTable="tblCompany">
                <Table name="tblCompany">
                        <SQL dialect="generic">
                            tblCompany.polymorphic_type != 'TntCLMCompany'
                        </SQL>
                </Table>

                <Level name="Company" column="id" type="Integer" captionColumn="name" ordinalColumn="name" />
            </Hierarchy>
        </Dimension>

        <Dimension name="Invoice Country" foreignKey="company_id">
            <Hierarchy hasAll="true" allMemberName="All Invoice Countries" primaryKey="company_id" primaryKeyTable="tblCompanyAddress">
                <Join leftKey="address_id" rightKey="id">
                    <Table name="tblCompanyAddress" >
                        <SQL dialect="generic">
                            tblCompanyAddress.is_invoice = 1
                        </SQL>
                    </Table>
                    <Join leftKey="country_id" rightKey="id">
                        <Table name="tblAddress" />
                        <Table name="tblCountry" />
                    </Join>
                </Join>

                <Level name="Country" table="tblCountry" column="id" type="Integer" captionColumn="name" ordinalColumn="name" />

            </Hierarchy>
        </Dimension>


        <!-- person dimensions -->

        <Dimension name="Gender" foreignKey="person_id">
            <Hierarchy hasAll="true" allMemberName="All Genders" primaryKey="id" primaryKeyTable="tblPerson">
                <Table name="tblPerson" />
                <Level name="Gender" column="gender" type="String" captionColumn="gender" ordinalColumn="gender" />
            </Hierarchy>
        </Dimension>


        <Dimension name="Person Country" foreignKey="person_id">

            <Hierarchy hasAll="true" allMemberName="All Person Countries" primaryKey="id" primaryKeyTable="tblPerson">
                <Join leftKey="address_id" rightKey="id">
                    <Table name="tblPerson" />
                    <Join leftKey="country_id" rightKey="id">
                        <Table name="tblAddress" />
                        <Table name="tblCountry" />
                    </Join>
                </Join>

                <Level name="Country" table="tblCountry" column="id" type="Integer" captionColumn="name" ordinalColumn="name" />

            </Hierarchy>
        </Dimension>

        <Dimension name="Time" foreignKey="participant_time_id" type="TimeDimension">
            <Hierarchy hasAll="true" primaryKey="id" name="Quarter">
                <Table name="aggTimeQuarter"/>
                <Level name="Year" column="year" type="Integer" uniqueMembers="true" levelType="TimeYears" ordinalColumn="year" />
                <Level name="Quarter" column="quarter" uniqueMembers="false" levelType="TimeQuarters"  ordinalColumn="quarter" />
                <Level name="Month" column="month" type="Integer" uniqueMembers="false" levelType="TimeMonths" ordinalColumn="month" />
                <Level name="Day" column="day" type="Integer" uniqueMembers="false" levelType="TimeDays" ordinalColumn="day" />
            </Hierarchy>
            <Hierarchy name="WeekISO8601" hasAll="true" primaryKey="id" caption="Week">
                <Table name="aggTimeWeekISO8601"/>
                <Level name="Year" column="year" type="Integer" uniqueMembers="true" levelType="TimeYears" ordinalColumn="year" />
                <Level name="Week" column="week" uniqueMembers="false" levelType="TimeWeeks" ordinalColumn="week" />
                <Level name="Day" column="day" type="Integer" uniqueMembers="false" levelType="TimeDays" ordinalColumn="day" />
            </Hierarchy>
            <Hierarchy name="WeekUS" hasAll="true" primaryKey="id" caption="Week">
                <Table name="aggTimeWeekUS"/>
                <Level name="Year" column="year" type="Integer" uniqueMembers="true" levelType="TimeYears" ordinalColumn="year" />
                <Level name="Week" column="week" uniqueMembers="false" levelType="TimeWeeks" ordinalColumn="week" />
                <Level name="Day" column="day" type="Integer" uniqueMembers="false" levelType="TimeDays" ordinalColumn="day" />
            </Hierarchy>
        </Dimension>


        <Measure name="PubStats Respondents" column="is_respondent" aggregator="sum" datatype="Integer" />
        <Measure name="PubStats Participants" column="is_participant" aggregator="sum" datatype="Integer" />
        <Measure name="PubStats Invitees" column="invitee_id" aggregator="count" datatype="Integer" />

        <CalculatedMember name="PubStats Response %%"
           dimension="Measures"
           formula="[Measures].[PubStats Respondents] / [Measures].[PubStats Invitees] * 100"
           formatString="#,###.00"
           visible="true">
        </CalculatedMember>

    </Cube>
        """ % {'name': self.name}

    xml = property(_xml)

class AbstractResponseCube(CubeBase):
    def _xml(self, fact_table):
        return """
    <Cube name="%(name)s" defaultMeasure="%(name)s Respondents">

        <Table name="%(fact_table)s" />

        <Dimension name="Item" foreignKey="item_id" >
            <Hierarchy hasAll="true" allMemberName="All Items" primaryKey="item_id" primaryKeyTable="aggSurveyItemQALocale">
                <Table name="aggSurveyItemQALocale" />

                <Level name="Locale" column="locale_id" type="Integer" />
                <Level name="Label" column="item_id" type="Integer" captionColumn="label" ordinalColumn="position" />
            </Hierarchy>
        </Dimension>

        <Dimension name="Question" foreignKey="question_id" >
            <Hierarchy hasAll="true" allMemberName="All Questions" primaryKey="item_element_id" primaryKeyTable="aggSurveyItemQAElementLocale">
                <Table name="aggSurveyItemQAElementLocale" alias="QuestionLocale">
                    <SQL dialect="generic">
                        QuestionLocale.is_question = 1
                    </SQL>
                </Table>

                <Level name="Item" table="QuestionLocale" column="item_id" type="Integer" />
                <Level name="Locale" table="QuestionLocale" column="locale_id" type="Integer" />
                <Level name="Label" table="QuestionLocale" column="item_element_id" type="Integer" captionColumn="label" ordinalColumn="position" />
            </Hierarchy>
        </Dimension>

        <Dimension name="Answer" foreignKey="answer_id">
            <Hierarchy hasAll="true" allMemberName="All Answers" primaryKey="item_element_id" primaryKeyTable="aggSurveyItemQAElementLocale">
                <Table name="aggSurveyItemQAElementLocale" alias="AnswerLocale">
                    <SQL dialect="generic">
                        AnswerLocale.is_question = 0
                    </SQL>
                </Table>

                <Level name="Item" table="AnswerLocale" column="item_id" type="Integer" />
                <Level name="Locale" table="AnswerLocale" column="locale_id" type="Integer" />
                <Level name="Label" table="AnswerLocale" column="item_element_id" type="Integer" captionColumn="label" ordinalColumn="position" />
            </Hierarchy>
        </Dimension>

        <Dimension name="Survey" foreignKey="publication_id" >
            <Hierarchy hasAll="true" allMemberName="All Surveys" primaryKey="id" primaryKeyTable="tblComposePublication">
                <Join leftKey="id" rightKey="publication_id">
                    <Table name="tblComposePublication" >
                        <SQL dialect="generic">
                            tblComposePublication.deleted = 0
                            AND tblComposePublication.status > 1
                        </SQL>
                    </Table>
                    <Join leftKey="survey_id" rightKey="id">
                        <Table name="tblSurveyPublication" />
                        <Table name="tblSurvey" >
                            <SQL dialect="generic">
                                tblSurvey.deleted = 0
                            </SQL>
                        </Table>
                    </Join>
                </Join>

                <Level name="Survey" table="tblSurvey" column="id" captionColumn="name" type="Integer" />
                <Level name="Publication" table="tblComposePublication" column="id"  captionColumn="label" type="Integer" ordinalColumn="begin_date" />
            </Hierarchy>
        </Dimension>

        <Dimension name="Company" foreignKey="company_id">
            <Hierarchy hasAll="true" allMemberName="All Companies" primaryKey="id" primaryKeyTable="tblCompany">
                <Table name="tblCompany">
                        <SQL dialect="generic">
                            tblCompany.polymorphic_type != 'TntCLMCompany'
                        </SQL>
                </Table>

                <Level name="Company" column="id" type="Integer" captionColumn="name" ordinalColumn="name" />
            </Hierarchy>
        </Dimension>

        <Dimension name="Invoice Country" foreignKey="company_id">
            <Hierarchy hasAll="true" allMemberName="All Invoice Countries" primaryKey="company_id" primaryKeyTable="tblCompanyAddress">
                <Join leftKey="address_id" rightKey="id">
                    <Table name="tblCompanyAddress" >
                        <SQL dialect="generic">
                            tblCompanyAddress.is_invoice = 1
                        </SQL>
                    </Table>
                    <Join leftKey="country_id" rightKey="id">
                        <Table name="tblAddress" />
                        <Table name="tblCountry" />
                    </Join>
                </Join>

                <Level name="Country" table="tblCountry" column="id" type="Integer" captionColumn="name" ordinalColumn="name" />

            </Hierarchy>
        </Dimension>


        <!-- person dimensions -->

        <Dimension name="Gender" foreignKey="person_id">
            <Hierarchy hasAll="true" allMemberName="All Genders" primaryKey="id" primaryKeyTable="tblPerson">
                <Table name="tblPerson" />
                <Level name="Gender" column="gender" type="String" captionColumn="gender" ordinalColumn="gender" />
            </Hierarchy>
        </Dimension>


        <Dimension name="Person Country" foreignKey="person_id">

            <Hierarchy hasAll="true" allMemberName="All Person Countries" primaryKey="id" primaryKeyTable="tblPerson">
                <Join leftKey="address_id" rightKey="id">
                    <Table name="tblPerson" />
                    <Join leftKey="country_id" rightKey="id">
                        <Table name="tblAddress" />
                        <Table name="tblCountry" />
                    </Join>
                </Join>

                <Level name="Country" table="tblCountry" column="id" type="Integer" captionColumn="name" ordinalColumn="name" />

            </Hierarchy>
        </Dimension>

        <!-- Participant used for cross referencing -->
        <Dimension name="Participant" foreignKey="participant_id">
            <Hierarchy hasAll="true" allMemberName="All Participants" primaryKey="id" primaryKeyTable="tblSurveyParticipant">
                <Table name="tblSurveyParticipant" />
                <Level name="PID" column="id" type="Integer" captionColumn="id" ordinalColumn="id" />
            </Hierarchy>
        </Dimension>

        <Dimension name="Time" foreignKey="participant_time_id" type="TimeDimension">
            <Hierarchy hasAll="true" primaryKey="id" name="Quarter">
                <Table name="aggTimeQuarter"/>
                <Level name="Year" column="year" type="Integer" uniqueMembers="true" levelType="TimeYears" ordinalColumn="year" />
                <Level name="Quarter" column="quarter" uniqueMembers="false" levelType="TimeQuarters"  ordinalColumn="quarter" />
                <Level name="Month" column="month" type="Integer" uniqueMembers="false" levelType="TimeMonths" ordinalColumn="month" />
                <Level name="Day" column="day" type="Integer" uniqueMembers="false" levelType="TimeDays" ordinalColumn="day" />
            </Hierarchy>
            <Hierarchy name="WeekISO8601" hasAll="true" primaryKey="id" caption="Week">
                <Table name="aggTimeWeekISO8601"/>
                <Level name="Year" column="year" type="Integer" uniqueMembers="true" levelType="TimeYears" ordinalColumn="year" />
                <Level name="Week" column="week" uniqueMembers="false" levelType="TimeWeeks" ordinalColumn="week" />
                <Level name="Day" column="day" type="Integer" uniqueMembers="false" levelType="TimeDays" ordinalColumn="day" />
            </Hierarchy>
            <Hierarchy name="WeekUS" hasAll="true" primaryKey="id" caption="Week">
                <Table name="aggTimeWeekUS"/>
                <Level name="Year" column="year" type="Integer" uniqueMembers="true" levelType="TimeYears" ordinalColumn="year" />
                <Level name="Week" column="week" uniqueMembers="false" levelType="TimeWeeks" ordinalColumn="week" />
                <Level name="Day" column="day" type="Integer" uniqueMembers="false" levelType="TimeDays" ordinalColumn="day" />
            </Hierarchy>
        </Dimension>

        <Measure name="%(name)s Respondents" column="participant_id" aggregator="distinct-count" datatype="Integer" />
        <Measure name="%(name)s Response Count" column="id" aggregator="distinct-count" datatype="Integer" />
        <Measure name="%(name)s Companies" column="company_id" aggregator="distinct-count" datatype="Integer" />
        <!--<Measure name="%(name)s Responses Text Count" column="response_text" formatString="Standard" datatype="Integer" aggregator="count" />-->
        <Measure name="%(name)s Responses Int Sum" column="response_int" formatString="Standard" datatype="Integer" aggregator="sum" />
        <Measure name="%(name)s Responses Float Sum" column="response_float" formatString="#,###.0" datatype="Integer" aggregator="sum" />

        <Measure name="%(name)s Weighted Int Sum" column="weighted_int" formatString="Standard" datatype="Integer" aggregator="sum" />
        <Measure name="%(name)s Weighted Float Sum" column="weighted_float" formatString="#,###.0" datatype="Integer" aggregator="sum" />


        <CalculatedMember name="%(name)s Responses Int Avg"
           dimension="Measures"
           formula="[Measures].[%(name)s Responses Int Sum] / [Measures].[%(name)s Response Count]"
           formatString="#,###.00"
           visible="true">
        </CalculatedMember>

        <CalculatedMember name="%(name)s Responses Float Avg"
           dimension="Measures"
           formula="[Measures].[%(name)s Responses Float Sum] / [Measures].[%(name)s Response Count]"
           formatString="#,###.00"
           visible="true">
        </CalculatedMember>

        <CalculatedMember name="%(name)s Responses Int Avg per Respondent"
           dimension="Measures"
           formula="[Measures].[%(name)s Responses Int Sum] / [Measures].[%(name)s Respondents]"
           formatString="#,###.00"
           visible="true">
        </CalculatedMember>

        <CalculatedMember name="%(name)s Responses Float Avg per Respondent"
           dimension="Measures"
           formula="[Measures].[%(name)s Responses Float Sum] / [Measures].[%(name)s Respondents]"
           formatString="#,###.00"
           visible="true">
        </CalculatedMember>

        <CalculatedMember name="%(name)s Weighted Int Avg"
           dimension="Measures"
           formula="[Measures].[%(name)s Weighted Int Sum] / [Measures].[%(name)s Response Count]"
           formatString="#,###.00"
           visible="true">
        </CalculatedMember>

        <CalculatedMember name="%(name)s Weighted Float Avg"
           dimension="Measures"
           formula="[Measures].[%(name)s Weighted Float Sum] / [Measures].[%(name)s Response Count]"
           formatString="#,###.00"
           visible="true">
        </CalculatedMember>

    </Cube>

    """ % {'name': self.name, 'fact_table': fact_table}

class ResponseCube(AbstractResponseCube):
    name = 'Response'

    def __xml(self):
        return self._xml('aggFullIQuestResponse')

    xml = property(__xml)

class FilteredResponseCube(AbstractResponseCube):
    name = 'FilteredResponse'

    def __xml(self):
        return self._xml('aggFilteredIQuestResponse')

    xml = property(__xml)

class VirtualFilteredResponseCube(CubeBase):
    name = 'VirtualFilteredResponse'

    def _get_xml(self):

        m = ['Respondents', 'Companies', 'Responses Int Sum', 'Responses Float Sum',
             'Responses Int Avg', 'Responses Int Avg per Respondent', 'Weighted Int Sum', 'Weighted Int Avg',
             'Responses Float Avg', 'Responses Float Avg per Respondent', 'Weighted Float Sum', 'Weighted Float Avg',
             'Response Count']

        measures = []
        cm = []
        for x in m:
            measures.append("""<VirtualCubeMeasure cubeName="Response" name="[Measures].[Response %s]" />""" % x)
            measures.append("""<VirtualCubeMeasure cubeName="FilteredResponse" name="[Measures].[FilteredResponse %s]" />""" % x)
            cm.append("""
        <CalculatedMember name="%(name)s"
           dimension="Measures"
           formula="IIF([Company].CurrentMember IS [Company].[All Companies], [Measures].[FilteredResponse %(name)s], [Measures].[Response %(name)s])"
           visible="true">
        </CalculatedMember>
            """ % {'name': x})


        return """
    <VirtualCube name="VirtualFilteredResponse">

        <CubeUsages>
            <CubeUsage cubeName="Response" />
            <CubeUsage cubeName="FilteredResponse" />
        </CubeUsages>

        <VirtualCubeDimension cubeName="Response" name="Company" />
        <VirtualCubeDimension cubeName="FilteredResponse" name="Item" />
        <VirtualCubeDimension cubeName="FilteredResponse" name="Question" />
        <VirtualCubeDimension cubeName="FilteredResponse" name="Answer" />
        <VirtualCubeDimension cubeName="FilteredResponse" name="Survey" />
        <VirtualCubeDimension cubeName="FilteredResponse" name="Participant" />
        <VirtualCubeDimension cubeName="FilteredResponse" name="Time" />
        <VirtualCubeDimension cubeName="FilteredResponse" name="Gender" />
        <VirtualCubeDimension cubeName="FilteredResponse" name="Invoice Country" />
        <VirtualCubeDimension cubeName="FilteredResponse" name="Person Country" />

        %s
        %s

    </VirtualCube>
    """ % ("\n".join(measures), "\n".join(cm))

    xml = property(_get_xml)

class VirtualResponseCube(CubeBase):
    name = 'VirtualResponse'

    def _get_xml(self):

        m = ['Respondents', 'Companies', 'Responses Int Sum', 'Responses Float Sum',
             'Responses Int Avg', 'Responses Int Avg per Respondent', 'Weighted Int Sum', 'Weighted Int Avg',
             'Responses Float Avg','Responses Float Avg per Respondent', 'Weighted Float Sum', 'Weighted Float Avg',
             'Response Count']

        measures = []
        cm = []
        for x in m:
            measures.append("""<VirtualCubeMeasure cubeName="Response" name="[Measures].[Response %s]" />""" % x)
            cm.append("""
        <CalculatedMember name="%(name)s"
           dimension="Measures"
           formula="[Measures].[Response %(name)s]"
           visible="true">
        </CalculatedMember>
            """ % {'name': x})


        return """
    <VirtualCube name="VirtualResponse">

        <CubeUsages>
            <CubeUsage cubeName="Response" />
        </CubeUsages>

        <VirtualCubeDimension cubeName="Response" name="Company" />
        <VirtualCubeDimension cubeName="Response" name="Item" />
        <VirtualCubeDimension cubeName="Response" name="Question" />
        <VirtualCubeDimension cubeName="Response" name="Answer" />
        <VirtualCubeDimension cubeName="Response" name="Survey" />
        <VirtualCubeDimension cubeName="Response" name="Participant" />
        <VirtualCubeDimension cubeName="Response" name="Time" />
        <VirtualCubeDimension cubeName="Response" name="Gender" />
        <VirtualCubeDimension cubeName="Response" name="Invoice Country" />
        <VirtualCubeDimension cubeName="Response" name="Person Country" />

        %s
        %s

    </VirtualCube>
    """ % ("\n".join(measures), "\n".join(cm))

    xml = property(_get_xml)

class AccountRole():

    def __init__(self, session, ds):
        self.sa_session = session
        self.ds = ds

    def get_all(self):
        return (
            self.ResponseCube_account_role,
        )

    def ResponseCube_account_role(self, account, surveys, cube, validated_publications):
        """
        Method used for cubes with same dimensions and hierarchies
        """

        rc = IQubeRoleController(self.sa_session, account, self.ds, cube)
        rc.grant_cube(IQubeRole.ACCESS_ALL)

        has_survey_access = False
        for survey in surveys:

            publications = survey.publications
            privileged_publications = []
            for publication in publications:
                if publication not in validated_publications:
                    continue
                privileged_publications.append(publication)
            # account does have privs on the survey, but not on any of its publication
            # don't give access to the Survey dimension
            if len(privileged_publications) == 0:
                continue

            rc.grant_member('[Survey].[%s]' % survey.id, IQubeRole.ACCESS_ALL)

            # having this means it'll return ALL publications for this survey
            for publication in privileged_publications:
                member = '[Survey].[%s].[%s]' % (survey.id, publication.id)
                rc.grant_member(member, IQubeRole.ACCESS_ALL)

            has_survey_access = True

        if has_survey_access:
            rc.grant_hierarchy('[Survey]', IQubeRole.ACCESS_CUSTOM, IQubeRole.ROLLUP_POLICY_PARTIAL)
        else:
            rc.grant_hierarchy('[Survey]', IQubeRole.ACCESS_NONE)

        # code below disabled for now.. huge slowdown
        """
        # companies
        # notice: rollup policy is set to full. this is required for anonymous surveys to work.
        # if its set to partial then anonymous surveys wont give result because it joins with company
        contact_controller = ContactController(self.sa_session, account.id)
        rc.grant_hierarchy('[Company]', DataSourceRole.ACCESS_CUSTOM, DataSourceRole.ROLLUP_POLICY_FULL)
        try:
            contacts = contact_controller.get_contacts(permission='view_survey_publication_response')
        except PermissionDeniedException: # we do not have rights to contacts
            contacts = []

        company_ids = []
        for contact in contacts:
            if contact.company is not None and contact.company.id not in company_ids:
                company_ids.append(contact.company.id)

                member = '[Company].[%s]' % (contact.company.id)
                rc.grant_member(member, DataSourceRole.ACCESS_ALL)
        """

        return rc.to_list()

class ResponseCubeRole(RoleBase):

    def get_role_list(self, account_id):
        """
        returns a role list with access on survey/publication level
        """

        c = AbstractController(self.sa_session, account_id=account_id)
        c.assert_permissions('view_survey_publication_response')
        account = c.get_account()

        publications = c.get_privileged_object(SurveyPublication, priv=SurveyPublication.PRIV_VIEW_RESPONSE,
            where=SurveyPublication.status.in_([SurveyPublication.STATUS_ONLINE, SurveyPublication.STATUS_FINISHED]))
        surveys = set([p.survey for p in publications if c.has_privileges_on_object(p.survey, Survey.PRIV_VIEW)])
        if len(surveys) == 0:
            raise PublicSecurityException('no access to report data')

        rv = []
        account_roles = AccountRole(self.sa_session, self.ds)
        for f in account_roles.get_all():
            for cube in ['VirtualResponse', 'VirtualFilteredResponse', 'PublicationStatistics']:
                rv += f(account, surveys, cube, publications)

        return rv

projects = (IQuestProject, )

