# Copyright (c) 2012 BaseSoft B.V.
# See LICENSE for details.

import datetime

from django.conf import settings
from django.utils.translation import ugettext as _
from django.utils import simplejson

from basesoft.util.ordered_dict import OrderedDict
from basesoft.web.contrib.contact.controllers import ContactController
from basesoft.web.core.exceptions import PermissionDeniedException

from iqportal.iqube.core.controllers import DataSourceController, MondrianMemberSerialize,\
    ConfigurableWidgetController
from iqportal.iquest.compose.controllers import ComposeController
from iqportal.iqreport.core import rpc as iqreport_rpc
from iqportal.iquest.report.controllers import IQuestReportPublicationStateController
from iqportal.iqube.compose.controllers import MdxCalculatedMemberController

class IQuestReportRPCHandler:
    def __init__(self, request, **kwargs):
        self.sa_session = request.Session()
        self.account_id = request.session['user_id']
        self.dsc = DataSourceController(self.sa_session, account_id=self.account_id)
        self.cube = 'VirtualResponse'

    def get_available_dimensions(self, dimensions=None):
        ds = self.dsc.get_datasource_by('mondrian_project', '%s.IQuest' % settings.SETTINGS_MODULE).pop()
        client = ds.connect(self.sa_session, self.account_id)

        possible = ['Survey', 'Question', 'Answer', 'Gender', 'Person Country', 'Measures', 'Invoice Country', 'Time']
        if dimensions is not None:
            possible = dimensions.keys()

        dims = client.get_dimensions(self.cube)
        rv = []
        for dim in dims:
            if dim['dimension_caption'] not in possible:
                continue
            x = client.get_dimension_levels(self.cube, dim['dimension_unique_name'])
            l = x.pop(0)
            if l['level_type'] == '1':
                l = x.pop(0)
            rv.append({'label': dim['dimension_caption'], 'value': l['level_unique_name']})
        return sorted(rv, key=lambda k: k['label'])

    def _serialize_member(self, x):
        # separators used to be compact like the JS one.
        return {'label': x['member_caption'], 'value': MondrianMemberSerialize(x)}

    def _get_calculated_members(self, ds, dimension):
        mcmc = MdxCalculatedMemberController(self.sa_session, account_id=self.account_id)
        cms = []
        for cm in mcmc.get_calculated_members_for(ds, self.cube, dimension=dimension):
            cms.append({'value': cm.id, 'label': cm.label})
        return cms

    def get_publications(self, survey_id=None, presets=False, grouped=True):
        if survey_id is None or survey_id == '':
            return []
        # TODO: add all member? #3264

        # get XMLA result for survey.children
        ds = self.dsc.get_datasource_by('mondrian_project', '%s.IQuest' % settings.SETTINGS_MODULE).pop()
        client = ds.connect(self.sa_session, self.account_id)
        pubs = [self._serialize_member(i) for i in client.get_member_tree_op(self.cube, '[Survey].[%s]' % survey_id, client.MDTREEOP_CHILDREN)]
        pubs.reverse() # we want them from new to old (#3766).
        if not presets:
            return pubs

        ret = OrderedDict()
        preset_list = []
        preset_def = {
            'children': {
                'caption': _('All publications'),
                'method': '.Children'
            },
            'last_child': {
                'caption': _('Latest publication'),
                'method': '.LastChild'
            },
            'first_child': {
                'caption': _('First publication'),
                'method': '.FirstChild'
            }
        }
        for i in presets:
            if not preset_def.has_key(i):
                continue

            definition = preset_def[i]

            d = client.get_member_tree_op(self.cube, '[Survey].[%s]' % survey_id, client.MDTREEOP_SELF)[0]
            d['member_caption'] = definition['caption'];
            d['member_unique_name'] = d['member_unique_name'] + definition['method'];
            preset_list.append(self._serialize_member(d))

        # TODO: should this be per Survey?
        cms = self._get_calculated_members(ds, 'Survey')

        if grouped:
            ret.update([(_('Presets'), preset_list)])
            ret.update([(_('Publication'), pubs)])
            if len(cms) > 0: # TODO: check if enabled in config.
                ret.update([(_('Calculated members'), cms)])
            return ret
        return preset_list+pubs

    def get_q_and_a_by_qa(self, item_id, survey_id, locale_id=None):
        controller = ComposeController(self.sa_session, account_id=self.account_id)
        if survey_id is None or survey_id == '':
            return

        if locale_id is None:
            survey = controller.get_surveys(survey_id).pop()
            locale_id = survey.fallback_locale.id

        ds = self.dsc.get_datasource_by('mondrian_project', '%s.IQuest' % settings.SETTINGS_MODULE).pop()
        client = ds.connect(self.sa_session, self.account_id)

        questions = [self._serialize_member(i)['value'] for i in client.get_member_tree_op(self.cube, '[Question].[%s].[%s]' % (item_id, locale_id), client.MDTREEOP_CHILDREN)]
        answers = [self._serialize_member(i)['value'] for i in client.get_member_tree_op(self.cube, '[Answer].[%s].[%s]' % (item_id, locale_id), client.MDTREEOP_CHILDREN)]
        return {'Question': questions, 'Answer': answers}

    def get_qas_by_survey(self, survey_id, locale_id=None):
        if survey_id is None or survey_id == '':
            return []

        controller = ComposeController(self.sa_session, account_id=self.account_id)
        survey = controller.get_surveys(survey_id).pop()

        if locale_id is None:
            locale_id = survey.fallback_locale.id
        else:
            locale_id = int(locale_id)
            assert locale_id in [l.id for l in survey.available_locales]

        ds = self.dsc.get_datasource_by('mondrian_project', '%s.IQuest' % settings.SETTINGS_MODULE).pop()
        client = ds.connect(self.sa_session, self.account_id)

        items = survey.itemQAs
        ret = []
        for item in items:
            item_data = client.get_member_tree_op(self.cube, '[Item].[%s].[%s]' % (locale_id, item.id), client.MDTREEOP_SELF)
            ret.append(self._serialize_member(item_data[0]))

        return ret

    def get_survey_qa_items(self, survey_id, types=None, obj_types=None, locale_id=None):
        if survey_id is None or survey_id == '':
            return []

        controller = ComposeController(self.sa_session, account_id=self.account_id)
        survey = controller.get_surveys(survey_id).pop()

        if locale_id is None:
            locale_id = survey.fallback_locale.id
        else:
            locale_id = int(locale_id)
            assert locale_id in [l.id for l in survey.available_locales]

        if types is not None:
            if not isinstance(types, list):
                types = [types]

        if obj_types is not None:
            if not isinstance(obj_types, list):
                obj_types = [obj_types]

        ds = self.dsc.get_datasource_by('mondrian_project', '%s.IQuest' % settings.SETTINGS_MODULE).pop()
        client = ds.connect(self.sa_session, self.account_id)

        items = survey.itemQAs
        ret = []
        for item in items:
            if types is not None and item.type not in types:
                continue
            if obj_types is not None and item.object_name not in obj_types:
                continue
            item_data = client.get_member_tree_op(self.cube, '[Item].[%s].[%s]' % (locale_id, item.id), client.MDTREEOP_SELF)
            if len(item_data) == 0:
                return []
            ret.append(self._serialize_member(item_data[0]))

        return ret

    def get_elements_by_item(self, survey_id, item_id):
        if survey_id in ['', None]:
            return []
        if item_id in ['', None]:
            return []

        return {
            'questions': self.get_elements_by_survey(survey_id, item_id, type=1, by_item=False),
            'answers': self.get_elements_by_survey(survey_id, item_id, by_item=False)
        }

    def get_elements_by_survey(self, survey_id, item_ids=None, locale_id=None, type=0, by_item=True):
        """
        @param survey_id: id of the survey to get the Q or A for
        @param item_ids: ids of the specific items we need
        @param locale_id: id of the locale, fallback is the default
        @param type: 0 - Answer, 1 - Question
        """
        if survey_id is None or survey_id == '':
            return []
        controller = ComposeController(self.sa_session, account_id=self.account_id)
        survey = controller.get_surveys(survey_id).pop()

        if locale_id is None:
            locale_id = survey.fallback_locale.id
        else:
            locale_id = int(locale_id)
            assert locale_id in [l.id for l in survey.available_locales]

        ds = self.dsc.get_datasource_by('mondrian_project', '%s.IQuest' % settings.SETTINGS_MODULE).pop()
        client = ds.connect(self.sa_session, self.account_id)

        items = survey.itemQAs
        if item_ids not in [None, '']:
            if not isinstance(item_ids, list):
                item_ids = [item_ids]
            item_ids = [int(i) for i in item_ids]
            seli = [i for i in items if i.id in item_ids]
            items = seli

        elements = OrderedDict()
        rv = []
        for item in items:
            el = 'Question'
            if type == 0:
                el = 'Answer'
            item_data = client.get_member_tree_op(self.cube, '[Item].[%s].[%s]' % (locale_id, item.id), client.MDTREEOP_SELF)
            # just item caption is not enough because multiple items can have the same caption
            # so lets add item position also
            members = client.get_member_tree_op(self.cube, '[%s].[%s].[%s]' % (el, item.id, locale_id), client.MDTREEOP_CHILDREN)
            serialized = [self._serialize_member(i) for i in members]
            elements.update([('%s - %s' % (item.position+1, item_data[0]['member_caption']), serialized)])
            rv = rv + serialized

        # TODO: should this be done by element? -> This could require a change on Calculated Member level (hierarchy level vs dimension)
        if len(rv) > 0:
            cms = self._get_calculated_members(ds, 'Answer' if type == 0 else 'Question')
            if len(cms) > 0:
                elements.update([(_('Calculated members'), cms)])
                rv = rv + cms

        if by_item:
            return elements
        return rv

    def get_questions_by_survey(self, survey_id, item_ids=None, locale_id=None):
        return self.get_elements_by_survey(survey_id, item_ids, locale_id, 1)

    def get_answers_by_survey(self, survey_id, item_ids=None, locale_id=None):
        return self.get_elements_by_survey(survey_id, item_ids, locale_id, 0)

    def get_time_children(self, ds, cube, config, survey_id, values):
        """
        this method is an extension of get_dimension_options() but receives an
        extra argument survey_id which is used to determine the min/max years based on
        its publications
        """

        # number of years to show extra in case no end_date is specified
        EXTRA_YEARS = 4

        controller = ComposeController(self.sa_session, account_id=self.account_id)
        if survey_id is None or survey_id.strip() == '':
            min_year = 1970
            max_year = 2050
        else:
            survey = controller.get_surveys(survey_id).pop()

            min_year = None
            max_year = None

            for publication in survey.publications:

                if min_year is None:
                    min_year = publication.begin_date.year
                elif publication.begin_date.year < min_year:
                    min_year = publication.begin_date.year

                if publication.end_date is None:
                    max_year = datetime.datetime.now().year + EXTRA_YEARS
                elif max_year is None:
                    max_year = publication.end_date.year

                if publication.end_date is not None and publication.end_date.year > max_year:
                    max_year = publication.end_date.year

            if min_year is None:
                min_year = 1970

            if max_year is None:
                max_year = max_year = datetime.datetime.now().year + EXTRA_YEARS

        # use the original get_dimension_options() to get the result. but then filter per group for min/max years
        ds = self.dsc.get_datasource_by('mondrian_project', '%s.IQuest' % settings.SETTINGS_MODULE).pop()
        cwc = ConfigurableWidgetController(self.sa_session, account_id=self.dsc.get_account().id)
        grouped_rv = cwc.get_dimension_options(ds, cube, config, values)

        # group_name can be year, quarter, week, etc.. check the first member and see
        # if its part of the year level. then limit those years to the ones for this survey
        for group_name, members in grouped_rv.iteritems():

            if len(members) > 0:
                member_0 = simplejson.loads(members[0]['value'])
                if member_0['level_unique_name'].endswith('.[Year]'):

                    new_members = []
                    for member_str in members:

                        year = int( simplejson.loads(member_str['value'])['member_name'])
                        if year >= min_year and year <= max_year:
                            new_members.append(member_str)

                    grouped_rv[group_name] = new_members

        return grouped_rv


    def get_company_members(self, *args, **kwargs):
        """
        we dont use roles anymore for getting available companies.. so here we use the contactcontroller and limit the result
        """

        contact_controller = ContactController(self.sa_session, self.dsc.get_account().id)
        try:
            contacts = contact_controller.get_contacts(permission='view_survey_publication_response')
        except PermissionDeniedException: # we do not have rights to contacts
            contacts = []

        rv = {}
        rv['Companies'] = []

        company_ids = []
        for contact in contacts:
            if contact.company is not None and contact.company.id not in company_ids:
                company_ids.append(contact.company.id)

                rv['Companies'].append({
                    'value': simplejson.dumps({
                        "dimension_unique_name": "[Company]",
                        "member_unique_name": "[Company].[%s]" % contact.company.id,
                        "depth": "1",
                        "level_unique_name": "[Company].[Company]",
                        "member_name": contact.company.id,
                        "hierarchy_unique_name": "[Company]"
                        }),
                    'label': contact.company.name
                })


        return rv

class ReportPublicationHandler(iqreport_rpc.ReportPublicationHandler):
    controller_class = IQuestReportPublicationStateController

handlers = (IQuestReportRPCHandler, ReportPublicationHandler, )
