# Copyright (c) 2010 BaseSoft B.V.
# See LICENSE for details.

from sqlalchemy import case, func, and_
from django.utils.translation import ugettext as _

from basesoft.web.core.util.custom_query import CustomQuery, CustomQueryField, CustomQueryModel, DATA_TYPE_DATE, DATA_TYPE_BOOLEAN
from basesoft.web.contrib.contact.models import Person, Company, Contact

from iqportal.iquest.response.models import SurveyParticipantSession, SurveyParticipant, SurveyQAResponse
from iqportal.iquest.core.models import Survey, SurveyPublication
from basesoft.web.contrib.compose.models import ComposeInvitee

class PublicationInviteesCustomQuery(CustomQuery):
    def validate_access(self):
        self.assert_permissions('view_survey_publication_response')

    def tables(self):
        return (
            CustomQueryModel(Survey),
            CustomQueryModel(SurveyPublication,
                             onclause=Survey.id==SurveyPublication.survey_id),
            CustomQueryModel(ComposeInvitee,
                             onclause=ComposeInvitee.publication_id==SurveyPublication.id),
            CustomQueryModel(SurveyParticipant,
                             onclause=ComposeInvitee.id==SurveyParticipant.invitee_id, use_outerjoin=True),
            CustomQueryModel(SurveyParticipantSession,
                             onclause=SurveyParticipantSession.participant_id==SurveyParticipant.id, use_outerjoin=True),
            CustomQueryModel(Contact, onclause=ComposeInvitee.contact_id==Contact.id, use_outerjoin=True, show_archived=True),
            CustomQueryModel(Person, onclause=Contact.person_id==Person.id, use_outerjoin=True),
            CustomQueryModel(Company, onclause=Contact.company_id==Company.id, use_outerjoin=True),
        )

    def fields(self):
        return (
            CustomQueryField(ComposeInvitee.id, _('Invitee ID'), key='invitee_id'),
            CustomQueryField(func.concat(case([(Person.first_name!=None, Person.first_name)], else_=""),
                                        " ",
                                        case([(Person.middle_name!=None, Person.middle_name)], else_=""),
                                        " ",
                                        case([(Person.last_name!=None, Person.last_name)], else_=""),
                                        ), _('Participant'), key='participant'),
            CustomQueryField(Company.name, _('Company'), key='company', is_filter=True),
            CustomQueryField(func.max(SurveyParticipantSession.created), _('Last accessed'), key='last_accessed',
                             type=DATA_TYPE_DATE),
            CustomQueryField(Person.email, _('Email'), key='email'),
            CustomQueryField(case([(SurveyParticipant.id!=None, 1)], else_=0), _('Is participant'), key='is_participant',
                             is_filter=True, type=DATA_TYPE_BOOLEAN),
            CustomQueryField(case([(SurveyParticipant.is_respondent>0, 1)], else_=0), _('Is respondent'), key='is_respondent',
                             is_filter=True, type=DATA_TYPE_BOOLEAN),
            CustomQueryField(SurveyParticipant.id, _('Responses'), key='details'),
            CustomQueryField(SurveyParticipant.id, _('Participant ID'), key='participant_id'),
            CustomQueryField(ComposeInvitee.invitee_hash, _('Invitee hash'), key='invitee_hash'),
            CustomQueryField(SurveyPublication.id, _('Publication ID'), key='publication_id'),
        )

    def set_where(self):
        self.where = and_(ComposeInvitee.contact_id!=None, Survey.deleted==False, SurveyPublication.deleted==False)

    group_by = (ComposeInvitee.id, )


class PublicationResponsesCustomQuery(CustomQuery):
    def validate_access(self):
        self.assert_permissions('view_survey_publication_response')

    def tables(self):
        return (
            CustomQueryModel(Survey),
            CustomQueryModel(SurveyPublication,
                             onclause=Survey.id==SurveyPublication.survey_id),
            CustomQueryModel(ComposeInvitee,
                             onclause=ComposeInvitee.publication_id==SurveyPublication.id),
            CustomQueryModel(SurveyParticipant,
                             onclause=ComposeInvitee.id==SurveyParticipant.invitee_id),
            CustomQueryModel(SurveyParticipantSession,
                             onclause=SurveyParticipantSession.participant_id==SurveyParticipant.id, use_outerjoin=True),
            CustomQueryModel(Contact, onclause=ComposeInvitee.contact_id==Contact.id, use_outerjoin=True, show_archived=True),
            CustomQueryModel(Person, onclause=Contact.person_id==Person.id, use_outerjoin=True),
            CustomQueryModel(Company, onclause=Contact.company_id==Company.id, use_outerjoin=True),
        )

    def fields(self):
        return (
            CustomQueryField(ComposeInvitee.id, _('Invitee ID'), key='invitee_id'),
            CustomQueryField(func.concat(case([(Person.first_name!=None, Person.first_name)], else_=""),
                                        " ",
                                        case([(Person.middle_name!=None, Person.middle_name)], else_=""),
                                        " ",
                                        case([(Person.last_name!=None, Person.last_name)], else_=""),
                                        ), _('Participant'), key='participant'),
            CustomQueryField(Company.name, _('Company'), key='company'),
            CustomQueryField(func.max(SurveyParticipantSession.created), _('Last accessed'), key='last_accessed',
                             type=DATA_TYPE_DATE),
            CustomQueryField(Person.email, _('Email'), key='email'),
            CustomQueryField(SurveyParticipant.is_respondent, _('Responses type'), key='responses_type'),
            CustomQueryField(SurveyParticipant.id, _('Responses'), key='details'),
            CustomQueryField(SurveyParticipant.id, _('Participant ID'), key='participant_id'),
            CustomQueryField(ComposeInvitee.invitee_hash, _('Invitee hash'), key='invitee_hash'),
            CustomQueryField(SurveyPublication.id, _('Publication ID'), key='publication_id'),
        )

    def set_where(self):
        self.where = and_(SurveyParticipant.is_respondent>0, Survey.deleted==False, SurveyPublication.deleted==False)

    group_by = (SurveyParticipant.id, )


custom_queries = (PublicationInviteesCustomQuery, PublicationResponsesCustomQuery, )