# Copyright (c) 2008 BaseSoft B.V.
# See LICENSE for details.

"""
This file contains the objects, functions and variables to setup the response
part of the IQuest application.
"""

import datetime
from sqlalchemy import Column, Integer, ForeignKey, DateTime, DECIMAL, UnicodeText, Float, SmallInteger, UniqueConstraint
from basesoft.sqlalchemy.table import IQTable
from basesoft.web.contrib.license import models as license_models
from django.utils.translation import ugettext as _

__mapping_defined = False
__tables_defined = False
tables = {}

def reset():
    global __mapping_defined
    global __tables_defined

    __mapping_defined = False
    __tables_defined = False

def get_tables():
    return tables

def do_tables(metadata):
    global __tables_defined
    if not __tables_defined:

        #
        # SurveyParticipant is the bridge between the responses and survey.
        #
        # In case a survey is non-anonymous:
        #  - The table gets filled with everyone that can access the survey at the moment of publication by the scheduler
        #  - contact_id references to the contact that can fill in the survey
        #  - Even if a respondent comes back multiple times for the same SurveyPublication this table will get one entry.
        #    To see how often a respondent came back see SurveyParticipantSession.
        #
        # When the survey is anonymous:
        #  - The table will receive a new entry for each participant that accesses the survey.
        #  - contact_id remains null
        #
        tblSurveyParticipant = IQTable('tblSurveyParticipant', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('invitee_id', Integer, ForeignKey('tblComposeInvitee.id')),
            Column('locale_id', Integer, ForeignKey('tblLocale.id')),
            Column('is_respondent', SmallInteger, nullable=False, default=0),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        # SurveyParticipantSession gets an entry for every time a respondent accesses the survey for a certain
        # time (created to last_update)
        tblSurveyParticipantSession = IQTable('tblSurveyParticipantSession', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('participant_id', Integer, ForeignKey('tblSurveyParticipant.id'), nullable=False),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        # This holds the actual respondent data and links a SurveyParticipant to a Question/Answer.
        # For future reporting link the SurveyParticipantSession so we can see in which session he filled out the QA.
        tblSurveyQAResponse = IQTable('tblSurveyQAResponse', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('participant_id', Integer, ForeignKey('tblSurveyParticipant.id'), nullable=False),
            Column('question_id', Integer, ForeignKey('tblSurveyItemQAElement.id'), nullable=False),
            Column('answer_id', Integer, ForeignKey('tblSurveyItemQAElement.id'), nullable=False),
            Column('participant_session_id', Integer, ForeignKey('tblSurveyParticipantSession.id'), nullable=False),
            Column('response_text', UnicodeText(), nullable=True),
            Column('response_int', Integer, nullable=True),
            Column('response_float', DECIMAL(20,5), nullable=True),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now),
            UniqueConstraint('participant_id', 'question_id', 'answer_id', name='iquest_response_unique_index')
        )

        # This holds all responses that were deleted from tblSurveyQAResponse;
        tblSurveyQAResponseHistory = IQTable('tblSurveyQAResponseHistory', metadata,
            # add an autoincrement primary key, since having (participant_id, question_id, answer_id) as primary key
            # doesn't apply because there could be responses that will be removed more than once for a participant
            # and we need to store all of them
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('participant_id', Integer, ForeignKey('tblSurveyParticipant.id')),
            Column('question_id', Integer, ForeignKey('tblSurveyItemQAElement.id')),
            Column('answer_id', Integer, ForeignKey('tblSurveyItemQAElement.id')),
            Column('participant_session_id', Integer, ForeignKey('tblSurveyParticipantSession.id')),
            Column('response_text', UnicodeText(), nullable=True),
            Column('response_int', Integer, nullable=True),
            Column('response_float', DECIMAL(20,5), nullable=True),

            # also store the dates for when the response was originally created and updated
            # (from the original tblSurveyQAResponse)
            Column('response_created', DateTime(), nullable=False),
            Column('response_last_updated', DateTime(), nullable=False),

            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        tables['tblSurveyParticipant'] = tblSurveyParticipant
        tables['tblSurveyParticipantSession'] = tblSurveyParticipantSession
        tables['tblSurveyQAResponse'] = tblSurveyQAResponse
        tables['tblSurveyQAResponseHistory'] = tblSurveyQAResponseHistory

        __tables_defined = True

def do_mappings():
    """
    This function is called by Django middleware when needed.
    """
    global __mapping_defined
    if not __mapping_defined:
        from sqlalchemy.orm import mapper, relation
        from basesoft.web.contrib.i18n import models as i18n_models
        from basesoft.web.contrib.compose import models as compose_models
        from iqportal.iquest.core import models as core_models
        from iqportal.iquest.core.sa_datamapper import get_tables as core_tables
        from iqportal.iquest.response import models as response_models

        mapper(response_models.SurveyParticipant, tables['tblSurveyParticipant'],
               properties = {'invitee': relation(compose_models.ComposeInvitee),
                             'locale': relation(i18n_models.Locale),
                             })

        mapper(response_models.SurveyParticipantSession, tables['tblSurveyParticipantSession'],
               properties = {'participant': relation(response_models.SurveyParticipant)})

        mapper(response_models.SurveyQAResponse, tables['tblSurveyQAResponse'],
               properties = {'participant': relation(response_models.SurveyParticipant),
                             'question': relation(core_models.SurveyItemQAElement,
                                            primaryjoin=tables['tblSurveyQAResponse'].c.question_id==core_tables()['tblSurveyItemQAElement'].c.id),
                             'answer': relation(core_models.SurveyItemQAElement,
                                            primaryjoin=tables['tblSurveyQAResponse'].c.answer_id==core_tables()['tblSurveyItemQAElement'].c.id),
                             'participant_session': relation(response_models.SurveyParticipantSession),
                             })
        mapper(response_models.SurveyQAResponseHistory, tables['tblSurveyQAResponseHistory'],
               properties = {'participant': relation(response_models.SurveyParticipant),
                             'question': relation(core_models.SurveyItemQAElement,
                                            primaryjoin=tables['tblSurveyQAResponseHistory'].c.question_id==core_tables()['tblSurveyItemQAElement'].c.id),
                             'answer': relation(core_models.SurveyItemQAElement,
                                            primaryjoin=tables['tblSurveyQAResponseHistory'].c.answer_id==core_tables()['tblSurveyItemQAElement'].c.id),
                             'participant_session': relation(response_models.SurveyParticipantSession),
                             })

    __mapping_defined = True

def app_permissions_category():
    """
    Returns a category for all permissions defined for this app
    """
    return license_models.PermissionCategory(u'IQuest Response')

def app_permissions():
    """
    The permissions used within this application. Can be sync'd with the
    `iqsyncpermissions` command.
    """
    return [
        license_models.Permission(u'view_survey_publication_response', u"""
            This gives the right to view a survey publication's responses.
            """, _("View survey publication responses")),
        license_models.Permission(u'create_survey_publication_response', u"""
            This gives the right to create survey publication's responses.
            """, _("Create survey publication responses")),
        license_models.Permission(u'update_survey_publication_response', u"""
            This gives the right to update a survey publication's responses.
            """, _("Update survey publication responses")),
        license_models.Permission(u'delete_survey_publication_response', u"""
            This gives the right to delete a survey publication's responses.
            """, _("Delete survey publication responses")),
        ]

def app_configs():
    """
    The account configs available for this application. This can be sync'd with the
    `iqsyncpermissions` command.
    """
    from basesoft.web.contrib.account import models as account_models
    configs = []

    config = account_models.Config()
    config.name = 'basesoft.web.response.invitee_expiry_days'
    config.type = 'integer'
    config.default = 0 # 0 = never
    config.description = u'Number of days before an invitee hash expires.'
    configs.append(config)

    return configs
