# Copyright (c) 2011 BaseSoft B.V.
# See LICENSE for details.

"""
SAP Core Controller module
"""

from sqlalchemy import and_
from sqlalchemy.sql.expression import distinct
from django.conf import settings
from django.utils.translation import ugettext as _
from basesoft.web.core.util.controllers import AbstractController, transaction_decorator
from basesoft.web.core.exceptions import SecurityException, PublicUserException

from basesoft.web.contrib.license.controllers import LicenseController

from iqportal.sap.core.models import SAPProject, SAPComponentElement, SAPComponentElementGroup, SAPComponentElementType,\
    SAPProjectEstimate, SAPProjectActual, SAPProjectPeriod, SAPProjectComponentElement

class SAPCoreController(AbstractController):
    """
    """

    def get_projects(self, id=None, privs=SAPProject.PRIV_VIEW):
        """
        Return a list with SAPProject.
        """
        self.assert_permissions('view_sap_project')
        return self.get_privileged_object_by_id(SAPProject, id, privs)

    def get_components(self, id=None, type_id=None, privs=SAPComponentElement.PRIV_VIEW):
        """
        """
        self.assert_permissions('view_sap_component_element')
        filter, fo = self.get_query_filter_on_privileges(SAPComponentElement, SAPComponentElementGroup, privs)

        if id is not None:
            if not isinstance(id, list):
                id = [id]
            filter = and_(filter, SAPComponentElement.id.in_(id))

        if type_id is not None:
            if not isinstance(type_id, list): type_id = [type_id]
            filter = and_(filter, SAPComponentElement.type_id.in_(type_id))

        filter = and_(filter, and_(SAPComponentElement.deleted==False, SAPComponentElement.archived==False))

        return self.sa_session.query(SAPComponentElement).select_from(fo).filter(filter).all()

    def get_component_types_by(self, field=None, values=None):
        """
        """
        filter = None
        if field is not None:
            assert field in ['name', 'id']
            if values is not None:
                filter = getattr(SAPComponentElementType, field).in_(values)
        return self.sa_session.query(SAPComponentElementType).filter(filter).all()

    def get_project_state(self, id):
        project = self.get_projects(id)
        if len(project) == 0:
            raise SecurityException('Invalid project id %s' % id)
        project = project[0]

        return {
            'dbID': project.id,
            'name': project.name,
            # it should be just one license
            'license': [pg.group.license.id for pg in project.groups if pg.group.id != self.get_default_group().id][0],
            'components': [c.id for c in project.component_elements]
        }

    def load_project_components(self, project_id):
        """
        Returns project components hierarchy in order to build form for estimates and actuals.
        """
        project = self.get_projects(project_id)
        if len(project) == 0:
            raise SecurityException('Invalid project id %s' % project_id)
        project = project[0]

        pc = {}
        labels = {}
        for c in project.component_elements:
            # child node - must be added to it's parent
            if c.parent_component_element:
                if not pc.has_key(c.parent_component_element.id):
                    pc[c.parent_component_element.id] = []
                pc[c.parent_component_element.id].append(c.id)
                labels[c.parent_component_element.id] = c.parent_component_element.name
                labels[c.id] = c.name
            # parent node - must add all it's children as well
            else:
                pc[c.id] = []
                labels[c.id] = c.name
                for ch in c.child_component_elements:
                    pc[c.id].append(ch.id)
                    labels[ch.id] = ch.name
        return (pc, labels)

    def load_project_estimates(self, project_id, period_id=None):
        if period_id is None:
            return {}
        estimates = self.sa_session.query(SAPProjectEstimate).join(SAPProjectComponentElement).filter(and_(
             SAPProjectComponentElement.project_id==project_id, SAPProjectEstimate.project_period_id==period_id)).all()

        ret = {}
        for pe in estimates:
            ret['period_id'] = pe.project_period_id
            ret['period'] = pe.project_period.year
            for m in SAPProjectEstimate.cols_i18n:
                if getattr(pe, m) is not None:
                   key = '%s_%s' % (pe.project_component_element.component_element_id, m)
                   ret[key] = getattr(pe, m)
        return ret

    def load_project_actuals(self, project_id, period_id=None):
        if period_id is None:
            return {}
        actuals = self.sa_session.query(SAPProjectActual).join(SAPProjectComponentElement).filter(and_(
             SAPProjectComponentElement.project_id==project_id, SAPProjectActual.project_period_id==period_id)).all()

        ret = {}
        for pe in actuals:
            ret['period_id'] = pe.project_period_id
            ret['period'] = pe.project_period.year
            ret['actuals_date'] = pe.actuals_date
            for m in SAPProjectActual.cols_i18n:
                if getattr(pe, m) is not None:
                   key = '%s_%s' % (pe.project_component_element.component_element_id, m)
                   ret[key] = getattr(pe, m)
        return ret

    def _get_period(self, project, year):
        period = self.sa_session.query(SAPProjectPeriod).filter(and_(
                                        SAPProjectPeriod.project==project, SAPProjectPeriod.year==year)).all()
        if len(period) == 0:
            period = SAPProjectPeriod()
            period.project = project
            period.year = year
        else:
            period = period[0]
        return period

    @transaction_decorator
    def save_project_estimates(self, project_id, state):
        """
        """
        project = self.get_projects(project_id)
        if len(project) == 0:
            raise SecurityException('Invalid project id %s' % project_id)
        project = project[0]
        period = self._get_period(project, state['period'])
        project_component_elements = self.sa_session.query(SAPProjectComponentElement).filter(
                                    SAPProjectComponentElement.project==project).all()

        for pc in project_component_elements:
            c = pc.component_element
            # check if we have data for this period and this component
            estimate = self.sa_session.query(SAPProjectEstimate).filter(and_(SAPProjectEstimate.project_component_element==pc,
                                                                             SAPProjectEstimate.project_period==period)).all()
            if len(estimate) == 0:
                estimate = SAPProjectEstimate()
                estimate.project_period = period
                estimate.project_component_element = pc
            else:
                estimate = estimate[0]
            has_values = False
            for m in SAPProjectEstimate.cols_i18n:
                key = '%s_%s' % (c.id, m)
                if state.has_key(key) and state[key] not in ['', None]:
                    setattr(estimate, m, state[key])
                    has_values = True
            if has_values:
                self.sa_session.add(estimate)

    @transaction_decorator
    def save_project_actuals(self, project_id, state):
        """
        """
        project = self.get_projects(project_id)
        if len(project) == 0:
            raise SecurityException('Invalid project id %s' % project_id)
        project = project[0]
        period = self._get_period(project, state['period'])
        project_component_elements = self.sa_session.query(SAPProjectComponentElement).filter(
                                    SAPProjectComponentElement.project==project).all()

        for pc in project_component_elements:
            c = pc.component_element
            # check if we have data for this period and this component
            actual = self.sa_session.query(SAPProjectActual).filter(and_(SAPProjectActual.project_component_element==pc,
                                            SAPProjectActual.project_period==period,
                                            SAPProjectActual.actuals_date==state['actuals_date'])).all()
            if len(actual) == 0:
                actual = SAPProjectActual()
                actual.project_period = period
                actual.project_component_element = pc
                actual.actuals_date = state['actuals_date']
            else:
                actual = actual[0]
            has_values = False
            for m in SAPProjectActual.cols_i18n:
                key = '%s_%s' % (c.id, m)
                if state.has_key(key) and state[key] not in ['', None]:
                    setattr(actual, m, state[key])
                    has_values = True
            if has_values:
                self.sa_session.add(actual)


    @transaction_decorator
    def save_project(self, state):
        if state['dbID'] is not None:
            self.assert_permissions('update_sap_project')
            project = self.get_projects(state['dbID'], SAPProject.PRIV_UPDATE)
            if len(project) == 0:
                raise SecurityException('Invalid project id %s' % state['dbID'])
            project = project[0]
        else:
            self.assert_permissions('create_sap_project')
            project = SAPProject()

        project.name = state['name']

        state_components = self.get_components(state['components'])
        project_components = []
        for pc in project.component_elements:
            # leafs will be replaced by their parent if set
            if pc in state_components and pc.parent_component_element not in state_components:
                project_components.append(pc)
        for c in state_components:
            # only add if component or it's parent are not already set for this project
            if c not in project_components and c.parent_component_element not in project_components:
                project_components.append(c)
        project.component_elements = project_components

        self.sa_session.add(project)

        if state['dbID'] is None:
            self.save_default_privilege_object_group(project)

            # cannot update licenses for a project
            lc = LicenseController(self.sa_session, account_id=self.get_account().id)
            license = lc.get_licenses(state['license'])
            if len(license) == 0:
                raise SecurityException('Invalid license %s' % state['license'])
            group = license[0].default_group
            self.grant_privilege_to(project, SAPProject.PRIV_VIEW, False, True, False, group)

        return project

    def get_component_state(self, component):
        if not isinstance(component, SAPComponentElement):
            component = self.get_components(id)
            if len(component) == 0:
                raise SecurityException('Invalid component id %s' % component)
            component = component[0]
        return {
            'dbID': component.id,
            'name': component.name,
            'code': component.code,
            'type': component.type.name
        }
