# Copyright (c) 2011 BaseSoft B.V.
# See LICENSE for details.

import datetime

from sqlalchemy.orm import mapper, relation, backref
from sqlalchemy import Column, Integer, ForeignKey, String, DateTime, SmallInteger, DECIMAL, Unicode, UniqueConstraint
from django.utils.translation import ugettext as _

from basesoft.sqlalchemy.table import IQTable
from basesoft.web.contrib.license import models as license_models

__mapping_defined = False
__tables_defined = False
tables = {}

def reset():
    global __mapping_defined
    global __tables_defined

    __mapping_defined = False
    __tables_defined = False

def get_tables():
    return tables

def do_tables(metadata):
    global __tables_defined
    if not __tables_defined:

        # the project (a license can have only one project ?)
        tblSAPProject = IQTable('tblSAPProject', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),

            Column('name', String(40), nullable=False, unique=True),

            Column('deleted', SmallInteger, nullable=False, default=0),
            Column('archived', SmallInteger, nullable=False, default=0),

            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        tblSAPProjectGroup = IQTable('tblSAPProjectGroup', metadata,
            Column('project_id', Integer, ForeignKey('tblSAPProject.id', ondelete="CASCADE"), primary_key=True),
            Column('group_id', Integer, ForeignKey('tblGroup.id'), primary_key=True),
            Column('privilege', Integer, nullable=False, primary_key=True, autoincrement=False),
            Column('group_has_privilege', SmallInteger, default=0),
            Column('license_has_privilege', SmallInteger, default=0),
            Column('domain_has_privilege', SmallInteger, default=0),
            Column('world_has_privilege', SmallInteger, default=0),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now)
        )

        tblSAPComponentElementType = IQTable('tblSAPComponentElementType', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('name', Unicode(255), nullable=False),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now)
        )

        # privileged object
        tblSAPComponentElement = IQTable('tblSAPComponentElement', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('type_id', Integer, ForeignKey('tblSAPComponentElementType.id'), nullable=False),
            Column('parent_component_element_id', Integer, ForeignKey('tblSAPComponentElement.id'), nullable=True),

            Column('name', String(40), nullable=False, unique=True),
            Column('code', String(40), nullable=False, unique=True),

            Column('deleted', SmallInteger, nullable=False, default=0),
            Column('archived', SmallInteger, nullable=False, default=0),

            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now)
        )

        tblSAPComponentElementGroup = IQTable('tblSAPComponentElementGroup', metadata,
            Column('component_element_id', Integer, ForeignKey('tblSAPComponentElement.id', ondelete="CASCADE"), primary_key=True),
            Column('group_id', Integer, ForeignKey('tblGroup.id'), primary_key=True),
            Column('privilege', Integer, nullable=False, primary_key=True, autoincrement=False),
            Column('group_has_privilege', SmallInteger, default=0),
            Column('license_has_privilege', SmallInteger, default=0),
            Column('domain_has_privilege', SmallInteger, default=0),
            Column('world_has_privilege', SmallInteger, default=0),
            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now)
        )


        tblSAPProjectComponentElement = IQTable('tblSAPProjectComponentElement', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('project_id', Integer, ForeignKey('tblSAPProject.id', ondelete="CASCADE"), nullable=False),
            Column('component_element_id', Integer, ForeignKey('tblSAPComponentElement.id'), nullable=False),

            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now),

            UniqueConstraint('project_id', 'component_element_id')
        )

        # for now we use only year, can be extended later.
        tblSAPProjectPeriod = IQTable('tblSAPProjectPeriod', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('project_id', Integer, ForeignKey('tblSAPProject.id', ondelete="CASCADE"), nullable=False),
            Column('year', SmallInteger, nullable=False),

            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now),

            UniqueConstraint('project_id', 'year')
        )

        tblSAPProjectEstimate = IQTable('tblSAPProjectEstimate', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('project_component_element_id', Integer, ForeignKey('tblSAPProjectComponentElement.id', ondelete="CASCADE"), nullable=False),
            Column('project_period_id', Integer, ForeignKey('tblSAPProjectPeriod.id', ondelete="CASCADE"), nullable=False),

            Column('business_income', Integer, nullable=True),

            Column('percentage_income', DECIMAL(10,2), nullable=True),
            Column('percentage_expenses', DECIMAL(10,2), nullable=True),
            Column('part_of_income', DECIMAL(10,2), nullable=True),
            Column('part_of_expenses', DECIMAL(10,2), nullable=True),
            Column('improvement_income', DECIMAL(10,2), nullable=True),
            Column('improvement_expenses', DECIMAL(10,2), nullable=True),
            Column('income_ist', DECIMAL(16,5), nullable=True),
            Column('expenses_ist', DECIMAL(16,5), nullable=True),
            Column('income_soll', DECIMAL(16,5), nullable=True),
            Column('expenses_soll', DECIMAL(16,5), nullable=True),

            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now),

            UniqueConstraint('project_component_element_id', 'project_period_id')
        )

        tblSAPProjectActual = IQTable('tblSAPProjectActual', metadata,
            Column('id', Integer, primary_key=True, autoincrement=True),
            Column('project_component_element_id', Integer, ForeignKey('tblSAPProjectComponentElement.id', ondelete="CASCADE"), nullable=False),
            Column('project_period_id', Integer, ForeignKey('tblSAPProjectPeriod.id', ondelete="CASCADE"), nullable=False),
            Column('actuals_date', DateTime(), nullable=False),

            Column('income_now', DECIMAL(16,5), nullable=True),
            Column('expenses_now', DECIMAL(16,5), nullable=True),

            Column('created', DateTime(), nullable=False, default=datetime.datetime.now),
            Column('last_updated', DateTime(), nullable=False, default=datetime.datetime.now, onupdate=datetime.datetime.now),
        )

        tables['tblSAPProject'] = tblSAPProject
        tables['tblSAPProjectGroup'] = tblSAPProjectGroup
        tables['tblSAPComponentElementType'] = tblSAPComponentElementType
        tables['tblSAPComponentElement'] = tblSAPComponentElement
        tables['tblSAPComponentElementGroup'] = tblSAPComponentElementGroup
        tables['tblSAPProjectComponentElement'] = tblSAPProjectComponentElement
        tables['tblSAPProjectPeriod'] = tblSAPProjectPeriod
        tables['tblSAPProjectEstimate'] = tblSAPProjectEstimate
        tables['tblSAPProjectActual'] = tblSAPProjectActual

        __tables_defined = True


def do_mappings():
    """
    This function is only called by the django middleware
    when it is needed
    """
    global __mapping_defined
    if not __mapping_defined:
        from basesoft.web.contrib.license.hooks import PrivilegeObjectMapper
        from iqportal.sap.core import models as sap_models

        mapper(sap_models.SAPProject, tables['tblSAPProject'],
            extension = [PrivilegeObjectMapper()],
            properties = {
                'groups': relation(sap_models.SAPProjectGroup, backref='project', cascade="all", passive_deletes=True),
                'component_elements': relation(sap_models.SAPComponentElement, secondary=tables['tblSAPProjectComponentElement']),
            }
        )

        mapper(sap_models.SAPProjectGroup, tables['tblSAPProjectGroup'],
            properties = {
                'group': relation(license_models.Group)
            }
        )

        mapper(sap_models.SAPComponentElementType, tables['tblSAPComponentElementType'])

        mapper(sap_models.SAPComponentElement, tables['tblSAPComponentElement'],
            extension = [PrivilegeObjectMapper()],
            properties = {
                'groups': relation(sap_models.SAPComponentElementGroup, backref='component_element', cascade="all", passive_deletes=True),
                'type': relation(sap_models.SAPComponentElementType),
                'child_component_elements': relation(sap_models.SAPComponentElement, backref=backref('parent_component_element', remote_side=[tables['tblSAPComponentElement'].c.id]))
            }
        )

        mapper(sap_models.SAPComponentElementGroup, tables['tblSAPComponentElementGroup'],
            properties = {
                'group': relation(license_models.Group)
            }
        )

        mapper(sap_models.SAPProjectComponentElement, tables['tblSAPProjectComponentElement'],
            properties = {
                'project': relation(sap_models.SAPProject),
                'component_element': relation(sap_models.SAPComponentElement),
            }
        )

        mapper(sap_models.SAPProjectPeriod, tables['tblSAPProjectPeriod'],
            properties = {
                'project': relation(sap_models.SAPProject),
            }
        )

        mapper(sap_models.SAPProjectEstimate, tables['tblSAPProjectEstimate'],
            properties = {
                'project_component_element': relation(sap_models.SAPProjectComponentElement),
                'project_period': relation(sap_models.SAPProjectPeriod),
            }
        )

        mapper(sap_models.SAPProjectActual, tables['tblSAPProjectActual'],
            properties = {
                'project_component_element': relation(sap_models.SAPProjectComponentElement),
                'project_period': relation(sap_models.SAPProjectPeriod),
            }
        )

        __mapping_defined = True

def fill_data(conn):
    pass

def app_permissions_category():
    """
    Returns a category for all permissions defined for this app
    """
    return license_models.PermissionCategory(u'SAP')

def app_permissions():
    """
    The permissions used within this application. Can be sync'd with the
    `iqsyncpermissions` command.
    """
    return [

        license_models.Permission(u'create_sap_project', u"""
            This gives the right to create a SAP project.
            """, _("Create SAP project")),
        license_models.Permission(u'view_sap_project', u"""
            This gives the right to view a SAP project.
            """, _("View SAP project")),
        license_models.Permission(u'update_sap_project', u"""
            This gives the right to update a SAP project.
            """, _("Update SAP project")),
        license_models.Permission(u'delete_sap_project', u"""
            This gives the right to delete a SAP project.
            """, _("Delete SAP project")),
        license_models.Permission(u'sap_project_manager', u"""
            This gives the right to manage SAP projects.
            """, _("SAP project manager")),

        license_models.Permission(u'create_sap_component_element', u"""
            This gives the right to create a SAP Component Element.
            """, _("Create SAP Component Element")),
        license_models.Permission(u'view_sap_component_element', u"""
            This gives the right to view a SAP Component Element.
            """, _("View SAP Component Element")),
        license_models.Permission(u'update_sap_component_element', u"""
            This gives the right to create a SAP Component Element.
            """, _("Update SAP Component Element")),
        license_models.Permission(u'delete_sap_component_element', u"""
            This gives the right to delete a SAP Component Element.
            """, _("Delete SAP Component Element")),
        ]
